#!/bin/bash
#@version=1.0
# Disable rebuild initrd first boot time.

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

[ -f /etc/growroot-disabled ] && exit 0
grep -wq "vm_debug" /proc/cmdline && { echo 'Cmdline openstack debug!';exit 1; }

grep -qiE 'centos|red' /etc/issue && LINUX_VERSION="centos"
grep -qiE 'ubuntu|debian' /etc/issue && LINUX_VERSION="ubuntu"

resize_disk() {
    # check if resize partition success or not
    root_uuid=$(awk -F '=| ' '{for(i=1;i<=NF;i++){if($i == "UUID"){print $(i+1)}}}' /proc/cmdline)
    root_partition=$(blkid -o full| grep -i "${root_uuid}"|awk -F ':' '{print $1}')

    [ "x${root_partition}" = "x" ] && \
    { echo 'Can not found root partition !';return 1; }

    echo -n "Ready to resize filesystem "
    resize2fs $root_partition > /dev/null 2>&1
    touch /etc/growroot-disabled
    echo -e "\t\t[ OK ]"

#    echo -n "Rebuild initramfs "
#
#    if [ "$LINUX_VERSION" = "ubuntu" ];then
#        update-initramfs -u > /dev/null 2>&1
#    elif [ "$LINUX_VERSION" = "centos" ];then
#        dracut -f > /dev/null 2>&1
#    fi
#    echo -e "\t\t\t[ OK ]"
}

mount_disk() {
    tmp_file=$(mktemp)
    blkid -o full 2>/dev/null > $tmp_file

    root_uuid=$(awk -F '=| ' '{for(i=1;i<=NF;i++){if($i == "UUID"){print $(i+1)}}}' /proc/cmdline)
    root_partition=$(grep -i "${root_uuid}" $tmp_file|awk -F ':' '{print $1}')

    while read line;
    do
        datanode=""
        part="${line%%:*}"
        part_uuid=$(echo $line|awk -F '=| ' '{for(i=1;i<=NF;i++){if($i == "UUID"){print $(i+1)}}}'|tr -d '\"')
        part_type=$(echo $line|awk -F '=| ' '{for(i=1;i<=NF;i++){if($i == "TYPE"){print $(i+1)}}}'|tr -d '\"')
        [ "${root_uuid}" = "${part_uuid}" ] && continue
        grep -q "${part} " /etc/mtab && continue

        for num in $(seq 0 36);
        do
            [ $num -eq 0 ] && datanodes="/data" || datanodes="/data${num}"
            if [ -d $datanodes ];then
                mountpoint $datanodes >/dev/null 2>&1 || datanode="$datanodes"
            else
                mkdir $datanodes
                datanode=$datanodes
            fi
            [ "x${datanode}" != "x" ] && break
        done

        # Start to try mount partitions
        if mount -t ${part_type} ${part} ${datanode};then
            echo -e "UUID=${part_uuid}\t${datanode}\t${part_type}\tdefaults 0 0" >> /etc/fstab
        else
            # It so bad, I don't want to come here!
            FSTYPES="ext4 ext3 ext2 xfs vfat"
            for FSTYPE in $FSTYPES;
            do
                if mount -t ${FSTYPE} ${part} ${datanode};then
                    echo -e "UUID=${part_uuid}\t${datanode}\t${FSTYPE}\tdefaults 0 0" >> /etc/fstab
                    break
                fi
            done
        fi
    done < $tmp_file
    rm -f $tmp_file

}

move_dir() {
    mountpoint /data || return

    for dir in /var/log/.first_boot/*;
    do
        [ -d /data/${dir##*/} ] && rm -rf $dir || mv $dir /data/.
    done
    rm -rf /var/log/.first_boot
}

resize_disk
mount_disk
move_dir

exit 0
