#!/bin/bash
#@version=0.9

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

grep -wq "vm_debug" /proc/cmdline &&
{ echo 'Cmdline openstack debug!';exit 1; }

tmp_file="/tmp/vm_firstboot_run.log"
exec 200<>$tmp_file
flock -n -e 200 || { echo "$0 Already Running, exit!";exit 1; }

MODE="$1"
BASENAME="vm_firstboot_run"
log_file="/var/log/sysop_manager/user_privilege_change.log"
run_time=$(date "+%Y-%m-%d %H:%M:%S")
repos_file='/usr/local/i386/*/initial_system/repos_server.log'
ISPFILE='/usr/local/i386/public_repos/initial_system/cmdb/all_ipinfo.log'

function exit_func () {
    rm -f $tmp_file
}

trap 'exit_func;exit' 0 2 3 9 15

function get_baseos () {
    ip_addr=$(ifconfig| awk '{if($0 ~ /inet addr/){print $2}}'| \
            grep -v '127.0.0.1'|awk -F ':' '{print $2;exit}')
    system_uuid=$(dmidecode -s system-uuid|tr [A-Z] [a-z])
    pick_repos1=$(grep 'internal' $repos_file|awk '{print $1}'|sort -R|head -n2)
    pick_repos2=$(grep -E "wangtong|dianxin" $repos_file|awk '{print $1}'|sort -R|head -n2)
    pick_repos=$(echo -e "${pick_repos1}\n${pick_repos2}"|sort -R)
}

function clean_script () {
    sed -i "/$BASENAME/d" /etc/rc.local /etc/crontab
    rm -f /etc/init.d/${BASENAME}
    rm -f /etc/rc*.d/*${BASENAME}
}

function add_user () {
    local user=$1
    local user_uid=$2
    local user_key=$(echo "$3"|base64 -d)

    id ${user} > /dev/null 2>&1
    if [ $? -ne 0 ];then
        groupadd -g ${user_uid} $user
        useradd -s /bin/bash -m -u ${user_uid} -g ${user_uid} $user
        mkdir -p /home/${user}/.ssh
        echo "$user_key" > /home/${user}/.ssh/authorized_keys
        chmod 600 /home/${user}/.ssh/authorized_keys
        chmod 700 /home/${user}/.ssh
        chown -R ${user}:${user} /home/${user}
        usermod -G execute $user
        id ${user} >/dev/null 2>&1 &&
        echo "${run_time} adduser ${user} success from vm boot firstime" >> $log_file
    fi
}

function adduser_list () {
    [ "x${system_uuid}" == "x" ] && return 1

    for which_ip in "manager.repos.yy.duowan.com" $pick_repos;
    do
        wget --http-user=manifold --http-password=dw_manifold -q --timeout=4 --tries=2 -O $tmp_file "http://${which_ip}:63217/external/VM_query_add_user.php?ip=$ip_addr&uuid=$system_uuid&type=text" && break
    done
    grep -q "vm_no_user" $tmp_file && return 0
    if [ -s $tmp_file ];then
        while read user ID key;do
            add_user $user $ID $key
        done < $tmp_file
    fi
}

function calc_network () {
    local NIC=$1

    local NIC_IP=$(ifconfig $NIC | grep 'inet addr'| \
                awk '{print $2}'|awk -F ':' '{print $2}')
    local NIC_MASK=$(ifconfig $NIC | grep 'inet addr'| \
                awk '{print $NF}'|awk -F ':' '{print $2}')
    if [ "x${NIC_IP}" = "x" ] || [ "x${NIC_MASK}" = "x" ];then
        return
    fi
    host_num="$((256 - ${NIC_MASK##*.}))"
    network_unit="$((${NIC_IP##*.} / host_num))"
    network_en="$((network_unit * host_num))"

    local NIC_GW="${NIC_IP%.*}.$((network_en + 1))"
    echo "$NIC_IP $NIC_MASK $NIC_GW"
}

function pick_default_gw () {
    export default_gw DEF_ISP nofound_isp

    DEF_ISP=99
    for which_nic in $(ifconfig |cut -d ' ' -f1|grep -E 'eth'| grep -v '^$');
    do
        network_info=$(calc_network $which_nic)
        net_ip="${network_info%% *}"
        net_mask="${network_info#* }"
        net_gw="${net_mask##* }"
        net_mask="${net_mask%% *}"

        if [ -s $ISPFILE ] && [ "x${net_ip}" != "x" ];then
            ISP=$(grep -w -m1 "${net_ip}" $ISPFILE | awk '{print $3}')
            if [ "x${ISP}" == "x" ];then
                net_ip_suffix="${net_ip%.*}"
                ISP=$(grep -m1 "${net_ip_suffix}\." $ISPFILE|awk '{print $3}')
            fi

            if [ "x${ISP}" != "x" ];then
                if [ $ISP -lt $DEF_ISP ];then
                    DEF_ISP=$ISP
                    default_gw=$net_gw
                fi
            else
                nofound_isp='yes'
                break
            fi
        else
            nofound_isp='yes'
            break
        fi
    done
}

function change_default_gw () {
    change_gw=$1
    [ "x${change_gw}" = "x" ] && return 1

    get_old_gw=$(route -n|grep '^0.0.0.0'|awk '{print $2}')
    route add -net default gw $change_gw
    for which_gw in $get_old_gw;
    do
        [ "$which_gw" == "$change_gw" ] && continue
        route del -net default gw $which_gw
    done
    route add -net default gw $change_gw || true
    /bin/bash /usr/local/i386/*/auto/system_router.sh >/dev/null 2>&1
}

#############
#    main   #
############
get_baseos
[ "$MODE" = "netonly" ] || adduser_list
pick_default_gw
if [ "x${nofound_isp}" != 'xyes' ];then
    change_default_gw $default_gw
    clean_script
elif [ $(awk -F '.' '{print $1}' /proc/uptime) -ge 3600 ];then
    clean_script
else
    # Try again to obtain isp in next loop. Do nothing here.
    :
fi
true
