#!/bin/bash

export INTERFACES=$INTERFACE
export ACTIONS=$ACTION

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/urs/local/bin:/usr/local/sbin

[ "x${INTERFACES}" = "x" ] && exit 1

PIDFILE="/var/run/dhclient.${INTERFACES}.pid"
LEASEFILE="/var/run/dhclient.${INTERFACES}.leases"
ISPFILE="/usr/local/i386/public_repos/initial_system/cmdb/all_ipinfo.log"

function check_nic_configure_or_not () {
	local NIC=$1
	local status=0
	export LINUX_VERSION

	if grep -qiE 'centos|red' /etc/issue;then
		LINUX_VERSION="centos"
		for nic_conf in $(ls -d /etc/sysconfig/network-scripts/ifcfg-${INTERFACES}*);
		do
			grep -qw "$NIC" $nic_conf && status=1 && break;
		done
	elif grep -qiE 'ubuntu|debian' /etc/issue;then
		LINUX_VERSION="ubuntu"
		grep "auto .*${NIC}" /etc/network/interfaces && \
		status=1
	fi

	[ $status -eq 1 ] && [ "x${ACTIONS}" = "xadd" ] && exit 0
}

function enqueue_dhclient () {
	local NIC=$1

	[ -f "$PIDFILE" ] && rm -f $PIDFILE
	[ -f "$LEASEFILE" ] && rm -f $LEASEFILE

	if [ "$LINUX_VERSION" = "centos" ];then
		dhclient -1 -pf $PIDFILE -lf $LEASEFILE -R subnet-mask $NIC
	elif [ "$LINUX_VERSION" = "ubuntu" ];then
		dhclient -1 -pf $PIDFILE -lf $LEASEFILE -cf /etc/dhcp/dhclient_virtio.conf $NIC
	fi
	wait
}

function dequeue_dhclient () {
	local NIC=$1

	if [ "$LINUX_VERSION" = "centos" ];then
		[ -f $LEASEFILE ] && dhclient -pf $PIDFILE -lf $LEASEFILE -r $NIC
	elif [ "$LINUX_VERSION" = "ubuntu" ];then
		[ -f $LEASEFILE ] && \
		dhclient -pf $PIDFILE -lf $LEASEFILE -cf /etc/dhcp/dhclient_virtio.conf -x -r $NIC
	fi
	wait

	[ -f "$PIDFILE" ] && rm -f $PIDFILE
	[ -f "$LEASEFILE" ] && rm -f $LEASEFILE

	[ ! -d /sys/class/net/$NIC ] && sed -i "/$NIC/d" /etc/udev/rules.d/70-persistent-net.rules 2>/dev/null
}

function kill_dhclient_daemon () {
	local NIC=$1
	ps aux | grep -w "$LEASEFILE" | grep -v "grep"|awk '{print $2}' | xargs kill -9
	[ -f "$PIDFILE" ] && rm -f $PIDFILE
	[ -f "$LEASEFILE" ] && rm -f $LEASEFILE

	[ ! -d /sys/class/net/$NIC ] && sed -i "/$NIC/d" /etc/udev/rules.d/70-persistent-net.rules 2>/dev/null
}

function calc_network () {
	local NIC=$1

	local NIC_IP=$(ifconfig $NIC | grep 'inet addr'| \
				awk '{print $2}'|awk -F ':' '{print $2}')
	local NIC_MASK=$(ifconfig $NIC | grep 'inet addr'| \
				awk '{print $NF}'|awk -F ':' '{print $2}')
	if [ "x${NIC_IP}" = "x" ] || [ "x${NIC_MASK}" = "x" ];then
		return
	fi
	host_num="$((256 - ${NIC_MASK##*.}))"
	network_unit="$((${NIC_IP##*.} / host_num))"
	network_en="$((network_unit * host_num))"

	local NIC_GW="${NIC_IP%.*}.$((network_en + 1))"
	echo "$NIC_IP $NIC_MASK $NIC_GW"
}

function pick_default_gw () {
	export default_gw DEF_ISP

	DEF_ISP=99
	for which_nic in $(ifconfig |cut -d ' ' -f1|grep -E 'eth'| grep -v '^$');
	do
		network_info=$(calc_network $which_nic)
		net_ip="${network_info%% *}"
		net_mask="${network_info#* }"
		net_gw="${net_mask##* }"
		net_mask="${net_mask%% *}"

		if [ -s $ISPFILE ] && [ "x${net_ip}" != "x" ];then
			net_ip_suffix="${net_ip%.*}"
			ISP=$(grep -m1 "${net_ip_suffix}\." $ISPFILE|\
					awk '{print $3}')
			if [ "x${ISP}" = "x" ];then
				ISP_SUFFIX="${net_ip_suffix%.*}"
				ISP=$(grep -m1 "${ISP_SUFFIX}\." $ISPFILE|\
					awk '{print $3}')
			fi

			if [ "x${ISP}" != "x" ] && [ $ISP -lt $DEF_ISP ];then
				DEF_ISP=$ISP
				default_gw=$net_gw
			fi
		fi

		if echo $net_ip|grep -q '10\.';then
			CUR_INTERNAL_GW=$(route -n|grep '^10.0.0.0'|awk '{print $2}')
			if [ "x${CUR_INTERNAL_GW}" = "x" ];then
				INTERNAL_NET="yes"
				INTERNAL_GW=$net_gw
			fi
		fi
	done
}

function change_default_gw () {
	change_gw=$1

	[ "x${change_gw}" = "x" ] && return

	get_old_gw=$(route -n|grep '^0.0.0.0'|awk '{print $2}')

	route add -net default gw $change_gw

	for which_gw in $get_old_gw;
	do
		[ "$which_gw" == "$change_gw" ] && continue
		route del -net default gw $which_gw
	done

	[ "x${INTERNAL_NET}" = "xyes" ] &&
		route add -net 10.0.0.0/8 gw $INTERNAL_GW

	route add -net default gw $change_gw || true
}

check_nic_configure_or_not $INTERFACES
case $ACTIONS in
	add|register)
		enqueue_dhclient $INTERFACES
		kill_dhclient_daemon $INTERFACES
		;;
	remove|unregister)
		dequeue_dhclient $INTERFACES
		;;
esac

pick_default_gw
change_default_gw $default_gw
/bin/bash /usr/local/i386/*/auto/system_router.sh

true






