#!/bin/bash
#Date:2011-07-01
#Written by manifold.
#Readme:integrate /etc/sudoers /etc/ssh/ssh_config /etc/crontab /etc/motd /etc/skel ... monitor.

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games

# check linux version
cat /etc/issue|grep -E "CentOS|Red" && LINUX_VERSION="centos"
cat /etc/issue|grep -E "Ubuntu" && LINUX_VERSION="ubuntu"
if [ "$LINUX_VERSION" = "" ];then exit 1;fi

# basic define
BASENAME=`basename $0`
SHORT_NAME=base_config
LOG_DIR=/var/log/sysop_manager

if [ ! -d $LOG_DIR ];then
	mkdir -p $LOG_DIR
fi

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

# repository path
REPOS_NAME=comm_repos
COMM_DIR=/usr/local/i386/$REPOS_NAME
PUBLIC_DIR=/usr/local/i386/public_repos

# get ip address
IPADDR=`/sbin/ifconfig|grep Bcast|grep -vE "inet addr:192\.168|inet addr:172\.|inet addr:10\."|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
	INTERNAL_IP="yes"
fi

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:] ]*$SHORT_NAME" && exit 1

BACKUP_DATE=`date +%Y-%m-%d-%H-%M-%S`

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# define detect file function
Detect_file () {
	if [ ! -f $S_FILE ];then
		INSERT_MYSQL "no file $S_FILE."
	elif [ ! -f $D_FILE ];then
		/bin/cp $S_FILE $D_FILE
		if [ "$?" -eq 0 ];then
			INSERT_MYSQL "add file $D_FILE successful."
			CHANGE=yes
		else
			INSERT_MYSQL "add file $D_FILE failed."
		fi
	else
		S_MD5=`md5sum $S_FILE|awk '{print $1}'`
		D_MD5=`md5sum $D_FILE|awk '{print $1}'`
		if [ "$S_MD5" != "$D_MD5" ];then
			BACKUP_FILE="`echo $D_FILE|sed 's#/#_#g'`.$BACKUP_DATE"
			/bin/cp $D_FILE $LOG_DIR/$BACKUP_FILE
			cat $S_FILE > $D_FILE
			if [ "$?" -eq 0 ];then
				CHANGE=yes
				INSERT_MYSQL "$D_FILE changed success"
			else
				INSERT_MYSQL "$D_FILE changed failed"
			fi
		fi
	fi
}

function Lock_detect_file () {
	local ower="$1"
	local mode="$2"
	if [ ! -f $S_FILE ];then
		INSERT_MYSQL "no file $S_FILE."
	elif [ ! -f $D_FILE ];then
		/bin/cp $S_FILE $D_FILE
		if [ "$?" -eq 0 ];then
			INSERT_MYSQL "add file $D_FILE successful."
			CHANGE=yes
		else
			INSERT_MYSQL "add file $D_FILE failed."
		fi
	else
		S_MD5=`md5sum $S_FILE|awk '{print $1}'`
		D_MD5=`md5sum $D_FILE|awk '{print $1}'`
		if [ "$S_MD5" != "$D_MD5" ];then
			BACKUP_FILE="`echo $D_FILE|sed 's#/#_#g'`.$BACKUP_DATE"
			/bin/cp $D_FILE $LOG_DIR/$BACKUP_FILE
			chattr -i $D_FILE
			cat $S_FILE > $D_FILE
			chown $ower $D_FILE
			chmod $mode $D_FILE
			chattr +i $D_FILE
			if [ "$?" -eq 0 ];then
				CHANGE=yes
				INSERT_MYSQL "$D_FILE changed success"
			else
				INSERT_MYSQL "$D_FILE changed failed"
			fi
		fi
	fi
}

# define detect exclude list function.
Detect_exclude () {
	SHORT_NAME="$1"
	EXCLUDE_DETECT_LINE=`cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:] ]*$SHORT_NAME"|wc -l`
	if [ "$EXCLUDE_DETECT_LINE" -eq 0 ] && [ ! -f $LOG_DIR/stop_run_$SHORT_NAME ];then
		RUN=yes
	fi
}

######################################################################################################
######################################################################################################

# timezone setting
unset RUN
Detect_exclude timezone
if [ "$RUN" = "yes" ];then
	S_FILE=/usr/share/zoneinfo/Asia/Shanghai
	D_FILE=/etc/localtime
	unset CHANGE
	Detect_file
	if [ "$CHANGE" = "yes" ];then
		INSERT_MYSQL "change timezone to Asia/Shangehai."
	fi
fi

######################################################################################################
######################################################################################################

# /etc/sudoers setting
unset RUN
Detect_exclude sudo
if [ "$RUN" = "yes" ];then
	[ ! -d /etc/sudoers.d ] && mkdir /etc/sudoers.d && chmod 755 /etc/sudoers.d
	S_FILE=$COMM_DIR/initial_system/sudoers.$LINUX_VERSION
	D_FILE=/etc/sudoers
	unset CHANGE
	Detect_file
	chmod 440 /etc/sudoers /etc/sudoers.d/*
	chown root:root /etc/sudoers /etc/sudoers.d/*

	for csudo in /etc/sudoers.d/*;
	do
		visudo -c -f $csudo >/dev/null 2>&1 || \
		mv $csudo $LOG_DIR/sudo_$(basename $csudo)_$(date +%s)
	done
fi

######################################################################################################
######################################################################################################

# ssl setting
unset RUN
Detect_exclude ssl
if [ "$RUN" = "yes" ];then
	if [ ! -d /etc/ssl ];then
		mkdir -p /etc/ssl
	fi
	S_FILE=$COMM_DIR/platform/yy-cacert.pem
	D_FILE=/etc/ssl/yy-cacert.pem
	unset CHANGE
	Detect_file
fi

######################################################################################################
######################################################################################################

# environment setting
unset RUN
Detect_exclude skel
if [ "$RUN" = "yes" ];then
	if [ "$LINUX_VERSION" = "ubuntu" ];then
		FILE_LIST () {
		(
		cat <<'EOF'
/usr/local/i386/comm_repos/initial_system/skel/.bash_logout	/etc/skel/.bash_logout
/usr/local/i386/comm_repos/initial_system/skel/.bashrc	/etc/skel/.bashrc
/usr/local/i386/comm_repos/initial_system/skel/.profile	/etc/skel/.profile
/usr/local/i386/comm_repos/initial_system/skel/.vimrc	/etc/skel/.vimrc
/usr/local/i386/comm_repos/initial_system/skel/.bash_logout	/home/yuwanfu/.bash_logout
/usr/local/i386/comm_repos/initial_system/skel/.bashrc	/home/yuwanfu/.bashrc
/usr/local/i386/comm_repos/initial_system/skel/.profile	/home/yuwanfu/.profile
/usr/local/i386/comm_repos/initial_system/skel/.vimrc	/home/yuwanfu/.vimrc
EOF
		)
		}

		FILE_LIST | while read S_FILE D_FILE
		do
			Detect_file
		done
		if [ "$CHANGE" = "yes" ];then
			chown -R yuwanfu:yuwanfu /home/yuwanfu
		fi

		BASH_BASHRC_ENTRY () {
		(
		cat <<'EOF'
alias ls='ls --color=auto'
alias ll='ls -l'
alias grep='grep --color=auto'
alias go='ssh -p 32200'
alias vi='/usr/bin/vim'
alias yy='cd /home/dspeak/release/bin'
alias yylog='cd /data/yy/log'
export HISTSIZE=1000
export HISTFILESIZE=5000
export HISTTIMEFORMAT="%F %T: "
EOF
		)
		}

		# /etc/bash.bashrc configure
		if [ -f /etc/bash.bashrc ];then
			BASH_BASHRC_ENTRY | while read ALIAS_ENTRY
			do
				ALIAS_HEAD=`echo $ALIAS_ENTRY|awk -F '=' '{print $1}'`
				ALIAS_LINE=`cat /etc/bash.bashrc|grep "$ALIAS_HEAD"|wc -l`
				NEW_ALIAS_LINE=`cat /etc/bash.bashrc|grep -w "$ALIAS_ENTRY"|wc -l`
				if [ "$ALIAS_LINE" -eq 0 ];then
					echo "$ALIAS_ENTRY" >> /etc/bash.bashrc
				elif [ "$ALIAS_LINE" -gt 0 ] && [ "$NEW_ALIAS_LINE" -eq 0 ];then
					sed -i '/^'"${ALIAS_HEAD}"'=.*/d' /etc/bash.bashrc
					echo "$ALIAS_ENTRY" >> /etc/bash.bashrc
				fi
			done
		fi

		# change PS
		cat /etc/bash.bashrc |grep "export.*PS1\="
		if [ "$?" -ne 0 ];then
			cat /etc/bash.bashrc|grep "PS1\=" && sed -i 's/PS1\=/export PS1\=/g' /etc/bash.bashrc
		fi

		# check bind clear screen
		cat /etc/bash.bashrc |grep "bind.*-x.*C-l.*clear"
		if [ "$?" -ne 0 ];then
			echo bind -x \'\"\\C-l\":clear\' >> /etc/bash.bashrc
		fi
	fi
fi

######################################################################################################
######################################################################################################

# /etc/motd setting
unset RUN
Detect_exclude motd
if [ "$RUN" = "yes" ];then
	# old update-motd detect
	if [ -f /etc/cron.d/update-motd ];then
		S_FILE=$COMM_DIR/initial_system/motd/update-motd
		D_FILE=/etc/cron.d/update-motd
		unset CHANGE
		Detect_file
	fi
	if [ -d /etc/update-motd.d ];then
		if [ ! -d /etc/update-motd.d/bak ];then
			mkdir /etc/update-motd.d/bak
			cd /etc/update-motd.d && mv * bak
		fi
		if [ ! -f /etc/update-motd.d/motd ];then
			echo "cat /etc/motd" > /etc/update-motd.d/motd
			chmod 755 /etc/update-motd.d/motd
		fi
	fi
	# reconfigure sshd
	cat /etc/pam.d/sshd|grep pam_motd.so && sed -i '/pam_motd.so/ s/^#*//' /etc/pam.d/sshd
	# old sysinfo rc.local detect
	cat /etc/rc.local|grep /usr/share/landscape/landscape-sysinfo.wrapper && sed -i '/landscape-sysinfo.wrapper/d' /etc/rc.local
	# generate motd
	CURRENT_TIME=`date "+%Y-%m-%d-%H"`
	CHECK_TIME=`cat $LOG_DIR/check_motd.log`
	if [ "$CURRENT_TIME" != "$CHECK_TIME" ];then
		/bin/bash $COMM_DIR/initial_system/motd/generate_motd.sh && echo "$CURRENT_TIME" > $LOG_DIR/check_motd.log
		SCRIPT_NAME="base_config.sh"
		MESSAGE="update motd."
		echo "`date +%Y-%m-%d-%T` $SCRIPT_NAME $MESSAGE" >> $LOG_DIR/sysop_manager.log 2>&1
	fi
fi

######################################################################################################
######################################################################################################

# check /etc/ssh/sshd_config
unset RUN
Detect_exclude ssh
if [ "$RUN" = "yes" ];then
LINE=`cat /etc/ssh/sshd_config |grep -v ^#|grep -E "(Port 32200|PasswordAuthentication no|PermitRootLogin no|PubkeyAuthentication yes)"|wc -l`
if [ "$LINE" -ne 4 ];then
	INSERT_MYSQL "SSH configure error!!!"
fi

# check /etc/ssh/ssh_config
S_FILE=$COMM_DIR/initial_system/ssh_config
D_FILE=/etc/ssh/ssh_config
unset CHANGE
Detect_file

# check ssh process
if [ "$LINUX_VERSION" = "ubuntu" ];then
	SSH_PATH=/etc/init.d/ssh
else
	SSH_PATH=/etc/init.d/sshd
fi
ps aux|grep -v grep|grep /usr/sbin/sshd
if [ "$?" -ne 0 ];then
	$SSH_PATH restart
	if [ "$?" -eq 0 ];then
		INSERT_MYSQL "No ssh process,and restart successful."
	else
		INSERT_MYSQL "No ssh process,but restart failed."
	fi
fi
fi

######################################################################################################
######################################################################################################

# check /etc/sysctl.conf
unset RUN
Detect_exclude sysctl
if [ "$RUN" = "yes" ];then
#	if [ -d $PUBLIC_DIR ];then
#		EXCLUDE_IP_LINE_SYSCTL=`cat $COMM_DIR/monitor/sar_dip.log|grep -w $IPADDR|wc -l`
#		if [ "$EXCLUDE_IP_LINE_SYSCTL" -gt 0 ];then
#			SYSCTL_FILE=`cat $PUBLIC_DIR/initial_system/sysctl/sysctl_version.log|grep -v ^#|awk '{if($1=="'"${IPADDR}"'") print $2}'|uniq|head -n 1`
#			if [ "$SYSCTL_FILE" != "" ];then
#				S_FILE=$PUBLIC_DIR/initial_system/sysctl/$SYSCTL_FILE
#			else
#				S_FILE=$PUBLIC_DIR/initial_system/sysctl/sysctl.conf_oom
#			fi
#		else
#			SYSCTL_FILE=`cat $PUBLIC_DIR/initial_system/sysctl/sysctl_version.log|grep -v ^#|awk '{if($1=="'"${IPADDR}"'") print $2}'|uniq|head -n 1`
#			if [ "$SYSCTL_FILE" != "" ];then
#				S_FILE=$PUBLIC_DIR/initial_system/sysctl/$SYSCTL_FILE
#			else
#				# жǷweb serverܣǵĻҪtw_recycleص
#				ps aux|grep -v grep|grep -E "nginx|apache|httpd|keepalived" && S_FILE=$PUBLIC_DIR/initial_system/sysctl/sysctl.conf_${REPOS_NAME}_web || S_FILE=$PUBLIC_DIR/initial_system/sysctl/sysctl.conf_${REPOS_NAME}
#			fi
#		fi
#		D_FILE=/etc/sysctl.conf
#		unset CHANGE
#		Detect_file
#		if [ "$CHANGE" = "yes" ];then
#			sysctl -p
#		fi
#	fi
	[ -s $PUBLIC_DIR/initial_system/monitor/sysctl_common.conf ] && sysctl -p $PUBLIC_DIR/initial_system/monitor/sysctl_common.conf
fi

######################################################################################################
######################################################################################################

# check /etc/security/limits.conf
unset RUN
Detect_exclude limits
if [ "$RUN" = "yes" ];then
	S_FILE=$COMM_DIR/initial_system/etc_security_limits.conf.content
	D_FILE=/etc/security/limits.conf
	unset CHANGE
	Detect_file
fi

######################################################################################################
######################################################################################################

# check system's crontab task

######################################################################################################
######################################################################################################

# check user's crontab task
unset RUN
Detect_exclude dispatcher
if [ "$RUN" = "yes" ];then
	S_FILE=$COMM_DIR/initial_system/dispatcher.content
	unset CHANGE
	if [ "$LINUX_VERSION" = "ubuntu" ];then
		D_FILE=/var/spool/cron/crontabs/root
		CRON_BIN_PATH=/etc/init.d/cron
		Lock_detect_file "root:crontab" "0600"
	else
		D_FILE=/var/spool/cron/root
		CRON_BIN_PATH=/etc/init.d/crond
		Lock_detect_file "root:root" "0600"
	fi
	if [ "$CHANGE" = "yes" ];then
		$CRON_BIN_PATH restart
		if [ "$?" -eq 0 ];then
			INSERT_MYSQL "$CRON_BIN_PATH restart success"
		else
			INSERT_MYSQL "$CRON_BIN_PATH restart failed"
		fi
	fi
fi

######################################################################################################
######################################################################################################

# host sign in function.
echo $*|grep -w nosign
if [ "$?" -ne 0 ];then
	/bin/bash $COMM_DIR/auto/host_sign_in.sh
fi

######################################################################################################
######################################################################################################

# logcheck function for system.
# unset RUN
# Detect_exclude logcheck
# if [ "$RUN" = "yes" ];then
# 	# invoke the time generate by motd function.
	# if [ "$CURRENT_TIME" != "$CHECK_TIME" ];then
# 		# Not tainted
		# LOGCHECK_MEG=`tail -n 5000 /var/log/messages|grep -v -E "yyms_agent_d|tcpdump|proxyserver_d|nagios|httpd|promiscuous|im-vm08|IPTABLES|dump_server|libc|error count|test|segfault|dhclient|province|ZMODEM|dhcp|default_wake_function"|grep -E -i "out of|promiscuous mode|skip device|dst cache overflow|error|block type is not|fault|attempted to kill|Kernel Panic|abnormal status|mismatch|Link is down|task abort|degraded|timed out|failure|overflow|Dazed|invalid|dead|reject|EDAC"`
		# if [ "$LOGCHECK_MEG" != "" ];then
			# INSERT_MYSQL "$LOGCHECK_MEG"
		# fi
	# fi
# fi

######################################################################################################
######################################################################################################

: <<EOF
1: "two-lane",
2: "two-lane-telecom",
3: "two-lane-unicom",
4: "telecom",
5: "unicom",
6: "bgp",
7: "edu",
8: "cmcc",
10: "Intranet"
EOF

# check server's isp
unset RUN
Detect_exclude check_isp
if [ "$RUN" = "yes" ];then
	if [ -f /home/dspeak/yyms/hostinfo ];then
		cat /home/dspeak/yyms/hostinfo|python -mjson.tool|awk '{if($1~/isp/) print $2}'|sort -u > $LOG_DIR/isp.log 2>&1
	fi
fi

######################################################################################################
######################################################################################################
# jifen_rsync
S_FILE=$COMM_DIR/initial_system/rsync/rsync_jifen.sh
D_FILE=/home/dspeak/rsync_jifen.sh
unset CHANGE
Detect_file

S_FILE=$COMM_DIR/initial_system/rsync/check_rsync_jifen.sh
D_FILE=/home/dspeak/check_rsync_jifen.sh
unset CHANGE
Detect_file

