#!/bin/bash
#Date:2011-05-15
#Written by manifold.
#Readme:manage smokeping.

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
exit 0

function set_variables
{	
	# basic define
	BASENAME=`basename $0`
	SHORT_NAME=smokeping
	LOG_DIR=/var/log/sysop_manager
	BACKUP_DATE=`date +%Y-%m-%d-%H-%M-%S`
	# repository path
	REPOS_NAME=comm_repos
	COMM_DIR=/usr/local/i386/$REPOS_NAME
	# get ip address 
	IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
	[ "$IPADDR" = "" ] && IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
	
}

set_variables

function initializing
{
	# if not a ubuntu system,exit.
	cat /etc/issue|grep -E "Ubuntu" || exit 1
	# stop run configure
	if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ]
	then
			exit 1
	fi
	# exclude list
	cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR .*$SHORT_NAME" && exit 1
}

initializing

# if not a smokeping server,exit
cat $COMM_DIR/smokeping/smokeping_ipaddress.log | grep -w $IPADDR || exit 1


# define insert alert function
INSERT_MYSQL () 
{
	/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# check smokeping host file's version


function check_smokeping_process
{
	# check smokeping process exist or not
	SMOKEPING_PROCCESS_EXPECTED_COUNT=5
	CURRENT_SMOKEPING_PROCCESS_COUNT=$(ps aux|grep -E "/usr/sbin/smokeping[[:space:]]*\[FPing.+\]"|grep -E -v "grep|mysql" | wc -l) 
	if [ $CURRENT_SMOKEPING_PROCCESS_COUNT -lt $SMOKEPING_PROCCESS_EXPECTED_COUNT ]
	then
		sleep 5
		CURRENT_SMOKEPING_PROCCESS_COUNT=$(ps aux|grep -E "/usr/sbin/smokeping[[:space:]]*\[FPing.+\]"|grep -E -v "grep|mysql" | wc -l) 
		if [ $CURRENT_SMOKEPING_PROCCESS_COUNT -lt $SMOKEPING_PROCCESS_EXPECTED_COUNT ]
		then
			/etc/init.d/smokeping restart
			if [ "$?" -eq 0 ];then
				INSERT_MYSQL "Smokeping proccess dead or abnormal, but restarted successfully."
			else
				INSERT_MYSQL "Smokeping proccess dead or abnormal, and failed to restart."
			fi
		fi
	fi
}

check_smokeping_process

#test target file exist or not, and difference from source file.

function diff_configure_file
{
	FILE=$1
	TARGET_FILE=$2
	if [ ! -f $FILE ] 
	then
		return 1
	fi
	
	CURRENT_SMOKEPING_MD5=$(md5sum $FILE | awk '{print $1}')
	TARGET_SMOKEPING_MD5=$(md5sum $TARGET_FILE | awk '{print $1}')
	if [ "$CURRENT_SMOKEPING_MD5" != "$TARGET_SMOKEPING_MD5" ] 
	then
		return 2
	fi
	
}

#test all smokeping configure file
function check_configure_files
{
	FILE_LIST="config bgp_and_double_line double_line telecom telecom2 netcom_and_others"
	[ ! -d /etc/smokeping/config.d ] && mkdir -p /etc/smokeping/config.d
	for FILENAME in $(echo "$FILE_LIST")
	do
		SMOKEPING_CONFIG_DIR="/etc/smokeping"
		[ "$FILENAME" = "config" ] || SMOKEPING_CONFIG_DIR="$SMOKEPING_CONFIG_DIR/config.d"
		diff_configure_file $SMOKEPING_CONFIG_DIR/$FILENAME $COMM_DIR/smokeping/conf/$FILENAME	
		STATUS=$?
		if [  $STATUS -eq 2 ] 
		then 
			/bin/cp -f $SMOKEPING_CONFIG_DIR/$FILENAME  $LOG_DIR/SMOKEPING_CONF_OF_$FILENAME.$BACKUP_DATE
			/bin/cp -f $COMM_DIR/smokeping/conf/$FILENAME $SMOKEPING_CONFIG_DIR/$FILENAME
			CHANGE=yes
		elif [ $STATUS -eq 1 ]
		then
			/bin/cp -f $COMM_DIR/smokeping/conf/$FILENAME $SMOKEPING_CONFIG_DIR/$FILENAME
			CHANGE=yes
		fi
	done
	#ûԶӼؽڵ
    if [[ ! -f $COMM_DIR/smokeping/conf/smokeping.add ]]||[[ ! -f /etc/smokeping/config.d/smokeping.add ]];then
		touch $COMM_DIR/smokeping/conf/smokeping.add
        cp $COMM_DIR/smokeping/conf/smokeping.add /etc/smokeping/config.d/.
	fi
	
}


function reload_smokeping
{
#		if [ /usr/sbin/smokeping --check ]
#		if [ "$?" -eq 0 ]
	 /etc/init.d/smokeping  reload 
	if [ "$?" -eq 0 ]
	then
#			COUNT=1
#			while  [ $COUNT -le 10 ]
#			do
#				NOW=$(date +%R)
#				/etc/init.d/smokeping  reload
#				sleep 5
#				START_TIME=$(ps aux | grep -E "/usr/sbin/smokeping \[FPing.\]"|awk '{ print $9 }'|uniq)
#				[ "$START_TIME" = "$NOW" ] && break
#				let COUNT+=1
#			done
#				
#			if [ $COUNT -le 10 ]
#			then
		INSERT_MYSQL "smokeping configuration files changed,smokeping reloaded."
#					INSERT_MYSQL "smokeping configuration files changed, restarted smokeping."
#			else
#					INSERT_MYSQL "smokeping configuration files changed, but restarted failed."
#			fi

	else
		INSERT_MYSQL "smokeping configuration files changed, but errors exist in configuration file."
	fi

}

check_configure_files
[ "$CHANGE" = "yes" ] && reload_smokeping

function check_apache_process
{
	# check apache process exist or not
	if [ `ps aux|grep -v grep|grep "/usr/sbin/apache2 -k start"|wc -l` -eq 0 ];then
		sleep 5
		if [ `ps aux|grep -v grep|grep "/usr/sbin/apache2 -k start"|wc -l` -eq 0 ];then
			/etc/init.d/apache2 restart
			if [ "$?" -eq 0 ];then
				INSERT_MYSQL "Smokeping server: No apache running, successfully restarted."
			else
				INSERT_MYSQL "Smokeping server: No apache running and failed to restart."
			fi
		fi
	fi
}

check_apache_process

function afterwards
{
	# define query time.
	QUERY_DATE=`date "+%Y-%m-%d"`

	# smokeping data for sysop
	sh $COMM_DIR/smokeping/fetch_smokeping_data.sh loss 10minute > /dev/null 2>&1
	sh $COMM_DIR/smokeping/fetch_smokeping_data.sh rtt 10minute > /dev/null 2>&1

	# ÿٳ01:1*ݸ£ǰűÿ3ӵһΣ01:1*3ݸ»ᣬⵥεдʧ
	if [ `date "+%H-%M"|cut -c 1-4` = "00-1" ];then
		sh $COMM_DIR/smokeping/fetch_smokeping_data.sh loss yesterday_1930_2230 "$QUERY_DATE" > /dev/null 2>&1
		sh $COMM_DIR/smokeping/fetch_smokeping_data.sh rtt yesterday_1930_2230 "$QUERY_DATE" > /dev/null 2>&1
		sh $COMM_DIR/smokeping/fetch_smokeping_data.sh loss month_1930_2230 "$QUERY_DATE" > /dev/null 2>&1
		sh $COMM_DIR/smokeping/fetch_smokeping_data.sh rtt month_1930_2230 "$QUERY_DATE" > /dev/null 2>&1
	fi
}

afterwards

