#!/bin/bash
#Date:2011-05-15
#Written by manifold.
#Readme:monitor the content of /etc/syslog-ng/syslog-ng.conf and /etc/logrotate.d/syslog-ng.
#Change:2012-05-03
#Readme:新增lograte进程的解锁。
#Change:2012-06-08
#Readme:新增手工编译的syslog-ng.3.3.2版本的管理。

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
source /usr/local/i386/*/auto/function.sh

# basic define
BASENAME=`basename $0`
SHORT_NAME=syslog-rotate
LOG_DIR=/var/log/sysop_manager

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || \
	[ -f $LOG_DIR/stop_run_$BASENAME ] || \
	[ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi
test_dept_game
[ "x$?" == "x0" ] && exit 1

# repository path
REPOS_NAME=comm_repos
COMM_DIR=/usr/local/i386/$REPOS_NAME
PUBLIC_DIR=/usr/local/i386/public_repos

# get ip address
IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
fi

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:] ]*$SHORT_NAME" && exit 1

BACKUP_DATE=`date +%Y-%m-%d-%H-%M-%S`
[ ! -d /var/log/system_log/ ] && mkdir -p /var/log/system_log/

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# define detect file function
Detect_file () {
	if [ ! -f $S_FILE ];then
		INSERT_MYSQL "no file $S_FILE."
	elif [ ! -f $D_FILE ];then
		/bin/cp $S_FILE $D_FILE
		if [ "$?" -eq 0 ];then
			INSERT_MYSQL "add file $D_FILE successful."
			CHANGE=yes
		else
			INSERT_MYSQL "add file $D_FILE failed."
		fi
	else
		S_MD5=`md5sum $S_FILE|awk '{print $1}'`
		D_MD5=`md5sum $D_FILE|awk '{print $1}'`
		if [ "$S_MD5" != "$D_MD5" ];then
			BACKUP_FILE="`echo $D_FILE|sed 's#/#_#g'`.$BACKUP_DATE"
			/bin/cp $D_FILE $LOG_DIR/$BACKUP_FILE
			cat $S_FILE > $D_FILE
			if [ "$?" -eq 0 ];then
				CHANGE=yes
				INSERT_MYSQL "$D_FILE changed success"
			else
				INSERT_MYSQL "$D_FILE changed failed"
			fi
		fi
	fi
}

#grep -wq "^${IPADDR}" $PUBLIC_DIR/initial_system/syslog/conf_v2.log && \
#switch_conf="v2"
#
#if [ "x${switch_conf}" = "x" ];then
#	IDCID=$(split_tag "idc_id" /home/dspeak/yyms/hostinfo.ini '=')
#	IDCID="${IDCID:-0}"
#	grep -wq "^sid=${IDCID}" $PUBLIC_DIR/initial_system/syslog/conf_v2.log && \
#	switch_conf="v2"
#fi

switch_conf="v2"

# detect syslog-ng package is exist or not.
if [ "$LINUX_VERSION" = "ubuntu" ];then
	PACKAGE_LINE=`dpkg -l|grep syslog-ng|wc -l`
	V3_LINE=`dpkg -l|grep "syslog-ng.* 3\."|wc -l`
elif [ "$LINUX_VERSION" = "centos" ];then
	PACKAGE_LINE=`rpm -qa|grep syslog-ng|wc -l`
	V3_LINE=`rpm -qa|grep syslog-ng-3|wc -l`
fi
if [ "$PACKAGE_LINE" -eq 0 ];then
	INSERT_MYSQL "syslog-ng have not installed."
	exit 1
fi

# syslog-ng configure file revision
SYS_NUM=`cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR .*syslog"|wc -l`
if [ "$SYS_NUM" -eq 0 ];then
	# syslog-ng配置检测（分手工编译版本和deb版本两种配置）
	if [ -f /usr/local/sbin/syslog-ng ];then
		SYSLOG_BIN=/usr/local/sbin/syslog-ng
		$SYSLOG_BIN --version|grep 3.3
		if [ "$?" -eq 0 ];then
			SYSLOG_V3_3="yes"
			S_FILE=$PUBLIC_DIR/initial_system/syslog/syslog-ng.conf${switch_conf}.${LINUX_VERSION}.v3.3
			D_FILE=/usr/local/etc/syslog-ng.conf
		else
			SYSLOG_V3_3="no"
		fi
	else
		if [ -f /sbin/syslog-ng ];then
			SYSLOG_BIN=/sbin/syslog-ng
		else
			SYSLOG_BIN=/usr/sbin/syslog-ng
		fi
		SYSLOG_FILE=`cat $PUBLIC_DIR/initial_system/syslog/syslog_version.log|grep -v ^#|awk '{if($1=="'"${IPADDR}"'") print $2}'|uniq|head -n 1`
		if [ "$SYSLOG_FILE" != "" ];then
			S_FILE=$PUBLIC_DIR/initial_system/syslog/$SYSLOG_FILE
		else
			# whether v3 syslog-ng
			if [ "$V3_LINE" -gt 0 ];then
				$SYSLOG_BIN --version|grep -q "3\.3"
				if [ "$?" -eq 0 ];then
					S_FILE=$PUBLIC_DIR/initial_system/syslog/syslog-ng.conf${switch_conf}.${LINUX_VERSION}.v3.3
				else
					S_FILE=$PUBLIC_DIR/initial_system/syslog/syslog-ng.conf${switch_conf}.${LINUX_VERSION}.v3
				fi
				OTHER_PARAMETER="--no-caps --no-caps"
			else
				S_FILE=$PUBLIC_DIR/initial_system/syslog/syslog-ng.conf${switch_conf}.${LINUX_VERSION}.v2
			fi
		fi
		D_FILE=/etc/syslog-ng/syslog-ng.conf
	fi

	unset CHANGE
	Detect_file
	if [ "$CHANGE" = "yes" ];then
		pkill syslog-ng
		wait
		rm -f /var/run/syslog-ng.pid > /dev/null 2>&1
		sleep 2
		$SYSLOG_BIN -p /var/run/syslog-ng.pid $OTHER_PARAMETER || INSERT_MYSQL "/etc/syslog-ng/syslog-ng.conf changed,but syslog-ng process restart failed!!!"
	fi
fi

# syslog-ng control file revision
if [ "$SYSLOG_V3_3" = "yes" ];then
	S_FILE=$PUBLIC_DIR/initial_system/syslog/syslog-ng_ctl.ubuntu.v3.3
	D_FILE=/etc/init.d/syslog-ng
	unset CHANGE
	Detect_file
	if [ "$CHANGE" = "yes" ];then
		INSERT_MYSQL "$D_FILE changed"
	fi
fi

function logrotate_v2() {
	S_FILE=$PUBLIC_DIR/initial_system/syslog/logrotate_attention
	[ "$LINUX_VERSION" = "ubuntu" ] && \
	D_FILE=/etc/logrotate.d/syslog-ng || \
	D_FILE=/etc/logrotate.d/syslog
	unset CHANGE
	Detect_file

	S_FILE=$PUBLIC_DIR/initial_system/syslog/logrotate.sh
	D_FILE=/etc/cron.daily/logrotate
	unset CHANGE
	Detect_file
	chmod 755 /etc/cron.daily/logrotate

	if ! grep '/etc/cron.daily/logrotate' /etc/crontab| grep -q '10 \*';then
		sed -i "/\/etc\/cron.daily\/logrotate/d" /etc/crontab
		echo '10 * * * *      root    /etc/cron.daily/logrotate' >> /etc/crontab
	fi
}

function logrotate_old() {
    CONF_FILE=$1
    if [ "x${CONF_FILE}" != "x" ];then
        S_FILE=$PUBLIC_DIR/initial_system/syslog/$CONF_FILE
    else
        S_FILE=$PUBLIC_DIR/initial_system/syslog/logrotate.d_syslog-ng.$LINUX_VERSION
    fi
	[ "$LINUX_VERSION" = "ubuntu" ] && \
	D_FILE=/etc/logrotate.d/syslog-ng || \
	D_FILE=/etc/logrotate.d/syslog
    unset CHANGE
    Detect_file
}


# logrotate configure file revision
LOG_NUM=`cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR .*logrotate"|wc -l`
if [ "$LOG_NUM" -eq 0 ];then
	LOGROTATE_FILE=$(grep -v "^#" $PUBLIC_DIR/initial_system/syslog/logrotate_version.log|\
    	grep -v ^#|awk '{if($1=="'"${IPADDR}"'") print $2}'|uniq|head -n 1)

	if [ "x${switch_conf}" != "x" ] && [ "x$LOGROTATE_FILE" = "x" ];then
		logrotate_v2
	else
		logrotate_old "$LOGROTATE_FILE"
	fi

    [ "$CHANGE" = "yes" ] && INSERT_MYSQL "$D_FILE update."
fi

# 检查logrotate进程数
UNLOCK_PROCESS () {
	/bin/bash $COMM_DIR/auto/unlock_process.sh "$1" "$2"
}
UNLOCK_PROCESS "/usr/sbin/logrotate /etc/logrotate.conf" "1800"


# 检查syslog-ng进程数
if [[ `ps aux|grep -v grep|grep "syslog-ng -p /var/run/syslog-ng.pid"|wc -l` -ne 1 ]] || [[ ! -f /var/run/syslog-ng.pid ]];then
	for PID in $(pidof syslog-ng);do
		kill -9 $PID
	done
	rm -f /var/run/syslog-ng.pid > /dev/null 2>&1
	sleep 2
	$SYSLOG_BIN -p /var/run/syslog-ng.pid $OTHER_PARAMETER
	INSERT_MYSQL "Detected syslog-ng process not equal 1,restart it."
fi


