#!/bin/bash
# common functions lib
#

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
set +H
export LANG="en_US.utf8"
export LC_ALL="en_US.utf8"

LOG_DIR=/var/log/sysop_manager
DOMAIN_NAME="manager.repos.yy.duowan.com"
HOSTFILE="/home/dspeak/yyms/hostinfo.ini"
REPOS_DIR="/usr/local/i386"

cat /etc/issue|grep -iqE "CentOS|Red" && export LINUX_VERSION="centos"
cat /etc/issue|grep -iqE "Ubuntu|Debian" && export LINUX_VERSION="ubuntu"
[ "x${LINUX_VERSION}" = "x" ] && exit 1

[ ! -d /dev/sysop ] && mkdir /dev/sysop

# OUTPUT_GREEN [-n] "string message"
function OUTPUT_GREEN() {
    if [ "$1" == '-n' ];then
        echo -en "\033[32;40;1m$2\033[0m"
    else
        echo -e "\033[32;40;1m$1\033[0m"
    fi
}

# OUTPUT_RED [-n] "string message"
function OUTPUT_RED() {
    if [ "$1" == '-n' ];then
        echo -en "\033[31;40;1m$2\033[0m"
    else
        echo -e "\033[31;40;1m$1\033[0m"
    fi
}

# OUTPUT_YELLOW [-n] "string message"
function OUTPUT_YELLOW(){
    if [ "$1" == '-n' ];then
        echo -en "\033[33;40;1mWarning:$2\033[0m"
    else
        echo -e "\033[33;40;1mWarning:$1\033[0m"
    fi
}

#split_tag "ip_isp_list" $HOSTFILE "="
function split_tag (){
    if [ $# -eq 3 ];then
        local query_tag=$1
        local query_file=$2
        local tag="$3"

        if [ -s $query_file ];then
            grep -Ew "^${query_tag}" $query_file |\
            tail -n1 |cut -d "$tag" -f 2-|sed  's/^ *//'
        fi
    fi
}

# total_isp
function total_isp () {

    local isp_info=$(split_tag "ip_isp_list" $HOSTFILE "=")

    local stack
    local which_isp
    local isp
    stack=$IFS
    IFS=,
    for which_isp in $isp_info;
    do
        isp="${which_isp##*:} ${isp}"
    done
    IFS=$stack

    [ "x${isp}" != "x" ] && echo $isp

}

# ip_from_isp "CNC"
function ip_from_isp () {
    local ISP=$1

    split_tag "ip_isp_list" $HOSTFILE "=" | \
    awk -F "=|,|:" '{for(i=1;i<=NF;i++){if($i == "'${ISP}'"){print $(i-1)}}}'

}

function isp_from_ip () {
    local IP
    IP=$1

    split_tag "ip_isp_list" $HOSTFILE "=" | \
    awk -F "=|,|:" '{for(i=1;i<=NF;i++){if($i == "'${IP}'"){print $(i+1)}}}'
}

function get_one_ip () {

    ifconfig | grep -m1 "inet addr"| \
    awk -F ':' '{print $2}'|awk '{print $1}'
}

function get_global_ip () {
    ifconfig | grep "inet addr"| grep -v "127.0.0.1" | \
    awk -F ':' '{print $2}'|awk '{print $1}'
}

function get_all_ip () {
    ip addr list |sed -n "s/\(.*inet \)\([0-9\.]*\).*/\2/p"
}

function get_repos () {
    local which_local
    local repos

    for which_local in $(ls --color=never -d /usr/local/i386/*);
    do
        [ ! -d $which_local ] && continue
        repos="${which_local##*/}"
        [ "x${repos}" = "xpublic_repos" ] && continue
        echo $repos
    done
}

function get_internal_ip () {
    get_global_ip | grep -E '^10\.'
}

function get_external_ip () {
    get_global_ip | grep -vE '^10\.|^192.168'
}

function select_log_ip () {

    local LOG_SERVER_TEL="183.57.81.207"
    local LOG_SERVER_CNC="122.13.201.143"
    local LOG_SERVER_INTERNAL="10.20.81.93"
    local NIC
    local ISP
    local LOCAL_IP
    local LOG_SERVER

    NIC=$(route -n |grep '^0.0.0.0'|awk '{print $NF}')
    LOCAL_IP=$(get_one_ip)

    ISP=$(isp_from_ip "$LOCAL_IP")
    [ "x${ISP}" = "x" ] && return

    case $ISP in
        "CTL")
            LOG_SERVER=$LOG_SERVER_TEL
            ;;
        "CNC")
            LOG_SERVER=$LOG_SERVER_CNC
            ;;
        "INTRANET")
            LOG_SERVER=$LOG_SERVER_INTERNAL
            ;;
        *)
        LOG_SERVER=$LOG_SERVER_TEL
            ;;
    esac
    echo $LOG_SERVER
}

function upload_logfile () {
    local context="$1"
    local log_ip=$(select_log_ip)
    local sid=$(split_tag "server_id" "$HOSTFILE" "=")

    if [ "x${log_ip}" = "x" ] || \
        [ ! -f "$context" ] || \
        [ "x${sid}" = "x" ];then
        return 1
    fi
    curl -m 5 --retry 3 -T "$context" \
    "http://${log_ip}:6081/${sid}_$(basename $context)"
    if [ $? -ne 0 ];then
        rsync --port=8731 "$context" \
        ${log_ip}::upload_log/${sid}_$(basename $context)
        return $?
    else
        return 0
    fi
}

function test_dept_game () {
    # 0: true; 1: false
    local e_code=1
    [ -f /etc/is_yygame ] && e_code=0 || e_code=1
    return $e_code
}

function noblock_lock () {
    local lockfile
    lockfile="${1:-/dev/sysop/$(basename $0).lock}"
    exec 200<>$lockfile
    flock -n -e 200 || { echo "$0 Already Running, exit!";exit 1; }
}

function noblock_unlock () {
    local lockfile
    lockfile="${1:-/dev/sysop/$(basename $0).lock}"
    exec 200<>$lockfile
    flock -u 200
    rm -f $lockfile
}






