#!/bin/bash
#Date:2011-07-21
#Written by manifold.
#Readme:host sign in function.
#Change:2012-05-08
#Readme:新增ipmitool的维护以及BMC管理IP的上报。
#Change:2012-06-28
#Readme:新增ip mac的对照关系。

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
export LANG=C
export LC_ALL=C

# check linux version
cat /etc/issue|grep -i -E "CentOS|Red" && LINUX_VERSION="centos"
cat /etc/issue|grep -i -E "Ubuntu" && LINUX_VERSION="ubuntu"
if [ "$LINUX_VERSION" = "" ];then exit 1;fi

# basic define
BASENAME=`basename $0`
LOG_DIR=/var/log/sysop_manager

if [ ! -d $LOG_DIR ];then
	mkdir -p $LOG_DIR
fi

# repository path
REPOS_NAME=comm_repos
COMM_DIR=/usr/local/i386/$REPOS_NAME
PUBLIC_DIR=/usr/local/i386/public_repos

REPOS_DB_ADDRESS_FILE=$LOG_DIR/repos_db_address.log

IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
fi

if [ ! -f $REPOS_DB_ADDRESS_FILE ] || [ ! -s $REPOS_DB_ADDRESS_FILE ];then
	svn info $COMM_DIR|awk -F ':' '{if($1~/URL/) print $3}'|tr -d '\/' > $REPOS_DB_ADDRESS_FILE
fi

MYSQL_ADDRESS=`cat $REPOS_DB_ADDRESS_FILE`
DEPARTMENT="yy"

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# different from other INSERT_MYSQL function
WGET_LOG_FILE=$LOG_DIR/host_sign_in_wget.log
HOST_SIGN_IN () {
	ACTION="$1"
	if [ "$ACTION" = "ip_mac" ];then
		wget --http-user=manifold --http-passwd=dw_manifold --output-document=$WGET_LOG_FILE --timeout=8 --tries=1 --quiet "http://$MYSQL_ADDRESS:63217/host_sign_in_ip_mac.php?ip=$IFCONFIG_IP&mac=$IFCONFIG_IP_MAC&NET_CARD_PORT=$NET_CARD_PORT&NET_CARD_NAME=$NET_CARD_NAME&NET_CARD_SPEED=$NET_CARD_SPEED"
	elif [ "$ACTION" = "ip_bmc" ];then
		wget --http-user=manifold --http-passwd=dw_manifold --output-document=$WGET_LOG_FILE --timeout=8 --tries=1 --quiet "http://$MYSQL_ADDRESS:63217/host_sign_in.php?ip=$IPADDR&bmc_ip_address=$BMC_IP_ADDRESS&department=$DEPARTMENT"
	fi
}

# about ipmi function
cat $PUBLIC_DIR/initial_system/snmp/ext/bmc_ip_address.log|grep -v ^#|grep $IPADDR
if [ "$?" -eq 0 ];then
	# check ipmotool package
	if [ "$LINUX_VERSION" = "centos" ];then
		PACKAGE_LINE=`rpm -qa|grep ipmitool|wc -l`
		if [ "$PACKAGE_LINE" -eq 0 ];then
			yum -y install ipmitool
			if [ "$?" -eq 0 ];then
				INSERT_MYSQL "$LINUX_VERSION install ipmitool successful."
			else
				INSERT_MYSQL "$LINUX_VERSION install ipmitool failed."
			fi
		fi
	elif [ "$LINUX_VERSION" = "ubuntu" ];then
		PACKAGE_LINE=`dpkg -l|grep ipmitool|wc -l`
		if [ "$PACKAGE_LINE" -eq 0 ];then
			aptitude -y install ipmitool
			if [ "$?" -eq 0 ];then
				INSERT_MYSQL "$LINUX_VERSION install ipmitool successful."
			else
				INSERT_MYSQL "$LINUX_VERSION install ipmitool failed."
			fi
		fi
	fi

	# check module
	IPMI_MOD_LINE=`lsmod|grep ipmi|wc -l`
	if [ "$IPMI_MOD_LINE" -lt 5 ];then
		modprobe ipmi_watchdog
		modprobe ipmi_poweroff
		modprobe ipmi_devintf
		modprobe ipmi_msghandler
		modprobe ipmi_si
	fi

	# check ipmi configure
	if [ "$PACKAGE_LINE" -gt 0 ];then
		CHECK_IPMI_INFO_LINE=`ipmitool lan print|grep -E "IP Address Source       : Static Address|IP Address              : |Default Gateway IP      : "|grep -v "0.0.0.0"|wc -l`
		if [ "$CHECK_IPMI_INFO_LINE" -lt 3 ];then
			INSERT_MYSQL "ipmi configure error,please check."
		fi
		BMC_IP_ADDRESS=`ipmitool lan print|grep "IP Address              : "|awk -F ':' '{print $2}'|column -t`
	fi
else
	BMC_IP_ADDRESS="0.0.0.0"
fi

SEND_MESSAGE_DETECT () {
	ACTION="$1"
	SELECT_ROLE="mysql"
	address_array=($(cat $COMM_DIR/initial_system/repos_server.log|grep -v $MYSQL_ADDRESS|grep "$SELECT_ROLE"|awk '{print $1}'))
	until [ "${#address_array[*]}" -eq 0 ];do
		address_array=(${address_array[*]})
		address_array_num=${#address_array[*]}
		xuhao=$((RANDOM%address_array_num))
		ADDRESS=${address_array[$xuhao]}
		unset address_array[$xuhao]
		MYSQL_ADDRESS="$ADDRESS"
		echo "MESSAGE_ADDRESS:$MYSQL_ADDRESS"
		HOST_SIGN_IN $ACTION
		RESULT=`cat $WGET_LOG_FILE`
		if [ "$RESULT" = "0" ];then
			echo "$MYSQL_ADDRESS" > $REPOS_DB_ADDRESS_FILE
			break
		fi
	done
}

SCRIPT_NAME="$BASENAME"
MESSAGE="host sign in,ip address $IPADDR."
echo "`date +%Y-%m-%d-%T` $SCRIPT_NAME $MESSAGE" >> $LOG_DIR/sysop_manager.log 2>&1
sleep $((RANDOM%20))

function HOST_SIGN_IN_ACTION () {
	ACTION="$1"
	HOST_SIGN_IN $ACTION
	RESULT=`cat $WGET_LOG_FILE`
	[ "$RESULT" != "0" ] && SEND_MESSAGE_DETECT $ACTION

}

# ip bmc地址注册
HOST_SIGN_IN_ACTION ip_bmc

# 相关信息获取
# base64超过76个字符就断行了，断行后wget url数据没有返回
#        -w, --wrap=COLS
#               Wrap encoded lines after COLS character (default 76).  Use 0 to disable line wrapping.
NET_CARD_PORT="eth0"
NET_CARD_PORT=`printf "$NET_CARD_PORT"|base64 -w0`
NET_CARD_NAME=`lspci|grep "Ethernet controller"|awk -F 'Ethernet controller:' '{print $2}'|sort -u|head -n 1|sed -r 's/^[ ]{1,}//g'`
NET_CARD_NAME=`printf "$NET_CARD_NAME"|base64 -w0`
NET_CARD_SPEED=`ethtool eth0|awk '{if($1~/Speed:/) print $2}'`
NET_CARD_SPEED=`printf "$NET_CARD_SPEED"|base64 -w0`

# 注册IP地址对应的MAC到数据库
for IFCONFIG_IP in `ifconfig|grep "inet addr"|awk '{print $2}'|awk -F ':' '{print $2}'|grep -v ^127.`
do
	IFCONFIG_IP_MAC=`ifconfig|grep $IFCONFIG_IP -B 1|grep HWaddr|awk '{print $5}'`
	if [ "$IFCONFIG_IP" != "" ] && [ "$IFCONFIG_IP_MAC" != "" ];then
		
		# ip mac地址及其他相关信息注册
		HOST_SIGN_IN_ACTION ip_mac
		
	fi
done

