#!/bin/bash
# Date 2013-01-06
# Written by wuhaiting
# Readme 分配监控/etc/resolv.conf
# Date 2013-03-19
# 退出前检查以防/etc/resolv.conf为空

source /usr/local/i386/*/auto/function.sh
noblock_lock
trap "noblock_unlock;" 0 1 2 9 15

# basic define
BASENAME=`basename $0`
SHORT_NAME=mk_resolv
LOG_DIR=/var/log/sysop_manager
COMM_DIR=/usr/local/i386/*
PUBLIC_DIR=/usr/local/i386/public_repos
DNS_ROOM_FILE=$PUBLIC_DIR/initial_system/resolv.conf/dns_from_cmdb.log
ENABLE_RESOLV_V2=$PUBLIC_DIR/initial_system/resolv.conf/enable_resolv_v2

export PATH=/usr/sbin:/usr/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin
export LANG=C
export LC_ALL=C
# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

if [ -s /home/dspeak/yyms/hostinfo.ini ];then
    IDC_ID=$(grep "^room_id" /home/dspeak/yyms/hostinfo.ini| \
        awk -F "=" '{print $2}')
    [ "x${IDC_ID}" != "x" ] && grep -wq "^${IDC_ID}" $ENABLE_RESOLV_V2 && \
    { /bin/bash $COMM_DIR/auto/mk_resolv_v2.sh;exit; }
fi

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# get ip address
IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
IPADDR_ALL=`/sbin/ifconfig|grep "inet addr"|awk '{print $2}'|awk -F ":" '{print $2}'|grep -v -E "^127\.0|^10\.|^192\.|^172\."`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
fi

# exclude list
IPADDR_ALL_ex=$(echo $IPADDR_ALL|tr -s ' ' '|')
grep "$SHORT_NAME" $COMM_DIR/initial_system/exclude_list.log | grep -wE "$IPADDR_ALL_ex" && exit 1

# 如果是内网机器，直接exit
[[ "$IPADDR_ALL" == "" ]] && exit 0

which dig
if [ "$?" -ne 0 ];then
	if [ "`cat /etc/issue|grep -iq ubuntu && echo 0`" == "0" ];then
		apt-get -y install dnsutils
	fi
fi

# 临时文件
TMP_FILE1=`mktemp /tmp/tmp_file.XXXXX`

function default_resolv_server () {
	if [[ ! -s /etc/resolv.conf ]] || [[ "`cat /etc/resolv.conf | grep '^nameserver' | wc -l`" == "0" ]];then
		# 使用万能DNS解析服务器
		echo -e "nameserver 114.114.114.114\nnameserver 114.114.115.115\nnameserver 8.8.8.8" > /etc/resolv.conf
	fi
	echo "$BASENAME" >> $LOG_DIR/sysop_manager.log
	cat /etc/resolv.conf >> $LOG_DIR/sysop_manager.log
}

# 如果cmdb里面没有相关的记录，使用就的机制匹配
function detect_resolv_file () {
    ISP_FILE=/var/log/sysop_manager/isp.log
    if [ -s "$ISP_FILE" ];then
        ISP_LINE=`cat $ISP_FILE|grep -v -E -w "10|11"|sort -u|sed '/^$/d'|wc -l`
        ISP_NUM=`cat $ISP_FILE|grep -v -E -w "10|11"|sort -u|sed '/^$/d'`
        #single line...telecom,unicom,edu,bgp,cmcc,oversea.
        if [ "$ISP_LINE" == "1" ];then
            case $ISP_NUM in
                "4")
                    S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_telecom;;
                "5")
                    S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_unicom;;
                "6")
                    S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_double_and_bgp;;
                "7")
                    S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_edu;;
                "8")
                    S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_cmcc;;
				14|16)
					S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_haiwai;;
                *)
                    S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_telecom;;
            esac
        elif [ "$ISP_LINE" == "2" ];then
            # unicom and cmcc server
			echo $ISP_NUM|grep "5" && echo $ISP_NUM|grep "8"
            if [ "$?" == "0" ];then
                S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_unicom_and_cmcc
            else
                # telecom and unicom
                S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_double_and_bgp
            fi
        elif [ "$ISP_LINE" == "3" ];then
            S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_three_line
        else
            S_FILE=$PUBLIC_DIR/initial_system/resolv.conf/resolv.conf_telecom
        fi
    fi

	TMP_FILE=`cat $PUBLIC_DIR/initial_system/resolv.conf/resolv_version.log|grep -v ^#|awk '{if($1=="'"${IPADDR}"'") print $1}'|uniq|head -n 1`
	if [ "$TMP_FILE" != "" ];then
		return
	fi

	[ "x${S_FILE}" != "x" ] && cat $S_FILE
}

# 防止/etc/resolv.conf有其他字符
if [ "`cat /etc/resolv.conf|grep -v -E '^$|^#' |grep -v '^nameserver' |wc -l `" != "0" ];then
	cat /etc/resolv.conf|grep -E '^$|^#|^nameserver' > $TMP_FILE1
	[[ -s $TMP_FILE1 ]] && cat $TMP_FILE1 > /etc/resolv.conf
	cat /dev/null > $TMP_FILE1
fi

MD5_NEW=`md5sum $DNS_ROOM_FILE|awk '{print $1}'`
MD5_OLD=`md5sum $LOG_DIR/dns_from_cmdb.log | awk '{print $1}'`

if [[ "$MD5_NEW" != "$MD5_OLD" ]] && [[ -s $DNS_ROOM_FILE ]];then
	cp -f $DNS_ROOM_FILE $LOG_DIR/dns_from_cmdb.log
	CHANGE=yes
fi

OLD_RESOLV=`cat /etc/resolv.conf|grep nameserver|awk '{print $2}'|tr -s "\n" " "`

if [ -s /home/dspeak/yyms/hostinfo.ini ];then
	ROOM_ID=`cat /home/dspeak/yyms/hostinfo.ini|grep "^room_id"|awk -F "=" '{print $2}'`
else
	# 机器刚上线，yyms同步未完成，采用就的方式检测
	detect_resolv_file | grep "^nameserver" > $TMP_FILE1
	[[ -s $TMP_FILE1 ]] && cp -f $TMP_FILE1 /etc/resolv.conf
	rm $TMP_FILE1
	default_resolv_server
	exit 0
fi

NEW_RESOLV=`cat $DNS_ROOM_FILE|grep "^$ROOM_ID\>"|awk '{print $3}'|tr -s "," " "`
detect_resolv_file | grep "^nameserver" > $TMP_FILE1
TMP_RESOLV=`cat $TMP_FILE1|awk '{print $2}'|tr -s "\n" " "`
NEW_RESOLV_ALL=`echo $NEW_RESOLV $TMP_RESOLV`

# 如果cmdb里面没有机房dns信息，采用旧策略
if [ "$NEW_RESOLV" == "" ];then
	detect_resolv_file | grep "^nameserver" > $TMP_FILE1
	[[ -s $TMP_FILE1 ]] && cp -f $TMP_FILE1 /etc/resolv.conf

	for a in `cat /etc/resolv.conf|awk '{print $2}'`;do
		dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.baidu.com > /dev/null 2>&1 && continue;
		dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.sina.com.cn >/dev/null 2>&1 && continue;
		FAIL_RESOLV=`echo $FAIL_RESOLV $a`
	done
	unset a
	if [ "`echo $FAIL_RESOLV|awk '{print NF}'`" == "`cat /etc/resolv.conf|grep -v -E '^$|^#' | wc -l`" ];then
		INSERT_MYSQL "resolv_file all fail,check"
		# 使用万能DNS解析服务器
		echo -e "nameserver 114.114.114.114\nnameserver 114.114.115.115\nnameserver 8.8.8.8" > /etc/resolv.conf
	fi
	rm $TMP_FILE1
	default_resolv_server
	exit 0
fi

# 检查是否使用了未授权的DNS地址
if [ "$OLD_RESOLV" != "" ];then
	for a in `echo $OLD_RESOLV`;do
		echo $NEW_RESOLV_ALL|grep -wq $a >/dev/null 2>&1
		[[ "$?" -ne 0 ]] && CHANGE=yes
	done
	unset a
else
	CHANGE=yes
fi

# 检查是否使用CMDB里的信息，每天只是check一遍
if [ "`date +%H:%M`" == "15:00" ];then
	for a in `echo $OLD_RESOLV`;do
		echo $NEW_RESOLV|grep -wq $a >/dev/null 2>&1
		[[ $? -ne 0 ]] && CHANGE=yes
	done

	[[ "`echo $OLD_RESOLV|awk '{print NF}'`" != "`echo $NEW_RESOLV|awk '{print NF}'`" ]] && CHANGE=yes
	unset a
fi


# 检查现有dns服务器解析是否有问题。
if [ "$CHANGE" != "yes" ];then
	for a in `cat /etc/resolv.conf|awk '{print $2}'`;do
		dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.baidu.com > /dev/null 2>&1 && continue;
		dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.sina.com.cn >/dev/null 2>&1 && continue;
		FAIL_RESOLV=`echo $FAIL_RESOLV $a`
		CHANGE=yes
	done
	if [[ "$FAIL_RESOLV" != "" ]] && [[ "$CHANGE" == "yes" ]];then
		FAIL_NUM=`echo $FAIL_RESOLV|awk '{print NF}'`
		# 有2个或以上的不可用dns解析
		if [ "$FAIL_NUM" == "1" ];then
			sed -i "/nameserver.*$FAIL_RESOLV/d" /etc/resolv.conf
			rm $TMP_FILE1
			default_resolv_server
			exit 0
		fi
	fi
	unset a FAIL_NUM FAIL_RESOLV
fi

# ok,到此的话需要更新/etc/resolv.conf，但是为了保证所有都能够使用还是需要用dig检查
if [ "$CHANGE" == "yes" ];then
	cat /dev/null > $TMP_FILE1
	for a in `echo $NEW_RESOLV`;do
		dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.baidu.com > /dev/null 2>&1 || IS_OK=no
		dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.sina.com.cn > /dev/null 2>&1 || IS_OK=no
		dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.duowan.com > /dev/null 2>&1 || IS_OK=no
		[[ "$IS_OK" != "no" ]] && echo "nameserver $a" >> $TMP_FILE1
	done
	# 如果是3线的机器，不能够只是使用移动的dns
	ISP_NUM=`cat $LOG_DIR/isp.log|grep -v -E "10|11" |wc -l`
	if [[ "$ISP_NUM" -ge 3 ]] && [[ "`cat $TMP_FILE1|wc -l`" -le 1 ]];then
		detect_resolv_file | grep "^nameserver" > $TMP_FILE1
		[[ -s $TMP_FILE1 ]] && cp -f $TMP_FILE1 /etc/resolv.conf
		cat /dev/null > $TMP_FILE1
	fi

	if [ -s $TMP_FILE1 ];then
		cp -f $TMP_FILE1 /etc/resolv.conf
	else
		#cmdb里面提供dns不可用，采用旧的策略，先写再测试
		detect_resolv_file | grep "^nameserver" > $TMP_FILE1
		[[ -s $TMP_FILE1 ]] && cp -f $TMP_FILE1 /etc/resolv.conf
		cat /dev/null > $TMP_FILE1

		for a in `cat /etc/resolv.conf|awk '{print $2}'`;do
			dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.baidu.com > /dev/null 2>&1 && continue;
			dig +time=1 +tries=1 +retry=1 +nofail +short @$a www.sina.com.cn >/dev/null 2>&1 && continue;
			FAIL_RESOLV=`echo $FAIL_RESOLV $a`
		done
		if [ "`echo $FAIL_RESOLV|awk '{print NF}'`" == "`cat /etc/resolv.conf|grep -v -E '^$|^#' | wc -l`" ];then
			INSERT_MYSQL "resolv_file all fail,check"
			# 使用万能DNS解析服务器
			echo -e "nameserver 114.114.114.114\nnameserver 114.114.115.115\nnameserver 8.8.8.8" > /etc/resolv.conf
		fi
	fi
fi

rm $TMP_FILE1
default_resolv_server
exit 0

