#!/bin/bash
# Rewrite nameserver check scripts
# (1). If you want to configure nameserver about idc, go to CMDB.
# (2). If CMDB's nameserver all resolv fail. All of rule will be
#       imported in /etc/hosts in enable localdns.
#

BASENAME=$(basename $0)
SHORT_NAME=mk_resolv
COMM_DIR=/usr/local/i386
PUBLIC_DIR=${COMM_DIR}/public_repos
DNS_ROOM_FILE=$PUBLIC_DIR/initial_system/resolv.conf/dns_from_cmdb.log
NAME_CONTEXT=$PUBLIC_DIR/initial_system/resolv.conf/all_context_rule.log
source ${COMM_DIR}/*/auto/function.sh
HOSTSLOCK="/etc/.hosts_lock"
IPADDR=$(get_one_ip)
IPADDR_EXT=$(get_external_ip)
IPADDR_ALL=$(get_all_ip|grep -v '^127\.')
RESOLVFILE="/etc/resolv.conf"
DEFAULT_NS="114.114.114.114 223.5.5.5"
IDC_ID=$(split_tag 'idc_id' "$HOSTFILE" '=')

export CHANGE="no"
export old_ns=""

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || \
    [ -f $LOG_DIR/stop_run_$BASENAME ] || \
    [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

function check_is_run() {
    # If the server only have internal ip, exit
    [ "x${IPADDR_EXT}" = "x" ] && exit 1

    unset exclude_ip
    for ck_ip in $IPADDR_ALL;
    do
        if [ "x${exclude_ip}" = "x" ];then
            exclude_ip="$ck_ip"
        else
            exclude_ip="${exclude_ip}|${ck_ip}"
        fi
    done
    grep "$SHORT_NAME" $COMM_DIR/*/initial_system/exclude_list.log| \
    grep -wE "$exclude_ip" && exit 1

    grep -wE "$exclude_ip" $PUBLIC_DIR/initial_system/resolv.conf/resolv_version.log && exit 1

}

function set_ns() {
    ns_entry="$@"
    > $RESOLVFILE
    for ns in $ns_entry;
    do
        echo "nameserver $ns" >> $RESOLVFILE
    done
    sync
}

function check_ns_valid() {
    local check_nss="$@"
    local valid_ns
    local return_code=0

    for ns in $check_nss;
    do
        unset fail_ns
        for ck_host in "www.baidu.com" "www.sina.com.cn" "www.duowan.com";
        do
            dig +time=1 +tries=1 +retry=1 +nofail +short \
            @${ns} ${ck_host} >/dev/null 2>&1 || { fail_ns="yes";break; }
        done
        [ "x${fail_ns}" != "xyes" ] && \
        valid_ns="${valid_ns} $ns" || return_code=1
    done
    [ "x${valid_ns}" = "x" ] && return_code=2
    echo "valid_ns:${valid_ns}"
    return $return_code
}

function select_old_ns() {
    RESOLV_DIR="$PUBLIC_DIR/initial_system/resolv.conf/resolv_v2"
    ISP_FILE=/var/log/sysop_manager/isp.log
    ISP_COUNT=$(grep -vE "^$|10|11" $ISP_FILE|wc -l)
    ISP_CS=$(grep -vE "10|11" $ISP_FILE)

    if [ "$ISP_COUNT" = "1" ];then
        case $ISP_NUM in
            "4")
                S_FILE=$RESOLV_DIR/resolv.conf_telecom;;
            "5")
                S_FILE=$RESOLV_DIR/resolv.conf_unicom;;
            "6")
                S_FILE=$RESOLV_DIR/resolv.conf_double_and_bgp;;
            "7")
                S_FILE=$RESOLV_DIR/resolv.conf_edu;;
            "8")
                S_FILE=$RESOLV_DIR/resolv.conf_cmcc;;
            14|16)
                S_FILE=$RESOLV_DIR/resolv.conf_haiwai;;
            *)
                S_FILE=$RESOLV_DIR/resolv.conf_telecom;;
        esac
    elif [ "$ISP_COUNT" = "2" ];then
        # unicom and cmcc server
        if [ "$(printf "$ISP_CS"|grep -cE "5|8")" == "2" ];then
            S_FILE=$RESOLV_DIR/resolv.conf_unicom_and_cmcc
        else
            # telecom and unicom
            S_FILE=$RESOLV_DIR/resolv.conf_double_and_bgp
        fi
    elif [ "$ISP_COUNT" = "3" ];then
        S_FILE=$RESOLV_DIR/resolv.conf_three_line
    else
        S_FILE=$RESOLV_DIR/resolv.conf_telecom
    fi
    [ "x${S_FILE}" != "x" ] && \
    echo "valid_ns:$(awk '{print $2}' $S_FILE|tr '\n' ' ')"
}

function select_new_ns() {
    if [ "x${IDC_ID}" != "x" ];then
        new_ns=$(grep "^${IDC_ID}\>" $DNS_ROOM_FILE | \
            awk '{print $3}'| tr -s "," " ")
        echo "valid_ns:${new_ns}"
    fi
}

function select_ns() {
    nss=$(select_new_ns|awk -F ':' '{if($0 ~ /valid_ns/){print $2}}')
    [ "x${nss}" = "x" ] && \
    nss=$(select_old_ns |awk -F ':' '{if($0 ~ /valid_ns/){print $2}}')
    echo "valid_ns:$nss"
}

function import_to_hostfile() {
    cp -f /etc/hosts $HOSTSLOCK
    cat $NAME_CONTEXT >> /etc/hosts
    sync
    pidof nscd >/dev/null 2>&1 && nscd -i hosts
    chattr +i /etc/hosts
    chattr +i $HOSTSLOCK
}

function unpack_from_hostfile() {
    chattr -i /etc/hosts
    chattr -i $HOSTSLOCK
    [ -s $HOSTSLOCK ] && cat $HOSTSLOCK > /etc/hosts
    rm -f $HOSTSLOCK
    pidof nscd >/dev/null 2>&1 && nscd -i hosts
}

function check_nameserver_file_change() {
    MD5_OLD=$(md5sum $LOG_DIR/dns_from_cmdb.log|awk '{print $1}')
    MD5_NEW=$(md5sum $DNS_ROOM_FILE | awk '{print $1}')

    if [ "${MD5_OLD}" != "${MD5_NEW}" ] && [ -s $DNS_ROOM_FILE ];then
        cp -f $DNS_ROOM_FILE $LOG_DIR/dns_from_cmdb.log
        CHANGE="yes"
        return 0
    fi
    return 1
}

function check_illegal_ns() {
    # If any untrust nameserver in /etc/hosts ?
    local pickup_ns="$@"
    if [ "$(date +%M)" = "00" ] && [ "x${pickup_ns}" != "x" ];then
        for ns in $old_ns;
        do
            echo "$pickup_ns" | grep -wq "$ns" || CHANGE="yes"
        done
        i=$(echo "$pickup_ns"|awk '{print NF}')
        v=$(echo "$old_ns"|awk '{print NF}')
        [ "$i" != "$v" ] && CHANGE="yes"
        rm -f /data/core_files/core-dig-*
    fi
    [ "x${CHANGE}" = "xyes" ] && return 1 || return 0
}

function main() {
    check_is_run

    old_ns=$(grep -vE "^$|^#" /etc/resolv.conf | \
        grep -E "[0-9]{1,3}\.[0-9]{1,3}\."| \
        awk '{print $NF}'| tr -s '\n' ' ')
    [ "x${old_ns}" = "x" ] && CHANGE="yes"

    check_ns_valid "$old_ns" > /dev/null || CHANGE="yes"

    check_nameserver_file_change && CHANGE="yes"

    nss=$(select_ns |awk -F ':' '{if($0 ~ /valid_ns/){print $2}}')
    check_illegal_ns "$nss" || CHANGE="yes"

    if [ "x${CHANGE}" != "xyes" ];then
        [ -f $HOSTSLOCK ] && unpack_from_hostfile
    else
        nss=$(select_ns|awk -F ':' '{if($0 ~ /valid_ns/){print $2}}')
        [ "x${nss}" = "x" ] && nss="$DEFAULT_NS"

        nss_last=$(check_ns_valid "$nss")
        new_ns_status="$?"

        if [ "x${new_ns_status}" = "x0" ] || \
            [ "x${new_ns_status}" = "x1" ];then
            nss_last=$(echo "$nss_last" | \
                awk -F ':' '{if($0 ~ /valid_ns/){print $2}}')
            set_ns "$nss_last"
            [ -f $HOSTSLOCK ] && unpack_from_hostfile

        elif [ "x${new_ns_status}" = "x2" ];then
            # All of nameserver has fail. we need /etc/hosts file
            [ ! -f $HOSTSLOCK ] && import_to_hostfile
        fi

    fi
}

main



