#!/bin/bash
#Date:2012-02-22
#Written by manifold.
#Readme:check arp info.
#change:2012-07-13
#Readme:原有规则是写到iptables里面，作用到的是3层，但arp是底层协议，用于ip地址到物理地址的转换，所以，需要用arp命令去绑定。

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games

# basic define
BASENAME=`basename $0`
SHORT_NAME=mon_arp
LOG_DIR=/var/log/sysop_manager

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

# repository path
REPOS_NAME=comm_repos
COMM_DIR=/usr/local/i386/$REPOS_NAME

# get ip address
IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
fi

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:]]*$SHORT_NAME" && exit 1

# define insert alert function
INSERT_MYSQL () {
    sleep $((RANDOM%15))
	/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# 检测是否有arping工具，arping由iputils-arping提供
which arping
if [ "$?" -ne 0 ];then
	#INSERT_MYSQL "ARP:no arping tool,please install."
	exit 1
fi

function CHECK_ARP () {

	# 检测默认路由是否正常，只有默认路由正常才能获取到正确的网关
	# 有些ubuntu主机偶尔会丢失默认路由，或者出现多个默认路由等情况
	# /etc/network/interfaces配置了错误的网关，在network restart后是会有两个网关的，过几分钟后网络才会断掉

	# 获取默认网关IP
	GATEWAY_IP_DEFAULT_ORI=`route -n|grep -w "0\.0\.0\.0.*0\.0\.0\.0"|awk '{print $2}'`
	GATEWAY_IP_DEFAULT=`echo "$GATEWAY_IP_DEFAULT_ORI"|awk -F '.' '{if($1>0 && $1<256 && $2>=0 && $2<256 && $3>=0 && $3<256 && $4>=0 && $4<256) print $0}'`
	if [ "$GATEWAY_IP_DEFAULT" = "" ];then
		INSERT_MYSQL "ARP:GATEWAY_IP_DEFAULT error,GATEWAY_IP_DEFAULT_ORI:$GATEWAY_IP_DEFAULT_ORI"
		exit 1
	fi

	# 获取网通网关IP（若有）
	GATEWAY_IP_CNC_ORI=`route -n|awk '{print $2}'|sort|uniq -c|awk '{if($1>600) print $2}'`
	GATEWAY_IP_CNC=`echo "$GATEWAY_IP_CNC_ORI"|awk -F '.' '{if($1>0 && $1<256 && $2>=0 && $2<256 && $3>=0 && $3<256 && $4>=0 && $4<256) print $0}'`

	# 分别绑定电信、网通网关及其对应MAC地址
	for GATEWAY_IP in $GATEWAY_IP_DEFAULT #$GATEWAY_IP_CNC
	do

		# 对于arping的-c 8参数，ubuntu会ping 8次，而centos 6会ping 4次
		# 判断是否有arp冲突
		arping -b $GATEWAY_IP -c 8 > $LOG_DIR/arping.log 2>&1

		# 若arping探测网关IP无响应，说明可能换了交换机（IP不变，但MAC变），此时需要解绑MAC
		cat $LOG_DIR/arping.log|grep -E "Received 0 response|0 packets received"
		if [ "$?" -eq 0 ];then
			arp -na|grep PERM|tr -d \(\)|awk 'NR>=1{system("arp -d " $2)}'
			INSERT_MYSQL "ARP:$GATEWAY_IP:unbind all mac address from arp table,reason:arping received 0 response"
			exit 1
		fi

		# 若arping有响应，根据response数据做相关检测
		GATEWAY_IP_MAC_LINE=`cat $LOG_DIR/arping.log|grep "Unicast reply from $GATEWAY_IP"|awk '{print $5}'|grep ".*:.*:.*"|sort -u|wc -l`
		if [ "$GATEWAY_IP_MAC_LINE" -eq 1 ];then
			# 获取并验证mac的正确性
			GATEWAY_IP_MAC=`cat $LOG_DIR/arping.log|grep "Unicast reply from $GATEWAY_IP"|awk '{print $5}'|tr -d \[\]|sort -u`
			MAC_CHAR_NUM=`echo "$GATEWAY_IP_MAC"|grep ":"|wc -c`
			if [ "$MAC_CHAR_NUM" -ne 18 ];then
				INSERT_MYSQL "ARP:$GATEWAY_IP:mac error,not equal 18 char"
				exit 1
			fi
		else
			INSERT_MYSQL "ARP:$GATEWAY_IP:GATEWAY_IP_MAC_LINE:$GATEWAY_IP_MAC_LINE,something error"
			exit 1
		fi

		# 检测是否已经做了网关的arp绑定
		ARP_BINDING_LINE=`arp -a -n|grep -i -w "$GATEWAY_IP.*PERM"|wc -l`
		# 若未绑定，进行绑定
		if [ "$ARP_BINDING_LINE" -eq 0 ];then
			arp -s $GATEWAY_IP $GATEWAY_IP_MAC && INSERT_MYSQL "ARP:$GATEWAY_IP:binding GATEWAY_IP:$GATEWAY_IP to GATEWAY_IP_MAC:$GATEWAY_IP_MAC"
		# 若已绑定，判断是否匹配，是否需要重新绑定（在刚换交换机时，用arping探测是通的，但若过几分钟就会不通）
		else
			# 若绑定的网关地址与现有MAC地址不匹配，解除绑定（若有冲突，无论绑定不绑定arp，arping都会出现两条不同的网关mac记录）
			ARP_BINDING_LINE=`arp -a -n|grep -i -w "$GATEWAY_IP.*$GATEWAY_IP_MAC.*PERM"|wc -l`
			if [ "$ARP_BINDING_LINE" -eq 0 ];then
				# 还没查清楚是什么问题，原来的语句是（顺序反了）：
				# arp -na|grep PERM
				# + grep --color=auto PERM
				# + arp -na
				# ? (221.204.232.97) at 54:89:98:c0:72:46 [ether] PERM on eth0
				arp -na|grep PERM|tr -d \(\)|awk 'NR>=1{system("arp -d " $2)}'
				# arpResult=$(arp -na|grep PERM|tr -d \(\))
				# awk 'NR>=1{system("arp -d " $2)}' <<< "$arpResult"
				INSERT_MYSQL "ARP:$GATEWAY_IP:unbind all mac address from arp table,reason:the mac from arp table not match to the mac from arping"
				exit 1
			fi
		fi

	done

}

function APR_UNBIND () {

	if arp -an | grep -iwq 'PERM';then
		arp -na|grep PERM|tr -d \(\)|awk 'NR>=1{system("arp -d " $2)}'
	fi
}

function ARP_IP_LIST () {
cat <<EOF
test
EOF
}

#EXECUTE_OR_NOT=`ARP_IP_LIST|grep -w $IPADDR|wc -l`
#if [ "$EXECUTE_OR_NOT" -gt 0 ];then
#	CHECK_ARP
#fi

#CHECK_ARP
#APR_UNBIND
