#!/bin/bash
#Date:2012-01-05
#Written by manifold.
#Readme:check some information.

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
export LANG=C
export LC_ALL=C

# basic define
BASENAME=`basename $0`
SHORT_NAME=mon_function
LOG_DIR=/var/log/sysop_manager

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

# repository path
REPOS_NAME=comm_repos
COMM_DIR=/usr/local/i386/$REPOS_NAME
PUBLIC_DIR=/usr/local/i386/public_repos

# get ip address
IPADDR=`/sbin/ifconfig|grep Bcast|grep -vE "inet addr:192.168|inet addr:172\.|inet addr:10\."|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
	IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
	INTERNAL_IP="yes"
	INTERNAL_PLATFORM="10.20.81.107"
fi

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:]]*$SHORT_NAME" && exit 1

cur_status=$(awk -F '=' '{if($0 ~ /^status/){print $2}}' /home/dspeak/yyms/hostinfo.ini)
wall_time_sec=$(date "+%s")
wall_time_min="$((wall_time_sec/60%60))"
wall_time_hour="$(((wall_time_sec+8*3600)/60/60%24))"

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

################################################################################
################################################################################

# ntpdate时间同步不单独放在/etc/crontab里面，改为半小时同步一遍
function ntp_time_sync () {
	API=$(ntpdate -u $(cat /var/log/sysop_manager/repos_db_address.log) 2>&1)
	if [ $? -eq 0 ];then
		echo $API >> $LOG_DIR/sysop_manager.log
		hwclock -w
		return
	fi
	# 从8个ip里面随机抽一个主，一个备
	api_address_array=($(cat $COMM_DIR/initial_system/repos_server.log|grep "$API_ROLE"|awk '{print $1}'))
	api_address_array=(${api_address_array[*]})
	api_address_array_num=${#api_address_array[*]}
	api_num_index1=$((RANDOM%api_address_array_num))
	API_ADDRESS1=${api_address_array[$api_num_index1]}
	api_num_index2=$((RANDOM%api_address_array_num))
	API_ADDRESS2=${api_address_array[$api_num_index2]}
	until [ "$API_ADDRESS1" != "$API_ADDRESS2" ]
	do
		api_num_index2=$((RANDOM%api_address_array_num))
		API_ADDRESS2=${api_address_array[$api_num_index2]}
	done

	API1=$(ntpdate -u $API_ADDRESS1 2>&1)
	if [ "$?" -ne 0 ];then
		API2=$(ntpdate -u $API_ADDRESS2 2>&1)
		if [ "$?" -ne 0 ];then
            api_num_index3=$((RANDOM%api_address_array_num))
            API_ADDRESS3=${api_address_array[$api_num_index3]}
            until [[ "$API_ADDRESS3" != "$API_ADDRESS1" ]] && [[ "$API_ADDRESS3" != "$API_ADDRESS2" ]]
            do
                api_num_index3=$((RANDOM%api_address_array_num))
                API_ADDRESS3=${api_address_array[$api_num_index3]}
            done
		    API3=$(ntpdate -u $API_ADDRESS3 2>&1)
            if [ "$?" -ne 0 ];then
				INSERT_MYSQL "ntp_time_sync fail from $API_ADDRESS1 and $API_ADDRESS2 and $API_ADDRESS3 ."
				echo $API3 >> $LOG_DIR/sysop_manager.log
			else
				echo $API3 >> $LOG_DIR/sysop_manager.log
				hwclock -w
			fi
		else
			echo $API2 >> $LOG_DIR/sysop_manager.log
			hwclock -w
		fi
	else
		echo $API1 >> $LOG_DIR/sysop_manager.log
		hwclock -w
	fi

}
ntp_time_sync

# dir detect
if [ -d /data ] && [ ! -d /data/core_files ];then
	mkdir -p /data/core_files
fi

# change kernel clock_source from tsc to hpet,if machine support
function change_tsc_hpet () {
	KERNEL_CLOCK_DIR=/sys/devices/system/clocksource/clocksource0
	grep -wq "hpet" $KERNEL_CLOCK_DIR/available_clocksource && \
	grep -wq "hpet" $KERNEL_CLOCK_DIR/current_clocksource || echo hpet > $KERNEL_CLOCK_DIR/current_clocksource
}
change_tsc_hpet


function check_disk_space () {
		for PARTITION in `cat /etc/mtab |grep ^/dev |awk '{print $2}'`
		do
			DISK_SPACE=`df -lh|grep -w "$PARTITION"$|tr -d \%|awk '{print $NF,$(NF-1)}'|awk '{if($2>95) print}'`
			if [ "$DISK_SPACE" != "" ];then
				INSERT_MYSQL "DISK_SPACE:$DISK_SPACE"
			fi
		done
}
check_disk_space

function check_disk_status () {
	# /bin/bash /usr/local/i386/public_repos/initial_system/snmp/ext/check_disk.sh|grep -w 1
	for DIR in `cat /etc/mtab |grep ^/dev |awk '{print $2}'`
	do
		TEMP_FILE=`mktemp -p $DIR`
		if [ "$?" -ne 0 ];then
			if [ "`echo $IPADDR|grep -qE "^172\.|^192\.|^10\." && echo 0`" == "0" ];then
				INSERT_MYSQL "$IPADDR  DISK_READ:readonly,partition:$DIR"
			else
				INSERT_MYSQL "DISK_READ:readonly,partition:$DIR"
			fi
			python /home/dspeak/yyms/yymp/yymp_report_script/yymp_report_alarm.py 10754 74894 0 "readonly,partition:$DIR"
		else
			rm -f $TEMP_FILE
			echo 0
		fi
	done
}
check_disk_status

function detect_core_files {
		CORE_PATTERN_LINE=`cat /etc/sysctl.conf|grep -w "kernel.core_pattern"|wc -l`
		if [ "$CORE_PATTERN_LINE" -eq 1 ];then
			CORE_PATTERN_LINE=`cat /etc/sysctl.conf|grep -w "kernel.core_pattern = /data/core_files/core-%e-%p-%t"|wc -l`
			if [ "$CORE_PATTERN_LINE" -eq 0 ];then
				sed -i '/kernel.core_pattern/d' /etc/sysctl.conf
				echo "kernel.core_pattern = /data/core_files/core-%e-%p-%t" >> /etc/sysctl.conf
				sysctl -w "kernel.core_pattern=/data/core_files/core-%e-%p-%t"
			fi
		else
			# no record,or multi record
			sed -i '/kernel.core_pattern/d' /etc/sysctl.conf
			echo "kernel.core_pattern = /data/core_files/core-%e-%p-%t" >> /etc/sysctl.conf
			sysctl -w "kernel.core_pattern=/data/core_files/core-%e-%p-%t"
		fi

	if [ ! -f /data/core_files/.core_info ];then
		touch  /data/core_files/.core_info
	fi

        cd  /data/core_files
	GDB=$(which gdb)
        OLD_CORE_FILES="$(cat /data/core_files/.core_info |xargs)"
        NOW_CORE_FILES=$( /bin/ls core-* 2>/dev/null|xargs )
        CORE_COUNT=$( /bin/ls core-* 2>/dev/null| wc -l )
        NEW_CORE_FILE=$( /bin/ls core-* 2>/dev/null )

        if [ $CORE_COUNT -le 0 ];then
                # now, there is no core down files, do nothing
                :
        elif [ "$OLD_CORE_FILES" = "" ];then
                # completely new files, alarm directly.
                #INSERT_MYSQL "Detected new core file in /data/core_files : $NOW_CORE_FILES . Altogether $CORE_COUNT files."
		for FILE in $NOW_CORE_FILES
		do
			[ "$GDB" != "" ] && \
				PS_NAME=$(sudo $GDB -batch -q -c /data/core_files/$FILE | grep 'Core was generated by'|cut -d\` -f2|cut -d\' -f1)
			if [ "$PS_NAME" = "" ]
			then
				sudo file /data/core_files/$FILE |grep -q 'from' && \
				PS_NAME="$(sudo file /data/core_files/$FILE | sed 's/^.*from //'|tr -d \')" || \
				PS_NAME="$(echo "$FILE"|cut -d- -f2)"
			fi
			DATE="$(sudo ls -l --time-style=long-iso $FILE|awk '{print $6}')"
			TIME="$(sudo ls -l --time-style=long-iso $FILE|awk '{print $7}')"
			[[ "$PS_NAME" =~ ^(grep|pidof)(.*) ]] || INSERT_MYSQL "NEW_CORE_FILE#$FILE#$PS_NAME#$DATE#$TIME"
		done
        else
                for FILE in $OLD_CORE_FILES
                do
                        NEW_CORE_FILE=$(echo "$NEW_CORE_FILE"|grep -Ev "^$FILE$")
                done
                # OK , new core down files found, alarm.
                if [ "$NEW_CORE_FILE" != "" ];then
                        #INSERT_MYSQL "Detected new core file in /data/core_files : $(echo $NEW_CORE_FILE|xargs) . Altogether $CORE_COUNT files."
			for FILE in $NEW_CORE_FILE
			do
			[ "$GDB" != "" ] && \
				PS_NAME=$(sudo $GDB -batch -q -c /data/core_files/$FILE | grep 'Core was generated by'|cut -d\` -f2|cut -d\' -f1)
			if [ "$PS_NAME" = "" ]
			then
				sudo file /data/core_files/$FILE |grep -q 'from' && \
				PS_NAME="$(sudo file /data/core_files/$FILE | sed 's/^.*from //'|tr -d \')" || \
				PS_NAME="$(echo "$FILE"|cut -d- -f2)"
			fi
				DATE="$(sudo ls -l --time-style=long-iso $FILE|awk '{print $6}')"
				TIME="$(sudo ls -l --time-style=long-iso $FILE|awk '{print $7}')"
				[[ "$PS_NAME" =~ ^(grep|pidof)(.*) ]] || INSERT_MYSQL "NEW_CORE_FILE#$FILE#$PS_NAME#$DATE#$TIME"
                [[ "$PS_NAME" =~ ^(grep|pidof)(.*) ]] || /home/dspeak/yyms/yymp/yymp_report_script/yymp_report_alarm.py 10466 "$PS_NAME" "New core dump of $PS_NAME"
			done
                fi
        fi
	# delete core dump files exist over 7 days.
	find /data/core_files -type f -mtime +7 -name "core-*" -exec /bin/rm -f {} \;

        /bin/ls core-* 2>/dev/null  > /data/core_files/.core_info
        return 0
}
rm -f /data/core_files/core-sysop_manager.*
[ -d /data/core_files ] && [ "$cur_status" != "7" ] && detect_core_files
##################################################################################
##################################################################################

# ntp进程检查
function detect_ntp_process () {
	PROCESS_NUM=`ps aux|grep -v grep|grep -w ntpd|wc -l`
	if [ "$PROCESS_NUM" -gt 0 ];then
		/etc/init.d/ntp stop
		wait
		date `date +"%m%d%H%M%C%y.%S"`
		LINK_FILE=`/bin/ls -l /etc/rc2.d|awk '{if($NF=="../init.d/ntp") print $(NF-2)}'`
		if [ "$LINK_FILE" != "" ];then
			unlink /etc/rc2.d/$LINK_FILE
		fi
		sleep 2
		PROCESS_NUM=`ps aux|grep -v grep|grep -w ntpd|wc -l`
		if [ "$PROCESS_NUM" -gt 0 ] || [ -L /etc/rc2.d/$LINK_FILE ];then
			INSERT_MYSQL "turn off ntpd process or unlink ntpd from rc2.d failed,please check."
		else
			INSERT_MYSQL "turn off ntpd process and unlink ntpd from rc2.d successful."
		fi
	fi
}
detect_ntp_process
################################################################################
################################################################################

# 每个小时30分钟执行，硬件健康监控。整点硬件信息上报
half_hour=$(($wall_time_min/30))
if [ "$half_hour" == "1" ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/monitor/mon_hardware.sh
else
	echo "****************************************" >> /var/log/sysop_manager/collect_hardware_info.log
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/collect_hardware_info_to_cmdb.v2.sh >> /var/log/sysop_manager/collect_hardware_info.log 2>&1 &
fi

######################################################################################
######################################################################################
function check_dspeak_yyms () {
	id dspeak 2>&1
	if [[ "$?" -ne 0 ]] || [[ ! -d /home/dspeak/release ]];then
		useradd -s /bin/bash -d /home/dspeak -m dspeak
		mkdir -p /home/dspeak/.ssh
		echo "ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAIEA7NtU7UFJ0EtWvPo/3tV/8Iia3HBhn9/O2wbfeCKZTmZtv4gc/Q6VtnmGmUvqbLqYRbT00QFLsc1QXRx3lJvKZfLaCpniIHhdeBUTVPgRq0TZF7AxGiRS1AZQv/EAvG1tRqs7q5QhyUIO9ZBbmm+ax93K/W0qDnpGb2oKopQHmus=" > /home/dspeak/.ssh/authorized_keys
		chmod 700 /home/dspeak/.ssh
		chmod 600 /home/dspeak/.ssh/authorized_keys
		chown -R dspeak:dspeak /home/dspeak

		# 创建应用系统所需目录
		INSERT_MYSQL "创建应用系统所需目录: /home/dspeak/release/bin,/data/yy/log"
		mkdir -p /home/dspeak/release/bin
		mkdir -p /home/dspeak/yyms/proc_info
		mkdir -p /data/yy/log
		chsh dspeak -s /sbin/nologin

		pkill iptables
		mkdir -p /home/dspeak/iptables
		chown dspeak:dspeak -R /home/dspeak/iptables
		/bin/cp /usr/local/i386/$role/initial_system/iptables/iptables.sh /home/dspeak/iptables/iptables.sh
		iptables -t raw -F
		iptables -t raw -L

		# ipdb网通地址段
		/bin/cp /usr/local/i386/$role/initial_system/route/CncIpDb.txt /home/dspeak/release/bin/CncIpDb.txt
		/bin/cp /usr/local/i386/$role/initial_system/route/CmccIpDb.txt /home/dspeak/release/bin/CmccIpDb.txt
		/bin/cp /usr/local/i386/$role/initial_system/route/EduIpDb.txt /home/dspeak/release/bin/EduIpDb.txt

		echo 1qazxcv > /etc/rsync.scr
		chmod 600 /etc/rsync.scr
    else
        if [ ! -s /etc/rsync.scr ];then
            echo 1qazxcv > /etc/rsync.scr
            chmod 600 /etc/rsync.scr
        fi
    fi
}
check_dspeak_yyms

#######################################################################################
#######################################################################################

# 检查服务器ip、掩码、网关配置（告警）。管理网每天检查一遍，自动刷新ip
function network_configure() {
	TMP_FILE=`mktemp` || return 1
	Room_id=`awk -F '=' '{if($0 ~ /^room_id/){print $2}}' /home/dspeak/yyms/hostinfo.ini`
	[[ "$Room_id" == "" ]] && Room_id="unknow"
	IP_LAST=$(echo $IPADDR | awk -F '.' '{print $NF}')

	function check_active_config() {
		IPADDR_ALL=`ifconfig |grep "inet addr:"|awk '{print $2}'|awk -F ":" '{print $2}'|grep -v "127\.0"|sort -u|tr -s "\n" " "`
		for active_ip in `echo $IPADDR_ALL`;do
			[ -s $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log ] && \
			grep -w "$active_ip" $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log && (rm -f $TMP_FILE;return)

			active_netmask=`ifconfig|grep -w "$active_ip"|awk '{print $NF}'|awk -F ":" '{print $2}'`
			active_gateway_suffix=`echo $active_ip|awk -F "." '{printf("%s.%s.%s",$1,$2,$3)}'`
			active_gateway=$(route -n|grep -v -E "Kernel|Destination"|grep "^0\.0\.0\.0.*$active_gateway_suffix"|awk '{print $2}'|head -n1)
			[ -z "$active_gateway" ] && active_gateway=`route -n|grep -v -E "Kernel|Destination"|awk '{print $2}'|grep "$active_gateway_suffix"|sort -u|tail -n1`
			[[ "$active_netmask" == "255.255.255.255" ]] && continue;

			# check with network_configure.ini
			dead_netmask=`cat $LOG_DIR/network_configure.ini|grep -w "ip:$active_ip"|awk '{print $3}'|awk -F ":" '{print $2}'`
			dead_gateway=`cat $LOG_DIR/network_configure.ini|grep -w "ip:$active_ip"|awk '{print $4}'|awk -F ":" '{print $2}'`
			if [[ "$dead_netmask" != "" ]] && [[ "$dead_netmask" != "$active_netmask" ]];then
				if [ "$1" == "" ];then
					WARN=yes;
				else
					INSERT_MYSQL "Network configure: $active_ip $Room_id MASK:$active_netmask no equal cmdb_info $dead_netmask"
				fi
			fi
			if [[ "$dead_gateway" != "" ]] && [[ "$active_gateway" != "" ]] && [[ "$dead_gateway" != "$active_gateway" ]];then
				echo $active_gateway | grep -Eq "^10\.|^172\.|^192\."
				if [ $? -ne 0 ];then
					if [ "$1" == "" ];then
						WARN=yes;
					else
						INSERT_MYSQL "Network configure: $active_ip $Room_id GW:$active_gateway no equal cmdb_info $dead_gateway"
					fi
				fi
			fi
			unset active_ip active_netmask active_gateway dead_netmask dead_gateway active_gateway_suffix
		done
	}

	if [ "`grep -wq "^isp" $LOG_DIR/network_configure.ini && echo 0`" != "0" ] || [ "`expr $IP_LAST % 24`" -eq "`date +%H`" ];then
		/bin/bash $PUBLIC_DIR/initial_system/snmp/ext/get_cmdb_info/getServer_ip.sh $IPADDR > $TMP_FILE
		grep -q "^isp" $TMP_FILE && cat $TMP_FILE > $LOG_DIR/network_configure.ini
	fi

	[[ -s $LOG_DIR/network_configure.ini ]] && check_active_config
	if [ "$WARN" == "yes" ];then
		# try to update network_configure.ini file
		/bin/bash $PUBLIC_DIR/initial_system/snmp/ext/get_cmdb_info/getServer_ip.sh $IPADDR > $TMP_FILE
		grep -q "^isp" $TMP_FILE && cat $TMP_FILE > $LOG_DIR/network_configure.ini
		unset WARN
		# try to check again , warn ?
		[[ -s $LOG_DIR/network_configure.ini ]] && check_active_config "msg"
	fi

	# 检查服务器是否使用了ip，但是cmdb没有相关的记录
	if [ -s $LOG_DIR/network_configure.ini ];then
		for check_ip in `ip addr list |grep -w inet|awk '{print $2}'`;do
			echo $check_ip | grep -Eq "^127\.0\.0\.1|^169\.254|^10\.101|^10\.100" && continue;
			check_ip=`echo ${check_ip%\/*}`

			[ -s $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log ] && \
			grep -w "$check_ip" $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log && continue
			grep -wq "$check_ip" $LOG_DIR/network_configure.ini
			[[ "$?" -ne 0 ]] && alloc_ip_warn=yes
		done

		if [ "$alloc_ip_warn" == "yes" ];then
			# try to update network_configure.ini file
			/bin/bash $PUBLIC_DIR/initial_system/snmp/ext/get_cmdb_info/getServer_ip.sh $IPADDR > $TMP_FILE
			grep -q "^isp" $TMP_FILE && cat $TMP_FILE > $LOG_DIR/network_configure.ini
			if [ -s $LOG_DIR/network_configure.ini ];then
				for check_ip in `ip addr list |grep -w inet|awk '{print $2}'`;do
					echo $check_ip | grep -Eq "^127\.0\.0\.1|^169\.254|^10\.101|^10\.100" && continue;
					check_ip=`echo ${check_ip%\/*}`

					[ -s $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log ] && \
					grep -w "$check_ip" $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log && continue
					grep -wq "$check_ip" $LOG_DIR/network_configure.ini
					[[ "$?" -ne 0 ]] && INSERT_MYSQL "Network configure: $Room_id CMDB no allocate:$check_ip"
				done
			fi
		fi
	fi

	# 第一阶段的掩码、网关检查已经完毕。下面开始第二阶段检查是否漏了ip没有配置到服务器上面。
	unset dead_ip active_ip
	cat $LOG_DIR/network_configure.ini|grep -v "isp:11"|while read a b c d;
	do
		dead_ip=`echo $b|awk -F ":" '{print $2}'`
		[ -s $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log ] && \
			grep -w "$dead_ip" $PUBLIC_DIR/initial_system/cmdb/exclude_lvs_ip.log && continue
		ip addr list|grep -w inet |grep "$dead_ip" > /dev/null 2>&1
		if [ "$?" != "0" ];then
			if [ "$NO_NEED_UPDATE" != "yes" ];then
				# get cmdb information
				/bin/bash $PUBLIC_DIR/initial_system/snmp/ext/get_cmdb_info/getServer_ip.sh $IPADDR > $TMP_FILE
				grep -q "^isp" $TMP_FILE && cat $TMP_FILE > $LOG_DIR/network_configure.ini
				NO_NEED_UPDATE=yes
			fi

			if [ -s $LOG_DIR/network_configure.ini ];then
				# try to check again
				dead_ip=`echo $b|awk -F ":" '{print $2}'`
				ip addr list|grep -w inet |grep "$dead_ip" > /dev/null 2>&1
				if [ "$?" -ne 0 ];then
					INSERT_MYSQL "Network configure: $Room_id system no ip:$dead_ip"
					break;
				fi
			fi
		fi
	done
	# 检查是否有多个默认路由，导致网络不正常
	default_route_line=`route -n|grep -r "0\.0\.0\.0[[:space:]]*[0-9\.].*[[:space:]]0\.0\.0\.0"|sort -u|wc -l`
	[[ $default_route_line -ge 2 ]] && INSERT_MYSQL "Network configure: $Room_id Have two defaults ruote rule,check!"

	rm $TMP_FILE
}
if ! grep -q "dw_yangzhilin" /home/dspeak/yyms/hostinfo.ini;then
	RUN_MINS=$wall_time_min
	[ "$RUN_MINS" -ge 30 ] && network_configure
	rm -f $TMP_FILE
fi
#####################################################################
#####################################################################

function etc_hosts_file() {

print_hosts_dx(){
(
cat << 'EOF'
221.228.79.31 	bc.yy.duowan.com
221.228.79.32	bc2.yy.duowan.com
121.14.37.153	config.yy.duowan.com
58.215.46.21 	mirror.yy.duowan.com
121.14.43.142   oams.yy.duowan.com
106.38.255.130 oams2.yy.duowan.com
EOF
)
}

print_hosts_wt(){
(
cat << 'EOF'
153.35.190.31	bc.yy.duowan.com
153.35.190.32	bc2.yy.duowan.com
58.249.119.217	config.yy.duowan.com
112.81.159.21 	mirror.yy.duowan.com
122.13.149.142   oams.yy.duowan.com
111.206.234.95  oams2.yy.duowan.com
EOF
)
}

ISP_TYPE=`cat /var/log/sysop_manager/isp.log`
ISP_LINE_NUM=`cat /var/log/sysop_manager/isp.log|wc -l`
if [[ "$ISP_LINE_NUM" == "1" ]] && [[ "$ISP_TYPE" == "5" ]];then
	print_hosts_wt|while read a b;do
		[ "$INTERNAL_IP" = "yes" ] && [ "$b" = "mirror.yy.duowan.com" ] && a="10.20.176.2"
		entry_num=`cat /etc/hosts|grep -w "$b"|wc -l`
		entry_ip=`cat /etc/hosts|grep -w "$b"|tail -n1|awk '{print $1}'`
		if [[ "$entry_num" != "1" ]] || [[ "$entry_ip" != "$a" ]];then
			[ "$HOST_ENTRY_BACKUP" != "yes" ] && cp -f /etc/hosts $LOG_DIR/host_${wall_time_sec}
			HOST_ENTRY_BACKUP="yes"
			sed -i "/$b/d" /etc/hosts
			echo -e "$a\t$b" >> /etc/hosts
			INSERT_MYSQL "change_hosts:$b change $entry_ip to $a"
		fi
	done
else
	# 电信和其他双线的机器默认走电信点
	print_hosts_dx|while read a b;do
		[ "$INTERNAL_IP" = "yes" ] && [ "$b" = "mirror.yy.duowan.com" ] && a="10.20.176.2"
		entry_num=`cat /etc/hosts|grep -w "$b"|wc -l`
		entry_ip=`cat /etc/hosts|grep -w "$b"|tail -n1|awk '{print $1}'`
		if [[ "$entry_num" != "1" ]] || [[ "$entry_ip" != "$a" ]];then
			[ "$HOST_ENTRY_BACKUP" != "yes" ] && cp -f /etc/hosts $LOG_DIR/host_${wall_time_sec}
			HOST_ENTRY_BACKUP="yes"
			sed -i "/$b/d" /etc/hosts
			echo -e "$a\t$b" >> /etc/hosts
			INSERT_MYSQL "change_hosts:$b change $entry_ip to $a"
		fi
	done
fi
}
[ "$cur_status" != "7" ] && [ "$cur_status" != "8" ] && etc_hosts_file

#####################################################################################
#####################################################################################
# recompiled /bin/bash check function.
function check_bash () {
	BASH_LOG_FILE=/data/yy/log/bash/bash.log
	#if [ ! -f $BASH_LOG_FILE ];then
	#	INSERT_MYSQL "no bash log file."
	#fi

function BASH_MD5_LIST () {
		cat <<EOF
		adda55477ed230022aa0cda849f896c5
		4d58fdee8255f0c89022ceb4722a2a94
		c5e63426bf5dbd362598a62058584bd1
		cb1df322fbd6a3d64831bacf1eca3ee2
		e084b9938149b606abc1289bc9059d07
		828bc40c1b19de4275fd4295c316a4ab
		ce010d3f176fac4fd765231265f11365
		e08411991254fb846b9338f3f909360b
EOF
		}
	D_MD5=`md5sum /bin/bash|awk '{print $1}'`
	BASH_MD5_LIST|grep -w $D_MD5
	if [ "$?" -ne 0 ];then
		if [ "${wall_time_hour}:${wall_time_min}" == "10:30" ];then
            sleep $((RANDOM%20))
			/bin/bash /usr/local/i386/*/other/bash_update.sh
		else
			INSERT_MYSQL "/bin/bash may be not recompile yet,please check."
		fi
	fi
}
check_bash

####################################################################################
####################################################################################
# check /etc/crontab. if some user has been delete, it will keep in file(/etc/crontab), that will cause /etc/crontab not run.
function check_etc_crontab () {
	CRON_FILE=/etc/crontab
	a_tmp=`mktemp /tmp/tmp.XXXX`;b_tmp=`mktemp /tmp/tmp.XXXX`;cp -f $CRON_FILE $a_tmp
	for user in `awk '{if($1 !~ /^#/ && NF >= 7){print $6}}' $CRON_FILE | sort|uniq`;do
		id $user > /dev/null 2>&1
		if [ "$?" -ne 0 ];then
			error_uer=$(echo $user | awk '{gsub("/","\\/",$0);print $0}')
			awk '{if($1 ~ /^#/){print $0}else if($1 ~ /.*=/ || NF == 0 ){print $0}else if(NF >= 7 && $6 !~ /'"$error_uer"'/){if($3 != 0 && $4 != 0){print $0}else{print "#(syntax error)"$0}}}' $a_tmp > $b_tmp
			cat $b_tmp > $a_tmp
		fi
	done
	if [[ "`md5sum $CRON_FILE|awk '{print $1}'`" != "`md5sum $a_tmp|awk '{print $1}'`" ]] && [[ -s $a_tmp ]];then
		cp $CRON_FILE $LOG_DIR/_etc_crontab.${wall_time_sec}
		cat $a_tmp > $CRON_FILE
		INSERT_MYSQL "/etc/crontab have some error, and recover."
	fi
	rm -f $a_tmp $b_tmp

}
check_etc_crontab

#####################################################################################
#####################################################################################
function change_privilege_dir () {
	for dir in "/data/yy" "/data/yy/log"
	do
		dir_pri=`stat -c %a $dir`
		[[ "$dir_pri" != "777" ]] && chmod 777 $dir
	done
}
change_privilege_dir

#####################################################################################
#####################################################################################
function collect_os_info () {
	LINUX_BIT=`getconf LONG_BIT|head -n1`
    cat /etc/issue|grep -i "Ubuntu" && LINUX_DIS=ubuntu
    cat /etc/issue|grep -E "debian|Debian" &&  LINUX_DIS=debian
    cat /etc/issue|grep -i "CentOS" && LINUX_DIS=centos
    cat /etc/issue|grep -E "Red|red" && LINUX_DIS=Redhat

	if [ -s /boot/grub/menu.lst ]; then
		CUR_KERNEL=$(grep '[(linux)(kernel)].*vmlinuz' /boot/grub/menu.lst | grep -v -m1 '#' | awk '{for(i=1;i<=NF;i++){if($i ~ /vmlinuz/){print $i;exit}}}')
		CUR_KERNEL="${CUR_KERNEL#*-}"
	elif [ -s /boot/grub/grub.cfg ]; then
		CUR_KERNEL=$(grep '[(linux)(kernel)].*vmlinuz' /boot/grub/grub.cfg | grep -v -m1 '#' | awk '{for(i=1;i<=NF;i++){if($i ~ /vmlinuz/){print $i;exit}}}')
		CUR_KERNEL="${CUR_KERNEL#*-}"
	fi

    if [ "$LINUX_DIS" == "ubuntu" ] || [ "$LINUX_DIS" == "debian" ];then
        DIS_VERSION=`cat /etc/issue 2>/dev/null|head -n1|awk '{print $2}'`
        KERNEL=`uname -r |awk -F "-" '{printf("%s-%s",$1,$2)}'`
		CUR_KERNEL=`echo $CUR_KERNEL|awk -F "-" '{printf("%s-%s",$1,$2)}'`
    elif [ "$LINUX_DIS" == "centos" ] || [ "$LINUX_DIS" == "Redhat" ];then
        DIS_VERSION=`cat /etc/issue 2>/dev/null|head -n1|awk '{print $(NF-1)}'`
        KERNEL=`uname -r | awk -F '.' '{printf("%s.%s.%s",$1,$2,$3)}'`
		CUR_KERNEL=`echo $CUR_KERNEL | awk -F '.' '{printf("%s.%s.%s",$1,$2,$3)}'`
    else
        LINUX_DIS=unknow
    fi

	time=`uptime`;time=`echo ${time#*up}`;time=`echo ${time%%,*}`
	server_room=`awk -F '=' '{if($0 ~ /^room=/){print $2}}' /home/dspeak/yyms/hostinfo.ini`
	tech_admin=`awk -F '=' '{if($0 ~ /^responsibleAdmin=/){print $2}}' /home/dspeak/yyms/hostinfo.ini`
	sysop_admin=`awk -F '=' '{if($0 ~ /^sysopResponsibleAdmin=/){print $2}}' /home/dspeak/yyms/hostinfo.ini`
	server_id=`awk -F '=' '{if($0 ~ /^server_id/){print $2}}' /home/dspeak/yyms/hostinfo.ini`
	sleep $((RANDOM%50))
    INSERT_MYSQL "collect_os_info: $server_id|$LINUX_DIS|$DIS_VERSION|$LINUX_BIT|$KERNEL|$CUR_KERNEL|$time|$server_room|$tech_admin|$sysop_admin"

}
if [ "${wall_time_hour}" -eq "02" ];then
	collect_os_info
	/bin/bash $COMM_DIR/misc/collect_mac_ip.sh
fi

#####################################################################################
#####################################################################################
function change_command_pri () {
	[ ! -s $PUBLIC_DIR/initial_system/monitor/command_and_dir.pri.log ] && return 1
	for msg in $(grep -vE "^#|^$" $PUBLIC_DIR/initial_system/monitor/command_and_dir.pri.log);
	do
		change_dst=${msg%%:*}
		change_pri=$(echo $msg|awk -F ':' '{print $2}')
		change_type=$(echo $msg|awk -F ':' '{print $3}')
		change_rec=${msg##*:}

		if [ "$change_type" == "c" ];then
			for dst_sub in $(which -a $change_dst 2>/dev/null);
			do
				[ -L $dst_sub ] && continue
				cur_pri=$(stat -c %a $dst_sub 2>/dev/null)
				[ "$cur_pri" != "$change_pri" ] && chmod $change_pri $dst_sub
			done
		elif [ "$change_type" == "d" ];then
			if [ "$change_rec" == "R" ];then
				chmod -R $change_pri $change_dst
			else
				chmod $change_pri $change_dst
			fi
		fi
	done
}
change_command_pri

#####################################################################################
#####################################################################################
#	每天备份重要系统文件
function backup_system_file () {
	BACKUP_DIR=/data/.backup
	[ ! -d $BACKUP_DIR ] && mkdir $BACKUP_DIR
	cur_time=$(date "+%H:%M")
	cur_hour="${cur_time%%:*}"
	if [ "$cur_hour" = "05" ]; then
		cur_date=$(date "+%Y%m%d")
		if [ ! -d $BACKUP_DIR/backup_${cur_date} ]; then
			mkdir $BACKUP_DIR/backup_${cur_date} || return 1
			grep -E 'IP_Address|OS_info' /etc/hardware_info > $BACKUP_DIR/backup_${cur_date}/system_info.log
			time=`uptime`;time=`echo ${time#*up}`;time=`echo ${time%%,*}`
			echo -e "UPTIME:\t\t$time" >> $BACKUP_DIR/backup_${cur_date}/system_info.log


			cd / && tar cpjf $BACKUP_DIR/backup_${cur_date}/etc.tar.bz2 etc/
			cd -
			cp /usr/local/virus/iptables/iptables.* $BACKUP_DIR/backup_${cur_date}/.
			ls -l /home --color=never | awk '{print $NF}' \
			| grep -Ev 'duowan|dspeak|backup|ubuntu|centos|pxe_kenjee' > $BACKUP_DIR/backup_${cur_date}/home_user.log

			binary_busybox=$(which busybox)
			[ -x "$binary_busybox" ] && cp -p $binary_busybox $BACKUP_DIR/backup_${cur_date}/.

			# 备份回滚60天
			DEL_AGO=60
			del_date=$(date "+%Y%m%d" -d "$DEL_AGO day ago")
			for check_backup_dir in $BACKUP_DIR/* ;
			do
				check_time="${check_backup_dir##*_}"
				if [ $check_time -ge $del_date ] ; then
					continue
				else
					echo $check_backup_dir | grep -q "${BACKUP_DIR}/backup" \
						&& rm -rf $check_backup_dir
				fi
			done
		fi
	fi

}
backup_system_file

#####################################################################################
#####################################################################################
# Update OS scripts
function update_os_script () {

UPDATE_SCRIPT_FILE () {
		cat <<EOF
		/usr/local/i386/$REPOS_NAME/initial_system/other/udev_check.sh $LOG_DIR/udev_check.sh
EOF
}

	UPDATE_SCRIPT_FILE|while read SRC_FILE DST_SCRIPT;do
		SRC_MD5=$(md5sum $SRC_FILE|cut -d ' ' -f 1)
		DST_MD5=$(md5sum $DST_SCRIPT|cut -d ' ' -f 1)
		if [ "$SRC_MD5" != "$DST_MD5" ]; then
			cat $SRC_FILE > $DST_SCRIPT
			[ $? -eq 0 ] && /bin/bash $SRC_FILE
		fi
	done
}
update_os_script

#####################################################################################
#####################################################################################
# Initial BMC user and password every day
function update_bmc() {
	random_key="${IPADDR##*.}"
	random_key="$((random_key%24))"
	if [ "$random_key" -eq "$wall_time_hour" ] && \
		[ "$wall_time_min" -lt 10 ];then
		bash $COMM_DIR/other/initial_bmc.sh randomcode
	fi
}
update_bmc

