#!/bin/bash
#DATA:2012-09-13
#Written by wuhaiting.
#README:monitor hardware. fan,tempture,memory,disk,accelerator

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
DISK_TMP_FILE=`mktemp /tmp/disk_status.XXXX`

# check linux version
cat /etc/issue|grep -i -E "CentOS|Red" >/dev/null 2>&1 && LINUX_VERSION="centos"
cat /etc/issue|grep -i -E "Ubuntu|Debian" >/dev/null 2>&1 && LINUX_VERSION="ubuntu"
SYSTEM_VERSION=`getconf LONG_BIT`
if [ "$LINUX_VERSION" = "" ] || [ "$SYSTEM_VERSION" == "" ];then exit 1;fi

# basic define
BASENAME=`basename $0`
SHORT_NAME=mon_hardware
LOG_DIR=/var/log/sysop_manager
LOG_MARK=$LOG_DIR/hardware.mark
LOG_FILE=/var/log/hardware.log

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

# repository path
REPOS_NAME=comm_repos
COMM_DIR=/usr/local/i386/$REPOS_NAME
PUBLIC_DIR=/usr/local/i386/public_repos

# get ip address 
IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
fi 

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:] ]*$SHORT_NAME" && exit 1

# define insert alert function
# $2=warning־platform档$2=error yyms
INSERT_MYSQL () {
MSG=$1;wall_time=`date "+%Y%m%d %H:%M:%S"`
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$MSG"
echo -e "$wall_time $MSG\n$3" >> $LOG_FILE
echo "$MSG"
}

# wget function
wget_action () {
	wget --http-user=manifold --http-passwd=dw_manifold --output-document=$1 --timeout=$2 --tries=$3 "http://$4:63217/$5"
}

DMIDECODE_DIR=`which dmidecode`
MANUFACTURER=`$DMIDECODE_DIR -s system-manufacturer`
RAID_DEVICE=`lspci|grep -E -w "RAID|SCSI|SATA"`

function hard_disk_check () {
    for DIR in `cat /etc/mtab |grep ^/dev |awk '{print $2}'`
    do
        TEMP_FILE=`mktemp -p $DIR`
        if [ "$?" -ne 0 ];then
            INSERT_MYSQL "DISK_READ:readonly,partition:$DIR" "error"
        else
            rm -f $TEMP_FILE
        fi  
    done
	
	# пӲ̹
	if [ ! -z "$RAID_DEVICE" ];then
	# LSI MegaRaid array 
		if [ "`lspci|grep -E -w "RAID|SCSI"|grep -E -w "MegaRAID SAS|MegaSAS" > /dev/null && echo "MegaRAID"`" == "MegaRAID" ];then
			if [ ! -f /usr/sbin/megacli ];then
				if [ "$LINUX_VERSION" == "ubuntu" ];then
					cd /tmp && aptitude -y download megacli ;dpkg -i megacli* ;rm -rf megacli* ;cd -
				elif [ "$LINUX_VERSION" == "centos" ];then
					wget_action /tmp/megacli-8.00.29.tar.bz2 8 2 121.14.36.27 package/megacli-8.00.29.tar.bz2
					[[ "$?" -eq 0 ]] && cd /tmp && tar jxf megacli-8.00.29.tar.bz2 && rpm -i lib_utils-1.00-08.noarch.rpm && rpm -i megacli-8.00.29-1.i386.rpm
					[[ -x /opt/MegaRAID/MegaCli/MegaCli64 ]] && ln -s /opt/MegaRAID/MegaCli/MegaCli64 /usr/sbin/megacli
					cd - && rm -f /tmp/megacli.* /tmp/lib_utils.*
				fi
				[[ ! -f /usr/sbin/megacli ]] && return 1
			fi
			# ӲraidǷ⡢ǷлֻǳȡпӲ״̬ϢпִС
			DISK_STATUS=`/usr/sbin/megacli -cfgdsply -aall|grep -E "^State|^Firmware state:|Error Count:|Failure Count:"|grep -v -w -E "0|Optimal|Online, Spun Up"`
			[[ -n $DISK_STATUS ]] && INSERT_MYSQL "Hardware check: Some Disks or Raid have error." "error" "$DISK_STATUS"
			DISK_POLICY=`/usr/sbin/megacli -cfgdsply -aall|grep -w -E "Current Cache Policy:"|grep -w -v "WriteBack, ReadAheadNone, Direct, No Write Cache if Bad BBU"`
			[[ -n $DISK_POLICY ]] && INSERT_MYSQL "Hardware check: disk policy abnormal, it will affect I/O or safety." "warning" "$DISK_POLICY"
			BATTERY_REPLACE=`/usr/sbin/megacli -adpbbucmd -aall|grep -E "Battery Replacement required|Remaining Capacity Low"|grep -v -w "No"`
			[[ -n $BATTERY_REPLACE ]] && INSERT_MYSQL "Hardware check: array controller battery error, please replace!" "error" "$BATTERY_REPLACE"
			rm MegaSAS.log
			return 0
		
		elif [ "`lspci|grep -E -w "RAID|SCSI"|grep -w "Smart Array" > /dev/null && echo "Smart_Array"`" == "Smart_Array" ];then
			if [ ! -f /usr/sbin/hpacucli ];then
				if [ "$LINUX_VERSION" == "ubuntu" ];then
					cd /tmp && apt-get update >/dev/null;aptitude -y download hpacucli;dpkg -i hpacucli* ;rm -rf hpacucli* ;cd -
				elif [ "$LINUX_VERSION" == "centos" ];then
					wget_action /tmp/hpacucli-8.75-12.0.noarch.rpm 8 2 121.14.36.27 package/hpacucli-8.75-12.0.noarch.rpm
					[[ "$?" -eq 0 ]] && rpm -i /tmp/hpacucli-8.75-12.0.noarch.rpm
					rm -f /tmp/hpacucli-8.75-12.0.noarch.rpm
				fi
				[[ ! -f /usr/sbin/hpacucli ]] && return 1
			fi
			DISK_STATUS=`/usr/sbin/hpacucli ctrl all show config 2>/dev/null|grep -E "physicaldrive|logicaldrive"|grep -v "OK"`
			[[ -n $DISK_STATUS ]] && INSERT_MYSQL "Hardware check: Some Disks or Raid have error." "error" "$DISK_STATUS"
			ARRAY_STATUS=`/usr/sbin/hpacucli ctrl all show status 2>/dev/null|grep -E "Controller Status:|PROBLEM DETECTED"|grep -v "OK"`
			[[ -n $ARRAY_STATUS ]] && INSERT_MYSQL "Hardware check: Array controller have some error." "error" "$ARRAY_STATUS"
		else
			# Use smartmontools to monitor disk status,which use disk's S.M.A.R.T table
			if [ ! -x /usr/sbin/smartctl ];then
				if [ "$LINUX_VERSION" == "ubuntu" ];then
					cd /tmp;aptitude download smartmontools > /dev/null 2>&1;dpkg -i smartmontools*;rm -f /etc/rc2.d/S20smartmontools /tmp/smartmontools*;cd -
				elif [ "$LINUX_VERSION" == "centos" ];then
					if [ "`lsb_release -a|grep "Release"|awk -F ":" '{print $2}'|awk -F "." '{print $1}'`" == "5" ];then
						wget http://mirror.centos.org/centos/5/os/i386/CentOS/smartmontools-5.38-3.el5.i386.rpm
						[[ "$?" -eq 0 ]] && rpm -i smartmontools-5.38-3.el5.i386.rpm
					else
						yum -y install smartmontools
					fi
				fi
				[[ ! -x /usr/sbin/smartctl ]] && return 1
			fi
			for a in `ls /dev/sd?`;do
				DISK_BASENAME=`basename $a`
				smartctl -a $a > $DISK_TMP_FILE
				cat $DISK_TMP_FILE |grep "does not support SMART" > /dev/null 2>&1 && continue
				DISK_STATUS=`cat $DISK_TMP_FILE |grep -E "SMART Health Status|grown defect list|Non-medium error|SMART.*test result:|Error Count:"|awk -F ":" '{print $2}'|awk '{print $1}'`
				set -- $DISK_STATUS
				# SAS DISK: SMART Health Status: OK    SATA disk: SMART overall-health self-assessment test result: PASSED
				[[ "$1" != "" ]] && if [ "$1" == "OK" ];then :;elif [ "$1" == "PASSED" ];then :;else
					INSERT_MYSQL "Hardware check: $a SMART status Fail, check!" "error"
				fi
				sed -i "/smartctl $DISK_BASENAME SMART status:.*/d" $LOG_MARK
				echo "smartctl $DISK_BASENAME SMART status: $1" >> $LOG_MARK
				# Elements in grown defect list: 0
				[[ "$2" != "" ]] && [[ "$2" != "0" ]] && INSERT_MYSQL "Hardware check: $a grown_defect_list: $2, check!" "error"
				sed -i "/smartctl $DISK_BASENAME grown_defect_list:.*/d" $LOG_MARK
				echo "smartctl $DISK_BASENAME grown_defect_list: $2" >> $LOG_MARK
				# Non-medium error count: 0
				MEDIUM_ERROR=`grep "$DISK_BASENAME none_medium_error" $LOG_MARK|tail -n1|awk -F ":" '{print $NF}'`
				[[ "$3" != "" ]] && [[ "$MEDIUM_ERROR" != "" ]] && [[ "$3" -ne "$MEDIUM_ERROR" ]] && INSERT_MYSQL "Hardware check: $a none_medium_error $3 not equal old value $MEDIUM_ERROR" "warning"
				sed -i "/smartctl $DISK_BASENAME none_medium_error:.*/d" $LOG_MARK
				echo "smartctl $DISK_BASENAME none_medium_error: $3" >> $LOG_MARK
				# Error counter log:
				# Errors Corrected by           Total   Correction     Gigabytes    Total
                # ECC          rereads/    errors   algorithm      processed    uncorrected
                # fast | delayed   rewrites  corrected  invocations   [10^9 bytes]  errors
                # read:    3647480     0         0   3647480    3647480          2.280           0
                # write:      0        0         0         0          0        413.131           0
                # verify:      61       0         0        61         61          0.000           0
				unset DISK_STATUS
				DISK_STATUS=`cat $DISK_TMP_FILE |grep -E "read:|write:|verify:|Raw_Read_Error_Rate|Seek_Error_Rate"|awk '{print $NF}'`
				set -- $DISK_STATUS
				[[ "$1" != "" ]] && [[ "$1" != "0" ]] && INSERT_MYSQL "Hardware check: $a READ uncorrected_error $1" "error"
				sed -i "/smartctl $DISK_BASENAME read uncorrected_error.*/d" $LOG_MARK
				echo "smartctl $DISK_BASENAME read uncorrected_error: $1" >> $LOG_MARK
				
				[[ "$2" != "" ]] && [[ "$2" != "0" ]] && INSERT_MYSQL "Hardware check: $a WRITE/SEEK uncorrected_error $2" "error"
				sed -i "/smartctl $DISK_BASENAME write\/seek uncorrected_error.*/d" $LOG_MARK
				echo "smartctl $DISK_BASENAME write/seek uncorrected_error: $2" >> $LOG_MARK
			done
		fi
	fi
	
	
}

log_read_write () {
	FILE=$1;BASENAME_FILE=`basename $1`;MASK=$2
	if [ -f $FILE ];then
		OLD_LINE_NUM=`cat $LOG_MARK|grep "$MASK"|tail -n1|awk -F ":" '{print $NF}'`
		END_LINE_NUM=`cat $FILE|wc -l`
		eval "sed -i "/$MASK.*/d" $LOG_MARK"
		echo "$MASK $END_LINE_NUM" >> $LOG_MARK
		if [ "$OLD_LINE_NUM" == "" ] || [ $OLD_LINE_NUM -gt $END_LINE_NUM ];then
			OLD_LINE_NUM=0
		fi
		OLD_LINE_NUM=`expr $OLD_LINE_NUM + 1`
		eval "sed -n '$OLD_LINE_NUM,$END_LINE_NUM p' $FILE |grep -E \"$HARDWARE_MASK\"|grep -v -E \"$V_MASK\"|sort -nu > $DISK_TMP_FILE"
		RESULT=`cat $DISK_TMP_FILE|grep -v "^#"|wc -l`
		[[ "$RESULT" != "0" ]] && INSERT_MYSQL "Hardware check: system [$BASENAME_FILE]log found $RESULT error!" "error"
		[[ -s $DISK_TMP_FILE ]] && cat $DISK_TMP_FILE|tail -n100 >> $LOG_FILE
	fi
}

# ϵͳ/var/log־ҪĹؼ
function log_prase () {
	# ϵͳ־вӲ̴־,ubuntu messagessyslogerror,centos kernmessages
	# Error Dectect and Corrected (EDAC)ԶصӲģ顣ubuntu910ں˱ȽϾãûi7core_edac
	# HARDWARE_MASK ־Ĺؼƥ
	HARDWARE_MASK="Medium Error|I/O error|Device offline|Unrecovered read error|EXT*-fs error|EDAC"
	V_MASK="EDAC.*Ver:|Giving out device|EDAC.*Driver loaded|EDAC.*Removed device"
	if [ "$LINUX_VERSION" == "ubuntu" ];then
		log_read_write "/var/log/syslog" "ubuntu_syslog_last_check_line:"
		log_read_write "/var/log/messages" "ubuntu_messages_last_check_line:"
		log_read_write "/var/log/kern.log" "ubuntu_kern_log_last_check_line:"
		log_read_write "/var/log/dmesg" "ubuntu_dmesg_last_check_line:"
	elif [ "$LINUX_VERSION" == "centos" ];then
		log_read_write "/var/log/messages" "centos_messages_last_check_line:"
		log_read_write "/var/log/kern" "centos_kern_last_check_line:"
		log_read_write "/var/log/dmesg" "centos_dmesg_last_check_line:"
	fi
}

function ipmi_log_prase () {
	OLD_IPMI_NUM=`cat $LOG_MARK|grep "ipmi log last_check_line:"|tail -n1|awk -F ":" '{print $NF}'`
	ipmitool sel list > $DISK_TMP_FILE 2>/dev/null
	NOW_IPMI_NUM=`cat $DISK_TMP_FILE |wc -l`
	[[ $NOW_IPMI_NUM -gt 2000 ]] && ipmitool sel clear >/dev/null 2>&1
	cat $LOG_MARK|grep "ipmi log last_check_line" >/dev/null 2>&1 || ret=yes
	sed -i "/ipmi log last_check_line/d" $LOG_MARK
	echo "ipmi log last_check_line: $NOW_IPMI_NUM" >> $LOG_MARK
	[[ "$ret" == "yes" ]] && return 1
	if [[ "$OLD_IPMI_NUM" == "" ]] && [[ $OLD_IPMI_NUM -gt $NOW_IPMI_NUM ]];then
		# reset LINE NUM
		OLD_IPMI_NUM=0
	fi
	OLD_IPMI_NUM=`expr $OLD_IPMI_NUM + 1`
	eval "sed -ni '$OLD_IPMI_NUM,$NOW_IPMI_NUM p' $DISK_TMP_FILE"
	IPMI_ERROR=`cat $DISK_TMP_FILE|awk -F "|" '{printf("%s,%s\n",$4,$5)}'|sort -u`
	[[ "$IPMI_ERROR" != "" ]] && INSERT_MYSQL "Hardware check: BMC $IPMI_ERROR" "error"
}

# main
[[ ! -f $LOG_FILE ]] && touch $LOG_FILE
[[ ! -f $LOG_MARK ]] && touch $LOG_MARK

# Ӳ̼
hard_disk_check
# ־Ϣ
log_prase
# BMC ־鿴
ls -l /dev/ipmi* >/dev/null
[[ "$?" -eq 0 ]] && ipmi_log_prase

rm -f $DISK_TMP_FILE
exit 0