#!/bin/bash
#Date:2011-05-15
#Written by manifold.
#Readme:monitor the sysop_manager.sh process.
#Change:2012-07-05
#Readme:check host entry in /etc/hosts about manager.repos.yy.duowan.com.

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
REPOS_NAME=comm_repos
source /usr/local/i386/$REPOS_NAME/auto/function.sh
noblock_lock
trap "noblock_unlock;" 0 1 2 9 15

# basic define
BASENAME=$(basename $0)
SHORT_NAME=sysop_manager
LOG_DIR=/var/log/sysop_manager

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

# repository path
COMM_DIR=/usr/local/i386/$REPOS_NAME

# get ip address
IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
fi

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:] ]*$SHORT_NAME" && exit 1

BACKUP_DATE=`date +%Y-%m-%d-%H-%M-%S`

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# grant execute atttibute
if [ ! -x $COMM_DIR/auto/sysop_manager.sh ];then
	chmod 755 $COMM_DIR/auto/sysop_manager.sh
fi

detect_file () {
	if [ ! -f $S_FILE ];then
		INSERT_MYSQL "no file $S_FILE."
	elif [ ! -f $D_FILE ];then
		/bin/cp $S_FILE $D_FILE
		if [ "$?" -eq 0 ];then
			INSERT_MYSQL "add file $D_FILE successful."
			CHANGE=yes
		else
			INSERT_MYSQL "add file $D_FILE failed."
		fi
	else
		S_MD5=`md5sum $S_FILE|awk '{print $1}'`
		D_MD5=`md5sum $D_FILE|awk '{print $1}'`
		if [ "$S_MD5" != "$D_MD5" ];then
			BACKUP_FILE="`echo $D_FILE|sed 's#/#_#g'`.$BACKUP_DATE"
			/bin/cp $D_FILE $LOG_DIR/$BACKUP_FILE
			cat $S_FILE > $D_FILE
			if [ "$?" -eq 0 ];then
				CHANGE=yes
				INSERT_MYSQL "$D_FILE changed success"
			else
				INSERT_MYSQL "$D_FILE changed failed"
			fi
		fi
	fi
}

# check sysop_manager.sh revision.
if [ ! -f $LOG_DIR/sysop_manager.sh ];then
	/bin/cp $COMM_DIR/auto/sysop_manager.sh $LOG_DIR/sysop_manager.sh
	CHANGE=yes
	STEP="no file"
else
	REAL_md5=`md5sum $COMM_DIR/auto/sysop_manager.sh|awk '{print $1}'`
	NOW_md5=`md5sum $LOG_DIR/sysop_manager.sh|awk '{print $1}'`
	if [ "$REAL_md5" != "$NOW_md5" ];then
		/bin/cp $LOG_DIR/sysop_manager.sh $LOG_DIR/sysop_manager.sh.$BACKUP_DATE
		cat $COMM_DIR/auto/sysop_manager.sh > $LOG_DIR/sysop_manager.sh
		CHANGE=yes
		STEP="file change"
	fi
fi

# check host entry(sysop_manager.sh also have this function)
DOMAIN_NAME=manager.repos.yy.duowan.com
REPOS_HOST_LINE=`cat /etc/hosts|grep -w "$DOMAIN_NAME"|wc -l`
if [ "$REPOS_HOST_LINE" -eq 0 ];then
	echo "58.215.46.20 $DOMAIN_NAME" >> /etc/hosts
	INSERT_MYSQL "add $DOMAIN_NAME entry in /etc/hosts"
elif [ "$REPOS_HOST_LINE" -gt 1 ];then
	sed -i '/.*[[:space:]]\+[[:alnum:] ]*'"${DOMAIN_NAME}"'/d' /etc/hosts
	echo "58.215.46.20 $DOMAIN_NAME" >> /etc/hosts
	INSERT_MYSQL "add $DOMAIN_NAME entry in /etc/hosts"
fi

# check sysop_manager.sh running or not.if running check process num.
PROCESS_LINE=`ps aux|grep "/bin/bash ./sysop_manager.sh"|grep -v grep|wc -l`
if [ "$PROCESS_LINE" -eq 0 ];then
	CHANGE=yes
	STEP="no process"
elif [ "$PROCESS_LINE" -eq 1 ];then
	SLEEP_STATUS=`ps aux|grep -v grep|awk '{if($8=="D" && $(NF-1)=="/bin/bash" && $NF=="./sysop_manager.sh") print}'`
	if [ "$SLEEP_STATUS" != "" ];then
		CHANGE=yes
		STEP="uninterruptible sleep"
	fi
elif [ "$PROCESS_LINE" -gt 1 ];then
	sleep 10
	PROCESS_LINE=`ps aux|grep "/bin/bash ./sysop_manager.sh"|grep -v grep|wc -l`
	if [ "$PROCESS_LINE" -gt 1 ];then
		CHANGE=yes
		STEP="more than 1 process"
		ps aux|grep "/bin/bash ./sysop_manager.sh"|grep -v grep >> $LOG_DIR/sysop_manager.log
	fi
fi

# if sysop_manager not running or version updated,restart it.
if [ "$CHANGE" = "yes" ];then
	ps aux|grep '/bin/bash ./sysop_manager.sh'|grep -v -E "grep|mysql"|awk '{print $2}'|xargs kill -9
	cd $COMM_DIR/auto
	./sysop_manager.sh &
	if [ "$?" -eq 0 ];then
		INSERT_MYSQL "sysop_manager.sh change result in step:$STEP,and restart successful."
	else
		INSERT_MYSQL "sysop_manager.sh change result in step:$STEP,and restart failed."
	fi
fi

# check svn locked process.
for PID in `ps aux|grep -v -E "grep|su|bash"|grep -E "svn switch|svn update -r"|awk '{print $2}'`
do
	PROCESS_JIFFIES=`cat /proc/$PID/stat | cut -d" " -f22`
	SYSTEM_UPTIME=`grep btime /proc/stat | cut -d" " -f2`
	PROCESS_START_SEC=$(( $SYSTEM_UPTIME + $PROCESS_JIFFIES / 100 ))
	NOW_SEC=`date -u --date 'now' "+%s"`
	PROCESS_RUN_SEC=$(( $NOW_SEC - $PROCESS_START_SEC ))
	# if svn switch or update process locked more than 600 seconds,kill the process.
	if [ "$PROCESS_RUN_SEC" -gt 600 ];then
		INSERT_MYSQL "have found svn process:$PID locked $PROCESS_RUN_SEC seconds,then kill it."
		ps aux|awk '{if($2~/'"$PID"'/) print}' >> $LOG_DIR/sysop_manager.log
		kill -9 $PID
	fi
done

# ϵͳoomֹoomʱɱؼcronsshd
for PRO_PID in `ps aux |grep -wE "/usr/sbin/sshd|syslog-ng" |grep -vwE "ssh|grep" |grep "^root" | awk '{print $2}'`;do
	if [ -f /proc/$PRO_PID/oom_score_adj ];then
		cron_value=`cat /proc/$PRO_PID/oom_score_adj`
		[[ "$cron_value" != "-1000" ]] && echo -1000 > /proc/$PRO_PID/oom_score_adj
	elif [ -f /proc/$PRO_PID/oom_adj ];then
		cron_value=`cat /proc/$PRO_PID/oom_adj`
		[[ "$cron_value" != "-17" ]] && echo -17 > /proc/$PRO_PID/oom_adj
	fi

done
