#!/bin/bash
#Date:2012-02-12
#Written by manifold.
#Readme:repository update script.
#Change:2012-06-19
#Readme:change get revision function from 'svn info' to 'cat entries',for reduce io usage rate.
#Change:2012-07-09
#Readme:increase update frequency.

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
export LANG=C

# check linux version
cat /etc/issue|grep -E "CentOS|Red" && LINUX_VERSION="centos"
cat /etc/issue|grep -E "Ubuntu|Debian" && LINUX_VERSION="ubuntu"
if [ "$LINUX_VERSION" = "" ];then exit 1;fi

# prepare the log directory
LOG_DIR=/var/log/sysop_manager
if [ ! -d "$LOG_DIR" ];then
	mkdir $LOG_DIR
fi

# basic define
BASENAME=`basename $0`
SHORT_NAME=repository
RUN_USER=yuwanfu
FORCE_SYNC=yes
DOMAIN_NAME=manager.repos.yy.duowan.com

# detect run or not.
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
	echo "Detect stop run file in directory $LOG_DIR and quit." && exit 0
fi

# repository path
REPOS_NAME=comm_repos
UP_REPOS_NAME="$REPOS_NAME public_repos"
COMM_DIR=/usr/local/i386/$REPOS_NAME
PUBLIC_DIR=/usr/local/i386/public_repos

# get ip address
function GET_LOCAL_IP () {
	IPADDR=`/sbin/ifconfig|grep Bcast|grep -vE "inet addr:192\.168|inet addr:172\.|inet addr:10\."|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
	unset INTERNAL_IP
	if [ "$IPADDR" = "" ];then
		IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -v 127.0.0.1|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
		INTERNAL_IP="yes"
	fi
}
GET_LOCAL_IP

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v ^# | grep "^$IPADDR[[:space:]]\+[[:alnum:] ]*$SHORT_NAME" && exit 1

# set svn client timeout value
HTTP_TIMEOUT_LINE=`cat /home/$RUN_USER/.subversion/servers|grep "^http-timeout = .*"|wc -l`
if [ "$HTTP_TIMEOUT_LINE" -eq 0 ];then
	echo "http-timeout = 15" >> /home/$RUN_USER/.subversion/servers
else
	sed -i 's/^http-timeout = .*/http-timeout = 15/' /home/$RUN_USER/.subversion/servers
fi

# set subversion 1.6 compatible function,such as store-plaintext-passwords
STORE_PALAINTEXT_PASSWORD_LINE=`cat /home/$RUN_USER/.subversion/servers|grep "^store-plaintext-passwords = .*"|wc -l`
if [ "$STORE_PALAINTEXT_PASSWORD_LINE" -eq 0 ];then
	echo "store-plaintext-passwords = yes" >> /home/$RUN_USER/.subversion/servers
else
	sed -i 's/^store-plaintext-passwords = .*/store-plaintext-passwords = yes/' /home/$RUN_USER/.subversion/servers
fi

# set svn client ssl-authority-files.
SSL_AUTHORITY_FILE_LINE=`cat /home/$RUN_USER/.subversion/servers|grep "^ssl-authority-files = .*"|wc -l`
if [ "$SSL_AUTHORITY_FILE_LINE" -eq 0 ];then
	echo "ssl-authority-files = /etc/ssl/yy-cacert.pem" >> /home/$RUN_USER/.subversion/servers
else
	sed -i 's#^ssl-authority-files = .*#ssl-authority-files = /etc/ssl/yy-cacert.pem#' /home/$RUN_USER/.subversion/servers
fi

# echo alarm message,and use php api to insert alarm message to mysql database.
INSERT_MYSQL () {
	/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# execute svn update interval
ACTION_INTERVAL=30

# recheck update version times
for DEFINE_REPOS in $UP_REPOS_NAME
do
	eval RECHECK_UPDATE_VERSION_TIMES_"$DEFINE_REPOS"=0
	eval FORCE_UPDATE_TIME_"$DEFINE_REPOS"=0
done
RECHECK_UPDATE_VERSION_TIMES_REPOS_NAME=0
FORCE_UPDATE_TIME_DEFINE_REPOS=0
unset DEFINE_REPOS

# endless loop,used to update.
while :;do

# ##########################################################################################
# ##########################################################################################

	# detect run or not.
	if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
		INSERT_MYSQL "Data_sync:Detect stop run file in directory $LOG_DIR and quit."
		sleep $ACTION_INTERVAL
		continue
	fi

	# check cron process running or not.(if this script process not run,cron will start it.
	CRON_LINE=`ps aux|grep -e "\<cron"|grep -v -E "ssh|grep|mysql|python" | wc -l`
	if [ "$CRON_LINE" -le "0" ];then
		/etc/init.d/cron* start
		INSERT_MYSQL "Data_sync:Cron process not running."
	fi

	/bin/bash /usr/local/i386/public_repos/initial_system/snmp/ext/jiji_word/jiji_change_w0rd.sh

	# check crontab task.(if all the crontab task be terminate,the process will be the last check method.)
	if [ "$LINUX_VERSION" = "centos" ];then
		CRON_FILE=/var/spool/cron/root
	else
		CRON_FILE=/var/spool/cron/crontabs/root
	fi
	DISP_FILE=$COMM_DIR/initial_system/dispatcher.content
	CRON_MD5=`md5sum $CRON_FILE|awk '{print $1}'`
	DISP_MD5=`md5sum $DISP_FILE|awk '{print $1}'`
	if [ "$CRON_MD5" != "$DISP_MD5" ];then
		INSERT_MYSQL "Data_sync:crontab task error."
	fi

	# if repository public_repos doesn't exist,check out it.
	if [ ! -d /usr/local/i386/public_repos ];then
		# REPOS_ADDRESS=`svn info $COMM_DIR|awk -F ':' '{if($1~/URL/) print $3}'|tr -d '\/'`
		REPOS_ADDRESS=`cat $COMM_DIR/.svn/entries|grep https://|awk -F ':' '{print $2}'|tr -d '\/'|uniq|head -n 1`
		echo $REPOS_ADDRESS|grep -E "$DOMAIN_NAME|[[:digit:]]*\.[[:digit:]]*\.[[:digit:]]*\.[[:digit:]]*"
		GREP_RESULT="$?"
		if [ "$REPOS_ADDRESS" = "" ] || [ "$GREP_RESULT" -ne 0 ];then
			REPOS_ADDRESS=`svn info $COMM_DIR|awk -F ':' '{if($1~/URL/) print $3}'|tr -d '\/'`
		fi
		su - $RUN_USER -c "cd /usr/local/i386 && echo p|svn co https://$REPOS_ADDRESS:63579/svn/public_repos"
		if [ "$?" -eq 0 ];then
			INSERT_MYSQL "Data_sync:check out repos public_repos successful."
		else
			INSERT_MYSQL "Data_sync:check out repos public_repos failed."
		fi
	fi

	[ ! -d /dev/sysop ] && mkdir /dev/sysop

	# update repos
	for REPOS_NAME in $UP_REPOS_NAME
	do

	# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

		# unset UPDATE_VERSION
		unset UPDATE_VERSION

		# sleep random time
		sleep $((RANDOM%15))

		# redefine variable REPOS_DIR
		REPOS_DIR=/usr/local/i386/$REPOS_NAME

		# get revision file function
		wget_action () {
			wget --http-user=manifold --http-passwd=dw_manifold --output-document=$LOG_DIR/$1 --timeout=5 --tries=1 --quiet "http://$2:63217/revision/$REPOS_NAME.revision.html"
		}

		# control loop and check revision.
		control_loop_and_check_revision () {
			if [ "$REPOS_NAME" = "public_repos" ];then
				sleep $ACTION_INTERVAL
				[[ -f $COMM_DIR/auto/monitor_user_privileges.sh ]] && /bin/bash $COMM_DIR/auto/monitor_user_privileges.sh &
				[ -d /usr/local/i386/$REPOS_NAME ] && [ ! -d /usr/local/i386/$REPOS_NAME/initial_system ] && rm -rf /usr/local/i386/$REPOS_NAME
			fi
			ALARM_THRESHOLD=3
			if [ -n "$UPDATE_VERSION" ];then
				get_local_revision
				if [ `expr $UPDATE_TO_REVISION - $LOCAL_REVISION` -ge "$ALARM_THRESHOLD" ];then
					INSERT_MYSQL "Data_sync:$REPOS_NAME:LOCAL_REVISION:$LOCAL_REVISION not equal API_REVISION:$UPDATE_TO_REVISION,may be data sync error,please check."
				fi
			fi
			continue
		}

		# get remote revision
		get_remote_revision () {
			API_ROLE="$1"
			for net_flag in $INTERNAL_IP "DXLT"; do
				if [ "$net_flag" = "DXLT" ];then
					api_address_array=($(grep "$API_ROLE" $COMM_DIR/initial_system/repos_server.log|awk '{print $1}'))
				else
					api_address_array=($(grep "internal" $COMM_DIR/initial_system/repos_server.log|grep "$API_ROLE"|awk '{print $1}'))
				fi
				until [ "${#api_address_array[*]}" -eq 0 ];do
					# gather all array element as a new array(regenerate a new array).
					api_address_array=(${api_address_array[*]})
					# calculate the amount of the new array's element.
					api_address_array_num=${#api_address_array[*]}
					# get a random element.
					api_num_index=$((RANDOM%api_address_array_num))
					API_ADDRESS=${api_address_array[$api_num_index]}
					# drop the element which already used.
					unset api_address_array[$api_num_index]
					echo "${API_ROLE}_array_num[$api_address_array_num]:$API_ADDRESS"
					# for api alarm message which send to mysql.
					eval API_ADDRESS_"$API_ROLE"=$API_ADDRESS
					if [ "$API_ROLE" = "api1" ];then
						REVISION_LOG_FILE=$REPOS_NAME.revision.1.log
						API_ADDRESS_API1=API_ADDRESS_"$API_ROLE"
						API_ADDRESS_API1=${!API_ADDRESS_API1}
					elif [ "$API_ROLE" = "api2" ];then
						REVISION_LOG_FILE=$REPOS_NAME.revision.2.log
						API_ADDRESS_API2=API_ADDRESS_"$API_ROLE"
						API_ADDRESS_API2=${!API_ADDRESS_API2}
					fi
					wget_action $REVISION_LOG_FILE $API_ADDRESS && return
				done
			done
		}
		get_remote_revision api1
		get_remote_revision api2

		# get local revision
		get_local_revision () {
			# LOCAL_REVISION=`su - $RUN_USER -c "export LANG=C;svn info /usr/local/i386/$REPOS_NAME"|awk -F ': ' '{if($1~/Last Changed Rev/) print $2}'`
			LOCAL_REVISION=`cat /usr/local/i386/$REPOS_NAME/.svn/entries|grep https:// -B1|grep -v https`
			echo $LOCAL_REVISION|grep "^[0-9]*$"
			GREP_RESULT="$?"
			if [ "$LOCAL_REVISION" = "" ] || [ "$GREP_RESULT" -ne 0 ];then
				LOCAL_REVISION=`su - $RUN_USER -c "export LANG=C;svn info /usr/local/i386/$REPOS_NAME"|awk -F ': ' '{if($1~/Last Changed Rev/) print $2}'`
				if [ "$LOCAL_REVISION" = "" ];then
					LOCAL_REVISION=`su - $RUN_USER -c "export LC_ALL=en_US.UTF-8;svn info /usr/local/i386/$REPOS_NAME"|awk -F ': ' '{if($1~/Last Changed Rev/) print $2}'`
					if [ "$LOCAL_REVISION" = "" ];then
						SVN_INFO_LINE=`su - $RUN_USER -c "export LANG=C;svn info /usr/local/i386/$REPOS_NAME"|wc -l`
						if [ "$SVN_INFO_LINE" -eq 8 ];then
							LOCAL_REVISION=`su - $RUN_USER -c "export LANG=C;svn info /usr/local/i386/$REPOS_NAME"|awk -F ': ' '{if($1~/Revision/) print $2}'`
							LOCAL_REVISION=$(($LOCAL_REVISION-1))
						fi
					fi
				fi
			fi
		}
		get_local_revision

		# if variable $LOCAL_REVISION is null,step to next loop.
		if [ -z "$LOCAL_REVISION" ];then
			INSERT_MYSQL "Data_sync:repos $REPOS_NAME LOCAL_REVISION null."
			sleep $ACTION_INTERVAL
			break
		fi

		# judge repository must be update or not,and if must be do,when the rollback revision more than 'threshold values',refuse to rollback.
		REPOS_UPDATE_OR_NOT () {
			if [ "$LOCAL_REVISION" -ne "$UPDATE_TO_REVISION" ];then
				# threshold value
				THRESHOLD_VALUE_ROLLBACK="5"
				if [ `expr $LOCAL_REVISION - $UPDATE_TO_REVISION` -ge "$THRESHOLD_VALUE_ROLLBACK" ];then
					INSERT_MYSQL "Data_sync:Refuse rollback $REPOS_NAME's revision from $LOCAL_REVISION to $UPDATE_TO_REVISION."
				else
					UPDATE_VERSION="$UPDATE_TO_REVISION"
				fi
			else
				# UPDATE_VERSION=""
				echo "`date +%Y-%m-%d-%T` $BASENAME $REPOS_NAME's LOCAL_REVISION:$LOCAL_REVISION equal REMOTE_REVISION:$UPDATE_TO_REVISION"
			fi
		}

		# if API1 web server and API2 web server normal,fetch revision data and compare them
		if [ -s $LOG_DIR/$REPOS_NAME.revision.1.log ] && [ -s $LOG_DIR/$REPOS_NAME.revision.2.log ];then
			if [ "`cat $LOG_DIR/$REPOS_NAME.revision.1.log`" = "`cat $LOG_DIR/$REPOS_NAME.revision.2.log`" ];then
				eval RECHECK_UPDATE_VERSION_TIMES_"$REPOS_NAME"=0
				UPDATE_TO_REVISION=`cat $LOG_DIR/$REPOS_NAME.revision.1.log`
				REPOS_UPDATE_OR_NOT
			else
				# if revison not match,recheck it next time.
				RECHECK_UPDATE_VERSION_TIMES_REPOS_NAME=RECHECK_UPDATE_VERSION_TIMES_"$REPOS_NAME"
				echo RECHECK_UPDATE_VERSION_TIMES_"$REPOS_NAME":${!RECHECK_UPDATE_VERSION_TIMES_REPOS_NAME}
				if [ "${!RECHECK_UPDATE_VERSION_TIMES_REPOS_NAME}" -le 1 ];then
					eval RECHECK_UPDATE_VERSION_TIMES_"$REPOS_NAME"=`expr ${!RECHECK_UPDATE_VERSION_TIMES_REPOS_NAME} + 1`
					# if repos is public_repos,sleep long time.because of the first repos $REPOS_NAME.
					control_loop_and_check_revision
				else
					# RECHECK_UPDATE_VERSION_TIMES=0
					# UPDATE_VERSION=""
					INSERT_MYSQL "Data_sync:Repos revision about $REPOS_NAME not match,API1:$API_ADDRESS_API1:`cat $LOG_DIR/$REPOS_NAME.revision.1.log`,API2:$API_ADDRESS_API2:`cat $LOG_DIR/$REPOS_NAME.revision.2.log`,Local:$LOCAL_REVISION"
				fi
			fi
		# if API1 web server error,use the API2's web revision data,and vice versa.
		elif [ ! -s $LOG_DIR/$REPOS_NAME.revision.1.log ] && [ -s $LOG_DIR/$REPOS_NAME.revision.2.log ];then
			INSERT_MYSQL "Data_sync:Revision error:API1:$API_ADDRESS_API1 revision file about $REPOS_NAME null."
			eval RECHECK_UPDATE_VERSION_TIMES_"$REPOS_NAME"=0
		#	UPDATE_TO_REVISION=`cat $LOG_DIR/$REPOS_NAME.revision.2.log`
		#	REPOS_UPDATE_OR_NOT
		elif [ -s $LOG_DIR/$REPOS_NAME.revision.1.log ] && [ ! -s $LOG_DIR/$REPOS_NAME.revision.2.log ];then
			INSERT_MYSQL "Data_sync:Revision error:API2:$API_ADDRESS_API2 revision file about $REPOS_NAME null."
			eval RECHECK_UPDATE_VERSION_TIMES_"$REPOS_NAME"=0
		#	UPDATE_TO_REVISION=`cat $LOG_DIR/$REPOS_NAME.revision.1.log`
		#	REPOS_UPDATE_OR_NOT
		elif [ ! -s $LOG_DIR/$REPOS_NAME.revision.1.log ] && [ ! -s $LOG_DIR/$REPOS_NAME.revision.2.log ];then
			eval RECHECK_UPDATE_VERSION_TIMES_"$REPOS_NAME"=0
			# UPDATE_VERSION=""
			INSERT_MYSQL "Data_sync:Two repos version file about $REPOS_NAME null,please check network and system."
			GET_LOCAL_IP
		fi

		# if variable $UPDATE_VERSION is null,step to next loop.
		if [ -z "$UPDATE_VERSION" ];then
			FORCE_UPDATE_TIME_REPOS_NAME=FORCE_UPDATE_TIME_"$REPOS_NAME"
			echo FORCE_UPDATE_TIME_"$REPOS_NAME":${!FORCE_UPDATE_TIME_REPOS_NAME}
			if [ "${!FORCE_UPDATE_TIME_REPOS_NAME}" -ge 150 ];then
				eval FORCE_UPDATE_TIME_"$REPOS_NAME"=0
				UPDATE_VERSION=$LOCAL_REVISION
			else
				eval FORCE_UPDATE_TIME_"$REPOS_NAME"=`expr ${!FORCE_UPDATE_TIME_REPOS_NAME} + 1`
				control_loop_and_check_revision
			fi
		else
			eval FORCE_UPDATE_TIME_"$REPOS_NAME"=0
		fi

		# repository ip address
		# REPOS_ADDRESS=`svn info $REPOS_DIR|awk -F ':' '{if($1~/URL/) print $3}'|tr -d '\/'`
		REPOS_ADDRESS=`cat $REPOS_DIR/.svn/entries|grep https://|awk -F ':' '{print $2}'|tr -d '\/'|uniq|head -n 1`
		echo $REPOS_ADDRESS|grep -E "$DOMAIN_NAME|[[:digit:]]*\.[[:digit:]]*\.[[:digit:]]*\.[[:digit:]]*"
		GREP_RESULT="$?"
		if [ "$REPOS_ADDRESS" = "" ] || [ "$GREP_RESULT" -ne 0 ];then
			REPOS_ADDRESS=`svn info $REPOS_DIR|awk -F ':' '{if($1~/URL/) print $3}'|tr -d '\/'`
		fi
		REPOS_OLD_ADDRESS=$REPOS_ADDRESS

		# retry update function
		REPO_UPDATE_DETECT () {

			# check repos address in /etc/hosts(mon_sysop_manager.sh also have this function)
			REPOS_HOST_LINE=`cat /etc/hosts|grep -w "$DOMAIN_NAME"|wc -l`
			if [ "$REPOS_HOST_LINE" -eq 0 ];then
				echo "$REPOS_ADDRESS $DOMAIN_NAME" >> /etc/hosts
			elif [ "$REPOS_HOST_LINE" -gt 1 ];then
				sed -i '/.*[[:space:]]\+[[:alnum:] ]*'"${DOMAIN_NAME}"'/d' /etc/hosts
				echo "$REPOS_ADDRESS $DOMAIN_NAME" >> /etc/hosts
			fi

			# get real repos address
			REPOS_ADDRESS=`cat /etc/hosts|grep -w "$DOMAIN_NAME"|awk '{print $1}'`

			SELECT_ROLE="repos"
			address_array=($(cat $COMM_DIR/initial_system/repos_server.log|grep -v $REPOS_ADDRESS|grep "$SELECT_ROLE"|awk '{print $1}'))
			until [ "${#address_array[*]}" -eq 0 ];do
				# gather all array element as a new array(regenerate a new array).
				address_array=(${address_array[*]})
				# calculate the amount of the new array's element.
				address_array_num=${#address_array[*]}
				# get a random element.
				num_index=$((RANDOM%address_array_num))
				ADDRESS=${address_array[$num_index]}
				# drop the element which already used.
				unset address_array[$num_index]
				echo "svn_array_num[$address_array_num]:$ADDRESS"

				# change repos address in /etc/hosts
				sed -i 's/.*[[:space:]]\+[[:alnum:] ]*'"${DOMAIN_NAME}"'/'"${ADDRESS}"' '"${DOMAIN_NAME}"'/' /etc/hosts
				RESULT="$?"

				# svn cleanup
				su - $RUN_USER -c "cd $REPOS_DIR;svn cleanup"

				# for old version compatable
				if [ "$REPOS_OLD_ADDRESS" != "$DOMAIN_NAME" ];then
					su - $RUN_USER -c "cd $REPOS_DIR;echo p|svn switch --relocate https://$REPOS_OLD_ADDRESS:63579/svn/$REPOS_NAME https://$DOMAIN_NAME:63579/svn/$REPOS_NAME --username=clientupdatea276c728eccbae78089bcc08e0658cae --password=e40a3775ad30455352775a4a144704c7"
					RESULT="$?"
				fi

				if [ "$RESULT" -eq 0 ];then
					INSERT_MYSQL "Data_sync:Repos $REPOS_NAME changed,from $REPOS_ADDRESS to $ADDRESS."
					su - $RUN_USER -c "export LANG=C;export LC_ALL=C;cd $REPOS_DIR;time svn update -r $UPDATE_VERSION" && break
				fi
			done
		}

		# svn update action.
		su - $RUN_USER -c "export LANG=C;export LC_ALL=C;cd $REPOS_DIR;svn update -r $UPDATE_VERSION"
		# if update failed,retry once.
		if [ "$?" -ne 0 ];then
			sleep 8
			REPOS_UPDATE_STATUS_LOG_FILE=$LOG_DIR/repository_update.log
			su - $RUN_USER -c "export LANG=C;export LC_ALL=C;cd $REPOS_DIR;svn cleanup;sleep 2;svn update -r $UPDATE_VERSION" > $REPOS_UPDATE_STATUS_LOG_FILE 2>&1
			if [ "$?" -ne 0 ];then

				# check whether the revision which user commit in web api is wrong or other problem.
				function check_error_situation () {
					FILTER_MESSAGE="$1"
					case "$FILTER_MESSAGE" in
						"delete-entry")
							ERROR_FILE=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "delete-entry"|awk '{print $NF}'|tr -d \'`
							INSERT_MYSQL_MESSAGE="Data_sync:$REPOS_NAME:Unable to lock $ERROR_FILE."
							;;
						"No space left")
							INSERT_MYSQL_MESSAGE="Data_sync:$REPOS_NAME:no space left on device."
							;;
						"svn: No such revision")
							INSERT_MYSQL_MESSAGE="Data_sync:No such revison $UPDATE_VERSION about $REPOS_NAME,please check web api."
							;;
						"Conflict discovered in")
							ERROR_FILE=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "Conflict discovered in"|awk '{print $NF}'|sed 's/\.$//'|tr -d \'`
							INSERT_MYSQL_MESSAGE="Data_sync:$REPOS_NAME:Conflict discovered in $ERROR_FILE.action:del file"
							;;
						"not under version control")
							ERROR_FILE=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "not under version control" |awk '{print $2}'|tr -d \'`
							INSERT_MYSQL_MESSAGE="Data_sync:$REPOS_NAME:$ERROR_FILE is not under version control.action:del it"
							;;
						"not a working copy directory")
							ERROR_FILE=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "not a working copy directory" |awk '{print $2}'|tr -d \'`
							INSERT_MYSQL_MESSAGE="Data_sync:$REPOS_NAME:$ERROR_FILE is not a working copy directory.action:del it"
							;;
						"object of the same name already exists")
							ERROR_FILE=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "object of the same name already exists"|awk -F ':' '{print $2}'|awk -F \' '{print $2}'|tr -d \'`
							INSERT_MYSQL_MESSAGE="Data_sync:$REPOS_NAME:object named $ERROR_FILE already exists.action:del file"
							;;
						"an unversioned .* of the same name already exists")
							ERROR_FILE=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "an unversioned .* of the same name already exists"|awk -F ':' '{print $2}'|awk '{print $NF}'|tr -d \'`
							INSERT_MYSQL_MESSAGE="Data_sync:$REPOS_NAME:an unversioned file or directory named $ERROR_FILE already exists.action:del it"
							;;
					esac

					# force sync or not.
					if [ "$FORCE_SYNC" = "yes" ];then
						if [ "$ERROR_FILE" != "" ];then
							cd $REPOS_DIR && rm -rf $ERROR_FILE
						fi
					fi
					INSERT_MYSQL "$INSERT_MYSQL_MESSAGE"
					control_loop_and_check_revision
				}

				ERROR_SITUATION_1=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "delete-entry"|wc -l`
				ERROR_SITUATION_2=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "No space left"|wc -l`
				ERROR_SITUATION_3=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "svn: No such revision"|wc -l`
				ERROR_SITUATION_4=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "Conflict discovered in"|wc -l`
				ERROR_SITUATION_5=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "not under version control"|wc -l`
				ERROR_SITUATION_6=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "not a working copy directory"|wc -l`
				ERROR_SITUATION_7=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "object of the same name already exists"|wc -l`
				ERROR_SITUATION_8=`cat $REPOS_UPDATE_STATUS_LOG_FILE|grep "an unversioned .* of the same name already exists"|wc -l`

				if [ "$ERROR_SITUATION_1" -gt 0 ];then
					check_error_situation "delete-entry"
				elif [ "$ERROR_SITUATION_2" -gt 0 ];then
					check_error_situation "No space left"
				elif [ "$ERROR_SITUATION_3" -gt 0 ];then
					check_error_situation "svn: No such revision"
				elif [ "$ERROR_SITUATION_4" -gt 0 ];then
					check_error_situation "Conflict discovered in"
				elif [ "$ERROR_SITUATION_5" -gt 0 ];then
					check_error_situation "not under version control"
				elif [ "$ERROR_SITUATION_6" -gt 0 ];then
					check_error_situation "not a working copy directory"
				elif [ "$ERROR_SITUATION_7" -gt 0 ];then
					check_error_situation "object of the same name already exists"
				elif [ "$ERROR_SITUATION_8" -gt 0 ];then
					check_error_situation "an unversioned .* of the same name already exists"
				else
					REPO_UPDATE_DETECT
					if [ "$RESULT" -ne 0 ];then
						ping $ADDRESS -w 4 > $LOG_DIR/repository_update.sh.ping.log
						SVN_LATENCY=`cat $LOG_DIR/repository_update.sh.ping.log|awk -F '/' '{if($2~/avg/) print $5" ms"}'`
						PACKAGE_LOSS=`cat $LOG_DIR/repository_update.sh.ping.log|awk '{if($2~/packets/) print $6" loss"}'`
						REPOS_ADDRESS=$ADDRESS
						INSERT_MYSQL "Data_sync:Repos $REPOS_NAME up failed!!! CONN => $REPOS_ADDRESS. Loss: $PACKAGE_LOSS. Rtt: $SVN_LATENCY."
					fi
				fi
			else
				control_loop_and_check_revision
			fi
		else
			control_loop_and_check_revision
		fi

	# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	done

	# sleep $ACTION_INTERVAL

	# del useless log
	# find $LOG_DIR -name "sysop_manager*log" -mtime +60 -exec rm -fr {} \;

# ##########################################################################################
# ##########################################################################################

done >> $LOG_DIR/sysop_manager.log 2>&1
