#!/bin/bash
#VERSION:1.3

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games
export LANG=en_US.utf8
export LANGUAGE=en_US.utf8
export LANG=LC_ALL

# basic define
BASENAME=`basename $0`
SHORT_NAME=router
LOG_DIR=/var/log/sysop_manager

# repository path
REPOS_NAME=comm_repos
COMM_DIR=/usr/local/i386/$REPOS_NAME
PUBLIC_DIR=/usr/local/i386/public_repos
ADDR_FAMILY=/home/dspeak/release/bin/addr_family.log

# stop run configure
if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || [ -f $LOG_DIR/stop_run_$BASENAME ] || [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

[ "$(grep -cE "buss_name.*VPN|sysop.*dw_yangzhilin" /home/dspeak/yyms/hostinfo.ini)" = "2" ] && exit 1

# check linux version
cat /etc/issue|grep -i -E "CentOS|Red" >/dev/null 2>&1 && LINUX_VERSION="centos"
cat /etc/issue|grep -i -E "Ubuntu|Debian" >/dev/null 2>&1 && LINUX_VERSION="ubuntu"

# get ip address
IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10\.|inet addr:169\.254"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
if [ "$IPADDR" = "" ];then
    IPADDR=`/sbin/ifconfig|grep "inet addr:"|grep -vE "127.0.0.1|169\.254" |awk -F 'Bcast' '{print $1}'|awk -F : '{print $NF}'`
fi

# exclude list
cat $COMM_DIR/initial_system/exclude_list.log|grep -v "^#" | grep "^$IPADDR[[:space:]]\+[[:alnum:] ]*$SHORT_NAME" && exit 1

BACKUP_DATE=`date +%Y-%m-%d-%H-%M-%S`

# define insert alert function
INSERT_MYSQL () {
/bin/bash $COMM_DIR/auto/insert_message.sh "$BASENAME" "$1"
}

# delete "exit 0" from /etc/rc.local
# 2011-04-19 remove the function delete "exit" entry.
# sed -i '/exit.*0/d' /etc/rc.local

detect_file () {
	if [ ! -f $S_FILE ];then
		INSERT_MYSQL "no file $S_FILE."
	elif [ ! -f $D_FILE ];then
		/bin/cp $S_FILE $D_FILE
		if [ "$?" -eq 0 ];then
			CHANGE=yes
		else
			INSERT_MYSQL "add file $D_FILE failed."
		fi
	else
		S_MD5=`md5sum $S_FILE|awk '{print $1}'`
		D_MD5=`md5sum $D_FILE|awk '{print $1}'`
		if [ "$S_MD5" != "$D_MD5" ];then
			BACKUP_FILE="`echo $D_FILE|sed 's#/#_#g'`.$BACKUP_DATE"
			[ "$1" = "nobackup" ] || /bin/cp $D_FILE $LOG_DIR/$BACKUP_FILE
			cat $S_FILE > $D_FILE
			if [ "$?" -eq 0 ];then
				CHANGE=yes
			else
				INSERT_MYSQL "$D_FILE changed failed"
			fi
		fi
	fi
}

#########################################################################
# ip db file for yy process (in all server)
# CncIpDb.txt CmccIpDb.txt EduIpDb.txt 仅作保留，后续版本不会再使用
[ ! -d /home/dspeak/release/bin ] && mkdir -p /home/dspeak/release/bin

S_FILE=$PUBLIC_DIR/initial_system/cmdb/all_ipinfo.log
D_FILE=/home/dspeak/release/bin/all_ipinfo.log
detect_file nobackup

unset CHANGE
S_FILE=$PUBLIC_DIR/initial_system/cmdb/addr_family.log
D_FILE=$ADDR_FAMILY
detect_file nobackup

S_FILE=$PUBLIC_DIR/initial_system/route/CncRouteDb.txt
D_FILE=$LOG_DIR/CncRouteDb.txt
detect_file nobackup

##########################################################################

# last check,two ip address
ROUTE_LINE=0
IP_NUM=`/sbin/ifconfig|grep Bcast|grep -vE "inet addr:192\.|inet addr:172\.|inet addr:10\.|inet addr:169\.254"|awk -F '.' '{print $1,$2,$3}'|sort -u|wc -l`
if [ "$IP_NUM" -ge 2 ];then
	ROUTE_LINE=`route -n |grep -w "0.0.0.0"|grep -v -E "^192\.|^10\.|^172\."|wc -l`
fi

# judge in which list,and genetate route script for system
if [ "$ROUTE_LINE" -gt 2 ];then
	IP_ROUTE_MD5=`/sbin/ifconfig|grep Bcast|awk '{print $2,$NF}'|grep -v -E "^192\.|^172\.|^10\.|169\.254"|sort|md5sum|awk '{print $1}'`
	LOCAL_MD5=`cat $LOG_DIR/route_ip.md5`
	echo "$IP_ROUTE_MD5" > $LOG_DIR/route_ip.md5
	
	if [[ "$CHANGE" == "yes" ]] || [[ "$LOCAL_MD5" != "$IP_ROUTE_MD5" ]] || [[ ! -s /root/system_router.sh ]];then
		DEFAULT_GW_NUM=`/sbin/route -n|grep "^0\.0\.0\.0"|wc -l`
		if [ $DEFAULT_GW_NUM -ne 1 ];then
			if [ $DEFAULT_GW_NUM -eq 0 ];then
				# 默认网关不见了, why？
				INSERT_MYSQL "default gateway no found."
				[[ "$LINUX_VERSION" == "ubuntu" ]] && NO_DEFAULT_GW=`grep "gateway" /etc/network/interfaces |sort -u|awk '{print $2}'`
				[[ "$LINUX_VERSION" == "centos" ]] && NO_DEFAULT_GW=`grep "GATEWAY" /etc/sysconfig/network-scripts/ifcfg-eth* |sort -u|awk -F '=' '{print $2}'`
				# 增加默认路由
				[[ "$NO_DEFAULT_GW" != "" ]] && /sbin/route add -net 0.0.0.0 netmask 0.0.0.0 gw $NO_DEFAULT_GW
				[[ $? -eq 0 ]] && INSERT_MYSQL "default gateway no found. and try to add default route rule , gw:$NO_DEFAULT_GW"
				DEFAULT_GW=$NO_DEFAULT_GW
			else
				INSERT_MYSQL "found $DEFAULT_GW_NUM default gateway."
				# choice the smallest isp num as default gateway
				for ip_address in `/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10\.|inet addr:169\.254"|awk '{print $2}'|awk -F : '{print $2}'`;do
					GATEWAY_SUFFIX="${ip_address%.*}"
					ISP_NUM=$(grep -m1 -w "$GATEWAY_SUFFIX" $ADDR_FAMILY | awk '{print $4}')
					[ -z "$ISP_NUM" ] && continue
					if [ -z "$SMALLEST_ISP" ] || [ "$ISP_NUM" -lt "$SMALLEST_ISP" ]; then
						SMALLEST_ISP=$ISP_NUM
						DEFAULT_GW=$ip_address
					fi
				done
				if [ -z "$DEFAULT_GW" ]; then
					DEFAULT_GW=`/bin/bash $COMM_DIR/other/calc_gateway.sh|awk -F ':' '{print $2}'`
				fi
				for which_del_gw in `/sbin/route -n|grep "^0\.0\.0\.0"|awk '{print $2}'`;do
					[ "$which_del_gw" = "$DEFAULT_GW" ] && continue
					route del -net 0/0 gw $which_del_gw
				done
				INSERT_MYSQL "found $DEFAULT_GW_NUM default gateway, and choice $DEFAULT_GW as default"
			fi
		else
			DEFAULT_GW=`/sbin/route -n | grep "^0\.0\.0\.0" | awk '{print $2}'`
		fi
		DEFAULT_GW_SUFFIX=`echo ${DEFAULT_GW%.*}`
		
		cat /etc/rc.local|grep /root/system_router.sh
		if [ "$?" -ne 0 ];then
			echo "/bin/bash /root/system_router.sh" >> /etc/rc.local
		fi
		export ROUTE_DIR=`which route`
		echo > /root/system_router.tmp
		
		(
cat << 'EOF' >> /root/system_router.tmp

NOW_MD5=`/sbin/ifconfig|grep Bcast|awk '{print $2,$NF}'|grep -v -E "^192\.|^172\.|^10\.|169\.254"|sort|md5sum|awk '{print $1}'`
LOCAL_MD5=`cat /var/log/sysop_manager/route_ip.md5`
[ "$LOCAL_MD5" != "$NOW_MD5" ] && exit 1

EOF
)
		
		for ip_address in `/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10\.|inet addr:169\.254"|awk '{print $2}'|awk -F : '{print $2}'`;do
			ip_address_suffix=`echo ${ip_address%.*}`;
			ip_address_suffix_suffix=`echo $ip_address | awk -F '.' '{print $1"."$2}'`
			
			TMP_ISP=$(grep -m1 -w "$ip_address_suffix" $ADDR_FAMILY | awk '{print $4}')
			if [ -z "$TMP_ISP" ]; then
				TMP_ISP=$(grep -m1 -w "$ip_address_suffix_suffix" $ADDR_FAMILY | awk '{print $4}')
			fi
			[ -z "$TMP_ISP" ] && continue;
			
			[ "$ip_address_suffix" = "$DEFAULT_GW_SUFFIX" ] && UPDATE_ISP="$UPDATE_ISP $TMP_ISP" && continue;
			echo "$UPDATE_ISP" | grep -wq "$TMP_ISP" && continue;
			
			CHOICE_IP_MASK=$(/sbin/ifconfig|grep -m1 "${ip_address_suffix}.*Mask:" | awk -F ':' '{print $2,$4}'|awk '{print $1,$3}')
			CHOICE_IP="${CHOICE_IP_MASK% *}"
			CHOICE_MASK="${CHOICE_IP_MASK#* }"
			CHOICE_GATEWAY=$(/bin/bash $COMM_DIR/other/calc_gateway.sh $CHOICE_IP $CHOICE_MASK | awk -F ':' '{print $2}')
			
			if [ "$TMP_ISP" = "5" ];then
				for CNC_NET in `cat $PUBLIC_DIR/initial_system/route/CncRouteDb.txt`
				do
					echo "$ROUTE_DIR add -net $CNC_NET gw $CHOICE_GATEWAY" >> /root/system_router.tmp
				done
			else
				for IP_NET in `awk '{if($4 == '$TMP_ISP'){printf("%s/%s\n",$1,$2)}}' $ADDR_FAMILY`
				do
					NET_SUFFIX="${IP_NET#*/}"
					NET_IP=$(echo $IP_NET| awk -F '.' '{printf("%d.%d.%d.0",$1,$2,$3)}')
					if [ "$NET_SUFFIX" = "32" ];then
						echo "$ROUTE_DIR add -net ${NET_IP}/0 gw $CHOICE_GATEWAY" >> /root/system_router.tmp
					else
						echo "$ROUTE_DIR add -net $IP_NET gw $CHOICE_GATEWAY" >> /root/system_router.tmp
					fi
				done
			fi
			UPDATE_ISP="$UPDATE_ISP $TMP_ISP"
		done
	
(
cat <<'EOF' >> /root/system_router.tmp
EXIST_ROUTE_LINE=$(route -n |wc -l)
SHOULD_EXIST=$(grep -c 'route add' $0)
SHOULD_EXIST=$(echo $SHOULD_EXIST|awk '{print int(0.9*$1)}')
[ $EXIST_ROUTE_LINE -ge $SHOULD_EXIST ] && RESULT=0 || RESULT=1
if [ -f /root/system_router_add.sh ];then
    /bin/bash /root/system_router_add.sh
fi
echo RESULT:$RESULT
EOF
)
		# 到此，新的静态路由表已经生成。删除就的路由缓冲
		# 为了防止路由不规范添加，导致删除了别人的路由。只能删除自己的路由
		grep 'route.*add.*gw' /root/system_router.sh |sed "s/add/del/g" > $LOG_DIR/del_route.log.$BACKUP_DATE
		/bin/bash $LOG_DIR/del_route.log.$BACKUP_DATE
		route -n|grep -v -w 0.0.0.0|grep -v "Kernel.*table"|grep -v "Destination.*Gateway.*Genmask"|grep -v -E "^10\.|^192\.|^172\." > $LOG_DIR/del_route.log.$BACKUP_DATE
		cat $LOG_DIR/del_route.log.$BACKUP_DATE|awk '{print $1,$2,$3}'| while read DESTINATION GATEWAY GENMASK
		do
			grep -wq "$DESTINATION" /root/system_router.tmp || continue;
			route del -net $DESTINATION netmask $GENMASK gw $GATEWAY
		done
		
		mv /root/system_router.sh $LOG_DIR/system_router.sh.$BACKUP_DATE
		mv /root/system_router.tmp /root/system_router.sh
		
		# 应用新路由条目
		RESULT=`/bin/bash /root/system_router.sh|grep RESULT|awk -F ':' '{print $2}'`
		# /bin/sh /root/system_router_add.sh
		INSERT_MYSQL "RouteDB file changed,and exit code:$RESULT."
		
		[[ "$RESULT" != "0" ]] && echo > $LOG_DIR/route_ip.md5
	
	fi	

	# verify the result (if user commit reduplicate network entry,i used sort and uniq to ensure precise line)
	S_LINE=`cat /root/system_router.sh | grep "$ROUTE_DIR.*gw" |uniq|wc -l`
	if [ -f /root/system_router_add.sh ];then
		SYSTEM_ROUTE_ADD_LINE=`cat /root/system_router_add.sh|wc -l`
		S_LINE=`expr $S_LINE + $SYSTEM_ROUTE_ADD_LINE`
	fi
	# S_LINE=`expr $S_LINE + 5`
	# D_LINE=`route -n|wc -l`
	D_LINE=`route -n|grep -v -w 0.0.0.0|grep -v "Kernel.*table"|grep -v "Destination.*Gateway.*Genmask"|wc -l`
	if [ "$S_LINE" -ne "$D_LINE" ];then
		if [ "$D_LINE" -eq 0 ];then
			rm -f $LOG_DIR/route_ip.md5
		fi
		if [ "`expr $S_LINE - $D_LINE`" -gt 50 ];then
			INSERT_MYSQL "Warnning: `basename $PUBLIC_DIR/initial_system/route/CncRouteDb.txt` have $S_LINE lines,but route table heve $D_LINE lines,please check."
		fi
	fi
	
	# last check,two ip address
	SHOULD_EXIST=$(grep -c 'route add' /root/system_router.sh)
	SHOULD_EXIST=$(echo $SHOULD_EXIST|awk '{print int(0.9*$1)}')
	if [ -n "$D_LINE" ];then
		if [ "$D_LINE" -lt "$SHOULD_EXIST" ];then
			INSERT_MYSQL "Have double ip address,route line should:$SHOULD_EXIST,but now have $D_LINE."
		fi
	fi
	
	SHOULD_TYPE_GATEWAY=$(/sbin/ifconfig|grep Bcast|grep -vE "inet addr:192.168|inet addr:172.16|inet addr:10\.|inet addr:169\.254"| \
						awk '{print $2}'|awk -F ':' '{split($2,a,".");printf("%d.%d.%d\n",a[1],a[2],a[3])}'|sort -u |wc -l)
	NOW_TYPE_GATEWAY=$(route -n | grep -vE 'Kernel|Destination'| awk '{print $2}'| grep -vE '0.0.0.0|^192\.168|^10\.|^172\.'|sort -u|wc -l)
	[ "$NOW_TYPE_GATEWAY" != "$SHOULD_TYPE_GATEWAY" ] && echo > $LOG_DIR/route_ip.md5
	
	# Check if default gateway change or not
	DEFAULT_GW_CHANGE=`/sbin/route -n | grep "^0\.0\.0\.0" | awk '{print $2}'`
	grep 'route add.*gw' /root/system_router.sh| awk '{print $NF}'|sort -u|grep -wq "$DEFAULT_GW_CHANGE" && echo > $LOG_DIR/route_ip.md5
	
else
	# 单线服务器
	if [ -f /root/system_router.sh ];then
		sed -i "s/add/del/g" /root/system_router.sh
		grep "route del -net.*" /root/system_router.sh > /root/system_router.tmp
		/bin/bash /root/system_router.tmp
		rm -f /root/system_router.sh /root/system_router.tmp
		[[ -f /root/system_router_add.sh ]] && rm -f /root/system_router_add.sh
	fi
	cat /etc/rc.local|grep -q '/root/system_router.sh' && sed -i "/system_router.sh/d" /etc/rc.local
fi

# 内网加一条路由把10.0.0.0/8指到内网网关
inner_msg=`/sbin/ifconfig|grep Bcast | grep -m1 -E "addr:10\."`
if [ "$inner_msg" != "" ];then
	inner_ip=`echo ${inner_msg#*addr:}`;inner_ip=`echo ${inner_ip%% *}`
	inner_mask=`echo ${inner_msg##*Mask:}`
	inner_gateway=`/bin/bash $COMM_DIR/other/calc_gateway.sh $inner_ip $inner_mask | awk -F ':' '{print $2}'`
	route -n | grep -wq "^10\.0\.0\.0.*$inner_gateway"
	if [ $? -ne 0 ];then
		del_inner_gw=`route -n | grep "^10\.0\.0\.0"| awk '{print $2}'| tr -s "\n" " "`
		for del_gw_index in $del_inner_gw
		do
			INSERT_MYSQL "found 10.0.0.0 gw $del_gw_index not equal $inner_gateway"
			route del -net 10.0.0.0/8 gw $del_gw_index
		done
		route add -net 10.0.0.0/8 gw $inner_gateway
	elif [ "`route -n | grep -c "^10\.0\.0\.0"`" -ge 2 ];then
		del_inner_gw=`route -n | grep "^10\.0\.0\.0"|grep -wv "$inner_gateway" | awk '{print $2}'| tr -s "\n" " "`
		for del_gw_index in $del_inner_gw
		do
			INSERT_MYSQL "found 10.0.0.0 gw $del_gw_index not equal $inner_gateway"
			route del -net 10.0.0.0/8 gw $del_gw_index
		done
	fi
fi

echo
