#!/bin/bash
# Written by wuhaiting
# Date 2012-12-10
# Readme 收集服务器硬件信息

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
PUBLIC_DIR=/usr/local/i386/public_repos/
HARDWARE_INFO=/etc/hardware_info

cat /etc/issue|grep -i -E "ubuntu|debian" > /dev/null 2>&1 && LINUX_VERSION=ubuntu
cat /etc/issue|grep -i -E "centos|red" > /dev/null 2>&1 && LINUX_VERSION=centos

IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
IPADDR_ALL=`/sbin/ifconfig|grep "inet addr"|awk '{print $2}'|awk -F ":" '{print $2}'|grep -v -E "^127\."|tr -s "\n" " "`
MOUNT_DATA=`cat /etc/mtab|awk '{print $2}'|grep data |tr -s "\n" " "`

function cpu_info () {
	CPU_INFO_FILE=/proc/cpuinfo
	cpu_num=`cat $CPU_INFO_FILE|grep "physical id"|sort -u|wc -l`
	cpu_processor=`cat $CPU_INFO_FILE|grep "^processor"|wc -l`
	cpu_model=`cat $CPU_INFO_FILE|grep -w "model name"|awk -F ":" '{print $2}'|sed -r "s/^[[:space:]]+//"|tail -n1`
	_cpu_model=`echo $cpu_model|awk '{printf("%s%s%s",$(NF-2),$(NF-1),$NF)}'`
}

function mem_info () {
	system_mem=`cat /proc/meminfo|grep "^MemTotal"|awk '{print $(NF-1)/1024/1024 "GB"}'`
	physical_memory=`dmidecode | sed -n -r '/^[[:space:]]+Size/p' | grep "MB"|awk '{print $2}'`
	physical_memory_all=`echo $physical_memory|awk '{i=0;sum=0;while(i<NF){i++;sum=sum+$i};print(sum/1024)}'`
	physical_mem_type=`dmidecode | sed -n -r '/^[[:space:]]+Size/p' | grep "MB"|awk '{print $2}'|sort -u`
	for type in `echo $physical_mem_type`;do
		for type_num in `echo $physical_memory`;do
			if [ "$type_num" == "$type" ];then num=`expr $num + 1`;fi
		done
		physical_mem_type_info="$physical_mem_type_info ${type}MB*$num"
		unset num
	done
	unset 
}

function ethernet_info () {
	which ethtool > /dev/null 2>&1
	[[ "$?" -ne 0 ]] && apt-get -y install ethtool > /dev/null 2>&1
	eth_num=`lspci|grep -E "Ethernet controller:"|wc -l`
	eth_num=`expr $eth_num - 1`
	ethernet_driver=`lsmod|grep -E "^bnx|^e1000|^igb"|awk '{print $1}'|tr -s "\n" " "`
	for ethernet_num in `seq 0 $eth_num`;do
		eth_speed=`ethtool eth${ethernet_num} 2>/dev/null|grep "Speed:"|awk '{print $2}'`
		ethernet_speed="$ethernet_speed eth${ethernet_num}:${eth_speed}"
		unset eth_speed
	done
}

function os_info () {
	[[ "$LINUX_VERSION" == "ubuntu" ]] && SYSTEM_VERSION=`cat /etc/issue|head -n1|awk '{print $2}'`
	[[ "$LINUX_VERSION" == "centos" ]] && SYSTEM_VERSION=`cat /etc/issue|head -n1|awk '{print $(NF-1)}'`
	LINUX_BIT=`getconf LONG_BIT`
	KERNEL_VERSION=`uname -r`
	SWAP_ZONE=`swapon -s|grep -v "Filename"|wc -l`
	IS_DATA_ZONE=`cat /etc/mtab|awk '{if($2 ~ /\/data/){print "yes"}}'|tail -n1`
	
}

function disk_info () {
	tmp_file1=`mktemp /tmp/tmp1.XXXX`
	tmp_file2=`mktemp /tmp/tmp2.XXXX`
	tmp_file3=`mktemp /tmp/tmp3.XXXX`
	
	fdisk -l 2>/dev/null|grep -w "Disk /dev/.*:"|grep -v "MB"|awk '{printf("%s %s%s\n",$2,$3,$4)}'|tr -d "," > $tmp_file1
	# LSI MegaRaid array 
	if [ "`lspci|grep -E -w "RAID|SCSI"|grep -E -w "MegaRAID SAS|MegaSAS" > /dev/null && echo "MegaRAID"`" == "MegaRAID" ];then
		[[ ! -f /usr/sbin/megacli ]] && /bin/bash $PUBLIC_DIR/initial_system/monitor/mon_hardware.sh > /dev/null 2>&1
		/usr/sbin/megacli -ldpdinfo -aall > $tmp_file3
		disk_type_info=`cat $tmp_file3|grep "^Raw Size"|awk '{print $3}'|sort -u`
		for disk_type in `echo $disk_type_info`;do
			disk_type_num=`cat $tmp_file3|grep "^Raw Size"|grep -w "$disk_type"|wc -l`
			c_unit=`cat $tmp_file3|grep "^Raw Size"|grep -w "$disk_type"|sort -u|awk '{print $4}'`
			disk_type_and_num="$disk_type_and_num $disk_type$c_unit*$disk_type_num"
			unset disk_type_num
		done
			
		cat $tmp_file3|grep -E "^Raw Size:|^RAID Level|^Number Of Drives|^Span Depth|Device Speed|Inquiry Data|PD Type:"|awk -F '[ :]' '{
			if($0 ~ /RAID Level.*/){
				i=$(NF-4);
				j=$(NF-3);
				m=1;
				h="";
			}else if($0 ~ /Number Of Drives.*/){
				k=$NF;
			}else if($0 ~ /Span Depth.*/){
				l=$NF;
			}else if($0 ~ /PD Type:/){
				q=$NF;
			}else if($0 ~ /Raw Size/){
				n=$4$5;
			}else if($0 ~ /Inquiry Data/){
				for(g=1;g<=NF;g++){
					if($g ~ /INTEL/){
						if($(g+1) ~ /SSD.*/){
							h=" SSD";
						}
					}
				}
			}else if(m == 1 && $0 ~ /Device Speed/){
				if(l > 1){
					printf("%s*%s %s%s (RAID%s+%s) \n",n,k*l,q,h,i,"0");
					m=0;
				}else{
					printf("%s*%s %s%s (RAID%s) \n",n,k*l,q,h,i);
				};
				m=0;
			}
		}'|sed -e "s/"Primary-"//" -e "s/"Secondary-"//"|tr -d "," > $tmp_file2
	
	elif [ "`lspci|grep -E -w "RAID|SCSI"|grep -w "Smart Array" > /dev/null && echo "Smart_Array"`" == "Smart_Array" ];then
		[[ ! -f /usr/sbin/megacli ]] && /bin/bash $PUBLIC_DIR/initial_system/monitor/mon_hardware.sh > /dev/null 2>&1
		hpacucli ctrl all show config > $tmp_file3
		disk_type_info=`cat $tmp_file3|grep "physicaldrive"|awk '{for(n=1;n<=NF;n++){if($n ~ /GB,|TB,/){print $(n-1)}}}'|sort -u`
		for disk_type in `echo $disk_type_info`;do
			disk_type_num=`cat $tmp_file3|grep "physicaldrive"|awk '{for(n=1;n<=NF;n++){if($n ~ /GB,|TB,/){print $(n-1)}}}'|grep "$disk_type"|wc -l`
			c_unit=`cat $tmp_file3|grep "physicaldrive"|grep "$disk_type"|awk '{for(n=1;n<=NF;n++){if($n ~ /GB,|TB,/){print $n}}}'|sort -u|tail -n1|tr -d ","`
			disk_type_and_num="$disk_type_and_num $disk_type$c_unit*$disk_type_num"
			unset disk_type_num
		done
		
		cat $tmp_file3|awk '{
			if($1 ~ /logicaldrive/){
				i=0;
				j=$6;
				if(l == 1){
					printf("(RAID%s)\n",j);
				};
				l=1;
			};
			if($1 ~ /physicaldrive/){
				i++;
				for(n=1;n<=NF;n++){
					if($n ~ /GB,|TB,/){
						k=$(n-1)$n;
						g=$(n-2);
					}
				}
				l=0;
			};
			if(i != 0 && $0 ~ /^$/){
				printf("%s*%s %s (RAID%s)\n",k,i,g,j);
				i=0;
			};
		}'|tr -d "," > $tmp_file2
		
	else
		which lsiutil > /dev/null 2>&1
		if [ "$?" -eq 0 ];then
			lsiutil -p1 -a8,0 > $tmp_file3
			LSI_DISK_NUM=`cat $tmp_file3|grep -E "PhysDisk|Disk"|wc -l`
			FDISK_DISK_NUM=`cat $tmp_file1|wc -l`
			
			if [ "$FDISK_DISK_NUM" != "$LSI_DISK_NUM" ];then
				lsiutil -p1 -a21,1,0,0 > $tmp_file3
				cat $tmp_file3|grep -E "^Volume|Volume Size"|awk '{
					if($0 ~ /^Volume.*Type/){
						if($0 ~ /^Volume.*Type IM/){
							i="RAID1";
						}else{
							i="RAID0";
						};
					};
					if($0 ~ /Volume Size/){
						printf(" %sGB*%s(%s)",int($3/1024),$5,i);
					};
				}' > $tmp_file2
				
				disk_type_and_num=`cat $tmp_file2|awk -F "(" '{print $1}'`
				if [ "$disk_type_and_num" == "" ];then
					disk_type_info=`cat $tmp_file1|awk '{print $NF}'|sort -u`
					for disk_type in `echo $disk_type_info`;do
						disk_type_num=`cat $tmp_file1|awk '{print $NF}'|grep "$disk_type"|wc -l`
						disk_type_and_num="$disk_type_and_num $disk_type*$disk_type_num"
						unset disk_type_num
					done
				fi
				
			else
				disk_type_info=`cat $tmp_file1|awk '{print $NF}'|sort -u`
				for disk_type in `echo $disk_type_info`;do
					disk_type_num=`cat $tmp_file1|awk '{print $NF}'|grep "$disk_type"|wc -l`
					disk_type_and_num="$disk_type_and_num $disk_type*$disk_type_num"
					unset disk_type_num
				done
			fi
		else
			disk_type_info=`cat $tmp_file1|awk '{print $NF}'|sort -u`
			for disk_type in `echo $disk_type_info`;do
				disk_type_num=`cat $tmp_file1|awk '{print $NF}'|grep "$disk_type"|wc -l`
				disk_type_and_num="$disk_type_and_num $disk_type*$disk_type_num"
				unset disk_type_num
			done
		fi
	fi
	_PARTITION_DISK=`paste -d " " $tmp_file1 $tmp_file2`
	PARTITION_DISK=`paste -d " " $tmp_file1 $tmp_file2 |tr -d "\n"`
	rm -rf $tmp_file1 $tmp_file2 $tmp_file3
		
}

manufacturer=`dmidecode -s system-manufacturer|grep -v "^#"`
product=`dmidecode -s system-product-name|grep -v "^#"|sed "s/\-\[.*\]\-//"`
SN_NUMBER=`dmidecode -s system-serial-number|grep -v "^#"`

cpu_info
mem_info
ethernet_info
os_info
disk_info

echo "
Manufacturer 	: $manufacturer	
Product-name 	: $product	
serial-number 	: $SN_NUMBER
CPU 		: $cpu_model	
Processor 	: $cpu_processor
Physical_CPU 	: $cpu_num	
Physical_memory : $physical_mem_type_info	
Disk_info : 
$_PARTITION_DISK	

Ethernet 	: ($ethernet_driver)$ethernet_speed
IP_Address	: $IPADDR_ALL
OS_info 	: $LINUX_VERSION-$SYSTEM_VERSION($KERNEL_VERSION) $LINUX_BIT	
System_memory 	: $system_mem	
"    > $HARDWARE_INFO
	    

echo "$IPADDR | $product| $SN_NUMBER | $_cpu_model*$cpu_num |$physical_mem_type_info | $system_mem |$disk_type_and_num | $LINUX_VERSION-$SYSTEM_VERSION($KERNEL_VERSION) $LINUX_BIT |$ethernet_speed"
echo "$IPADDR_ALL| $MOUNT_DATA"
