#!/bin/bash

print_udev_check() {
(
cat << 'EOF'
#!/bin/sh
#@version: 1.3
#Wirtten by wuhaiting
#Date 2012-10-16
#Readme #服务器更换主板之后，由于udev之前根据mac地址生成的文件，导致物理网卡1跟eth0对应不上

PATH=/bin:/sbin:/usr/local/bin:/usr/local/sbin:/usr/bin:/usr/sbin
LOG_FILE=/var/log/sysop_manager/sysop_manager.log

COMMAND="$1"
MAC_FILE=/etc/udev/rules.d/70-persistent-net.rules
TMP_FILE=/tmp/mac.sdfes
CHANGE=no

if [ -z "$COMMAND" ];then
	echo "Usage: $0 COMMAND"
	exit 1
fi

cat /etc/issue|grep -E "CentOS|Red" && LINUX_VERSION="centos"
cat /etc/issue|grep -E "Ubuntu" && LINUX_VERSION="ubuntu"
if [ "$LINUX_VERSION" = "" ];then exit 1;fi

udev_clean_network_mac () {
	if [ "$LINUX_VERSION" = "ubuntu" ];then
		cat /etc/network/interfaces|grep -w "auto"|grep -w "eth[0-9]"|awk '{print $NF}'|sed "/^$/d"|grep -v ':' > $TMP_FILE
	elif [ "$LINUX_VERSION" = "centos" ];then
		ls /etc/sysconfig/network-scripts/ifcfg-eth[0-9]|awk -F "-" '{print $NF}'|sed "/^$/d" > $TMP_FILE
	fi
	for ETH_TMP in `cat $TMP_FILE`
	do
		cat /proc/net/dev|awk -F ":" '{print $1}'|grep -w "eth[0-9]"|grep $ETH_TMP > /dev/null 2>&1
		if [ "$?" -ne 0 ];then
			rm $MAC_FILE
			CHANGE=yes
			break
		fi
	done
		
	if [ "$CHANGE" = "yes" ];then
		[ -f /etc/sysconfig/network-scripts/ifcfg-eth0 ] && sed -i "/"HWADDR"/d" /etc/sysconfig/network-scripts/ifcfg-eth0
		[ -f /etc/sysconfig/network-scripts/ifcfg-eth1 ] && sed -i "/"HWADDR"/d" /etc/sysconfig/network-scripts/ifcfg-eth1
		[ -f /etc/network/interfaces ] && sed -i "/"hwaddress"/d" /etc/network/interfaces
		echo "$(date "+%Y-%m-%d %H:%M:%S") [step:$COMMAND] delete $MAC_FILE" >> $LOG_FILE
	fi
	if [ ! -s $MAC_FILE ] && [ "$CHANGE" = "yes" ];then
		for a in `cat $TMP_FILE`
		do
			grep -wq "$a" /proc/net/dev && echo add > /sys/class/net/$a/uevent
		done
	fi
}

check_igb_need_configure () {

# Because of some igb driver(eg: 4.0.17) won't open RSS default,
# so we need to manually open it. 
# Attention: Some igb driver won't have parm (eg:IntMode,RSS). So
# this functions is needed to check current kernel's igb dirver if
# support this param or not.

	TARGET_FILE="/etc/modprobe.d/igb.conf"
	GRUB_097_FILE="/boot/grub/menu.lst"
	GRUB_2_FILE="/boot/grub/grub.cfg"
	[ ! -s "$TARGET_FILE" ] && return 0
	[ -s $GRUB_097_FILE ] && GRUB_FILE=$GRUB_097_FILE || GRUB_FILE=$GRUB_2_FILE
	
	Boot_first_kernel=$(grep 'vmlinuz-' $GRUB_FILE | grep -m1 -vE '#| *#' \
						|awk -F 'vmlinuz-' '{split($2,a," ");print a[1]}')	
	
	Next_boot_use_igb=$(grep 'igb.ko' /lib/modules/${Boot_first_kernel}/modules.dep|cut -d ':' -f 1)
	
	[ ! -f /lib/modules/${Boot_first_kernel}/${Next_boot_use_igb} ] && return 1
	
	modinfo /lib/modules/${Boot_first_kernel}/${Next_boot_use_igb} > $TMP_FILE
	for which_param in $(grep 'options' $TARGET_FILE |awk '{$1="";$2="";print $0}'| tr -s '\n' ' ');
	do
		which_param="${which_param%%=*}"
		[ -z "$which_param" ] && continue
		
		if ! grep -q "parm.*${which_param}" $TMP_FILE ; then
			rm -f $TARGET_FILE
			[ $? -eq 0 ] && echo "$(date "+%Y-%m-%d %H:%M:%S") [step:$COMMAND] delete /etc/modprobe.d/igb.conf no parm $which_param" >> $LOG_FILE
			return 0
		fi
	done

}

reset_network_attribute () {
# Reset network-device attribute(eg: ringbuffer, tx_queue_len)

	for which_device in /sys/class/net/eth[0-9] ;
	do
		DEVICE="${which_device##*/}"
		ethtool -G $DEVICE rx 4096 tx 4096 \
			&& echo "$(date "+%Y-%m-%d %H:%M:%S") reset $DEVICE ringbuffer success" >> $LOG_FILE
		
		echo 8000 > $which_device/tx_queue_len \
			&& echo "$(date "+%Y-%m-%d %H:%M:%S") reset $DEVICE tx_queue_len success" >> $LOG_FILE
		
		if ! ethtool -i $DEVICE | grep -q "igb"; then
			[ -f /sys/class/net/$DEVICE/queues/rx-0/rps_cpus ] && \
				echo ff > /sys/class/net/$DEVICE/queues/rx-0/rps_cpus \
				&& echo "$(date "+%Y-%m-%d %H:%M:%S") reset $DEVICE rps_cpus success" >> $LOG_FILE
		fi
	
	done
}

case $COMMAND in
start|restart)
	udev_clean_network_mac;
	check_igb_need_configure;
	reset_network_attribute
	;;
stop)
	check_igb_need_configure
	;;
*)
	echo "Usage: $0 COMMAND"
	exit 1
esac
rm -f $TMP_FILE
exit 0

EOF
)
}

TARGET_FILE="/etc/init.d/udev_check"
NOW_MD5=$(print_udev_check | md5sum | cut -d ' ' -f 1)
OLD_MD5=$(md5sum $TARGET_FILE | cut -d ' ' -f 1)

if [ "$NOW_MD5" != "$OLD_MD5" ] ; then
	print_udev_check > $TARGET_FILE
	chmod 755 $TARGET_FILE
	ln -s $TARGET_FILE /etc/rc0.d/S02udev_check
	ln -s $TARGET_FILE /etc/rc1.d/S02udev_check
	ln -s $TARGET_FILE /etc/rc2.d/S02udev_check
	ln -s $TARGET_FILE /etc/rc3.d/S02udev_check
	ln -s $TARGET_FILE /etc/rc6.d/S02udev_check
fi