#!/bin/bash

# rewrite by boblin on 2014-07-07

# jifen file is write by linkd_d/sessiond . New
# file will be created everye 2 hours , so this
# script wake up every 30 minutes , and try to 
# sync the new-est jifen file to jifen server

ENABLE_INFO_LOG='yes'

ENABLE_DEBUG_LOG='yes'

ENABLE_NOTE_LOG='yes'

ENABLE_WARN_LOG='yes'

ENABLE_ERR_LOG='yes'


source /data/pkg/public-scripts/func/misc.sh

source /data/pkg/public-scripts/func/common.sh

source /data/pkg/public-scripts/func/common-log.sh

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

#-----------------------------------#
# keep log file size not exeecd 50M #
#-----------------------------------#

rotate_log /home/dspeak/rsync_jifen.log "50M"

(
   #------------------------#
   # delete old jifen files #
   #------------------------#

   info_short_log "cleaning old jifen files (>10 days)"
   
   for jifenDir in /home/dspeak/release/bin/old_jifen /data/yy/data/linkd_d/old_jifen /data/yy/data/old_jifen /home/dspeak/release/old_jifen; do
   
       find $jifenDir -type f -name 'jifen.*.*' -mtime +10 -exec rm -f {} \; 2>/dev/null
   
   done
   
   #--------------#
   # get local ip #
   #--------------#
   
   ip=$(get_local_ip eth0)
   
   : ${ip:=$(get_local_ip eth1)}
   
   : ${ip:=$(get_local_ip eth2)}
   
   info_short_log "ip=$ip"
   
   info_short_log "check isp type with file /var/log/sysop_manager/isp.log"
   
   debug_short_log "$(cat /var/log/sysop_manager/isp.log)"
   
   # check isp
   
   ispType=$(cat /var/log/sysop_manager/isp.log | head -n1)
   
   if [ "$ispType" = "5" ] ; then

	    # CNC
   
            rsyncServerList="58.249.117.105  122.13.149.246  111.206.234.120"
   else
	    # OTHER ISP

             rsyncServerList="121.14.39.137  121.14.43.217  106.38.255.155"
   fi
   
   info_short_log "ispType=$ispType , rsync server list = [ $rsyncServerList ]"
   
   #-----------------#
   # main work begin #
   #-----------------#

   allJifenDir=
   
   syncList=''
   
   for jifenDir in /home/dspeak/release/bin /data/yy/data/linkd_d ; do
   
      info_short_log
   
      info_short_log ">>> checking dir $jifenDir ...."	
   
      curJifenFileList=$(find $jifenDir -maxdepth 1 -type f -name 'jifen.*.*' 2>/dev/null)
   
      if [ -z "$curJifenFileList" ] ; then
   
           info_short_log "no jifen file found under dir $jifenDir , skip"
   
   	       continue
      else
    	   info_short_log "find jifen files under dir $jifenDir ...."
   
           for i in $curJifenFileList ; do
           
                createTs=$(echo $i | awk -F '.' '{print $3}')
           
                alive=$(( $(date +%s) - $createTs ))
           
                aliveHours=$(awk -v x=$alive 'BEGIN {printf "%.2f\n",x/3600}')
           
                if  [ $alive -ge 7200 ] ; then
           
                     info_short_log "file [$i] create at $(date -d@$createTs +'%F %T') , alive for ($alive) seconds [$aliveHours Hours]"
           
                     info_short_log "== add file [$i] to sync list =="
           
                     syncList="${syncList}${syncList:+ }$i"  # update sync list
                else
                     info_short_log "file [$i] create at $(date -d@$createTs +'%F %T') , only ($alive) seconds [$aliveHours Hours] , no need to sync"
                fi
           
          done
         
     fi
      
   done
   
   if [ -z "$syncList" ] ; then
   
        info_short_log
   
   	    info_short_log "no files need to sync , exit now"
   
   	    exit 0
   else
        info_short_log "print sync list"
        
        info_short_log "$(repeatStr '#' '95')"
        
        info_short_log "$(ls -l $syncList)"
        
        info_short_log "$(repeatStr '#' '95')"
        
        info_short_log "begin to sync"
   
     	for file in $syncList ; do
     
            flag=0
     
            for rsyncServer in $rsyncServerList ; do
      
         		  rsync -az --password-file=/etc/rsync.scr $file jifen@${rsyncServer}::jifen/$ip/ ; rc=$?
         		  
         		  if [ $rc -eq 0 ] ; then
         		 	 
                       info_short_log "sync file [$file] to [$rsyncServer] succeed , rc=[$rc]"
         		  else
                       err_short_log "sync file [$file] to [$rsyncServer] failed , rc=[$rc]"
                  fi
         
         		  ((flag+=$rc))
            done
      
            if [ $flag -eq 0 ] ; then
      
                 info_short_log "file [$i] all succeed , safe to backup"
      
                 backupDir="$(readlink -f $(dirname $file)/../old_jifen)"
      
                 mkdir -p $backupDir &>/dev/null
      
                 debug_short_log "$(mv -v $file $backupDir)"
            else		 
                 err_short_log  "file [$i] not all succeed , keep it"
            fi
  
       done

   fi
   
   info_short_log
   
   info_short_log "--DONE--"
   
) >> /home/dspeak/rsync_jifen.log 2>&1
