#!/bin/bash
#Date:2012-05-18
#Writen by wuhaiting
#bash-3.2.48打上审计补丁，更新全网bash
#本脚本依赖于auto/insert_message.sh
#2012-06-05,增加了213-220行回滚检测
#2014-09-26 由于CVE-2014-6271，更新bash至到3.2.52
#2014-12-26,更新bash至到3.2.57

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

# 基础变量定义
BASENAME=`basename $0`

read SOURCE_ADDRESS < /var/log/sysop_manager/repos_db_address.log

LOG_DIR=/var/log/sysop_manager
if [ ! -d "$LOG_DIR" ];then
	mkdir $LOG_DIR
fi

# 检查linux版本
cat /etc/issue | grep -E -i "Ubuntu"
if [ "$?" -eq 0 ];then
	LINUX_VERSION=ubuntu
	SYSTEM_VERSION=`cat /etc/issue|grep -v ^$|awk '{print $2}'|awk -F "." '{print $1}'`
else
	cat /etc/issue | grep -E -i "CentOS|Red"
	if [ "$?" -eq 0 ];then
		LINUX_VERSION=centos
		SYSTEM_VERSION=`cat /etc/issue|head -n 1|awk -F "." '{print $1}'|awk '{print $NF}'`
	fi
fi

# 在前面去export PATH路径出去，加了绝对路径有时候会适得其反
ARCH_BIT=`getconf LONG_BIT`

# 告警及前端打印
function INSERT_MYSQL () {
	/bin/bash /usr/local/i386/*/auto/insert_message.sh "$BASENAME" "$1"
	echo -e "\033[31;32;1m$BASENAME $1\033[0m"
}

# 校验当前bash是否已经升级过
if [ "$LINUX_VERSION" == "ubuntu" ];then
	if [ "$SYSTEM_VERSION" -ge "9" ];then
		if [ "$ARCH_BIT" == "64" ];then
			SOURCE_MD5="ce010d3f176fac4fd765231265f11365"
		elif [ "$ARCH_BIT" == "32" ];then
			SOURCE_MD5="e08411991254fb846b9338f3f909360b"
		fi
	elif [ "$SYSTEM_VERSION" -eq "8" ];then
		if [ "$ARCH_BIT" == "64" ];then
			SOURCE_MD5="adda55477ed230022aa0cda849f896c5"
		elif [ "$ARCH_BIT" == "32" ];then
			SOURCE_MD5="4d58fdee8255f0c89022ceb4722a2a94"
		fi
	fi
elif [ "$LINUX_VERSION" == "centos" ];then
	if [ "$SYSTEM_VERSION" -eq "6" ];then
		if [ "$ARCH_BIT" == "64" ];then
			SOURCE_MD5="828bc40c1b19de4275fd4295c316a4ab"
		elif [ "$ARCH_BIT" == "32" ];then
			SOURCE_MD5="e084b9938149b606abc1289bc9059d07"
		fi
	elif [ "$SYSTEM_VERSION" -eq "5" ];then
		if [ "$ARCH_BIT" == "64" ];then
			SOURCE_MD5="c5e63426bf5dbd362598a62058584bd1"
		elif [ "$ARCH_BIT" == "32" ];then
			SOURCE_MD5="cb1df322fbd6a3d64831bacf1eca3ee2"
		fi
	fi
fi
BASH_MD5=`md5sum /bin/bash|awk '{print $1}'`
if [ "$BASH_MD5" == "$SOURCE_MD5" ];then
	INSERT_MYSQL "Already recompile bash,exit now."
	exit 1
fi

# 若未检测到相应系统版本，直接退出
if [ "$LINUX_VERSION" == "" ];then
	INSERT_MYSQL "linux version error,please check."
	exit 1
fi

# bash回滚函数
function bash_recover () {
	install /bin/bash.ori /bin/bash
	INSERT_MYSQL "dpkg or rpm install error,bash-recover."
	exit 1
}

# syslog_update,logrotate功能由管理机去预分发，不需要通过此脚本去实现相关功能
# function syslog_update() {
# (
# # 要新建一个source，因为全网机器上的source不统一
# cat << 'EOF'
# source s_bash { unix-dgram("/dev/log"); file("/proc/kmsg"); };
# filter f_bash { match("bash:"); };
# destination d_bash { udp("121.14.47.198" port(61430)); file("/data/yy/log/bash/bash.log"); };
# log { source(s_bash); filter(f_bash); destination(d_bash); };
# EOF
# ) >> /etc/syslog-ng/syslog-ng.conf
	# /etc/init.d/syslog-ng restart
# }

# function logrotate_update() {
# (
# cat << 'EOF'

# /data/yy/log/bash/bash.log {
	# weekly
	# compress
	# dateext
	# create
	# postrotate
	# /etc/init.d/syslog-ng reload >/dev/null
	# endscript
# }
# EOF
# )
# }

# 下载包函数
function wget_action () {
	BASH_PACKAGE="$1"
	BASH_MD5="$2"
	#检测$LOG_DIR/$1是否存在，如果存在，删除它重新下载
	if [ -f $LOG_DIR/$BASH_PACKAGE ];then
		eval rm -f $LOG_DIR/$BASH_PACKAGE
	fi
	wget -w 10s --http-user=manifold --http-passwd=dw_manifold --output-document=$LOG_DIR/$1 --timeout=30 --tries=2 --quiet "http://$SOURCE_ADDRESS:63217/revision/package/$1"
	if [ $? -ne 0 ];then
		SOURCE_ADDRESS=$(awk '{if($0 ~ /manager.repos.yy.duowan.com/){print $1}}' /etc/hosts)
		wget -w 10s --http-user=manifold --http-passwd=dw_manifold --output-document=$LOG_DIR/$1 --timeout=30 --tries=2 --quiet "http://$SOURCE_ADDRESS:63217/revision/package/$1"
	fi
	if [ ! -f $LOG_DIR/$BASH_PACKAGE ];then
		INSERT_MYSQL "download $BASH_PACKAGE from $SOURCE_ADDRESS failed"
		exit 1
	fi

	MD5=`/usr/bin/md5sum $LOG_DIR/$BASH_PACKAGE|awk '{print $1}'`
	if [ "$MD5" != "$BASH_MD5" ];then
		INSERT_MYSQL "download $BASH_PACKAGE successful,but md5 error"
		exit 1
	fi

}

# bash检查函数
function check_bash () {
	# 检查应用新bash后是否有异常
	which bash
	if [ "$?" -ne 0 ];then
		INSERT_MYSQL "recompile bash finished,but now can not find bash."
		bash_recover
		exit 1
	fi

	# bash版本校验
	BASH_MD5=`md5sum /bin/bash|awk '{print $1}'`
	if [ "$BASH_MD5" != "$SOURCE_MD5" ];then
		INSERT_MYSQL "recompile bash finished,but now found that the bash version error."
		bash_recover
		exit 1
	fi

	# 调用新bash打印测试日志
	/bin/bash -c "echo bash_test"
	if [ "$?" -ne 0 ];then
		bash_recover
		INSERT_MYSQL "recompile bash finished,but execute failed."
		exit 1
	else
		INSERT_MYSQL "recompile bash successful."
		exit 0
	fi

	# 检查syslog是否可以正常输出新bash的日志
	# cat /data/yy/log/bash/bash.log|grep "bash_test"
	# if [ "$?" -ne 0 ];then
		# INSERT_MYSQL "recompile bash finished,but can not find bash log in /data/yy/log/bash/bash.log."
		# exit 1
	# else
		# INSERT_MYSQL "recompile bash successful."
		# exit 0
	# fi
}

# 手工编译函数
function manual_compile () {
	# 下载相关包
	aptitude -y install bison m4
	BASH_TAR=bash-3.2.48-patch.tar.gz
	if [ ! -f $LOG_DIR/$BASH_TAR ];then
		wget_action "$BASH_TAR" "3e241631e167d64ab5cbc3e47c2e4a3d"
	fi

	tar zxvf $LOG_DIR/$BASH_TAR -C $LOG_DIR

	cd $LOG_DIR/bash-3.2.48
	./configure --exec-prefix=./ >/dev/null
	/usr/bin/make >/dev/null
	/usr/bin/make test >/dev/null
	if [ "$?" -ne 0 ];then
		INSERT_MYSQL "manual_compile bash fail"
		exit 1
	fi

	# 校验编译后的bash版本是否正确
	chmod 755 /bin/bash
	chown root:root /bin/bash
	BASH_MD5=`md5sum /bin/bash|awk '{print $1}'`
	if [ "$BASH_MD5" != "$SOURCE_MD5" ];then
		INSERT_MYSQL "recompile bash failed,version error."
		exit 1
	fi

	# 校验编译后的bash是否可以正确运行
	./bash -c 'echo bash_test'
	if [ "$?" -ne 0 ];then
		INSERT_MYSQL "recompile bash failed,command execute error."
		exit 1
	fi

	# 备份旧bash，用新bash覆盖旧bash
	if [ ! -f /bin/bash.ori ];then
		cp -f /bin/bash /bin/bash.ori
	fi
	cp -f ./bash /bin/bash
	chmod 755 /bin/bash
	chown root:root /bin/bash

	check_bash
}

ubuntu_bash_update() {
	# bash.deb包现在只是在ubuntu 9.04以上版本测试过
	if [ "$SYSTEM_VERSION" -ge "9" ];then

		# 64位操作系统
		if [ "$ARCH_BIT" == "64" ];then

			BASH_DEB="bash_3.2.57_amd64.deb"
			wget_action "$BASH_DEB" "f8a8d5845a8d82c9f1e2b7c02f4a67db"

		# 32位操作系统
		elif [ "$ARCH_BIT" == "32" ];then

			BASH_DEB="bash_3.2.57_i386.deb"
			wget_action "$BASH_DEB" "4aba2526816b31b8aa7cdf10a232e690"

		fi
	elif [ "$SYSTEM_VERSION" -eq "8" ];then
		apt-get -y install bison m4
		# 64位操作系统
		if [ "$ARCH_BIT" == "64" ];then

			BASH_DEB=bash_3.2.48_ubuntu8_amd64.deb
			wget_action "$BASH_DEB" "6cabc0cf6dd3ad54d3472945d0be8a8d"

		# 32位操作系统
		elif [ "$ARCH_BIT" == "32" ];then
			BASH_DEB=bash_3.2.48_1ubuntu8_i386.deb
			wget_action "$BASH_DEB" "9fb8882739d14fa2382dd766d663f9c0"

		fi
	fi



	# 检测是否已经存在旧的bash备份文件
	[ ! -x /bin/bash.ori ] && install -m 755 /bin/bash /bin/bash.ori

	# 安装新的bash
	/usr/bin/dpkg -i $LOG_DIR/$BASH_DEB
	if [ "$?" -ne 0 ];then

		INSERT_MYSQL "dpkg -i $BASH_DEB error,bash reciver now."
		bash_recover
		exit 1

	else

		check_bash

	fi
}

centos_bash_update() {
	#bash.rpm包现在只是在centos5以上的版本测试过
	if [ "$SYSTEM_VERSION" -eq "5" ];then

		# 5版本需要调用外部库文件libncurses，系统起来的时候需要调用sh->bash挂载分区，导致/usr 挂载失败
		/bin/mountpoint -q /usr
		if [ "$?" == "0" ];then
			INSERT_MYSQL "bash_update: found /usr partition,exit."
			exit 1
		fi

		if [ "$ARCH_BIT" == "64" ];then

			BASH_RPM=bash-3.2.48-el5.x86_64.rpm
			wget_action $BASH_RPM "f8d5fa606fc6cc8bb47fe4d7ca22a868"

		elif [ "$ARCH_BIT" = "32" ];then

			BASH_RPM=bash-3.2.48-el5.i386.rpm
			wget_action $BASH_RPM "1ba0864fe2322da7a054b5256ee243c8"

		fi
	elif [ "$SYSTEM_VERSION" -eq "6" ];then
		if [ "$ARCH_BIT" == "64" ];then

			BASH_RPM="bash-3.2.57-1.x86_64.rpm"
			wget_action $BASH_RPM "c267b6d17a030f0ce9f1c1fc3c9a1c88"
		elif [ "$ARCH_BIT" == "32" ];then
			BASH_RPM="bash-3.2.52-1.i686.rpm"
			wget_action $BASH_RPM "ec09e57e5e553a22b1f11f7a544bdc0b"

		fi
	fi

		# 检测是否已经存在旧的bash备份文件
	[ ! -x /bin/bash.ori ] && install -m 755 /bin/bash /bin/bash.ori

	# 因为把该bash为低版本，而且在不卸载原来版本的情况下安装，rpm要加上--froce参数才行
	/bin/rpm -ivh --force $LOG_DIR/$BASH_RPM
	if [ "$?" -ne 0 ];then

		INSERT_MYSQL "rpm -ivh $BASH_RPM error,bash reciver now."
		bash_recover
		exit 1

	else

		check_bash

	fi
}

# 检测运行用户是否为root
if [ `whoami` != "root" ];then
	INSERT_MYSQL "no permissioin,use root instead.exit."
	exit 1
fi

# 判断操作系统
if [ "$LINUX_VERSION" == "ubuntu" ];then
	ubuntu_bash_update
elif [ "$LINUX_VERSION" == "centos" ];then
	centos_bash_update
else
	#manual_compile
	:
fi

