#!/bin/bash
# 2012-07-25 Written  by wuhaiting.
# 2015-02-05 Upload user password info to Platform every day.

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
export LANG=C
export LC_ALL=C

if ! source /usr/local/i386/*/auto/function.sh 2>/dev/null;then
    NOREPORT="yes"
    grep -qE "Ubuntu|Debian" /etc/issue && LINUX_VERSION="ubuntu"
    grep -qE "Ubuntu|Debian" /etc/issue && LINUX_VERSION="ubuntu"
fi
[ "x${LINUX_VERSION}" == "x" ] && exit 1


# Define global variables
BASENAME=$(basename $0)
SHORT_NAME="initial_bmc"
KNL_VER=$(uname -r)
RESULT="1"
LOG_DIR="/var/log/sysop_manager"
USER="admin"
USERID="4"

# Define input parameter.
PASSWD="$1"
if [ "x${PASSWD}" = "xrandomcode" ];then
    PASSWD=$(tr -dc "A-Za-z1-9@$" < /dev/urandom | head -c10)
    PASSWD="${PASSWD}@"
fi
if [ "x${2}" != "x" ];then
    BMC_IP="$2"
    BMC_NETMASK="${3:-255.255.255.0}"
    BMC_GATEWAY="${BMC_IP%.*}.1"
fi

INSERT_MYSQL () {
    /bin/bash /usr/local/i386/*/auto/insert_message.sh "$BASENAME" "$1"
}

# Some kernel ipmi version will hang up when som bmc controller no response.
function exit_func() {
    if echo $KNL_VER | grep '3.5.0';then
        modprobe -r ipmi_poweroff 2>/dev/null
        modprobe -r ipmi_watchdog 2>/dev/null
        modprobe -r ipmi_devintf 2>/dev/null
        modprobe -r ipmi_msghandler 2>/dev/null
        modprobe -r ipmi_si 2>/dev/null

        [ -d /lib/modules/$KNL_VER/kernel/drivers/char/ipmi ] && \
        mv /lib/modules/$KNL_VER/kernel/drivers/char/ipmi \
        /var/log/sysop_manager/ipmi >/dev/null 2>&1
        depmod -a
    fi
    exit $RESULT
}

function help_func() {
cat << EOF
usage:$BASENAME password
usage:$BASENAME password [IP]
usage:$BASENAME password [IP] [NETMASK] [GATEWAY]
usage:$BASENAME randomcode      #set PASSWORD with random key.

EOF
return 0
}

function load_ipmi() {
    CUR_IPMI_NUM=$(lsmod| grep -c 'ipmi')
    if [ "${LINUX_VERSION}" = "ubuntu" ] && [ $CUR_IPMI_NUM -lt 3 ];then
        reload='yes'
    elif [ "$LINUX_VERSION" = "centos" ];then
        lsmod | grep -q 'ipmi_devintf' || reload='yes'
    fi
    if [ "x${reload}" = 'xyes' ];then
        if echo $KNL_VER| grep -q '3.5.0-23';then
            [ -d /var/log/sysop_manager/ipmi ] && \
            mv /var/log/sysop_manager/ipmi \
            /lib/modules/$KNL_VER/kernel/drivers/char/
            depmod -a
        fi
        modprobe ipmi_si 2>/dev/null
        modprobe ipmi_devintf 2>/dev/null
        modprobe ipmi_msghandler 2>/dev/null
        modprobe ipmi_watchdog 2>/dev/null

        if [ "$LINUX_VERSION" = 'ubuntu' ];then
            module_prepost="/etc/modules"
        elif [ "$LINUX_VERSION" = 'centos' ];then
            module_prepost='/etc/sysconfig/modules/ipmi.modules'
        fi
        sed -i "/ipmi_/d" $module_prepost 2>/dev/null
        for cmodule in ipmi_si ipmi_devintf ipmi_msghandler ipmi_watchdog
        do
            if modinfo $cmodule >/dev/null 2>&1;then
                [ "$LINUX_VERSION" = 'ubuntu' ] && echo "$cmodule" >> $module_prepost
                [ "$LINUX_VERSION" = 'centos' ] && echo "modprobe $cmodule" >> $module_prepost
            fi
        done
    fi
}

function check_ipmi_status() {
    # Return code:
    # 0:normal, 1: BMC not support, 2: ipmi kernel thread hang up.
    [ -c /dev/ipmi[0-9] ] || return 1

    ipmi_cpu_usage=$(ps -eo pcpu,comm|awk '{if($0 ~ /kipmi/){print int($1)}}')
    if [ "x${ipmi_cpu_usage}" != "x" ] && [ $ipmi_cpu_usage -ge 80 ];then
        INSERT_MYSQL "kipmi cpu usage high: ${ipmi_cpu_usage}"
        return 2
    fi

    return 0
}

function select_manu() {
    SYSTEM_MANUFACTURER=$(split_tag 'Manufacturer' /etc/hardware_info ':' 2>/dev/null)
    [ "x${SYSTEM_MANUFACTURER}" = "x" ] && \
    SYSTEM_MANUFACTURER=$(dmidecode -s system-manufacturer 2>/dev/null | grep -v '^#')
    case $SYSTEM_MANUFACTURER in
        "HP")
            BMC_CHANNEL=2;;
        "IBM")
            BMC_CHANNEL=1;;
        "Powerleader")
            BMC_CHANNEL=1;;
        "Intel")
            BMC_CHANNEL=1;;
        "Lenovo")
            BMC_CHANNEL=1;;
        Dell*|dell*)
            BMC_CHANNEL=1
            USER="root";;
        Inspur*)
            BMC_CHANNEL=8;;
        *)
            BMC_CHANNEL=1;;
    esac
}


function initial_ipmi_user() {
    #check user and set all of access denied
    for u in $(ipmitool user list $BMC_CHANNEL|grep -vw "ID"|awk '{print $1}')
    do
        ipmitool channel setaccess $BMC_CHANNEL $u callin=off ipmi=off link=off privilege=15 >/dev/null 2>&1
        ipmitool user disable $u >/dev/null 2>&1
    done

    #检查用户名是否已经存在，某些服务器ipmi新建同一用户不同id会报错
    CHECK_USER=$(ipmitool user list $BMC_CHANNEL | grep -Ev "ID|grep" | \
        grep -w $USER | head -n1| awk '{print $2}')
    if [ "x${CHECK_USER}" == "x" ];then
        ipmitool user set name 3 $USER >/dev/null 2>&1
        if [ "$?" -ne 0 ];then
            ipmitool user set name 4 $USER >/dev/null 2>&1
            if [ "$?" -ne 0 ];then
                ipmitool user set name 5 $USER >/dev/null 2>&1
                if [ "$?" -ne 0 ];then
                    echo "ipmi SET name error"
                    RESULT='1'
                    exit $RESULT
                fi
            fi
        fi
        #新建用户和ID，但是HP的一些机器用户和ID不一定和我们设置的一样
        USERID=$(ipmitool user list $BMC_CHANNEL | grep -v "grep" | \
            grep -w "$USER" | awk '{print $1}')
    elif [ "x${CHECK_USER}" = "x${USER}" ];then
        USERID=$(ipmitool user list $BMC_CHANNEL | grep -v "grep" | \
            grep -w $USER | awk '{print $1;exit}')
    fi

    ipmitool user set password $USERID $PASSWD
    #设置用户权限
    ipmitool channel setaccess $BMC_CHANNEL $USERID callin=on ipmi=on link=on privilege=4 2>/dev/null
    [ $? -ne 0 ] && ipmitool channel setaccess $BMC_CHANNEL $USERID callin=on ipmi=on privilege=4
    ipmitool user enable $USERID
    ipmitool user test $USERID 16 $PASSWD
    RESULT="$?"

    return $RESULT
}

function configure_ipmi_ip() {
    if [ "x${BMC_IP}" != "x" ];then
    # 增加ip设置功能
        ipmitool lan set $BMC_CHANNEL arp respond on
        ipmitool lan set $BMC_CHANNEL arp generate on
        ipmitool lan set $BMC_CHANNEL ipsrc static
        ipmitool lan set $BMC_CHANNEL netmask $BMC_NETMASK && NETMASK_STATUS=$?
        ipmitool lan set $BMC_CHANNEL ipaddr $BMC_IP && IP_STATUS=$?
        ipmitool lan set $BMC_CHANNEL defgw ipaddr $BMC_GATEWAY && GATEWAY_STATUS=$?
        echo "Reflash IPMI IP:$IP_STATUS, netmask:$NETMASK_STATUS, gateway:$GATEWAY_STATUS"

        if [ "$NETMASK_STATUS" = "0" ] && \
            [ "$IP_STATUS" = "0" ] && \
            [ "$GATEWAY_STATUS" = "0" ];then
            RESULT=0
        else
            RESULT=1
        fi
    else
        RESULT=0
    fi
    return $RESULT
}

function upload_ipmi_info() {
    tmp_info=$(ipmitool lan print $BMC_CHANNEL| tr -d '%')
    CUR_BMC_IP=$(printf "$tmp_info"|grep -v 'IP Address Source' | \
        grep 'IP Address'| awk -F ':' '{print $NF}')
    CUR_BMC_IP="${CUR_BMC_IP// /}"

    CMDB_BMC_IP=$(ip_from_isp 'BMC' 2>/dev/null)
    if [ "x${CMDB_BMC_IP}" != "x" ] && \
        [ "${CUR_BMC_IP}" != "${CMDB_BMC_IP}" ];then
        python /home/dspeak/yyms/yymp/yymp_report_script/yymp_report_alarm.py "12221" "78201" "0" "管理网不一致通知YY运维基础平台处理" >/dev/null
        INSERT_MYSQL "bmc ip inconsistency"
    fi

    if echo "$CUR_BMC_IP" | grep -q '10\.';then
        [ ! -d /dev/sysop ] && r='/dev' || r='/dev/sysop'
        BMC_MAC=$(printf "$tmp_info"| awk '{if($0 ~ /^MAC Address/){print $NF}}')
        BMC_MAC="${BMC_MAC// /}"

        SN=$(split_tag 'serial-number' /etc/hardware_info ':' 2>/dev/null)
        [ "x${SN}" = "x" ] && SN=$(dmidecode -s system-serial-number|grep -v '^#'| tail -n1)
        SN="${SN// /}"

        SID=$(split_tag 'server_id' /home/dspeak/yyms/hostinfo.ini '=' 2>/dev/null)
        SID="${SID:-0}"

        wget -q --timeout=4 --tries=3 --user=manifold \
        --password=dw_manifold -O $r/ipmi_update_result \
        "http://manager.repos.yy.duowan.com:63217/external/record_ipmi_info.php?bmc_mac=${BMC_MAC}&sn=${SN}&sid=${SID}&npw=${PASSWD}&channel=${BMC_CHANNEL}&user=${USER}&user_id=${USERID}&bmc_ip=${CUR_BMC_IP}"

        if [ $? -eq 0 ] && grep -q 'success.*true' $r/ipmi_update_result ;then
            echo "$PASSWD" > $LOG_DIR/bmc_pw
        else
            return 1
        fi
    fi
    return 0
}

if [ -f $LOG_DIR/stop_run_$SHORT_NAME ] || \
    [ -f $LOG_DIR/stop_run_$BASENAME ] || \
    [ -f $LOG_DIR/stop_run_all ];then
        exit 1
fi

# main
trap "exit_func;" 0 1 2 9 15

if [ $# -eq 0 ];then
    help_func
    exit $RESULT
fi

load_ipmi
check_ipmi_status
if [ $? -eq 0 ];then
    select_manu
    initial_ipmi_user
    configure_ipmi_ip
    upload_ipmi_info
    if [ $? -ne 0 ];then
        PASSWD=$(cat $LOG_DIR/bmc_pw)
        [ "x${PASSWD}" != "x" ] && initial_ipmi_user
    fi
fi




