#!/bin/bash
#@version=0.4
#如果有必要更新网卡驱动
#打开网卡多队列

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
export LANG=en_US.utf8
export LC_ALL=en_US.utf8

function clean_fun ()
{
	rm -rf igb-5.0.6*
}

trap "clean_fun;exit;" 0 1 2 9 15

grep -iqE 'Ubuntu|Debian' /etc/issue && LINUX_VERSION="ubuntu"
grep -iqE 'CentOS|Red' /etc/issue && LINUX_VERSION="centos"

function help_print () {
	cat << "EOF"
usage: $0 [OPTIONS]

    --noreboot	Update driver and configure without rebooting. Nic multiqueue will be opened in next reboot time

EOF
}

OUTPUT_GREEN() {
	echo -e "\033[32;40;1m$1\033[0m"
}

OUTPUT_YELLOW(){
	echo -e "\033[33;40;1mWarning:$1\033[0m"
}

OUTPUT_RED(){
	echo -e "\033[31;40;1mError_msg:$1\033[0m"
}

until [ -z $1 ];do
	PARAM=$1
	case $PARAM in
		-h|help)
			help_print
			exit 
			shift;;
		--noreboot)
			NOREBOOT="yes"
			shift;;
	esac
done

[ "$(whoami)" != "root" ] && { echo "No permission!";exit 1; }

[ "$LINUX_VERSION" = "centos" ] && { OUTPUT_YELLOW "centos脚本不支持";exit 1; }

read queue_count nic < <(sed -n 's/.*\(eth[0-9]\).*/\1/p' /proc/interrupts |sort | uniq -c|sort| head -n1)
read nic_mac < /sys/class/net/${nic}/address
cur_nic_driver=$(grep -i "$nic_mac" /etc/hardware_info |awk -F '&' '{print $4}')

if [ "$cur_nic_driver" = "igb" ];then
	cur_igb_ver=$(modinfo igb| awk '{if($1 ~ /version/){print $NF;exit}}')
	
	if dpkg --compare-versions ${cur_igb_ver} lt 4.0.0 >/dev/null 2>&1;then
		OUTPUT_YELLOW "准备更新网卡igb驱动"	
		IGB_PAK="igb-5.0.6"
		wget --quiet --timeout=4 --tries=2 "http://mirror.yy.duowan.com:63782/packages/${IGB_PAK}.tar.gz"
		if [ $? -eq 0 ];then
			tar zxf ${IGB_PAK}.tar.gz
			cd ${IGB_PAK}/src
			make;make install
			wait
	
			new_igb_ver=$(modinfo igb| awk '{if($1 ~ /version/){print $NF;exit}}')
			if dpkg --compare-versions ${new_igb_ver} gt 4.0.0 >/dev/null 2>&1;then
				echo 'options igb RSS=0,0,0,0 IntMode=2,2,2,2' > /etc/modprobe.d/igb.conf
				OUTPUT_GREEN "安装igb驱动... success"
				update-initramfs -u
				RELOAD_IGB="yes"
			else
				OUTPUT_RED "驱动安装更新... 失败"
				exit 1
			fi
			cd -
		else
			OUTPUT_YELLOW "驱动下载失败"
			exit 1
		fi
	else
		if [ $queue_count -le 2 ];then
			echo 'options igb RSS=0,0,0,0 IntMode=2,2,2,2' > /etc/modprobe.d/igb.conf
			update-initramfs -u
			RELOAD_IGB="yes"
		fi
	fi
	
	if [ "x$RELOAD_IGB" = "xyes" ];then
		if [ "x${NOREBOOT}" = "xyes" ];then
			OUTPUT_GREEN "网卡多队列在下次重启后生效"
		else
			echo -e -n "reload网卡驱动,网络会中断10s左右,过长时间可能是由于reload失败需要重启 ........"
			ifdown -a --exclude=lo > /dev/null 2>&1
			modprobe -r igb;sleep 1;modprobe igb;
			ifup -a > /dev/null 2>&1
			OUTPUT_GREEN "驱动reload成功"
			bash -c "bash /usr/local/i386/*/auto/system_router.sh >/dev/null 2>&1 &"
		fi
	else
		
		eth_nic=$(grep -i "$nic_mac" /etc/hardware_info|awk '{for(i=1;i<=NF;i++){if($i ~ /[0-9]+[a-zA-Z]*/){print $i;exit}}}')
		OUTPUT_GREEN  "网卡:${eth_nic}, queue=${queue_count} 已经打开多队列"
	fi
else
	echo "非igb驱动网卡无需更新"
fi


if [ ! -s /etc/init.d/nic_irq_affinity ];then
	cp /usr/local/i386/*/initial_system/other/nic_irq_affinity.sh  /etc/init.d/nic_irq_affinity
	chmod 755 /etc/init.d/nic_irq_affinity
	ln -s /etc/init.d/nic_irq_affinity /etc/rc2.d/S25nic_irq_affinity 
	ln -s /etc/init.d/nic_irq_affinity /etc/rc3.d/S25nic_irq_affinity 
	ln -s /etc/init.d/nic_irq_affinity /etc/rc4.d/S25nic_irq_affinity 
	ln -s /etc/init.d/nic_irq_affinity /etc/rc5.d/S25nic_irq_affinity 
	ln -s /etc/init.d/nic_irq_affinity /etc/cron.hourly/start_nic_irq_affinity
	/etc/init.d/nic_irq_affinity start > /dev/null 2>&1

fi

exit 0