#!/bin/bash
# Date 2013-04-08
# Written by wuhaiting
# lsiutil阵列卡标准函数

# 支持阵列卡  02:00.0 SCSI storage controller: LSI Logic / Symbios Logic SAS1068E PCI-Express Fusion-MPT SAS (rev 08)
# 管理包 lsiutil、smartctl

# 返回标准格式：
# logicaldrive 1 ( 600GB, RAID1, OK, PD-num )
# physicaldrive 1I:1:1 ( SATA, 1000GB, OK, Serial-number, model )
#
# 容量大小转换成标准格式：64GB 73GB 146GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 4000GB
# RAID 0、1、1+0、5、50、none
# 接口类型 IDE、SCSI、SATA、SAS、SSD
# 状态：ok、fail、Optimal、online、Degraded、rebuild

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

which lsiutil >/dev/null 2>&1
if [ "$?" -ne 0 ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/raid_package_install.sh
	which lsiutil > /dev/null 2>&1 || exit 1
fi

tmp_file1=`mktemp` || exit 1
tmp_file2=`mktemp`
[ "$1" = "status" ] && DEBUG_TAG=yes

lsiutil -p1 -a21,1,0,0 > $tmp_file1 2>&1
ld_index=$(awk '{if($0 ~ /^Volume.*Bus.*Target/){print $2}}' $tmp_file1)
sdmap=$(ls -l --color=never /dev/disk/by-path/ | grep -vE 'part|usb' |awk -F ':' '{if($(NF-1) == "'$ld_index'"){print $0}}')
sdmap="${sdmap##*/}"

ld_msg=`cat $tmp_file1 | grep -E "^Volume.*Type|Volume Size|volume.*physical|Volume.*State" | awk '{
					if($0 ~ /volume.*physical/){
						for(k=1;k<=NF;k++){
							if($k == "volume"){n=$(k-1)};
							if($k == "physical"){m=$(k-1)};
						}
						if(n == m){
							ld_raid="RAID0";
						}else if(m > n){
							ld_raid="RAID1";
						};
					}else if($0 ~ /^Volume.*Type/){
						for(i=1;i<=NF;i++){
							if($i == "Type"){
								if($(i+1) == "IM"){ld_raid="RAID1"}else{ld_raid="RAID0"};
							}
						}
					}else if($0 ~ /Volume State/){
						if($0 ~ /optimal/ || $0 ~ /OK/ || $0 ~ /ok/){
							ld_status="OK";
						}else{
							if($0 ~ /resync in progress/){
								ld_status="Rebuild";
							}else{
								if("'$DEBUG_TAG'"=="yes"){
									printf("1:2:2@0@Raid fail status=%s\n",ld_status);
								}
								ld_status="fail";
							}
						}
					}else if($0 ~ /Volume Size/){
						g=int($3/1024);
						a=999999999;
						split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
						for(var in b){
							c=b[var]-g;
							if(c>=0 && c<a){a=c;ld_volume=b[var];};
						};
						if("'$DEBUG_TAG'"!="yes"){
							printf("logicaldrive '$ld_index' ( %sGB, %s, %s, %s, '$sdmap' )",ld_volume,ld_raid,ld_status,m);
						}
					};
				}'`

if [ "$ld_msg" != "" ];then
	ctrl_model=$(cat /proc/scsi/mptsas/* 2>/dev/null | cut -d " " -f 2)
	[ "$DEBUG_TAG" != "yes" ] && echo "accelerator none ( none, none, none, $ctrl_model )"
	echo $ld_msg;

	lsiutil -p1 -a21,2,0,0 | awk '{
		if($0 ~ /^PhysDisk.*Bus.*Target/){
			ld_num=$2;
		}else if($0 ~ /PhysDisk State/){
			ld_state=$NF;
			if(ld_state !~ /online/ && ld_state !~ /sync/){
				if("'$DEBUG_TAG'"=="yes"){
					printf("1:1:4@0:0:%s@Disk fail status=%s\n",ld_num,ld_state);
				};
				ld_state="fail";
			}else{
				ld_state="OK";
			};
		}else if($0 ~ /PhysDisk Size/){
			g=int($3/1024);
			pd_model=$(NF-1);

			a=999999999;
			split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
			for(var in b){
				c=b[var]-g;
				if(c>=0 && c<a){a=c;pd_volume=b[var];};
			}
			printf("physicaldrive 0I:0:%s ( SAS, %sGB, %s, none, %s )\n",ld_num,pd_volume,ld_state,pd_model);
		}
	}'

fi

rm -f $tmp_file1 $tmp_file2

