#!/bin/bash
# Date 2013-04-08
# Written by wuhaiting
# HBA卡标准函数

# 支持一系列不做raid的HBA卡
# 管理包 smartmontools

# 返回标准格式：
# physicaldrive none ( SATA, 1000GB, OK, Serial-number, model )
# 容量大小转换成标准格式：64GB 73GB 146GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 4000GB
# 接口类型 IDE、SCSI、SATA、SAS、SSD
# 状态：ok、fail、Optimal、online、Degraded、rebuild

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

which smartctl > /dev/null 2>&1
if [ "$?" -ne 0 ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/raid_package_install.sh
	which smartctl > /dev/null 2>&1 || exit 1
fi

tmp_file1=`mktemp` || exit 1
tmp_file2=`mktemp`
[ "$1" = "status" ] && DEBUG_TAG=yes

lspci | grep -E -w "RAID|SCSI|SATA" | grep -qEw "Fusion-MPT"
if [ "$?" -eq 0 ];then
	ctrl_model=$(cat /proc/scsi/mptsas/* 2>/dev/null | cut -d " " -f 2)
else
	ctrl_model=`lspci|grep -E -w "SCSI|SATA"|awk -F ':' '{gsub(/\(.*\)/,"",$NF);print $NF}'|tail -n1`
fi
[ "$DEBUG_TAG" != "yes" ] && echo "accelerator none ( none, none, none, $ctrl_model )"
[ "$DEBUG_TAG" != "yes" ] && echo "logicaldrive none ( none, none, none, none )"

fdisk -l 2>/dev/null|grep -w "Disk /dev/.*:"|grep -v "MB"|awk '{
			a=99999999;
			g=$3;
			split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
			for(var in b){
				c=b[var]-g;
				if(c<0){c=-c;}
				if(c>=0 && c<a){a=c;d=b[var];};
			}
			printf("%s %s%s\n",$2,d,$4);
		}'| tr -d ",:" > $tmp_file1

cat $tmp_file1 |while read disk_index disk_volume;do
			sdmap="${disk_index##*/}"
			smartctl -a $disk_index |grep -iE "Device:|Device Model:|Product:|ATA Standard|Transport protocol|^Serial Number|SMART Health Status|SMART.*test result:" | \
			awk '{
				if($0 ~ /^Device/ || $0 ~ /^Product/){
					if($0 ~ /LOGICAL/ || $0 ~ /VIRSUAL/ || $0 ~ /RAID/){no_need="yes"};
					if($0 ~ /^Product/){disk_model=$NF;};
					if($0 ~ /Device:/){disk_model=$3};
					if($0 ~ /Device Model:/){
						disk_model=$4;
						if(disk_model == ""){disk_model=$3}
					};
				}else if($0 ~ /^Serial/){
					disk_sn=$NF;
				}else if($0 ~ /Transport protocol/ || $0 ~ /ATA Standard/){
					if($0 ~ /SAS/){
						disk_type="SAS";
					}else if($0 ~ /ATA/){
						disk_type="SATA";
					}else if($0 ~ /Parallel SCSI/){
						disk_type="SCSI";
					};
				}else if($0 ~ /SMART Health Status/ || $0 ~ /SMART.*test result/){
					disk_status=$NF;
				}
			}
			END{
				if(disk_status == "OK" || disk_status == "PASSED" || disk_status == ""){
					disk_status="OK";
				}else{
					if("'$DEBUG_TAG'"=="yes"){
						printf("1:1:4@none@Disk fail status=%s\n",disk_status);
					};
					disk_status="fail";
				};
				if(disk_model ~ /SSD/){disk_type="SSD"};
				if(no_need == "yes" || disk_type == ""){exit;};
				if("'$DEBUG_TAG'"!="yes"){
					printf("physicaldrive none ( %s, '$disk_volume', %s, %s, %s, '$sdmap' )\n",disk_type,disk_status,disk_sn,disk_model);
				}
			}'

done
rm -f $tmp_file1 $tmp_file2
