#!/bin/bash
# Date 2013-04-04
# Written by wuhaiting
# RocketRAID阵列卡标准函数

# 支持阵列卡  RAID bus controller: HighPoint Technologies, Inc. RocketRAID 2640 SAS/SATA Controller
# 管理包 hptraidconf、hptsvr 采用C-S模式进行管理，太麻烦。直接从/proc/scsi/rr26xx/X获取信息

# 返回标准格式：
# 从/proc获取信息不支持阵列卡方面信息,统一为none
# accelerator 1 ( none, none, none, none )
# logicaldrive 1 ( 600GB, RAID1, OK, PD-num )
# physicaldrive 1I:1:1 ( SATA, 1000GB, OK, Serial-number, model )
#
# 容量大小转换成标准格式：64GB 73GB 146GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 4000GB
# RAID 0、1、1+0、5、none
# 接口类型,从/proc只能通过容量大概判断。
# 状态：ok、fail、Optimal、online、Degraded、rebuild

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

[ -f /proc/scsi/rr26xx/0 ] && exit 1

tmp_file1=`mktemp` || exit 1
tmp_file2=`mktemp`

source_file=/proc/scsi/rr26xx/*
[ "$1" = "status" ] && DEBUG_TAG=yes

controll_num=`grep "^Controller" $source_file | awk '{gsub(":","",$2);print $2}' | tr -s "\n" " "`
ctrl_model=`lspci | grep -w "RocketRAID" | awk '{for(i=1;i<=NF;i++){if($i == "RocketRAID"){print "RocketRAID"" "$(i+1)}}}'`
for controll_index in $controll_num;
do
	controll_index_start="Controller ${controll_index}:"
	controll_index_next=`grep "^Controller.*:" $source_file | \
				grep -A1 "$controll_index_start" | grep -v "$controll_index_start"`
				
	if [ "$controll_index_next" != "" ];then
		sed -n "/$controll_index_start/,/$controll_index_next/p" $source_file | \
		grep -v "$controll_index_next" > $tmp_file1
	else
		sed -n "/$controll_index_start/,/$controll_index_next/p" $source_file > $tmp_file1
	fi
	
	echo "accelerator $controll_index ( none, none, none, $ctrl_model )"
	
	ld_num=`grep -c "\[DISK.*_.*\]" $tmp_file1`;ld_num=`expr $ld_num - 1`
	
	for ld_index in `seq 0 $ld_num`;
	do
		ld_start="DISK.*_${ld_index}.*RAID"
		ld_next=`grep "\[DISK.*_.*RAID" $tmp_file1 | grep -A1 "$ld_start" | \
					grep -v "$ld_start" | sed -e "s/\[/\./g" -e "s/\]/\./g"`
					
		if [ "$ld_next" != "" ];then
			sed -n "/$ld_start/,/$ld_next/p" $tmp_file1 | grep -v "$ld_next" > $tmp_file2
		else
			sed -n "/$ld_start/,/$ld_next/p" $tmp_file1 > $tmp_file2
		fi
		
		ld_index_msg=`grep "DISK.*_${ld_index}.*RAID" $tmp_file2`
		ld_index_raid=`echo ${ld_index_msg#*\(}`;ld_index_raid=`echo ${ld_index_raid%\)*}`
		
		ld_index_volume=`echo $ld_index_msg | awk '{print $5}' | tr -d "MGB,"`
		ld_index_volume=`echo $ld_index_volume | awk '{
						g=int($1/1024);
						a=999999999;
						split("64 73 146 240 300 500 600 1000 2000 2500 4000",b," ");
						for(var in b){
							c=b[var]-g;
							if(c>=0 && c<a){a=c;d=b[var];};
						}
						printf("%s",d);
				}'`
				
		ld_index_stats=`echo $ld_index_msg| \
			awk -F ',' '{gsub(" ","",$NF);stat=$NF;if(stat ~ /Normal/ || stat ~ /Rebuilding/){print stat}else{print "fail"}}'`
		if [ "$ld_index_stats" = "fail" -a "$DEBUG_TAG" = "yes" ];then
			echo "1:2:2@$controll_index:$ld_index@Raid fail status=$ld_index_stats"
		fi
		
		ld_index_disk_num=`grep -c "/.*/.*" $tmp_file2`
		sdmap=$(ls -l --color=never /dev/disk/by-path/ | grep -vE 'part|usb' |awk -F ':' '{if($(NF-1) == "'$ld_index'"){print $0}}')
		sdmap="${sdmap##*/}"
		
		[ "$DEBUG_TAG" != "yes" ] && \
			echo "logicaldrive $controll_index:$ld_index ( ${ld_index_volume}GB, ${ld_index_raid}, $ld_index_stats, $ld_index_disk_num, $sdmap )"
		
		pd_num=`grep "/.*/.*" $tmp_file2 | awk '{if($1 == "*"){print $2}else{print $1}}' | tr -s "\n" " "`
		for pd_index in $pd_num;
		do
			pd_index_msg=`grep "^$pd_index" $tmp_file1`
			
			pd_index_position=`echo $pd_index | awk -F '/' '{printf("%sI:%s:%s",$1,$3,$2)}'`
			
			pd_index_model_sn=`echo $pd_index_msg | awk '{gsub(",","",$3);print $3}'`
			pd_index_model=`echo ${pd_index_model_sn%-*}`
			pd_index_sn=`echo ${pd_index_model_sn#*-}`
			
			pd_index_volume=`echo $pd_index_msg | awk '{gsub("MB,","",$4);
					g=int($4/1024);
					a=999999999;
					split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
					for(var in b){
						c=b[var]-g;
						if(c>=0 && c<a){a=c;d=b[var];};
					}
					printf("%s",d);
				}'`
			
			[[ $pd_index_volume -gt 600 ]] && pd_index_type=SATA || pd_index_type=SAS
			echo $pd_index_model | grep -q "SSD" && pd_index_type=SSD	
			
			pd_index_stats=`echo $pd_index_msg | awk '{print $5}'`
			if [ "$pd_index_stats" == "Normal" ];then
				pd_index_stats=OK 
			else
				[ "$DEBUG_TAG" = "yes" ] && echo "1:1:4@$pd_index_position@Disk fail status=$pd_index_stats"
				pd_index_stats=fail
			fi
			
			[ "$DEBUG_TAG" != "yes" ] && \
				echo "physicaldrive $pd_index_position ( $pd_index_type, ${pd_index_volume}GB, $pd_index_stats, $pd_index_sn, $pd_index_model )"
			
			unset pd_index_msg pd_index_model_sn pd_index_model pd_index_sn pd_index_volume
			unset pd_index_stats
		done
		unset ld_start ld_next ld_index_msg ld_index_raid ld_index_volume ld_index_stats
	done
	unset controll_index_start controll_index_next ld_num
done

rm $tmp_file1 $tmp_file2

