#!/bin/bash
# Date 2013-04-03
# Written by wuhaiting
# Adaptec阵列卡标准函数

# 支持阵列卡  01:00.0 RAID bus controller: Adaptec AAC-RAID (rev 09)
# 管理包 arcconf

# 返回标准格式：
# accelerator 1 ( Cache-size, OK, Serial-number, model )
# logicaldrive 1 ( 600GB, RAID1, OK, PD-num )
# physicaldrive 1I:1:1 ( SATA, 1000GB, OK, Serial-number, model )
#
# 容量大小转换成标准格式：64GB 73GB 146GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 4000GB
# RAID 0、1、1+0、5、none
# 接口类型 IDE、SCSI、SATA、SAS、SSD
# 状态：ok、fail、Optimal、online、Degraded、rebuild

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

which arcconf > /dev/null 2>&1
if [ "$?" -ne 0 ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/raid_package_install.sh
	which arcconf > /dev/null 2>&1 || exit 1
fi

tmp_file1=`mktemp` || exit 1
tmp_file2=`mktemp`
tmp_file3=`mktemp`
[ "$1" = "status" ] && DEBUG_TAG=yes

arcconf GETCONFIG 1 | awk '{a=$1;$1="";print a$0}' > $tmp_file1

# accelerator msg
grep -iE 'Controller Status|Controller Model|Controller Serial Number|Physical Slot|Installed memory' $tmp_file1 | \
awk -F ':' '{
	if($0 ~ /Status/){
		a=$2;
	}else if($0 ~ /Model/){
		b=$2;
	}else if($0 ~ /Serial Number/){
		c=$2;
	}else if($0 ~ /Slot/){
		d=$2;
	}else if($0 ~ /memory/){
		e=$2;
		gsub(" ","",e);
		if(e !~ /MB/){e=none};
	}}
	END{
		if(a !~ /Optimal/ && "'$DEBUG_TAG'"=="yes"){
			printf("3:1@%s@Controller fail status=%s\n",d,a);
		};
		if("'$DEBUG_TAG'"!="yes"){
			printf("accelerator %s ( %s, %s, %s, %s )\n",d,e,a,c,b);
		};
	}'
	
# logical and physical msg
grep -iE 'Logical device number|RAID level|Status of logical|^Size|Segment|^State|^Transfer Speed|^Model|^Serial number' $tmp_file1 > $tmp_file2
ld_num=`grep -c 'Logical device number' $tmp_file2`
ld_num_1=`expr $ld_num - 1`
# cacluate ld
for ld_index in `seq 0 $ld_num_1`;
do
	start_line="Logical device number ${ld_index}"
	next_line=`grep 'Logical device number' $tmp_file2 | grep -A1 "$start_line" | grep -v "$start_line"`
	if [ "$next_line" != "" ];then
		sed -n "/${start_line}$/,/$next_line/p" $tmp_file2 | grep -v "$next_line" > $tmp_file1
	else
		sed -n "/${start_line}$/,/$next_line/p" $tmp_file2 > $tmp_file1
	fi
	
	ld_index_raid=`grep "RAID level" $tmp_file1 | awk '{print $NF}'`
	ld_index_volume=`grep "^Size" $tmp_file1| head -n1 | awk '{print int($3/1024)}'`
	ld_index_status=`grep "Status of logical" $tmp_file1`;ld_index_status=`echo ${ld_index_status#*: }`
	if [ "$ld_index_status" == "Optimal" ]; then
		ld_index_status=OK
	else
		[ "$DEBUG_TAG" = "yes" ] && echo "1:2:2@$ld_index@Raid fail status=$ld_index_status"
		ld_index_status=fail
	fi
	
	disk_num=`grep -c "Segment" $tmp_file1`
	[ "$DEBUG_TAG" != "yes" ] && echo "logicaldrive $ld_index ( ${ld_index_volume}GB, RAID${ld_index_raid}, $ld_index_status, $disk_num )"
	
	# cacluate pd
	disk_sn=`grep "Segment" $tmp_file1 | awk '{print $NF}' | tr -s "\n" " "`
	for disk_sn_index in $disk_sn;
	do
		grep -v "Segment" $tmp_file1 | grep -B3 -A1 "$disk_sn_index" > $tmp_file3
		disk_index_state=`grep -i "^State" $tmp_file3`;disk_index_state=`echo ${disk_index_state#*: }`;
		if [ "$disk_index_state" == "Online" ];then
			disk_index_state=OK
		else
			[ "$DEBUG_TAG" = "yes" ] && echo "1:1:4@$ld_index@Disk fail status=$disk_index_state"
			disk_index_state=fail
		fi
		
		disk_index_type=`awk -F '[ |:]' '{if($0 ~ /Transfer Speed/){print $5}}' $tmp_file3`
		
		disk_index_model=`grep -i "^Model" $tmp_file3`;
		disk_index_model=`echo ${disk_index_model#*: }`;
		disk_index_model=`echo ${disk_index_model% ?}`
		echo $disk_index_model | grep -q "SSD" && disk_index_type=SSD
		
		disk_index_volume=`awk '{
			if($0 ~ /^Size/){
				g=int($3/1024);
				a=999999999;
				split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
				for(var in b){
					c=b[var]-g;
					if(c>=0 && c<a){a=c;d=b[var];};
				}
				printf("%s",d);
			};
		}' $tmp_file3`
		
		disk_index_position=`grep "Segment.*$disk_sn_index" $tmp_file1 | awk '{print $(NF-1)}' | \
																awk -F '[:|,]' '{gsub(")","",$NF);printf("%sI:%s:%s",$2,$4,$6)}'`
		
		[ "$DEBUG_TAG" != "yes" ] && \
			echo "physicaldrive $disk_index_position ( $disk_index_type, ${disk_index_volume}GB, $disk_index_state, $disk_sn_index, $disk_index_model )"
		unset disk_index_model disk_index_state disk_index_volume disk_index_type disk_index_position
	done
	unset disk_num ld_index_status ld_index_volume ld_index_raid start_line next_line
done

rm $tmp_file1 $tmp_file2 $tmp_file3
exit 0


		
	
	


	