#!/bin/bash
# Date 2013-04-16
# Written by wuhaiting
# cliib阵列卡标准函数

# 支持阵列卡  06:00.0 RAID bus controller: Promise Technology, Inc. 81384 [SuperTrak EX SAS and SATA RAID Controller] (rev 09) 
# 管理包 cliib

# 返回标准格式：
# accelerator 1 ( Cache-size, OK, Serial-number, model )
# logicaldrive 1 ( 600GB, RAID1, OK, PD-num )
# physicaldrive 1I:1:1 ( SATA, 1000GB, OK, Serial-number, model )
#
# 容量大小转换成标准格式：64GB 73GB 146GB 240GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 4000GB
# RAID 0、1、1+0、5、50、none
# 接口类型 IDE、SCSI、SATA、SAS、SSD
# 状态：ok、fail、Optimal、online、Degraded、rebuild

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

which cliib > /dev/null 2>&1
if [ "$?" -ne 0 ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/raid_package_install.sh
	which cliib > /dev/null 2>&1 || exit 1
fi

tmp_file1=`mktemp` || exit 1
tmp_file2=`mktemp`
tmp_file3=`mktemp`
tmp_file4=`mktemp`

[ "$1" = "status" ] && DEBUG_TAG=yes
export LD_LIBRARY_PATH=/usr/local/lib/webpampro/agent
(
expect << EOF
set timeout 4
spawn /usr/sbin/cliib
expect "cliib>"
send "ctrl -v\n"
expect "cliib>"
send "logdrv -v\n"
expect "cliib>"
send "phydrv -v\n"
expect "cliib>"
send "exit\n"
exit
EOF
) |tr -d "\r" > $tmp_file1

ctrl_num=`grep -w "^CtrlId" $tmp_file1 | awk '{print $NF}' | tr -s "\n" " "`
for ctrl_index in $ctrl_num;
do
	ctrl_start="CtrlId: *${ctrl_index}"
	ctrl_next=`grep "^CtrlId:" $tmp_file1 |grep -A1 "$ctrl_start" | \
					grep -v "$ctrl_start"`
					
	if [ "$ctrl_next" != "" ];then
		sed -n "/${ctrl_start}$/,/$ctrl_next/p" $tmp_file1 | grep -v "$ctrl_next" > $tmp_file2
	else
		sed -n "/${ctrl_start}$/,/$ctrl_next/p" $tmp_file1 > $tmp_file2
	fi
	
	ctrl_stats=`grep -A2 "$ctrl_start" $tmp_file2 | \
					awk -F '[ |:]' '{if($0 ~ /OperationalStatus/){print $3}}'`
	[ "$DEBUG_TAG" = "yes" ] && [ "$ctrl_stats" != "OK" ] && \
		echo "3:1@$ctrl_index@Controller fail status=$ctrl_stats"
	
	awk -F '[ |:]' '{
		if($0 ~ /Model:/){
			ctrl_model=$NF;
		}else if($0 ~ /MemSize:/){
			ctrl_cache=$NF;
		}else if($0 ~ /SerialNo/){
			ctrl_sn=$NF;
		}else if($0 ~ /BIOSVersion/ && "'$DEBUG_TAG'" != "yes"){
			printf("accelerator %s ( %s, %s, %s, %s )\n","'$ctrl_index'",ctrl_cache,"'$ctrl_stats'",ctrl_sn,ctrl_model);
		}
	}' $tmp_file2
	
	ld_num=`grep -w "^LdId" $tmp_file2 | awk '{print $NF}'| tr -s "\n" " "`
	for ld_index in $ld_num;
	do
		ld_start="LdId: *${ld_index}"
		ld_next=`grep "^LdId:" $tmp_file2 | grep -A1 "$ld_start" | grep -v "$ld_start"`
		
		if [ "$ld_next" != "" ];then
			sed -n "/${ld_start}$/,/$ld_next/p" $tmp_file2 | grep -v "$ld_next" > $tmp_file3
		else
			sed -n "/${ld_start}$/,/$ld_next/p" $tmp_file2 > $tmp_file3
		fi
		
		ld_stats=`grep -A2 "$ld_start" $tmp_file3 | \
					awk -F '[ |:]' '{if($0 ~ /OperationalStatus/){gsub(",","",$3);print $3}}'`
		[ "$DEBUG_TAG" = "yes" ] && [ "$ld_stats" != "OK" ] && \
			echo "1:2:2@$ctrl_index:$ld_index@Raid fail status=$ld_stats"
		
		awk -F '[ |:]' '{
			if($0 ~ /RAIDLevel/){
				ld_raid=$3;
			}else if($0 ~ /^Capacity/){
				ld_volume=$3;
			}else if($0 ~ /NumOfUsedPD/ && "'$DEBUG_TAG'" != "yes"){
				ld_pd_num=$3;
				printf("logicaldrive '$ctrl_index':'$ld_index' ( %s, %s, %s, %s )\n",ld_volume,ld_raid,"'$ld_stats'",ld_pd_num);
			}
		}' $tmp_file3
		
		pd_num=`grep -E "PdId:|ConfigStatus:" $tmp_file1 | grep -B1 "ConfigStatus: Array${ld_index}" |grep "PdId:" | awk '{print $NF}' | tr -s "\n" " "`
		for pd_index in $pd_num;
		do
			pd_start_tmp="ConfigStatus: Array${ld_index}"
			pd_start=`grep -E "PdId:|ConfigStatus:" $tmp_file1 |grep -B1 "$pd_start_tmp" | \
						grep "PdId: *${pd_index}"`
			pd_next=`grep "^PdId:" $tmp_file1 | grep -A1 "$pd_start$" | grep -v "$pd_start"`
			
			if [ "$pd_next" != "" ];then
				sed -n "/${pd_start}$/,/$pd_next/p" $tmp_file1 | grep -v "$pd_next" > $tmp_file4
			else
				sed -n "/${pd_start}$/,/$pd_next/p" $tmp_file1 > $tmp_file4
			fi
			
			pd_stats=`grep -A2 "$pd_start" $tmp_file4 | \
					awk -F '[ |:]' '{if($0 ~ /OperationalStatus/){print $3}}'`
			[ "$DEBUG_TAG" = "yes" ] && [ "$pd_stats" != "OK" ] && \
				echo "1:1:4@$ctrl_index:$ld_index:$pd_index@Disk fail status=$pd_stats"
					
			awk -F '[ |:]' '{
				if($0 ~ /PhysicalCapacity/){
					if($NF ~ /GB/){
						gsub("GB","",$3);
						g=$3;
						a=999999999;
						split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
						for(var in b){
							c=b[var]-g;
							if(c>=0 && c<a){a=c;d=b[var];};
						}
						pd_volume=d"GB";
					}else if($NF ~ /TB/){
						gsub("TB","",$3);
						g=$3;
						a=999999999;
						split("1 2 2.5 3 4",b," ");
						for(var in b){
							c=b[var]-g;
							if(c>=0 && c<a){a=c;d=b[var];};
						}
						pd_volume=d"TB";
					}
				}else if($0 ~ /^ModelNo/){
					for(i=4;i<=NF;i++){
						if($i != ""){pd_model_tmp=$i;break;};
					};
					pd_model=$3" "pd_model_tmp;
					gsub("VisibleTo","",pd_model);
					if(pd_model_tmp != "VisibleTo"){gsub(" ","-",pd_model)};
				}else if($0 ~ /^SerialNo/){
					pd_sn=$3;
				}else if($0 ~ /DriveInterface/){
					pd_type=$3;
					if(pd_model ~ /SSD/ || pd_sn ~ /SSD/){
						pd_type="SSD";
					}
				}
			}
			END{
				if("'$DEBUG_TAG'" != "yes"){
					printf("physicaldrive '$ctrl_index'I:'$ld_index':'$pd_index' ( %s, %s, '$pd_stats', %s, %s )\n",pd_type,pd_volume,pd_sn,pd_model);
				}
			}' $tmp_file4
			
			unset pd_start pd_next pd_stats
		done
		unset ld_start ld_next ld_stats
	done
	unset ctrl_start ctrl_next ctrl_stats
done

(
expect << EOF
set timeout 4
spawn /usr/sbin/cliib
expect "cliib>"
send "spare\n"
expect "cliib>"
send "exit\n"
exit
EOF
) |tr -d "\r" > $tmp_file2

hotspare_num=`grep -Ew "Dedicated|Global" $tmp_file2 | awk '{print $3}' | tr -s "\n" " "`
for hotspare_index in $hotspare_num;
do
	hotspare_start="PdId: *${hotspare_index}"
	hotspare_next=`grep "^PdId:" $tmp_file1 | grep -A1 "$hotspare_start" | grep -v "$hotspare_start"`
	
	if [ "$hotspare_next" != "" ];then
		sed -n "/${hotspare_start}$/,/$hotspare_next/p" $tmp_file1 | grep -v "$hotspare_next" > $tmp_file4
	else
		sed -n "/${hotspare_start}$/,/$hotspare_next/p" $tmp_file1 > $tmp_file4
	fi
	
	hotspare_stats=`grep -A2 "$hotspare_start" $tmp_file4 | \
			awk -F '[ |:]' '{if($0 ~ /OperationalStatus/){print $3}}'`
	[ "$hotspare_stats" != "OK" ] && [ "$DEBUG_TAG" = "yes" ] && \
		echo "1:1:4@$ctrl_index:$ld_index:$hotspare_index@Disk fail status=$hotspare_stats"
	[ "$hotspare_stats" != "OK" ] && hotspare_stats="fail" || hotspare_stats="OK-Hostspare"
			
	awk -F '[ |:]' '{
		if($0 ~ /PhysicalCapacity/){
			if($NF ~ /GB/){
				gsub("GB","",$3);
				g=$3;
				a=999999999;
				split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
				for(var in b){
					c=b[var]-g;
					if(c>=0 && c<a){a=c;d=b[var];};
				}
				pd_volume=d"GB";
			}else if($NF ~ /TB/){
				gsub("TB","",$3);
				g=$3;
				a=999999999;
				split("1 2 2.5 3 4",b," ");
				for(var in b){
					c=b[var]-g;
					if(c>=0 && c<a){a=c;d=b[var];};
				}
				pd_volume=d"TB";
			}
		}else if($0 ~ /^ModelNo/){
			for(i=4;i<=NF;i++){
				if($i != ""){pd_model_tmp=$i;break;};
			};
			pd_model=$3" "pd_model_tmp;
			gsub("VisibleTo","",pd_model);
			if(pd_model_tmp != "VisibleTo"){gsub(" ","-",pd_model)};
		}else if($0 ~ /^SerialNo/){
			pd_sn=$3;
		}else if($0 ~ /DriveInterface/){
			pd_type=$3;
			if(pd_model ~ /SSD/ || pd_sn ~ /SSD/){
				pd_type="SSD";
			}
		}
	}
	END{
		if("'$DEBUG_TAG'" != "yes"){
			printf("physicaldrive '$ctrl_index'I:'$ld_index':'$hotspare_index' ( %s, %s, '$hotspare_stats', %s, %s )\n",pd_type,pd_volume,pd_sn,pd_model);
		}
	}' $tmp_file4
	
	unset hotspare_start hotspare_next hotspare_stats
done

rm -f $tmp_file1 $tmp_file2 $tmp_file3 $tmp_file4