#!/bin/bash
# Date 2013-04-05
# Written by wuhaiting
# hpacucli阵列卡标准函数

# 支持阵列卡  05:00.0 RAID bus controller: Hewlett-Packard Company Smart Array G6 controllers
# 管理包 hpacucli

# 返回标准格式：
# accelerator 1 ( Cache-size, OK, Serial-number, model )
# logicaldrive 1 ( 600GB, RAID1, OK, PD-num )
# physicaldrive 1I:1:1 ( SATA, 1000GB, OK, Serial-number, model )
#
# 容量大小转换成标准格式：64GB 73GB 146GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 4000GB
# RAID 0、1、1+0、5、50、none
# 接口类型 IDE、SCSI、SATA、SAS、SSD
# 状态：ok、fail、Optimal、online、Degraded、rebuild

# lightup 返回码:  0[success], 1[some disk lightup fail], 2[all of fail], 3[not found bad disk to lightup]

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

which hpacucli > /dev/null 2>&1
if [ "$?" -ne 0 ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/raid_package_install.sh
	which hpacucli > /dev/null 2>&1 || exit 1
fi
[ "$1" = "status" ] && DEBUG_TAG=yes
[ "$1" = "lightup" ] && LIGHT=yes && DEBUG_TAG=yes
return_code='0'
tmp_file1=$(mktemp) || exit 1
tmp_file2=$(mktemp)
tmp_file3=$(mktemp)
tmp_file4=$(mktemp)
tmp_file5=$(mktemp)

trap "rm -f $tmp_file1 $tmp_file2 $tmp_file3 $tmp_file4 $tmp_file5;" 0 1 2 9 15

HP_BIN="hpacucli"
$HP_BIN ctrl all show config detail | awk '{a=$1;$1="";print a$0}' > $tmp_file1
if grep -qE 'APPLICATION UPGRADE|No controllers detected' $tmp_file1 && \
	[ -x /usr/sbin/hpssacli ];then
	HP_BIN="hpssacli"
	$HP_BIN ctrl all show config detail | \
	awk '{a=$1;$1="";print a$0}' > $tmp_file1
fi

ctrl_num=$(awk '{if($0 ~ /^Slot:/){print $NF}}' $tmp_file1 |tr -s "\n" " ")
for ctrl_index in $ctrl_num;
do
	ctrl_start="Slot: ${ctrl_index}"
	ctrl_next=$(grep "^Slot:" $tmp_file1| \
		grep -A1 "$ctrl_start" | \
		grep -v "$ctrl_start")

	if [ "x${ctrl_next}" != "x" ];then
		sed -n "/${ctrl_start}/,/$ctrl_next/p" $tmp_file1 | grep -v "$ctrl_next" > $tmp_file2
	else
		sed -n "/${ctrl_start}/,/$ctrl_next/p" $tmp_file1 > $tmp_file2
	fi

	ctrl_model=$(grep -B2 "$ctrl_start" $tmp_file1|grep "Smart.*in Slot"|awk '{print $3}')
	ctrl_model="${ctrl_model:-unknown}"

	ctrl_ld_start="Logical Drive"
	sed -n "/${ctrl_start}/,/$ctrl_ld_start/p" $tmp_file2 > $tmp_file3

	awk -F '[ |:]' '{
		if($0 ~ /Serial Number/){
			ctrl_sn=$NF;
		}else if($0 ~ /Total Cache Size/){
			ctrl_cache_size=int($(NF-1))$NF;
		}else if($0 ~ /Controller Status/){
			ctrl_status=$NF;
		}else if($0 ~ /Cache Status:/){
			ctrl_cache_status=$NF;
		}else if($0 ~ /Battery.*Status/){
			ctrl_battery_status=$NF;
		}
	}
	END{
		if(ctrl_cache_status == ""){ctrl_cache_status="OK"};
		if(ctrl_battery_status == ""){ctrl_battery_status="OK"};
		if(ctrl_status != "OK" || ctrl_cache_status != "OK" || ctrl_battery_status != "OK"){
			if("'$DEBUG_TAG'"=="yes"){
				if(ctrl_status != "OK"){printf("'$ctrl_index'@Controller fail status=%s\n",ctrl_status)};
				if(ctrl_cache_status != "OK"){printf("'$ctrl_index'@Controller cache fail status=%s\n",ctrl_cache_status)};
				if(ctrl_battery_status != "OK"){printf("'$ctrl_index'@Controller battery fail status=%s\n",ctrl_battery_status)};
			};
			ctrl_status="fail"
		};
		if(ctrl_cache_size == ""){ctrl_cache_size="none"};
		if("'$DEBUG_TAG'" != "yes"){
			printf("accelerator %s ( %s, %s, %s, %s)\n",'$ctrl_index',ctrl_cache_size,ctrl_status,ctrl_sn,"'$ctrl_model'");
		}
	}' $tmp_file3

	ld_num=$(grep "Logical Drive:" $tmp_file2 | awk '{print $NF}' | tr -s "\n" " ")
	for ld_index in $ld_num;
	do
		ld_start="Logical.*Drive: *$ld_index"
		ld_next=$(grep "Logical.*Drive:" $tmp_file2 | \
						grep -A1 "$ld_start$" | grep -v "$ld_start")

		if [ "$ld_next" != "" ];then
			sed -n "/${ld_start}$/,/$ld_next/p" $tmp_file2 | grep -v "$ld_next" > $tmp_file3
		else
			sed -n "/${ld_start}$/,/$ld_next/p" $tmp_file2 > $tmp_file3
		fi

		ld_pd_start="Logical Drive Label:"
		sed -n "/${ld_start}/,/$ld_pd_start/p" $tmp_file3 > $tmp_file4

		pd_num=$(grep physicaldrive $tmp_file3 | awk '{print $1" "$2}'| sort -u|wc -l)
		hotspare_num=$(grep -ci "Spare Drive" $tmp_file3)
		pd_num=$((pd_num - hotspare_num))
		awk -F '[ |:]' '{
			if($0 ~ /^Size/){
				if($NF ~ /TB/){
					ld_size=$(NF-1)$NF;
				}else{
					ld_size=int($(NF-1))$NF;
				}
			}else if($0 ~ /Fault Tolerance/){
				ld_raid=$NF;
			}else if($0 ~ /^Status/){
				ld_status=$NF;
			}else if($0 ~ /Disk Name/){
				for(g=1;g++;g<=NF){
					if($g ~ /dev/){
						len=split($g,sdmap_array,"/");
						if(ld_status!="OK"){
							if("'$DEBUG_TAG'" == "yes"){
								printf("'$ctrl_index':'$ld_index'@Raid fail status=%s\n",ld_status);
							};
							ld_status="fail";
						}
						break;
					}
				}
			}
		}
		END{
			if("'$DEBUG_TAG'" != "yes"){
				printf("logicaldrive '$ctrl_index':'$ld_index' ( %s, RAID%s, %s, %s, %s )\n",ld_size,ld_raid,ld_status,"'$pd_num'",sdmap_array[len]);
			}
		}' $tmp_file4
		unset pd_num

		pd_num=$(grep physicaldrive $tmp_file3 | awk '{print $2}'| sort -u| tr -s "\n" " ")
		for pd_index in $pd_num;
		do
			pd_start="^physicaldrive.*${pd_index}"
			#pd_next=`grep "^physicaldrive" $tmp_file3 | grep -v "(.*)" | grep -A1 "$pd_start" | grep -v "$pd_start"`

			pd_next="PHY Transfer Rate"

			if [ "$ld_next" != "" ];then
				sed -n "/${pd_start}$/,/$pd_next/p" $tmp_file3 | grep -v "$ld_next" > $tmp_file4
			else
				sed -n "/${pd_start}$/,/$pd_next/p" $tmp_file3 > $tmp_file4
			fi

			awk -F '[ |:]' '{
				if($0 ~ /^physicaldrive .*:.*:.*/){
					LOCATE=$(NF-2)":"$(NF-1)":"$NF;
				}else if($0 ~ /^Status/){
					pd_status=$NF;
					if(pd_status!="OK" && pd_status!="Rebuilding"){
						if("'$DEBUG_TAG'" == "yes"){
							printf("'$ctrl_index'@%s@Disk fail status=%s\n", LOCATE, pd_status);
						};
						pd_status="fail";
					}
				}else if($0 ~ /Spare Drive/){
					hotspare=1;
					exit;
				}else if($0 ~ /Interface Type/){
					pd_type=$NF;
					if($0 ~ /Solid State/){pd_type="SSD";};
				}else if($0 ~ /^Size/){
					pd_volume=int($(NF-1))$NF;
				}else if($0 ~ /Serial Number/){
					pd_sn=$NF;
				}else if($0 ~ /^Model/){
					pd_model=$NF;
					if(pd_model ~ /SSD/){pd_type="SSD";};
				}
			}END{
				if("'$DEBUG_TAG'" != "yes" && hotspare != 1){
					printf("physicaldrive %s ( %s, %s, %s, %s, %s )\n",LOCATE,pd_type,pd_volume,pd_status,pd_sn,pd_model);
				}
			}' $tmp_file4 | tee -a $tmp_file5

			unset pd_start pd_next
		done
		unset pd_num ld_start ld_next ld_pd_start
	done
	unset ctrl_start ctrl_next ctrl_ld_start
done

hotspare_num=$(grep -ci "Spare Drive" $tmp_file1)
if [ $hotspare_num -gt 0 ];then
	[ "$DEBUG_TAG" != "yes" ] && echo "logicaldrive none ( none, Hotspare, none, $hotspare_num )"

		pd_num=$(grep -E "physicaldrive|Spare Drive" $tmp_file1 | \
			grep -B1 "Spare Drive"|awk '{if($0 ~ /physicaldrive/){print $NF}}'| \
			sort -u| tr -s "\n" " ")
		for pd_index in $pd_num;
		do
			pd_start="^physicaldrive.*${pd_index}"
			pd_next="PHY Transfer Rate"

			sed -n "/${pd_start}$/,/${pd_next}/p" $tmp_file1 > $tmp_file4

			awk -F '[ |:]' '{
				if($0 ~ /^physicaldrive .*:.*:.*/){
					LOCATE=$(NF-2)":"$(NF-1)":"$NF;
				}else if($0 ~ /^Status/){
					pd_status=$NF;
					if(pd_status!="OK" && pd_status!="Rebuilding"){
						if("'$DEBUG_TAG'" == "yes"){
							printf("'$ctrl_index'@%s@Disk fail status=%s\n", LOCATE, pd_status);
						};
						pd_status="fail";
					}else{
						pd_status="OK-Hotspare";
					}
				}else if($0 ~ /Spare Drive/){
					hotspare=1;
				}else if($0 ~ /Interface Type/){
					pd_type=$NF;
					if($0 ~ /Solid State/){pd_type="SSD";};
				}else if($0 ~ /^Size/){
					pd_volume=int($(NF-1))$NF;
				}else if($0 ~ /Serial Number/){
					pd_sn=$NF;
				}else if($0 ~ /^Model/){
					pd_model=$NF;
					if(pd_model ~ /SSD/){pd_type="SSD";};
				}
			}END{
				if("'$DEBUG_TAG'" != "yes" && hotspare = 1){
					printf("physicaldrive %s ( %s, %s, %s, %s, %s )\n",LOCATE,pd_type,pd_volume,pd_status,pd_sn,pd_model);
				}
			}' $tmp_file4 | tee -a $tmp_file5

			unset pd_start pd_next
		done
fi

if [ "x$LIGHT" = "xyes" ];then
	for locate in $(grep -w "Disk" $tmp_file5|awk -F '@' '{printf("%s@%s",$1,$2)}'|sort -u);
	do
		CTRL="${locate%%@*}"
		disk_locate="${locate##*@}"
		$HP_BIN ctrl slot=${CTRL} physicaldrive ${disk_locate} modify led=on >/dev/null 2>&1
		if [ "$?" -eq 0 ];then
			echo "success lightup slot=$CTRL $disk_locate disk."
			success_light='yes'
		else
			echo "fail lightup slot=$CTRL $disk_locate disk."
			fail_light='yes'
		fi
	done
fi

if [ "x${LIGHT}" = "xyes" ];then
	[ "x${success_light}" = "xyes" ] && [ "x${fail_light}" = "x" ] && return_code='0'
	[ "x${success_light}" = "xyes" ] && [ "x${fail_light}" = "xyes" ] && return_code='1'
	[ "x${success_light}" = "x" ] && [ "x${fail_light}" = "xyes" ] && return_code='2'
	[ "x${success_light}" = "x" ] && [ "x${fail_light}" = "x" ] && return_code='3'
fi

exit $return_code


