#!/bin/bash
# Date 2013-04-07
# Written by wuhaiting
# megacli阵列卡标准函数

# 支持阵列卡  10:00.0 RAID bus controller: LSI Logic / Symbios Logic LSI MegaSAS 9260 (rev 05)
# 管理包 megacli

# 返回标准格式：
# accelerator 1 ( Cache-size, OK, Serial-number, model )
# logicaldrive 1 ( 600GB, RAID1, OK, PD-num )
# physicaldrive 1I:1:1 ( SATA, 1000GB, OK, Serial-number, model )
#
# 容量大小转换成标准格式：64GB 73GB 146GB 240GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 4000GB
# RAID 0、1、1+0、5、50、none
# 接口类型 IDE、SCSI、SATA、SAS、SSD
# 状态：ok、fail、Optimal、online、Degraded、rebuild

# lightup 返回码:  0[success], 1[some disk lightup fail], 2[all of fail], 3[not found bad disk to lightup]

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

which megacli > /dev/null 2>&1
if [ "$?" -ne 0 ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/raid_package_install.sh
	which megacli > /dev/null 2>&1 || exit 1
fi

[ "$1" = "status" ] && DEBUG_TAG=yes
[ "$1" = "lightup" ] && LIGHT=yes && DEBUG_TAG=yes
return_code='0'
tmp_file1=$(mktemp) || exit 1
tmp_file2=$(mktemp)
tmp_file3=$(mktemp)
tmp_file4=$(mktemp)
tmp_file5=$(mktemp)
tmp_file6=$(mktemp)

trap "rm -f $tmp_file1 $tmp_file2 $tmp_file3 $tmp_file4 $tmp_file5 $tmp_file6 MegaSAS.log CmdTool.log;" 0 1 2 9 15

/usr/sbin/megacli -AdpGetPciInfo -aall |tr -d '\r' > $tmp_file1
for control_id in $(grep 'Controller' $tmp_file1|awk '{print $NF}');
do
	CTL_BUS_ARRAY[$control_id]=$(grep -A2 -E "Controller .*${control_id}$" $tmp_file1| \
							awk -F ':' '{if($0 ~ /Bus Number/){print $2}}'|sed 's/ //g')
	CTL_DEV_ARRAY[$control_id]=$(grep -A3 -E "Controller .*${control_id}$" $tmp_file1| \
							awk -F ':' '{if($0 ~ /Device Number/){print $2}}'|sed 's/ //g')
done

/usr/sbin/megacli -ldpdinfo -aall |tr -d '\r' > $tmp_file1
/usr/sbin/megacli -adpallinfo -aall |tr -d '\r' > $tmp_file2

if [ "${#CTL_BUS_ARRAY[@]}" -eq 0 ];then
	grep "^Adapter.*#" $tmp_file1 | cut -d "#" -f 2 | sort -n > $tmp_file3
	ls -l --color=never /dev/disk/by-path/| \
	grep -vE 'part|usb|sr[0-9]|total' | \
	awk -F 'pci-' '{print $NF}' | \
	awk -F ':' '{split($3,b,".");printf("%s:%s\n"),$2,b[1]}'| \
	sort -nu > $tmp_file4
	for PCI_MSG in $(paste -d ':' $tmp_file3 $tmp_file4);
	do
		control_id="${PCI_MSG%%:*}"
		CTL_BUS_ARRAY[$control_id]=$(echo $PCI_MSG | awk -F ':' '{print $2}')
		CTL_DEV_ARRAY[$control_id]=$(echo $PCI_MSG | awk -F ':' '{print $3}')
	done
fi

fail_disk_num=$(awk '{if($0 ~ /Failed Disks/){print $NF}}' $tmp_file2)
ctrl_num=$(grep "^Adapter.*#" $tmp_file1 | cut -d "#" -f 2 | sort -n)
for ctrl_index in $ctrl_num;
do
	ctrl_start="Adapter *#${ctrl_index}"
	ctrl_next=$(grep "^Adapter.*#" $tmp_file2 | grep -A1 "$ctrl_start" | grep -v "$ctrl_start")

	if [ "$ctrl_next" != "" ];then
		sed -n "/${ctrl_start}$/,/$ctrl_next/p" $tmp_file1 | grep -v "$ctrl_next" > $tmp_file3
		sed -n "/${ctrl_start}$/,/$ctrl_next/p" $tmp_file2 | grep -v "$ctrl_next" > $tmp_file4
	else
		sed -n "/${ctrl_start}/,/$ctrl_next/p" $tmp_file1 > $tmp_file3
		sed -n "/${ctrl_start}/,/$ctrl_next/p" $tmp_file2 > $tmp_file4
	fi

	awk '{
		if($0 ~ /^Product Name/){
			for(i=1;i<=NF;i++){if($i ~ /RAID/ || $i ~ /PERC/){ctrl_model=$i$(i+1)}};
			if(ctrl_model == ""){for(i=1;i<=NF;i++){if($i ~ /[0-9]/){ctrl_model=$i}}};
		}else if($0 ~ /^Serial No/){
			ctrl_sn=$NF;
			if(ctrl_sn == ":"){ctrl_sn="none"}
		}else if($0 ~ /^Memory Size/){
			ctrl_cache=$NF;
		}else if($0 ~ /^ECC Bucket Count/){
			ecc_count=$NF;
		}else if($0 ~ /Memory Correctable Errors/){
			mem_ecc_count=$NF;
		}else if($0 ~ /Memory Uncorrectable Errors/){
			mem_unecc_count=$NF;
		}
	}
	END{
		if(ecc_count == "0"){ecc_count="OK"};
		if(mem_ecc_count == "0"){mem_ecc_count="OK"};
		if(mem_unecc_count == "0"){mem_unecc_count="OK"};
		if(ecc_count != "OK" || mem_ecc_count != "OK" || mem_unecc_count != "OK"){
			if("'$DEBUG_TAG'"=="yes"){
				if(ecc_count!="0"){printf("3:4@'$ctrl_index'@Controller Ecc Bucket count=%s\n",ecc_count)};
				if(mem_ecc_count!="0"){printf("3:4@'$ctrl_index'@Controller Memory Correctable Errors %s\n",mem_ecc_count)};
				if(mem_unecc_count!="0"){printf("3:2@'$ctrl_index'@Controller Memory Uncorrectable Errors %s\n",mem_unecc_count)};
			}
			ctrl_status="fail";
		}else{
			ctrl_status="OK";
		}
		if("'$DEBUG_TAG'" != "yes"){
			printf("accelerator '$ctrl_index' ( %s, %s, %s, %s )\n",ctrl_cache,ctrl_status,ctrl_sn,ctrl_model);
		}
	}' $tmp_file4

	ld_num=$(grep '^Virtual Drive:' $tmp_file3 | awk '{print $3}' | tr -s "\n" " ")
	for ld_index in $ld_num;
	do
		ld_start="Virtual.*Drive: *${ld_index}"
		ld_next=$(grep "^Virtual.*Drive:" $tmp_file3 | \
			grep -A1 -w "$ld_start" | grep -v -m1 "$ld_start")

		if [ "$ld_next" != "" ];then
			sed -n "/${ld_start}\>/,/$ld_next/p" $tmp_file3 | \
										grep -v "$ld_next" > $tmp_file4

		else
			sed -n "/${ld_start}/,/$ld_next/p" $tmp_file3 > $tmp_file4
		fi

		partical_ld_index=$(grep "Virtual.*Drive" $tmp_file4|tr -d "()"|awk '{print $NF}')

		sdmap=$(ls -l --color=never /dev/disk/by-path/ | \
				grep -vE 'part|usb|sr[0-9]' | awk -F 'pci-' '{print $NF}'|\
				awk -F ':' '{
					c=split($3,d,".");
					if($(NF-1) == "'$partical_ld_index'" && \
					$2 == '${CTL_BUS_ARRAY[$ctrl_index]}'){
						if(d[1] != "" && \
							d[1] == '${CTL_DEV_ARRAY[$ctrl_index]}'){
							a=split($0,b,"/");
							print b[a];
							exit;
						}
					}
				}')

		awk -F '[ :]' '{
			if($0 ~ /^RAID Level/){
				gsub("Primary-","",$(NF-4));
				gsub(",","",$(NF-4));
				ld_raid_pri=$(NF-4);
			}else if($0 ~ /^Size/){
				ld_volume=$(NF-1)$NF;
			}else if($0 ~ /^State/){
				ld_state=$NF;
				if(ld_state!="Optimal"){
					if("'$DEBUG_TAG'"=="yes"){
						printf("1:2:2@'$ctrl_index':'$ld_index'@Raid fail status=%s\n",ld_state);
					};
					ld_state="fail";
				}
			}else if($0 ~ /^Number Of Drives/){
				ld_pd_num=$NF;
			}else if($0 ~ /^Span Depth/ && "'$DEBUG_TAG'" != "yes"){
				ld_span=$NF;

				if(ld_span > 1){
					printf("logicaldrive '$ctrl_index':'$ld_index' ( %s, RAID%s+0, %s, %d, '$sdmap' )\n",ld_volume,ld_raid_pri,ld_state,ld_pd_num*ld_span);
				}else{
					printf("logicaldrive '$ctrl_index':'$ld_index' ( %s, RAID%s, %s, %d, '$sdmap' )\n",ld_volume,ld_raid_pri,ld_state,ld_pd_num*ld_span);
				};
				exit
			}
		}' $tmp_file4

		pd_num=$(grep "^Slot Number:" $tmp_file4 | awk '{print $NF}' | tr -s "\n" " ")
		for pd_index in $pd_num;
		do
			pd_start="Slot.*Number: *${pd_index}"
			pd_next=$(grep "^Slot.*Number:" $tmp_file4 | \
			grep -w -A1 "$pd_start" | grep -v -m1 "$pd_start")
			if [ "$pd_next" != "" ];then
				sed -n "/${pd_start}$/,/$pd_next/p" $tmp_file4 | \
											grep -v "$pd_next" > $tmp_file5
			else
				sed -n "/${pd_start}/,/$pd_next/p" $tmp_file4 > $tmp_file5
			fi
			EDID=$(grep -B1 "${pd_start}$" $tmp_file4| awk '{if($0 ~ /^Enclosure Device ID/){print $NF}}')
			gbl_location="${gbl_location} $ctrl_index:$EDID:$pd_index"

			awk '{
				if($0 ~ /^Media Error Count/){
					pd_media_count=$NF;
				}else if($0 ~ /^Other Error Count/){
					pd_other_count=0;
				}else if($0 ~ /^Predictive Failure Count/){
					pd_pre_count=$NF;
				}else if($0 ~ /^PD Type/){
					pd_type=$NF;
				}else if($0 ~ /^Raw Size/){
					a=9999999999;
					if($4 == "GB"){
						split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
						for(var in b){
							c=b[var]-$3;
							if(c>=0 && c<a){a=c;d=b[var];};
						}
						pd_volume=d"GB";
					}else if($4 == "TB"){
						split("1 2 2.5 3 4",b," ");
						for(var in b){
							c=b[var]-$3;
							if(c>0 && c<a){a=c;d=b[var];};
						}
						pd_volume=d"TB";
					}
				}else if($0 ~ /^Firmware state/){
					o=$3$4$5;
					if(o != "Online,SpunUp" && o != "Online,Spundown" && o != "Hotspare,SpunUp" && o != "Rebuild"){
						pd_tmp_status="Fail";
					}else {
						if(o !~ /Rebuild/){
							pd_tmp_status="OK";
						}else{
							pd_tmp_status="Rebuild";
						}
					}
				}else if($0 ~ /^Inquiry Data/){
					pd_sn=$3;
					$1="";$2="";$3="";
					gsub("^ *","",$0);
					gsub(" ","-",$0);
					pd_model=$0;
				}else if($0 ~ /^Media Type/){
					if(pd_sn ~ /SSD/ || pd_model ~ /SSD/ || $0 ~ /Solid State/){pd_type="SSD"};

					if(pd_media_count > 100 || pd_other_count != "0" || pd_pre_count != "0" || pd_tmp_status == "Fail"){
						if("'$DEBUG_TAG'"=="yes"){
							if(pd_media_count!="0"){printf("'$ctrl_index':'$EDID':'$pd_index'@Disk media error count %s\n",pd_media_count)};
							if(pd_other_count!="0"){printf("'$ctrl_index':'$EDID':'$pd_index'@Disk other error count %s\n", pd_other_count)};
							if(pd_pre_count!="0"){printf("'$ctrl_index':'$EDID':'$pd_index'@Disk Predictive Failure Count %s\n", pd_pre_count)};
							if(pd_tmp_status=="Fail"){printf("'$ctrl_index':'$EDID':'$pd_index'@Disk Firmware state=%s\n", o)};
						};
						pd_status="fail";
					}else{
						pd_status=pd_tmp_status
					};

					if("'$DEBUG_TAG'"!="yes"){
						printf("physicaldrive '$ctrl_index'I:'$EDID':'$pd_index' ( %s, %s, %s, %s, %s )\n",pd_type,pd_volume,pd_status,pd_sn,pd_model);
					}
				}
			}' $tmp_file5 | tee -a $tmp_file6

			unset pd_start pd_next
		done
		unset ld_start ld_next pd_num
	done
	unset ctrl_start ctrl_next
done

# 检查是否有Hotspare
/usr/sbin/megacli -pdlist -aall |tr -d '\r' > $tmp_file1
hotspare_num_line=$(grep -Ec "Firmware state: Hotspare|Copyback" $tmp_file1)
if [ "$hotspare_num_line" -ge 1 ];then
	[ "$DEBUG_TAG" != "yes" ] && echo "logicaldrive none ( none, Hotspare, none, $hotspare_num_line )"
	hotspare_num=`grep -E "Firmware.*state.*Hotspare|Slot Number:|Firmware.*Copyback" $tmp_file1 | \
							grep -EB1 "Firmware.*state.*Hotspare|Firmware.*Copyback" | \
							awk -F ':' '{if($0 ~ /^Slot Number/){print $NF}}' |tr -s "\n" " "`
	for hotspare_index in $hotspare_num;
	do
		hotspare_start="Slot.*Number:.*${hotspare_index}"
		hotspare_next=$(grep "^Slot.*Number:" $tmp_file1 | \
			grep -A1 "$hotspare_start" |grep -v "$hotspare_start")
		if [ "$hotspare_next" != "" ];then
			sed -n "/${hotspare_start}$/,/$hotspare_next/p" $tmp_file1 | \
										grep -v "$hotspare_next" > $tmp_file2
		else
			sed -n "/${hotspare_start}/,/$hotspare_next/p" $tmp_file1 > $tmp_file2
		fi
		EDID=$(grep -B1 "${hotspare_start}$" $tmp_file1| awk '{if($0 ~ /^Enclosure Device ID/){print $NF}}')
		ctrl_index=$(grep -E "${hotspare_start}$|^Adapter.*#" $tmp_file1|\
			grep "^Adapter.*#"| cut -d "#" -f 2 | sort -n)
		gbl_location="${gbl_location} $ctrl_index:$EDID:$hotspare_index"
		awk '{
			if($0 ~ /^Media Error Count/){
				pd_media_count=$NF;
			}else if($0 ~ /^Other Error Count/){
				pd_other_count=$NF;
			}else if($0 ~ /^Predictive Failure Count/){
				pd_pre_count=$NF;
			}else if($0 ~ /^PD Type/){
				pd_type=$NF;
			}else if($0 ~ /^Raw Size/){
				a=9999999999;
				if($4 == "GB"){
					split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
					for(var in b){
						c=b[var]-$3;
						if(c>=0 && c<a){a=c;d=b[var];};
					}
					pd_volume=d"GB";
				}else if($4 == "TB"){
					split("1 2 2.5 3 4",b," ");
					for(var in b){
						c=b[var]-$3;
						if(c>0 && c<a){a=c;d=b[var];};
					}
					pd_volume=d"TB";
				}
			}else if($0 ~ /^Firmware state/){
				o=$3$4$5;
				if(o != "Online,SpunUp" && o != "Online,Spundown" && o != "Hotspare,SpunUp" && o != "Hotspare,Spundown" && o != "Rebuild" && o != "Copyback"){
					pd_tmp_status="Fail";
				}else {
					pd_tmp_status="OK-Hotspare";
				}
			}else if($0 ~ /^Inquiry Data/){
				pd_sn=$3;
				$1="";$2="";$3="";
				gsub("^ *","",$0);
				gsub(" ","-",$0);
				pd_model=$0;

				if(pd_sn ~ /SSD/ || pd_model ~ /SSD/){pd_type="SSD"};

				if(pd_media_count == "0" && pd_other_count == "0" && pd_pre_count == "0" && pd_tmp_status ~ /OK/){
					pd_status="OK-Hotspare";
				}else{
					if("'$DEBUG_TAG'"=="yes"){
							if(pd_media_count!="0"){printf("'$ctrl_index':'$EDID':'$hotspare_index'@Disk media error count %s\n", pd_media_count)};
							if(pd_other_count!="0"){printf("'$ctrl_index':'$EDID':'$hotspare_index'@Disk other error count %s\n", pd_other_count)};
							if(pd_pre_count!="0"){printf("'$ctrl_index':'$EDID':'$hotspare_index'@Disk Predictive Failure Count %s\n", pd_pre_count)};
							if(pd_tmp_status=="Fail"){printf("'$ctrl_index':'$EDID':'$hotspare_index'@Disk(Hotspare) Firmware state=%s\n", o)};
					};
					pd_status="fail";
				};
				if("'$DEBUG_TAG'"!="yes"){
					printf("physicaldrive '$ctrl_index'I:'$EDID':'$hotspare_index' ( %s, %s, %s, %s, %s )\n",pd_type,pd_volume,pd_status,pd_sn,pd_model);
				}
			}
		}' $tmp_file2 | tee -a $tmp_file6
		unset hotspare_start hotspare_next
	done
fi

# Locate bad disk, and light led up
if [ "x${DEBUG_TAG}" = "xyes" ];then
	pre_fail_locate=$(grep 'Disk' $tmp_file6|awk -F '@' '{print $1}'| sort -u)

	if [ "$fail_disk_num" -ge 1 ] && [ "x${pre_fail_locate}" == "x" ];then
		# First, if some disk that we missing.
		ALL_DISK=$(awk '{
			if($0 ~ /^Adapter/){
				CTRL=$NF;
			}else if($0 ~ /Enclosure Device ID/){
				EDID=$NF;
			}else if($0 ~ /Slot Number/){
				SLOT=$NF;
				printf("%s:%s:%s ",CTRL, EDID, SLOT);
			}
		}' $tmp_file1|tr -d '#')

		for which_disk in $ALL_DISK;
		do
			echo "$gbl_location"|grep -wq "$which_disk" || \
			echo "${which_disk}@Disk Failed" | tee -a $tmp_file6
		done
		pre_fail_locate=$(grep 'Disk' $tmp_file6|awk -F '@' '{print $1}'| sort -u)
	fi

	if [ "x${LIGHT}" = "xyes" ];then
		for mix_locate in $pre_fail_locate;
		do
			ctrl_num="${mix_locate%%:*}"
			EDID="${mix_locate#*:}"
			EDID="${EDID%:*}"
			SLOT="${mix_locate##*:}"
			megacli -pdlocate -start -physdrv[${EDID}:${SLOT}] -a${ctrl_num} >/dev/null 2>&1
			if [ "$?" -eq 0 ];then
				echo "success lightup ${ctrl_num}:${EDID}:${SLOT} disk."
				success_light='yes'
			else
				echo "fail lightup ${ctrl_num}:${EDID}:${SLOT} disk."
				fail_light='yes'
			fi
		done
	fi
fi

if [ "x${LIGHT}" = "xyes" ];then
	[ "x${success_light}" = "xyes" ] && [ "x${fail_light}" = "x" ] && return_code='0'
	[ "x${success_light}" = "xyes" ] && [ "x${fail_light}" = "xyes" ] && return_code='1'
	[ "x${success_light}" = "x" ] && [ "x${fail_light}" = "xyes" ] && return_code='2'
	[ "x${success_light}" = "x" ] && [ "x${fail_light}" = "x" ] && return_code='3'
fi

exit $return_code



