#!/bin/bash
# Date 2013-04-08
# Written by wuhaiting
#
# 01:00.0 Serial Attached SCSI controller: LSI Logic / Symbios Logic SAS2008 PCI-Express Fusion-MPT SAS-2 [Falcon] (rev 03)
# Fusion-MPT SAS-2 其实是hba卡，支持做raid,但是脚本这里只会显示相关raid信息

# 返回标准格式：
# physicaldrive 1I:1:1 ( SATA, 1000GB, OK, Serial-number, model )
# 容量大小转换成标准格式：64GB 73GB 146GB 300GB 500GB 600GB 1000GB 2000GB 2500GB 3000GB 4000GB
# 接口类型 IDE、SCSI、SATA、SAS、SSD
# 状态：ok、fail、Optimal、online、Degraded、rebuild

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LANG=C
export LC_ALL=C

[ "$1" = "status" ] && DEBUG_TAG=yes

which sas2ircu > /dev/null 2>&1
if [ "$?" -ne 0 ];then
	/bin/bash /usr/local/i386/public_repos/initial_system/hardware/raid_package_install.sh
	which sas2ircu > /dev/null 2>&1 || exit 1
fi

DEBUG_PARAM=$1

tmp_file=`mktemp` || exit 1
tmp_file1=`mktemp`
tmp_file2=`mktemp`

[ -c /dev/mpt3ctl ] && MNG_UTIL="sas3ircu" || MNG_UTIL="sas2ircu"

for PCI_ADDRESS in $(ls /sys/block/[svh]d*/device/sas_address);do
	echo "$(echo $PCI_ADDRESS|cut -d / -f4) $(awk -F'0x' '{print $2}' $PCI_ADDRESS)"
done > $tmp_file2

CONTOLLER_NUM=$($MNG_UTIL list | grep -A1 '-'| grep -v '-'| awk '{print $1}')
for CONTROLL_INDEX in $CONTOLLER_NUM; do
	$MNG_UTIL $CONTROLL_INDEX display > $tmp_file
	ctrl_start="Controller information"
	ctrl_next="IR Volume information"
	sed -n "/${ctrl_start}/,/${ctrl_next}/p" $tmp_file > $tmp_file1

	ctrl_slot_position=$(awk -F ':' '{if($1 ~ /Slot/){print $2}}' $tmp_file1| tr -d ' ')
	ctrl_model=$(awk -F ':' '{if($1 ~ /Controller type/){print $2}}' $tmp_file1| tr -d ' ')
	CONTOLL_PRINT=0

	ld_num=$(grep 'IR volume' $tmp_file |tr -s ' ' ':'|cut -d ':' -f3)
	for ld_index in $ld_num; do
		[ "$DEBUG_TAG" != "yes" ] && [ "$CONTOLL_PRINT" = "0" ] \
			&& CONTOLL_PRINT=1 \
			&& echo "accelerator $ctrl_slot_position ( none, none, none, $ctrl_model )"

		ld_start="IR volume ${ld_index}"
		ld_next=`grep "IR volume" $tmp_file|grep -A1 "$ld_start$"| grep -v "$ld_start"`
		if [ "$ld_next" != "" ]; then
			sed -n "/${ld_start}\>/,/${ld_next}\>/p" $tmp_file | \
				grep -v "$ld_next$" > $tmp_file1
		else
			sed -n "/${ld_start}\>/,//p" $tmp_file > $tmp_file1
		fi

		WWID=$(grep 'Volume wwid' $tmp_file1|cut -d : -f2)
		sdmap=$(grep "$WWID" $tmp_file2|awk '{print $1}')

		awk '{
			if($0 ~ /Status of volume/){
				ld_status=$(NF-1);
				if(ld_status != "Okay" && "'$DEBUG_TAG'"=="yes"){
					printf("1:2:2@'$ctrl_slot_position':'$ld_index'@Raid Fail status=%s\n",ld_status);
				}
				if(ld_status != "Okay"){ld_status="fail"}else{ld_status="OK"};
			}else if($0 ~ /RAID level/){
				ld_raid=$NF;
			}else if($0 ~ /Size.*MB/){
				g=$NF/1000;
				ld_volume=g"GB";
			}else if($0 ~ /PHY.*Enclosure.*Slot/){
				pd_num += 1
			}
		}END{
			if("'$DEBUG_TAG'" != "yes"){
				printf("logicaldrive '$ctrl_slot_position':'$ld_index' ( %s, %s, %s, %s, '$sdmap' )\n",ld_volume,ld_raid,ld_status,pd_num);
			}
		}' $tmp_file1

		pd_num=$(grep "PHY.*Enclosure.*Slot" $tmp_file1 | awk '{print $NF}'|tr -s "\n" ' ')
		for pd_index in $pd_num; do
			enc_id="${pd_index%:*}"
			slot_id="${pd_index#*:}"

			awk '{
					if($0 ~ /Device is a Hard disk/){
						a=1;
					}else if($0 ~ /Enclosure.*: '$enc_id'/){
						if(b!=1){print $0};
						b=1;
					}else if($0 ~ /Slot.*: '$slot_id'/ && b==1){
						c=1;
						print $0;
					}else if(a==1 && b==1 && c==1){
						print $0;
						if($0 ~ /Drive Type/){exit};
					};
				}' $tmp_file > $tmp_file1

			awk '{
				if($0 ~ /State/){
					pd_state=$(NF-1);
					if(pd_state!="Optimal" && "'$DEBUG_TAG'"=="yes"){
						printf("1:1:4@'$ctrl_slot_position':'$pd_index'@Disk fail status=%s\n",pd_state);
					};
                    if(pd_state!="Optimal" && tolower(pd_state) !~ /rebuild/){
                        pd_state="fail"
                    }else{
                        if(tolower(pd_state) ~ /rebuild/){pd_state="Rebuild"}else(pd_state="OK")
                    };
				}else if($0 ~ /Size.*MB.*sectors/){
					split($NF,b,"/");
					g=b[1]/1000;
					a=999999999;
					split("64 73 146 240 300 480 500 600 1000 1500 2000 2500 4000 5000 6000 8000",b," ");
					for(var in b){
						c=b[var]-g;
						if(c>=0 && c<a){a=c;d=b[var];};
					}
					pd_volume=d"GB";
				}else if($0 ~ /Model Number/){
					pd_model=$NF;
				}else if($0 ~ /Serial No/){
					pd_sn=$NF;
				}else if($0 ~ /Protocol/){
					pd_type=$NF;
				};
			}
			END{
				if("'$DEBUG_TAG'"!="yes"){
					printf("physicaldrive '$ctrl_slot_position'I:'$pd_index' ( %s, %s, %s, %s, %s )\n",pd_type,pd_volume,pd_state,pd_sn,pd_model);
				}
			}' $tmp_file1
			unset enc_id slot_id
		done
		unset ld_start ld_next sdmap
	done
	unset ctrl_start ctrl_next ctrl_slot_position ctrl_model
done

rm -f $tmp_file $tmp_file1 $tmp_file2
