#!/bin/bash
#@version=0.9

# 1. check:     服务器上架验收(硬件信息, 系统环境, 运维基础环境)
# 2. cheos:     检查系统环境非硬件(系统环境, 运维基础环境)
# 3. game:      游戏事业部验收(硬件信息, 系统环境)
# 4. view:      定制光盘硬件配置查看(硬件信息)

# Usage Help:
# /bin/bash script.sh [check|cheos|game|view]

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
export LANG=en_US.UTF-8
export LC_ALL=en_US.UTF-8

TMPFILE=$(mktemp)
# 验收告警信息
TMPFILE1=$(mktemp)
# 机器验收是否合格
RESULT_FILE=$(mktemp)

function exit_func () {
    local CODE="${1:-0}"
    cat $TMPFILE1 >> $log_file
    rm -f $TMPFILE $TMPFILE1 $RESULT_FILE
    exit $CODE
}
trap "exit_func;exit" 0 1 2 3 9 15
exec 200<>$0
flock -n -e 200 || { echo "Script Already Running,exit";exit 1; }

COMMAND=$1
# 验收接口code定义, 0:success, other's code fail
export global_warning="0"
echo $global_warning > $RESULT_FILE

BASENAME=$(basename $0)
base_dir="/usr/local/i386"
public_dir="${base_dir}/public_repos"
hw_base_dir="/var/log/.hardware_check"
log_dir="/var/log"
log_file="$log_dir/install_os.log"
hardware_file="/etc/hardware_info"
reposfile="repos_server.log"
IP_ADDR=$(ifconfig|grep "inet addr"|awk '{print $2}'| \
    awk -F ":" '{print $2}'|grep -v -E '^127\.'|head -n1)


grep -iqE "ubuntu|debian" /etc/issue > /dev/null 2>&1 && LINUX_VERSION=ubuntu
grep -iqE "centos|red" /etc/issue > /dev/null 2>&1 && LINUX_VERSION=centos
RUN_TIME=$(date "+%Y-%m-%d %H:%M:%S")
export gblserver_info=""
export gblserver_model=""

function OUTPUT_GREEN (){
    echo $1 >> $TMPFILE1
    echo -e "\033[32;40;1m$1\033[0m"
}

function OUTPUT_RED (){
    echo $1 >> $TMPFILE1
    echo '1' > $RESULT_FILE
    global_warning="1"
    echo -e "\033[31;40;1m$1\033[0m"
}

function OUTPUT_YELLOW (){
    echo $1 >> $TMPFILE1
    echo '1' > $RESULT_FILE
    global_warning="1"
    echo -e "\033[33;40;1m$1\033[0m"
}

function sysop_wget_info () {
    local status
    if [ "$1" = '-n' ];then
        # wget with not username and password
        wget --quiet --output-document=$3 --timeout=4 --tries=2 "$2"
        status=$?
    else
        # wget with username and password
        wget --http-user=initial_system --http-passwd=ini_sys_1289UYH \
            --quiet --output-document=$2 --timeout=4 --tries=2 "$1"
        status=$?
    fi
    return $status
}

function get_file_info (){
    if [ $# -eq 2 ];then
        local query_tag=$1
        local query_file=$2
    else
        local query_tag=$1
        local query_file=$log_file
    fi

    if [ -s $query_file ];then
        grep -Ew "^${query_tag}" $query_file |\
        tail -n1 | cut -d ':' -f 2-|sed  's/^ *//'
    fi
}

function get_vari_info() {
    local src tag
    if [ $# -eq 2 ];then
        tag=$1
        src=$2
        eval printf \"\$$src\" | grep -Ew "^${tag}"| \
        tail -n1 | cut -d ':' -f 2-|sed 's/^ *//'
    elif [ $# -eq 3 ] && [ "$1" == "line" ];then
        tag=$2
        src=$3
        eval printf \"\$$src\" | grep -iE "$tag"
    fi
}

function UPLOAD_WARN () {
    # upload_code is determine whether server acceptance ok or fail
    upload_code="${1:-0}"
    upload_msg=$(base64 -w0 $2|sed 's/+/%2B/g')

    #UPLOAD_URL="http://test166.sysop.duowan.com:8099/webservice/serverOnline/updateAuthorizeResult.do?"
    UPLOAD_URL="http://dcm.sysop.duowan.com:9098/webservice/serverOnline/updateAuthorizeResult.do?"
    assert_id=$(get_file_info "assert_id")
    [ "x${assert_id}" = "x" ] && \
    { echo "assert_id null upload fail!" >> $log_file;return 1; }

    post_data="assert_id=${assert_id}&code=${upload_code}&message=${upload_msg}"

    wget -q --post-data "$post_data" --timeout=6 \
        --tries=2 "$UPLOAD_URL" --output-document /dev/null
    wget_status="$?"
    if [ "$wget_status" -ne 0 ];then
        echo "upload DCM URL fail, code:$wget_status" >> $log_file

        BASE64=$(echo "${UPLOAD_URL}${post_data}" |base64 -w0)
        wget --http-user=manifold --http-passwd=dw_manifold \
        --output-document=/dev/null --timeout=6 --tries=3 --quiet \
        "http://manager.repos.yy.duowan.com:63217/proxy_queue_api.php?GET_QUEUE=$BASE64"

        [ $? -ne 0 ] && echo "platform porxy upload to DCM fail" >> $log_file
    fi

}

function compare_string () {
    # 0:equal, 1 different
    local string1=$(echo $1|tr "[a-z]" "[A-Z]")
    local string2=$(echo $2|tr "[a-z]" "[A-Z]")
    local warn_code=$3
    local warn_msg="$4"

    if [ "${string1}" != "${string2}" ];then
        OUTPUT_RED "$warn_msg, 当前:${string1} ===> 应该为:${string2}"
        return 1
    else
        return 0
    fi
}

function compare_like_str () {
    # It will return 0 when $2 is $1 subset, other return 1.
    local string1=$(echo $1|tr "[a-z]" "[A-Z]")
    local string2=$(echo $2|tr "[a-z]" "[A-Z]")
    local warn_code=$3
    local warn_msg=$4


    local num=$(echo "$string1"| wc -w)
    until [ $num -eq 0 ];do
        local str1=$(echo "$string1"|awk '{c='$num';print $c}')
        local str2=$(echo "$string2"|awk '{c='$num';print $c}')
        num=$((num-1))

        if ! echo "$str1"|grep -iq "$str2";then
            OUTPUT_RED "$warn_msg, 当前:${string1} ===> 应该为:${string2}"
            return 1
        fi
    done
}

function data_partition_is_ok () {
    export MOUNT_DATA=$(awk '{print $2}' /etc/mtab|grep data |tr -s "\n" " ")
    local IS_DATA_ZONE=$(df -l | \
        awk '{if($NF == "/data" && $2 > 50000000){print "yes";exit}}')

    if [ "x${IS_DATA_ZONE}" != "xyes" ];then
        OUTPUT_YELLOW "/data分区小于50G"
    fi
}

function swap_is_ok () {
    local swap_status
    local SWAP_ZONE=$(swapon -s|grep -cv "Filename")
    if [ "$SWAP_ZONE" -lt 1 ];then
        compare_string "0" "1" "158" "swap分区没有挂载"
        swap_status=1
    else
        swap_status=0
    fi
    return $swap_status
}

function package_is_ok () {
    local packages error_pak

    if [ "$LINUX_VERSION" = "ubuntu" ];then
        packages="iptables ntpdate ethtool wget libssl0.9.8 mcelog \
        coreutils lrzsz vim syslog-ng ia32-libs"
        for pak in $packages;
        do
            dpkg -l $pak | grep -qE "^ii *${pak}" || \
            error_pak="${error_pak} $pak"
        done

        SSL_VERSION=$(dpkg -l libssl1.0.0 2>/dev/null| \
            grep '^ii.*libssl1.0.0'|awk '{print $3}')
        if [ "x${SSL_VERSION}" != "x" ] && dpkg --compare-versions \
            "${SSL_VERSION}" lt "1.0.1-4ubuntu5.12";then
            OUTPUT_RED "openssl 版本低于1.0.1-4ubuntu5.12"
        fi

    elif [ "$LINUX_VERSION" = "centos" ];then
        packages="ntpdate iptables ethtool coreutils pciutils \
        openssh-clients bind-utils libstdc++"
        packages_num=$(echo $packages| awk '{print NF}')

        rpm -qa $packages > $TMPFILE

        for pak in $packages;
        do
            grep -qw "${pak}" $TMPFILE || \
            error_pak="${error_pak} $pak"
        done

    fi

    [ "x${error_pak}" != "x" ] && OUTPUT_YELLOW "系统包安装有问题:$error_pak"
}

function data_directory_is_ok () {
    local error_dir check_dir

    for check_dir in bizop pkg services;
    do
        [ ! -d /data/$check_dir ] && error_dir="${error_dir} $check_dir"
    done

    if [ "x${error_dir}" != "x" ];then
        OUTPUT_RED "/data目录文件夹初始化有问题,影响包发布"
        return 1
    else
        return 0
    fi
}

function kernel_is_acceptable () {
    local knl_version exit_status

    cur_knl_version=$(uname -r)
    cur_runlevel=$(runlevel|awk '{print $2}')

    case $(uname -m) in
        x86_64)
            cur_bit=64
            ;;
        i386)
            cur_bit=32
            ;;
    esac

    if [ "$LINUX_VERSION" = "ubuntu" ];then
        DISTRIB_RELEASE=$(awk -F '=' '{if($1 == "DISTRIB_RELEASE"){print $2;exit}}' /etc/lsb-release)
        need_runlevel=2

        case "$DISTRIB_RELEASE" in
            9.04)
                knl_version="2.6.28-11-server"
                ;;
            10.04|11.04)
                knl_version="2.6.38-15-server"
                ;;
            12.04)
                knl_version="3.2.0-72-generic"
                knl_version1='3.2.0-23-generic'
                ;;
            *)
                OUTPUT_YELLOW "Ubuntu 发行版:$DISTRIB_RELEASE"
                ;;
        esac

        if [ "x${knl_version1}" != "x" ];then
            [ "x${knl_version}" != "x${cur_knl_version}" ] && \
            [ "x${knl_version1}" != "x${cur_knl_version}" ] && \
            compare_string "${cur_knl_version}" "${knl_version}" "158" "发行版$DISTRIB_RELEASE"
        else
            compare_string "${cur_knl_version}" "${knl_version}" "158" "发行版$DISTRIB_RELEASE"
        fi

    elif [ "$LINUX_VERSION" = "centos" ];then
        DISTRIB_RELEASE=$(awk '{for(i=1;i<=NF;i++){if($i == "release"){print $(i+1);exit}}}' /etc/redhat-release)
        need_runlevel=3

    fi

    [ "$cur_runlevel" != "$need_runlevel" ] && \
        { compare_string "$cur_runlevel" "$need_runlevel" "158" "runlevel不正确";exit_status=1; }

    if [ "x${OSINFO}" != 'x' ];then
        local cur_os_info=$(get_file_info "OS_info" $hardware_file)
        compare_string "${cur_os_info%%-*}" "${OSINFO%%/*}" "158" "系统发行版不对"

        cur_os_v="${cur_os_info#*-}";cur_os_v="${cur_os_v%%(*}"
        need_os_v="${OSINFO#*/}";need_os_v="${need_os_v%/*}"
        compare_string "${cur_os_v}" "${need_os_v}" "158" "系统版本不对"
    fi

    return "${exit_status:-0}"
}

function cpu_is_ok () {

    # C-state check
    local state_dir="/sys/devices/system/cpu/cpu0/cpuidle"
    max_cstate=1
    max_cstate_name=""
    if [ -d $state_dir ];then
        for states in $(ls -d $state_dir/state*);
        do
            states="${states##*/}"
            states=$(echo $states|tr -d a-z)
            [ $states -eq 0 -o $states -eq 1 ] && continue

            if [ $states -gt $max_cstate ];then
                read max_cstate_name < $state_dir/state${states}/name
            fi
        done

        #[ "x${max_cstate_name}" != "x" ] && \
        #    OUTPUT_YELLOW "服务器BIOS没有关闭C-state,当前:${max_cstate_name}"
    fi

    # numa and turbo check
    phy_cpu_count=$(get_file_info "Physical_CPU" $hardware_file)
    node_num=$(ls -d /sys/devices/system/node/node*|wc -l)
    compare_string "$node_num" "$phy_cpu_count" "158" "BIOS没有打开numa"

    scale_frequency_file="/sys/devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies"
    [ -f "$scale_frequency_file" ] && \
    scale_frequency=$(awk '{print NF}' $scale_frequency_file)
    #if [ "x${scale_frequency}" = "x" ] || [ $scale_frequency -le 1 ];then
    #    OUTPUT_YELLOW "不支持turbo"
    #    return_status=1
    #fi

    return "${return_status:-0}"
}

function display_info () {
    local CHECK_DISK DISK_NUM DISK_TYPE
    local CHECK_NICS CHECK_NIC_SPEED NIC_TYPE
    local dis_product=$(get_file_info "Product-name" $hardware_file)
    local dis_serial=$(get_file_info "serial-number" $hardware_file)
    local dis_cpu=$(get_file_info "CPU" $hardware_file| \
            awk '{for(i=1;i<=NF;i++){if($i == "CPU"){print $(i+1);exit}}}')
    local dis_phy_cpu=$(get_file_info "Physical_CPU" $hardware_file)
    local dis_processor=$(get_file_info "Processor" $hardware_file)
    local dis_phy_mem=$(get_file_info "Physical_memory" $hardware_file)
    local dis_sys_mem=$(get_file_info "System_memory" $hardware_file)
    local dis_os_info=$(get_file_info "OS_info" $hardware_file)
    local yyos=$(head -n1 /etc/yos_version 2>/dev/null)
    local dis_ip=$(get_file_info "IP_Address" $hardware_file)

    head $log_file|grep -q '清除全部硬盘' && \
    local install_type="不保留" || \
    local install_type="保留"

    for CHECK_DISK in $(grep 'physicaldrive' $hardware_file|awk '{print $5}'|uniq)
    do
        CHECK_DISK="${CHECK_DISK%%,}"
        DISK_NUM=$(grep -c "physicaldrive.*${CHECK_DISK}" $hardware_file)
        DISK_TYPE="${DISK_TYPE} ${CHECK_DISK}*${DISK_NUM}"
    done

    for CHECK_NICS in $(ls -d /sys/class/net/*);
    do
        CHECK_NIC="${CHECK_NICS##*/}"
        echo $CHECK_NIC | grep -q 'eth' || continue
        CHECK_NIC_SPEED=$(ethtool ${CHECK_NIC} | \
            awk -F ': ' '{if($1 ~ /Speed/){print $2}}'|awk '{print $1}')
        NIC_TYPE="${NIC_TYPE} ${CHECK_NIC}:${CHECK_NIC_SPEED}"
    done

    echo "$dis_product| $OMSCA_NAME |$dis_serial |${dis_cpu}*${dis_phy_cpu}*${dis_processor}|$dis_phy_mem| $dis_sys_mem |$DISK_TYPE|$dis_os_info $yyos $install_type|$NIC_TYPE"|tr -d '\t'
    echo "$dis_ip|$MOUNT_DATA"
}

function partition_mount_is_ok () {
    local error_par

    grep -wE "[vsh]d[a-z]{1,2}[1-9]{1,2}" /proc/partitions| \
        awk '{print $NF}' > $TMPFILE
    for check_par in $(grep -wE "[vsh]d[a-z]{1,2}" /proc/partitions| \
        awk '{print $NF}');
    do
        grep -q "$check_par" $TMPFILE || error_par="${error_par} $check_par"
    done
    if [ "x${error_par}" != "x" ];then
        OUTPUT_RED "${error_par}没有分区"
    fi

    unset error_par
    blkid -o full > $TMPFILE
    for ddev in $(awk -F ':' '{print $1}' $TMPFILE);
    do
        ddev_par="${ddev##*/}"
        ddev_r=$(awk '{if($4 == "'$ddev_par'" && $3 <= 10240){print "yes"}}' /proc/partitions)
        [ "x$ddev_r" == 'xyes' ] && sed -i "/\/dev\/$ddev_par/d" $TMPFILE
    done

    for check_par in $(grep -wE "[vsh]d[a-z]{1,2}[1-9]{1,2}" /proc/partitions | awk '{if($3 > 1000000){print $NF}}');
    do
        grep -q "/${check_par}:" $TMPFILE || \
            error_par="${error_par} $check_par"
    done
    if [ "x${error_par}" != "x" ];then
        OUTPUT_RED "${error_par}没有格式化"
    fi

    unset error_par
    for check_par in $(grep -vE 'swap' $TMPFILE|cut -d ':' -f1);do
        if ! grep -qw "$check_par" /proc/mounts;then
            check_uuid=$(grep "$check_par" $TMPFILE | \
                awk '{for(i=1;i<=NF;i++){if($i ~ /UUID/){split($i,a,/=/);print a[2]}}}'|tr -d [\"])
            grep -iq "$check_uuid" /proc/mounts || \
            error_par="${error_par} $check_par"
        fi
    done
    if [ "x${error_par}" != "x" ];then
        OUTPUT_RED "${error_par}分区没有挂载"
    fi

    unset error_par
    for check_par in $(cat $TMPFILE);do
        echo $check_par| grep -qi 'UUID=' || continue

        check_par=$(echo $check_par|tr -d '\"'|awk -F '=' '{print $2}')
        if ! grep -iq "${check_par}" /etc/fstab;then
            OUTPUT_YELLOW "存在分区不是用UUID挂载"
            break
        else
            check_path=$(grep -i "${check_par}" /etc/fstab| awk '{print $2}')
            [ "x${check_path}" == "x" ] || [ "x${check_path}" == "xswap" ] && continue
            touch $check_path/.acceptance_check.ck 2>/dev/null && \
            rm -f $check_path/.acceptance_check.ck || \
            OUTPUT_RED "${check_path}分区只读"
        fi
    done

    error_par=$(mount | awk '{print $3}'| sort|uniq -c| \
        awk '{if($1 > 1){print $2}}'| tr -s '\n' ' ')
    if [ "x${error_par}" != "x" ];then
        OUTPUT_RED "存在相同分区挂载点${error_par}"
    fi
}

function power_elec_num () {
    local manufacturer=$(get_file_info "Manufacturer" $hardware_file| \
            tr [A-Z] [a-z])
    should_elec_num=$(get_vari_info "elec_num" gblserver_model)
    for manu in "dell" "hp" "ibm" "huawei" "lenovo";
    do
        if echo $manufacturer| grep -iq "$manu" ;then
            manufacturer=$manu
            break;
        fi
    done

    which ipmitool > /dev/null 2>&1
    ipmitool sdr list > $TMPFILE

    if [ -s $TMPFILE ];then
        case "$manufacturer" in
            dell)
                elec_num=$(grep -iw "Voltage" $TMPFILE|grep -civ "disable")
                ;;
            hp)
                elec_num=$(grep -i "power supply" $TMPFILE|awk '{for(i=1;i<=NF;i++){if(tolower($i) ~ /watts/){if($(i-1)>0)print $(i-1)}}}'|wc -l)
                ;;
            ibm)
                elec_num=$(grep -i "power supply" $TMPFILE|grep -c '0x01')
                ;;
            lenovo)
                elec_num=$(grep -i "psu.*status" $TMPFILE| grep -c '0x01')
                ;;
            huawei)
                elec_num=$(grep -i "power[0-9]" $TMPFILE|awk '{for(i=1;i<=NF;i++){if(tolower($i) ~ /watts/){if($(i-1)>0)print $(i-1)}}}'|wc -l)
        esac

        # [ "x${elec_num}" = "x" ] && OUTPUT_YELLOW "无法识别服务器电源数量"
    fi
    [ "x${elec_num}" != "x" ] && \
    compare_string "${elec_num}" "$should_elec_num" "160" "电源数量不对"

}

function disk_strip_size () {
    local count

    if lspci|grep -iqw "LSI";then
        count=$(megacli -ldinfo -lall -aall|grep "Strip Size"|grep -vc '64')
    fi
    if lspci | grep -qEw "Smart Array";then
        count=$(hpacucli ctrl all show config detail| grep 'Strip Size'|grep -vc '64')
    fi

    [ "x${count}" != "x" ] && [ "$count" -ge 1 ] && \
    OUTPUT_YELLOW "阵列卡逻辑盘strip size不等于64KB"
}

function configure_ip_is_ok () {

    #for ipport in "manager.repos.yy.duowan.com:63217" "183.57.81.221:63217" "10.20.176.3:63217" "121.14.36.27:63217" "10.20.81.107:63217" "122.13.201.157:63217";
    #do
        #sysop_wget_info "http://$ipport/pxe_inst/get_ip_info.php?ip=$IP_ADDR" "$TMPFILE"
        #[ $? -eq 0 ] && break;
    #done
    #grep -q "^isp:" $log_file || return 1

    get_vari_info 'line' "^isp" "gblserver_info" | \
    while read isp_type conf_ip conf_mask conf_gw;
        do
            conf_ip="${conf_ip##*:}"
            conf_mask="${conf_mask##*:}"
            conf_gw="${conf_gw##*:}"
            [ "$isp_type" = "isp:11" ] && continue

            if ! ifconfig | grep -wq "$conf_ip";then
                OUTPUT_RED "没有配置IP:$conf_ip"
            else
                ifconfig | grep -q "${conf_ip}.*${conf_mask}" || \
                OUTPUT_RED "IP:${conf_ip}掩码不对"
            fi
        done

    default_gw=$(get_vari_info 'line' '^isp:' "gblserver_info" | \
        cut -d ':' -f 2- |sort -n|head -n 1|awk -F ':' '{print $NF}')
    cur_def_gw=$(route -n | grep "^0.0.0.0"|awk '{print $2}')
    compare_string "$default_gw" "${cur_def_gw}" "162" "网关配置不对"

    local dis_ip=$(get_file_info "IP_Address" $hardware_file)
    for check_ip in $dis_ip;
    do
        printf "$gblserver_info" | grep -qw "$check_ip" || \
        OUTPUT_RED "服务器配置多余IP:$check_ip"
    done

    [ "x${GAME_DEPARTMENT}" != "xyes" ] && \
    local exec_dir="/usr/local/i386/*/initial_system/" || \
    local exec_dir=$hw_base_dir

    # Check platform ip
    isp_count=$(get_vari_info "line" "^isp:" "gblserver_info"|awk '{print $1}')
    cat /dev/null > $TMPFILE
    for which_isp in $isp_count
    do
        case $which_isp in
            isp:4)
                EXTERNAL_IP="yes"
                grep 'dianxin' $exec_dir/$reposfile 2>/dev/null|awk '{print $1}' >> $TMPFILE
                ;;
            isp:5)
                EXTERNAL_IP="yes"
                grep 'wangtong' $exec_dir/$reposfile 2>/dev/null|awk '{print $1}' >> $TMPFILE
                ;;
            isp:10)
                INTERNAL_IP="yes"
                grep 'internal' $exec_dir/$reposfile 2>/dev/null|awk '{print $1}' >> $TMPFILE
                ;;
            isp:11)
                ;;
            *)
                EXTERNAL_IP="yes"
                diff_isp="(跨ISP)"
                ;;
        esac
    done

    if [ ! -s $TMPFILE ];then
        [ "x${INTERNAL_IP}" = "xyes" ] && \
        grep 'internal' $exec_dir/$reposfile 2>/dev/null| awk '{print $1}' >> $TMPFILE

        [ "x${EXTERNAL_IP}" = "xyes" ] && \
        grep -v 'internal' $exec_dir/$reposfile 2>/dev/null| awk '{print $1}' >> $TMPFILE
    fi

    ping_fail_ip=$(fping -u -f $TMPFILE)
    ping_fail_status=$?
    if [ "${ping_fail_status}" != "0" ];then
        ping_fail_ip=$(echo $ping_fail_ip|tr -s '\n' ' ')
        OUTPUT_YELLOW "网络连通性${diff_isp},ping不可达ip:${ping_fail_ip}"
    fi
}

function sysop_sync_is_ok() {
    for tmpdir in $(ls -d $base_dir/{*_repos,web*}/auto 2>/dev/null);
    do
        REPOS=$(echo $tmpdir|cut -d '/' -f 5)
        if [ ! -d $public_dir ];then
            OUTPUT_YELLOW "等候同步$public_dir"
            #wait second = $loop * 2
            loop=15
            until [ $loop -eq 0 ];do
                sleep 2;
                ((loop-=1))
                echo -n ". "
                if [ -d $public_dir ];then
                    echo "正在同步"
                    sleep 10
                    break;
                fi
            done

            [ $loop -eq 0 ] && { OUTPUT_RED "管理机同步失败";exit 2; }
        else
            if [ ! -d $base_dir/$REPOS/initial_system ] || \
                [ ! -d $public_dir/initial_system/snmp ] || \
                [ ! -d $public_dir/initial_system/hardware ];then
                OUTPUT_YELLOW "管理机库文件同步不完整"
                exit 2
            fi
        fi
        break
    done
    if [ "x${REPOS}" = "x" ];then
        OUTPUT_YELLOW "管理机库文件同步失败"
        exit 2
    fi
}

function cpu_hw_check() {
    # CPU型号
    cur_cpu_name=$(get_file_info "CPU" $hardware_file| \
                    awk -F 'CPU|@' '{print $2}'|tr -d ' ')
    should_cpu_name=$(get_vari_info "cpu_model_name" "gblserver_model"|tr -d ' ')
    compare_string "${cur_cpu_name}" "$should_cpu_name" "159" "CPU型号不对"

    # CPU频率
    cur_cpu_frecuency=$(get_file_info "CPU" "$hardware_file"|awk '{print $NF}')
    should_cpu_frecuency=$(get_vari_info "cpu_frequency" "gblserver_model")
    cpu_frecuency_delta=$(echo "$cur_cpu_frecuency" "$should_cpu_frecuency"| \
        tr -d [GHZghz]| awk '{a=($1==$2)?0:1;print a;}')
    [ "$cpu_frecuency_delta" != "0" ] && \
    compare_string "$cur_cpu_frecuency" "$should_cpu_frecuency" \
                    "150" "cpu频率不符"

    cur_phy_cpu=$(get_file_info "Physical_CPU" "$hardware_file")
    cur_processor=$(get_file_info "Processor" "$hardware_file")
    should_CPU=$(get_vari_info "cpu_num" "gblserver_model")
    should_phy_cpu="${should_CPU%%~*}"
    should_porcessor="${should_CPU##*~}"
    compare_string "${cur_phy_cpu}" "${should_phy_cpu}" "153" "CPU数量不符"
    compare_string "${cur_processor}" "${should_porcessor}" "154" \
                    "逻辑核不符,是否开了超线程"
}

function memory_hw_check() {
    # Memory check
    cur_phy_mem=$(get_file_info "Physical_memory" $hardware_file)
    cur_mem_size="${cur_phy_mem%%B\**}"
    cur_mem_num="${cur_phy_mem##*\*}"
    should_mem_size=$(get_vari_info "mem_size" "gblserver_model")
    should_mem_num=$(get_vari_info "mem_num" "gblserver_model")
    compare_string "$cur_mem_size" "$should_mem_size" "151" "内存容量规格不符"
    compare_string "$cur_mem_num" "$should_mem_num" "152" "内存数量不符"

    # 内存频率检查，临时hardcode
    mem_freq=$(dmidecode | grep 'Clock Speed'| grep -v 'Unknown'| \
    awk '{for(i=1;i<=NF;i++){if($i ~ /MHz/ || $i ~ /mhz/){print $(i-1)}}}'|sort -u)
    mem_freq_count=$(unset i;for f in $mem_freq;do (( i+=1 ));done;echo $i)
    [ "x${mem_freq_count}" != "x" ] && [ $mem_freq_count -gt 1 ] && \
    compare_string "$mem_freq_count" "1" "152" "内存不同频率混插"

    for mfreq in $mem_freq;
    do
        if [ "${OMSCA_NAME}" = "C2_3" ];then
            [ $mfreq -lt 1600 ] && \
            OUTPUT_RED "${OMSCA_NAME}服务器内存频率低于1600 MHZ"
        else
            [ $mfreq -lt 1333 ] && \
            OUTPUT_RED "${OMSCA_NAME}服务器内存频率低于1333 MHZ"
        fi
    done
}

function base_board_check() {
    cur_product_name=$(get_file_info "Product-name" "$hardware_file")
    compare_like_str "$cur_product_name" "$should_product_name" \
                    "159" "服务器型号不符"

    cur_sn=$(get_file_info "serial-number" "$hardware_file")
    compare_string "${cur_sn}" "${should_sn}" "149" "服务器序列号不符"
}

function ipmi_hw_check() {
    IPMI_IP=$(get_vari_info "line" "isp:11" 'gblserver_info')
    [ "x${IPMI_IP}" = "x" ] && return 1
    IPMI_IP=$(echo $IPMI_IP|awk '{print $2}'|awk -F ':' '{print $2}')
    IPMI_MASK="255.255.255.0"
    IPMI_GW="${IPMI_IP%.*}.1"

    ls /dev/ipmi* >/dev/null 2>&1
    [ $? -ne 0 ] && OUTPUT_RED "服务器没有带外管理卡"

    local SYSTEM_MANUFACTURER=$(get_file_info "Manufacturer" $hardware_file |\
             tr '[a-z]' '[A-Z]')
    case $SYSTEM_MANUFACTURER in
        "HP")
            BMC_CHANNEL=2;;
        "IBM")
            BMC_CHANNEL=1;;
        "POWERLEADER")
            BMC_CHANNEL=1;;
        "INTEL")
            BMC_CHANNEL=1;;
        "LENOVO")
            BMC_CHANNEL=1;;
        DELL*)
            BMC_CHANNEL=1
            USER="root";;
        INSPUR*)
            BMC_CHANNEL=8;;
        *)
            BMC_CHANNEL=1;;
    esac

    export gblserver_ipmi=$(ipmitool lan print $BMC_CHANNEL)
    cur_ipmi_ip=$(get_vari_info "IP Address" "gblserver_ipmi")
    cur_ipmi_mask=$(get_vari_info "Subnet Mask" "gblserver_ipmi")
    cur_ipmi_gw=$(get_vari_info "Default Gateway IP" "gblserver_ipmi")

    compare_string "$cur_ipmi_ip" "${IPMI_IP}" "158" "管理网IP不对"
    compare_string "$cur_ipmi_mask" "${IPMI_MASK}" "158" "管理网掩码不对"
    compare_string "$cur_ipmi_gw" "${IPMI_GW}" "158" "管理网网关不对"

}

function nic_hw_check() {
    get_vari_info "line" "net-" "gblserver_model"|cut -d ':' -f 2- | \
    while read nic_name nic_num nic_model_iface;
    do
        nic_model_iface="${nic_model_iface##*=}"
        nic_model="${nic_model_iface%%~*}"
        nic_iface="${nic_model_iface##*~}"
        should_nic_num="${nic_num##*=}"

        cur_nic_num=$(grep -ciw "$nic_model" $hardware_file)

        if [ "x${nic_model_iface}" = "x" ] || \
            [ "x${nic_model}" = "x" ] || \
            [ "x${nic_iface}" = "x" ];then
            OUTPUT_YELLOW "CMDB没有网卡采购信息,检查！！！"
            break;
        fi
        [ "$cur_nic_num" -lt "$should_nic_num" ] && \
        compare_string "${cur_nic_num}" "${should_nic_num}" \
                        "155" "网卡型号:${nic_model}数量不符"

        for check_mac in $(grep -iw "$nic_model" $hardware_file| \
            awk -F '&' '{print $NF}');
        do
            if grep -qi "$check_mac" /proc/net/bonding/bond* > /dev/null 2>&1 ;then
                nic_alias=$(grep -Ew "Interface:|HW addr:" /proc/net/bonding/bond*| grep -B1 -iw "$check_mac" | \
                awk -F ':' '{if($0 ~ /Interface:/){print $2;exit}}'| \
                sed 's/ *//g')
            else
                nic_alias=$(ifconfig -a|grep -iw "$check_mac"| \
                    cut -d ' ' -f 1|sed '/:/d'|head -n1)
            fi

            # 检查网卡速率
            cur_nic_port_speed=$(ethtool $nic_alias|\
                awk -F ':' '{if($0 ~ /Speed:/){print $2}}'|\
                tr -d '[[:alpha:][:space:]/\!]')
            if [ "x${cur_nic_port_speed}" != "x" ] && \
                [ $cur_nic_port_speed -le 100 ];then
                OUTPUT_RED "网口:${nic_alias}速率低于1000Mb/s"
            fi

            # 检查网卡传输介质
            cur_nic_iface=$(ethtool ${nic_alias} |\
                awk -F ':' '{if($0 ~ /Port:/){print $2}}'|\
                sed "s/^ *//g")
            compare_string "$cur_nic_iface" "$nic_iface" \
                            "158" "网口:${nic_alias}传输介质不符"

            eth_driver=$(ethtool -i $nic_alias | \
                awk '{if($1 ~ /driver/){print $NF}}')
            bus=$(ethtool -i $nic_alias|grep bus-info| \
                    awk -F ":" '{print $3":"$4}')
            msi=$(lspci -vvv -s $bus|grep -i "MSI-X"| \
                    awk -F " " '{print $5}'|awk -F "=" '{print $2}')
            [ "${eth_driver}" = "tg3" ] && continue
            if lspci -vvv -s $bus |grep -i "IOV" >/dev/null; then
                vfs=$(lspci -vvv -s $bus|grep VFs|awk '{print $6}'|tr -d [,])
                sriov="YES, VFs:$vfs"
            else
                sriov="NO"
            fi

            if [ "$sriov" = "NO" ] && [ $vfs -lt 8 ];then
                OUTPUT_RED "网卡$nic_alias, $bus, MSI-X:$msi, SRIOV:$sriov"
            fi
        done

        nic_multiqueue_num=$(ls -d /proc/irq/*/eth[0-9]-* 2>/dev/null|\
                        sed -n 's/.*\(eth[0-9]\).*/\1/p'|sort -n| \
                        uniq -c|head -n1|awk '{print $1}')
        [ "x${nic_multiqueue_num}" != "x" ] && \
        [ $nic_multiqueue_num -lt 4 ] && \
        OUTPUT_RED "网卡没有开启多队列, 当前队列数:$nic_multiqueue_num"
    done
}

function accerator_hw_check() {
    sed -n -e "/Array controller/,/Disk_info/p" -e "/accelerator/p" $hardware_file > $TMPFILE
    printf "$gblserver_model"|grep "^raid-"|cut -d ':' -f 2- | \
    while read cache_size accerator_model;
    do
        cache_size="${cache_size##*=}"
        should_cache_size=$(echo "$cache_size" | awk '{
            if($0 ~ /G|g/){
                split($0,a,/G|g/);
                printf("%dGB %dMB",a[1],a[1]*1024)
            }else{
                split($0,a,/M|m/);
                printf("%dGB %dMB",a[1]/1024,a[1])
            }}')

        accerator_model="${accerator_model##*=}"
        if grep -qw "$accerator_model" $TMPFILE;then
            if [ "x${cache_size}" != "x" ];then
                for scs in $should_cache_size
                do
                    grep -qiw "${scs}" $TMPFILE && scs_found=yes && break
                done
                [ "x${scs_found}" != "xyes" ] && OUTPUT_RED "阵列卡:$accerator_model cache size不符"
            fi
        else
            if grep -iq "accelerator.*none" $TMPFILE && lspci | \
                grep -iE 'scsi|raid'|grep -q "$accerator_model";then
                :
            else
                OUTPUT_RED "阵列卡:$accerator_model 型号不符"
            fi
        fi
    done
}

function disk_hw_check() {
    # Disk check
    # RAID10 4 ( SAS 600GB OK IBM-ESXSST600MM0006 B56KS0M1Q2371118B5C3 )
    # RAID1, 2, ( SAS, 300GB, OK, SEAGATE, ST300MM0006-LS08S0K23GEP )
    # physicaldrive none ( SSD 300GB OK BTWL347203LX300PGN SSDSC2BB300G4 sdb )
    grep -E 'logicaldrive|physicaldrive' "$hardware_file"|\
    awk '{
        if($1 == "logicaldrive"){i=1;a=$5;b=$7};
        if($1 == "physicaldrive" && i == 1 && $2 != "none"){
            i=0;$1=a;$2=b;
            if($5 ~ /TB|T/){split($5,f,/T/);$5=1000*f[1]"GB"};
            print $0;
        };
        if($1 == "physicaldrive" && $2 == "none"){print toupper($0)};
    }' | tr -d [+,] > $TMPFILE

    # 1 SAS 2 300G RAID1
    # 4 SATA 2 4T RAID1
    # 4 SSD 1 480G RAID0
    get_vari_info "line" 'disk-' "gblserver_model"|cut -d ':' -f 2- | \
    awk '{
        for(i=1;i<=NF;i++){
            if($i ~ /=/){split($i,a,/=|~/);$i=a[2];}
            if(i == 3 && $i ~ /T/){split($3,f,/T/);$3=1000*f[1]"GB"};
        };
        print toupper($0);
    }' | sort | uniq -c | \
    while read raid_disk_type_num disk_iface disk_num volume_type raid_type;
    do
        if echo "$raid_type"|grep -iq "no";then
            cur_hba_disk=$(grep -ic "physicaldrive none.*${disk_iface}.*${volume_type}" $TMPFILE)
            compare_string "$cur_hba_disk" "$disk_num" "158" "硬盘:${volume_type}数量不对"
        else
            [ "${raid_type}" == "HS" ] && raid_type="Hotspare"
            cur_raid_type_num=$(grep -ic "${raid_type}.*${disk_num}.*${disk_iface}.*${volume_type}"  $TMPFILE)
            compare_string "$cur_raid_type_num" "$raid_disk_type_num" \
            "157" "Raid阵列数量不符(${raid_type} $disk_iface ${volume_type}*${disk_num})"
        fi
    done

    for disk_model in $(get_vari_info 'line' 'disk-' "gblserver_model" | \
        awk '{split($2,a,/~/);print a[2]}'|sort -u|uniq);
    do
        should_disk_model_num=$(get_vari_info "line" "$disk_model" \
            "gblserver_model" | \
            awk '{for(i=1;i<=NF;i++){if($i ~ /num=/){split($i,a,/=/);f=f+a[2]}}}END{print f}')

        cur_disk_model_num=$(grep -ci "$disk_model" $hardware_file)
        check_model=$(get_vari_info "line" "$disk_model" "gblserver_model"| \
            head -n1 | awk '{for(i=1;i<=NF;i++){if($i ~ /=/){split($i,a,/=|~/);$i=a[2];}};printf("%s(%s)",$4,$2);}')
        compare_string "$cur_disk_model_num" "$should_disk_model_num" \
            "158" "硬盘${check_model}型号:${disk_model}不符"
    done

    # 检查逻辑盘是否跟原始容量相差过大
    grep -E 'logicaldrive|physicaldrive' "$hardware_file"|\
    awk '{
        if($1 == "logicaldrive"){i=1;a=$5;b=$7;c=$4;
            if(c ~ /TB|T/){split(c,g,/T/);c=1000*g[1];};
        };
        if($1 == "physicaldrive" && i == 1 && $2 != "none"){
            i=0;$1=a;$2=b;
            if($5 ~ /TB|T/){split($5,f,/T/);$5=1000*f[1];};
            printf("%s %s %s %s\n",toupper(a),int(c),int(b),int($5));
        };
    }' | tr -d [+,] | \
    while read raidtype raidsize num disk_one;
    do
        case "$raidtype" in
            "RAID0")
                rawsize=$(($num * $disk_one));;
            "RAID1")
                rawsize="$disk_one";;
            "RAID10")
                rawsize="$((($num * $disk_one) / 2))";;
            "RAID5")
                rawsize="$((($num - 1) * $disk_one))";;
            "HOTSPARE")
                continue;;
        esac
        deltasize=$(($rawsize*2/10))
        [ "$(($rawsize - $raidsize))" -ge $deltasize ] && \
        OUTPUT_RED "阵列原始容量不符$raidtype(${num}*${disk_one}GB), 当前:${raidsize}GB ===> 应该为:${rawsize}GB"
    done
}

function hw_accept() {
    [ "x${new_machine}" != "xyes" ] && return 1
    # 获取硬件采购配置信息
    for ipport in "manager.repos.yy.duowan.com:63217" \
        "183.57.81.221:63217" "121.14.36.27:63217" \
        "10.20.81.107:63217" "10.20.176.3:63217" \
        "122.13.201.157:63217";
    do
        sysop_wget_info "http://$ipport/external/query_server_model.php?type=text&model=${OMSCA_NAME}&model_id=${MODEL_ID}" "$TMPFILE"
        if [ $? -eq 0 ];then
            cat $TMPFILE >> $log_file
            gblserver_model=$(cat $TMPFILE)
            echo "$RUN_TIME [success] download server model:$OMSCA_NAME $MODEL_ID from $ipport" >> $log_file
            break;
        fi
    done

    cpu_hw_check
    memory_hw_check
    base_board_check
    nic_hw_check
    ipmi_hw_check
    accerator_hw_check
    disk_strip_size
    disk_hw_check
}

function base_os_accept() {
    configure_ip_is_ok
    kernel_is_acceptable
    cpu_is_ok
    swap_is_ok
    data_partition_is_ok
    partition_mount_is_ok
    package_is_ok
}

function yy_initial_env() {
    [ "x${GAME_DEPARTMENT}" == "xyes" ] && return 1
    sysop_sync_is_ok
    data_directory_is_ok

}

function get_hw_info() {
    [ "x${GAME_DEPARTMENT}" != "xyes" ] && \
    local exec_dir="$public_dir/initial_system/hardware" || \
    local exec_dir=$hw_base_dir
    if [ -s $exec_dir/collect_hardware_info_to_cmdb.v2.sh ];then
        exec_file="$exec_dir/collect_hardware_info_to_cmdb.v2.sh"
    elif [ -s $public_dir/initial_system/hardware/collect_hardware_info_to_cmdb.v2.sh ];then
        exec_file="$public_dir/initial_system/hardware/collect_hardware_info_to_cmdb.v2.sh"
    elif [ -s $hw_base_dir/collect_hardware_info_to_cmdb.v2.sh ];then
        exec_file="$hw_base_dir/collect_hardware_info_to_cmdb.v2.sh"
    fi
    /bin/bash $exec_file query > /dev/null 2>&1
    wait

}

function get_host_info() {
    for ipport in "183.57.81.221:65327" "58.215.46.20:65327" \
                    "10.20.81.107:65327" "10.20.176.3:65327" \
                    "122.13.201.157:65327";
    do
        sysop_wget_info "http://$ipport/get_ip_info.php?ip=$IP_ADDR" "$TMPFILE"
        [ $? -eq 0 ] && cat $TMPFILE >> $log_file && \
        gblserver_info=$(cat $TMPFILE) && break;
    done

    # 新服务器上架从DCM获取得到的配置信息
    export OSINFO=$(get_vari_info "os_info" gblserver_info)
    export OMSCA_NAME=$(get_vari_info "omsca_name" gblserver_info)
    export MODEL_ID=$(get_vari_info "model_id" gblserver_info)
    export should_product_name=$(get_vari_info "model_name" gblserver_info)
    export should_sn=$(get_vari_info "serial_no" gblserver_info)

    # 旧服务器从CMDB获取得到的信息
    export OSID=$(get_vari_info "os_id" gblserver_info)

    # 珠海游戏事业部服务器
    get_vari_info "line" 'tech_admin|sysop_admin|buss' 'gblserver_info' | \
    grep -iEq '游戏事业部|zhaoyiqiang|liuyadan|huangjianlong|zhangkai1' && \
    export GAME_DEPARTMENT="yes"

    if [ "x${OSINFO}" = "x" ] && [ "x${OSID}" = "x" ];then
        OUTPUT_RED "获取CMDB信息失败，是否没录入资产或不是上架单机器"
        export CMDB_INFO="fail"
    fi

    [ "x${CMDB_INFO}" != "xfail" ] && [ "x${OSINFO}" != "x" ] && \
    export new_machine="yes"
}

function post_check() {
    [ "x${GAME_DEPARTMENT}" = "xyes" ] && GAME_MSG="珠海游戏"
    result_tag=$(cat $RESULT_FILE)
    if [ "${global_warning}" = "0" ] && [ "${result_tag}" = "0" ];then
        [ "x${new_machine}" = "xyes" ] && \
        OUTPUT_GREEN "${GAME_MSG}新机器上架通过配置验收！！" || \
        OUTPUT_GREEN "${GAME_MSG}机器重装通过验收！！"
    else
        global_warning="1"
    fi
    [ "x${new_machine}" = "xyes" ] && UPLOAD_WARN "$global_warning" $TMPFILE1
    grep -qw "$BASENAME" /etc/crontab && sed -i "/$BASENAME/d" /etc/crontab

}

case $COMMAND in
    view)
        get_hw_info
        cat /etc/hardware_info| grep -v 'Virtual machine or no server_id'
        data_partition_is_ok
        display_info
        ;;
    "-h|help")
        echo "/bin/bash $0 [view]"
        exit 1
        ;;
    *)
        get_host_info
        get_hw_info
        hw_accept
        base_os_accept
        yy_initial_env
        post_check
        display_info
        ;;
esac


