#!/bin/bash

export PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin
export LC_ALL=en_US.UTF-8
export LANG=en_US.UTF-8

grep -iqE "ubuntu|debian" /etc/issue > /dev/null 2>&1 && LINUX_VERSION=ubuntu
grep -iqE "centos|red" /etc/issue > /dev/null 2>&1 && LINUX_VERSION=centos

PUBLIC_DIR="/usr/local/i386/public_repos"
HARDWARE_INFO="/etc/hardware_info"
export ERROR_RESULT=""

IPADDR=$(/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}')
MOUNT_DATA=$(cat /etc/mtab|awk '{print $2}'|grep data |tr -s "\n" " ")

function exit_func () {
	exit 0
}

OUTPUT_RED(){
	echo -e "\033[31;40;1m$1\033[0m"
}

function check_system_initialization () {
	local error_msg="";

	for CHECK_DIR in '*/auto' 'public_repos/initial_system';
	do
		[ ! -d /usr/local/i386/${CHECK_DIR} ] && \
		error_msg="${error_msg} ${CHECK_DIR}"
	done

	if [ "$LINUX_VERSION" = 'ubuntu' ];then
		for CHECK_PACKAGE in 'ia32-libs' 'libssl0.9.8';
		do
			dpkg -l | grep -qE "^ii *${CHECK_PACKAGE}" || \
			error_msg="${error_msg} ${CHECK_PACKAGE}"
		done
	elif [ "$LINUX_VERSION" = 'centos'  ];then
		rpm -qa | grep -qE "libstdc.*.i686"  || \
		error_msg="libstdc++.i686"
	fi

	[ "x${error_msg}" != "x" ] && \
	OUTPUT_RED "找不到:$error_msg"	
}

check_system_initialization

hwCheck="${PUBLIC_DIR}/initial_system/hardware/collect_hardware_info_to_cmdb.v2.sh"
[ ! -s ${hwCheck} ] && exit 1;

eval "/bin/bash $hwCheck ns > /dev/null 2>&1"
wait;

[ ! -s ${HARDWARE_INFO} ] && OUTPUT_RED "找不到文件:${HARDWARE_INFO}"


Product=$(awk -F ':' '{if($1 ~ /Product-name/){print $2;exit}}' $HARDWARE_INFO \
			|tr -d '\t')
serial=$(awk -F ':' '{if($1 ~ /serial-number/){print $2;exit}}' $HARDWARE_INFO)
CPU=$(awk -F ':' '{if($1 ~ /^CPU/){print $2;exit}}' $HARDWARE_INFO| \
	awk '{for(i=1;i<=NF;i++){if($i == "CPU"){print $(i+1);exit}}}')
Physical_cpu=$(awk -F ':' '{if($1 ~ /^Physical_CPU/){print $2;exit}}' $HARDWARE_INFO \
				|sed 's/ *//g'|tr -d '\t')
Processor=$(awk -F ':' '{if($1 ~ /^Processor/){print $2;exit}}' $HARDWARE_INFO \
				|sed 's/ *//g')
Physical_memory=$(awk -F ':' '{if($1 ~ /^Physical_memory/){print $2;exit}}' $HARDWARE_INFO)
System_memory=$(awk -F ':' '{if($1 ~ /^System_memory/){print $2;exit}}' $HARDWARE_INFO \
				| sed 's/ *//g'|tr -d '\t')
OS_info=$(awk -F ':' '{if($1 ~ /^OS_info/){print $2;exit}}' $HARDWARE_INFO \
			| tr -d '\t')
IP_Address=$(awk -F ':' '{if($1 ~ /^IP_Address/){print $2;exit}}' $HARDWARE_INFO)

for CHECK_DISK in $(grep 'physicaldrive' $HARDWARE_INFO|awk '{print $5}'|uniq)
do
	CHECK_DISK="${CHECK_DISK%%,}"
	DISK_NUM=$(grep -c "physicaldrive.*${CHECK_DISK}" $HARDWARE_INFO)
	DISK_TYPE="${DISK_TYPE} ${CHECK_DISK}*${DISK_NUM}"
done

for CHECK_NICS in $(ls -d /sys/class/net/*);
do
	CHECK_NIC="${CHECK_NICS##*/}"
	echo $CHECK_NIC | grep -q 'eth' || continue
	CHECK_NIC_SPEED=$(ethtool ${CHECK_NIC} | awk -F ': ' '{if($1 ~ /Speed/){print $2}}'|awk '{print $1}')
	NIC_TYPE="${NIC_TYPE} ${CHECK_NIC}:${CHECK_NIC_SPEED}"
done

SWAP_ZONE=$(swapon -s|grep -cv "Filename")
[ "$SWAP_ZONE" -lt 1 ] && OUTPUT_RED "swap分区挂载失败"
IS_DATA_ZONE=$(df -l | awk '{if($NF == "/data" && $2 > 100000000){print "yes";exit}}')
[ "x${IS_DATA_ZONE}" != "xyes" ] && OUTPUT_RED "/data挂载有问题"

echo "$IPADDR|$Product|$serial |${CPU}*${Physical_cpu}*${Processor}|$Physical_memory| $System_memory |$DISK_TYPE|$OS_info|$NIC_TYPE"
echo "$IP_Address|$MOUNT_DATA"
echo "---------------------------------------------------------------"






