#!/bin/bash
#Author:huangweisong
#update:2013/10/29
#version:1.0
#install netatop/atop and set crontab task
set +H
export LANG=C
export LC_ALL=C
export PATH=/usr/bin:/bin:/sbin:/usr/sbin

if [ $# -ne 2 ];then
	echo "Usage:$BASH_SOURCE starttime stoptime"
	echo "aaRESULT=1"
	exit 1
fi

function error_exit() {
    echo "aaRESULT=1"
	exit 1
}

ipL=`/sbin/ifconfig eth0|grep 'inet addr'|awk -F'[: ]*' '{print $4}'`
cd $(dirname $(readlink -ef $BASH_SOURCE))
kversion=`uname -r|cut -d'-' -f1`
stime=`echo $1|sed 's/ *//g'`
dtime=`echo $2|sed 's/ *//g'`


#check time format
for time in "$stime" "$dtime"
do
   if ! echo "$time"|grep -qE '^[0-9]{12}$';then
	   echo "starttime/stoptime must a number(YYYYmdHMS),example `date +%Y%m%d%H%M`"
	   error_exit
   fi
done


kill -9 CheckFlowbyProc.sh &>/dev/null
tar zxvf atopinstall.tgz
Basedir=/data/atop
Logdir=/data/atop/log
mkdir -p $Logdir

function create_intranet_list() {
    local intranet_temp_list=$Logdir/intranet_temp_lst
    local intranet_host_ini=/home/dspeak/yyms/intranet_ip_list.ini
    local intranet_list=$Basedir/intranet.lst
    local md5_file_intranet_host_ini=$Logdir/intranet_host_ini.md5
    local md5_intranet_host_ini=`md5sum $intranet_host_ini|awk '{print $1}'`
    if [ ! -f $intranet_list ];then
          awk -F'.' '{print $1"."$2"."$3".0/24"}' $intranet_host_ini|sort -u|grep -vE '^10\.|^192\.168\.|^172.1[6-9]\.|^172.2[0-9]\.|^172.31\.|^$' >$intranet_temp_list
          if [ `cat $intranet_temp_list|wc -l` -ge 1 ];then
               cp -af $intranet_temp_list $intranet_list &>/dev/null
               echo $md5_intranet_host_ini >$md5_file_intranet_host_ini
          fi
          return
    elif [ $md5_intranet_host_ini != `head -1 $md5_file_intranet_host_ini` ];then
           awk -F'.' '{print $1"."$2"."$3".0/24"}' $intranet_host_ini|sort -u|grep -vE '^10\.|^192\.168\.|^172.1[6-9]\.|^172.2[0-9]\.|^172.31\.|^$' >$intranet_temp_list
           if [ `cat $intranet_temp_list|wc -l` -ge 1 ];then
               cp -af $intranet_list $intranet_list.old &>/dev/null
               cp -af $intranet_temp_list $intranet_list &>/dev/null
               echo $md5_intranet_host_ini >$md5_file_intranet_host_ini
           fi
    fi
}

#create intranet iplist
create_intranet_list

#install netatop kernel module
if [ -f /etc/init.d/netatop ];then
	/etc/init.d/netatop stop
fi	
aptitude -y -v install  zlib1g-dev libncurses5-dev
if [ ! -d /lib/modules/`uname -r`/extra ];then
 	 mkdir /lib/modules/`uname -r`/extra
fi
chown root:root netatop*
cp -af netatop.ko_$kversion /lib/modules/`uname -r`/extra/netatop.ko
chmod 0744 /lib/modules/`uname -r`/extra/netatop.ko
depmod
cp -af netatopd_$kversion /usr/sbin/netatopd
cp -af netatop.init /etc/init.d/netatop
chmod 0755 /usr/sbin/netatopd /etc/init.d/netatop
service netatop start || error_exit


#add crontab mask
chmod +x  CheckFlowbyProc.sh
cp -af CheckFlowbyProc.sh  /data/atop/
chmod +r netatop.conf
cp -af netatop.conf /data/atop/
rm -f /data/atop/atopmonitor.log
rm -f /data/atop/log/atop_result.log
sed -i '/check netflow by process/d' /etc/crontab
sed -i '/data\/atop\/CheckFlowbyProc.sh/d' /etc/crontab
echo '#check netflow by process' >>/etc/crontab
echo '* * * * * root /data/atop/CheckFlowbyProc.sh 20 >>/data/atop/atopmonitor.log 2>/dev/null &' >>/etc/crontab
rm -f  netatop.init netatop.ko_* netatopd_* CheckFlowbyProc.sh
if [ ! -f /data/atop/atop-2.0.2.tar.gz ] && [ -f atop-2.0.2.tar.gz ];then
    cp -af atop-2.0.2.tar.gz /data/atop/
fi	

#install atop package
cd /data/atop
if [ ! -f /usr/bin/atop ];then
	if [ ! -f atop-2.0.2.tar.gz ];then
		 which wget &>/dev/null || apt-get install  -y --force-yes wget || error_exit
	     wget -t 5 -w 1 -q http://www.atoptool.nl/download/atop-2.0.2.tar.gz -P /data/atop/
    fi		
    if [ -f atop-2.0.2.tar.gz ];then
         tar zxvf atop-2.0.2.tar.gz
         cd atop-2.0.2
		 which make &>/dev/null || apt-get install  -y --force-yes make || error_exit
		 which gcc &>/dev/null || apt-get install  -y --force-yes gcc || error_exit
		 dpkg -l libc6-dev &>/dev/null || apt-get install  -y --force-yes libc6-dev || error_exit
		 dpkg -l libncurses5-dev &>/dev/null || apt-get install  -y --force-yes libncurses5-dev || error_exit
		 dpkg -l zlib1g-dev &>/dev/null || apt-get install  -y --force-yes zlib1g-dev || error_exit
		 make && make install || error_exit
    fi
fi
	
if [ -e /etc/init.d/atop -o -e /etc/atop ];then
     update-rc.d atop disable &>/dev/null
     killall -9 atop &>/dev/null
     rm -rf /var/log/atop 
     rm -f /var/run/atop.pid /etc/init.d/atop 
     rm -rf /etc/atop 
fi

#add start/stop time config
echo "STARTTIME:$stime" >runtime.conf
echo "STOPTIME:$dtime" >>runtime.conf
echo "aaRESULT=0"
