#!/bin/bash

trap 'if [ -n "$dstFile" ] ; then 

         chattr -i $dstFile &>/dev/null 

      fi' EXIT TERM KILL HUP

set +H

export PATH="/usr/bin:/bin:/sbin:/usr/sbin"

export LANG=C

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T')| $line"

   done < <(echo "$*")

   return
}


function exit_on_ok {

    aaRESULT=0
    
    echo "aaRESULT=$aaRESULT"
    
    exit $aaRESULT
}


function exit_on_err {

   aaRESULT=1

   echo "aaRESULT=$aaRESULT" 

   exit $aaRESULT
}


if [ "$UID" -ne 0 ] ; then

   print_log "Err! only root user can run this script"

   exit_on_err 
fi

chmod a+x $0

print_log "$0 $*"

# set xml syntax check python code

code=$( 
cat <<EOF
#!/usr/bin/env python
from xml.dom.minidom import parse
import sys
if not sys.argv[1]:
        sys.exit(1)
try:
        parse(sys.argv[1])
except:
        sys.exit(1)
EOF
)

# paramater check

if [ $# -lt 1 ] || [ $# -gt 2 ] ; then
	
     print_log "Err! $0 <base64-encoded paras | ip-string> [/path/to/file]"

     exit_on_err
else
     # check whether file exist

     if [ $# -eq 1 ] ; then 

          dstFile=/home/dspeak/release/bin/allow.xml

     elif [ -z "$1" ] ; then

          print_log "Err! $0 <base64-encoded paras | ip-string> [/path/to/file]"

          exit_on_err
     else
          dstFile="$2"
     fi 

     print_log "whitelist file = $dstFile"

     if [ ! -f "$dstFile" ] ; then

          print_log "Err! white list file [$dstFile] not a file or not exist"

          exit_on_err 
     else
          # set dstFile to the full-path 

          dstFile="$(readlink -f $dstFile)"

          print_log "whitelist file full-path = $dstFile"

          tmpFile="${dstFile}.tmp"

          print_log "tmp file = $tmpFile"
     fi

     pattern='^[[:space:]]*([0-9]{1,3}\.){3}[0-9]{1,3}[[:space:]]*$'

     ipList=$(echo "$1" | egrep -v '^[[:space:]]*$' | sed -r "s/$pattern//g")

     if [ -n "$ipList" ] ; then

          # try decoded base64-encoded string

          ipList=$(base64 -d <(echo "$1")) || { print_log "Err! invalid base64 string [$1], decode failed";

                                                exit_on_err ; }
          if [ -z "$ipList" ] ; then

               print_log "Err! decode result is Empty !!!"

               exit_on_err
          else
               print_log "Ok! base64 decode succeed"
          fi
     else
          print_log

          print_log "Ok! input type is ip-string , no need to decode"

          # re-assign the ipList var to the input content

          ipList=$(echo "$1" | egrep -v '^[[:space:]]*$')
     fi

     # check ip list format if it contains invalid characters

     x=$(echo "$ipList" | sed -r "s/$pattern//g")

     if [ -n "$x" ] ; then

          print_log "ip list contain invalid characters : [$x]"

          print_log "it should be one IP per-line" 
     
          exit_on_err
     fi

     # check whether <allows> line exist

     symbolLine='<allows>'

     endSymbolLine='<\/allows>'

     if ! egrep -q "^[[:space:]]*${symbolLine}[[:space:]]*$" $dstFile ; then

          print_log "Err! $symbolLine line NOT found in $dstFile"

          exit_on_err

     elif [ $(egrep -c "^[[:space:]]*${symbolLine}[[:space:]]*$" $dstFile) -gt 1 ] ; then

          print_log "Err! more than 1 $symbolLine line found in $dstFile"

          exit_on_err
     fi

     # check whether is opened by vi

     swpFile="$(dirname $dstFile)/.$(basename $dstFile).swp"

     if [ -f "$swpFile" ] ; then

        print_log "Err! found swp file $swpFile , may be opened by vim"

        exit_on_err
     fi

     # change $dstFile to readonly

     if ! chattr +i $dstFile ; then

        print_log "Err! set file [$dstFile] read-only failed"

        exit_on_err
     fi           

     # create a tmp file

     cp $dstFile $tmpFile 

     # begin to work

     unset delStr

     delStr="/$symbolLine/,/$endSymbolLine/{"

     while read ip ; do

          # delete old line

          delStr="${delStr}${delStr:+ }/>${ip//./\\.}</d;"

     done <<< "$ipList"

     delStr="${delStr} }"

     print_log "delete old lines ...."

     sed -i -r -e "$delStr" $tmpFile

     # clear the <host></host> pair statments

     #cat $tmpFile | xargs -i -n 1 echo -n {}"#@#" | sed -r 's!<host>#@#[[:space:]]*</host>#@#[[:space:]]*!!g' | sed -r 's/#@#/\n/g' > ${tmpFile}.x

     #tr '\n' '&' < $tmpFile | sed 's!<host>&[[:space:]]*</host>&[[:space:]]*!!g' | sed -r 's/&/\n/g' > ${tmpFile}.x

     sed -r 's/$/\x00/' $tmpFile | tr -d '\n' | sed -r 's/[[:space:]]*<host>\x00[[:space:]]*<\/host>\x00//g' | sed -r 's/\x00/\n/g' > ${tmpFile}.x

     mv ${tmpFile}.x $tmpFile

     # check xml syntax

     if python <(echo "$code") ${tmpFile:?XML file must be gived} ; then

          print_log "Ok! $tmpFile XML syntax check passed"
     else
          print_log "Err! $tmpFile XML syntax check failed"

          chattr -i $dstFile
 
          exit_on_err
     fi

     # backup $dstFile and overwrite it with $tmpFile

     backupDir="$(dirname $dstFile)/bak"

     mkdir -p $backupDir

     bakFile="$backupDir/$(basename $dstFile).$(date +'%Y%m%d_%H%M')"
   
     if cp $dstFile $bakFile ; then

         print_log "Ok! $dstFile backup succeed"

         chattr -i $dstFile

         mv $tmpFile $dstFile

         # reload

         localIp=$(ifconfig eth0 |grep 'inet addr:' | head -n1 | awk -F '[ :]+' '{print $4}')

         print_log "local ip = $localIp"

         (sleep 1;echo "reload_allow";sleep 1) | telnet $localIp 2020 2>&1 | grep 'reloaded' 

         if [ $? -eq 0 ] ; then

              print_log "Ok! reload new conf succeed"

              diff -u $dstFile $bakFile

              print_log "Finished"

              exit_on_ok 
         else
              print_log "Err! reload new conf failed , begin to rollback"

              cp $dstFile ${dstFile}.bad
           
              mv $bakFile $dstFile

              (sleep 1;echo "reload_allow";sleep 1) | telnet $localIp 2020 

              print_log "rollback to old whitelist file succeed"

              print_log "Finished"

              exit_on_err 
         fi    
    else
         print_log "Err! $dstFile backup failed"

         chattr -i $dstFile 

         exit_on_errr 
    fi
fi
