#!/bin/bash

trap 'if [ -n "$dstFile" ] ; then 

         chattr -i $dstFile &>/dev/null 

      fi' EXIT TERM KILL HUP

set +H

export PATH="/usr/bin:/bin:/sbin:/usr/sbin"

export LANG=C

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T')| $line"

   done < <(echo "$*")

   return

}

if [ "$UID" -ne 0 ] ; then

     print_log "Err! only root user can run this script"

     aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT
fi

chmod a+x $0

print_log "$0 $*"

# set xml syntax check python code

code=$( 
cat <<EOF
#!/usr/bin/env python
from xml.dom.minidom import parse
import sys
if not sys.argv[1]:
        sys.exit(1)
try:
        parse(sys.argv[1])
except:
        sys.exit(1)
EOF
)

# paramater check

if [ $# -lt 1 ] || [ $# -gt 2 ] ; then
	
     print_log "Err! $0 <base64-encoded paras> [/path/to/file]"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
else
     # check whether file exist

     if [ $# -eq 1 ] ; then 

          dstFile=/home/dspeak/release/bin/server.xml
     else
          dstFile="$2"
     fi

     print_log "whitelist file = $dstFile"

     if [ ! -f "$dstFile" ] ; then

          print_log "Err! white list file [$dstFile] not a file or not exist"

          aaRESULT=1; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     else
          # set dstFile to the full-path 

          dstFile="$(readlink -f $dstFile)"

          print_log "whitelist file full-path = $dstFile"

          tmpFile="${dstFile}.tmp"

          print_log "tmp file = $tmpFile"
     fi

     # decoded base64-encoded string

     ipList=$(base64 -d <(echo "$1")) || { print_log "Err! invalid base64 string [$1], decode failed"; 

                                           aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT; }

     if [ -z "$ipList" ] ; then

        print_log "Err! decode result is Empty !!!"

        aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi

     # check ip list format if it contains invalid characters

     x=$(echo "$ipList" | sed -r 's/([0-9]{1,3}\.){3}[0-9]{1,3}//g')

     if [ -n "$x" ] ; then

        print_log "ip list contain invalid characters : [$x]"

        print_log "it should be one IP per-line"
     
        aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi
     
     # check whether is opened by vi

     swpFile="$(dirname $dstFile)/.$(basename $dstFile).swp"

     if [ -f "$swpFile" ] ; then

        print_log "Err! found swp file $swpFile , may be opened by vim" ; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT

     fi

     # change $dstFile to readonly

     if ! chattr +i $dstFile ; then

        print_log "Err! set file [$dstFile] read-only failed"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi           

     # create a backup file

     cp $dstFile $tmpFile 

     # begin to work

     while read ip ; do 

         print_log "ip=$ip"

         print_log $(egrep "<allow +[dwjyb]ip=\"${ip}\"" $tmpFile)

         # delete line

         sed -i -r "/<allow +.+ip=\"${ip}\"/d" $tmpFile && print_log "Ok! ${ip} delete succeed"

     done <<< "$ipList"

     # check xml syntax

     if python <(echo "$code") ${tmpFile:?XML file must be gived} ; then

          print_log "Ok! $tmpFile XML syntax check passed"
     else
          print_log "Err! $tmpFile XML syntax check failed"

          chattr -i $dstFile 

          aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi

     # backup $dstFile and overwrite it with $tmpFile

     backupDir="$(dirname $dstFile)/bak"

     mkdir -p $backupDir

     bakFile="$backupDir/$(basename $dstFile).$(date +'%Y%m%d_%H%M')"
   
     if cp $dstFile $bakFile ; then

         print_log "Ok! $dstFile backup succeed"

         chattr -i $dstFile

         mv $tmpFile $dstFile

         # reload

         (sleep 1;echo "reloadconf";sleep 3)|telnet 0 4501 2>/dev/null | grep 'reloadconf succeed'

         if [ $? -eq 0 ] ; then

              print_log "Ok! reload new conf succeed"

              print_log "Finished"

              aaRESULT=0 ; echo "aaRESULT=$aaRESULT"; exit $aaRESULT 
         else
              print_log "Err! reload new conf failed , begin to rollback"

              cp $dstFile ${dstFile}.bad
           
              mv $bakFile $dstFile

              (sleep 1;echo "reloadconf";sleep 1)|telnet 0 4501

              print_log "rollback to old whitelist file succeed"

              print_log "Finished"

              aaRESULT=1; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
         fi    
    else
         print_log "Err! $dstFile backup failed"

         chattr -i $dstFile 

         aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
    fi
fi
