#!/bin/bash

trap 'if [ -n "$dstFile" ] ; then 

         chattr -i $dstFile &>/dev/null 

      fi' EXIT TERM KILL HUP

set +H

export PATH="/usr/bin:/bin:/sbin:/usr/sbin"

export LANG=C

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T')| $line"

   done < <(echo "$*")

   return

}

if [ "$UID" -ne 0 ] ; then

     print_log "Err! only root user can run this script"

     aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
fi

chmod a+x $0

print_log "$0 $*"

# paramater check

if [ $# -lt 1 ] || [ $# -gt 2 ] ; then
	
	 print_log "Err! $0 <base64-encoded paras> [/path/to/file]"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
else
     # check whether file exist

     if [ $# -eq 1 ] ; then 

          dstFile="/usr/local/virus/iptables/iptables.allow.yycookie"
     else
          dstFile="$2"
     fi

     print_log "whitelist file = $dstFile"

     if [ ! -f "$dstFile" ] ; then

          print_log "Err! white list file [$dstFile] not a file or not exist"

          aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     else

          # set dstFile to the full-path 

          dstFile="$(readlink -f $dstFile)"

          print_log "whitelist file full-path = $dstFile"

          tmpFile="${dstFile}.tmp"

          print_log "tmp file = $tmpFile"
     fi

     # decoded base64-encoded string

     ipList=$(base64 -d <(echo "$1")) || { print_log "Err! invalid base64 string [$1], decode failed"; 

                                           aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT; }

     if [ -z "$ipList" ] ; then

        print_log "Err! decode result is Empty !!!"

        aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi

     # check ip list format if it contains invalid characters

     x=$(echo "$ipList" | sed -r 's/([0-9]{1,3}\.){3}[0-9]{1,3}//g')

     if [ -n "$x" ] ; then

          print_log "ip list contain invalid characters : [$x]"

          print_log "it should be one IP per line"
     
          aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi

     ipList=$(echo "$ipList" |sort -t '.' -k4,4n -u)

     # check whether is opened by vi

     swpFile="$(dirname $dstFile)/.$(basename $dstFile).swp"

     if [ -f "$swpFile" ] ; then

        print_log "Err! found swp file $swpFile , may be opened by vim" ; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT

     fi
     
     # change $dstFile to readonly

     if ! chattr +i $dstFile ; then

          print_log "Err! set file [$dstFile] read-only failed"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi           

     # create a backup file

     cp $dstFile $tmpFile 

     # begin to work

     for ip in $ipList ; do

          print_log "$ip"

          sed -i -r "/^[[:space:]]*${ip}[[:space:]]*$/d" $tmpFile
     done

     # backup $dstFile and overwrite it with $tmpFile
   
     bakFile="$dstFile.$(date +'%Y%m%d_%H%M')"

     if cp $dstFile $bakFile ; then

         print_log "Ok! $dstFile backup succeed"

         chattr -i $dstFile

         mv $tmpFile $dstFile

         # reload

         bash /usr/local/virus/iptables/iptables.sh

         if [ $? -eq 0 ] ; then

              print_log "Ok! reload new conf succeed"

              print_log "Finished"

              aaRESULT=0 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
         else
              print_log "Err! reload new conf failed , begin to rollback"

              cp $dstFile ${dstFile}.bad
           
              mv $bakFile $dstFile

              bash /usr/local/virus/iptables/iptables.sh

              print_log "rollback to old whitelist file succeed"

              print_log "Finished"

              aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
         fi    
    else
         print_log "Err! $dstFile backup failed"

         chattr -i $dstFile 

         aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
    fi
fi
