#!/bin/bash

trap 'if [ -n "$dstFile" ] ; then 

         chattr +i $dstFile &>/dev/null 

      fi' EXIT TERM KILL HUP

set +H

export PATH="/usr/bin:/bin:/sbin:/usr/sbin"

export LANG=C

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T')| $line"

   done < <(echo "$*")

   return

}

if [ "$UID" -ne 0 ] ; then

     print_log "Err! only root user can run this script"

     aaRESULT=1 
              
     echo "aaRESULT=$aaRESULT" 
     
     exit $aaRESULT 
fi

chmod a+x $0

print_log "$0 $*"

# set xml syntax check python code

code=$( 
cat <<EOF
#!/usr/bin/env python
from xml.dom.minidom import parse
import sys
if not sys.argv[1]:
        sys.exit(1)
try:
        parse(sys.argv[1])
except:
        sys.exit(1)
EOF
)

# paramater check

if [ $# -lt 1 ] || [ $# -gt 2 ] ; then
	
     print_log "Err! $0 <base64-encoded paras> [/path/to/file]"
              
     aaRESULT=1 
     
     echo "aaRESULT=$aaRESULT" 
     
     exit $aaRESULT 
else
     # check whether file exist

     if [ $# -eq 1 ] ; then 

          dstFile=/home/dspeak/release/bin/allow.xml

     elif [ -z "$1" ] ; then

          print_log "Err! $0 <base64-encoded paras> [/path/to/file]"

          aaRESULT=1

          echo "aaRESULT=$aaRESULT" 

          exit $aaRESULT
     else
          dstFile="$2"
     fi

     print_log "whitelist file = $dstFile"

     if [ ! -f "$dstFile" ] ; then

          print_log "Err! white list file [$dstFile] not a file or not exist"

          aaRESULT=1 
              
          echo "aaRESULT=$aaRESULT" 
          
          exit $aaRESULT 
     else
          # set dstFile to the full-path 

          dstFile="$(readlink -f $dstFile)"

          print_log "whitelist file full-path = $dstFile"

          tmpFile="${dstFile}.tmp"

          print_log "tmp file = $tmpFile"
     fi

     # decoded base64-encoded string

     pattern='^(([0-9]{1,3}\.){3}[0-9]{1,3}[[:space:]]+[[:alpha:]]+[[:space:]]*,?[[:space:]]*)+'

     ipList=$(echo "$1" | egrep -v '^[[:space:]]*$' | sed -r "s/$pattern//g") 

     if [ -n "$ipList" ] ; then

          # try base64 decode

          ipList=$(base64 -d <(echo "$1")) || { print_log "Err! invalid base64 string [$1], decode failed";

                                                aaRESULT=1 
              
                                                echo "aaRESULT=$aaRESULT" 
              
                                                exit $aaRESULT
              
                                               }

          if [ -z "$ipList" ] ; then

               print_log "Err! decode result is Empty !!!"

               aaRESULT=1 
              
               echo "aaRESULT=$aaRESULT" 
              
               exit $aaRESULT 
          else
               print_log "Ok! base64 decode succeed"
          fi
     else
          print_log

          print_log "Ok! input type is ip-isp string , no need to decode"

          # re-assign the ipList var to the input content

          ipList=$(echo "$1" | egrep -v '^[[:space:]]*$')
     fi

     # check ip list format if it contains invalid characters

     x=$(echo "$ipList" | sed -r "s/$pattern//g") 

     if [ -n "$x" ] ; then

          print_log "ip list contain invalid characters : [$x]"

          print_log "it should be \"<ip1> <isp1>[,<ip2> <isp2>, ...]\" format"
     
          aaRESULT=1 
              
          echo "aaRESULT=$aaRESULT" 
          
          exit $aaRESULT 
     fi
     
     # trasfer to UPPER case and split them

     ipList=$(echo "$ipList" | egrep -v '^[[:space:]]*$' | \

              tr '[[:lower:]]' '[[:upper:]]' | sed -r 's/([[:space:]]+)?,([[:space:]]+)?/,/' | \

              sort -t '.' -k1,1n -k2,2n -k3,3n -k4,4n) 

     # check whether duplicate ip exist

     x=$(echo "$ipList" | xargs -d ',' -n 1 | sort | awk '{print $1}' | uniq -c | awk '$1>1 {print $2}' | sort -u | xargs)

     if [ -n "$x" ] ; then

        print_log "Err! duplicate ip found [ $x ]"

        aaRESULT=1

        echo "aaRESULT=$aaRESULT" 

        exit $aaRESULT
     fi

     # check whether <allows> line exist

     symbolLine='<allows>'

     endSymbolLine='<\/allows>'

     if ! egrep -q "^[[:space:]]*${symbolLine}[[:space:]]*$" $dstFile ; then

          print_log "Err! $symbolLine line NOT found in $dstFile"

          aaRESULT=1 
              
          echo "aaRESULT=$aaRESULT" 
          
          exit $aaRESULT 

     elif [ $(egrep -c "^[[:space:]]*${symbolLine}[[:space:]]*$" $dstFile) -gt 1 ] ; then

          print_log "Err! more than 1 $symbolLine line found in $dstFile"

          aaRESULT=1 
              
          echo "aaRESULT=$aaRESULT" 
          
          exit $aaRESULT
     fi

     # check whether is opened by vi

     swpFile="$(dirname $dstFile)/.$(basename $dstFile).swp"

     if [ -f "$swpFile" ] ; then

        print_log "Err! found swp file $swpFile , may be opened by vim"

        aaRESULT=1

        echo "aaRESULT=$aaRESULT"

        exit $aaRESULT

     fi

     # change $dstFile to readonly

     if ! chattr +i $dstFile ; then

        print_log "Err! set file [$dstFile] read-only failed"

        aaRESULT=1

        echo "aaRESULT=$aaRESULT"

        exit $aaRESULT 
     fi           

     # create a tmp file

     cp $dstFile $tmpFile 

     # begin to work

     unset insertStr

     unset delStr

     delStr="/$symbolLine/,/$endSymbolLine/{"

     while read line ; do

           oldIFS="$IFS"

           IFS=','

           set -- $line

           IFS="$oldIFS"

           insertStr="${insertStr}${insertStr:+\n}        <host>"

           for ((i=1;i<=$#;i++)) ; do

                 ip=$(echo "${!i}" | awk -F ' ' '{print $1}')

                 isp=$(echo "${!i}" | awk -F ' ' '{print $2}')

                 print_log "ip=$ip , isp=$isp"

                 # generate the del command and insert command string

                 delStr="${delStr}${delStr:+ }/>${ip//./\\.}</d;"

                 insertStr="${insertStr}\n            <ip type=\"$isp\">${ip}</ip>"
           done

           insertStr="${insertStr}\n        </host>"

     done <<< "$ipList"

     delStr="${delStr} }"

     print_log "delete old lines ...."
 
     sed -i -r -e "$delStr" $tmpFile

     print_log "insert all new lines" 

     echo -e "$insertStr" | sed "s/^/$(date +'%F %T')| /"

     insertCmd="/$symbolLine/a \\${insertStr}"        

     sed -i -r -e "$insertCmd" $tmpFile

     # clear the <host></host> pair statments

     #cat $tmpFile | xargs -i -n 1 echo -n {}"#@#" | sed -r 's!<host>#@#[[:space:]]*</host>#@#[[:space:]]*!!g' | sed -r 's/#@#/\n/g' > ${tmpFile}.x

     #tr '\n' '&' < $tmpFile | sed 's!<host>&[[:space:]]*</host>&[[:space:]]*!!g' | sed -r 's/&/\n/g' > ${tmpFile}.x 

     sed -r 's/$/\x00/' $tmpFile | tr -d '\n' | sed -r 's/[[:space:]]*<host>\x00[[:space:]]*<\/host>\x00//g' | sed -r 's/\x00/\n/g' > ${tmpFile}.x

     mv ${tmpFile}.x $tmpFile

     # check xml syntax

     if python <(echo "$code") ${tmpFile:?XML file must be gived} ; then

          print_log "Ok! XML syntax check passed"
     else
          print_log "Err! XML syntax check failed"

          chattr -i $dstFile

          aaRESULT=1

          echo "aaRESULT=$aaRESULT"

          exit $aaRESULT 
     fi

     # backup $dstFile and overwrite it with $tmpFile

     backupDir="$(dirname $dstFile)/bak"

     mkdir -p $backupDir
   
     bakFile="$backupDir/$(basename $dstFile).$(date +'%Y%m%d_%H%M')"

     if cp $dstFile $bakFile ; then

         print_log "Ok! $dstFile backup succeed"

         chattr -i $dstFile

         mv $tmpFile $dstFile

         # reload

         localIp=$(ifconfig eth0 |grep 'inet addr:' | head -n1 | awk -F '[ :]+' '{print $4}')

         print_log "local ip = $localIp"

         (sleep 1;echo "reload_allow";sleep 1) | telnet $localIp 2020 2>&1 | grep 'reloaded' 

         if [ $? -eq 0 ] ; then

              print_log "Ok! reload new conf succeed"

              #diff -u $dstFile $bakFile

              print_log "Finished"

              aaRESULT=0 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
         else
              print_log "Err! reload new conf failed , begin to rollback"

              cp $dstFile ${dstFile}.bad
           
              mv $bakFile $dstFile

              (sleep 1;echo "reload_allow";sleep 1) | telnet $localIp 2020 

              print_log "rollback to old whitelist file succeed"

              print_log "Finished"

              aaRESULT=1 
              
              echo "aaRESULT=$aaRESULT" 
              
              exit $aaRESULT 
         fi    
    else
         print_log "Err! $dstFile backup failed"

         chattr -i $dstFile 

         aaRESULT=1 
              
         echo "aaRESULT=$aaRESULT" 
         
         exit $aaRESULT 
    fi
fi
