#!/bin/bash

# FILENAME     : gen_rsync.bash
# DESCRIPTION  : generate rsync configuration for
# AUTHOR       : Juliam
# CREATED DATE : 2013-12-20
# LAST MODIFIED: 2013-12-20

export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

if [ "$(whoami)" != "root" ]
then
	echo "----- MUST be run by root! ------"
	exit 1
fi

# Edit by linrunjin 20131231
# avoid to run mutiple rsync instance
# -----------------------------------------------
RSYNC_PIDS=$(pidof rsync)
for pid in $RSYNC_PIDS
do
    RSYNC_PATH=$(readlink -f /proc/$pid/cwd)
    if [ "$RSYNC_PATH" == "/usr/local/i386/public_repos/initial_system/snmp/ext/bizop/public-scripts/bin" ]
    then
        echo "bizop rsync already running"
        exit
    fi
done

# -----------------------------------------------

if [ $# -lt 1 ]
then
	echo "------ Must specify at least one allow ip! ------"
	echo "Usage: bash $0 allow_ip1[ allow_ip2 ... ]"
	exit 2
fi

RSYNC_CONF=/tmp/rsync_bizop.conf
USER_FILE=/tmp/rsync_bizop.user
DIR=/
MODULE=bizop
USER=web_bizop
CODE='W*g-SVF15GIVT\|,ovYp6-.r9+7C8YUNoy69bt,y8-kFkm5_Y.CK36l.tbF\:dX+2XIQ'
ALLOW_IP=$*

cat > $RSYNC_CONF <<EOF
uid = root
gid = root
use chroot = yes
port = 2875
pid file = /tmp/rsync_bizop.pid
lock file = /tmp/rsync_bizop.lock
log file = /tmp/rsync_bizop.log

[rsync_bizop]
        path = $DIR
        comment = for bizop web sync files.
        read only = yes
        ignore errors
        strict modes = yes
        auth users = $USER
        secrets file = $USER_FILE
	hosts allow = $ALLOW_IP
	hosts deny = *
EOF

cat > $USER_FILE <<EOF
$USER:$CODE
EOF


chown root:root $USER_FILE
chmod 600  $USER_FILE

rsync --daemon --config $RSYNC_CONF
