#!/bin/bash

trap 'if [ -n "$dstFile" ] ; then 

         chattr -i $dstFile &>/dev/null 

      fi' EXIT TERM KILL HUP

set +H

export PATH="/usr/bin:/bin:/sbin:/usr/sbin"

export LANG=C

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T')| $line"

   done < <(echo "$*")

   return

}

if [ "$UID" -ne 0 ] ; then

     print_log "Err! only root user can run this script"

     aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
fi

chmod a+x $0

print_log "$0 $*"

# paramater check

if [ $# -ne 1 ] ; then 
	
     print_log "Err! $0 <base64-encoded paras>"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
else
     # check whether file exist

     if [ $# -eq 1 ] ; then 

          dstFile=/etc/hosts
     else
          dstFile="$2"
     fi

     print_log "host file = $dstFile"

     if [ ! -f "$dstFile" ] ; then

          print_log "Err! white list file [$dstFile] not a file or not exist"

          aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     else
          # set dstFile to the full-path 

          dstFile="$(readlink -f $dstFile)"

          print_log "host file full-path = $dstFile"

          tmpFile="${dstFile}.tmp"

          print_log "tmp file = $tmpFile"
     fi

     # decoded base64-encoded string

     #pattern='^[[:space:]]*(([0-9]+\.){3}[0-9],?)+[[:space:]]+[0-9A-Za-z._-]+[[:space:]]*$'

     pattern='^[[:space:]]*((([0-9]+\.){3}[0-9]+)+,?)+[[:space:]]+[0-9A-Za-z._-]+[[:space:]]*$'

     ipList=$(echo "$1" | egrep -v '^[[:space:]]*$' | sed -r "s/$pattern//g")

     if [ -n "$ipList" ] ; then

          # try base64 decode

          ipList=$(base64 -d <(echo "$1")) || { print_log "Err! invalid base64 string [$1], decode failed"

                                               aaRESULT=1

                                               echo "aaRESULT=$aaRESULT"

                                               exit $aaRESULT ; }

          if [ -z "$ipList" ] ; then

               print_log "Err! decode result is Empty !!!"

               aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
          fi

     else
          print_log

          print_log "Ok! input type is ip-isp string , no need to decode"

          # re-assign the ipList var to the input content

          ipList=$(echo "$1" | egrep -v '^[[:space:]]*$')
     fi

     # check ip list format if it contains invalid characters

     x=$(echo "$ipList" | sed -r "s/$pattern//g") 

     if [ -n "$x" ] ; then

          print_log "ip list contain invalid characters : [$x]"

          print_log "it should be \"<ip> <domain>\" format"
     
          aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi

     # check whether is opened by vi

     swpFile="$(dirname $dstFile)/.$(basename $dstFile).swp"

     if [ -f "$swpFile" ] ; then

        print_log "Err! found swp file $swpFile , may be opened by vim" ; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT

     fi
     
     # change $dstFile to readonly

     if ! chattr +i $dstFile ; then

          print_log "Err! set file [$dstFile] read-only failed"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi           

     # create a backup file

     cp $dstFile $tmpFile 

     # begin to work

     while read ips domain ; do

          sed -i -r "/[[:space:]]+${domain}[[:space:]]*$/d" $tmpFile

          for ip in ${ips//,/ } ; do

              echo "$ip $domain" >> $tmpFile

              print_log "[$ip $domain] added"
         done

     done <<< "$ipList"

     # backup $dstFile and overwrite it with $tmpFile
   
     bakFile="$dstFile.$(date +'%Y%m%d_%H%M')"

     if cp $dstFile $bakFile ; then

         print_log "Ok! $dstFile backup succeed"

         chattr -i $dstFile && mv $tmpFile $dstFile && print_log "Finished"

         aaRESULT=0 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 

    else
         print_log "Err! $dstFile backup failed"

         chattr -i $dstFile 

         aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
    fi
fi
