#!/bin/bash

trap 'if [ -n "$dstFile" ] ; then 

         chattr -i $dstFile &>/dev/null 

      fi' EXIT TERM KILL HUP

set +H

export PATH="/usr/bin:/bin:/sbin:/usr/sbin"

export LANG=C

sleep 10

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T')| $line"

   done < <(echo "$*")

   return

}

if [ "$UID" -ne 0 ] ; then

     print_log "Err! only root user can run this script"

     aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
fi

chmod a+x $0

print_log "$0 $*"

# paramater check

if [ $# -ne 1 ] ; then 

	 print_log "Err! args=$*"
	
	 print_log "Err! $0 <base64-encoded paras>"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
else
     # check whether file exist

     if [ $# -eq 1 ] ; then 

          dstFile=/etc/rc.local
     else
          dstFile="$2"
     fi

     print_log "kernel param file = $dstFile"

     if [ ! -f "$dstFile" ] ; then

          print_log "Err! kernel param file [$dstFile] not a file or not exist"

          aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     else
          # set dstFile to the full-path 

          dstFile="$(readlink -f $dstFile)"

          print_log "kernel param file full-path = $dstFile"

          tmpFile="${dstFile}.tmp"

          print_log "tmp file = $tmpFile"
     fi

     # decoded base64-encoded string

     paramList=$(base64 -d <(echo "$1")) || { print_log "Err! invalid base64 string [$1], decode failed"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT; }

     if [ -z "$paramList" ] ; then

        print_log "Err! decode result is Empty !!!"

        aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi

     echo "$paramList"

     paramList=$(echo "$paramList" |tr '=' ' ') 

     # check whether is opened by vi

     swpFile="$(dirname $dstFile)/.$(basename $dstFile).swp"

     if [ -f "$swpFile" ] ; then

        print_log "Err! found swp file $swpFile , may be opened by vim" ; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT

     fi

     # change $dstFile to readonly

     if ! chattr +i $dstFile ; then

          print_log "Err! set file [$dstFile] read-only failed"; aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
     fi           

     # create a backup file

     cp $dstFile $tmpFile 

     # begin to work

     while read param value ; do

         if sysctl -w ${param}="$value" >/dev/null ; then

              print_log "${param}=$value"
         else
              print_log "Err! set $param to $value failed"

              aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
         fi 

         sed -i -r "/sysctl -w ${param}=/d" $tmpFile
 
         echo "sysctl -w ${param}=\"$value\"" >> $tmpFile


     done <<< "$paramList"

     # backup $dstFile and overwrite it with $tmpFile
   
     bakFile="$dstFile.$(date +'%Y%m%d_%H%M')"

     if cp $dstFile $bakFile ; then

         print_log "Ok! $dstFile backup succeed"

         chattr -i $dstFile

         mv $tmpFile $dstFile

         aaRESULT=0 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT

    else
         print_log "Err! $dstFile backup failed"

         chattr -i $dstFile 

         aaRESULT=1 ; echo "aaRESULT=$aaRESULT" ; exit $aaRESULT 
    fi
fi
