#!/bin/bash
export PATH=/usr/bin:/bin:/sbin:/usr/sbin

function is_valid_interface {

   local int=${1:?$FUNCNAME Err! interface must be gived}

   /sbin/ifconfig -a |grep 'Link encap:'|awk '{print $1}' |grep -x $int -q

   return $?
}

function get_local_ip {

   local int=$1 ip

   is_valid_interface $int || { echo "Err! invalid interface $int" >&2; return 1; }

   ip=$(/sbin/ifconfig |egrep "^${int}[[:space:]]+" -A1 |egrep 'inet addr:[^ ]+' -o |cut -d':' -f 2)

   if [ -n "$ip" ] ; then

        echo "$ip"

        return
   else
        return 1
   fi
}

function json_output(){
	
	if [[ "$#" -lt 2 ]];then
		echo 'json_output usage: json_output status info'
	fi
	status=$1
	shift
	info=$*
	info=$(echo "$info" | tr '"' "'")
	echo "{ \"status\" : \"$status\" , \"info\" : \"$info\" }"
	exit $status
}

status=0
info=''

proc_name="lg_login_d"
lg_pids=$(pidof $proc_name)

if [[ -z "$lg_pids" ]];then
	status=1
	info="no $proc_name's pids in this ip"
	json_output $status $info
fi

start_time=$(date +"%F %T")

for i in $lg_pids ; do
	port=$(ls /home/dspeak/yyms/proc_info/${i}* | awk -F"-" "{print \$3}")
	info="$info\nlogind (pid:$i) is using port $port"
	nc -q 15 0 $port <<< "agent get proc" | grep "pname.*lg" > /dev/null

	if [[ $? = 0 ]] ; then
		nc -q 15 0 $port <<< "config reload all" | grep -v "^[[:space:]]*$"
		if [[ "$?" -ne 0 ]] ; then
			$status=1
			info="$info\nreload port=$port failed"
			json_output $status $info
		fi
	else
		status=1
        info="$info\nthe process listening on port $port does not seem to be a logind"
		json_output $status $info
	fi
done

sleep 5

end_time=$(date +"%F %T")

eth0Ip=$(get_local_ip "eth0" | head -1)

status=0
info="ip=$eth0Ip start_time=$start_time end_time=$end_time\n"
json_output $status $info
