#/bin/bash
#Author:huangweisong
#update:2013/11/27
#Comment:check CO or slave redis health state

set +H
export LANG=C
export LC_ALL=C
export PATH=/usr/bin:/bin:/sbin:/usr/sbin

#common function

function report()
{
  echo "{\"status\":$1,\"info\":\"$2\",\"data\":\"$3\"}"
}

function Usage() 
{
  info="Usage:cul_health_check.sh CO/SLAVE"
  report 1 "$info"
  exit 1
}

function check_connect() 
{
  host=$1
  port=$2
  for i in {1..5}
  do	  
     nc -z $host $port
	 if [ $? -eq 0 ];then
	   	 echo "  SUCCESS:$host $port connect sucess"
		 return 0
	 fi
	 sleep 0.5
  done
  echo "  ERROR:$host $port connect fail"
  Flag=1
}  

function check_process()
{
  proname=$1	
  pronum=$2
  pidnum=`ps aux |grep $proname|grep -v grep|wc -l`
  if [ $pidnum -ne $pronum ];then
	  echo "  ERROR:$proname process number: $pidnum,not equal $pronum"
	  Flag=1
  else
	  echo "  SUCCESS:$proname process number: $pidnum"
  fi
}

function stat_co_master()
{
  bt_ports=`netstat -nlp |grep bt_co_d|awk -F '[ :]+' '{print $5}'`	
  for port in $bt_ports
  do
	  dsinfo="$dsinfo\n"`echo 'ds'|nc 0 $port -q 1 |grep 'total session'`
  done
  res=`echo -e "$dsinfo"|awk '{sum+=$4}END{print sum}'`
  echo ${res:-0}
}	

function stat_db_master()
{
   if ! which mysql &>/dev/null ;then
      aptitude -y install ` aptitude search  mysql-client-core|grep -v ':i386'|head -1|awk  '{print $2}'` &>/dev/null
   fi	  
   sessnum=`mysql -h mysql.appdaemin.yy.com -P 3306 -u root -e 'select count(*) from appDaemon.serveridInfo where isMaster=1' -B|tail -1`
   echo ${sessnum:-0}
}	

function count_rate()
{
  if [ $1 -ge $2 ];then
	  bigint=$1;smallint=$2
  else
	  bigint=$2;smallint=$1
  fi
  rate=`echo "scale=0;$((((bigint - smallint))*100))/$bigint"|bc -l`
  echo $rate
}

function check_master_sess_server_num()
{
  diff_rate=${1:-5}	
  sess_co_num=`stat_co_master`
  sess_db_num=`stat_db_master`
  rate=`count_rate $sess_co_num $sess_db_num`
  if [ $rate -ge $diff_rate ];then
	   echo "  ERROR:CO_connect_master_session_server:$sess_co_num  appdaemon_mysql_master_session_server:$sess_db_num  diff_rate:$rate%"
	   Flag=1
  else
	   echo "  SUCCESS:CO_connect_master_session_server:$sess_co_num  appdaemon_mysql_master_session_server:$sess_db_num  diff_rate:$rate%"
  fi	   
}

function get_db_keys()
{
  host=$1
  port=$2
  echo "`$redis_path -h $host -p $port 'info'|grep 'db.*keys'|sed 's/,expires.*//g'`"
}

function check_master_slave_redis_keys()
{
  process_name=$1	
  diff_rate=${2:-5}
  ports=`awk -F ':' '/^ *listen[: ]+/{print $2}' /data/services/bt_cul_acc_redis_slave*/bin/bt_cul_redis_slave.conf`
  for port in $ports
  do
    confile="$confile $port.redis.conf"
  done	
  for ins in $confile
  do
	  Sport=`echo $ins|awk -F'.' '{print $1}'`
	  Mport=`grep '^slaveof' /data/services/$process_name-*/bin/$ins |sed 's/
//g'|awk '{print $NF}'`
	  if [ `grep '^slaveof' /data/services/$process_name-*/bin/$ins|sed 's/
//g'|wc -w` -eq 3 ];then
	       Mhost=`grep '^slaveof' /data/services/$process_name-*/bin/$ins|sed 's/
//g' |awk '{print $2}'`
      fi
	  Mhost=${Mhost:-127.0.0.1}
      Mport_info=`get_db_keys ${Mhost} $Mport`
	  Sport_info=`get_db_keys 127.0.0.1 $Sport`
	  if [ `echo "$Mport_info"|wc -l` -ne `echo "$Sport_info"|wc -l` ];then
		   echo "  ERROR:master_redis and slave_redis db number is not same"
		   Flag=1
		   return 
	  fi	   
	  for db in `echo "$Mport_info"|awk -F':' '{print $1}'`
	  do
		  Mkeys=`echo "$Mport_info"|grep -w "^$db"|awk -F'=' '{print $2}'`
		  Skeys=`echo "$Sport_info"|grep -w "^$db"|awk -F'=' '{print $2}'`
		  Skeys=${Skeys:-0}
          Mkeys=${Mkeys:-0}
          rate=`count_rate $Mkeys $Skeys`
		  if [ $rate -ge $diff_rate ];then
			   Flag=1
		  else
			   echo "  SUCCESS:master_${Mhost}_${Mport}_${db}_keys:$Mkeys slave_${Shost}_${Sport}_${db}_keys:$Skeys diff_rate:$rate%"
          fi
	   done	
  done	   
}

############################ MAIN #################################
#check parameter
if [ $# -ne 1 ];then
	 Usage
fi
if [ $1 != 'CO' ] && [ $1 != 'SLAVE' ];then
	 Usage
fi

Shost=`/sbin/ifconfig eth0|grep 'inet addr'|awk -F'[: ]*' '{print $4}'`
redis_path=$(dirname $(readlink -ef $BASH_SOURCE))/redis-cli
Flag=0

###
# modified by linrunjin 2013/11/29
# install bc if needed
if ! which bc &>/dev/null ; then
   aptitude -y install bc &>/dev/null
fi
# append redis-cli execute permission
if [ ! -x $redis_path ] ; then
   	chmod a+x $redis_path
fi
###

if [ A$1 = A'CO' ];then
	if ! ls -d /data/services/bt_co_d-* &>/dev/null ;then
       info="$Shost not a CO server,please check"
	   report 1 "$info"
	   exit 1
    fi   
	#1.check mysql.appdaemin.yy.com 3306
	   info="1.check connect to mysql.appdaemin.yy.com 3306:"
	   info="$info\n`check_connect mysql.appdaemin.yy.com 3306`"
	#2.check bt_co_d master_redis first_slave_redis process
       info="$info\n2.check bt_co_d master_redis first_slave_redis process:"
	   bt_co_d_num=`grep '^count' /data/services/bt_co_d*/admin/start.sh|awk -F'=' '{print $2}'`
	   info="$info\n`check_process bt_co_d $bt_co_d_num`"
       m_redis_num=`ls /data/services/bt_co_d*/mpRedis/1637*.redis.conf|wc -w`
	   info="$info\n`check_process redis-server $m_redis_num`"
	   first_redis_num=`sed -n '/redisPorts="/,/"/p' /data/services/bt_cul_redis_slave-*/admin/start.sh|sed -e 's/redisPorts="//g' -e 's/"//g'|sed "s/\s/\n/g"|grep -v '^$'|wc -l`
	   info="$info\n`check_process bt_cul_redis_slave $first_redis_num`"
	#3.check master session server number
	   info="$info\n3.check master session server number:"
       info="$info\n`check_master_sess_server_num 1`"
	#4.check master and first slave redis key number
	   info="$info\n4.check master and first slave redis key number:"
	   info="$info\n`check_master_slave_redis_keys bt_cul_redis_slave 3`"
fi	

if [ A$1 = A'SLAVE' ];then
	if ! ls -d /data/services/bt_cul_*_redis_slave-* &>/dev/null ;then
	   info="$Shost not a slave redis,please check"
	   report 1 "$info"
	   exit 1
    fi   
	#1.check bt_cul_<idc>_redis_slves proces
	   info="1.check bt_cul_<idc>_redis_slves proces:"
	   second_redis_num=`awk -F ':' '/^ *listen[: ]+/{print $2}' /data/services/bt_cul_acc_redis_slave*/bin/bt_cul_redis_slave.conf|wc -l`
	   process_name=`grep '^app_name=' /data/services/bt_cul_*_redis_slave-*/init.conf|awk -F'"' '{print $2}'`
	   info="$info\n`check_process $process_name $second_redis_num`"
	#2.check second_redis_key and first_redis_key number
	   info="$info\n2.check second_redis_key and first_redis_key number:"
       info="$info\n`check_master_slave_redis_keys $process_name 2`"
fi	   

if [ $Flag -eq 0 ];then
	 data=`echo -e "$info"`
	 info=""
else
	 info=`echo -e "$info"`
fi	 
report $Flag "$info" "$data"
