#!/bin/bash

#!/bin/bash
# NAME            : $0 
# Ver             : 0.1
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2013-11-19
# Summary         : query Service/CUL Redis data 


#----------------------------#
# Set Basic Environment Zone #
#----------------------------#

set +H

MYPID=$$

export LANG=C

export LC_ALL=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

export PROG="$(basename $0)"

scriptPath=$(readlink -fe $BASH_SOURCE)

scriptName=$(basename "$scriptPath")

scriptDir=$(dirname $scriptPath)

queryProg="$scriptDir/redis-cli"

QUERY_TYPE_LIST='Available Query Type :
        00 : (topsid=>sid list) list all subsid for given topsid which "smembers > 0"
        01 : (topsid:subsid=>uid list) list all uid for given topsid:subsid
        10 : (uid:topsid=>subsid:role) output the user role for given subsid
        20 : (topsid=>master sessionid) output the master session for given topsid'

USAGE="Usage :
       $BASH_SOURCE -q [OPTIONS] : do redis query
       $BASH_SOURCE -l : list available query type
       $BASH_SOURCE -v : output script version
       $BASH_SOURCE -d : dump channel user role mapping
       $BASH_SOURCE -h : display this message

       Option ::
       -q : query mode
       -t <query type> : see below
       -u <uid>        : used for query type '10'
       -S <topsid>     : used for ALL query types
       -s <subsid>     : used for query type '01'
       -i <redis ip1,ip2,...>   : connect to given IPs
       -p <redis port> : connect to given port

$QUERY_TYPE_LIST
"

channelRoler='        NUL_ROLE = 0,
        VISITOR = 20,           //VISITOR
        NORMAL = 25,            //U
        DELETED = 50,           //unuse
        TMPVIP = 66,            //TMPVIP
        VIP = 88,               //VIP
        MEMBER = 100,           //R
        CMANAGER = 150,         //CA
        PMANAGER = 175,         //PA
        MANANGER = 200,         //MAN
        VICE_OWNER = 230,       //VO
        OWNER = 255,            //OW
        KEFU = 300,             //CS
        POLICE = 400,           //
        SA = 1000,              //SA'

function exit_on_err {

   local info="$@"

   echo "{\"status\":1,\"info\":\"$info\",\"data\":\"\"}"

   exit ${code:=1}
}

weight=0


# load common bash functions

source /data/bizop/public-scripts/func/common.sh || { exit_on_err "Err! can't load common.sh" ; exit 1 ; } 

# Syntax Check

if [ $# -eq 0 ] ; then

     exit_on_err "Err! syntax wrong" ; exit 1
fi

# Command Line process

while getopts ":t:u:S:s:i:p:qhlvd" opt ; do

	  case $opt in

                  'q') mode="query" ;;

		  't') queryType="$OPTARG" ;;

		  'u') uid="$OPTARG"       ; ((weight+=1)) ;;

		  'S') topSid="$OPTARG"    ; ((weight+=2)) ;;

		  's') subSid="$OPTARG"    ; ((weight+=4)) ;;

		  'i') redisIps="$OPTARG"   ;;

		  'p') redisPort="$OPTARG" ;;

		  'h') echo "$USAGE"       ; exit 0 ;;

		  'l') echo "$QUERY_TYPE_LIST" ; exit 0 ;; 

                  'd') echo "$channelRoler" ; exit 0 ;; 

                  'v') ver=$(grep '^# Ver' $BASH_SOURCE | head -1| awk -F ':' '{print $2}' | tr -d '[[:space:]]') ; echo "$ver" ; exit 0 ;;

		  *)   exit_on_err "Err! syntax wrong" ; exit 1 ;;

      esac

done

# simple arg check

if [ -n "$redisPort" ] ; then

     #if [ -n "${redisPort//[0-9]}" ]; then

     if ! is_positive_integer $redisPort ; then

          exit_on_err "Err! invalid redis port $redisPort"

     elif [ $redisPort -gt 65535 ] || [ $redisPort -le 0 ] ; then

         exit_on_err "Err! invalid redis port $redisPort"
     fi
fi

if [ -n "$redisIps" ] ;then
   for redisIp in `echo "$redisIps" |sed -e 's/ +//g' -e 's/,/ /g'`
   do	
      if ! is_ip "$redisIp" ; then
         exit_on_err "Err! invalid redis ip $redisIp"
      fi
   done
fi

if [ -n "$uid" ] && ! is_positive_integer "$uid" ; then

    exit_on_err "Err! invalid uid $uid"
fi

if [ -n "$topSid" ] && ! is_positive_integer "$topSid" ; then

    exit_on_err "Err! invalid uid $topSid"
fi

if [ -n "$subSid" ] && ! is_positive_integer "$subSid" ; then

    exit_on_err "Err! invalid uid $subSid"
fi

# set default redis ip
if [ -z "$redisIps" ] ; then

	redisIp=(114.111.163.30 183.57.81.243 58.215.46.18 119.147.176.12)
else
	redisIp=(`echo "$redisIps" |sed -e 's/ +//g' -e 's/,/ /g'`)
fi

# find redis-cli under working directory

if [ ! -f $queryProg ] ; then

     exit_on_err "Err! Can't find $(basename $queryProg) under $scriptDir"
else
     chmod a+x $queryProg
fi

# main work

function check_redis() {
if [ "$mode" = "query" ] ; then

     case $queryType in 
     
       '00') if [ $weight -eq 2 ] ; then
     
                  x=$($queryProg -h $1 -p ${redisPort:=26380} -n 1 smembers $topSid 2>$scriptDir/$ip-err.log) ; rc=$?
             else
                  exit_on_err "Err! syntax wrong"
             fi ;;
     
       '01') if [ $weight -eq 6 ] ; then
     
                  x=$($queryProg -h $1 -p ${redisPort:=26380} -n 0 smembers $topSid:$subSid 2>$scriptDir/$ip-err.log) ; rc=$?
             else
                  exit_on_err "Err! syntax wrong"
             fi ;;
      
     
       '10') if [ $weight -eq 3 ] ; then
     
                  x=$($queryProg -h $1 -p ${redisPort:=26381} hgetall $uid:$topSid 2>$scriptDir/$ip-err.log) ; rc=$?
            else
                  exit_on_err "Err! syntax wrong"
            fi ;;
     
     
       '20') if [ $weight -eq 2 ] ; then
     
                  x=$($queryProg -h $1 -p ${redisPort:=26382} get $topSid 2>$scriptDir/$ip-err.log) ; rc=$?
             else
                  exit_on_err "Err! syntax wrong"
             fi ;;

       '') exit_on_err "Err! syntax wrong" ;;
     
       *) exit_on_err "Err! invalid query type" ;;
     
     esac

fi
}

i=0
for ip in ${redisIp[@]}
do  
	x="";rc=""
	check_redis $ip
	result[$i]="$x"
	info[$i]=`cat $scriptDir/$ip-err.log`
	Flag[$i]=$rc
	let i=i+1
done	

# packaged into json string
rc=0
x=""

for flag in ${Flag[@]}
do 
	if [ $flag -eq 1 ];then
		 rc=1	
	fi	
done

let j=${#Flag[@]}-1
if [ $rc -eq 0 ] ; then
	 for i in `seq 0 $j`
	 do
		 res="$res\"${redisIp[$i]}\":\"${result[$i]}\","
	 done	 
	 res=${res%,}
     echo "{\"status\":0,\"info\":\"succeed\",\"data\":{$res}}"
else
	 for i in `seq 0 $j`
	 do
		 res="$res\"${redisIp[$i]}\":\"${info[$i]}\","
	 done	 
	 res=${res%,}
     echo "{\"status\":1,\"info\":{\"${redisIp[0]}:${info[0]}\",\"${redisIp[1]}:${info[1]}\",\"${redisIp[2]}:${info[2]}\",\"${redisIp[3]}:${info[3]}\"},\"data\":\"\"}"
fi
