#!/bin/bash

#-----------------#
# User Check Zone #
#-----------------#


function err_log {

   local content

   if [ "$ENABLE_ERR_LOG" = 'yes' ] ; then

        while read content; do
     
           printf "%s | %30s | %30s | [ERR ] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2
     
        done < <(echo "$*")
   fi

   return
}


function note_log {

   local content

   if [ "$ENABLE_NOTE_LOG" = 'yes' ] ; then

        while read content; do
     
           printf "%s | %30s | %30s | [NOTE] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2
     
        done < <(echo "$*")
   fi

   return
}


function info_log {

   local content

   if [ "$ENABLE_INFO_LOG" = 'yes' ] ; then

        while read content  ; do
     
           printf "%s | %30s | %30s | [INFO] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content"
     
        done < <(echo "$*")
   fi

   return
}

function warn_log {

   local content

   if [ "$ENABLE_WARN_LOG" = 'yes' ] ; then

        while read content; do
     
           printf "%s | %30s | %30s | [WARN] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2
     
        done < <(echo "$*")
   fi

   return
}


function debug_log {

   local content

   if [ "$ENABLE_DEBUG_LOG" = 'yes' ] ; then

        while read content; do
     
           printf "%s | %30s | %30s | [DEBU] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2
     
        done < <(echo "$*")
   fi

   return
}


function check_user {

   local user=${1:?$FUNCNAME Err! user must be gived}

   if [ "$USER" = "$user" ] ; then

        return 0
   else
        return 1
   fi
}


function check_uid {

   local uid=${1:?$FUNCNAME Err! uid must be gived}

   if [ "$UID" = "$uid" ] ; then

        return 0
   else
        return 1
   fi

}

function which_cmd {

   local cmd=${1:?$FUNCNAME Err! command must be gived}

   which $cmd

   return $?

}


#----------------------------#
# file & directory functions #
#----------------------------#

function is_exist {

   local file=${1:?$FUNCNAME Err! file must be given}

   [ -e "$file" ] && return 0 || return 1
}


function is_file {

   local file=${1:?$FUNCNAME Err! file must be gived}

   ( [ -f "$file" ] && [ ! -L "$file" ] ) && return 0 || return 1
}

function is_link {

   local file=${1:?$FUNCNAME Err! file must be gived}

   [ -L "$file" ] && return 0 || return 1
}

function is_dir {

   local dir=${1:?$FUNCNAME Err! dir must be gived}

   [ -d "$dir" ] && return 0 || return 1
}

function is_symbol_link {

   local file=${1:?$FUNCNAME Err! file must be gived}

   [ -L "$file" ] && return 0 || return 1
}

function is_readable {

   local file=${1:?$FUNCNAME Err! file must be gived}

   [ -r "$file" ] && return 0 || return 1
}

function is_writable {

   local file=${1:?$FUNCNAME Err! file must be gived}

   [ -w "$file" ] && return 0 || return 1
}

function is_readable_file {

   local file=${1:?$FUNCNAME Err! file must be gived}

   ( is_file "$file" && is_readable "$file" ) && return 0 || return 1
}

function is_executable_dir {

   local dir=${1:?$FUNCNAME Err! dir must be gived}

   ( [ -d "$dir" ] && [ -x "$dir" ] ) && return 0 || return 1
}

function is_executable_file {

   local file=${1:?$FUNCNAME Err! file must be gived}

   ( [ -f "$file" ] && [ -n "$(file $file | grep -wi 'executable')" ] ) && return 0 || return 1
}


function is_readable_dir {

   local dir=${1:?$FUNCNAME Err! dir must be gived}

   ( is_dir "$dir" && is_readable "$dir" && is_executable_dir "$dir" ) && return 0 || return 1
}

function is_writable_file {

   local file=${1:?$FUNCNAME Err! file must be gived}

   ( is_file "$file" && is_writable "$file" ) && return 0 || return 1
}

function is_writable_dir {

   local dir=${1:?$FUNCNAME Err! dir must be gived}

   ( is_dir "$dir" && is_writable "$dir" && is_executable_dir "$dir" ) && return 0 || return 1
}

function is_empty_file {

   local file=${1:?$FUNCNAME Err! file must be gived}

   ( is_file "$file" && [ ! -s "$file" ]) && return 0 || return 1
}


#--------------#
# os functions #
#--------------#


function get_longbit {

   local x

   x=$(getconf LONG_BIT)

   echo $x

   return
}


function check_os {

   if cat /proc/version | egrep -i -q 'ubuntu' ; then

        echo "ubuntu"

   elif cat /proc/version | egrep -i -q '(red *hat)|centos' ; then

        echo "redhat"
   fi
}

function check_os_release {

   local x

   x=$(check_os)

   case $x in

        'ubuntu') cat /etc/issue | egrep -v '^[[:space:]]*$' ;;

        'redhat') cat /etc/redhat-release | egrep -v '^[[:space:]]*$' ;;
   esac

   return
}

function check_kernel_release {

   uname -r
}


#--------------------#
# numberic functions # 
#--------------------#

function dec_to_n {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   local base=${2:?$FUNCNAME [ERR ] | base is NULL}

   if [ ! -z $(tr -d '[0-9]' <<< "$x") ] ; then
      
        echo "$FUNCNAME Err! $x is not a decimal number" >&2

        return 1

   elif [ ! -z $(tr -d '[0-9]' <<< "$base") ] ; then

        echo "$FUNCNAME Err! $base is not a number" >&2

        return 1
   else
        echo "obase=${base};ibase=10;$x" | bc
   fi
}


function dec_to_bin {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   dec_to_n "$x" 2
}


function dec_to_oct {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   dec_to_n "$x" 8
}


function dec_to_hex {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   dec_to_n "$x" 16
}


function n_to_dec {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   local base=${2:?$FUNCNAME [ERR ] | base is NULL}

   if [ ! -z $(tr -d '[0-9]' <<< "$base") ] ; then

        echo "$FUNCNAME Err! $base is not a number" >&2

        return 1
   else
        echo "obase=10;ibase=$base;$x" | bc
   fi

}


function hex_to_dec {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   if [ ! -z $(tr -d '[0-9a-fA-F]' <<< "$x") ] ; then

        echo "$FUNCNAME Err! $x is not a HEX number" >&2

        return 1
   else
        n_to_dec "$(tr '[[:lower:]]' '[[:upper:]' <<< $x)" 16

   fi
}


function oct_to_dec {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   if [ ! -z $(tr -d '[0-7]' <<< "$x") ] ; then

        echo "$FUNCNAME Err! $x is not a Octet number" >&2

        return 1
   else
        n_to_dec "$(tr '[[:lower:]]' '[[:upper:]' <<< $x)" 8

   fi
}


function bin_to_dec {

   local x=${1:?$FUNCNAME [ERR ] | x is NULL}

   if [ ! -z $(tr -d '[01]' <<< "$x") ] ; then

        echo "$FUNCNAME Err! $x is not a Binary number" >&2

        return 1
   else
        n_to_dec "$(tr '[[:lower:]]' '[[:upper:]' <<< $x)" 2 

   fi

}


function ascii_to_char {

   local n=${1:?$FUNCNAME [ERR ] | n is NULL}

   if [ ! -z "$(tr -d '[0-9]' <<< "$n")" ] ; then

        echo "$n is not a number" >&2

        return 1

   elif [ $n -lt 0 ] || [ $n -gt 127 ] ; then

        echo "$n invalid  (0<=n<=127)" >&2

        return 1
   else
        awk -v _n=$n 'BEGIN {printf "%c\n",_n}'
   fi

}



function char_to_ascii {

   local char=${1:?$FUNCNAME [ERR ] | char is NULL}

   if [ ${#char} -gt 1 ] ; then

        echo "$FUNCNAME [ERR ] | only a single character allowd" >&2

        return 1
   else
        printf "%d\n" "'$char"
   fi 
}


function compare_version_fmt {

   local ver1 ver2 result x y

   if [ $# -ne 2 ] ; then

        print_log "$FUNCNAME Err! $FUNCNAME <ver1> <ver2>"

        return 1
   else
        ver1="$1" ver2="$2"

        x=$(echo "$ver1" | grep -o '\.' | wc -l)

        y=$(echo "$ver2" | grep -o '\.' | wc -l)

        if [ $x -eq 0 ] && [ $y -eq 0 ] ; then 

             return 0

        elif ( [ $x -eq 2 ] || [ $x -eq 3 ] ) && ( [ $y -eq 2 ] || [ $y -eq 3 ] ) ; then 

             return 0
        else
             return 1
        fi
   fi

}

function compare_version {

   local oldVer newVer result x y

   newVer="$1" oldVer="$2"

   # expand '.0' if ver=x.y.z

   if [ $(echo "$newVer" | grep -o '\.' | wc -l) -eq 2 ] ; then

         newVer="${newVer}.0"
   fi

   if [ $(echo "$oldVer" | grep -o '\.' | wc -l) -eq 2 ] ; then

         oldVer="${oldVer}.0"
   fi

   result=$(echo -e "${oldVer}\n${newVer}" |sort -t'.' -k1,1n -k2,2n -k3,3n -k4,4n|uniq)

   if [ "$(echo $result|wc -w)" -eq 1 ]  ; then

         return 1 # ver1 = ver2
   else
         result=$(echo "$result" |tail -1)

         if [ "$result" = "$newVer" ] ; then

              return 0  # dst version > src version
         else
              return 1
         fi
   fi
}


function is_integer {

   local pattern

   pattern='^[+-]?[0-9]+$'

   if [[ "$1" =~ $pattern ]] ; then

        return 0
   else
        return 1
   fi
}

function is_float {

   local pattern

   pattern='^[+-]?[0-9]+(\.[0-9]+)?$'

   if [[ "$1" =~ $pattern ]] ; then

        return 0
   else
        return 1
   fi
}

function is_ip {

   local x

   x=$(echo "$1" | awk -F '.' 'NF==4 && (0<=$1 && $1<=255) &&  (0<=$2 && $2<=255) &&  (0<=$3 && $3<=255) &&  (0<=$4 && $4<=255)')

   if [ "$x" = "$1" ] && [ -n "$x" ] ; then

        return 0
   else
        return 1
   fi
}

function is_positive {

   x=$(echo "$1" |awk '$1 > 0')

   if [ -n "$x" ] ; then

        return 0
   else
        return 1
   fi
}

function is_positive_integer {

   if is_integer $1 && is_positive $1 ; then

        return 0
   else
        return 1
   fi
}


function is_positive_float {

   if is_float $1 && is_positive $1 ; then

        return 0
   else
        return 1
   fi
}


function conv_dec_to_hex {

   local num=$1

   local bits=$2

   if is_positive_integer $num ; then

      printf "%0${bits}x\n" $num
   fi
}

#------------------------#
# network functions      #
#------------------------#

function is_valid_interface {

   local int=${1:?$FUNCNAME Err! interface must be gived}

   /sbin/ifconfig -a |grep 'Link encap:'|awk '{print $1}' |grep -x $int -q

   return $?
}


function get_interface_status {

   local int=${1:?$FUNCNAME Err! interface must be gived}

   is_valid_interface $int || { echo "Err! invalid interface $int" >&2; return 1; }

   /sbin/ifconfig $int |egrep 'UP (BROADCAST|LOOPBACK)' -q

   if [ $? -eq 0 ] ; then

        echo "UP" ; return 0
   else
        echo "DOWN" ; return 1
   fi

}

function get_all_ip {

   local x privateIp

   #x=$(ifconfig |grep 'inet addr' |grep -v 127.0.0.1 |awk -F '[ :]+' '{print $4}' | xargs | sed 's/ /,/g')

   privateIp='inet +addr: *(127\.|192\.168\.|10\.|172\.(1[6-9]|2[0-9]|3[0-1])\.)'

   x=$(ifconfig |grep 'inet addr' | egrep -v "$privateIp" |awk -F '[ :]+' '{print $4}' | xargs | sed 's/ /,/g')

   echo "$x"
  
   return

}


function get_first_ip {

   local x

   # private ip is filterd by get_all_ip already

   x=$(get_all_ip | xargs | awk -F '[, ]+' '{print $1}')

   echo "$x"

   return
}


function get_second_ip {

   local x

   # private ip is filterd by get_all_ip already

   x=$(get_all_ip | xargs | awk -F '[, ]+' '{print $2}')
 
   echo "$x"

   return
}


function get_local_ip {

   local int=$1 ip

   is_valid_interface $int || { echo "Err! invalid interface $int" >&2; return 1; }

   ip=$(/sbin/ifconfig |egrep "^${int}[[:space:]]+" -A1 |egrep 'inet addr:[^ ]+' -o |cut -d':' -f 2)

   if [ -n "$ip" ] ; then

        echo "$ip"

        return
   else
        return 1
   fi
}


#-----------------#
# Sting Functions #
#-----------------#

function repeatStr {

   local string=${1:?$FUNCNAME [ERR ] | string is NULL}
 
   local count=${2:?$FUNCNAME [ERR ] | count is NULL}

   local delimeter=${3:-''}

   local x

   x=$(for ((i=1;i<=$count;i++)) ; do echo -n "${string}${delimeter}" ; done;echo)

   echo "$x"

   return
}


#-----------------#
# Array Functions #
#-----------------#

function append {

   local _name_=${1:?$FUNCNAME [ERR ] | _name_ is NULL}
   
   shift
   
   if ! declare -p ${_name_} 2>/dev/null | grep '^declare -a' -q ; then
   
        echo "${_name_} is not an array or not set"
   
        return 1
   else
        # add each element to the array
   
        for i in "$@" ; do
   
            eval "${_name_}=(" " \"\${${_name_}[@]}\" "  "  \"$i\" "  ")"
          
        done
   
        return
   fi

}


#----------------------#
# Filesystem Functions #
#----------------------#

function partition_usage {

   local partition=${1:?$FUNCNAME [ERR ] | partition is NULL}

   local x

   partition="/${partition}"

   x=$(df $partition 2>/dev/null| awk '{print $5}' | tail -n +2)

   if [ -n "$x" ] ; then

        echo ${x//%}

        return
   else
        echo "invalid partition $partition"
 
        return 1
   fi

}

