# -*- coding: utf-8 -*-

import traceback
import os
import re
import md5
from subprocess import Popen, PIPE

def read_whole_file(filepath):
    try:
        f = open(filepath)
        data = f.read()
        f.close()
        return data
    except:
        print traceback.print_exc()
        return None;

def write_file(filepath, data):
    try:
        f = open(filepath, 'w')
        f.write(data)
        f.close()
        return True
    except:
        print traceback.print_exc()
        return False;
        
def md5_small_data(data):
    try:
        return md5.new(data).hexdigest()
    except:
        print traceback.print_exc()
        return None;

class HostInfo:
    ip_pattern = re.compile("inet\s+addr:(?P<ip>\d+.\d+.\d+.\d+)\s+Bcast:", 
                          re.DOTALL | re.IGNORECASE)
    uuid_path = "/home/dspeak/yyms/uuid" 
    hostinfo_path = "/home/dspeak/yyms/hostinfo"
    hostinfo_ini_path = "/home/dspeak/yyms/hostinfo.ini"   
    isp_map = {
            1: "two-lane",
            #2: "two-lane-telecom",
            #3: "two-lane-unicom",
            4: "CTL", # "telecom",
            5: "CNC", # "unicom",
            6: "BGP",
            7: "EDU",
            8: "MOB",
            10: "INTRANET",
     }
   
    @staticmethod
    def is_unicom(hostinfo_obj):
        if(not hostinfo_obj):
            return False
        ips = hostinfo_obj.get("ips", [])
        for m in ips:
            if m.get("isp", 0) == 5:
                return True
        return False
   
    @staticmethod
    def read_file(filepath):
        try:
            f = open(filepath)
            data = f.read()
            f.close()
            return data
        except:
            print traceback.print_exc()
            return None;
    
    @staticmethod
    def get_uuid():
        try:
            if(not os.path.exists(HostInfo.uuid_path)):
                return None
            data = read_whole_file(HostInfo.uuid_path)
            if data:
                data = data.strip()
                if(len(data) != 36):
                    return None
            return data
        except:
            print traceback.print_exc()
            pass;
        pass
        
    @staticmethod
    def get_local_ip_list(): 
        try:
            ip_list = [] 
            if os.name == "posix":
                pp = Popen(['/sbin/ifconfig'], stdout=PIPE)
                data = pp.stdout.read()
                ms = re.findall(HostInfo.ip_pattern, data)
                for m in ms:
                    ip_list.append(m)
            return ip_list
        except:
            print traceback.print_exc()
            return None    
        
    @staticmethod
    def get_hostinfo():     
        """ 
            return hostinfo object
        """
        ret = None
        try:
            if(not os.path.exists(HostInfo.hostinfo_path)):
                return ret
            data = read_whole_file(HostInfo.hostinfo_path)
            if data:
                return eval(data)
        except:
            print traceback.print_exc()
            os.remove(HostInfo.hostinfo_path)
            os.remove(HostInfo.hostinfo_ini_path)
        return ret   
        
    @staticmethod
    def hostinfo_to_ini(obj):
        lines = []
        for k,v in obj.iteritems():
            if(k != "ips"):
                lines.append("%s=%s" % (k, v))
            if(k == "ips"):
                ipisp_list = []
                for m in v:
                    ipisp_list.append("%s:%s" % (m["ip"], HostInfo.isp_map.get(m["isp"], 0)))
                lines.append("ip_isp_list=%s" % (",".join(ipisp_list)))
        return '\n'.join(lines)
        pass
    
    @staticmethod
    def write_hostinfo(data, guid, hostinfo_obj):
        # 为比较md5, 服务端是用 ordered map, 直接写数据 - 改为不比较！
        write_file(HostInfo.hostinfo_path, data) 
        write_file(HostInfo.hostinfo_ini_path, HostInfo.hostinfo_to_ini(hostinfo_obj))
        # 主机 guid 生成后不再改变
        if(not os.path.exists(HostInfo.uuid_path)):
            write_file(HostInfo.uuid_path, guid)
    
    @staticmethod
    def write_not_exists_hostinfo(data, guid, hostinfo_obj):
        if(not os.path.exists(HostInfo.hostinfo_path)):
            write_file(HostInfo.hostinfo_path, data) 
        if(not os.path.exists(HostInfo.uuid_path)):
            write_file(HostInfo.uuid_path, guid)
        if(not os.path.exists(HostInfo.hostinfo_ini_path)):
            write_file(HostInfo.hostinfo_ini_path, HostInfo.hostinfo_to_ini(hostinfo_obj))
            
    @staticmethod
    def update_hostinfo(data):
        try:
            hostinfo_obj = eval(data)
            if(not hostinfo_obj):
                return False
            if(not hostinfo_obj.has_key("guid")):
                print '''not hostinfo_obj.has_key("guid")'''
                return False
            guid = hostinfo_obj["guid"].strip()
            if(len(guid) != 36):
                print '''if(len(guid) != 36):'''
                return False
            ts = hostinfo_obj.get("ts", 0)
            current_uuid = HostInfo.get_uuid()
            current_hostinfo_obj = HostInfo.get_hostinfo()
            if(current_hostinfo_obj): 
                current_ts = current_hostinfo_obj.get("ts", 0)
                # 机器的guid生成后不应该改变
                if(guid != current_uuid or guid != current_hostinfo_obj.get("guid", None)): 
                    # 告警
                    print "guid != current_uuid"
                    return False     
                if(ts > current_ts):                
                    HostInfo.write_hostinfo(data, guid, hostinfo_obj)
                else:
                    HostInfo.write_not_exists_hostinfo(data, guid, hostinfo_obj)
            else:
                HostInfo.write_hostinfo(data, guid, hostinfo_obj) 
            return True                       
        except:
            print traceback.print_exc()
            return False
        
    
from unittest import TestCase  
class TestHostInfo(TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass
    def test_get_uuid(self):
        self.assertEquals("661dd7b6-5727-47bd-afc7-745071295bd1", HostInfo.get_uuid())
        pass
    def test_get_local_ip(self):
        local_ip_list = HostInfo.get_local_ip_list()
        self.assertEqual(1, len(local_ip_list))
        self.assertEqual("113.106.100.63", local_ip_list[0])
        pass
    def test_get_hostinfo(self):
        obj = HostInfo.get_hostinfo()
        # md5sum /home/dspeak/yyms/hostinfo
        print obj
        pass
    def test_hostinfo2ini(self):
        data = '''{"room":"亚太双线", "status":1, "dept":"dw-yy",
        "guid":"7eaba9dd-f4ad-474a-86c2-b3599147974f", 
        "pri_group_id":"111",
        "sec_group_id":"",
        "ips":[{"ip": "121.14.37.138", "isp": 4}, {"ip": "58.249.119.202", "isp": 5}]}'''
        obj = eval(data)
        txt = HostInfo.hostinfo_to_ini(obj)
        self.assertEquals('''status=1
room=亚太双线
sec_group_id=
pri_group_id=111
ip_isp_list=121.14.37.138:CTL,58.249.119.202:CNC
dept=dw-yy
guid=7eaba9dd-f4ad-474a-86c2-b3599147974f''', txt)        
    pass
      
if __name__ == "__main__":
    import unittest
    unittest.main()
