#!/bin/bash
# Written by wuhaiting
# Date 2012-12-10
# 修改网卡配置 IP、netmask

PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/usr/sbin:/sbin:/bin

# check linux version
cat /etc/issue|grep -i -E "CentOS|Red" >/dev/null 2>&1 && LINUX_VERSION="centos"
cat /etc/issue|grep -i -E "Ubuntu|Debian" >/dev/null 2>&1 && LINUX_VERSION="ubuntu"

IPADDR=`/sbin/ifconfig|grep Bcast|grep -v -E "inet addr:192.168|inet addr:172.16|inet addr:10.0"|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`
[[ "$IPADDR" == "" ]] && IPADDR=`/sbin/ifconfig|grep Bcast|head -n 1|awk '{print $2}'|awk -F : '{print $2}'`

LOG_DIR=/var/log/sysop_manager
ubuntu_interface_dir=/etc/network
centos_interface_dir=/etc/sysconfig/network-scripts
#ubuntu_interface_dir=/home/wuhaiting
#centos_interface_dir=/home/wuhaiting

# 工具返回码:
# 参数不对：1
# 相关ip掩码已存在：2
# ip掩码原来已经不存在：3
# 默认网关ip不能删除: 4
# 成功: 0

function printmsg () {
	echo -e -n "\033[32;40;1m$1\033[0m\n"
}

function printlog () {
	[[ $# -ne 0 ]] && echo $* >> $LOG_DIR/sysop_manager.log
}

function exit_tools_platform () {
	# 根据工具来设定 platform_result_tag
	platform_result_tag="result"
	echo -e "$platform_result_tag\t$IPADDR\t$*"

}

function cacle_prefix () {
	cacle_netmask=$1
	cacle_netmask=`echo ${cacle_netmask##*\.}`
	prefix=`echo $cacle_netmask | awk '{a=256-$1;b=log(a)/log(2);c=32-b;print c}'`
	echo $prefix
}

# 该函数返回参数为ip和mask的网卡配置信息
function param_interfaces_msg () {
	[ $# -lt 2 ] && return;
	param_ip=$1
	param_mask=$2

	if [ "$LINUX_VERSION" == "ubuntu" ];then
		awk '{
			$0=$1" "$2
			if($1 ~ /^auto/){
				a=1;
			}else if($0 ~ /address.*'$param_ip'$/){
				b=1;
				if(a==1){print $0};
			}else if(a==1 && b==1 && $0 ~ /netmask/){
				print $0;
			}else if(a==1 && b==1 && $0 ~ /gateway|broadcast|hwaddress/){
				print $0;
			}else if(a==1 && b==1){
				exit 2
			}
		}' $ubuntu_interface_dir/interfaces
	elif [ "$LINUX_VERSION" == "centos" ];then
		which_file=`grep -w "$param_ip" $centos_interface_dir/ifcfg-eth* | head -n1`
		which_file=`echo ${which_file%:*}`
		[[ -s $which_file ]] && grep -wE "IPADDR|NETMASK|GATEWAY" $which_file
	fi
}

# 新增ip,但是不会改变默认网关那一组ip
function add_new_ip () {
	add_ip=$1
	add_netmask=$2
	add_gateway=$(/bin/bash /usr/local/i386/*/other/calc_gateway.sh $add_ip $add_netmask|awk -F ':' '{print $2}')

	echo $add_ip | grep -Eq "^10\.|^192\.|^172\." && internal=1 || internal=0

	if [ "$LINUX_VERSION" == "ubuntu" ];then
		external_line=$(grep -cw "auto.*eth${internal}" $ubuntu_interface_dir/interfaces)
		if [ $external_line -eq 0 ];then

			if [ "$global_force" = "yes" ];then
				grep -vE "^#|^$" $ubuntu_interface_dir/interfaces | \
				grep -wq "gateway" || \
				echo -e "auto eth${internal}\niface eth${internal} inet static\n\taddress $add_ip\n\tnetmask $add_netmask\n\tgateway ${add_gateway}\n" >> $ubuntu_interface_dir/interfaces
				gw_tag='yes'
			else
				echo -e "auto eth${internal}\niface eth${internal} inet static\n\taddress $add_ip\n\tnetmask $add_netmask\n" >> $ubuntu_interface_dir/interfaces
			fi
			eth_tag="${internal}"
		else
			for max_external_line in `seq 1 30`
			do
				grep -qw "auto.*eth${internal}:${max_external_line}" $ubuntu_interface_dir/interfaces
				if [ $? -ne 0 ];then
					external_line=$max_external_line
					break;
				fi
			done

			echo -e "auto eth${internal}:${external_line}\niface eth${internal}:${external_line} inet static\n\taddress $add_ip\n\tnetmask $add_netmask\n" >> $ubuntu_interface_dir/interfaces
			eth_tag="${internal}:${external_line}"
		fi
	elif [ "$LINUX_VERSION" == "centos" ];then
		external_line=`ls $centos_interface_dir/ifcfg-eth* |grep -cw "ifcfg-eth${internal}"`
		if [ $external_line -eq 0 ];then
			echo -e "DEVICE=eth${internal}\nBOOTPROTO=static\nNM_CONTROLLED=yes\nONBOOT=yes" > $centos_interface_dir/ifcfg-eth${internal}
			echo -e "IPADDR=$add_ip\nNETMASK=$add_netmask" >> $centos_interface_dir/ifcfg-eth${internal}
			if [ "$global_force" = "yes" ];then
				grep -vE "^#|^$" $centos_interface_dir/ifcfg-eth* | grep -q 'GATEWAY' || \
				echo -e "GATEWAY=${add_gateway}" >> $centos_interface_dir/ifcfg-eth${internal}
				gw_tag='yes'
			fi
			eth_tag="${internal}"
		else
			for max_external_line in `seq 1 30`
			do
				ls $centos_interface_dir/ifcfg-eth* |grep -qw "ifcfg-eth${internal}:${max_external_line}"
				if [ $? -ne 0 ];then
					external_line=$max_external_line
					break;
				fi
			done

			echo -e "DEVICE=eth${internal}:${external_line}\nBOOTPROTO=static\nNM_CONTROLLED=yes\nONBOOT=yes" > $centos_interface_dir/ifcfg-eth${internal}:${external_line}
			echo -e "IPADDR=$add_ip\nNETMASK=$add_netmask\n" >> $centos_interface_dir/ifcfg-eth${internal}:${external_line}
			eth_tag="${internal}:${external_line}"
		fi
	fi

	if [ "$TAKE_EFFECT" = "yes" ];then
		add_auto="eth$eth_tag"
		ip addr list | grep -q "eth${internal}.*state DOWN" && ip link set eth${internal} up
		add_prefix=`cacle_prefix "$add_netmask"`
		v1="${add_gateway##*.}"
		v2="${add_netmask##*.}"
		v3=$(( 255-v2-1 ))
		v3=$(( v3+v1 ))
		add_ip_suffix="${add_ip%.*}.${v3}"
		ip addr add ${add_ip}/${add_prefix} broadcast ${add_ip_suffix} dev ${add_auto} label ${add_auto}

		if [ "$global_force" = "yes" ] && [ "$gw_tag" = "yes" ];then
			route add -net default gw ${add_gateway}
		fi
	fi
}

# 已存在的ip，但是掩码有问题，可以更正
function change_new_netmask () {
	change_ip=$1
	change_mask=$2
	change_gateway=$(/bin/bash /usr/local/i386/*/other/calc_gateway.sh $change_ip $change_mask|awk -F ':' '{print $2}')
	change_auto=`ip addr list | grep -w "${change_ip}.*global" |tail -n1| awk '{print $NF}'`

	if [ "$change_auto" != "" ];then
		if [ "$LINUX_VERSION" == "ubuntu" ];then
			sed -i "/auto.*${change_auto}$/,/netmask/s/"netmask.*"/netmask $change_mask/g" $ubuntu_interface_dir/interfaces
		elif [ "$LINUX_VERSION" == "centos" ];then
			sed -i "s/"NETMASK.*"/NETMASK=$change_mask/g" $centos_interface_dir/ifcfg-${change_auto}
		fi
	fi

	if [ "$TAKE_EFFECT" == "yes" ];then
		old_ip_prefix=`ip addr list | grep -w "${change_ip}.*global" | awk '{print $2}'`
		# 先增加，后删除
		change_prefix=`cacle_prefix "$change_mask"`
		change_ip_suffix=`echo ${change_ip%.*}.255`
		ip addr add ${change_ip}/${change_prefix} broadcast ${change_ip_suffix} dev ${change_auto} label ${change_auto}
		if [ $? -eq 0 ];then
			ip addr del ${old_ip_prefix} dev ${change_auto} label ${change_auto}
		fi
	fi
}

# 检查ip是否默认网关的ip
function del_old_ip () {
	del_ip=$1
	del_mask=$2
	del_gateway=$(/bin/bash /usr/local/i386/*/other/calc_gateway.sh $del_ip $del_mask|awk -F ':' '{print $2}')

	del_ip_prefix="${del_ip%\.*}"
	del_ip_prefix_line=$(ip -f inet addr list | grep -c "$del_ip_prefix")
	if [ "$LINUX_VERSION" == "ubuntu" ];then
		del_ip_prefix_line2=$(grep -c "address.*$del_ip_prefix" $ubuntu_interface_dir/interfaces)
	elif [ "$LINUX_VERSION" == "centos" ];then
		del_ip_prefix_line2=$(grep -c "IPADDR.*$del_ip_prefix" $centos_interface_dir/ifcfg-eth*)
	fi
	if [ "$del_ip_prefix_line" == "1" ] && \
	 [ "$del_ip_prefix_line2" == "1" ] && \
	 [ "$global_force" != "yes" ];then
		route -n | grep -q "^0.0.0.0.*$del_gateway"
		if [ "$?" -eq 0 ];then
			exit_tools_platform "code=4&&默认网关ip不能删除"
			return
		fi
	fi

	if [ "$LINUX_VERSION" == "ubuntu" ];then
		tmp_line=$(grep -vE "^$|^#" $ubuntu_interface_dir/interfaces | grep -E "auto|address" | grep -A1 "address.*$del_ip$" | tail -n1)
		echo $tmp_line | grep -q "auto eth" || tmp_line=""
		last_line=$(grep -v "^$" $ubuntu_interface_dir/interfaces| grep -B1 "$tmp_line" | head -n1)
		start_line=$(grep -vE "^$|^#" $ubuntu_interface_dir/interfaces | grep -E "auto|address" | grep -B1 "address.*$del_ip$" | head -n1)
		del_auto=$(echo $start_line | awk '{print $NF}')

		sed -i "/${start_line}$/,/$last_line/d" $ubuntu_interface_dir/interfaces
	elif [ "$LINUX_VERSION" == "centos" ];then
		del_auto=$(ip addr list | grep -w "${del_ip}.*global" | awk '{print $NF}')
		for del_file in `ls $centos_interface_dir/ifcfg-eth*`
		do
			grep -q "IPADDR=$del_ip" $del_file && rm -f $del_file
		done
	fi

	if [ "$TAKE_EFFECT" == "yes" ];then
		del_prefix=$(cacle_prefix "$del_mask")
		ip addr del ${del_ip}/${del_prefix} dev ${del_auto} label ${del_auto} 2>/dev/null
		if [ "$?" != "0" ];then
			del_line=`ip -f inet addr list | grep -cw "$del_ip.*global"`
			if [ "$del_line" == "1" ];then
				tmp_del_ip_mask=`ip -f inet addr list | grep -w "$del_ip.*global" | awk '{print $2}'`
				ip addr del ${tmp_del_ip_mask} dev ${del_auto} label ${del_auto} 2>/dev/null
			fi
		fi
		del_auto="${del_auto%:*}"
		ip -f inet addr show ${del_auto} | grep -q "inet.*global" || ip link set ${del_auto} down

		if [ "$global_force" = "yes" ];then
			route -n | grep -q "^0.0.0.0.*$del_gateway" && \
			route del -net default gw $del_gateway
		fi
	fi
}

function last_check () {
	last_ip=$1
	last_mask=$2
	last_file=$3

	param_interfaces_msg $last_ip $last_mask > $last_file

	LAST_NEW_IP=$(grep -E "address|IPADDR" $last_file| awk -F ' |=' '{for(i=NF;i>=1;i--){if($i != ""){print $i;exit}}}')
	LAST_NEW_MASK=$(grep -E "netmask|NETMASK" $last_file | awk -F ' |=' '{for(i=NF;i>=1;i--){if($i != ""){print $i;exit}}}')
}

# 参数 basename.sh [-f] [-i] options IP netmask gateway
# options: {add|del}|{addtool|deltool}

if [ "$1"  = "-f" ];then
	#强制支持默认网关的修改,不保证成功
	global_force="yes"
	shift
else
	global_force="no"
fi
if [ "$1" == "-i" ];then
	TAKE_EFFECT=yes
	TYPE=$2
	tmpTYPE=$TYPE
	echo $TYPE | grep -q "tool" && TYPE="${TYPE%t*}";
	IP=$3
	NETMASK=$4
	GATEWAY=$5
else
	TYPE=$1
	tmpTYPE=$TYPE
	echo $TYPE | grep -q "tool" && TYPE="${TYPE%t*}";
	IP=$2
	NETMASK=$3
	GATEWAY=$4
fi

echo $tmpTYPE | grep -q "tool" && platform_tools=yes

if [ $# -lt 4 ];then
	printmsg "basename.sh [-f] [-i] add|del IP netmask gateway"

	[[ "$platform_tools" == "yes" ]] && exit_tools_platform "参数个数不对 IP netmask gateway" || printmsg "参数个数不对"

	exit 1;
fi

TMP_FILE=`mktemp`
SUCCESS_ORNOT=1

# 检查interfaces是否带CRLF
file $ubuntu_interface_dir/interfaces | grep -q 'CRLF' && cat $ubuntu_interface_dir/interfaces | tr -d '\r' > $TMP_FILE && cat $TMP_FILE > $ubuntu_interface_dir/interfaces
param_interfaces_msg $IP $NETMASK > $TMP_FILE

# 如果IP 不存在，那么新增ip。如果IP存在，但是掩码不一致，修改配置
OLD_IP=$(grep -E "address|IPADDR" $TMP_FILE| awk -F ' |=' '{for(i=NF;i>=1;i--){if($i != ""){print $i;exit}}}')
OLD_MASK=$(grep -E "netmask|NETMASK" $TMP_FILE| awk -F ' |=' '{for(i=NF;i>=1;i--){if($i != ""){print $i;exit}}}')

[[ "$LINUX_VERSION" == "centos" ]] && pkill -9 -f NetworkManager

if [ "$TYPE" == "add" ];then
	if [ "$OLD_IP" == "" ];then
		add_new_ip $IP $NETMASK $GATEWAY
		last_check $IP $NETMASK $TMP_FILE
		if [ "$LAST_NEW_IP" == "$IP" ];then
			[[ "$platform_tools" == "yes" ]] && exit_tools_platform "成功添加$IP" || printmsg "成功添加$IP"
			SUCCESS_ORNOT=0
		fi
	elif [[ "$OLD_IP" == "$IP" ]] && [[ "$OLD_MASK" != "$NETMASK" ]];then
		change_new_netmask $IP $NETMASK $GATEWAY
		last_check $IP $NETMASK $TMP_FILE
		if [[ "$LAST_NEW_IP" == "$IP" ]] && [[ "$LAST_NEW_MASK" == "$NETMASK" ]];then
			[[ "$platform_tools" == "yes" ]] && exit_tools_platform "成功修改掩码" || printmsg "成功修改掩码"
			SUCCESS_ORNOT=0
		fi
	else
		[[ "$platform_tools" == "yes" ]] && exit_tools_platform "相关ip掩码已存在" || printmsg "相关ip掩码已存在"
		SUCCESS_ORNOT=1
	fi
elif [ "$TYPE" == "del" ];then
	if [ "$IP" == "$OLD_IP" ];then
		del_old_ip $IP $NETMASK $GATEWAY
		last_check $IP $NETMASK $TMP_FILE
		if [ "$LAST_NEW_IP" == "" ];then
			[[ "$platform_tools" == "yes" ]] && exit_tools_platform "成功删除$IP" || printmsg "成功删除$IP"
			SUCCESS_ORNOT=0
		fi
	else
		[[ "$platform_tools" == "yes" ]] && exit_tools_platform "ip掩码原来已经不存在" || printmsg "ip掩码原来已经不存在"
		SUCCESS_ORNOT=1
	fi

fi

rm -f $TMP_FILE
exit 0

