#!/bin/bash
# NAME            : $0 
# Ver             : 8.0
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2014-07-06 16:47
# Summary         : check proc not running & coredump , and auto-call monitor.sh/res_report.sh .
# Changelog       : from ver8.0 , no regular cron entries will be installed for each pkg (monitor/clear/md5sum/report) . this script take all the places 

# crontab control area , quit when finished

sleep 3

set +H

export LANG=C

export LC_ALL=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin


#----------------#
# load variables #
#----------------#

source /data/pkg/public-scripts/conf/check_proc_not_running.conf

#----------------#
# load functions # 
#----------------#

source $binDir/mypidof.sh

source $funcDir/get_available_fd.sh

source $funcDir/common.sh

source $funcDir/common-log.sh

source $funcDir/common-hosts.sh

source $funcDir/misc.sh

source $funcDir/check_proc_not_running_comm.sh


(
    flock --nonblock 200 || exit 1

    {
        # s2skill for invalid pids

        if [ "$ENABLE_S2S_KILLER" = "yes" ] ; then 

            ( s2skill $(find_s2s_pidFiles) ) &

			( s2skill_for_stale ) &
        fi

        if [ $(sys_uptime) -lt $NO_CHECK_IF_JUST_BOOTUP ] ; then

             wlog "system uptime is $(sys_uptime) seconds (<$NO_CHECK_IF_JUST_BOOTUP) , just quit"
        
             dlog "--end--"

             exit
        else
             log "system uptime is $(sys_uptime) seconds (>$NO_CHECK_IF_JUST_BOOTUP) , continue"   
        fi

        update_today_link
    
        rotate_log $logFile $maxLogSize
    
        clear_old_alarm_record
    
        dlog "$(cat $hostInfoFile)"
    
        dlog ""
      
        #quit_if_vhost || exit 1
    
        #quit_if_operating_host || exit 1
    
        # update cmdb procinfo cache file if needed
    
        if [ ! -e $cmdbProcCacheFile ] || [ $(date +%H%M) == "0000" ] || instance_status_changed ; then
    
              dlog "need to update cmdb cache file"

              localDns=$(get_local_dns)

              if [ -n "$localDns" ] ; then           
    
                   backup_cmdb_pinfo_file 
    
                   download_cmdb_pinfo || restore_cmdb_pinfo_file
          
                   show_cmdb_pinfo
    
                   cmdb_pinfo_format
              else
                   elog "no local dns found , can NOT update cmdb proc info , use cache if exist !!!"
              fi
        else
              dlog "cmdb cache file exist , or not 00:00 , or no instance status changed , no cmdb query needed"
        fi
    
        #----------------------------------#
        # collect all local proc.info file #
        #----------------------------------#
    
        #gen_local_pinfo_cache_file

        gen_complete_local_pinfo_cache 

        if [ ! -s $localProcCacheFile ] ; then

             dlog "$localProcCacheFile is empty , no proc need to check , quit"

             dlog ""

             dlog "--end--"

             exit
        fi
    
        #--------------------------------------------------#
        # open fd for procinfo file used by 'read' command #
        #--------------------------------------------------#
    
        tmpFileFD=$(get_available_fd)
    
        dlog "found available fd : [$tmpFileFD] , open $localProcCacheFile for reading ...."
    
        dlog "$(cat -A $localProcCacheFile)"
    
        eval "exec $tmpFileFD<$localProcCacheFile"
    
        #-----------------#
        # main work begin #
        #-----------------#
    
        while read -u $tmpFileFD line ; do
    
            curNum=0 coreFlag=0
    
            # get amount,package_name,.... variables
    
            eval $(echo "$line" | sed 's/,/;/g;s/:/=/g')

            # skip when proc name is 'null' or 'NULL'

            if [ "$(echo "$proc" | tr '[[:upper:]]' '[[:lower:]]')" = "null" ] ; then

                 wlog "package [$package_name]'s proc name is *NULL* , maybe is a script package"

                 wlog "no need to check , skipt it"

                 wlog ""

                 continue
            fi 

            binaryPath="${path}/${proc}"
    
            binaryPath=$(echo $binaryPath | sed -r 's/\/+/\//g')
    
            initFile="$path/../init.conf"
    
            commonVarFile="$path/../admin/common-var.conf"
    
            monitorScript="$path/../admin/monitor.sh"
    
            monitorScriptLog="$path/../admin/monitor.log"

            resReportScript="$path/../admin/report.sh"

            resReportScriptLog="$path/../admin/report.log"
    
            user=$(grep '^user=' $initFile 2>/dev/null |  awk -F '[="]+' '{print $2}')
    
            pname=$(grep '^pname=' $initFile 2>/dev/null |  awk -F '[="]+' '{print $2}')
    
            autoStart=$(grep 'export AUTO_START=' $commonVarFile 2>/dev/null |  awk -F '[="]+' '{print $2}')

            needResReport=$(grep 'export NEED_RESOURCE_REPORT=' $commonVarFile 2>/dev/null |  awk -F '[="]+' '{print $2}')

            : ${needResReport:=yes}
    
            : ${autoStart:=unknown}
    
            instanceStatusFile=$(echo "$binaryPath" | egrep -o '\/data\/services/[^/]+' | sed 's#$#/admin/instance.status#')
    
            instanceStatus=$(awk '{print $NF}' $instanceStatusFile 2>/dev/null)
    
            noAutoStartFlagFile=$(echo "$binaryPath" | egrep -o '\/data\/services/[^/]+' | sed 's#$#/admin/.no-autostart#')
    
            #-----------------------------#   
            # get process_level from cmdb #
            #-----------------------------#   
    
            #process_level=$(get_proc_level "$version_id")

            #if [ -z "$process_level" ] ; then

            #      dlog "process level for $proc is NULL , default to 3(normal)"

            #      process_level=3
            #fi
    
            dlog ">>> begin <<<"
    
            dlog "proc=[$proc]"

            dlog "path=[$binaryPath]"

            dlog "amount=[$amount]"
    
            #----------------------------------# 
            # check if a test or debug process #
            #----------------------------------#
    
            if is_test_debug_proc $proc ; then
    
                 wlog "$proc is a test or debug process , skip it"
    
                 dlog "<<< finished >>>"
    
                 continue
            fi
    
            #------------------------------------#
            # first , check if coredump happened #
            #------------------------------------#
    
            if  echo "$CORE_PROGS" | grep -q -x $proc ; then
    
                 coreFlag="1"

                 dlog "$CORE_FILES"
            else
                 coreFlag="0"
            fi
    
            #-----------------------------------------------------------#
            # try to find proc through /proc/pid/exe , limit to $amount #
            #-----------------------------------------------------------#
    
            pidExeList=$(find_proc_proc_exe "$proc" | head -n $amount)

            dlog "${pidExeList:-<empty>}"
    
            curNum=$(var_line_count "$pidExeList")

            #----------------------------------------------------------------------#
            # if can NOT find through /proc/pid/exe , try pidof , limit to $amount #
            #----------------------------------------------------------------------#
    
            if [ $curNum -eq 0 ] ; then
    
               pidExeList=$(find_proc_pidof "$proc" | head -n $amount)
            
               dlog "${pidExeList:-[INFO] | <empty>}"
             
               curNum=$(var_line_count "$pidExeList")
            fi
    
            #-----------------------------------------------------#
            # if pidof could NOT find , then try /proc/pid/status #
            #-----------------------------------------------------#
    
            if [ $curNum -eq 0 ] ; then
    
               pidExeList=$(find_proc_proc_status "$proc" | head -n $amount)
            
               dlog "${pidExeList:-[INFO] | <empty>}"
            
               curNum=$(var_line_count "$pidExeList")
            fi 
    
    
            #-----------------------------#
            # maybe it's a garbage record #
            #-----------------------------#
    
            if [ $curNum -lt $amount ] ; then
            
                  if binary_not_exist $binaryPath  ; then

                       continue  # garbage record , maybe  updated | uninstalled | rollbacked
    
                  elif instance_stopped "$binaryPath" "$instanceStatus" ; then
    
                       continue  # garbage record , maybe stopped
    
                  elif garbage_record "$binaryPath" "$noAutoStartFlagFile"; then
    
                       continue # record will be delete soon , just ignore
                  else
                       elog "[$binaryPath] not running detected"
                  fi
            else
                  # find some $proc
     
                  dlog "[$path/$proc] final count set to $curNum"
            fi
    
            #--------------------------------------------#
            # record if coredump or not running happened #
            #--------------------------------------------#
    
            if [ $curNum -lt $amount ] || [ "$coreFlag" = "1" ] ; then
    
                 lessNum=$((10#$amount - 10#$curNum))
    
                 elog "proc=$proc coredump=$coreFlag curNum=$curNum (<$amount)"
    
                 #-------------------------------------#
                 # determine if need to run monitor.sh #
                 #-------------------------------------#
    
                 if  [ $autoStart != "yes" ] ; then
    
                      # if autostart=no , send alarm immediately , no matter what levels
    
                      print_log "[INFO] | autostart = no , send alarm right now"
    
                      alarmStr="10310 75421 0 ${process_level} 2 ${version_id} $autoStart"
    
                      alarmStr="$alarmStr $amount $lessNum $curNum n/a $proc $path"
    
                      alarmStr="$alarmStr $coreFlag $pname"
    
                      not_run_alarm "$alarmStr"
    
                      log_not_running "$todayAlarmFile" "$proc" "$process_level" "$path" "$coreFlag" "$amount" "$curNum" "$lessNum" "$package_name" "$version" "$pname"
    
                      # no monitor.sh will be runned
    
                 else 
    
                      if [ $amount -gt 0 ] ; then
              
                           # if record min num in CMDB > 0 , then send alarm
              
                           dlog "send alarm !!!"
              
                           not_run_alarm 10310 75421 0 "${process_level}" "2" "${version_id}" "$autoStart" "$amount" "$lessNum" "$curNum" "n/a" "$proc" "$path" "$coreFlag" "$pname"
                           log_not_running "$todayAlarmFile" "$proc" "$process_level" "$path" "$coreFlag" "$amount" "$curNum" "$lessNum" "$package_name" "$version" "$pname"
                      else 
                           # if record min num in CMDB is 0 , then no alarm
             
                           dlog "CMDB record proc num=0 , NO alarm"
                      fi

                      if [ "$ENABLE_CALL_MONITOR_SCRIPT" = "yes" ] ; then

                           dlog "--- Call Monitor Script ENABLED ---"

                           #------------------------------------------------------------------------------#
                           # call monitor.sh , we DON'T have to check the .no-autostart flag file .       #  
                           # Because if it exists , the code above will just ignore this pkg and continue #
                           #------------------------------------------------------------------------------#
        
                           su - $user -c "nohup /bin/bash $monitorScript > $monitorScriptLog 2>&1 &"
        
                           log_call_monitor "$todayAlarmFile" "$proc" "$monitorScript" 
                      else
                           dlog "--- Call Monitor Script DISABLED ---"
                      fi
                 fi
         
            elif [ "$ENABLE_UNEXPECT_RESTART_CHECK" = "no" ] ; then

                 nlog ""
 
                 nlog "--- Unexpected Restart Check DISABLED ---"

            elif [ "$ENABLE_UNEXPECT_RESTART_CHECK" = "yes" ] ; then
 
                  nlog "--- Unexpected Restart Check ENABLED ---"

                  sysUpTime=$(awk '{printf "%.0f\n",$1}' /proc/uptime)

                  if [ $sysUpTime -gt 900 ] ; then

                      dlog "system uptime is $sysUpTime seconds (>900) , begin unexpect restart check ...."
    
                      dlog "proc=$proc curNum=$curNum (>=$amount)"   
        
                      dlog "still check proc start time"
        
                      #-------------------------------------------------#
                      # even proc num is right , still check start time #
                      #-------------------------------------------------#
                      
                      check_proc_startTime "$pidExeList" "$proc" "$path" "$process_level"
        
                      #------------------------------------------------#
                      # check unexpected restart times with 10 minutes #
                      #------------------------------------------------#
        
                      startTs=$(date -d '10 minutes ago' +%s)
        
                      unexpRestartLog=$(get_unexp_restart_log $startTs $todayAlarmFile "path=$path")
        
                      unexpRestartTimes=$(var_line_count "$unexpRestartLog")
        
                      if [ $unexpRestartTimes -ge 2 ] ; then
        
                           wlog "too many unexpected restart in 10 minutes [$unexpRestartTimes] times since $(date -d@$startTs +'%F %T')"
        
                           # too_many_restart_alarm 10310 75421 0 "$process_level" 2 "$version_id" "$package_name-$version restart too many times [$unexpRestartTimes in 10-minutes]"
                      
                           dlog "$unexpRestartLog"
    
                      elif [ $unexpRestartTimes -ge 1 ] ; then
    
                           dlog "only $unexpRestartTimes times unexpected restart in 10 minutes [ $(date -d@$startTs +'%F %T') ~ $(date +'%F %T') ]"
    
                           dlog "restart-too-many alarm threshold not reached"
    
                           dlog "$unexpRestartLog"
                      else
                           dlog "no unexpected restart in 10 minutes [ $(date -d@$startTs +'%F %T') ~ $(date +'%F %T') ]"
                      fi    
                 else
                      dlog "system uptime is $sysUpTime (<900) , skip unexpect restart check"
                 fi
            fi

            #------------------------------------------------------------#
            # call resource report script for current pkg-ver if enabled #
            #------------------------------------------------------------#

            if [ "$ENABLE_CALL_RES_REPORT_SCRIPT" = "yes" ] ; then

                dlog "--- Call Resource Report ENABLED ---"

                if [ "$needResReport" = "yes" ] ; then
    
                     dlog "call resource report script for pkg [$package_name] : $resReportScript"
    
                     su - $user -c "nohup /bin/bash $resReportScript >$resReportScriptLog 2>&1 &"
                else
                     dlog "NEED_RESOURCE_REPORT != yes , no cpu/mem report"
                fi

            else
                 dlog "--- Call Resource Report DISABLED ---"
            fi
    
            dlog "<<< finished >>>"

            dlog ""
    
        done
    
        # close fd
    
        eval "exec $tmpFileFD<&-"
    
        rm -f $tmpFile
        
        dlog "--end--"
    
        date +%s > $lastRunTsFile 
    
    } >>$logFile 2>&1

) 200>/data/pkg/public-scripts/data/.check_proc_not_running.lock && \

rm /data/pkg/public-scripts/data/.check_proc_not_running.lock
