#!/bin/bash
# NAME            : $0 
# Ver             : 2.4.3
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2014-12-24
# Summary         : DuoWan pkg release & cfg managerment system background scripts 
# Changelog       : report cfgId on install\update\rollback\reload (2.4.3) 

#----------------------------#
# Set Basic Environment Zone #
#----------------------------#

set +H

MYPID=$$

export LANG=C

export LC_ALL=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

export PROG="$(basename $0)"

prog="$(basename $0)"

binDir="$(cd $(dirname $0) && pwd -P)"

confDir="$binDir/../conf"

logDir="$binDir/../log"

chmod -c a+x $binDir/*.sh


#----------------------#
# Little Function Zone #
#----------------------#

function create_log_file {

   local files=$*

   local logDir i

   for i in $files ; do

       logDir="$(dirname $i)"

       if [ ! -e $logDir ] ; then
    
            if ! mkdir -p $logDir &>/dev/null ; then
    
               msg=$(err_log "mkdir $logDir failed" 2>&1)
    
               echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE 
    
               ( do_report "${_TASK_ID}" "${_SESSION_ID}" "$CODE_TASK_LOG_MKDIR_FAIL" "$msg" "$msg" "${_VERSION_ID}" "" )
    
               return $CODE_TASK_LOG_MKDIR_FAIL

            fi
    
       elif [ -f $logDir ] ; then
    
            msg=$(err_log "script log dir $logDir is a file" 2>&1)
    
            echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE 
    
            ( do_report "${_TASK_ID}" "${_SESSION_ID}" "$CODE_TASK_LOG_MKDIR_FAIL" "$msg" "$msg" "${_VERSION_ID}" "" )
    
            return $CODE_TASK_LOG_MKDIR_FAIL
       fi

       # enforce to remove file if already exists
    
       rm -f $i &>/dev/null
    
       cp /dev/null $i ; chmod 664 $i

   done

   return 0
}

function make_basic_dir {

   local dirs="$*"

   local i

   for i in $dirs ; do 
 
       if [ ! -e $i ] ; then
    
            if ! mkdir -p $i &>/dev/null ; then
    
                 msg=$(err_log "create dir $i failed" 2>&1)
    
                 echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE
    
                 ( do_report "${_TASK_ID}" "${_SESSION_ID}" "$CODE_MKDIR_FAIL" "$msg" "$msg" "${_VERSION_ID}" "" )
    
                 return $CODE_MKDIR_FAIL
            fi

       elif [ -d $i ] ; then

            continue
       else
            msg=$(err_log "$i alreay exist , but not a dir" 2>&1)

            echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE

            ( do_report "${_TASK_ID}" "${_SESSION_ID}" "$CODE_MKDIR_FAIL" "$msg" "$msg" "${_VERSION_ID}" "" )

            return $CODE_MKDIR_FAIL
       fi

   done

   return 0
}


function exit_on_err {

   local code=${1:?$prog Err! code MUST be gived}

   local msg=${2:?$prog Err! msg MUST be gived}

   echo "$(date +'%F %T')| $PROG [ERR ] | $msg"
   
   exit $code 
}


#-------------------------------------------#
# Load Main Config && Common Functions Zone #
#-------------------------------------------#

configList="$confDir/main.conf $confDir/cmdb.conf $confDir/auto-test.conf"

for config in $configList ; do

    if ! source "$config" 2>>$logDir/err.log ; then
    
         exit_on_err 112 "load $config failed"
    fi

done


for loadFile in $LOAD_SCRIPT_LIST ; do
         
    if ! source "$loadFile" 2>>$logDir/err.log ; then
         
         exit_on_err "$CODE_LOAD_FAIL" "load $loadFile failed !"
    fi
         
done 


#--------------------#
# User ID Check Zone #
#--------------------#

if ! ( check_uid 0 ) ; then
 
   exit_on_err "$CODE_NOT_ROOT" "only root can run this script !"
fi


#----------------------------------------#
# Result Report Python Script Check Zone #
#----------------------------------------#

if ! (is_readable_file $TASK_REPORT_SCRIPT) ; then

   exit_on_err "$CODE_NO_RPT_PY" "no $TASK_REPORT_SCRIPT found !"

fi


#-----------------------------------------------------------------#
# Scan the BASH_ARGV array and Set Task-releated Environment VARs # 
#-----------------------------------------------------------------#

for ((i=0;i<${#BASH_ARGV[*]};i++)); do

    if echo "${BASH_ARGV[$i]}" | egrep -q '[[:space:]]' ; then

         msg=$(err_log "\"${BASH_ARGV[$i]}\" contain space character" 2>&1)

         echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE

         # no "do_report" function will be called !!!

         exit_on_err "$CODE_PARA_CONTAIN_SPACE" "$msg" 

    elif ! echo "${BASH_ARGV[$i]}" | egrep -q '^[^ =]+=[^ =]+$' ; then

         msg=$(err_log "invalid format \"${BASH_ARGV[$i]}\" (item=value)" 2>&1)

         echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE

         # no "do_report" function will be called !!!

         exit_on_err "$CODE_PARA_INVALID_FMT" "$msg"
    
    elif ! eval "${BASH_ARGV[$i]}" 2>/dev/null ; then

         msg=$(err_log "${BASH_ARGV[$i]} eval error" 2>&1)

         echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE

         # no "do_report" function will be called !!!

         exit_on_err "$CODE_PARA_EVAL_ERR" "$msg"
    fi

done

#---------------------------------------#
# Export Task-releated Environment VARs #
#---------------------------------------#

declare -r -x _TASK_ID="$taskId"

declare -r -x _SESSION_ID="$sessionId"

declare -r -x _VERSION_ID="$versionId"

declare -r -x _GLOBAL_ID="${taskId}/${sessionId}/${versionId}"

declare -r -x _ACTION="$action"

declare -r -x ACTION_MODE=$action

true ${curVer:=$ver}    # if action=install|start|stop|restart|uninstall|reload

true ${curVer:=$oldVer} # if action=update

true ${curVer:=$srcVer} # if action=rollback



#-------------------#
# Create Pulic Dirs #
#-------------------#

( make_basic_dir $PKG_HOME_DIR $PKG_BACKUP_DIR $PKG_DOWNLOAD_DIR ) || exit $?

( make_basic_dir $PKG_DOWNLOAD_LOG_DIR $PKG_LOG_DIR $PKG_DATA_DIR ) || exit $?

( make_basic_dir $PKG_BAD_DIR $PKG_COMMON_CONF_DIR $PKG_COMMON_LIBS_DIR ) || exit $?

( make_basic_dir $PUB_SCRIPT_LOCK_DIR ) || exit $?



#------------------#
# Task Record Zone # 
#------------------#

taskMgrLog="${PUB_SCRIPT_TASK_MGR_LOG}.$pkg-$curVer"

taskStr="new task : pid $MYPID , taskId=$taskId , sid=$sessionId , verId=$versionId , pkg=$pkg , ver=$curVer"

info_log "$taskStr" >> $taskMgrLog

taskMgrLockFile=$PUB_SCRIPT_LOCK_DIR/$pkg-$curVer.lock

{

    # only one single task for each pkg-ver is allowed

    flock -n 200 || { err_log "MYPID=$MYPID Can't get lock on $taskMgrLockFile , I quit !!!" >> $taskMgrLog 2>&1 ; exit 101 ; }

	info_log "MYPID=$MYPID get lock on $taskMgrLockFile" >> $taskMgrLog
	
    #-----------------#
    # Task Clean Zone # 
    #-----------------#

    taskFile="${PUB_SCRIPT_TASK_FILE}.$pkg-$curVer"
    
    ( task_clean $taskFile $taskMgrLog )
    
    
    #---------------------------#
    # Duplicate Task Check Zone #
    #---------------------------#
    
    sleep $((10#$RANDOM % 5))
    
    pid=$(is_a_new_task "$taskFile" "$action" "$taskId" "$sessionId" "$versionId")
    
    rc=$?
    
    if [ $rc -eq 1 ] ; then
    
         msg=$(err_log "function \"is_a_new_task\" syntax err" 2>&1)
    
         echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE
    
         ( do_report "$taskId" "$sessionId" "$CODE_SYNTAX_ERR" "$msg" "$msg" "$versionId" "" )
    
         exit $CODE_SYNTAX_ERR
    
    elif [ $rc -eq 2 ] ; then
    
         {
           err_log "another same task ($taskId) , same session ($sessionId) , same version ($versionId) is running , and NOT zombies"
    
           err_log "!!! --- maybe the python channel send command twice --- !!!"
    
           err_log "[action=$action taskId=$taskId sessionId=$sessionId versionId=$versionId]" 
    
           err_log "the PID is $pid"
      
           err_log "$(ps --no-headers -o pid,lstart,cmd $pid)"
      
           err_log "duplicated task canceled , quit now"
      
           err_log "shell script [pid=$MYPID] terminated"
       
           err_log "!!! --- do __NOT__ report to web , just quit --- !!!"
    
         } 2>&1 | tee -a $PUB_SCRIPT_ERR_LOG_FILE $taskMgrLog
    
         # NOTE !!!! do NOT report when a duplicate task ,
    
         # because network problem may cause the directive to send more than 1 times ,
    
         # I just record this problem and quite silentlly
    
         exit $CODE_DUP_TASK
    
    elif [ $rc -eq 3 ] ; then
    
         {
           err_log "found log dir $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID} exist !!!"
    
           err_log "!!! --- the web front-end had send this task before --- !!!"
    
           while read line ; do
    
               err_log "$line"
    
           done < <(ls -l $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}) 
    
           err_log "duplicated task canceled , quit now"
    
           err_log "shell script [pid=$MYPID] terminated"
    
           err_log "!!! --- do __NOT__ report to web , just quit --- !!!"
    
         } 2>&1 | tee -a $PUB_SCRIPT_ERR_LOG_FILE $taskMgrLog
    
         # NOTE !!!! do NOT report when a duplicate task , 
    
         # because network problem may cause the directive to send more than 1 times ,
    
         # I just record this problem and quite silentlly
    
         exit $CODE_DUP_TASK
    fi
    
    
    #--------------------------#
    # Conflict Task Check Zone #
    #--------------------------#
    
    # Conflict task means that when a instance is doing a action , 
    
    # another task (different task , different session , same version) is arrived .
    
    # because a single instance can NOT do different task at the same time , 
    
    # so the new arrived task will be canceled .
    
    # ugly sleep 
    
    sleep $((10#$RANDOM % 5))
    
    pid=$(is_a_conflict_task "$taskFile" "$action" "$taskId" "$sessionId" "$pkg" "$curVer")
    
    rc=$?
    
    if [ $rc -eq 1 ] ; then
    
         msg=$(err_log "function \"is_a_conflict_task\" syntax err" 2>&1)
    
         echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE
    
         ( do_report "$taskId" "$sessionId" "$CODE_SYNTAX_ERR" "$msg" "$msg" "$versionId" "" )
    
         exit $CODE_SYNTAX_ERR
    
    elif [ $rc -eq 2 ] ; then
    
         {
            err_log "!!! --- another task for [$pkg-$curVer] is running --- !!!"
    
            err_log "only __1__ task is allowed at a time"
       
            err_log "the PID is $pid"
       
            err_log "N/A" "$(ps --no-headers -o pid,lstart,cmd $pid)"
       
            err_log "conflict task canceled , quit now"
       
            err_log "shell script [pid=$MYPID] terminated"
       
            err_log "!!! --- do __NOT__ report to web , just quit --- !!!"
    
         } 2>&1 | tee -a $PUB_SCRIPT_ERR_LOG_FILE $taskMgrLog
    
         exit $CODE_CONFLICT_TASK
    fi
    
    
    #---------------------------------#
    # Task Check Pass , Begin to Work #
    #---------------------------------#
    
    ( add_task_to_taskFile $MYPID $taskFile "action=$action" "taskId=$taskId" "sessionId=$sessionId" "pkg=$pkg" "ver=$curVer" "versionId=$versionId" )
    
    info_log "add task [taskId=$taskId sessionId=$sessionId verId=$versionId]" >> $taskMgrLog 

} 200>$taskMgrLockFile

rc=$?

if [ $rc -ne 0 ] ; then

   exit $rc 
fi

#------------------------#
# Task Log Creation Zone # 
#------------------------#

logFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_LOG_FILE"

briefLog="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_BRIEF_LOG"

rptLog="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_REPORT_LOG"

appLog="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_APP_LOG"

( create_log_file "$logFile" "$briefLog" "$rptLog" ) || exit $?

declare -x -r _LOG_FILE="$logFile"

declare -x -r _BRIEF_LOG="$briefLog"

declare -x -r _RPT_LOG="$rptLog"

declare -x -r _APP_LOG="$appLog"


#--------------------------#
# Main Log Start Line Zone #
#--------------------------#

echo "$0 $*" | sed -r 's/[^ ]+ /&\\\n/g' > $logFile

#----------------------#
# Paramater check Zone #
#----------------------#

function chk_para_ok {

   local para=$1 value=$2

   case $para in

        'taskId')     ( chk_taskId     "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'sessionId')  ( chk_sid        "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'versionId')  ( chk_verId      "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'pkg')        ( chk_pkg        "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'ver')        ( chk_ver        "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'md5')        ( chk_md5        "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'cfgmd5')     ( chk_md5        "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'urls')       ( chk_urls       "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'cfgpkgurls') ( chk_urls       "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'order')      ( chk_order      "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'startApp')   ( chk_startApp   "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'copyCfg')    ( chk_copyCfg    "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'rate')       ( chk_rate       "$value"  >>${_LOG_FILE} 2>&1 ) ;;

        'platform')   ( chk_platform   "$value"  >>${_LOG_FILE} 2>&1 ) ;;

   esac

   if [ $? -ne 0 ] ; then

        msg=$(err_log "invalid $para [$value]" 2>&1)

        echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE

        ( do_report "${_TASK_ID}" "${_SESSION_ID}" "$CODE_PARA_ERR" "$msg" "$(cat $logFile)" "${_VERSION_ID}" "" )

        return $CODE_PARA_ERR
   else
        return 0
   fi
}

banner="$(repeatStr '-' 19) script file info $(repeatStr '-' 19)"

info_log "$banner" >> $logFile

info_log "script path=$0" >> $logFile

info_log "dw_linbobo (linbobo@yy.com)" >> $logFile

info_log "" >> $logFile




#-----------------------------------------#
# Print OS-releated Environment Info Zone #
#-----------------------------------------#

banner="$(repeatStr '-' 19) OS releated info $(repeatStr '-' 19)"

info_log "$banner" >> $logFile

os=$(lsb_release -d | sed -r 's/^Description:[[:space:]]+//')

true ${os:=$(cat /etc/issue | head -n1)}

{
    info_log "OS = [$os]"

    info_log "MEM = [$(head -n1 /proc/meminfo | awk '{printf "%.0f\n",$2/1024/1024}')GB]"

    info_log "MY PID = [$MYPID]"

    info_log "LOCAL IP = [$(get_local_ip "eth0")]"
    
    info_log "PLATFORM = [$(getconf LONG_BIT)-bit]"
    
} >> $logFile



#------------------------------#
# Paramaters Sanity Check Zone #
#------------------------------#

banner="$(repeatStr '-' 19) parameters check $(repeatStr '-' 19)"

info_log "" >> $logFile

info_log "$banner" >> $logFile

case $action in 

 download) 

    {
      ( chk_para_ok "taskId" "$taskId" ) && \
    
      ( chk_para_ok "sessionId" "$sessionId" ) && \
    
      ( chk_para_ok "versionId" "$versionId" ) && \
    
      ( chk_para_ok "pkg" "$pkg" ) && \
    
      ( chk_para_ok "ver" "$ver" ) && \
    
      ( chk_para_ok "md5" "$md5" ) && \
    
      ( chk_para_ok "url" "$urls" ) && \
    
      ( chk_para_ok "rate" "$rate" ) && \
    
      ( chk_para_ok "platform" "$platform" )

   }  || exit $?

   ;;

 install)

   {
      ( chk_para_ok "taskId" "$taskId" ) && \
    
      ( chk_para_ok "sessionId" "$sessionId" ) && \
    
      ( chk_para_ok "versionId" "$versionId" )  && \
    
      ( chk_para_ok "pkg" "$pkg" ) && \
    
      ( chk_para_ok "ver" "$ver" ) && \
    
      ( chk_para_ok "md5" "$md5" ) && \
    
      ( chk_para_ok "urls" "$urls" ) && \
    
      ( chk_para_ok "startApp" "$startApp" ) && \
    
      ( chk_para_ok "rate" "$rate" ) && \
    
      ( chk_para_ok "platform" "$platform" ) && \
    
      ( chk_para_ok "cfgmd5" "$cfgmd5" ) && \
    
      ( chk_para_ok "cfgpkgurls" "$cfgpkgurls" )

   } || exit $?

   ;;


 reload)

   {
      ( chk_para_ok "taskId" "$taskId" ) && \
    
      ( chk_para_ok "sessionId" "$sessionId" ) && \
    
      ( chk_para_ok "versionId" "$versionId" ) && \
    
      ( chk_para_ok "pkg" "$pkg" ) && \
    
      ( chk_para_ok "ver" "$ver" ) && \
    
      ( chk_para_ok "rate" "$rate" ) && \
    
      ( chk_para_ok "cfgmd5" "$cfgmd5" ) && \
    
      ( chk_para_ok "cfgpkgurls" "$cfgpkgurls" )

   } || exit $?

   ;;


 uninstall|start|stop|restart)

   {
      ( chk_para_ok "taskId" "$taskId" ) && \
    
      ( chk_para_ok "sessionId" "$sessionId" ) && \
    
      ( chk_para_ok "versionId" "$versionId" ) && \
    
      ( chk_para_ok "pkg" "$pkg" ) && \
    
      ( chk_para_ok "ver" "$ver" )

   } || exit $?

   ;;

 rollback)
  
   {
      ( chk_para_ok "taskId" "$taskId" ) && \
    
      ( chk_para_ok "sessionId" "$sessionId" ) && \
    
      ( chk_para_ok "versionId" "$versionId" ) && \
    
      ( chk_para_ok "pkg" "$pkg" ) && \
    
      ( chk_para_ok "srcVer" "$srcVer" ) && \
    
      ( chk_para_ok "dstVer" "$dstVer" )

   } || exit $?

   ;;

 update)

   {
      ( chk_para_ok "taskId" "$taskId" ) && \
    
      ( chk_para_ok "sessionId" "$sessionId" ) && \
    
      ( chk_para_ok "versionId" "$versionId" ) && \
    
      ( chk_para_ok "pkg" "$pkg" ) && \
    
      ( chk_para_ok "oldVer" "$oldVer" ) && \
    
      ( chk_para_ok "newVer" "$newVer" ) && \
    
      ( chk_para_ok "md5" "$md5" ) && \
    
      ( chk_para_ok "urls" "$urls" ) && \
    
      ( chk_para_ok "copyCfg" "$copyCfg" ) && \
    
      ( chk_para_ok "startApp" "$startApp" ) && \
    
      ( chk_para_ok "order" "$order" ) && \
    
      ( chk_para_ok "rate" "$rate" ) && \
    
      ( chk_para_ok "platform" "$platform" ) && \
    
      ( chk_para_ok "cfgmd5" "$cfgmd5" ) && \
    
      ( chk_para_ok "cfgpkgurls" "$cfgpkgurls" )

   } || exit $?

   ;;

esac

{
    info_log "paramaters pre-check is passed"
    
    info_log "logFile  = [$logFile]"
    
    info_log "briefLog  = [$briefLog]"
    
    info_log "rptLog   = [$rptLog]"
    
} >> $logFile


#-----------------------------------------------------#
# Set DO_CFG_MGR According To The cfgmd5 & cfgpkgurls #
#-----------------------------------------------------#

if [ -n "$cfgmd5" ] && [ -n "$cfgpkgurls" ] ; then

     if [ "$cfgmd5" = 0 ] && [ "$cfgpkgurls" = "http://" ] ; then
     
          info_log "set DO_CFG_MGR=no" >> $logFile
     
          declare -r -x DO_CFG_MGR="no"
     else
          info_log "set DO_CFG_MGR=yes" >> $logFile
     
          declare -r -x DO_CFG_MGR="yes"
     
          # re-set the cfgpkgurls , add '?type=config' suffix
     
          cfgpkgurls=$(add_url_suffix "$cfgpkgurls")
     fi
else
     info_log "set DO_CFG_MGR=no" >> $logFile

     declare -r -x DO_CFG_MGR="no"
fi


#-----------------------------------#
# Call Actural Worker Function Zone #
#-----------------------------------#

{
   info_log "" 
    
   info_log "---------------- main ----------------" 

} >> $logFile

case $action in 

     install) 

        ( install_pkg "$pkg" "$ver" "$md5" "$startApp" "$urls" "$rate" "$platform" "$cfgmd5" "$cfgpkgurls" >> $logFile 2>&1 )

        rc=$?

        if [ $rc -eq 0 ] ; then

           if [ $startApp = "yes" ] ; then

               set_instance_status "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "running"
           fi
        fi

        detailMsg=$(split_task_log "$appLog" "rpt") || detailMsg=$(split_task_log "$logFile" "rpt")

        brief="$(cat $briefLog)"

        # remove task before report , so it doesn't have to wait the reporting to finish

        ( remove_self_task $MYPID $taskFile $taskMgrLog ) 

        ( do_report "$taskId" "$sessionId" "$rc" "$brief" "$detailMsg" "$versionId" "$(get_cfg_id_by_pkg_ver "$pkg" "$ver")" )

        clear_garbage $CLEAN_PERIOD &>/dev/null || :

        exit $rc ;;

     uninstall) 

        cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

        ( uninstall_pkg "$pkg" "$ver" >> $logFile 2>&1 )

        rc=$?

        if [ $rc -eq 0 ] ; then

           set_instance_status "$PKG_BACKUP_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "stop"
        fi

        detailMsg=$(split_task_log "$appLog" "rpt") || detailMsg=$(split_task_log "$logFile" "rpt")

        brief="$(cat $briefLog)"

        # remove task before report , so it doesn't have to wait the reporting to finish

        ( remove_self_task $MYPID $taskFile $taskMgrLog ) 

        ( do_report "$taskId" "$sessionId" "$rc" "$brief" "$detailMsg" "$versionId" "" )

        # keep the task file , do not remove , in case of alarm

        clear_garbage $CLEAN_PERIOD &>/dev/null || :

        exit $rc ;;

  update)

        export COPY_CFG=$copyCfg

        ( update_pkg "$pkg" "$oldVer" "$newVer" "$md5" "$copyCfg" "$order" "$urls" "$rate" "$startApp" "$platform" "$cfgmd5" "$cfgpkgurls" >> $logFile 2>&1 )

        rc=$?

        if [ $rc -eq 0 ] ; then

           set_instance_status "$PKG_BACKUP_DIR/$pkg-$oldVer/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "stop"

           set_instance_status "$PKG_HOME_DIR/$pkg-$newVer/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "running"
        fi

        detailMsg=$(split_task_log "$appLog" "rpt") || detailMsg=$(split_task_log "$logFile" "rpt")

        brief="$(cat $briefLog)"

        # remove task before report , so it doesn't have to wait the reporting to finish

        ( remove_self_task $MYPID $taskFile $taskMgrLog )

        ( do_report "$taskId" "$sessionId" "$rc" "$brief" "$detailMsg" "$versionId" "$(get_cfg_id_by_pkg_ver "$pkg" "$newVer")" )

        clear_garbage $CLEAN_PERIOD &>/dev/null || : 

        exit $rc ;;

 
     start|stop) 

        cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

        user=$(get_and_chk_user "$cfgFile")

        if [ -z "$user" ] ; then

           msg=$(err_log "get user info failed" 2>&1)

           echo "$msg" >> $logFile

           ( remove_self_task $MYPID $taskFile $taskMgrLog ) 

           ( do_report "$taskId" "$sessionId" "$CODE_GET_USER_FAIL" "$msg" "$msg" "$versionId" "" )

           exit $CODE_GET_USER_FAIL
        fi

        ( start_stop_app "$pkg" "$ver" "$user" "$action" >> $logFile 2>&1 )

        rc=$?

        if [ $rc -eq 0 ] ; then

            if [ $action = "start" ] ; then

                set_instance_status "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "running"
            else
                set_instance_status "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "stop"
            fi
        fi

        detailMsg=$(split_task_log "$appLog" "rpt") || detailMsg=$(split_task_log "$logFile" "rpt")

        brief="$(cat $briefLog)"

        # remove task before report , so it doesn't have to wait the reporting to finish

        ( remove_self_task $MYPID $taskFile $taskMgrLog )

        ( do_report "$taskId" "$sessionId" "$rc" "$brief" "$detailMsg" "$versionId" "" )

        clear_garbage $CLEAN_PERIOD &>/dev/null || : 

        exit $rc ;;


     restart)

        cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

        user=$(get_and_chk_user "$cfgFile")

	if [ -z "$user" ] ; then

           msg=$(err_log "get user info failed" 2>&1)

           echo "$msg" >> $logFile

           ( remove_self_task $MYPID $taskFile $taskMgrLog )

           ( do_report "$taskId" "$sessionId" "$CODE_GET_USER_FAIL" "$msg" "$msg" "$versionId" "" )

           exit $CODE_GET_USER_FAIL

        fi

        ( restart_app "$pkg" "$ver" "$user" >> $logFile 2>&1 )

        rc=$?

        if [ $rc -eq 0 ] ; then

           set_instance_status "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "running"
        fi

        detailMsg=$(split_task_log "$appLog" "rpt") || detailMsg=$(split_task_log "$logFile" "rpt")

        brief="$(cat $briefLog)"

        # remove task before report , so it doesn't have to wait the reporting to finish

        ( remove_self_task $MYPID $taskFile $taskMgrLog ) 

        ( do_report "$taskId" "$sessionId" "$rc" "$brief" "$detailMsg" "$versionId" "" )

        clear_garbage $CLEAN_PERIOD &>/dev/null || : 

        exit $rc 

        ;;

     rollback)

        ( rollback_pkg "$pkg" "$srcVer" "$dstVer" >>$logFile 2>&1 )

        rc=$?

        if [ $rc -eq 0 ] ; then

             set_instance_status "$PKG_HOME_DIR/$pkg-$dstVer/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "running"

             set_instance_status "$PKG_BACKUP_DIR/$pkg-$srcVer/$PKG_SCRIPT_DIR/$INSTANCE_STATUS_FILE" "stop"
        fi

        detailMsg=$(split_task_log "$appLog" "rpt") || detailMsg=$(split_task_log "$logFile" "rpt")

        brief="$(cat $briefLog)"

        # remove task before report , so it doesn't have to wait the reporting to finish

        ( remove_self_task $MYPID $taskFile $taskMgrLog )
	
        ( do_report "$taskId" "$sessionId" "$rc" "$brief" "$detailMsg" "$versionId" "$(get_cfg_id_by_pkg_ver "$pkg" "$dstVer")" )

        clear_garbage $CLEAN_PERIOD &>/dev/null || : 

        exit $rc ;;

     reload)

        ( reload_pkg "$pkg" "$ver" "$cfgmd5" "$cfgpkgurls" >> $logFile 2>&1)

        rc=$?

        detailMsg=$(split_task_log "$logFile" "rpt")

        brief="$(cat $briefLog)"

        # remove task before report , so it doesn't have to wait the reporting to finish

        ( remove_self_task $MYPID $taskFile $taskMgrLog )

        ( do_report "$taskId" "$sessionId" "$rc" "$brief" "$detailMsg" "$versionId" "$(get_cfg_id_by_pkg_ver "$pkg" "$ver")" )

        clear_garbage $CLEAN_PERIOD &>/dev/null || :

        exit $rc ;;
   
     *) msg=$(err_log "invalid action type [$action]" 2>&1)

        echo "$msg" >> $PUB_SCRIPT_ERR_LOG_FILE

        ( do_report "$taskId" "$sessionId" "$CODE_ACTION_INVALID" "$msg" "$msg" "$versionId" "" )

        clear_garbage $CLEAN_PERIOD &>/dev/null || :	

        exit $CODE_ACTION_INVALID ;;
esac
