#!/bin/bash

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

#-------------------#
# private functions #
#-------------------#

function brief_ok_log {

   local msg="${*:?$FUNCNAME [ERR ] | log msg is NULL}"

   local level="INFO"

   # one for task log

   printf "%s | %30s | %30s | [$level] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}" "${FUNCNAME[2]:=N/A}" "$msg" 

   # one for brief log

   printf "%30s | %30s | [$level] | %s\n" "${FUNCNAME[1]}" "${FUNCNAME[2]:=N/A}" "$msg" >${_BRIEF_LOG}

   return
}



function brief_err_log {

   local msg="${*:?$FUNCNAME [ERR ] | log msg is NULL}"

   local level="ERR "

   # one for task log

   printf "%s | %30s | %30s | [$level] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}" "${FUNCNAME[2]:=N/A}" "$msg" 1>&2

   # one for brief log

   printf "%30s | %30s | [$level] | %s\n" "${FUNCNAME[1]}" "${FUNCNAME[2]:=N/A}" "$msg" >${_BRIEF_LOG}

   return

}

function collect_app_log {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local action=${3:?$FUNCNAME [ERR ] | action is NULL}

   local stepN=${4:?$FUNCNAME [ERR ] | stepN is NULL}

   local cfgFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE"

   local appName syslogFile pkgLogFile content

   {

      if [ "$stepN" = "2" ] ; then

           echo ; echo  # print two blank lines for separation
      fi

      echo "$(date +'%F %T|') ip=[$(get_all_ip)] action=${_ACTION} taskId=${_TASK_ID} sessionId=${_SESSION_ID} verId=${_VERSION_ID}"

      echo "$(date +'%F %T|') detail log file = $PUB_SCRIPT_LOG_DIR/${_TASK_ID}/${_SESSION_ID}/${_VERSION_ID}/$PUB_SCRIPT_LOG_FILE"

      echo "$(date +'%F %T|') current mode = ${_ACTION} , step #${stepN} : $action $pkg-$ver"

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      echo "$(date +'%F %T|') $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/${action}.log (last 50 lines)"

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      content=$(cat $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/${action}.log 2>/dev/null | sed 's/^/                     /')

      # try antoher place for ${action}.log : user_script.log

      if [ -z "$content" ] ; then

            content=$(tail -n 50 $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_USER_SCRIPT_LOG | sed 's/^/                     /')
      fi

      echo "${content:=$(date +'%F %T|') (empty)}"

      echo

      appName=$(get_item_cfg "base_info" "$cfgFile" "app_name") || return $?

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      echo "$(date +'%F %T|') /data/yy/log/$appName/$appName.log (last 50 lines)"

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      syslogFile="${PKG_LOG_DIR}/${appName}/${appName}.log"

      content=$(tail -n 50 $syslogFile 2>/dev/null | sed 's/^/                     /')

      echo "${content:=$(date +'%F %T|') (empty)}"

      echo

      pkgLogFile="${PKG_LOG_DIR}/$pkg/$pkg.log"

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      echo "$(date +'%F %T|') /data/yy/log/$pkg/$pkg.log (last 50 lines)"

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      content=$(tail -n 50 $pkgLogFile 2>/dev/null | sed 's/^/                     /')

      echo "${content:=$(date +'%F %T|') (empty)}"

      echo

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      echo "$(date +'%F %T|') /data/services/$pkg-$ver/log/*.log (last 50 lines)"

      echo "$(date +'%F %T|') $(repeatStr '*' 100)"

      appLogFiles="$PKG_HOME_DIR/$pkg-$ver/$PKG_SYMBOL_LOG_DIR/*.log"

      contenet=$(tail -n 50 $appLogFiles 2>/dev/null | sed 's/^/                     /')

      echo "${content:=$(date +'%F %T|') (empty)}"

   } >> ${_APP_LOG} 2>&1

}



function bash_syn_chk {

   local scriptFile=${1:?$FUNCNAME [ERR ] | scriptFile is NULL}

   local str

   if [ ! -r "$scriptFile" ] ; then

        brief_err_log "can not read script file $scriptFile , syntax check failed"

        return 1

   elif grep -q '
' $scriptFile ; then

        brief_err_log "found DOS CRLF character , invalid"

        return 1
   else
        str=$(bash -n "$scriptFile" 2>&1)

        rc=$?

        if [ -n "$str" ] ; then

             brief_err_log "$(echo $str)" 
        else
             debug_log "syntax check for [$scriptFile] passed"
        fi

        return $rc
   fi

}



function get_section_cfg {

   local section=${1:?$FUNCNAME [ERR ] | section name is NULL}

   local cfgFile=${2:?$FUNCNAME [ERR ] | cfg file is NULL}

   local row1 row2 

   if [ ! -f "$cfgFile" ] ; then

        brief_err_log "config file $cfgFile not exist"

        return $CODE_PKG_INIT_FILE_NOT_FOUND

   elif [ ! -r "$cfgFile" ] ; then

        brief_err_log "config file $cfgFile read permission denied"

        return $CODE_PKG_INIT_FILE_NOT_FOUND
   fi

   # hardcode delemiter is "<>"

   row1=$(egrep -n "^[[:space:]]*<${section}>" $cfgFile  | cut -d':' -f1)

   row2=$(egrep -n "</${section}>[[:space:]]*$" $cfgFile | cut -d':' -f1)

   if [ -z "$row1" ] && [ -z "$row2" ] ; then

        brief_err_log "section $section not found"

        return $CODE_GET_SECTION_CFG_FAIL 

   elif [ -z "$row1" ] ; then

        brief_err_log "section $section start-line not found"

        return $CODE_GET_SECTION_CFG_FAIL
 
   elif [ -z "$row2" ] ; then

        brief_err_log "section $section end-line not found"

        return $CODE_GET_SECTION_CFG_FAIL
 
   elif [ "$row2" -lt "$row1" ] ; then

        brief_err_log "section $section start-line ($row1) is behind end-line ($row2)"

        return $CODE_GET_SECTION_CFG_FAIL

   elif [ "$(echo $row1 |wc -w)" -gt 1 ] ; then

        brief_err_log "found duplicate section $section start-line"

        return $CODE_DUP_SECTION

   elif [ "$(echo $row2 |wc -w)" -gt 1 ] ; then

        brief_err_log "found duplicate section $section end-line"

        return $CODE_DUP_SECTION

   elif [ "$row1" -eq "$row2" ] ; then

        sed -n "${row1}p" $cfgFile | sed "s/^<$section>//;s/<\/$section>$//"

        return
   else
        sed -n "${row1},${row2}p" $cfgFile | sed -n '1!p' |sed -n '$!p'

        return
   fi
        
}



function get_item_cfg {

   local section=${1:?$FUNCNAME [ERR ] | section name is NULL}

   local cfgFile=${2:?$FUNCNAME [ERR ] | cfg file is NULL}

   local item=${3:?$FUNCNAME [ERR ] | item name is NULL}

   local x

   x=$(get_section_cfg "$section" "$cfgFile") || return $? 

   y=$(echo "$x" | egrep "^[[:space:]]*${item}[[:space:]]*=[[:space:]]*.+$" |awk -F'=' '{print $2}' |sed 's/^"//;s/"$//')

   if [ -z "$y" ] ; then

        brief_err_log "can not find [$item] from section $section"

        return $CODE_GET_ITEM_CFG_FAIL
   else
        echo "$y"

        return 0
   fi
}



function install_precheck {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version num is NULL}

   if [ -d "$PKG_HOME_DIR/$pkg-$ver" ] ; then

        brief_err_log "$pkg-$ver is already installed on $PKG_HOME_DIR"

        err_log "install at [$(tail -1 $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_INSTALL_MODE_FILE)]"

        err_log "$(ls -l $PKG_HOME_DIR/$pkg-$ver 2>&1)"

        return $CODE_INSTALL_PRECHK_FAIL

   elif [ -e "$PKG_HOME_DIR/$pkg-$ver" ] && [ ! -d "$PKG_HOME_DIR/$pkg-$ver" ] ; then

        brief_err_log "$PKG_HOME_DIR/$pkg-$ver is a FILE !!!"

        err_log "$(ls -l $PKG_HOME_DIR/$pkg-$ver 2>&1)"

        return $CODE_INSTALL_PRECHK_FAIL
   else
        return 0
   fi

}



function get_pkg_fullname {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version num is NULL}

   local platform=${3:?$FUNCNAME [ERR ] | platform is NULL}

   local type=${4:-binary}

   if [ "$type" = "config" ] ; then

        echo "${pkg}-${ver}-${platform}_cfg.tar.gz"
   else
        echo "${pkg}-${ver}-${platform}.tar.gz"
   fi

   return
}



function chk_taskId {

   local taskId

   taskId=${1:?$FUNCNAME [ERR ] | task id is NULL}

   if ! is_positive_integer $taskId ; then

        err_log "taskId [$taskId] is not a positive number"

        return $CODE_TASK_ID_INVALID
   else
        debug_log "taskId [$taskId] is valid"

        return 0
   fi
}



function chk_sid {

   local sid

   sid=${1:?$FUNCNAME [ERR ] | session id is NULL}

   if ! is_positive_integer $sid ; then

        err_log "sessionId [$sid] is not a positive number"

        return $CODE_SESSION_ID_INVALID
   else
        debug_log "sessionId [$sid] is valid"

        return 0
   fi
}



function chk_verId {

   local verId

   verId=${1:?$FUNCNAME [ERR ] | version id is NULL}

   if ! is_positive_integer $verId ; then

        err_log "versionId [$verId] is not a positive number"

        return $CODE_VERSION_ID_INVALID
   else
        debug_log "versionId [$verId] is valid"

        return 0
   fi
}



function chk_pkg {

   local pkg

   pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   if [ -n "$(echo $pkg | tr -d '[[:alnum:]\-_]')" ] ; then

        err_log "pkg name [$pkg] contain invalid characters [$(echo $pkg | tr -d '[[:alnum:]\-_]')]" 

        err_log "Only allowed 0-9 a-z A-Z _ -"

        return $CODE_PKG_NAME_INVALID
   else
        debug_log "pkg name [$pkg] is valid"

        return 0
   fi
}



function chk_ver {

   local ver x

   ver=${1:?$FUNCNAME [ERR ] | version num is NULL}

   x=$(echo "$ver" | tr -d '[0-9a-zA-Z_.\-]')

   if [ -n "$x" ] ; then

        err_log "version number [$ver] contain invalid characters [$x]" 

        err_log "Only allowed 0-9 , a-z , A-Z , _ , - , and \".\""

        return $CODE_VERSION_ID_INVALID
   else
        debug_log "version number [$ver] is valid"

        return 0
   fi
}



function chk_md5 {

   local md5

   md5=${1:?$FUNCNAME [ERR ] | md5 is NULL}

   if [ -n "$(echo $md5 | tr -d '[[:lower:]][0-9]')" ] ; then

        err_log "md5 [$md5] contain invalid characters [$(echo $md5 | tr -d '[[:lower:]][0-9]')]"

        return $CODE_PKG_MD5_INVALID
   else
        debug_log "md5 [$md5] is valid"

        return 0
   fi
}



function chk_urls {

   local urls

   urls=${*:?$FUNCNAME [ERR ] | urls is NULL}

   for i in $urls ; do

       if ! echo $i | egrep -q '^http?://' ; then

            err_log "url [$i] is invalid"

            return $CODE_PKG_URLS_INVALID
       else
            debug_log "url [$i] is valid"

            return 0
       fi

   done
}



function chk_order {

   local order

   order=${1:?$FUNCNAME [ERR ] | order is NULL}

   if [ "$order" != "kill-first" ] && [ "$order" != "start-first" ] && [ "$order" != "noaction" ] ; then

         errlog "$FUNCNAME" "${FUNCNAME[1]:=N/A}" "order [$order] is invalid , it should be \"kill-first\" or \"start-first\" or \"noaction\""

         return $CODE_START_STOP_ORDER_INVALID
   else
         debug_log "order [$order] is valid"

         return 0
   fi 
}



function is_boolean {

   local booleanValue=${1:?$FUNCNAME [ERR ] | booleanValue is NULL}

   if echo "$booleanValue" | tr '[[:upper:]]' '[[:lower:]]' | egrep -w -i -q 'yes|no' ; then

        return 0
   else
        return $CODE_BOOLEAN_VALUE_INVALID 
   fi

}



function chk_startApp {

   local startApp

   startApp=${1:?$FUNCNAME [ERR ] | startApp is NULL}

   if is_boolean $startApp ; then

        debug_log "startApp [$startApp] is valid"
 
        return 0
   else
        err_log "startApp [$startApp] is invalid, it should be \"yes\" or \"no\""

        return $CODE_START_APP_INVALID 
   fi
}



function chk_copyCfg {

   local copyCfg 

   copyCfg=${1:?$FUNCNAME [ERR ] | copyCfg is NULL}

   if is_boolean $copyCfg ; then

        debug_log "copyCfg [$copyCfg] is valid"

        return 0
   else
        err_log "copyCfg [$copyCfg] is invalid, it should be \"yes\" or \"no\""

        return $CODE_OTHER_ERR 
   fi

}



function chk_rate {

   local rate

   rate=${1:?$FUNCNAME [ERR ] | rate is NULL}

   if ! is_positive_integer $rate ; then

        err_log "rate [$rate] is not a positive number"

        return $CODE_RATE_INVALID
   else
        debug_log "rate [$rate] is valid"

        return 0
   fi
}



function chk_platform {

   local platform

   platform=${1:?$FUNCNAME [ERR ] | platform is NULL}

   if [ "$platform" != '32' ] && [ "$platform" != '64' ] && [ "$platform" != '64_32' ] && [ "$platform" != '32_64' ] ; then

        err_log "platform [$platform] is invalid"

        err_log "it must be '32' or '64' or '32_64' or '64_32'"

        return $CODE_PLATFORM_INVALID

   elif [ "$platform" = '32' ] || [ "$platform" = '64' ] ; then

         if [ "$platform" = "$(getconf LONG_BIT)" ] ; then

               debug_log "platform [$platform] is matched again this host"

               return 0
         else
               err_log "platform [$platform] is incorrect , it should be $(getconf LONG_BIT)-bit" 

               return $CODE_PLATFORM_INVALID
         fi
   else
         debug_log "platform [$platform] is compatiable with both 32/64"

         return 0
   fi
}



function download_pkg {

   # this function may be called by YYMS , so I use the key=value method to get paramaters

   local wgetOption pkgMd5 logFile dstDir downloadFile pkgFullName 

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version num is NULL}

   local md5=${3:?$FUNCNAME [ERR ] | md5 num is NULL}

   local urls=${4:?$FUNCNAME [ERR ] | urls is NULL}

   local platform=${5:?$FUNCNAME [ERR ] | platform is NULL}

   local dstDir=${6:?$FUNCNAME [ERR ] | dst dir is NULL}

   local pkgType=${7:?$FUNCNAME [ERR ] | pkg type is NULL}

   local rate=${8:-$WGET_LIMIT_RATE}

   local isp=$(get_host_isp $HOST_INFO_FILE)

   local flag=0

   local count=0

   local rc
 
   local downloadLog

   local wgetLocalOption

   local deadLineTs

   # load env conf

   source /data/pkg/public-scripts/conf/environ.conf

   info_log "pkg type is [$pkgType]"

   info_log "platform = [$platform]"

   pkgFullName=$(get_pkg_fullname "$pkg" "$ver" "$platform" "$pkgType")

   info_log "pkg full name = [$pkgFullName]" 

   # check if wget installed

   if ! which_cmd wget &>/dev/null ; then

      brief_err_log "wget NOT installed , download fail"

      return $CODE_WGET_NOT_FOUND
   fi

   # find pkg in $dstDir if downloaded before

   pkgMd5=$(find $dstDir -type f -name "$pkgFullName" |xargs -r md5sum | awk '{print $1}')

   if [ "$pkgMd5" = "$md5" ] ; then

        debug_log "find matched pkg file $pkgFullName in $dstDir"

        debug_log "no need to download"
   
        return 0
   else
        info_log "no mathced pkg file found in $dstDir"

        info_log "download begin"
   fi 
 
   # check /data partition usage

   if [ $(partition_usage $BASE_PREFIX) -gt $DISK_FULL_THRESHOLD ] ; then

        brief_err_log "$BASE_PREFIX usage > ${DISK_FULL_THRESHOLD}% , no free disk space"

        return $CODE_PKG_DISK_FULL
   else
        debug_log "$BASE_PREFIX usage is under ${DISK_FULL_THRESHOLD}% , safe to download"
   fi

   mkdir -p $dstDir &>/dev/null

   if [ ! -d $dstDir ] ; then

        brief_err_log "can not create download dir $dstDir"

        return $CODE_MKDIR_DOWNLOAD_DIR_FAIL
   else
        info_log "create download dir $dstDir succeed"
   fi
   
   info_log "begin to download"

   # download from multiple urls seperated by comma

   if [ $rate -gt $WGET_MAX_RATE ] ; then
 
         rate=$WGET_MAX_RATE

         warn_log "user-speicified rate [$rate] is too large , enforce to $WGET_MAX_RATE"
   fi

   info_log "download dir is [$dstDir]"

   # because the download site may return 404 until it download the whole file from web front-end ,

   # and Wget _NOT_ retry when receive a  404 response , so I have to use a outside 'while-loop' to 

   # enforce the retry

   downloadLog="$PKG_DOWNLOAD_LOG_DIR/${pkgFullName}-log"

   info_log "download log set to $downloadLog"

   rm -f $downloadLog

   # special treatment for config package

   if [ "$pkgType" = "config" ] ; then

        if [ "$ENV" = "product" ] ; then

             #info_log "add origin node [121.14.43.114,113.106.100.74] for config package"

             info_log "add origin node [121.14.43.114] for config package"

             urls="http://121.14.43.114:8086/fileDownload/download2.do?md5=$md5,$urls"
        fi

        urls=$(echo "$urls" | sed 's/,/ /g' | xargs -n1)

   else  # package tyep is binary

	if ( [ "$isp" = "EDU" ] || [ "$isp" = "MOB" ] ) && [ "$ENV" = "product" ] ; then

	     warn_log "isp = $isp , ignore the original urls , enforce to use transparent route"

             urls="http://218.16.227.177:51200/static/$md5 http://119.97.153.186:51200/static/$md5 http://222.186.49.45:51200/static/$md5"
        else
             info_log "isp = $isp , use original urls"
	fi
			  
         urls=$(echo "$urls" | sed 's/,/ /g' | xargs -n1 |  { sort -R 2>/dev/null || sort ; } )
   fi

   deadLineTs=$(( $(date +%s) + $WGET_TOTAL_TIMEOUT ))

   while [ $count -lt $RETRY_NUM ] ; do

       for i in $urls ; do
    
           # enforce to 8Mb/sec
    
           wgetLocalOption="--append-output=$downloadLog --output-document=$pkgFullName"

           wgetOption="$wgetLocalOption $WGET_COMMON_OPT"
    
           ( cd $dstDir ; rm -f $pkgFullName ; wget $wgetOption "$i" &>/dev/null) ; rc=$?
    
           if [ $rc -ne 0 ] ; then

                 ((++flag))

                 if [ $(date +%s) -lt $deadLineTs ] ; then
    
                      warn_log "Wget RETURN CODE = [$rc]"
    
                      warn_log "download from [$i] failed"

                      warn_log "still has time to download"
    
                      continue
                 else
                      warn_log "Wget Total Timeout reached ($WGET_TOTAL_TIMEOUT secs) , Abort !!!"

                      break 2 
                 fi
           else
                 info_log "download from [$i] finished" 
    
                 downloadFile=$dstDir/$pkgFullName
    
                 pkgMd5="$(md5sum $downloadFile | awk '{print $1}')"
              
                 if [ "$pkgMd5" = $md5 ] ; then
    
                      flag=0
              
                      info_log "md5 of [$downloadFile] is correct"

                      info_log "md5 = $md5" 
    
                      return $flag
                 else
                      ((++flag)) 
    
                      err_log "MD5 incorrect ! [$pkgMd5 != $md5]"
     
                      rm -f $downloadFile
     
                      info_log "Try next url" ; continue
                 fi
            fi
    
       done
    
       ((count++))

       # because Wget _NOT_ retry after 404 , so I have to sleep every round

       info_log "sleep for $RETRY_INTERVAL seconds"

       info_log "randomize download url list ..."

       sleep $RETRY_INTERVAL

   done

   # Download failed !!!

   if [ $flag -ne 0 ] ; then

        brief_err_log "download $pkgFullName from ALL urls failed"

        return $CODE_PKG_DOWNLOAD_FAIL
   fi
}              

 

function install_files { 

   # this function may be called by YYMS , so I use key=value method to get paramaters

   local pkg=${1:?$FUNCNAME [ERR ] | pkg file fullpath is NULL}

   local dstDir=${2:?$FUNCNAME [ERR ] | dst dir is NULL}

   local name=${3:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${4:?$FUNCNAME [ERR ] | version number must be give}

   local cfgFile="$PKG_HOME_DIR/$name-$ver/$PKG_CFG_FILE"

   local appName

   local rc

   local i pkg dstDir content

   # pkg file permission check
  
   if ! is_readable_file $pkg ; then

        brief_err_log "pkg file [$pkg] not exist or permission denied"

        return $CODE_PKG_FILE_ACCESS_ERR
   fi

   # dst dir permission check

   if ! is_writable_dir $dstDir ; then

        brief_err_log "destination dir [$dstDir] do NOT have write permission"

        return $CODE_PKG_INSTALL_DIR_ACCESS_ERR
   else
        mkdir -p $dstDir &>/dev/null
   fi

   # get content , also is a test

   debug_log "verification begin ...."

   content=$(tar --to-stdout -zxf $pkg $name-$ver/$PKG_CFG_FILE 2>/dev/null) 

   if [ -z "$content" ] ; then

        brief_err_log "$name-$ver/$PKG_CFG_FILE NOT found , invalid pkg"

        return $CODE_PKG_INVALID

   elif ! echo "$content" | egrep -q -x "name=\"$name\"" ; then 

        brief_err_log "name=\"$name\" NOT found in $name-$ver/$PKG_CFG_FILE"

        return $CODE_PKG_NAME_NOT_MATCH 

   elif ! echo "$content" | egrep -q -x "version=\"$ver\"" ; then

        brief_err_log "version=\"$ver\" NOT found in $name-$ver/$PKG_CFG_FILE"

        return $CODE_PKG_VER_NOT_MATCH
   else
        debug_log "name=\"$name\" is matched against cmd-line paramater"

        debug_log "version=\"$ver\" is matched against cmd-line paramater" 
   fi

   debug_log "[$pkg] is a valid tar.gzip file"

   debug_log "install to dir [$dstDir]"

   info_log ""

   echo

   echo '----- Disk Usage -----' 

   df -hl

   echo

   #echo '----- Binary Pkg Info -----'

   tar -C $dstDir -zxf $pkg 

   rc=$?

   echo

   if [ $rc -ne 0 ] ; then

        brief_err_log "un-tar pkg [$pkg] failed"

        return $CODE_PKG_EXTRACT_FAIL
   else
        return 0
   fi
   
}



function get_area_id {

   local hostInfoFile=${1:?$FUNCNAME [ERR ] | hostInfoFile is NULL}

   local x

   if ! is_readable_file $hostInfoFile ; then

        brief_err_log "$hostInfoFile read error"

        return $CODE_GET_HOST_INFO_FAIL
   else
        x=$(egrep -i '^[[:space:]]*area_id[[:space:]]*=' $hostInfoFile | awk -F '=' '{print $2}')

        if [ -n "$x" ] ; then

             echo "$x"

             return
        else
             brief_err_log "area id is empty"

             return $CODE_GET_HOST_INFO_FAIL
        fi              
   fi
}



function get_city_id {

   local hostInfoFile=${1:?$FUNCNAME [ERR ] | hostInfoFile is NULL}

   local x
 
   if ! is_readable_file $hostInfoFile ; then

        brief_err_log "$hostInfoFile read error"

        return $CODE_GET_HOST_INFO_FAIL
   else
        x=$(egrep -i '^[[:space:]]*city_id[[:space:]]*=' $hostInfoFile | awk -F '=' '{print $2}')

        if [ -n "$x" ] ; then

             echo "$x"

             return
        else
             brief_err_log "city is is empty"

             return $CODE_GET_HOST_INFO_FAIL
        fi
   fi
}



function get_idc_id {

   local hostInfoFile=${1:?$FUNCNAME [ERR ] | hostInfoFile is NULL}

   local x

   if ! is_readable_file $hostInfoFile ; then

        brief_err_log "$hostInfoFile read error"

        return $CODE_GET_HOST_INFO_FAIL
   else
        x=$(egrep -i '^[[:space:]]*idc_id[[:space:]]*=' $hostInfoFile | awk -F '=' '{print $2}')

        if [ -n "$x" ] ; then

             echo "$x"
        else
             brief_err_log "idc id is empty"

             return $CODE_GET_HOST_INFO_FAIL
        fi
   fi
}



function get_pri_groupId {

   local hostInfoFile=${1:?$FUNCNAME [ERR ] | hostInfoFile is NULL}

   local x

   if ! is_readable_file $hostInfoFile ; then

        brief_err_log "$hostInfoFile read error"

        return $CODE_GET_HOST_INFO_FAIL
   else 
        x=$(egrep -i '^[[:space:]]*pri_group_id[[:space:]]*=' $hostInfoFile | awk -F '=' '{print $2}')

        if [ -n "$x" ] ; then

             echo "$x"

             return
        else
             brief_err_log "primary group id is empty"

             return $CODE_GET_HOST_INFO_FAIL 
        fi
   fi 

   return
}



function get_ip_isp_list {

   local hostInfoFile=${1:?$FUNCNAME [ERR ] | hostInfoFile is NULL} 

   local x

   if ! is_readable_file $hostInfoFile ; then

        brief_err_log "$hostInfoFile read error"

        return $CODE_GET_HOST_INFO_FAIL
   else
        x=$(egrep -i '^[[:space:]]*ip_isp_list[[:space:]]*=' $hostInfoFile | awk -F '=' '{print $2}')

        echo "$x"
   fi

   return
}



function get_ip_by_isp {

   local hostInfoFile=${1:?$FUNCNAME [ERR ] | hostInfoFile is NULL}

   local isp=${2:?$FUNCNAME [ERR ] | isp is NULL}

   local x

   x=$(get_ip_isp_list $hostInfoFile)

   if [ -z "$x" ] ; then

         brief_err_log "get ip list failed"

         return $CODE_GET_HOST_INFO_FAIL
   else
         echo "$x" |sed 's/,/ /g' |xargs -n 1 |awk -F ':' -v _isp=$isp '$2 == _isp {print $1}' | sort -u | head -n 1 
   fi
}



function get_host_isp {

   local hostInfoFile=${1:?$FUNCNAME [ERR ] | hostInfoFile is NULL}

   local x isp

   x=$(get_ip_isp_list $hostInfoFile)

   if [ -z "$x" ] ; then

         brief_err_log "get ip list failed"

         return $CODE_GET_HOST_INFO_FAIL
   else
         isp=$(echo "$x" |sed 's/,/ /g' |xargs -n 1 |awk -F ':' '{print $2}' |sort -u |egrep -v 'INTRANET|0|BMC' |xargs)
     
         if [ $(echo $isp |wc -w) -eq 1 ] ; then

              echo "$isp" ; return 0              
 
         elif [ $(echo $isp |wc -w) -eq 2 ] ; then

              echo "DUAL" ; return 0

         elif [ $(echo $isp |wc -w) -eq 3 ] ; then

              echo "TRIPLE" ; return 0

         elif [ $(echo $isp |wc -w) -eq 0 ] ; then

              if [ "$(echo "$x" |sed 's/,/ /g' |xargs -n 1 |awk -F ':' '{print $2}' |sort -u| egrep -v '0|BMC')" = "INTRANET" ] ; then

                   echo "INTRANET" ; return 0
              else
                   brief_err_log "get host isp failed"
              fi

              return $CODE_GET_HOST_INFO_FAIL
         fi
  fi     
}



function replace_pattern {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg file is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local groupId isp eth0Ip ctlIp cncIp eduIp mobIp bgpIp allIp firstIp secondIp

   local fileList newFileList

   local areaId cityId idcId serverId

   local rList pattern value i j x

   local appName=$(get_item_cfg "base_info" "$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE" "app_name" | sed 's/,/ /g')

   info_log ""

   info_log "$(repeatStr '+' 19) $HOST_INFO_FILE $(repeatStr '+' 19)"

   cat $HOST_INFO_FILE | xargs -n 1

   info_log "$(repeatStr '+' 19) $HOST_INFO_FILE $(repeatStr '+' 19)"

   groupId=$(get_pri_groupId $HOST_INFO_FILE) || return $?

   areaId=$(get_area_id $HOST_INFO_FILE)      || return $?

   cityId=$(get_city_id $HOST_INFO_FILE)      || return $?

   idcId=$(get_idc_id $HOST_INFO_FILE)        || return $?

   isp=$(get_host_isp $HOST_INFO_FILE)        || return $?

   serverId=$(get_server_id_simple)

   # add isp-code for V4

   case $isp in

     'CTL') ispcode=1   ;;

     'CNC') ispcode=2   ;;

     'EDU') ispcode=8   ;;

     'MOB') ispcode=32  ;; 

     'HK')  ispcode=128 ;;

     'BRA') ispcode=256 ;;
 
     'BGP') ispcode=64  ;;

     'WBN') ispcode=16  ;;

    'CNII') ispcode=4   ;;

    'DUAL') ispcode=3   ;;

    'INTRANET') ispcode=512 ;;
   
        *)  ispcode=0   ;;
   esac

   # follow symbol link and canonicalize

   fileList=$(find -L "$PKG_HOME_DIR/$pkg-$ver/$PKG_CONF_DIR" -type f -a ! -name '*.bak')  

   if [ -n "$fileList" ] ; then

        info_log "found files under $PKG_HOME_DIR/$pkg-$ver/$PKG_CONF_DIR"
   else
        info_log "no file found under $PKG_HOME_DIR/$pkg-$ver/$PKG_CONF_DIR , replace pattern done"

        return 0
   fi

   for i in $fileList ; do

       x=$(readlink -e $i)

       if [ -z "$x" ] ; then

            warn_log "file [$i] point to an invalid path , skip it"      

       elif [ "$x" != "$i" ] ; then # is a symbol link

            info_log "$i ==> $x"

            newFileList="$newFileList $x"
       else
            newFileList="$newFileList $i"
       fi           

   done

   if [ -z "$newFileList" ] ; then

        info_log "no file found under $PKG_HOME_DIR/$pkg-$ver/$PKG_CONF_DIR , replace pattern done"

        return 0
   fi

   # get host info

   ctlIp=$(get_ip_by_isp $HOST_INFO_FILE "CTL")
   
   cncIp=$(get_ip_by_isp $HOST_INFO_FILE "CNC")
   
   bgpIp=$(get_ip_by_isp $HOST_INFO_FILE "BGP")
   
   eduIp=$(get_ip_by_isp $HOST_INFO_FILE "EDU")
   
   mobIp=$(get_ip_by_isp $HOST_INFO_FILE "MOB")
   
   eth0Ip=$(get_local_ip "eth0" | head -1)

   eth1Ip=$(get_local_ip "eth1" | head -1)
   
   allIp=$(get_all_ip)
   
   firstIp=$(get_first_ip)
   
   secondIp=$(get_second_ip)
    
   installPath="$PKG_HOME_DIR/$pkg-$ver"

   # create key:value array for each pattern

   declare -a rList

   rList+=(
           "##APP_NAME##:$appName"

           "##PKG_NAME##:$pkg"

           "##VERSION##:$ver"

           "##PRI_GROUP_ID##:$groupId"

           "##AREA_ID##:$areaId"
           
           "##CITY_ID##:$cityId"
           
           "##IDC_ID##:$idcId"
           
           "##ISP##:$isp"
           
           "##ISP_CODE##:$ispcode"
           
           "##CTL_IP##:$ctlIp"
           
           "##CNC_IP##:$cncIp"
           
           "##BGP_IP##:$bgpIp"
           
           "##EDU_IP##:$eduIp"
           
           "##MOB_IP##:$mobIp"
           
           "##ETH0_IP##:$eth0Ip"

           "##ETH1_IP##:$eth1Ip"
           
           "##ALL_IP##:$allIp"
           
           "##FIRST_IP##:$firstIp"
           
           "##SECOND_IP##:$secondIp"
           
           "##INSTALL_PATH##:$installPath"

           "##SERVER_ID##:$serverId"
   )

   info_log ""

   for i in $newFileList ; do

       cp -af $i ${i}${REPLACE_BAK_SUFFIX} && \

       for j in "${rList[@]}" ; do

             #info_log "${j%%:*} = ${j##*:} for $i"

             pattern=$(echo "$j" | awk -F ':' '{print $1}')

             value=$(echo "$j" | awk -F ':' '{print $2}')

             if ! sed -r -i -e "s!$pattern!$value!g" $i  ; then

                brief_err_log "replace $pattern = [$value] for file $i failed"

                return $CODE_GET_HOST_INFO_FAIL 
             fi
       done 

       if [ "$(md5sum $i ${i}${REPLACE_BAK_SUFFIX} | awk '{print $1}' | sort -u | wc -w)" -eq 1 ] ; then

            #info_log "[not changed] [$i]" 

            #rm -f ${i}.bak

            mv ${i}${REPLACE_BAK_SUFFIX} $i
       else
            info_log "[changed] [$i]"
       fi

   done

   info_log ""

   return
}



function run_post-install {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg file is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local scriptFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_POST_INSTALL_SCRIPT_PATH

   local rc 

   info_log "$(repeatStr '-' 19) post-install $(repeatStr '-' 19)"

   info_log "RUN USER = [$(whoami)]"

   info_log "ACTION=[${_ACTION}]"

   set -x

   bash $scriptFile 

   rc=$?

   set +x

   if [ $rc -ne 0 ] ; then

        brief_err_log "run post-install script failed"

        return $CODE_RUN_POST_INSTALL_SCRIPT_FAIL 
   else
        info_log "run post-install script succeed"

        return 0
   fi
}



function make_link_to_dir {

   local src=${1:?$FUNCNAME [ERR ] | link name is NULL}

   local dst=${2:?$FUNCNAME [ERR ] | dst file is NULL}

   if is_file $src ; then

        brief_err_log "a regular file $src already exist"

        return $CODE_CREATE_SYMBOL_LINK_FAIL

   elif is_link $src ; then

        brief_err_log "a link file $src already exist"

        return $CODE_CREATE_SYMBOL_LINK_FAIL
   fi

   if ! is_exist $(dirname $dst) ; then

        brief_err_log "$dst not exist" 

        return $CODE_SYMBOL_LINK_DST_DIR_NOT_EXIST
   fi

   if ! is_writable_dir $(dirname $src) ; then

        brief_err_log "$(dirname $src) not writable"

        return $CODE_SYMBOL_LINK_DST_DIR_NOT_WRITABLE
   fi

   # create destination dir first if not exist , /data/yy/log/<pkg-ver>/ for example

   mkdir -p $dst 2>/dev/null

   # if src dir exist and files under it , then mv files under dst dir first 

   mv $src/* $dst &>/dev/null 

   # then remove src dir before make symbol link

   ( rmdir $src || rm $src ) &>/dev/null

   ln -s $dst $src

   if [ $? -eq 0 ] ; then
   
        debug_log "make link [$src] --> [$dst] succeed"

        return 0
   else
        brief_err_log "make link [$src] --> [$dst] failed"

        return $CODE_MAKE_SYMBOL_LINK_FAIL 
   fi 
}



function gen_comm_env_var {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local mode=${3:?$FUNCNAME [ERR ] | mode is NULL}

   local oldVer=${4:?$FUNCNAME [ERR ] | old version is NULL}

   local cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

   local user isp outputFile x

   local ctlIp cncIp mobIp bgpIp eduIp eth0Ip varname

   local allIp firstIp secondIp

   local pname

   local serverId

   outputFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_COMMON_ENV_VAR_PATH"

   if [ ! -e $(dirname $outputFile) ] ; then

       mkdir -p $(dirname $outputFile) &>/dev/null

       if [ $? -ne 0 ] ; then

           brief_err_log "mkdir $(dirname $outputFile) failed"

           return $CODE_MKDIR_COMMON_VAR_DIR_FAIL 
       fi
   fi 

   user=$(get_and_chk_user "$cfgFile") || return $?
  
   # get isp

   isp=$(get_host_isp $HOST_INFO_FILE)

   # get ctl/cnc/edu/mob/bgp ip

   for i in CTL CNC BGP EDU MOB ; do

       varname="$(tr '[[:upper:]]' '[[:lower:]]' <<< "$i")Ip"

       eval $varname=$(get_ip_by_isp $HOST_INFO_FILE "$i")
     
   done
 
   # get eth0 ip
 
   eth0Ip=$(get_local_ip "eth0" | head -1)

   eth1Ip=$(get_local_ip "eth1" | head -1)
 
   allIp=$(get_all_ip)
 
   firstIp=$(get_first_ip)
 
   secondIp=$(get_second_ip)
 
   # get pname
 
   pname=$(get_item_cfg "base_info" "$cfgFile" "pname")

   # get server id

   serverId=$(get_server_id_simple)

   true > $outputFile

   {
   
     echo "# this file is generated automatically by ${BASH_SOURCE} at $(date +'%F %T')"
     echo "#"
     echo "# please don't edit"
     echo "#"
     echo "# any problem , please contact linbobo (linbobo@yy.com)"
 
     x=$(grep '^MemTotal:' /proc/meminfo |awk '{printf "%.0f\n",$2/1024/1024}') && \
 
     echo "export MEM_SIZE=$x" >> $outputFile && \
 
     echo "export INSTALL_PATH=\"$PKG_HOME_DIR/$pkg-$ver\""  && \
 
     echo "export PKG_NAME=\"$pkg\"" && \
 
     echo "export VERSION=\"$ver\"" && \
 
     echo "export PKG_USER=\"$user\"" && \
 
     echo "export OLD_VERSION=\"$oldVer\"" && \
 
     echo "export PKG_CFG_FILE=\"$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE\"" && \
 
     echo "export DATA_DIR=\"$PKG_HOME_DIR/$pkg-$ver/$PKG_SYMBOL_DATA_DIR\""  && \
 
     echo "export ISP=\"$isp\"" && \
 
     echo "export CTL_IP=\"$ctlIp\"" && \
 
     echo "export CNC_IP=\"$cncIp\"" && \
 
     echo "export MOB_IP=\"$mobIp\"" && \
 
     echo "export BGP_IP=\"$bgpIp\"" && \
 
     echo "export EDU_IP=\"$eduIp\"" && \
 
     echo "export ETH0_IP=\"$eth0Ip\"" && \

     echo "export ETH1_IP=\"$eth1Ip\"" && \
 
     echo "export ALL_IP=\"$allIp\"" && \
 
     echo "export FIRST_IP=\"$firstIp\"" && \
 
     echo "export SECOND_IP=\"$secondIp\"" && \
 
     echo "export PNAME=\"$pname\"" && \
 
     echo "export APP_NAME=\"$(get_item_cfg "base_info" "$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE" "app_name" | sed 's/,/ /g')\"" && \

     echo "export SERVER_ID=\"$serverId\"" && \
 
     x=$(get_item_cfg "base_info" "$cfgFile" "auto_start") && \
 
     if [ "$x" = "yes" ] ; then
 
          echo "export AUTO_START=\"yes\""
     else
          echo "export AUTO_START=\"no\""
     fi 
 
   } >> $outputFile

  if [ $? -eq 0 ] ; then

       info_log "gen file [$outputFile] succeed"

       return 0
  else
       brief_err_log "gen file [$outputFile] failed" 

       return 1
  fi
}



function save_section_cfg {

   local section=${1:?$FUNCNAME [ERR ] | section name is NULL}

   local cfgFile=${2:?$FUNCNAME [ERR ] | cfg file is NULL}

   local outputFile=${3:?$FUNCNAME [ERR ] | output file is NULL}

   local permission=${4:?$FUNCNAME [ERR ] | permission is NULL}

   if [ ! -e $(dirname $outputFile) ] ; then

       mkdir -p $(dirname $outputFile) &>/dev/null

       if [ $? -ne 0 ] ; then

           brief_err_log "mkdir $(dirname $outputFile) failed"

           return $CODE_MKDIR_SECTION_CFG_FAIL
       fi
   fi

   ( get_section_cfg "$section" "$cfgFile" >$outputFile ) || return $?

   chmod $permission $outputFile 

   #info_log "gen [$outputFile] succeed" 

   debug_log "gen [$outputFile] succeed" 

   return 0
}



function save_section_cfg_N {

   # for history reason , I add this function for old-format (v1.0) init.conf

   # because <restar></restart> section had not been exist in init.conf v1.0 

   local section=${1:?$FUNCNAME [ERR ] | section name is NULL}

   local cfgFile=${2:?$FUNCNAME [ERR ] | cfg file is NULL}

   local outputFile=${3:?$FUNCNAME [ERR ] | output file is NULL}

   local permission=${4:?$FUNCNAME [ERR ] | permission is NULL}

   if [ ! -e $(dirname $outputFile) ] ; then

       mkdir -p $(dirname $outputFile) &>/dev/null

       if [ $? -ne 0 ] ; then

           brief_err_log "mkdir $(dirname $outputFile) failed" 

           return $CODE_MKDIR_SECTION_CFG_FAIL
       fi
   fi

   if egrep -q "^[[:space:]]*<$section>" "$cfgFile" && egrep -q "</$section>[[:space:]]*$" "$cfgFile" ; then

        debug_log "found <$section></$section> section , this is a V2.0 $(basename $cfgFile)"

        ( get_section_cfg "$section" "$cfgFile" >$outputFile ) || return $?

        ( insert_common_code "$outputFile" "$pkg" "$ver" ) || return $?

        echo '2.0' > $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CFG_VER_FILE
   else
        debug_log "no <$section></$section> section found , this is a V1.0 format $(basename $cfgFile)"

        debug_log "we just generate an empty $outputFile"

        true >$outputFile

        echo '1.0' > $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CFG_VER_FILE
   fi

   chmod $permission $outputFile

   debug_log "section config $section save to [$outputFile] succeed"

   return 0
}



function insert_common_code {

   local file=${1:?$FUNCNAME [ERR ] | file is NULL}

   local pkg=${2:?$FUNCNAME [ERR ] | pkg name is NULL}
   
   local ver=${3:?$FUNCNAME [ERR ] | version number is NULL}

   if ! is_writable_file $file ; then

        brief_err_log "can not write $file"

        return $CODE_INSERT_COMMON_CODE_FAIL
   fi

   sed -i -r "1i \
#!/bin/bash\n\
# this file is generated automatically by ${BASH_SOURCE} at $(date +'%F %T')\n\
#\n\
# please don't edit.\n\
#\n\
# any problem , please contact linbobo (linbobo@yy.com)\n\
#\n\
#\n\
set -x\n\
set +H\n\
shopt -s expand_aliases\n\
export LANG=C\n\
export LC_ALL=C\n\
alias sudo='sudo -b'\n\
source $PUB_SCRIPT_BIN_DIR/mypidof.sh\n\
source $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_COMMON_ENV_VAR_FILE\n\
source $PUB_SCRIPT_BIN_DIR/limit-run.sh\n" $file

   if [ $? -ne 0 ] ; then

        brief_err_log "insert common code to $file failed"

        return $CODE_INSERT_COMMON_CODE_FAIL
   else 
        #info_log "insert common code to $file succeed"

        debug_log "insert common code to $file succeed"

        return 0
   fi
}



function insert_comm_resol_code {

   local file=${1:?$FUNCNAME [ERR ] | file is NULL}

   local pkg=${2:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${3:?$FUNCNAME [ERR ] | version number is NULL}

   if ! is_writable_file $file ; then

        brief_err_log "can not write $file"

        return $CODE_INSERT_COMMON_CODE_FAIL
   fi

   sed -i -r "1i \
#!/bin/bash\n\
# this file is generated automatically by ${BASH_SOURCE} at $(date +'%F %T')\n\
#\n\
# please don't edit.\n\
#\n\
# any problem , please contact linbobo (linbobo@yy.com)\n\
#\n\
#\n\
myPid=\$\$\n\
ppid=\$PPID\n\
ancestorPid=\$(ps --no-header -oppid \$ppid)\n\
set -x\n\
set +H\n\
shopt -s expand_aliases\n\
export LANG=C\n\
export LC_ALL=C\n\
alias sudo='sudo -b'\n\
source $PUB_SCRIPT_BIN_DIR/mypidof.sh\n\
source $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_COMMON_ENV_VAR_FILE\n\
source $PUB_SCRIPT_BIN_DIR/limit-run.sh\n\
if [ -f $(readlink -e $PUB_SCRIPT_ALARM_PATH) ] ; then\n\
     bash -x $PUB_SCRIPT_ALARM_PATH \$APP_NAME \$AUTO_START \$INSTALL_PATH \$PKG_NAME \$VERSION \$PNAME \$PKG_USER | tee -a \$INSTALL_PATH/$PKG_SCRIPT_DIR/alarm.log\n\
else\n\
     echo \"no $PUB_SCRIPT_ALARM_PATH found\"\n\
fi\n\
if [ \"\$AUTO_START\" != yes ] ; then \n\
     echo \"AUTO_START is disabled , just quit\"\n\
     exit 0\n\
elif [ -f \"$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$NO_AUTO_START_FLAG_FILE\" ] ; then\n\
     echo \"AUTO_START is enabled , but found $NO_AUTO_START_FLAG_FILE file , maybe current version is stopped\"\n\
     echo \"just quit\"\n\
     exit 0\n\
else\n\
     echo \"AUTO_START is enabled , and no $NO_AUTO_START_FLAG_FILE found , prepare to run Y ^_^ Y\"\n\
     echo \"\$(date +'%F %T')| called by [\$(ps --no-header -opid,ppid,cmd \$ppid\)]\" >> \$INSTALL_PATH/$PKG_SCRIPT_DIR/$PKG_AUTO_START_LOG\n\
     echo \"\$(date +'%F %T')| ancestor is [\$(ps --no-header -opid,ppid,cmd \$ancestorPid)]\" >> \$INSTALL_PATH/$PKG_SCRIPT_DIR/$PKG_AUTO_START_LOG\n\
fi" $file

   if [ $? -ne 0 ] ; then

        brief_err_log "insert common resolve code to $file failed"

        return $CODE_INSERT_COMMON_CODE_FAIL
   else
        debug_log "insert common resolve code to $file succeed"

        return 0
   fi
}



function gen_md5sum_list {

   local cfgFile=${1:?$FUNCNAME [ERR ] | cfgFile name is NULL}

   local md5CfgFile=${2:?$FUNCNAME [ERR ] | md5 config file is NULL}

   local pkg ver

   local outputFile

   local count

   local fileList

   local file

   local rc

   pkg=$(get_item_cfg "base_info" "$cfgFile" "name") || return $? 

   ver=$(get_item_cfg "base_info" "$cfgFile" "version") || return $? 

   outputFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_MD5_RESULT_FILE"

   cp /dev/null $outputFile

   ( save_section_cfg "md5" "$cfgFile" "$PKG_HOME_DIR/$pkg-$ver/$PKG_MD5_CONF_PATH" 644 ) || return $? 

   for i in $(cat $md5CfgFile) ; do

       fileList=$(cd ${PKG_HOME_DIR}/$pkg-$ver/ ; find $i -type f 2>/dev/null)

       for file in $fileList ; do

           if is_readable_file $PKG_HOME_DIR/$pkg-$ver/$file ; then

                value=$(md5sum $PKG_HOME_DIR/$pkg-$ver/$file)

                echo "$value" >> $outputFile 
           fi

       done

   done

   return 
   
}



function save_op_mode {
  
   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}
   
   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local mode=${3:?$FUNCNAME [ERR ] | install mode is NULL}

   local outputFile=${4:-$PKG_HOME_DIR/$pkg-$ver/$PKG_INSTALL_MODE_PATH}

   if ! is_writable_dir $(dirname $outputFile) ; then

        brief_err_log "dir [$(dirname $outputFile)] write permission denied or not exist"

        return $CODE_WRITE_INSTALL_MODE_FILE_FAIL
   else
        debug_log "save_op_mode args : $*"

        debug_log "record installation mode to $outputFile"

        echo "$(date +'%F %T')| task=${_TASK_ID} sessionId=${_SESSION_ID} versionId=${_VERSION_ID} pkg=$pkg ver=$ver op_mode=$mode" >> $outputFile

        return 0
   fi
}

function save_server_id {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local outputFile=${3:-$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$SERVER_ID_FILE}

   echo $(get_server_id_simple) > $outputFile

   return

}

function save_pkg_verId {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local outputFile=${3:-$PKG_HOME_DIR/$pkg-$ver/$PKG_VERSION_ID_PATH}

   echo "${_VERSION_ID}" > $outputFile

   return
}


function save_pkg_name {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local outputFile=${3:-$PKG_HOME_DIR/$pkg-$ver/$PKG_NAME_PATH}

   echo "$pkg" > $outputFile

   return
}


function save_pkg_verName {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local outputFile=${3:-$PKG_HOME_DIR/$pkg-$ver/$PKG_VERSION_NAME_PATH}

   echo "$ver" > $outputFile

   return

}

function save_cfg_cron {

   local cfgFile=${1:?$FUNCNAME [ERR ] | cfg file is NULL}

   local pkg ver

   local outputFile

   local installDir

   local rc

   local user

   local fd

   local line

   pkg=$(get_item_cfg "base_info" "$cfgFile" "name") || return $? 

   ver=$(get_item_cfg "base_info" "$cfgFile" "version") || return $? 

   user=$(get_and_chk_user "$cfgFile") || return $?
   
   installDir="${PKG_HOME_DIR}/$pkg-$ver"

   outputFile="${PKG_HOME_DIR}/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CRONTAB_CONF_FILE"

   ( save_section_cfg "crontab" "$cfgFile" "$outputFile" 644) || return $CODE_SAVE_SECTION_CFG_FAIL 

   sed -r -i '/^[[:space:]]*$/d' $outputFile

   sed -r -i "s:${INSTALL_PATH_PLACE_HOLDER}:${installDir}:g" $outputFile

   sed -r -i "s:/+:/:" $outputFile

   sed -r -i "s/^[[:space:]]+//" $outputFile

   #-----------------------------------------------------#
   # for history reason , log clear & md5sum cron entry  #
   # did NOT work , but still include in the init.conf . #
   # so I remove them here                               #
   #-----------------------------------------------------#

   sed -i -r -e '/\/data\/services\/.+\/admin\/clear.sh/d' \
           -e '/\/data\/services\/.+\/admin\/md5sum.sh/d' \
           -e '/^# +\/data\/services\/.+ log clear/d' \
           -e '/^# +\/data\/services\/.+ md5 monitor/d'  $outputFile && \

   debug_log "log clear & md5sum cron entry removed (history reason)"

   fd=$(get_available_fd)

   if [ -z "$fd" ] ; then

      brief_err_log "get avaialbe FD failed !!!"

      return $CODE_GET_FD_FAILED
   fi

   eval "exec $fd<$outputFile" && \

   while read -u $fd line ; do

       x=$(echo "$line" | awk '{print $6}')

       if [ "${line#\#}" != "$line" ] ; then

            # comment line

            echo "$line"

       elif id $x &>/dev/null ; then

            debug_log "found valid run user [$x] for cron entry [$(echo "$line" | awk '{print $7}')]"

            echo "$line"
       else
            echo "$line" | awk -v runuser=$user '{ $5=$5" "runuser; print }'
       fi

   done > $outputFile.$$ && \

   eval "exec $fd<&-" && \

   if ! grep -q -F "*/2 * * * * $user $installDir/admin/report.sh" $outputFile.$$ ; then

        info_log "no resource monitor crontab , add default resource monitor crontab for $pkg-$ver"

        sed -i -r -e "/^#.* resource report|admin\/report.sh/d" $outputFile.$$ && \

        echo "# $installDir resource report" >> $outputFile.$$ && \

        echo "*/2 * * * * $user $installDir/admin/report.sh > $installDir/admin/report.log 2>&1" >> $outputFile.$$
   else
        info_log "already have default resource monitor crontab , no need to add"

   fi && \

   mv ${outputFile}.$$ $outputFile && \

   if [ $? -eq 0 ] ; then

        info_log "insert run user $user to $outputFile succeed"
   
        return 0
   else
        brief_err_log "insert run user $user to $outputFile failed"   

        return $CODE_INSERT_CRON_FAIL 
   fi
}



function add_cron {

   local cfgFile=${1:?$FUNCNAME [ERR ] | cfg file is NULL}

   local pkg ver 

   local installDir

   local inputFile

   local rc

   pkg=$(get_item_cfg "base_info" "$cfgFile" "name")    || return $? 

   ver=$(get_item_cfg "base_info" "$cfgFile" "version") || return $? 

   inputFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CRONTAB_CONF_FILE"

   if [ ! -f $inputFile ] ; then
 
        info_log "$inputFile not exist , call 'save_cfg_cron' function"

        ( save_cfg_cron "$cfgFile" ) || return $? 
   else
        info_log "$inputFile exist , no need to generate"

        info_log "$(ls -l $inputFile)"
   fi

   # crontab entry syntax check

   while read line ; do

       x=$(echo "$line" | awk '$1 !~ /[ 	]*#/ && NF < 7')

       if [ -n "$x" ] ; then

            brief_err_log "cron entry [$line] syntax err"

            return $CODE_USER_CRON_ENTRY_INVALID
       else
            if ! grep -q -x -F "$line" /etc/crontab ; then

                echo "$line" >> /etc/crontab

                info_log "add entry [$line]" 
           else
                info_log "entry [$line] existed" 
           fi                    
       fi

  done < $inputFile

  rc=$?

  # enforce to remove .no-autostart file under admin/

  info_log "add cron & clean $NO_AUTO_START_FLAG_FILE"

  info_log "$(rm -fv $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$NO_AUTO_START_FLAG_FILE)"

  return $rc
}



function del_cron {

   local cfgFile=${1:?$FUNCNAME [ERR ] | cfg file is NULL}

   local pkg ver

   local inputFile

   local installDir

   local noAutoStartFlagFile

   pkg=$(get_item_cfg "base_info" "$cfgFile" "name")    || return $? 

   ver=$(get_item_cfg "base_info" "$cfgFile" "version") || return $? 

   inputFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CRONTAB_CONF_FILE"

   noAutoStartFlagFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$NO_AUTO_START_FLAG_FILE"

   info_log "input = $inputFile"

   if ! is_readable_file $inputFile ; then

        brief_err_log "$inputFile read permission denied or not exist"

        return $CODE_READ_PKG_CRON_CONF_FILE_FAIL 
   fi
 
   # do not backup crontab file

   #cp /etc/crontab /etc/crontab.$(date +'%F_%T')

   while read line ; do

        grep -v -F "$line" /etc/crontab > /tmp/crontab.$$ && \

        mv /tmp/crontab.$$ /etc/crontab && \

        info_log "del entry [$line]"

   done < $inputFile 

   # enforce to craete .no-autostart file under admin/

   info_log "del cron & set $NO_AUTO_START_FLAG_FILE"

   echo "$(date +'%F %T') action=${_ACTION} taskId=${_TASK_ID} sessionId=${_SESSION_ID}" > $noAutoStartFlagFile

   info_log "$(ls -l $noAutoStartFlagFile)"

   info_log "$(cat $noAutoStartFlagFile)"
 
   return 
}



function install_scripts {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgFile=${3:?$FUNCNAME [ERR ] | cfg file is NULL}

   local md5CfgFile=${4:?$FUNCNAME [ERR ] | md5 cfg file is NULL}

   local mode=${5:?$FUNCNAME [ERR ] | installation mode is NULL}

   local user

   local rc

   local userLoginShell

   # remove any pre-backup of this version under $PKG_BACKUP_DIR

   rm -fr $PKG_BACKUP_DIR/$pkg-$ver &>/dev/null

   user=$(get_and_chk_user "$cfgFile") || return $?

   # no matter start app is yes or no , check user login shell . Or maybe can NOT update or uninstall next time

   userLoginShell=$(chk_user_shell "$user") || return $?

   ( save_section_cfg "clear_file" $cfgFile "${PKG_HOME_DIR}/$pkg-$ver/$PKG_CLEAR_CONF_PATH" 644 ) || return $?

   ( save_section_cfg "start" "$cfgFile" "${PKG_HOME_DIR}/$pkg-$ver/$PKG_START_SCRIPT_PATH" 755 )  || return $?

   ( insert_common_code "$PKG_HOME_DIR/$pkg-$ver/$PKG_START_SCRIPT_PATH" "$pkg" "$ver" ) || return $?

   ( bash_syn_chk "$PKG_HOME_DIR/$pkg-$ver/$PKG_START_SCRIPT_PATH" ) || return $?

   ( save_section_cfg "stop" "$cfgFile" "${PKG_HOME_DIR}/$pkg-$ver/$PKG_STOP_SCRIPT_PATH" 755 ) || return $?

   ( insert_common_code "$PKG_HOME_DIR/$pkg-$ver/$PKG_STOP_SCRIPT_PATH" "$pkg" "$ver" )  || return $? 

   ( bash_syn_chk "$PKG_HOME_DIR/$pkg-$ver/$PKG_STOP_SCRIPT_PATH" ) || return $?

   # /** special treatment for <restart></restart> section begin

   ( save_section_cfg_N "restart" "$cfgFile" "${PKG_HOME_DIR}/$pkg-$ver/$PKG_RESTART_SCRIPT_PATH" 755 ) || return $?

    ( bash_syn_chk "$PKG_HOME_DIR/$pkg-$ver/$PKG_RESTART_SCRIPT_PATH" ) || return $?

   # special treatment for <restart></restart> section end **/

   ( save_section_cfg "monitor" "$cfgFile" "${PKG_HOME_DIR}/$pkg-$ver/$PKG_MONITOR_SCRIPT_PATH" 755 ) || return $? 

   ( insert_common_code "$PKG_HOME_DIR/$pkg-$ver/$PKG_MONITOR_SCRIPT_PATH" "$pkg" "$ver" ) || return $? 

   ( bash_syn_chk "$PKG_HOME_DIR/$pkg-$ver/$PKG_MONITOR_SCRIPT_PATH" ) || return $?

   # /** special treatment for resolve.sh begin

   ( save_section_cfg "resolve" "$cfgFile" "${PKG_HOME_DIR}/$pkg-$ver/$PKG_RESOLVE_SCRIPT_PATH" 755 ) || return $?

   ( insert_comm_resol_code "$PKG_HOME_DIR/$pkg-$ver/$PKG_RESOLVE_SCRIPT_PATH" "$pkg" "$ver") || return $?

   ( bash_syn_chk "$PKG_HOME_DIR/$pkg-$ver/$PKG_RESOLVE_SCRIPT_PATH" ) || return $?

   # special treatment for resolve.sh end **/

   ( make_link_to_dir "${PKG_HOME_DIR}/$pkg-$ver/${PKG_SYMBOL_DATA_DIR}" "${PKG_DATA_DIR}/$pkg" ) || return $?

   #--------------------------------------------------------------------------------------------------#
   # do NOT enable symbol log/ , this may cause conflict with syslog-ng auto-roate set by wu-hai-ting #
   #--------------------------------------------------------------------------------------------------#

   #( make_link_to_dir "${PKG_HOME_DIR}/$pkg-$ver/${PKG_SYMBOL_LOG_DIR}" "${PKG_LOG_DIR}/$pkg-$ver" ) || return $?

   ( gen_md5sum_list "$cfgFile" "$md5CfgFile" ) || return $? 

   ln -s "$PUB_SCRIPT_MD5_MONITOR_PATH" "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PUB_SCRIPT_MD5_MONITOR_SCRIPT" ; rc=$?

   if [ $rc -ne 0 ] ; then

      brief_err_log "link $PUB_SCRIPT_MD5_MONITOR_PATH to $PKG_HOME_DIR/$PKG_SCRIPT_DIR failed"

      return $rc
   fi

   ln -s "$PUB_SCRIPT_RES_REPORT_PATH" "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_RES_REPORT_SCRIPT" ; rc=$?

   if [ $rc -ne 0 ] ; then

        brief_err_log "link $PUB_SCRIPT_RES_REPORT_PATH to $PKG_HOME_DIR/$PKG_SCRIPT_DIR failed"

        return $rc
   fi

   ln -s "$PUB_SCRIPT_ALARM_PATH" "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_ALARM_SCRIPT" ; rc=$?

   if [ $rc -ne 0 ] ; then

        brief_err_log "link $PUB_SCRIPT_ALARM_PATH to $PKG_HOME_DIR/$PKG_SCRIPT_DIR failed"

        return $rc
   fi

   ( save_op_mode "$pkg" "$ver" "$mode" ) || return $?

   ( save_pkg_verId "$pkg" "$ver" )

   ( save_pkg_name "$pkg" "$ver" )

   ( save_pkg_verName "$pkg" "$ver" )

   ( save_server_id "$pkg" "$ver" )

   mkdir -p $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CACHE_DATA_DIR &>/dev/null

   mkdir -p $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CACHE_DATA_DIR/$S2S_DIR &>/dev/null

   find $PKG_HOME_DIR/$pkg-$ver -type f -exec file {} \; |egrep -i 'executable' |awk -F ':' '{print $1}' |xargs -r chmod 755

   chmod -R 755 $PKG_HOME_DIR/$pkg-$ver/bin   &>/dev/null

   chmod -R 755 $PKG_HOME_DIR/$pkg-$ver/sbin  &>/dev/null

   chmod 777 $PKG_HOME_DIR/$pkg-$ver/unittest/*/data &>/dev/null

   chmod -R u+r $PKG_HOME_DIR/$pkg-$ver       &>/dev/null

   # chown all files to $user

   if [ "$user" != "root" ] ; then

        info_log "run user [$user] != root , need chown"

        info_log "chown $PKG_HOME_DIR/$pkg-$ver and ${PKG_DATA_DIR}/$pkg to [$user] now"

        chown -R $user $PKG_HOME_DIR/$pkg-$ver

        chown -R $user ${PKG_DATA_DIR}/$pkg

        chown -R $user ${PKG_LOG_DIR}/$pkg-$ver
   else
        info_log "run user = root , no need to chown"
   fi

   return 0 

}



function move_bad_pkg_dir {
  
   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   rm -fvr $PKG_BAD_DIR/$pkg-$ver && sleep 5 && mv -vf $PKG_HOME_DIR/$pkg-$ver $PKG_BAD_DIR

   if [ ! -e $PKG_HOME_DIR/$pkg-$ver ] && [ -d $PKG_BAD_DIR/$pkg-$ver ] ; then

        info_log "move dir $PKG_HOME_DIR/$pkg-$ver to $PKG_BAD_DIR succeed"

        info_log "$(ls -ld $PKG_HOME_DIR/$pkg-$ver 2>&1)"

        info_log "$(ls -l $PKG_BAD_DIR/$pkg-$ver 2>&1)"

        cd $PKG_BAD_DIR/$pkg-$ver/$PKG_SCRIPT_DIR && echo 'bad' > $PKG_DIR_STATUS_FILE
   fi

   return
}



function install_pkg {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local md5=${3:?$FUNCNAME [ERR ] | pkg md5 value is NULL}

   local startApp=${4:?$FUNCNAME [ERR ] | startApp is NULL}

   local urls=${5:?$FUNCNAME [ERR ] | URLs is NULL}

   local rate=${6:?$FUNCNAME [ERR ] | rate is NULL}

   local platform=${7:?$FUNCNAME [ERR ] | platform is NULL}

   local cfgmd5=${8:?$FUNCNAME [ERR ] | cfgmd5 is NULL}

   local cfgpkgurls=${9:?$FUNCNAME [ERR ] | cfgpkgurls is NULL}
   
   local cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

   local md5CfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_MD5_CONF_FILE

   local pkgFile=$PKG_DOWNLOAD_DIR/$(get_pkg_fullname "$pkg" "$ver" "$platform" "binary")

   local startAppScriptFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_START_SCRIPT_PATH

   local scriptFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_POST_INSTALL_SCRIPT_PATH

   local logFile user startApp rc syslogFile appLogFiles appName pkgLogFile
   
   local autoTestLogFile autoTestResponseFile localIp cfgId

   # pre-check

   ( install_precheck "$pkg" "$ver" ) ; rc=$?

   if [ $rc -ne 0 ] ; then

      brief_err_log "install pre-check failed" ; return $rc;
   fi

   # download pkg if needed

   ( download_pkg "$pkg" "$ver" "$md5" "$urls" "$platform" "$PKG_DOWNLOAD_DIR" "binary" "$rate" )

   rc=$?

   if [ $rc -ne 0 ] ; then

      brief_err_log "download pkg failed"

      return $CODE_PKG_DOWNLOAD_FAIL
   fi

   # un-tar and install files

   ( install_files "$pkgFile" "$PKG_HOME_DIR" "$pkg" "$ver" )

   rc=$?

   if [ $rc -ne 0 ] ; then

      ( move_bad_pkg_dir "$pkg" "$ver" )

      brief_err_log "install files failed"

      return $rc 
   fi

   # download && install cfg pkg if needed

   if [ "$DO_CFG_MGR" = "yes" ] ; then

        ( cfg_mgr_common "$pkg" "$ver" "$cfgmd5" "$cfgpkgurls" )

        rc=$?

        if [ $rc -ne 0 ] ; then
     
             ( move_bad_pkg_dir "$pkg" "$ver" )
     
             brief_err_log "install config pkg failed"
     
             return $rc
        else
             info_log "install config pkg succeed"

             info_log ""
        fi
   else
        info_log "DO_CFG_MGR != yes , no need to do config management job"

        info_log ""
   fi

   # generate common environment variables file

   ( gen_comm_env_var "$pkg" "$ver" "install" "-" )

   rc=$?

   if [ $rc -ne 0 ] ; then

      ( move_bad_pkg_dir "$pkg" "$ver" )

      brief_err_log "$(cat ${_BRIEF_LOG})"
 
      return $CODE_GEN_COMM_VAR_FILE_FAIL
   fi

   # save & run post-install script

   ( save_section_cfg "post_install" "$cfgFile" "${PKG_HOME_DIR}/$pkg-$ver/$PKG_POST_INSTALL_SCRIPT_PATH" 755 )

   if [ $? -ne 0 ] ; then
 
      ( move_bad_pkg_dir "$pkg" "$ver" )

      brief_err_log "save post-install script failed" 

      return $CODE_INSERT_COMMON_CODE_FAIL
   fi 

   ( insert_common_code "$PKG_HOME_DIR/$pkg-$ver/$PKG_POST_INSTALL_SCRIPT_PATH" "$pkg" "$ver" )

   if [ $? -ne 0 ] ; then

      ( move_bad_pkg_dir "$pkg" "$ver" )

      brief_err_log "save post-install script failed" 
 
      return $CODE_INSERT_COMMON_CODE_FAIL
   fi 

   # syntax check first

   ( bash_syn_chk "$scriptFile" ) 

   if [ $? -ne 0 ] ; then

      ( move_bad_pkg_dir "$pkg" "$ver" )

      brief_err_log "found syntax error in $scriptFile" 

      return $CODE_POST_INSTALL_SCRIPT_SYNTAX_ERR 
   fi

   ( run_post-install "$pkg" "$ver" )

   if [ $? -ne 0 ] ; then

      ( move_bad_pkg_dir "$pkg" "$ver" )

      brief_err_log "run post-install script failed"
 
      return $CODE_RUN_POST_INSTALL_SCRIPT_FAIL
   fi

   info_log ""

   # replace common variables

   if [ "$ENABLE_REPLACE_PATTERN" = 'yes' ] ; then

        ( replace_pattern "$pkg" "$ver" )
   
        if [ $? -ne 0 ] ; then
   
             ( move_bad_pkg_dir "$pkg" "$ver" )
   
             brief_err_log "replace common variables failed"
   
             return $CODE_GET_HOST_INFO_FAIL
        fi
   else
        warn_log "replace pattern disabled , skip ..."
   fi

   info_log ""

   # begin to install other things

   ( install_scripts "$pkg" "$ver" "$cfgFile" "$md5CfgFile" "install" )

   if [ $? -ne 0 ] ; then

      # we DON'T have to del_cron , because cron entry not installed yet ! 

      # so we just need to move the bad pkg dir

      ( move_bad_pkg_dir "$pkg" "$ver" )

      #brief_err_log "$(cat ${_BRIEF_LOG})"

      return $CODE_INSTALL_COMMON_FILES_FAIL_UNDER_INSTALL
   fi 

   # save crontab entry

   ( save_cfg_cron "$cfgFile" )

   if [ $? -ne 0 ] ; then

      brief_err_log "save cron config failed"

      return $CODE_SAVE_CRON_FAIL
   fi 

   info_log ""

   # start app if user required
   
   if   [ "$startApp" = 'yes' ] ; then
   
        info_log "startApp = yes"
      
        user=$(get_and_chk_user "$cfgFile") || return $?
  
        # when install succeed , but start app failed , return specific code : 100 (update db)

        ( start_stop_app "$pkg" "$ver" "$user" "start" ) ; rc=$?

        # collect app log

        collect_app_log "$pkg" "$ver" "start" "1"

        if [ $rc -ne 0 ] ; then

           # enforce to stop app ( for example : count set to 5 , but only 3 started , need to kill !!!

           err_log ""

           err_log "$(repeatStr '-' 19)  start $pkg-$ver failed , clean  $(repeatStr '-' 19)"

           info_log "enforce to stop any new version app if running"

           ( start_stop_app "$pkg" "$ver" "$user" "stop" )

           ( move_bad_pkg_dir "$pkg" "$ver" )

	       brief_err_log "start [$pkg-$ver] failed , move to $PKG_BAD_DIR"

   	       return $CODE_START_APP_FAIL_UNDER_INSTALL
        fi

        localIp=$(get_local_ip "eth0") 

        autoTestLogFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$AUTO_TEST_LOG_DIR/$CALL_AUTO_TEST_LOG"

        autoTestResponseFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$AUTO_TEST_LOG_DIR/$CALL_AUTO_TEST_RESPONSE_FILE"

        mkdir -p $(dirname $autoTestLogFile) &>/dev/null

        nohup wget --output-document=$autoTestResponseFile -a $autoTestLogFile -t 2 -w 5 --timeout=5 "${AUTO_TEST_URL}?taskid=${_TASK_ID}&ip=$localIp" &

        # save pkg cfg Id

        cfgId=$(get_current_task_cfg_id "$pkg" "$ver")

        ( save_cfg_id "$pkg" "$ver" "$cfgId" )

        brief_ok_log "install && start pkg done"
   else
        # save pkg cfg Id

        cfgId=$(get_current_task_cfg_id "$pkg" "$ver")

        ( save_cfg_id "$pkg" "$ver" "$cfgId" )

        info_log "startApp = no"

        brief_ok_log "install pkg done" 
   fi

   return 0

}



function install_cfg_pkg {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgmd5=${3:?$FUNCNAME [ERR ] | cfgmd5 is NULL}

   local cfgpkgurls=${4:?$FUNCNAME [ERR ] | cfgpkgurls is NULL}

   local dstDir="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/cfg"

   local pkgFullName

   mkdir -p $dstDir

   # download pkg if needed

   info_log "\$DO_CFG_MGR = [$DO_CFG_MGR] , prepare to download config pkg"

   ( download_pkg "$pkg" "$ver" "$cfgmd5" "$cfgpkgurls" "noarch" "$PKG_DOWNLOAD_DIR" "config" "$rate" )

   rc=$?

   if [ $rc -ne 0 ] ; then

      brief_err_log "download cfg-pkg failed , maybe /data partition is full"

      return $CODE_PKG_DOWNLOAD_FAIL
   fi

   # extract to the cfg/ dir under log/<taskid>/<sessionId>/<versionId> dir

   pkgFullName=$(get_pkg_fullname "$pkg" "$ver" "noarch" "config") 

   echo

   echo "$(repeatStr '-' 5) Config Pkg Info $(repeatStr '-' 5)"

   tar -C $dstDir -zxvf $PKG_DOWNLOAD_DIR/$pkgFullName

   rc=$?

   echo

   if [ $rc -ne 0 ] ; then

        brief_err_log "un-tar config-pkg [$pkgFullName] to $dstDir failed" 

        return $CODE_CFG_PKG_EXTRACT_FAIL
   else
        brief_ok_log "un-tar config-pkg [$pkgFullName] succeed" 

        return 0

  fi

}



function add_url_suffix {

   local cfgPkgUrls=${1:?$FUNCNAME [ERR ] | cfgPkgUrls is NULL}

   echo "$cfgPkgUrls" | xargs -d ',' -n 1 | egrep -v '^[[:space:]]*$' | sed -r '/^http:\/\/.*/s/$/?type=config/' | xargs | sed 's/ /,/g'

   return
}


function get_cfg_id_by_pkg_ver {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgIdFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_ID_PATH"

   local x

   debug_log "cfgIdFile=$cfgIdFile $(ls -l $cfgIdFile)"

   x=$(cat $cfgIdFile 2>/dev/null)

   if [ -z "$x" ] ; then
  
        debug_log "empty cfg Id for $cfgIdFile , force to 0"
   else 
        debug_log "cfgId=$x"
   fi

   echo "${x:=0}"

   return $?
}


function get_current_task_cfg_id {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgMgrFile="$PUB_SCRIPT_LOG_DIR/${_TASK_ID}/${_SESSION_ID}/${_VERSION_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   local cfgId

   if [ ! -f $cfgMgrFile ] && [ "$DO_CFG_MGR" != "yes" ]; then

        # means use default config or copy old config from old Ver

        if [ "${_ACTION}" = "install" ] ; then

              cfgId=0

        elif [ "${_ACTION}" = "update" ] ; then

             cfgId=$(get_cfg_id_by_pkg_ver "$pkg" "$ver")
        fi
   else
        cfgId=$(get_item_cfg "base_info" "$cfgMgrFile" "cfgId")
   fi

   if [ -z "$cfgId" ] ; then

        err_log "empty cfgId for $cfgMgrFile , enforce to [0]"

        cfgId=0
   fi 

   echo "$cfgId"

   return
}


function save_cfg_id {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgId=${3:?$FUNCNAME [ERR ] | cfgId is NULL}

   local cfgIdFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CFG_ID_FILE"

   echo "$cfgId" > $cfgIdFile

   return
}


function gen_cfg_file_list {

   local cfgFileDir="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/cfg"

   local x

   cd $cfgFileDir && ls -1 | grep -v '^cfgmgr.conf$' | sed -r 's!#@#!/!g'

   return $?
}



function get_inherit_rules {

   local cfgFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   local inheritRules

   inheritRules=$(get_section_cfg "inherit_rules" "$cfgFile" | egrep -v '^#|^[[:space:]]*$' ) || return $?

   echo "$inheritRules"
}



function get_inherit_list {

   local cfgFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   local inheritList

   inheritList=$(get_inherit_rules | sed -r 's!#@#!/!g') || return $?

   echo "$inheritList"

   return
}



function gen_overwrite_list {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local allList overwriteList inheritList

   allList=$(gen_cfg_file_list) || return $?

   inheritList=$(get_inherit_list)

   overwriteList=$(join -v 1 <(echo "$allList" |sort) <(echo "$inheritList"|sort))

   echo "$overwriteList"

   return
}



function overwrite_cfg_file {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local allList overwriteList inheritList srcFile dstFile oldMd5 newMd5

   info_log "begin to overwrite"

   info_log "" 

   info_log "--- all file list ---"

   allList=$(gen_cfg_file_list)

   while read line ; do

      info_log "$line"

   done <<< "$allList"

   info_log ""

   info_log "--- all inherit file list ---"

   inheritList=$(get_inherit_list)

   while read line ; do

      info_log "$line"

   done <<< "$inheritList"

   info_log ""

   info_log "--- all overwrite file list ---"

   overwriteList=$(gen_overwrite_list "$pkg" "$ver")

   while read line ; do

      info_log "$line"

   done <<< "$overwriteList"

   # begin to overwrite the contents

   if ! cd $(dirname $cfgFile) ; then

      brief_err_log "cd cfgfile dir $(dirname $cfgFile) failed"

      return $CODE_CFG_TMP_DIR_ACCESS_FAIL
   fi

   info_log ""

   # first backup all overwrited files

   for i in $overwriteList ; do

       srcFile="$PKG_HOME_DIR/$pkg-$ver/$i"

       dstFile="${srcFile}.old"

       if [ ! -f "$srcFile" ] ; then

            warn_log "$(ls -l $srcFile 2>&1)"

            #brief_err_log "$srcFile is NOT a FILE or not exist !!!"

            warn_log "$srcFile is NOT a FILE or not exist , no need to backup !!!"

            #return $CODE_BACKUP_OLD_CFG_FILE_FAIL

       elif [ -e "$dstFile" ] && [ ! -f "$dstFile" ] ; then

            brief_err_log "$dstFile exist , but NOT a FILE !!!"

            return $CODE_BACKUP_OLD_CFG_FILE_FAIL
       else       
            if cp -f "$srcFile" "$dstFile" ; then 
     
                 debug_log "backup file $srcFile succeed"
            else
                 brief_err_log "backup file $srcFile failed"
     
                 return $CODE_BACKUP_OLD_CFG_FILE_FAIL
            fi
       fi

   done

   info_log ""
 
   # begin to overwrite !!!!!!

   for i in $overwriteList ; do

       # translate the normal filename into the '#@#'-separated name

       info_log "processing $(echo $i | sed -r 's!/!#@#!g') ...."

       srcFile="$(pwd)/$(echo $i | sed -r 's!/!#@#!g')"

       dstFile="$PKG_HOME_DIR/$pkg-$ver/$i"

       oldMd5=$(md5sum $dstFile 2>/dev/null | awk '{print $1}')

       newMd5=$(md5sum $srcFile 2>/dev/null | awk '{print $1}')
            
       info_log "srcfile = $srcFile"
     
       info_log "dstFile = $dstFile"

       if [ "${oldMd5:=<empty>}" != "$newMd5" ] ; then

            info_log "begin to [overwrite] for file $i"
     
            if cp --preserve=mode,ownership -f $srcFile $dstFile ; then
     
                 info_log "old md5 = [$oldMd5] , new md5 = [$newMd5] , overwrite succeed"
     
                 #info_log "----- output diff result ( new .VS. old ) -----"

                 #while read line ; do

                 #    info_log "$line"

                 #done < <(diff $dstFile $dstFile.old)
            else
                 brief_err_log "overwrite failed"

                 return $CODE_OVERWRITE_CFG_FILE_FAIL
            fi
       else
            info_log "md5 not changed , no need to overwrite"

            rm -f "${srcFile}.old"
       fi

       info_log ""

   done

   return
}



function inherit_cfg_file {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   local inheritList srcFile dstFile inheritRules inheritVer inheritFile keyword

   if [ "$ACTION_MODE" != "update" ] ; then

       brief_err_log "can NOT inherit file under [$ACTION_MODE] mode"

       return $CODE_CFG_INHERIT_NOT_UPDATE 
   fi

   inheritVer=$(get_item_cfg "base_info" "$cfgFile" "srcVersion") || return $?

   info_log "begin to inherit from [$pkg-$inheritVer]"

   info_log ""

   inheritRules=$(get_inherit_rules)

   if [ -n "$inheritRules" ] ; then

        while read inheritFile ; do

              srcFile="$PKG_HOME_DIR/$pkg-$inheritVer/$(echo $inheritFile | sed -r 's!#@#!/!g')"

              dstFile="$PKG_HOME_DIR/$pkg-$ver/$(echo $inheritFile | sed -r 's!#@#!/!g')"

              if cp -af $dstFile $dstFile.old ; then

                   debug_log "backup file $dstFile succeed"
              else
                   brief_err_log "backup file $dstFile failed"

                   return $CODE_BACKUP_OLD_CFG_FILE_FAIL
              fi

              debug_log "srcfile = $srcFile"

              debug_log "dstFile = $dstFile"

              if ! cp -af $srcFile $dstFile ; then

                   brief_err_log "copy cfg file $srcFile failed"

                   return $CODE_CFG_COPY_OLD_VER_FILE_FAIL
              else
                   info_log "src file md5 = [$(md5sum $srcFile)]"

                   info_log "dst file md5 = [$(md5sum $dstFile)]"
                 
                   info_log "inherit file succeed"

                   info_log ""
              fi

         done <<< "$inheritRules"

   else
         info_log "NO inherit rules found"
   fi

   return

}



function run_reload_script {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   local initFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

   local user

   local userLoginShell

   local rc

   scriptFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_RELOAD_SCRIPT_PATH"

   # For history reason , the run user item named 'runUser'

   # try get user from init.conf

   user=$(get_item_cfg "base_info" "$cfgFile" "runUser" | cut -d ':' -f 1)

   # try get user from init.conf if can get user info

   if [ -z "$user" ] ; then

      info_log "try to get user info from $initFile"

      user=$(get_item_cfg "base_info" "$initFile" "user" | cut -d ':' -f 1)
 
      rc=$?
   fi   

   if [ -z "$user" ] ; then

        brief_err_log "can NOT get 'user' info"

        return $rc
   else
        # check whether user exist

        if id $user &>/dev/null ; then

             info_log "run user = [$user]"
        else
             brief_err_log "user [$user] not exist"

             return $CODE_USER_INVALID 
        fi
   fi

   userLoginShell=$(chk_user_shell "$user") || return $?
   
   ( save_section_cfg "reload" "$cfgFile" "$scriptFile" 755 ) || return $?

   ( insert_common_code "$scriptFile" "$pkg" "$ver" ) || return $?

   chown $user "$scriptFile"

   ( bash_syn_chk "$scriptFile" )

   if [ $? -ne 0 ] ; then

      brief_err_log "found syntax error in $scriptFile"

      return $CODE_POST_INSTALL_SCRIPT_SYNTAX_ERR
   fi

   info_log "---------------- reload ----------------"

   set -x

   su - $user -c "export ACTION_MODE=${_ACTION}; bash $scriptFile 2>&1"

   rc=$?

   set +x

   if [ $rc -ne 0 ] ; then

        brief_err_log "config applied , but run reload script failed"

        return $CODE_RUN_RELOAD_SCRIPT_FAIL 
   else
        info_log "run reload script succeed"

        return 0
   fi

}



function get_cfg_pkg_info {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   local cfgName cfgAuthor

   cfgName=$(get_item_cfg "base_info" "$cfgFile" "cfgName")  || return $?

   cfgAuthor=$(get_item_cfg "base_info" "$cfgFile" "author") || return $?

   info_log "cfg name = [$cfgName]"

   info_log "cfg author = [$cfgAuthor]"

   return
}



function cfg_mgr_common {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgmd5=${3:?$FUNCNAME [ERR ] | cfgmd5 is NULL}

   local cfgpkgurls=${4:?$FUNCNAME [ERR ] | cfgpkgurls is NULL}

   local cfgFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   # step0 - determine if src dir exist

   if [ ! -d "$PKG_HOME_DIR/$pkg-$ver" ] ; then

        brief_err_log "dir $PKG_HOME_DIR/$pkg-$ver not exist"

        return $CODE_DIR_NOT_EXIST_UNDER_RELOAD
   fi

   # step1 - download cfg pkg
 
   info_log ""

   info_log "downloading config pkg ...."

   ( install_cfg_pkg "$pkg" "$ver" "$cfgmd5" "$cfgpkgurls" )

   rc=$?

   if [ $rc -ne 0 ] ; then
   
       brief_err_log "install config pkg failed"
   
       return $rc
   fi

   info_log "config pkg basic info"

   ( get_cfg_pkg_info "$pkg" "$ver" )

   rc=$?

   if [ $rc -ne 0 ] ; then

       brief_err_log "get config pkg basic info failed" 

       return $rc 
   fi 

   # step2 - overwrie files

   ( overwrite_cfg_file "$pkg" "$ver" )

   rc=$?

   if [ $rc -ne 0 ] ; then

      brief_err_log "overwrite cfg file failed"

      return $CODE_CFG_PKG_EXTRACT_FAIL 

   fi

   # step3 - inherit files if needed

   if [ "$ACTION_MODE" = "update" ] ; then

        ( inherit_cfg_file "$pkg" "$ver" )

        rc=$?

        if [ $rc -ne 0 ] ; then

             brief_err_log "inherit cfg file failed"

             return $CODE_PKG_CFG_INHERIT_FAIL 
        fi
   else
        info_log "current action mode is $ACTION_MODE , skip inherit job"
   fi

   return

}



function reload_pkg {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgmd5=${3:?$FUNCNAME [ERR ] | cfgmd5 is NULL}

   local cfgpkgurls=${4:?$FUNCNAME [ERR ] | cfgpkgurls is NULL}

   local cfgFile="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PKG_CFG_TASK_DIR/$PKG_CFG_MGR_FILE"

   local cfgId

   # step1 - do common job

   ( cfg_mgr_common "$pkg" "$ver" "$cfgmd5" "$cfgpkgurls" ) || return $? 

   info_log ""

   # step2 - replace common var

   if [ "$ENABLE_REPLACE_PATTERN" = 'yes' ] ; then

        ( replace_pattern "$pkg" "$ver" )
     
        if [ $? -ne 0 ] ; then
     
           brief_err_log "replace common variables failed"
     
           return $CODE_GET_HOST_INFO_FAIL
        fi
   else
        warn_log "replace pattern disabled , skip ..."
   fi

   info_log ""

   # step3 - save cfg id

   cfgId=$(get_current_task_cfg_id "$pkg" "$ver")

   ( save_cfg_id "$pkg" "$ver" "$cfgId" )

   # step4 - run reload script

   # create .no-autostart flag file in case of cron entriy start app when stopping

   info_log "auto-start forbidden ..."

   info_log "set $NO_AUTO_START_FLAG_FILE"

   echo "$(date +'%F %T') action=$ACTION_MODE global-id=${_GLOBAL_ID}" > $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$NO_AUTO_START_FLAG_FILE

   info_log "$(ls -l $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$NO_AUTO_START_FLAG_FILE)"

   info_log "$(cat $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$NO_AUTO_START_FLAG_FILE)"

   run_reload_script "$pkg" "$ver" 

   if [ $? -ne 0 ] ; then

        return $CODE_RUN_RELOAD_SCRIPT_FAIL
   fi

   info_log "clean $NO_AUTO_START_FLAG_FILE"

   info_log "$(rm -fv $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$NO_AUTO_START_FLAG_FILE)"

   brief_ok_log "reload pkg succeed" 
}



function backup_pkg {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local backupFile="$PKG_BACKUP_DIR/$pkg-$ver"

   if [ -d $PKG_HOME_DIR/$pkg-$ver/ ] ; then

        if is_writable_dir $PKG_BACKUP_DIR ; then

            info_log "remove any pre-backup file"

            # remove the backup dir if exists and move the dir to /data/backup

            info_log "$(rm -rf $backupFile && mv -fv $PKG_HOME_DIR/$pkg-$ver $PKG_BACKUP_DIR)" 

            if [ ! -e $PKG_HOME_DIR/$pkg-$ver ] ; then

                 # touch the backup record file

                 echo "$pkg $ver backup at $(date +'%F %T')" > $PKG_BACKUP_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_BACKUP_RECORD_FILE

                 # confirm job

                 info_log "$(ls -ld $PKG_HOME_DIR/$pkg-$ver 2>&1)"

                 info_log "$(ls -ld $backupFile 2>&1)"

                 return 0
            else
                 # confirm job

                 info_log "$(ls -ld $PKG_HOME_DIR/$pkg-$ver 2>&1)" 
 
                 info_log "$(ls -ld $backupFile 2>&1)"

                 brief_err_log "dir $PKG_HOME_DIR/$pkg-$ver move to $backupFile failed" 

                 return $CODE_BACKUP_OLD_VER_FAIL 
             fi
        else 
             brief_err_log "dir $(dirname $backupFile) write permission denied or not exist, backup failed"

             err_log "$(ls -l $(dirname $backupFile) 2>&1)"

             err_log "unable to backup $pkg-$ver dir "

             return $CODE_BACKUP_DIR_ACCESS_FAIL
        fi
   else
        brief_err_log "src dir $PKG_HOME_DIR/$pkg-$ver not exist" ; 

        err_log "$(ls -l $PKG_HOME_DIR/$pkg-$ver 2>&1)"

        return $CODE_BACKUP_SRC_DIR_NOT_EXIST 
   fi

}



function uninstall_pkg {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

   local crontabCfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CRONTAB_CONF_FILE

   local rc
  
   if [ ! -e $PKG_HOME_DIR/$pkg-$ver ] ; then

        info_log "dir [$PKG_HOME_DIR/$pkg-$ver] not exist"

        info_log "$(ls -l $PKG_HOME_DIR/$pkg-$ver 2>&1)"

        brief_ok_log "uninstall pkg done" 

        return 0 
   else
        user=$(get_and_chk_user "$cfgFile")

        if [ -z "$user" ] ; then

           ( move_bad_pkg_dir "$pkg" "$ver" )

           brief_err_log "get user info failed"

           return $CODE_USER_INVALID 
        fi

        ( start_stop_app "$pkg" "$ver" "$user" "stop" ) ; rc=$?

        # collect log no matter succeed or failed

        collect_app_log "$pkg" "$ver" "stop" "1"

        if [ $rc -ne 0 ] ; then

           err_log "stop $pkg-$ver failed , add cron back to restore old environment !!!" 

           ( add_cron "$cfgFile" )
 
           brief_err_log "stop $pkg-$ver failed"

           return $CODE_STOP_APP_FAIL_UNDER_UNINSTALL
        fi

        # move the dir to /data/backup

        ( backup_pkg "$pkg" "$ver" ) || return $?

        info_log "pkg dir [$PKG_HOME_DIR/$pkg-$ver] removed"

        # add a record to install_file to indicate that this dir has been moved to /data/backup

        ( save_op_mode "$pkg" "$ver" "backup_on_uninstall/rollback" "$PKG_BACKUP_DIR/$pkg-$ver/$PKG_INSTALL_MODE_PATH" )

        brief_ok_log "uninstall pkg done" 

        return 0
   fi

}
 


function start_stop_app {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local user=${3:?$FUNCNAME [ERR ] | user is NULL}

   local mode=${4:?$FUNCNAME [ERR ] | mode is NULL}

   shift 4 

   local appList="$*"

   local cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

   local scriptFile

   local i  rc serverId cmdbIpList chosenIp firstStartFlagFile

   local line appName appFile hostStatus userLoginShell pids

   if [ ! -d "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR" ] ; then

       brief_err_log "dir [$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR] not exist"

       err_log "$(ls -l $PKG_HOME_DIR/$pkg-$ver 2>&1)"

       return $CODE_DIR_NOT_EXIST_UNDER_START_STOP
   fi

   # NOTE : when use this function to restart app , it should use V2.0 format init.conf

   # use "restart_app" function to restart app for ver 1.0 init.conf
 
   if [ "$mode" != "start" ] && [ "$mode" != "stop" ] && [ "$mode" != "restart" ] ; then

       brief_err_log "mode can only be 'start' or 'stop' or 'restart'"

       return $CODE_ACTION_INVALID 
   fi

   # set script file to execute

   if [ "$mode" = "start" ] ; then

        # NOTE !! We do not add cron here . we add cron after start app succeed

        scriptFile="$PKG_START_SCRIPT_FILE"

   elif [ "$mode" = "stop" ] ; then

        scriptFile="$PKG_STOP_SCRIPT_FILE"
   else
        scriptFile="$PKG_RESTART_SCRIPT_FILE"
   fi

   info_log "   MODE = [$mode]"

   info_log " ACTION = [${_ACTION}]"

   info_log "RUN USER = [$user]"

   info_log "COPY_CFG = [$COPY_CFG]" 


   # output debug script run log

   info_log "$(repeatStr '-' 19) $mode script $(repeatStr '-' 19)"

   info_log "script = $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$scriptFile"


   # user login shell check

   userLoginShell=$(chk_user_shell "$user") || return $?
   

   # syntax check first
   
   ( bash_syn_chk "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$scriptFile" ) || return $?


   # remove cron entries first when stop or restart app

   if [ "$mode" = "stop" ] || [ "$mode" = "restart" ] ; then

       info_log "remove cron entries first when $mode app"

       ( del_cron "$cfgFile" ) || return $?
   fi


   # real work begin

   touch $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_USER_SCRIPT_LOG

   chmod a+w $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_USER_SCRIPT_LOG 

   if [ -z "$appList" ] ; then

        # DO NOT USE 'bash /path/to/script' form , or it will cause error by limit-run.sh 

        # set -o pipefail

        su - $user -c "export ACTION_MODE=${ACTION_MODE}

                       export ACTION=${_ACTION}

                       cd $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/ && \

                       bash ./$scriptFile 2>&1" > $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_USER_SCRIPT_LOG 2>&1

        rc=$?

        #set +o pipefail
   else
        for i in $appList; do
        
            # DO NOT USE 'bash /path/to/script' form , or it will cause error by limit-run.sh 
        
            #set -o pipefail

            su - $user -c "export ACTION_MODE=${ACTION_MODE}
                       
                           export ACTION=${_ACTION}

                           cd $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR && \

                           bash ./$scriptFile $i 2>&1" > $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_USER_SCRIPT_LOG 2>&1
            ((rc+=$?))

            #set +o pipefail
       
        done
   fi

   info_log "$scriptFile return code = [$rc]"

   info_log "see $PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$PUB_SCRIPT_USER_SCRIPT_LOG"

   # collect log no matter succeed or failed

   # because on install , update ,rollback  mode , app log had collected , 

   # so on these 3 modes , we don't have to re-collect

   if [ "${_ACTION}" = "start" ] || [ "${_ACTION}" = "stop" ] || [ "${_ACTION}" = "restart" ] ; then
   
      collect_app_log "$pkg" "$ver" "${_ACTION}" "1"
   fi
 
   # exception handle : when restart app failed

   if [ $rc -ne 0 ] ; then

        # if mode is restart , even when stop or start app is _FAILED_ , 

        # still need to add cron back , or the app won't start again

        if [ "$mode" = "restart" ] ; then

           warn_log "although restart app failed"

           warn_log "still add cron back !!!"

           ( add_cron "$cfgFile" ) || return $?

        fi 

        brief_err_log "$mode app(${appList:-ALL}) failed"

        return $CODE_START_STOP_ERR
   else
        # everything is ok , almost done

        #--------------------------------------------#
        # add cron when start or restart app succeed #
        #--------------------------------------------#   

        if [ "$mode" = "start" ] || [ "$action" = "restart" ] ; then

           info_log "add cron entries back when $mode app succeed"

           ( add_cron "$cfgFile" ) || return $?

        fi

        #---------------------#
        # CMDB job if enabled #
        #---------------------#

        if [ "$ENABLE_CMDB_REPORT" = 'yes' ] ; then
    
             info_log "CMDB proc report enabled"
    
             ( cmdb_proc_mgr "$pkg" "$ver" "$mode" )
      
             brief_ok_log "$mode app done"
    
             return 0  # always return 0
        else
             warn_log "CMDB proc report disabled"
    
             brief_ok_log "$mode app done (no CMDB)"
        fi
    fi
} 



function restart_app {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | version number is NULL}

   local user=${3:?$FUNCNAME [ERR ] | user is NULL}

   shift 3 

   local appList="$*"

   local cfgFile=$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE

   local scriptFile=${PKG_HOME_DIR}/$pkg-$ver/$PKG_RESTART_SCRIPT_PATH

   local initCfgVerFile=$(cat $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CFG_VER_FILE 2>/dev/null) 

   if [ "$initCfgVerFile" = "1.0" ] || [ -z "$initCfgVerFile" ]  ; then

         # old format , no <restart></restart> section exist

         info_log "no <restart></restart> section found"

         info_log "old format $PKG_CFG_FILE"

         info_log "use default behavior : stop-and-start"

         ( start_stop_app "$pkg" "$ver" "$user" "stop" "$appList" )  || return $?

         sleep 2

         ( start_stop_app "$pkg" "$ver" "$user" "start" "$appList" ) || return $? 

         brief_ok_log "restart app ${appList:-} succeed"

         return 0

   elif  [ "$initCfgVerFile" = "2.0" ] ; then 

         # new format , <restart></restart> section exist

         info_log "found <restart></restart> section"

         info_log "new format $PKG_CFG_FILE"

         info_log "running script $scriptFile"

         ( start_stop_app "$pkg" "$ver" "$user" "restart" "$appList" ) || return $?

          # collect log no matter succeed or failed
          
         collect_app_log "$pkg" "$ver" "restart" "1" 

         return 0 
   else
         brief_err_log "invalid $PKG_CFG_FILE format version [$initCfgVerFile], it should be 1.0 or 2.0" 

         return $CODE_INVALID_INIT_CFG_VERSION
   fi
}



function update_prechk {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local oldVer=${2:?$FUNCNAME [ERR ] | old version number is NULL}

   local newVer=${3:?$FUNCNAME [ERR ] | new version number is NULL}

   if [ -d $PKG_HOME_DIR/$pkg-$newVer ] ; then

         brief_err_log "$pkg new version [$newVer] is already installed"

         err_log "install at [$(tail -1 $PKG_HOME_DIR/$pkg-$newVer/$PKG_SCRIPT_DIR/$PKG_INSTALL_MODE_FILE)]"

         err_log "$(ls -l $PKG_HOME_DIR/$pkg-$newVer 2>&1)"

         return $CODE_UPDATE_DST_VER_ALREAY_EXIST 

   elif [ -f $PKG_HOME_DIR/$pkg-$newVer ] ; then

         brief_err_log "$PKG_HOME_DIR/$pkg-$newVer is a FILE !!!"

         err_log "$(ls -l $PKG_HOME_DIR/$pkg-$newVer 2>&1)"

         return $CODE_UPDATE_DST_VER_ALREAY_EXIST

   elif [ ! -e $PKG_HOME_DIR/$pkg-$oldVer ] ; then

         brief_err_log "$pkg old version [$oldVer] not installed"

         err_log "$(ls -l $PKG_HOME_DIR/$pkg-$oldVer 2>&1)"

         return $CODE_UPDATE_SRC_VER_NOT_EXIST 
   fi 
      
}



function update_pkg {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local oldVer=${2:?$FUNCNAME [ERR ] | old version number is NULL}

   local newVer=${3:?$FUNCNAME [ERR ] | new version number is NULL}

   local md5=${4:?$FUNCNAME [ERR ] | pkg md5 value is NULL}

   local copyCfg=${5:?$FUNCNAME [ERR ] | option "copyCfg" is NULL (yes|no)}

   local order=${6:?$FUNCNAME [ERR ] | option "order" is NULL (kill-first|start-first|noaction)} 

   local urls=${7:?$FUNCNAME [ERR ] | urls is NULL}
 
   local rate=${8:?$FUNCNAME [ERR ] | rate is NULL}

   local startApp=${9:?$FUNCNAME [ERR ] | startApp is NULL}

   local platform=${10:?$FUNCNAME [ERR ] | platform is NULL}

   local cfgmd5=${11:?$FUNCNAME [ERR ] | cfgmd5 is NULL}

   local cfgpkgurls=${12:?$FUNCNAME [ERR ] | cfgpkgurls is NULL}

   local pkgFile=$PKG_DOWNLOAD_DIR/$(get_pkg_fullname "$pkg" "$newVer" "$platform" "binary")

   local newCfgFile=$PKG_HOME_DIR/$pkg-$newVer/$PKG_CFG_FILE

   local oldCfgFile=$PKG_HOME_DIR/$pkg-$oldVer/$PKG_CFG_FILE

   local md5CfgFile=$PKG_HOME_DIR/$pkg-$newVer/$PKG_SCRIPT_DIR/$PKG_MD5_CONF_FILE

   local cfgBakDir=$PKG_HOME_DIR/$pkg-$newVer/${PKG_CONF_DIR}.bak

   local oldCfgDir=$PKG_HOME_DIR/$pkg-$oldVer/${PKG_CONF_DIR}

   local newCfgDir=$PKG_HOME_DIR/$pkg-$newVer/${PKG_CONF_DIR}

   local scriptFile=$PKG_HOME_DIR/$pkg-$newVer/$PKG_POST_INSTALL_SCRIPT_PATH

   local oldAppName rc reason userLoginShell autoTestLogFile autoTestResponseFile localIp cfgId

   info_log "update from [$oldVer] --> [$newVer]"
   
   # precheck

   ( update_prechk "$pkg" "$oldVer" "$newVer" ) || return $?

   ( download_pkg "$pkg" "$newVer" "$md5" "$urls" "$platform" "$PKG_DOWNLOAD_DIR" "binary" "$rate" ) || return $?

   ( install_files "$pkgFile" "$PKG_HOME_DIR" "$pkg" "$newVer" )

   rc=$?

   if [ $rc -ne 0 ] ; then

      # no need to del_cron here !

      ( move_bad_pkg_dir "$pkg" "$newVer" )

      #brief_err_log "install files failed"

      return $rc
   fi

   # download && install cfg pkg if needed

   if [ "$DO_CFG_MGR" = "yes" ] ; then

        ( cfg_mgr_common "$pkg" "$newVer" "$cfgmd5" "$cfgpkgurls" ) ; rc=$?

        if [ $rc -ne 0 ] ; then
      
             # no need to del_cron here !

             ( move_bad_pkg_dir "$pkg" "$newVer" )

             #brief_err_log "install config pkg failed"

             return $rc
        else
             info_log ""

             info_log "install config pkg succeed"

             info_log ""
        fi
   else
        info_log ""

        info_log "DO_CFG_MGR != yes , no need to do config management job"

        info_log ""
   fi

   # gen comm env variables file

   ( gen_comm_env_var "$pkg" "$newVer" "update" "$oldVer" ) 

   if [ $? -ne 0 ] ; then

      # no need to del_cron here !

      ( move_bad_pkg_dir "$pkg" "$newVer" )
 
      #brief_err_log "$(cat ${_BRIEF_LOG})"

      return $CODE_GEN_COMM_VAR_FILE_FAIL
   fi

   # prepare to run post-install scripts

   ( save_section_cfg "post_install" "$newCfgFile" "${PKG_HOME_DIR}/$pkg-$newVer/$PKG_POST_INSTALL_SCRIPT_PATH" 755 ) 

   if [ $? -ne 0 ] ; then

      # no need to del_cron here !
  
      ( move_bad_pkg_dir "$pkg" "$newVer" )

      #brief_err_log "save post-install script failed"

      return $CODE_INSERT_COMMON_CODE_FAIL
   fi 
 
   ( insert_common_code "$PKG_HOME_DIR/$pkg-$newVer/$PKG_POST_INSTALL_SCRIPT_PATH" "$pkg" "$newVer" )

   if [ $? -ne 0 ] ; then

      # no need to del_cron here !
 
      ( move_bad_pkg_dir "$pkg" "$newVer" )
 
      #brief_err_log "save post-install script failed"

      return $CODE_INSERT_COMMON_CODE_FAIL
   fi 

   ( bash_syn_chk "$scriptFile" ) 

   if [ $? -ne 0 ] ; then

      # no need to del_cron here !

      ( move_bad_pkg_dir "$pkg" "$newVer" )

      #brief_err_log "found syntax error in $scriptFile"

      return $CODE_POST_INSTALL_SCRIPT_SYNTAX_ERR 
   fi

   ( run_post-install "$pkg" "$newVer" ) 

   if [ $? -ne 0 ] ; then

      # no need to del_cron here !

      ( move_bad_pkg_dir "$pkg" "$newVer" )

      #brief_err_log "run post-install script failed"

      return $CODE_RUN_POST_INSTALL_SCRIPT_FAIL 
   fi

   info_log ""

   # replace common variables

   if [ "$ENABLE_REPLACE_PATTERN" = 'yes' ] ; then

        ( replace_pattern "$pkg" "$newVer" ) 
     
        if [ $? -ne 0 ] ; then

           # no need to del_cron here !
     
           ( move_bad_pkg_dir "$pkg" "$newVer" )
     
           #brief_err_log "replace common variables failed"
     
           return $CODE_GET_HOST_INFO_FAIL
        fi
   else
        warn_log "replace pattern disabled , skip ..."
   fi

   info_log ""

   # install start|stop|restart|monitor|resolve script

   ( install_scripts "$pkg" "$newVer" "$newCfgFile" "$md5CfgFile" "update" )

   if [ $? -ne 0 ] ; then

      # because pkg-ver/ dir not install correctly , so do NOT need to remove cron

      #( del_cron "$newCfgFile" )

      ( move_bad_pkg_dir "$pkg" "$newVer" )

      return $CODE_INSTALL_COMMON_FILES_FAIL_UNDER_UPDATE 
   fi

   # save crontab entry

   ( save_cfg_cron "$newCfgFile" ) ; rc=$?

   if [ $? -ne 0 ] ; then

      #brief_err_log "save cron config failed"

      ( move_bad_pkg_dir "$pkg" "$newVer" )

      return $CODE_SAVE_CRON_FAIL
   fi

   # copy old config

   if [ "$copyCfg" = 'yes' ] ; then

      info_log "prepart to copy old version config [$pkg-$oldVer/$PKG_CONF_DIR] to [$pkg-$newVer]"

      info_log "new version config backup to [$PKG_HOME_DIR/$pkg-$newVer/${PKG_CONF_DIR}.bak]"

      # backup the original conf/ dir of new ver

      cp -af $PKG_HOME_DIR/$pkg-${newVer}/${PKG_CONF_DIR} $PKG_HOME_DIR/$pkg-${newVer}/${PKG_CONF_DIR}.bak

      if [ $? -eq 0 ] ; then

          info_log "backup new version config succeed"
     
          # remove the old conf dir

          rm -fr $PKG_HOME_DIR/$pkg-${newVer%%\/}/${PKG_CONF_DIR%%\/}

          # then copy the entire old config dir
 
          cp -af $PKG_HOME_DIR/$pkg-$oldVer/${PKG_CONF_DIR%%\/} $PKG_HOME_DIR/$pkg-${newVer%%\/} 

          if [ $? -eq 0 ] ; then

               info_log "copy ALL file under $pkg-$oldVer/$PKG_CONF_DIR to $pkg-$newVer/$PKG_CONF_DIR succeed"
          else
               ( del_cron "$newCfgFile" )

               ( move_bad_pkg_dir "$pkg" "$newVer" )  # move new version dir to $PKG_BAD_DIR

               brief_err_log "copy ALL file under $pkg-$oldVer/$PKG_CONF_DIR to $pkg-$newVer/$PKG_CONF_DIR failed" 

               return $CODE_COPY_OLD_VER_CFG_FAIL 
          fi

      else
           ( del_cron "$newCfgFile" )

           ( move_bad_pkg_dir "$pkg" "$newVer" )  # move new version dir to $PKG_BAD_DIR

           brief_err_log "unable to backup new version conf $pkg-$newVer/$PKG_CONF_DIR"

           return $CODE_PKG_BACKUP_OLD_CFG_FAIL 
      fi
  fi         

  # print out oldver app process info

  info_log ""

  oldAppName=$(get_item_cfg "base_info" "$PKG_HOME_DIR/$pkg-$oldVer/$PKG_CFG_FILE" "app_name" | sed 's/,/ /g')

  info_log "OLD APP NAME = $oldAppName"

  while read line ; do

     info_log "$line"

  done < <(pidof $oldAppName | xargs -r ps -o pid,ppid,lstart,cmd)

  info_log ""

  info_log "update mode : startApp = $startApp"

  if [ "$startApp" = "no" ] ; then

       if [ "$copyCfg" = "yes" ] ; then

            cfgId=$(get_cfg_id_by_pkg_ver "$pkg" "$oldVer")
       else
            cfgId=$(get_current_task_cfg_id "$pkg" "$newVer") # cfgid is given
       fi

       ( save_cfg_id "$pkg" "$newVer" "$cfgId" )

       # normally , startApp should be YES always

       brief_ok_log "update pkg (install-only) done"

       return 0
  else
       stopUser=$(get_item_cfg "base_info" "$oldCfgFile" "user") ; rc=$?

       if [ $rc -ne 0 ] ; then

          # remove any new installed cron

          ( del_cron "$newCfgFile" )
  
          ( move_bad_pkg_dir "$pkg" "$newVer" )

          brief_err_log "get stop user info failed"

          return $rc
       fi

       startUser=$(get_item_cfg "base_info" "$newCfgFile" "user") ; rc=$?

       if [ $rc -ne 0 ] ; then

          # remove any new installed cron

          ( del_cron "$newCfgFile" )

          ( move_bad_pkg_dir "$pkg" "$newVer" )

          brief_err_log "get start user info failed"
 
          return $rc
       fi

       case $order in

             'kill-first') action1=stop ; action1User=$stopUser ; action1Ver=$oldVer ;

                           action2=start ; action2User=$startUser ; action2Ver=$newVer ;

                           action1CfgFile=$oldCfgFile ; action2CfgFile=$newCfgFile ;;

            'start-first') action1=start ; action1User=$startUser ; action1Ver=$newVer ;

                           action2=stop ; action2User=$stopUser ; action2Ver=$oldVer ;

                           action1CfgFile=$newCfgFile ; action2CfgFile=$oldCfgFile ;;

                        *) ( del_cron "$newCfgFile" )

                           ( move_bad_pkg_dir "$pkg" "$newVer" )

                           brief_err_log "invalid value '$order' for option 'order'"

                           return $CODE_ORDER_INVALID ;;
       esac

       info_log "execute $action1 script"

       info_log "RUN USER = $action1User"

       ( start_stop_app "$pkg"  "${action1Ver}" "${action1User}"  "$action1" ) ; rc=$? 

       # collect log no matter succeed or failed

       collect_app_log "$pkg" "$action1Ver" "$action1" "1"

       if [ $rc -ne 0 ] ; then

          # enforce to stop newVer app even newVer app is not installed

          err_log ""

          err_log "$(repeatStr '-' 19)  $action1 $pkg-$action1Ver failed , begin to rollback to $oldVer  $(repeatStr '-' 19)"

          info_log "enforce to stop any new version app if running"

          info_log "new ver = $newVer" 

          # also remove new version cron when stop new version app

          ( start_stop_app "$pkg" "$newVer" "$stopUser" "stop" )

          ( move_bad_pkg_dir "$pkg" "$newVer" )

          # if stop old version failed , we WON'T start old ver app again , just add old ver cron and wait for auto-start of old ver 

          ( add_cron "$oldCfgFile" )

          brief_err_log "$action1 [$pkg-${action1Ver}] failed"

          return $CODE_START_STOP_APP_FAIL_UNDER_UPDATE 

       fi

       # step 1 succeed , continue ....

       info_log ""

       info_log "action [$action1] [$pkg-${action1Ver}] finished"

       info_log "NEXT action is [$action2] [$pkg-${action2Ver}]"

       ( start_stop_app "$pkg"  "${action2Ver}" "${action2User}"  "$action2" ) ; rc=$?

       collect_app_log "$pkg" "$action2Ver" "$action2" "2"

       if [ $rc -ne 0 ] ; then
          
          err_log ""

          err_log "---------------- $action2 $pkg-$action2Ver failed , begin to rollback to $oldVer ----------------"

          err_log "action [$action2] failed , stop [$pkg-$newVer] process , remove [$pkg-$newVer] crontab"

          err_log "rollback to [$pkg-$oldVer] process & crontab"

          if [ "$order" = "start-first" ] ; then

               # in this situation , I do NOT re-generate the $_APP_LOG !!!!
       
               # immediately start old ver app and add old ver cron back

               ( start_stop_app "$pkg" "$oldVer" "${action1User}" "start" )

               # enforce to kill new version app even it's not running and clear new new ver cron

               ( start_stop_app "$pkg" "$newVer" "${action1User}" "stop" )

          elif [ "$order" = "kill-first" ] ; then

               # in this situation , I do NOT re-generate the $_APP_LOG !!!!
          
               # restore old version app process if new version start failed 

               # while order=kill-first , then remove new version crontab

               # enforce to stop old ver , in case of new ver process num < $count

               # also remove new version cron when stop new ver app

               ( start_stop_app "$pkg" "$newVer" "${action1User}" "stop" )

               # recover old version app and add old ver cron back

               ( start_stop_app "$pkg" "$oldVer" "${action1User}" "start" )
          fi

          # move new pkg dir to /data/bad_pkg

          ( move_bad_pkg_dir "$pkg" "$newVer" )

          brief_err_log "$action2 [$pkg-${action2Ver}] failed" 

          return $CODE_START_STOP_APP_FAIL_UNDER_UPDATE 
       fi

       # move old version directory to $PKG_BACKUP_DIR

       info_log "Everything is fine , clear old version dir"

       ( backup_pkg "$pkg" "$oldVer" ) 

       if [ $? -ne 0 ] ; then

          ( del_cron "$oldCfgFile" )

          brief_err_log "bakup old version [$oldVer] failed"

          return $CODE_MV_OLD_DIR_FAIL_UNDER_UPDATE 

       else
            # add a record to install-file to indicate that old ver has been move to /data/backup

            ( save_op_mode "$pkg" "$oldVer" "backup_on_update" "$PKG_BACKUP_DIR/$pkg-$oldVer/$PKG_INSTALL_MODE_PATH" )

            ( save_pkg_verId "$pkg" "$newVer" )

            ( save_pkg_name "$pkg" "$newVer" )

            ( save_pkg_verName "$pkg" "$newVer" )
       fi

       if [ "$copyCfg" = "yes" ] ; then

             cfgId=$(get_cfg_id_by_pkg_ver "$pkg" "$oldVer")
       else
             cfgId=$(get_current_task_cfg_id "$pkg" "$newVer")
       fi

       ( save_cfg_id "$pkg" "$newVer" "$cfgId" )


       # call auto-test script by HTTP API

       localIp=$(get_local_ip "eth0") 
 
       autoTestLogFile="$PKG_HOME_DIR/$pkg-$newVer/$PKG_SCRIPT_DIR/$AUTO_TEST_LOG_DIR/$CALL_AUTO_TEST_LOG"

       autoTestResponseFile="$PKG_HOME_DIR/$pkg-$newVer/$PKG_SCRIPT_DIR/$AUTO_TEST_LOG_DIR/$CALL_AUTO_TEST_RESPONSE_FILE"

       mkdir -p $(dirname $autoTestLogFile) &>/dev/null

       nohup wget --output-document=$autoTestResponseFile -a $autoTestLogFile -t 2 -w 5 --timeout=5 "${AUTO_TEST_URL}?taskid=${_TASK_ID}&ip=$localIp" &

       brief_ok_log "update && start pkg done"
  fi
 
  
} 



function rollback_prechk {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local srcVer=${2:?$FUNCNAME [ERR ] | srcVer is NULL}

   local dstVer=${3:?$FUNCNAME [ERR ] | dstVer is NULL}

   if [ ! -d $PKG_HOME_DIR/$pkg-$srcVer ] ; then
     
          brief_err_log "src version [$srcVer] is not installed"
     
          err_log "$(ls -l $PKG_HOME_DIR/$pkg-$srcVer 2>&1)"
     
          return $CODE_SRC_VER_NOT_EXIST_UNDER_ROLLBACK

   elif [ -d $PKG_HOME_DIR/$pkg-$dstVer ] ; then
     
          brief_err_log "dst version [$dstVer] already installed"

          err_log "install at [$(tail -1 $PKG_HOME_DIR/$pkg-$dstVer/$PKG_SCRIPT_DIR/$PKG_INSTALL_MODE_FILE)]"
     
          err_log "$(ls -l $PKG_HOME_DIR/$pkg-$dstVer 2>&1)"
     
          return $CODE_DST_VER_ALREAY_EXIST_UNDER_ROLLBACK

    elif [ -e $PKG_HOME_DIR/$pkg-$dstVer ] && [ ! -d $PKG_HOME_DIR/$pkg-$dstVer ] ; then
     
           brief_err_log "$PKG_HOME_DIR/$pkg-$dstVer is a FILE !!!"

           err_log "$(ls -l $PKG_HOME_DIR/$pkg-$dstVer 2>&1)"

           return $CODE_DST_VER_ALREAY_EXIST_UNDER_ROLLBACK

    elif [ ! -d $PKG_BACKUP_DIR/$pkg-$dstVer ] ; then

           brief_err_log "dst version [$dstVer] not installed before"

           err_log "$(ls -l $PKG_BACKUP_DIR/$pkg-$dstVer 2>&1)"

           return $CODE_DST_VER_NOT_INSTALLED_BEFORE_UNDER_ROLLBACK
    else
           info_log "rollback conditoin check passed"

           return 0
    fi
}



function rollback_pkg {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg name is NULL}

   local srcVer=${2:?$FUNCNAME [ERR ] | src version number is NULL}

   local dstVer=${3:?$FUNCNAME [ERR ] | dst version number is NULL}

   local srcCfgFile="$PKG_HOME_DIR/$pkg-$srcVer/$PKG_CFG_FILE"

   local dstCfgFile="$PKG_HOME_DIR/$pkg-$dstVer/$PKG_CFG_FILE"

   local oldDstCfgFile="$PKG_BACKUP_DIR/$pkg-$dstVer/$PKG_CFG_FILE"

   local srcUser
 
   local dstUser

   local rc

   #-------------------------------------------------------#
   # NOTE :: rollback is ALWAYS stop (uninstall) and start #
   #-------------------------------------------------------#

   info_log "rollback from [$srcVer] to [$dstVer]"

   srcUser=$(get_and_chk_user "$srcCfgFile") || return $?

   ( rollback_prechk "$pkg" "$srcVer" "$dstVer" ) || return $?

   info_log "begin to uninstall src version [$pkg-$srcVer]"

   dstUser=$(get_and_chk_user "$oldDstCfgFile") || return $?

   # call the uninstall_pkg function to uninstall the src version

   ( uninstall_pkg "$pkg" "$srcVer" "$srcCfgFile" ) || return $? 

   # restore the backup dstVer and start the app

   mv -f $PKG_BACKUP_DIR/$pkg-$dstVer $PKG_HOME_DIR

   info_log ""

   info_log "action [uninstall] [$pkg-$srcVer] is finished , NEXT action is [start] [$pkg-$dstVer]"

   ( start_stop_app "$pkg" "$dstVer" "$dstUser" "start" ) ; rc=$?

   # in function uninstall_pkg , $_APP_LOG had generate already , 

   # so here I append the $_APP_LOG :)

   collect_app_log "$pkg" "$dstVer" "start" "2"

   if [ $rc -ne 0 ] ; then

        err_log ""
  
        err_log "---------------- start $pkg-$dstVer failed , begin to rollback ----------------"

        # also remove dst ver cron when stop dst ver app
  
        ( start_stop_app "$pkg" "$dstVer" "$dstUser" "stop" )
  
        info_log "move $pkg-$dstVer back to $PKG_BACKUP_DIR"
  
        mv -f $PKG_HOME_DIR/$pkg-$dstVer $PKG_BACKUP_DIR
  
        info_log "move $pkg-$srcVer back to $PKG_HOME_DIR"
  
        mv -f $PKG_BACKUP_DIR/$pkg-$srcVer $PKG_HOME_DIR
  
        # restore source version app if start dst version app fail
  
        info_log "begin to start $pkg-$srcVer"

        # also add src ver cron back when start src ver app
   
        ( start_stop_app "$pkg" "$srcVer" "$srcUser" "start" )
  
        brief_err_log "start [$pkg-$dstVer] failed, rollback failed"
  
        return $CODE_START_DST_VER_APP_FAIL_UNDER_ROLLBAK 
   fi

   ( save_op_mode "$pkg" "$dstVer" "rollback" )

   brief_ok_log "rollback && start pkg done"

   return 0
}



function clear_garbage {

   local period=${1:?$FUNCNAME [ERR ] | period is NULL}

   # remove downloaded pkg before N days 

   find ${PKG_DOWNLOAD_DIR:=/data/pkg_download} -type f -daystart -mtime +$period -delete
  
   # remove logfile before N days

   find $PUB_SCRIPT_LOG_DIR/[0-9]* -daystart -mtime +$period -delete

   # remove bad pkg before N dasy

   find ${PKG_BAD_DIR:=/data/bad_pkg} -mindepth 1 -maxdepth 1 -type d -mtime +$period -exec rm -fr {} \;

   # remove backup pkg before 180 days

   #find ${PKG_BACKUP_DIR:=/data/backup} -mindepth 1 -mtime +180

   return 0
}



function get_and_chk_user {

   local cfgFile=${1:?$FUNCNAME [ERR ] | cfgFile is NULL}

   local user

   user=$(get_item_cfg "base_info" "$cfgFile" "user" | cut -d ':' -f 1)

   if [ -z "$user" ] ; then

        brief_err_log "get user inifo failed"

        return $CODE_USER_INVALID

   elif ! id $user &>/dev/null ; then

        brief_err_log "user [$user] not exist"

        return $CODE_USER_INVALID
   else
        echo "$user"

        debug_log "user [$user] is exist"

        debug_log "$(id $user)"

        return
   fi

}



function chk_user_shell {

   local user=${1:?$FUNCNAME [ERR ] | user is NULL}

   local userLoginShell

   debug_log "begin to check user's login shell"

   userLoginShell=$(getent passwd $user | awk -F ':' '{print $7}')

   if [ "$userLoginShell" = "/bin/bash" ] ; then

        debug_log "user [$user] login shell is /bin/bash , default to ok"

        return 0

   elif [ -z "$userLoginShell" ] ; then

        brief_err_log "user [$user] login shell is empty!"

        return $CODE_USER_LOGIN_SHELL_INVALID

   elif ! grep -q -x -F "$userLoginShell" /etc/shells ; then

        brief_err_log "user [$user] login shell is [$userLoginShell] , not in /etc/shells"

        return $CODE_USER_LOGIN_SHELL_INVALID
   else
        echo "$userLoginShell"

        info_log "user [$user] login shell ok [$userLoginShell]"

        return 0
   fi
}



function is_test_proc {

   local appNmae=${1:?$FUNCNAME appName is NULL}

   if echo "$appName" | egrep -q '[-_.](test|debug)(_d)?$' ; then
   
        isTestProc="yes"
   
   elif echo "$appName" | egrep -q '[-_.]?(test|debug)[-_.]' ; then
   
        isTestProc="yes"
   
   elif echo "$appName" | egrep -q '[-_.](test|debug)[0-9]+' ; then
   
        isTestProc="yes"
   
   elif echo "$appName" | egrep -q '^(test|debug)([0-9]+)?[-_]?' ; then
   
        isTestProc="yes"

   elif echo "$appName" | egrep -q '^(test|debug)([[:punct:]]+|$)' ; then

        isTestProc="yes"
   else
        isTestProc="no"
   fi
   
   if [ "$isTestProc" = "yes" ] ; then
   
        info_log "$appName IS a test or debug process"
   
        return 0
   else
        info_log "$appName is NOT a test or debug process"

        return 1
   fi

}



function set_cmdb_rpt_node {

   local chosenIp

   # first , see what's in the cache file

   chosenIp=$(cat $CMDB_IP_CACHE_FILE)

   # if nothing , then return the whole ip list
   
   if [ -z "$chosenIp" ] ; then

        warn_log "no suitable ip found"

        warn_log "all Ip's pkg-loss rate > 0%"

        warn_log "enforce to choose two ip randomly"
   
        chosenIp="$(echo $DEFAULT_CMDB_IP_LIST $CMDB_CNC_PROXY_IP_LIST | xargs -n1 | { sort -R 2>/dev/null || sort ; } | xargs)"
   else
        debug_log "found suitable ip = [${chosenIp// /,}]"
   fi
   
   echo "$chosenIp"

   return

}



function appFile_location {

   # for history reason , vrouter.ko regeistered as a app 
   
   # name but it's a kernel module , not a process at all !

   # so I have to confirm whether the appFile is under bin/

   # or sbin/ . if not , then no report to cmdb at all !

   local appFile=${1:?$FUNCNAME [ERR ] | appFile is NULL}

   debug_log "try to find file \"$appName\""

   debug_log "searching $PKG_BINARY_DIR/ ..."

   debug_log "searching $PKG_SBIN_BINARY_DIR/ ..."

   if [ -f "$PKG_HOME_DIR/$pkg-$ver/$PKG_BINARY_DIR/$appName" ] ; then
   
        appFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_BINARY_DIR/$appName"
   
   elif [ -f "$PKG_HOME_DIR/$pkg-$ver/$PKG_SBIN_BINARY_DIR/$appName" ] ; then
   
        appFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SBIN_BINARY_DIR/$appName"
   else
        appFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_BINARY_DIR/$appName"

        warn_log "no \"$appName\" under ${PKG_BINARY_DIR}/"

        warn_log "no \"$appName\" under ${PKG_SBIN_BINARY_DIR}/"

        warn_log "set \$appFile set to $appFile"
   fi

   echo "$appFile"

   return

}



function get_server_id {

   local serverId

   serverId=$(grep '^server_id=' $HOST_INFO_FILE |awk -F '=' '{print $2}')
   
   if [ -z "$serverId" ] ; then  # server_id is empty ! can't report , abort it !
   
        err_log "can NOT get server id in $HOST_INFO_FILE , skip CMDB report !!!"
   
        err_log "won't touch $FIRST_START_FLAG_FILE"
   
        python $ALARM_PYTHON_SCRIPT 10390 72604 0 "server id is empty , call wu-erping !!!" &>/dev/null
   
        # just send alarm , but do NOT return on error

        return 1
   
   elif [ -n "${serverId//[0-9]}" ] ; then  # server_id is invalid , can't report , abort it !
   
        err_log "server_id contain non-digital characters : $serverId"
   
        python $ALARM_PYTHON_SCRIPT 10390 72604 0 "server id is invalid : $serverId , call wu-erping !!!" &>/dev/null
   
        # just send alarm , but do NOT return on error

        return 1
   
   elif [ $serverId = 0 ] ; then  # server_id is zero , can't report , abort it !
   
        err_log "server id is 0 , skip CMDB report !!!"
   
        err_log "won't touch $FIRST_START_FLAG_FILE"
   
        python $ALARM_PYTHON_SCRIPT 10390 72604 0 "server id is 0 , call wu-erping !!!" &>/dev/null
   
        # just send alarm , but do NOT return on error

        return 1
   
   else # server_id is fine , prepare to report

        debug_log "server id is [$serverId]"
   
        echo "$serverId" 

        return
   fi
}



function get_server_id_simple {

   local serverId

   serverId=$(grep '^server_id=' $HOST_INFO_FILE |awk -F '=' '{print $2}')

   echo "$serverId"

   return
} 



function get_match_pattern {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local patternList

   local i

   source "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_COMMON_ENV_VAR_FILE"

   patternList=$(egrep -v '^#|^[[:space:]]*$' $SPECIAL_PROC_LIST_FILE | awk '{print $1}')
   
   if [ -n "$patternList" ] ; then
   
        while read i ; do
   
              debug_log "check for pattern [$i]"
   
              if egrep -q "$i" <<< "$APP_NAME" ; then

                 # found matched pattern !!!
        
                 echo $i
              fi
   
        done <<< "$patternList"

        debug_log "check finished"

        return 0
   else 
        return 1
   fi
   
}



function match_special_count {

   local matchPattern=${1:?$FUNCNAME [ERR ] | matchPattern is NULL}

   local appName=${2:?$FUNCNAME [ERR ] | appName is NULL}
   
   local count

   local line

   debug_log  "match pattern is $matchPattern"

   #count=$(awk -v _p="$matchPattern" '$1 == _p {print $2}' $SPECIAL_PROC_LIST_FILE | head -n1)

   # only the first-match line will be used

   line=$(awk -v _p="$matchPattern" '$1 == _p' $SPECIAL_PROC_LIST_FILE | head -n1)

   count=$(echo "$line" | awk '{print $2}')

   if [ -z "$count" ] ; then
   
        count=1
   
        warn_log "$appName is special , NO default value"

        warn_log "set [count=$count]"
   
        warn_log "line=[$line]"

        echo $count
   else
        warn_log "$appName is special , found default value"

        warn_log "set [count=$count]"
   
        warn_log "line=[$line]"

        echo $count
   fi
}


function find_additional_prog {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local installPath="$PKG_HOME_DIR/$pkg-$ver/"

   local appName

   local additionalProgList

   local cfgFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE"

   appName=$(get_item_cfg "base_info" "$cfgFile" "app_name") || return $?

   additionalProgList=$(ls -1 /proc/[0-9]*/exe -d 2>/dev/null | xargs -i readlink -fe {} | sort -u | grep "^${installPath}" | xargs -r -n1 basename | grep -v "^${APP_NAME}$" | xargs)

   info_log "additional prog list = $additionalProgList"
}



function set_rpt_count {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local appName=${3:?$FUNCNAME [ERR ] | appName is NULL}

   local matchPattern

   local matchFlag

   local count defunctCount

   local psString

   local pids

   local installPath="$PKG_HOME_DIR/$pkg-$ver"

   local pidInfo

   local otherPids

   matchPattern=$(get_match_pattern "$pkg" "$ver")
   
   if [ -n "$matchPattern" ] ; then
   
        matchFlag="true"
   
        # is a open source program
   
        count=$(match_special_count "$matchPattern" "$appName")

        echo "$count"
   else 
        matchFlag="false"
   
        # not speical proc

        pidInfo=$(pidof $appName)

        pids=$(echo $pidInfo | xargs -r -n1 | xargs -r -i ls -l /proc/{}/cwd | awk -v _x="$installPath"  '$NF ~ "^"_x {print $9}' | cut -d '/' -f 3)

        otherPids=$(join -v 1 <(echo "$pidInfo" | xargs -n1 | sort) <(echo "$pids" | xargs -n1 | sort) | xargs)

        [ -z "$pids" ] && pids=$(ps -o pid,cmd -e | awk -v _name="/${appName}$|^${appName}$" '$2 ~ _name {print $1}' | xargs)
        
        if [ -n "$pids" ] ; then

              count=$(ps --no-headers -opid,cmd $pids | grep -v '<defunct>$' | awk '{print $1}' | xargs | wc -w)

              defunctCount=$(ps --no-headers -opid,cmd $pids | grep '<defunct>$' | awk '{print $1}' | xargs | wc -w)
        
              debug_log "$appName is not special" 
   
              debug_log "find count=[$count] , defunctCount=[$defunctCount] , otherPids=[$otherPids]"

              psString=$(ps -o pid,ppid,lstart,cmd $pids)
        
              while read line ; do
        
                    debug_log "$line"
              
              done <<< "$psString"
 
              otherPidsString=$(echo $otherPids | xargs -n1 | xargs -i -r ls -l /proc/{}/exe)

              debug_log "other pid is :"
 
              while read line ; do

                    debug_log "$line"

              done <<< "$otherPidsString"

              echo "$count"
       else
              debug_log "no process name $appName found , maybe it's a script ?"

              echo "0"
       fi
   fi   
   
}


function gen_local_pinfo {

   # check to see if app file is special (enforce count to given number)

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local procInfoStr=$(get_txt_pinfo "$pkg" "$ver") 

   mkdir -p $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CACHE_DATA_DIR &>/dev/null

   echo "$procInfoStr" > $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CACHE_DATA_DIR/$LOCAL_PROC_INFO_FILE

   return
}



function cmdb_add_proc {

   # check to see if app file is special (enforce count to given number)

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local count

   local cwd

   local cmdbIpList

   local output

   local logFile

   local jsonStr

   local opt

   local serverId

   local i

   local outputLink

   local logFileLink

   local urlParaStr

   local verId

   source "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_COMMON_ENV_VAR_FILE"

   count=$(set_rpt_count "$pkg" "$ver" "$APP_NAME")

   serverId=$(get_server_id)

   cmdbIpList=$(set_cmdb_rpt_node)

   cwd="$PKG_HOME_DIR/$pkg-$ver/bin"
   
   output="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CMDB_LOG_DIR/$CMDB_ADD_RESPONSE.${_ACTION}"
   
   logFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CMDB_LOG_DIR/$CMDB_ADD_DEBUG_LOG.${_ACTION}"

   mkdir -p $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CMDB_LOG_DIR 2>/dev/null

   verId=$(cat $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_VERSION_ID_FILE 2>/dev/null)
   
   jsonStr="{\"ip\":\"${ETH0_IP}\",\"exe\":\"${APP_NAME}\",\"count\":$count,\"cwd\":\"$cwd\",\"version_id\":$verId}"

   opt="--output-document=$output --debug -a $logFile -t 2 -w 5 --timeout=5"
 
   urlParaStr="ip=${ETH0_IP}&json=${jsonStr}&server_id=$serverId"

   for i in $cmdbIpList ; do

       wget $opt "http://${i}:8088/webservice/pkgRep/add.do?${urlParaStr}" 2>&1

       rc=$?
       
       if [ $rc -eq 0 ] ; then

            outputLink="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$CMDB_WGET_RESPONSE"

            logFileLink="$PUB_SCRIPT_LOG_DIR/${_GLOBAL_ID}/$CMDB_WGET_LOG"

            info_log "report to CMDB($i) succeed"

            info_log "rc=[$rc]"

            info_log "create symlinks to cmdb log & response"

            info_log "$(ln -vs $output $outputLink)"

            info_log "$(ln -vs $logFile $logFileLink)"

            break
       else
            err_log "report to [$i] failed , try another ip"
       fi

   done

   if [ $rc -ne 0 ] ; then

        err_log "add [$APP_NAME] to all CMDB failed !!!"

        err_log "rc=[$rc]"

        err_log "$(python $ALARM_PYTHON_SCRIPT 10400 72642 0 "add $APP_NAME to CMDB failed !!!")"
   fi

   info_log "$(ls -l $output)"

   info_log "$(ls -l $logFile)"

   # print the last succeed or failed URL

   info_log "http://${i}:${CMDB_PORT}/${CMDB_ADD_INTERFACE}?${urlParaStr}"

   return $rc
}



function touch_flag_file {

   local file=${1:?$FUNCNAME [ERR ] | file is NULL}

   # report to CMDB succed , now I can generate .first-start file

   info_log "create $FIRST_START_FLAG_FILE"

   # touch the first-start flag file

   echo "$(date +'%F %T')" > $file

   info_log "$(ls -l $file)"

   return
}

function get_basic_pinfo {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   ( source "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_COMMON_ENV_VAR_FILE"

     count=$(set_rpt_count "$pkg" "$ver" "$APP_NAME")

     verId=$(cat $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_VERSION_ID_FILE 2>/dev/null) 

     echo "$pkg,$ver,${INSTALL_PATH}/bin,$APP_NAME,$verId,$count"

   )

   return
}


function get_txt_pinfo {

  local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

  local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

  local str=$(get_basic_pinfo "$pkg" "$ver")

  local binPath verId count pids appName

  binPath=$(echo "$str" | awk -F ',' '{print $3}')

  count=$(echo "$str" | awk -F ',' '{print $6}')

  verId=$(echo "$str" | awk -F ',' '{print $5}')

  appName=$(echo "$str" | awk -F ',' '{print $4}')

  pids=$(pidof $appName | xargs -n 1 | sort -n | head -n $count | xargs | sed 's/ /_/g')  # not all pids , but only $count pids (nginx, .eg.)

  echo "datetime:$(date +'%Y-%m-%d_%H:%M:%S'),ts:$(date +'%s'),amount:$count,package_name:$pkg,path:$binPath,version_id:$verId,proc:$appName,version:$ver,pids:$pids"

  return
}



function add_proc_to_cmdb {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local flagFile=${3:?$FUNCNAME [ERR ] | flagFile is NULL}

   local appFile=${4:?$FUNCNAME [ERR ] | appFile is NULL}

   local rc 

   if [ ! -f "$flagFile" ] && ( is_executable_file "$appFile" ) ; then
   
        info_log "$appFile is a excutable , need report"
   
        info_log "this is the first time to ${_ACTION}"
   
        info_log "add record to CMDB now"
   
        # try to add a new record to CMDB
   
        ( cmdb_add_proc "$pkg" "$ver" ) ; rc=$?
   
        if [ $rc -eq 0 ] ; then
   
             # report to CMDB succed , create .first-start file
   
             ( touch_flag_file $flagFile )
        else
             # report to cmdb failed , send alarm 
   
             # in next start|restart {_ACTION} , it will try again
   
             err_log "no $FIRST_START_FLAG_FILE touched"
         fi
   
    elif [ -f "$flagFile" ] ; then
   
         # alreay add to CMDB before
   
         info_log "found $FIRST_START_FLAG_FILE , already report to CMDB before"
   
         info_log "no no need to add to CMDB"
   
         info_log "$(ls -l $flagFile)"
   
   elif ! is_executable_file $appFile ; then
   
         # $APP_NAME not a executable file , do NOT add to CMDB 
   
         info_log "$appFile not exist or not executable"
   
         info_log "no need to add to CMDB"
   
         info_log "$(ls -l $appFile 2>&1)"
   fi
   
}



function remove_flag_file {

   local file=${1:?$FUNCNAME [ERR ] | file is NULL}

   info_log "remove flag file"

   info_log "$(rm -fv $file)"

   return
}



function cmdb_del_proc {

   # check to see if app file is special (enforce count to given number)
   
   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}
   
   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}
   
   local cmdbIpList
   
   local output
   
   local logFile
   
   local jsonStr
   
   local opt

   local serverId
   
   local i

   local urlParaStr

   local verId

   local cwd
   
   source "$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_COMMON_ENV_VAR_FILE"  # we get $APP_NAME here

   cwd="$PKG_HOME_DIR/$pkg-$ver/bin"

   serverId=$(get_server_id)
 
   cmdbIpList=$(set_cmdb_rpt_node)

   verId=$(cat $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_VERSION_ID_FILE 2>/dev/null)

   jsonStr="{\"ip\":\"${ETH0_IP}\",\"exe\":\"${APP_NAME}\",\"cwd\":\"$cwd\",\"version_id\":$verId}"

   output="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CMDB_LOG_DIR/$CMDB_DEL_RESPONSE.${_ACTION}"

   logFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CMDB_LOG_DIR/$CMDB_DEL_DEBUG_LOG.${_ACTION}"

   mkdir -p $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CMDB_LOG_DIR 2>/dev/null

   opt="--output-document=$output --debug -a $logFile -t 2 -w 5 --timeout=5"

   urlParaStr="ip=${ETH0_IP}&json=${jsonStr}&server_id=$serverId"

   for i in $cmdbIpList ; do

   # test purpose

       wget $opt "http://${i}:8088/webservice/pkgRep/del.do?${urlParaStr}" 2>&1

       rc=$?
    
       if [ $rc -eq 0 ] ; then

            info_log "remove [$APP_NAME] from CMDB ($i) succeed"

            info_log "rc=[$rc]"

            break 
       else
            err_log "report to [$i] failed , try another ip"
       fi

   done

   if [ $rc -ne 0 ] ; then

        err_log "remove [$APP_NAME] from all CMDB failed !!!"

        err_log "rc=[$rc]"
     
        err_log "$(python $ALARM_PYTHON_SCRIPT 10400 72642 0 "remove $APP_NAME from CMDB failed !!!")"
   fi

   info_log "$(ls -l $output)"

   info_log "$(ls -l $logFile)"

   info_log "http://${i}:${CMDB_PORT}/${CMDB_DEL_INTERFACE}?${urlParaStr}"

   return $rc

}



function del_proc_from_cmdb {

  local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

  local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

  local flagFile=${3:?$FUNCNAME [ERR ] | flagFile is NULL}

  ( cmdb_del_proc "$pkg" "$ver" ) 

  ( remove_flag_file "$flagFile" ) 

  return

}



function cmdb_proc_mgr {

   local pkg=${1:?$FUNCNAME [ERR ] | pkg is NULL}

   local ver=${2:?$FUNCNAME [ERR ] | ver is NULL}

   local mode=${3:?$FUNCNAME [ERR ] | mode is NULL}

   local rc

   local status

   local appName

   local cfgFile

   local firstStartFlagFile

   local localDns hasLocalDns=1

   local isp

   localDns=$(get_local_dns)

   isp=$(get_host_isp $HOST_INFO_FILE)

   if [ -z "$localDns" ] ; then

        err_log "Mo local dns found , can NOT connect outside domain : cmdb.sysop.duowan.com"

        err_log "can NOT perform CMDB job , abort !"

        hasLocalDns=0

   elif [ "$isp" = "INTRANET" ] ; then

        note_log "this is a INTRANET host , but local dns found , CMDB job continue"
   fi

   # below is the CMDB job
   
   cfgFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_CFG_FILE"

   appName=$(get_item_cfg "base_info" "$cfgFile" "app_name") || return $?

   # find app file location
   
   appFile=$(appFile_location $appName)
   
   # add new proc record to CMDB if this is the first time to start
   
   firstStartFlagFile="$PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$FIRST_START_FLAG_FILE"
   
   if [ "$mode" = "start" ] || [ "$mode" = "restart" ] ; then

        ( gen_local_pinfo "$pkg" "$ver" )
   
        [ $hasLocalDns -eq 1 ] && ( add_proc_to_cmdb "$pkg" "$ver" "$firstStartFlagFile" "$appFile" )
   
   elif [ "$mode" = "stop" ] ; then 

        # remove local proc.info cache file

        rm -f $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CACHE_DATA_DIR/$LOCAL_PROC_INFO_FILE &>/dev/null

        debug_log "remove local proc info cache file : $PKG_HOME_DIR/$pkg-$ver/$PKG_SCRIPT_DIR/$PKG_CACHE_DATA_DIR/$LOCAL_PROC_INFO_FILE"
 
        [ $hasLocalDns -eq 1 ] && ( del_proc_from_cmdb "$pkg" "$ver" "$firstStartFlagFile" )
   fi
   
   # always return succeed

   return 0
   
}


function set_instance_status {

   local statusFile=${1:?$FUNCNAME [ERR ] | status file is NULL}

   local status=${2:?$FUNCNAME [ERR ] | status is NULL}

   echo "$(date +%s) $(date +'%F %T') $status" > $statusFile

   return

}
