#!/bin/bash
# NAME            : $0 
# Ver             : 0.1 
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2012-07-26 14:56
# Summary         : find out what pkgs/ver are installed on this host 

export LANG=C   

export LC_ALL=C

export PATH="/usr/bin:/bin:/sbin:/usr/sbin"

mkdir -p /data/pkg/public-scripts/data

(
    flock --nonblock 200 || exit 1

    rptScript='/data/pkg/public-scripts/bin/instance_report.py'

    toolsPlatformCfg='/data/tools-platform/src/tools_client/data/config.init'

    # log file control area

    maxLogSize=52428800
    
    logFile='/data/pkg/public-scripts/data/installed-pkg.json'
    
    logSize=$(stat --format='%s' $logFile 2>/dev/null)
    
    : ${logSize:=0}
    
    if [ $logSize -gt $maxLogSize ] ; then
    
         mv $logFile $logFile.bak
    
         touch $logFile
    fi

    #list=$(find /data/services -type f -name 'init.conf')

    list=$(find /data/services/*/admin -type f -name 'install-mode' | sed 's/\/admin\/install-mode$/\/init.conf/')
    
    privateIp='inet +addr: *(127\.|192\.168\.|10\.|172\.(1[6-9]|2[0-9]|3[0-1])\.)'
    
    ip=$(/sbin/ifconfig |egrep 'inet +addr:' | egrep -v "$privateIp" | awk -F '[ :]+' '{print $4}' |head -1)

    if [ -z "$ip" ] ; then

         echo "$(date +'%F %T')| no public ip found, exit" >> $logFile
    
         exit 1
    fi

    # if public ip found , continue
    
    result=$(for i in $list; do
    
        pkg=$(grep '^name=' $i | awk -F '=' '{print $2}')
    
        ver=$(grep '^version' $i | awk -F '=' '{print $2}')

        dir="/data/services/${pkg//\"}-${ver//\"}"

        statusFile="$dir/admin/pkg-dir.status"

        if [ -d "$dir" ] ; then

            if [ "$(cat $statusFile 2>/dev/null)" != "bad" ] ; then
    
                 echo "ip=$ip pkg=$pkg ver=$ver"
            else
                 echo "found invalid bad pkg dir : $dir" >&2 
            fi
        fi

    done)

    # get server_id first
    
    hostInfoFile='/home/dspeak/yyms/hostinfo.ini'
    
    serverId=$(grep '^server_id=' $hostInfoFile |awk -F '=' '{print $2}')
    
    #  temporary job for installed pkg report
    
    # check server id from tool-platform
    
    toolPlatformServerId=$(cat $toolsPlatformCfg | sed 's/,/\n/g' | egrep -o "'server_id':[[:space:]]+[0-9]+" | awk -F '[ :]+' '{print $2}')
    
    if [ "$toolPlatformServerId" != "$serverId" ] ; then
    
         do_alarm 11047 75365 0 "server id conflict ( tools : "$toolPlatformServerId" / hostinfo : "$serverId" )" &>/dev/null
    fi
    
    # even the server-id's is conflict , still report
    
    jsonStr=$(echo "$result" | awk -F '[ ="]+' 'NF == 7 {print $4,$6}' | sort | \
    
               sed -r 's/^/{"package":"/;s/ /","version":"/;s/$/"}/' | xargs -d '\n' | \
    
               sed "s/ /,/g;s/^/{\"ip\":\"$ip\",\"info\":[/;s/$/]}/")
    
    sleep $((10#$RANDOM % 5 ))
    
    python $rptScript "$jsonStr" &>/dev/null
    
    echo "$(date +'%F %T')| pkged json=$jsonStr" >> $logFile

) 200>/data/pkg/public-scripts/data/.installed-pkg.lock && \

rm /data/pkg/public-scripts/data/.installed-pkg.lock &>/dev/null
