#!/bin/bash
# NAME            : $0 
# Ver             : 3.0 
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2015-02-15 15:02
# Summary         : common limit running script using flock and ps 

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

myPid=$$

ppid=$PPID

script=$(basename $0)

pidFile="$(dirname $0)/.$script.pid"

lockFile="$(dirname $0)/.$script.lock"

log=$(dirname $0)/$script-err.log

# step1 - check if the previous instance is still running , if so , just quit

prePid=$(cat $pidFile 2>/dev/null)

preProc=$(basename $(ps --no-headers -o cmd $prePid | awk '{print $NF}'))

if [ -n "$prePid" ] ; then

   if  ps $prePid &>/dev/null && [ "$preProc" = "$script" ] ; then

        exit 222  # last xxx.sh still running , quit now
   fi
fi

# step2 - use 'flock' to avoid conflict

# touch pid file with flock to avoid two $0 running at the same time !

# Here I use '{}' not '()' ! because (start|stop|restart|monitor|resolve).sh 

# use this script with 'source' command .

# If I use '()' , it just quit the sub-shell , not the entire script !!!!

{
  flock -n 200 || { echo "$(date +'%F %T')| $myPid | Can't get lock on $lockFile , I quit !!!" | tee -a $log ; exit 1; }

  echo "$(date +'%F %T')| $myPid | Get lock succeed , called by [$(ps --no-header -opid,cmd $ppid)]" | tee -a $log

  echo "$(date +'%F %T')| $myPid | Insert my pid ($myPid) into $pidFile" | tee -a $log

  echo "$myPid" > $pidFile

} 200>$lockFile
