#!/bin/bash
# NAME            : $0 
# Ver             : 0.2
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2012-11-02 10:58
# Summary         : md5 check 

set +H

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

# source config file and common functions

binDir=$(cd $(dirname $0) && pwd -P)

md5ResultFile=$binDir/md5.result

prog=$(basename $0)

errList=$(md5sum -c $md5ResultFile 2>&1 | grep 'FAILED' | awk -F ':' '{print $1}')

alarmScript='/home/dspeak/yyms/yymp/yymp_report_script/yymp_report_alarm.py'

if [ -z "$errList" ] ; then

     echo "$(date +'%F %T')| $prog Ok! all file's md5 is correct"

     exit 0
else
     echo "$(date +'%F %T')| $prog FATAL! Found $(echo $errList | wc -w) file's md5 is incorrect"

     for file in $errList ; do

         # only check md5 for binary file or so/ko file

         if ! file $file | egrep -q -i 'ar archive|shared object|library file|executable|relocatable' ; then 

            # not binary or so / ko file

            echo "file [$file] is neither binary nor .a / .so / .la / .ko file , skipped"

            continue
         fi

         correctMd5=$(awk -v x=$file '$2 == x {print $1}' $md5ResultFile)

         currentMd5=$(md5sum $file 2>/dev/null | awk '{print $1}')

         : ${currentMd5:="<file not exist>"}

         echo "$(date +'%F %T')| $prog file=$file"

         echo "$(date +'%F %T')| $prog current md5=[$currentMd5] , it should be [$correctMd5]" 

         echo "$(date +'%F %T')| $prog --"

         msg="$file md5 changed from [$correctMd5] => [$currentMd5]"

         #python $alarmScript  10020 72228 0 "$msg"

     done

     exit 1
fi
