#!/bin/bash
# NAME            : $0 
# Ver             : 1.2
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2012-12-06 14:41
# Summary         : output & report local processes packaged status 

set +H

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

ppid=$PPID

# crontab control area , quit when finished

#if [ "$1" = "del-cron" ] ; then
#
#     sudo sed -r -i '/\/data\/pkg\/public-scripts\/bin\/pkg-stat\.sh/d' /etc/crontab
#
#     exit 0
#
#elif [ "$1" = "add-cron" ] ; then
#
#     if ! egrep -q '^ *0 +0 +\* +\* +\* +root +\/bin\/bash +\/data\/pkg\/public-scripts\/bin\/pkg-stat.sh +&>\/dev\/null' /etc/crontab ; then 
#        
#         sudo sed -r -i '/\/data\/pkg\/public-scripts\/bin\/pkg-stat\.sh/d' /etc/crontab && \
#    
#         sudo bash -c "echo '# /data/pkg/public-scripts/bin/pkg-stat.sh' >> /etc/crontab" && \
#    
#         sudo bash -c "echo '0 0 * * * root /bin/bash /data/pkg/public-scripts/bin/pkg-stat.sh &>/dev/null' >> /etc/crontab"
#     fi
#
#     exit
#
#elif [ "$1" = "show-cron" ] ; then
#
#     egrep '\/data\/pkg\/public-scripts\/bin\/pkg-stat.sh' /etc/crontab
#
#     exit
#fi

# install lsof if needed

if ! sudo which lsof &>/dev/null ; then

     sudo aptitude -y install lsof || exit 1
fi

# set whitelist file

ipWhitelistFile="$(dirname $0)/../conf/ip.whitelist" 

procWhitelistFile="$(dirname $0)/../conf/proc.whitelist"

# set & create output dir

outputDir="$(dirname $0)/../data"

outputFile="$outputDir/pkg-stat.log"

outputTmpFile="${outputFile}.tmp"

fakePkgDirLog="$outputDir/pkg-stat-fake-pkgdir.log"

nonPkgProcLog="$outputDir/pkg-stat-non-pkg-proc.log"

mkdir -p $outputDir

(
    # find proc which their executable file under /home/dspeak/release/bin or /usr/local or /data*
    
    # at first I assume that each one is a valid packaged dir
    
    procList=$(sudo lsof -n -P -a -i | egrep 'LISTEN|UDP \*:[0-9]+' | awk '{print $2}' |sort -u | \
    
               xargs -i readlink  /proc/{}/exe 2>/dev/null | egrep '^(\/home\/dspeak|\/data|\/usr\/local|\/usr\/bin\/memcached)' | \
    
               sort -u | sed "s/^/$(date +'%F %T') /" | awk '{print $1,$2,$3,"Y"}')
    
    # confirm init.conf file , if not exist , then it's a fake package dir
    
    exeList=$(echo "$procList" | awk '{print $3}')
    
    true > $fakePkgDirLog
    
    true > $nonPkgProcLog
    
    for i in $exeList ; do
     
          initFile=$(echo $i | awk -F '/' '{print "/"$2"/"$3"/"$4"/init.conf"}')
    
          if [ ! -f "$initFile" ] ; then
    
             if echo "$i" | grep -q "^\/data\/services\/" ; then
    
                  # fake package dir
    
                  procList=$(awk -v x=$i '{ if ($3 == x) $4 = "N" ; print }' <<< "$procList")
    
                  fakeRootDir=$(egrep -o '/data/services/[^/]+/' <<< $i)
             
                  # print the fake pkg-dir record
    
                  echo "$i rootDir=[$fakeRootDir]" >> $fakePkgDirLog
    
                  echo "$(ls -l $fakeRootDir)" >> $fakePkgDirLog
    
                  echo "---" >> $fakePkgDirLog
             else
                  # non-pkg proc , so it won't have any init.conf file
    
                  procList=$(awk -v x=$i '{ if ($3 == x) $4 = "N/A" ; print }' <<< "$procList")
    
                  # print the non-pkged record
    
                  echo "$i binDir=[$(dirname $i)]" >> $nonPkgProcLog
             fi
    
          fi
    done
    
    # generate tmp file , prepare for reporting
    
    if [ -n "$procList" ] ; then
    
        awk '{
       
             if ( $3 ~ /data\/services\// &&  $4 == "Y" )
    
                  $5="Y"
             else
                  $5="N"
               
             $0=$0
    
             print $0
    
           }' <<< "$procList" | awk '{gsub("/.+/","",$3); printf "%-15s %-15s %-40s %-20s %-20s\n",$1,$2,$3,$4,$5}' > $outputTmpFile
    
    else   # create a fake record
    
           echo "$(date +'%F %T') idle Y Y" > $outputTmpFile
    fi
    
    # create new output file && report
    
    cp /dev/null $outputFile
    
    procMapFile=$(cd $(dirname $0) && pwd -P)/../conf/procinfo.map
    
    # see if it's in ip whitelist
    
    isIpWhitelist='N'
    
    privateIp='inet +addr: *(127\.|192\.168\.|10\.|172\.(1[6-9]|2[0-9]|3[0-1])\.)'
    
    myIps=$(ifconfig |grep 'inet addr' | awk -F '[ :]+' '{print $4}' | egrep -v "^$privateIp")
    
    firstIp=$(echo "$myIps" | head -n1)
    
    for i in $myIps ; do
    
         if egrep -q "^ *${i} *$" $ipWhitelistFile ; then
    
            isIpWhitelist='Y'
    
            break
         fi
    done
    
    while read date time proc initFile status ; do
    
         # 
    
         # get bussiness for each proc
    
         business=$(grep -i -w "$proc" $procMapFile | awk '{print $2}'| head -n 1 | tr '[[:lower:]]' '[[:upper:]')
    
         # if not found in procinfo.map , auto set business according some rules
    
         if [ -z "$business" ] ;then
    
             if [ "${proc##app_}" != "$proc" ] ; then
        
                  business=A-APP_SERVER
    
             elif [ "${proc##app}" != "$proc" ] ; then
    
                  business=OPEN_PLATFORM
    
             elif [ "${proc##userapp_}" != "$proc" ] ; then
    
                  business=OPEN_PLATFORM
        
             elif [ "${proc##vip_}" != "$proc" ] ; then
        
                  business=VIP
        
             elif [ "${proc##bt_}" != "$proc" ] ; then
        
                  business=BASE_PLATFORM
         
             elif [ "${proc##db_}" != "$proc" ] ; then
        
                  business=DB_PLATFORM
    
             elif [ "${proc##shard}" != "$proc" ] ; then
    
                  business=DB_PLATFORM
    
             elif [ "${proc##img}" != "$proc" ] ; then
    
                  business=DB_PLATFORM
        
             elif [ "${proc##im}" != "$proc" ] ; then
        
                  business=IM
    
             elif [ "${proc##sconfigsvr_}" != "$proc" ] ; then
    
                  business=IM
    
             elif [ "${proc##Slot}" != "$proc" ] ; then
    
                  business=IM
    
             elif [ "${proc##sccln_}" != "$proc" ] ; then
    
                  business=IM
     
             elif [ "${proc##iocheck_}" != "$proc" ] ; then
    
                  business=IM
        
             elif [ "${proc##lg_}" != "$proc" ] ; then
        
                  business=UDB
        
             elif [ "${proc##uhs_}" != "$proc" ] ; then
         
                  business=UDB
        
             elif [ "${proc##music}" != "$proc" ] ; then
        
                  business=MUSIC
        
             elif [ "${proc##lichi_}" != "$proc" ] ; then
        
                  business=LICHI
        
             elif [ "${proc##slave_session_}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##smdisp}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##smnode}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##voice_}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##sessiond_}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##relay_media}" != "$proc" ] ; then
        
                  business=VOICE
         
             elif [ "${proc##relay_signal}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##linkd_}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##guangbo_}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##lbsd_}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##signal_}" != "$proc" ] ; then
        
                  business=VOICE
        
             elif [ "${proc##pkgloss_}" != "$proc" ] ; then
        
                  business=VOICE
    
             elif [ "${proc##corps_}" != "$proc" ] ; then
    
                  business=OPEN_PLATFORM
    			  
             elif [ "${proc##mobileapp_}" != "$proc" ] ; then
    
                  business=IM
    			  
             elif [ "${proc##mobile}" != "$proc" ] ; then
    
                  business=MOBILE_YY
    
             elif [ "${proc##mob_}" != "$proc" ] ; then
    
                  business=MOBILE_YY
      
             elif [ "${proc##wap_}" != "$proc" ] ; then
    
                  business=MOBILE_YY
    
             elif [ "${proc##card_}" != "$proc" ] ; then
    
                  business=BASE_OPERATION
    
             elif [ "${proc##xman_}" != "$proc" ] ; then
    
                  business=XMAN
    
             elif [ "$proc" = "idle" ] ; then
    
                  business=IDLE
             else
                  business=OTHER
             fi 
         fi
    
         # see if it's in proc whitelist
    
         if egrep -q "^ *${proc} *$" <( egrep -v '^#|^[[:space:]]*$|^\[RE\]' $procWhitelistFile) ; then
    
              isProcWhitelist='Y'
         else
              # search pattern
    
              #echo "[$proc] no found in plain pattern , try regex"
    
              patternList=$(grep '^\[[rR][Ee]\]' $procWhitelistFile | cut -c 5-)
    
              for i in $patternList ; do
    
                  if echo "$proc" | egrep -q "$i" ; then
    
                       #echo "[$proc] match with pattern [$i] , set to Y"
    
                       isProcWhitelist='Y'
    
                       break
                  else
                       #echo "[$proc] not match with pattern [$i] , set to N"
    
                       isProcWhitelist='N'
                  fi
    
              done
         fi
    
         fmt='%-15s %-15s %-15s %-30s %-20s %-20s %-20s %s\n'
    
         printf "$fmt" "$date" "$time" "$business" "$proc" "initFile=$initFile" "status=$status" "procW=$isProcWhitelist" "ipW=$isIpWhitelist" >> $outputFile
    
    done < <(egrep -v '^#|^[[:space:]]*$' $outputTmpFile | sort -k3,3)
    
    rm -f $outputTmpFile

	rm "$outputDir/stat-report.log"

	rm "$(dirname $0)/../data/pkg-stat-called.log"

	rm "$(dirname $0)/../data/fake-pkgdir.log"

	rm "$(dirname $0)/../data/non-pkg-proc.log"

	rm "$(dirname $0)/../data/stat.log"

	rm "$(dirname $0)/../data/stat.log.tmp"
    
) &>/dev/null
