#!/bin/bash
# NAME            : $0 
# Ver             : 2.1
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2012-09-19 16:19
# Summary         : report local running proc info 
# Note            : enforce at (*/3):30 , one time at least per-day , keep history for 5 days , exit on every cycle (1 minutes) to release lock

# crontab control area , quit when finished

set +H

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T')| $line"

   done < <(echo "$*")

   return
}


# global paramater

fid='10330'

sid='72478'

reloadInterval=1

includeDir='^(/home|^/data|^/usr\/local)'

includeDir='^(/home|^/data|^/usr/local)'

binDir=$(cd $(dirname $0) && pwd)

funcDir="$binDir/../func"

baseDir="/data/pkg/public-scripts/data"

logFile=$baseDir/detail-global.log

hostInfoFile='/home/dspeak/yyms/hostinfo.ini'

lastReportFile="$baseDir/detail-last"  # is a symbol link

forceReportAtMinute="30"

source $funcDir/get_available_fd.sh

(
    flock --nonblock 200 || exit 1

    print_log "Ok ! >>>> Get Lock succeed on every-$reloadInterval minutes <<<<" >> $logFile

    print_log "" >> $logFile

    while true ; do

        print_log "---- start ----" >> $logFile

        print_log "reload interval = $reloadInterval" >> $logFile

        #------------------------------#
        # create logdir for every task #
        #------------------------------#
 
        dayTime=$(date +'%Y%m%d-%H%M')

        historyDir="$baseDir/detail-history/$(date +%F)/$dayTime"

        txtFile="$historyDir/detail-data.txt"

        jsonFile="$historyDir/detail-data.json"

        reportLogFile="$historyDir/detail-report.log"

        [ ! -e $historyDir ] && mkdir -p $historyDir && print_log "Ok! create dir $historyDir succeed" >> $logFile

        #---------------#
        # set report IP #
        #---------------#

        privateIp='inet +addr: *(127\.|192\.168\.|10\.|172\.(1[6-9]|2[0-9]|3[0-1])\.)'

        firstIp=$(ifconfig |grep 'inet addr' |awk -F '[ :]+' '{print $4}' | egrep -v "^$privateIp" | head -n1 )

        serverId=$(grep '^server_id=' $hostInfoFile | awk -F'=' '{print $2}')

        if [ -z "$serverId" ] ; then

             print_log "Err! can NOT get server id in $hostInfoFile" >> $logFile

             exit 1

        elif [ -n "${serverId//[0-9]}" ] ; then 

             print_log "server_id contain non-digital characters : $serverId" >> $logFile

             exit 1

        elif [ $serverId = 0 ] ; then

             print_log "server id is 0 , skip cmdb report !!!" >> $logFile

             exit 1
        else 
             print_log "server_id=$serverId" >> $logFile
        fi

        #-----------#
        # find pids #
        #-----------#
    
        pidList=$(ls /proc/[0-9]*/exe -l 2>/dev/null | awk '$(NF-1) ~ /->/ {print $(NF-2)}' | sed 's/\/exe$//'| awk -F '/' '{print $NF}')

        #----------------#
        # main work here #
        #----------------#

        for i in $pidList ; do
        
             unset author pname name version auto_start
        
             exe=$(readlink /proc/$i/exe | egrep "$includeDir" ) ; [ -z "$exe" ] && continue
        
             cwd=$(readlink /proc/$i/cwd ) ; [ -z "$cwd" ] && continue
        
             user=$(ps -ouser --no-headers $i)
        
             # find author , pname , pkg-name
        
             if echo  "$cwd" | grep '^\/data\/services\/' -q ; then
        
                # this is a packaged process
        
                initConfFile="$(dirname $exe)/../init.conf"
        
                eval $(egrep '^(name|pname|author|version|auto_start)=' $initConfFile 2>/dev/null)
             fi
        
             : ${author:='unkn'}
        
             : ${pname:='unkn'}
        
             : ${name:='unkn'}
        
             : ${version:='unkn'}
        
             : ${auto_start:='unkn'}
        
             echo "$user" "$(basename $exe)" "$cwd" "$author" "$pname" "$name" "$version" "$auto_start"
        
        done | sort -k2,2 | uniq -c | tr -s '[[:space:]]' > $txtFile
        
        # conver to json format
        
        if [ -n "$(cat $txtFile)" ] ; then
        
             echo '[' > $jsonFile 

             txtFileFD=$(get_available_fd)

             if [ -z "$txtFileFD" ] ; then

                  print_log "[ERR ] | can NOT get availabe fd for file $txtFile , quit now !" >> $logFile
 
                  exit 1
             else
                  eval "exec $txtFileFD<$txtFile"

                  print_log "get availabe fd [$txtFileFD]" >> $logFile

                  ls -l /proc/self/fd >> $logFile
             fi 
        
             while read -u $txtFileFD count user exe cwd author pname name version auto_start ; do
        
                 str="{\"ip\":\"$firstIp\",\"count\":$count,\"user\":\"$user\",\"exe\":\"$(basename $exe)\""
         
                 str="$str,\"cwd\":\"$cwd\",\"author\":\"$author\",\"pname\":\"$pname\",\"name\":\"$name\""
        
                 str="$str,\"version\":\"$version\",\"auto_start\":\"$auto_start\"},"
        
                 echo "$str"
        
             done >> $jsonFile 
        
             # replace the ending comma in last line to ']'

             sed -i -r '$s/,$/]/' $jsonFile

        else # no process found under the specified dir

             echo '' > $jsonFile
        fi

        print_log "$(ls -l $jsonFile)" >> $logFile
   
        print_log "$(rm -fv $lastReportFile)" >> $logFile

        print_log "$(ln -vs $jsonFile $lastReportFile)" >> $logFile

        print_log "history dir = $historyDir" >> $logFile              
        
        print_log "---- end ----" >> $logFile

        print_log "" >> $logFile

        # remove old history dir N days ago

        find $baseDir/detail-history/ -mindepth 2 -maxdepth 2 -mtime +5 -exec rm -fr {} \; 
    
        # exit or sleep 

        if [ $(( 10#$(date +'%-M') % $reloadInterval )) = 0 ] ; then

             print_log "Ok! <<<< Release Lock succeed on every-$reloadInterval minutes >>>>" >> $logFile

             print_log "" >> $logFile

             cat $jsonFile

             exit
        else
             cat $jsonFile

             sleep 60
        fi

    done

) 200>/data/pkg/public-scripts/data/.detail-proc-report.lock && \

rm /data/pkg/public-scripts/data/.detail-proc-report.lock

