#!/bin/bash
# NAME            : $0 
# Ver             : 1.0
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2014-02-25
# Summary         : repair cmdb proc data (fill the 'pkg' and 'ver' fields)

failFlag=0

forceRepairMode=0

if [ "$1" = "-f" ] ; then

      echo "force repair mode"

      forceRepairMode=1

elif [ "$(cat /tmp/.need-repair 2>/dev/null)" = 0 ] ; then

     echo "all repaired , done"

     ls -l /tmp/.need-repair

     exit
fi

source /data/pkg/public-scripts/conf/main.conf

source /data/pkg/public-scripts/func/common.sh

source /data/pkg/public-scripts/bin/functions.sh

status=$(get_host_status 2>/dev/null)

if [ $status -ne 1 ] ; then

    echo "host status not running , quit"

    exit 0
fi

ETH0_IP=$(get_local_ip "eth0")

if [ $forceRepairMode -eq 0 ] ; then

     hour=$(date +%H)

     lastDigit=${hour#[0-9]}

     lastIpDigit=$(echo ${ETH0_IP} | awk -F '.' '{print $4}' | awk -F '' '{print $NF}')

     if [ $lastDigit -ne $lastIpDigit ] ; then

          echo "not my turn (time:$lastDigit != ip:$lastIpDigit) , just quit"

          exit 0
     else
          echo "my turn (time:$lastDigit = ip:$lastIpDigit) , let's go"
     fi
fi

serverId=$(get_server_id 2>/dev/null) || exit 1

list=$(ls /data/services/*/init.conf)

for file in $list ; do

    pkgDir=$(dirname $file)

    initFile="$pkgDir/init.conf"

    if [ -e $initFile ] ; then

         APP_NAME=$(grep 'app_name=' $initFile | awk -F '"' '{print $2}')

         #if  is_test_proc "$APP_NAME" &>/dev/null ; then

         #    echo "$APP_NAME is test or debug , skip"

         #    echo "--"

         #    continue  # next pkg
         #fi   

         if [ -e $pkgDir/bin/$APP_NAME ] || [ -e $pkgDir/sbin/$APP_NAME ] ; then

               # set cwd (enforce to bin/ dir)

               cwd="$pkgDir/bin"

               # get verId

               verId=$(cat $pkgDir/admin/pkg.verId 2>/dev/null)

               [ -z "$verId" ] && continue  # next pkg

               # get count

               # first check if special

#               pkg=$(grep '^name="' $file | awk -F '"' '{print $2}')
#
#               ver=$(grep '^version="' $file | awk -F '"' '{print $2}')

#               countX=$(set_rpt_count "$pkg"  "$ver" "$APP_NAME" 2>/dev/null)
#
#               pids=$(pidof $APP_NAME)
#
#               [ -z "$pids" ] && pids=$(ps -o pid,cmd -e | awk -v _name="/${APP_NAME}$|^${APP_NAME}$" '$2 ~ _name {print $1}' | xargs)
#
#               countY=$(echo $pids | wc -w)
#
#               if [ $countY -eq 0 ] ; then 
#
#                    echo "$pkg $ver [$APP_NAME] acutally proc count = 0 , no add , maybe it was stopped through yydeploy system"
#
#                    echo '--'
#                   
#                    continue  # no report
#
#               else
#                    count=$countX
#
#                    echo "$pkg $ver [$APP_NAME] countX=$countX , countY=$countY , set count=$count"
#               fi

               opt="--output-document=/tmp/repair-cmdb-pkg.out -a /tmp/repair-cmdb-pkg.log -t 2 -w 5 --timeout=5"
 
               # call cmdb interface

               rc=0

               cmdbIpList=$(get_cmdb_iplist)

               for i in $cmdbIpList ; do

                    delJsonStr="{\"ip\":\"${ETH0_IP}\",\"exe\":\"${APP_NAME}\",\"cwd\":\"$cwd\",\"version_id\":$verId}"

                    delUrlParaStr="ip=${ETH0_IP}&json=${delJsonStr}&server_id=$serverId"

                    echo "$delJsonStr"

                    echo "del $pkgDir from $i"

                    wget $opt "http://${i}:8088/webservice/pkgRep/del.do?${delUrlParaStr}" &>/dev/null

                    if  [ $? -eq 0 ] ; then

                          echo "[del-ok]-$pkgDir"

                          # reset rc when del succeed

                          rc=0

                          if  is_test_proc "$APP_NAME" &>/dev/null ; then

                              echo "$APP_NAME is test or debug , skip"

                              echo "[no-add]"

                              echo "--"

                              continue 2 # next pkg
                          fi   

                          # re-add to cmdb

                          # get count

                          # first check if special

                          pkg=$(grep '^name="' $file | awk -F '"' '{print $2}')

                          ver=$(grep '^version="' $file | awk -F '"' '{print $2}')

                          countX=$(set_rpt_count "$pkg"  "$ver" "$APP_NAME" 2>/dev/null)
           
                          pids=$(pidof $APP_NAME)
           
                          [ -z "$pids" ] && pids=$(ps -o pid,cmd -e | awk -v _name="/${APP_NAME}$|^${APP_NAME}$" '$2 ~ _name {print $1}' | xargs)
           
                          countY=$(echo $pids | wc -w)
           
                          if [ $countY -eq 0 ] ; then
           
                               echo "$pkg $ver [$APP_NAME] acutally proc count = 0 , no add , maybe it was stopped through yydeploy system"

							   echo "[no-add]"
           
                               echo '--'
           
                               continue 2 # next pkg
           
                          else
                               count=$countX
           
                               echo "$pkg $ver [$APP_NAME] countX=$countX , countY=$countY , set count=$count"
                          fi

                          echo "re-add $pkgDir to $i"
               
                          addJsonStr="{\"ip\":\"${ETH0_IP}\",\"exe\":\"${APP_NAME}\",\"count\":$count,\"cwd\":\"$cwd\",\"version_id\":$verId}"
               
                          addUrlParaStr="ip=${ETH0_IP}&json=${addJsonStr}&server_id=$serverId"

                          echo "$addJsonStr"

                          wget $opt "http://${i}:8088/webservice/pkgRep/add.do?${addUrlParaStr}" &>/dev/null

                          if [ $? -eq 0 ] ; then

                               echo "[add-ok]-$pkgDir"

                               break  # next pkg
                          else
                               ((++rc))
                          fi
                    else
                          echo "[del-bad]-$pkgDir"
                        
                          ((++rc))
                   fi

               done
                          
               if [ $rc -gt 0 ] ; then

                    echo "[report-bad]-$pkgDir"

                    ((++failFlag))
               fi
          else
               echo "[bad]-$pkgDir : found init.conf , but no $APP_NAME found"
          fi
     else 
          echo "[bad]-$pkgDir : no init.conf found"
     fi

     echo "--"

done

if [ $failFlag -gt 0 ] ; then

     echo '1' > /tmp/.need-repair
else
     echo '0' > /tmp/.need-repair
fi
