#!/bin/bash
# NAME            : $0 
# Ver             : 0.1
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2012-10-23 16:06
# Summary         : cpu/mem usaage & req report only for pkged process , call by every report.sh under admin/

set +H

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

# global paramaters

cpuId=105

memId=106

binDir=$(dirname $0)

maxLogSize=5242880

logFile=$(dirname $0)/report-content.log

PUB_PKG_SCRIPT_DIR=/data/pkg/public-scripts/

pidFileDir=/home/dspeak/yyms/proc_list

cpuCoreNum=$(cat /proc/cpuinfo | egrep '^processor[[:space:]]*:' -c)

maxCpuUsage=$(( 10#${cpuCoreNum} * 100 ))

maxReportProcs=10

reportPy='/home/dspeak/yyms/yymp/yymp_report_script/yymp_report.py'

# get pkg releated common variables

source $PUB_PKG_SCRIPT_DIR/conf/main.conf

source $PUB_PKG_SCRIPT_DIR/bin/mypidof.sh

source $PUB_PKG_SCRIPT_DIR/func/common.sh

source $PUB_PKG_SCRIPT_DIR/func/common-log.sh

source $PUB_PKG_SCRIPT_DIR/func/misc.sh

# get $APP_NAME from common-var.conf

source $binDir/$PKG_COMMON_ENV_VAR_FILE

# set lockfile name

lockFile="$PKG_HOME_DIR/${PKG_NAME}-${VERSION}/$PKG_SCRIPT_DIR/$PKG_LOCK_DIR/report.sh.lock"

mkdir -p $(dirname $lockFile)

rotate_log $logFile '50M'

# main work

(
    flock --nonblock 200 || exit 1

    log "get lock succed [$lockFile]"

    list=$(pidof $APP_NAME)

    # if list is empty ,try to find pid through /proc/pid/status

    if [ -z "$list" ] ; then

       list=$(egrep "^Name:[[:space:]]+${APP_NAME}$" /proc/[0-9]*/status -l 2>/dev/null | awk -F '/' '{print $3}')
    fi		

    # cut the first $maxReportProcs pids , in case of process like php-fpm may have more than 2000 sub-procs

    count=$(wc -w <<< "$list")
    
    if [ $count -gt $maxReportProcs ] ; then

         elog "$APP_NAME has too many procs : [$count] , cut to [$maxReportProcs]"

         list=$(echo "$list" | xargs -n 1 | sort -n | tail -n $maxReportProcs | xargs)
    else
         log "$APP_NAME count = [$count] , less than $maxReportProcs"
    fi

    log "$APP_NAME pid is : "
    
    log "$list"
    
    if [ -z "$list" ] ; then
    
         # no running $APP_NAME found , just exit !
    
         elog "no running $APP_NAME found"
    
         ps -ef |grep "$APP_NAME" |grep -v grep
    
         exit 0
    
    else
         # begin to work
    
         log "found running $APP_NAME"
    
         log "pid="$list
    
         echo "$list" | xargs -r ps -lf
    
         # install pidstat if needed
    
         if ! which pidstat &>/dev/null ; then
    
            aptitude -y install sysstat &>/dev/null
         fi
    
         # get cpu usage in 3 secs in batch
    
         unset pidArg
    
         for i in $list ; do
    
            pidArg="$pidArg -p $i"
    
         done
    
         log 
    
         log "prepare to run pidstat (interval=1 / count=3) for 3 seconds for all $APP_NAME"
    
         result=$(pidstat -u $pidArg 1 3)
    
         # sanity check
    
         invalidResult=$(awk -v _max=$maxCpuUsage '$6 >= _max' <<< "$result")
    
         if [ -n "$invalidResult" ] ; then
    
              elog "found invalid cpu usage data (>$maxCpuUsage)"
    
              log "$invalidResult"
    
              log "try again"
    
              result=$(pidstat -u $pidArg 1 2)
    
              invalidResult=$(awk -v _max=$maxCpuUsage '$6 >= _max' <<< "$result")
    
              if [ -n "$invalidResult" ]  ; then
    
                   elog "foud invalid cpu usage data again (>$maxCpuUsage)"
    
                   log "$invalidResult"
    
                   elog "quit now"
    
                   exit 1
              else
                   log "the data is good this time , continue"
              fi
         else
              log "all cpu usage data is good (<$maxCpuUsage) , continue"
         fi
    
         log "$result"
    
         for i in $list ; do
    
             # get pname
    
             x=$(ls $pidFileDir | awk -F '.' -v _pid=$i '$NF == _pid')
    
             pname="${x%.*}"
    
             if [ -n "$pname" ]  ; then
    
                  log "Ok! found pname [$pname] for pid [$i] , their cpu & mem usage are reported by YYMS-2 Agent"
    
                  log "I just quit"
    
                  log "$(ls -l $pidFileDir/$x)"

                  touch $INSTALL_PATH/admin/.res_report_by_yyms2
    
                  continue
             else
                  log "no pname found for pid [$i] , use pkg name [$PKG_NAME] instead"
    
                  pname=$PKG_NAME

                  touch $INSTALL_PATH/admin/.res_report_by_pkg
             fi
    
             # get cpu usage
    
             cpuUsage=$(echo "$result" | egrep '^Average:' | awk -v _pid=$i '$2 == _pid {print $6}')

             : ${cpuUsage:=0}
    
             log "pid=$i usage=$cpuUsage pname=$pname id=$cpuId"
    
             log "$(python $reportPy "id=${cpuId}&value=${cpuUsage}&pname=${pname}&pid=${i}")"
    
             # mem usage (Bytes)
    
             memUsage=$(cat /proc/$i/status | grep '^VmRSS' | awk '{printf "%.0f\n", $2*1024}')
    
             : ${memUsage:=0}
    
             log "pid=$i usage=$memUsage pname=$pname id=$memId"
    
             log "$(python $reportPy "id=${memId}&value=${memUsage}&pname=${pname}&pid=${i}")"
    
         done

         log "-- END --"

         log ""
         
    fi

) >> $logFile 2>&1  200>$lockFile && \
rm -f $lockFile
