#!/bin/bash
# NAME            : $0 
# Ver             : 0.5
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2015-01-16
# Summary         : Scan listening port info and report

set +H

MYPID=$$

export LANG=C

export LC_ALL=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

source /data/pkg/public-scripts/conf/main.conf

source /data/pkg/public-scripts/func/common.sh

source /data/pkg/public-scripts/func/misc.sh

reportUrl='http://security.yy.com/admin/intf/postPortInfo.jsp'

reportLog='/data/pkg/public-scripts/data/collect_listen_port.out'

jsonFile='/data/pkg/public-scripts/data/listen_port.info'

localIp=$(get_local_ip "eth0")

if [ -z "$localIp" ] ; then

     if is_valid_interface "bond0" ; then

         localIp=$(get_local_ip "bond0" 2>/dev/null)
     else
          localIp=$(get_local_ip "eth1")
     fi
fi

ts=$(date +'%F %T')

x=$(ss -4lntup | tail -n +2 | sort -k4,4n -k5,5n)

echo "$x" > /data/pkg/public-scripts/data/listen_ports.log

tempResult=$(while read line ; do

   unset port portNum addr pids proc exe cwd name version author

   unset sinceWhen firstPid procUser md5 mtime

   port=$(echo "$line" | egrep -m1 -o '[0-9.*:]+:[0-9]+')

   portNum=$(echo "$port" | awk -F ':' '{print $NF}')

   addr=$(echo "$port" | awk -F ':' '{print $1}')

   if [ -z "$addr" ] || [ "$addr" = "*" ] ; then

      addr="0.0.0.0"
   fi

   proto=$(echo "$line" | awk '{if ($1=="tcp") print "tcp" ;else print "udp"}')

   pids=$(echo "$line" | egrep -o 'users:(.*)' | sed -r 's/\),/\n/g' | \
   
          awk -F ',' '{print $2}' | xargs -r ps --no-headers -opid | \
		  
          sort -u)

   if [ -z "$pids" ] ; then

      continue  # temp proc , already quit
   fi	   
   
   proc=$(echo "$line" | egrep -o 'users:(.*)' | sed -r 's/\),/\n/g' | \
   
          awk -F ',' '{print $2}' | xargs -r ps --no-headers -ocmd | \
		  
          sort -u | awk '{print $1}' | xargs -r -i basename {} |sort -u)

   if [ -z "$proc" ] ; then

        continue # temp proc , already quit
   else		 
        sinceWhen=$(date -d "$(ps --no-headers -o lstart $(echo "$pids"|sort -n|head -n1))" +'%F %T')

        firstPid=$(ps --no-headers -opid,lstart $pids | sort --month-sort -k3,3 -k4,4n | awk 'NR==1 {print $1}')

        procUser=$(ps -ouser --no-headers $firstPid)
   fi

   exe=$(readlink /proc/$(echo "$line" | egrep -o 'users:(.*)' | \
   
         sed -r 's/\),/\n/g' | awk -F ',' '{print $2}' | \
		 
         xargs -r ps --no-headers -opid | head -n 1 | xargs)/exe | awk '{print $1}')

   if [ ! -f "$exe" ] ; then

	exe="(deleted)"

        md5="(deleted)"

        mtime="(deleted)"
   else
	md5=$(md5sum $exe | awk '{print $1}') 

        mtime=$(date -d @$(stat --format='%Y' $exe) +'%F %T')
   fi	   
   
   cwd=$(readlink /proc/$(echo "$line" | egrep -o 'users:(.*)' | \
   
         sed -r 's/\),/\n/g' | awk -F ',' '{print $2}' | \
		 
         xargs -r ps --no-headers -opid | head -n 1 | xargs)/cwd | awk '{print $1}')
   
   if [ ! -d "$cwd" ] ; then

        cwd="(deleted)"
   fi	   
   
   if [ "$exe" != "(deleted)" ] && [ "$cwd" != "(deleted)" ] ; then

        # set variable "name"

        eval $(cat $(dirname $exe)/../init.conf 2>/dev/null | egrep '^name=')

         # set variable "version"

         eval $(cat $(dirname $exe)/../init.conf 2>/dev/null | egrep '^version=')  

         # set variable "author"

         eval $(cat $(dirname $exe)/../init.conf 2>/dev/null | egrep '^author=')   
   fi

   : ${name:=-}

   : ${version:=-}

   : ${author:=-}

   echo "{\"proto\":\"$proto\",\"addr\":\"$addr\",\"port\":$portNum,\"proc\":\"$proc\",\"procUser\":\"$procUser\",\"exe\":\"$exe\",\"cwd\":\"$cwd\",\"md5\":\"$md5\",\"mtime\":\"$mtime\",\"pkg\":\"$name\",\"ver\":\"$version\",\"author\":\"$author\",\"firstPid\":$firstPid,\"sinceWhen\":\"$sinceWhen\"}"
   
done <<< "$x")

jsonResult=$(echo "$tempResult" | sort -u | sed -r 's/port":([0-9]+),/port \1 /' | \

             sort -k2,2n | sed -r 's/port ([0-9]+) /port\":\1,/' | sed '$!s/}$/},/' | \

             sed -e '1s/^/[/' -e '$s/$/]/')

jsonResult="{\"ts\":\"$ts\",\"ip\":\"$localIp\",\"info\":${jsonResult}}"

echo "$jsonResult" > $jsonFile 

wget -o $reportLog --post-data "data=$(echo "$jsonResult")" -O /dev/null $reportUrl 
