#!/bin/bash

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

function print_log {

   local line

   while read line ; do

      echo "$(date +'%F %T') | $line" | awk '{$3=sprintf("%22s | ",$3);print $0}'

   done < <(echo "$*")

   return

}

function task_clean {

   local taskFile=${1:?$FUNCNAME [ERR ] | task input file is NULL}

   local logFile=${2:?$FUNCNAME [ERR ] | task manager log file is NULL}

   local pid action taskId sessionId

   while read pid action taskId sessionId pkg ver verId ; do

      if ! ps $pid &>/dev/null ; then          # task was finished , but still remain in taskFile , remove it !

           sed -r -i.bak "/^[[:space:]]*$pid[[:space:]]+/d" $taskFile && \

           print_log "$FUNCNAME [INFO] | pid=$pid is finished , removed it" >> $logFile

           print_log "$FUNCNAME [INFO] | remove task=[$pid] ${action:=-} ${taskId:=-} ${sessionId:=-} ${pkg:=-} from $taskFile" >> $logFile

      elif ps --no-headers $pid | grep -q -F '<defunct>' ; then    # task is running , but is zombies , ignore it

           sed -r -i.bak "/^[[:space:]]*$pid[[:space:]]+/d" $taskFile && \

           print_log "$FUNCNAME [INFO] | pid=$pid is running , but became zombies , removed it" >> $logFile

           print_log "$FUNCNAME [INFO] | remove task=[$pid] ${action:=-} ${taskId:=-} ${sessionId:=-} ${pkg:=-} from $taskFile" >> $logFile
 
      fi

   done < <(egrep -v '^#|^[[:space:]]*$' $taskFile 2>/dev/null)
}



function is_a_conflict_task {

   # operation on same pkg same ver

   local taskFile=${1:?$FUNCNAME [ERR ] | task file is NULL}

   local action=${2:?$FUNCNAME [ERR ] | action is NULL}

   local taskId=${3:?$FUNCNAME [ERR ] | taskId is NULL}

   local sessionId=${4:?$FUNCNAME [ERR ] | sessionId is NULL}

   local pkg=${5:?$FUNCNAME [ERR ] | pkg name is NULL}

   local ver=${6:?$FUNCNAME [ERR ] | version number is NULL}

   #local verId=${7:?$FUNCNAME [ERR ] | versionId is NULL}

   local pid

   touch $taskFile

   #pid=$(awk -v x=$pkg -v y=$ver -v z=$verId '(NF == 7) && ($5 == "pkg="x) && ($6 == "ver="y) && ($7 == "versionId="z) {print $1}' $taskFile)

   pid=$(awk -v x=$pkg -v y=$ver '(NF == 7) && ($5 == "pkg="x) && ($6 == "ver="y) {print $1}' $taskFile)

   if [ -n "$pid" ] ; then

        echo "$pid" 

        return 2
   else
        return 0
   fi

}


function is_a_new_task {

   local taskFile=${1:?$FUNCNAME [ERR ] | task file is NULL}

   local action=${2:?$FUNCNAME [ERR ] | action is NULL}

   local taskId=${3:?$FUNCNAME [ERR ] | taskId is NULL}

   local sessionId=${4:?$FUNCNAME [ERR ] | sessionId is NULL}

   local verId=${5:?$FUNCNAME [ERR ] | versionId is NULL}

   local pid

   touch $taskFile 

   pid=$(awk -v x=$taskId -v y=$sessionId -v z=$verId '(NF == 7) && ($3 == "taskId="x) && ($4 == "sessionId="y) && ($7 == "versionId="z) {print $1}' $taskFile)

   if [ -n "$pid" ] ; then

        echo "$pid" 

        return 2 

   elif [ -d $PUB_SCRIPT_LOG_DIR/$taskId/$sessionId/$verId ] ; then

        return 3
   else 
        return 0
   fi 

}

function add_task_to_taskFile {

    local scriptId=${1:?$FUNCNAME [ERR ] | scriptId is NULL}

    local taskFile=${2:?$FUNCNAME [ERR ] | taskFile is NULL}

    local actionStr=${3:?$FUNCNAME [ERR ] | actionStr is NULL}

    local taskIdStr=${4:?$FUNCNAME [ERR ] | taskIdStr is NULL}

    local sessionIdStr=${5:?$FUNCNAME [ERR ] | sessionIdStr is NULL}

    local pkgStr=${6:?$FUNCNAME [ERR ] | pkgStr is NULL}

    local verStr=${7:?$FUNCNAME [ERR ] | versionStr is NULL}

    local verIdStr=${8:?$FUNCNAME [ERR ] | versionIdStr is NULL}

    echo "$scriptId $actionStr $taskIdStr $sessionIdStr $pkgStr $verStr $verIdStr" >> $taskFile
}

function remove_self_task {

    local scriptId=${1:?$FUNCNAME [ERR ] | scriptId is NULL}

    local taskFile=${2:?$FUNCNAME [ERR ] | taskFile is NULL}

    local logFile=${3:?$FUNCNAME [ERR ] | logfile is NULL}

    sed -r -i.bak "/^[[:space:]]*$scriptId[[:space:]]+/d" $taskFile

    if ! egrep -q "^[[:space:]]*$scriptId[[:space:]]+" $taskFile ; then

         print_log "$FUNCNAME [INFO] | ok! my job is done (pid=$scriptId) , remove script [$scriptId] from $taskFile succeed" >> $logFile
 
         return 0
    else
         print_log "$FUNCNAME [ERR ] | remove script [$scriptId] from $taskFile failed" >> $logFile

         return 1
    fi
}
