#!/bin/bash

export LC_ALL=C

export LANG=C

export PATH=/usr/bin:/bin:/sbin:/usr/sbin

source /data/pkg/public-scripts/conf/environ.conf

function split_task_log {

   local logFile=${1:?$FUNCNAME [ERR ] | logFile is NULL}

   local prefix=${2:?$FUNCNAME [ERR ] | suffix is NULL}

   local detailMsg

   ( 
      if [ -f $logFile ] ; then

          cd $(dirname $logFile)
    
          split --suffix-length=2 -d -b $MAX_RPT_LOG_BYTES $logFile "$prefix"
    
          detailMsg="$(cat ${prefix}00)"
    
          if [ $(ls ${prefix}?? | wc -w) -gt 1 ] ; then
    
               detailMsg=$(echo -e "${detailMsg}\n([truncated because > $(( 10#${MAX_RPT_LOG_BYTES} / 1024 ))KB])")
          fi
    
          rm -f ${prefix}?? &>/dev/null
    
          echo "$detailMsg"
     else
          return 1
     fi
   )

   return
}


function do_report {

  if [ $# -lt 6 ] ; then

       echo "$(date +'%F %T')| $FUNCNAME [ERR ] | deployReport.py <taskId> <sessId> <code> <msg> <detail> <verId> [<cfgId>]"

       echo "$*"

       return 1
  else
       local taskId=${1:?$FUNCNAME [ERR ] | task id is NULL}

       local sessionId=${2:?$FUNCNAME [ERR ] | session ID is NULL}

       local code=${3:?$FUNCNAME [ERR ] | return code is NULL}

       local msg=${4:?$FUNCNAME [ERR ] | return msg is NULL}

       local detailLog=${5:?$FUNCNAME [ERR ] | detail log file is NULL}

       local versionId=${6:?$FUNCNAME [ERR ] | versionId is NULL}

       local cfgId=$7

       local count cfgId

       mkdir -p $PUB_SCRIPT_LOG_DIR/$taskId/$sessionId/ &>/dev/null

       (

         echo "msg=\"$msg\""

         msg=$(echo "$msg" | sed -r 's/^[0-9]{4}-[0-9]{2}-[0-9]{2} ([0-9]{2}:?){3} \|[[:space:]]+//') 

         echo "msg=\"$msg\""

         echo "detailMsg=\"$detailLog\""

         if [ "$ENV" = "product" ] ; then

              echo "reportProg=/home/dspeak/yyms/yyms_agent_2_d/cron/deployReport.py"

              ls -l /home/dspeak/yyms/yyms_agent_2_d/cron/deployReport.py
         else
              echo "reportProg=/data/deploy_test/deployReport.py"

              ls -l /data/deploy_test/deployReport.py
         fi  

         msgEncoded=$(echo "$msg"      | base64 -w0) 

         detailMsg=$(echo "$detailLog" | base64 -w0) 

         mkdir -p $PUB_SCRIPT_LOG_DIR/$taskId/$sessionId

         if [ "$ENV" = "product" ] ; then

              cd /home/dspeak/yyms/yyms_agent_2_d/cron/
         else
              cd /data/deploy_test
         fi
         
         set -x
    
         count=1

         until [ $count -gt 5 ] ; do
   
              rs=$(python deployReport.py "{'taskid':$taskId,'sessid':$sessionId,'code':$code,'msg':'$msgEncoded','detailed':'$detailMsg','versionId':$versionId,'cfgId':${cfgId:=''}}")

              rs=$(echo "$rs" | tr -d '[[:space:]]')

              if [ "$rs" = "0" ] ; then
	
		   echo "report succeed"

                   break
              else
                   echo "#$count report failed , try again after $(( 10#${count} * 2 )) seconds ...."

                   sleep $(( 10#${count} * 2 ))

                   ((count+=1))
              fi
                   
         done

         # send alarm if report result failed

         if [ "$rs" != "0" ] ; then
  
              # send alarm msg
  
              print_log "$FUNCNAME [ERR ] | rs=$rs after 5 retries , send alarm" 
  
              ( cd /usr/local/i386/*/auto/ && sudo bash ./insert_message.sh "yydeploy: $(get_all_ip) report failed($rs). taskId=$taskId sid=$sessionId vid=$versionId" )
         
              set +x
  
              return 1
         else
              print_log "$FUNCNAME [INFO] | rs=$rs, count=$count , report ok" 
  
              set +x

              return 0
         fi

         

       ) &> $PUB_SCRIPT_LOG_DIR/$taskId/$sessionId/$versionId/report.log
  fi
}
