#!/bin/bash
# NAME            : $0 
# Ver             : 0.1
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2013-03-14 16:46
# Summary         : duowan pkg release system background scripts 

export LANG=C

export LC_ALL=C

if [ $# -lt 2 ] || [ $# -gt 3 ] ; then

     echo "{'code':1,'message':'语法错误! $0 <srcdir> <dstdir> [<maxMB>]'}"
 
     exit 1
else
     srcDir=$1
 
     dstDir=$2

     maxSize=${3:-100}

     # force to integer

     maxSize=$(printf "%.0f" $maxSize) 

     if [ "${dstDir#.}" != "$dstDir" ] ; then

        echo "{'code':1,'message':'错误! 目标目录 $dstDir 必须是全路径格式'}"

        exit 1
     fi 

     if [ -d "$srcDir" ] && [ -r "$srcDir" ] && [ -x "$srcDir" ] ; then
  
         if [ -d "$dstDir" ] && [ -w "$dstDir" ] && [ -x "$dstDir" ] ; then

              # find file

              x=$(find "$srcDir" -type f | xargs -n1 | grep -xv "${srcDir%\/}/cfgmgr.conf")

              if [ -z "$x" ] ; then

                   echo "{'code':2,'message':'错误! 源目录 $srcDir 下没有找到任何文件'}"

                   exit 1
              fi 
     
              # prepare to tar

              tmpPkgName="$pkg-$ver-noarch-$$.tar.gz"

              cd  $srcDir && tar -zcf $dstDir/$tmpPkgName *

              size=$(( $(stat --format='%s' $dstDir/$tmpPkgName) / 1024 / 1024 ))

              if [ $size -gt $maxSize ] ; then

                 rm -f $dstDir/$tmpPkgName

                 echo "{'code':1,'message':'错误! 生成的 TAR 包大小 (${size}MB) 超过 ${maxSize}MB'}"

                 exit 1
              fi

              md5=$(md5sum $dstDir/$tmpPkgName|awk '{print $1}')

              # rename the pkg file

              mv $dstDir/$tmpPkgName $dstDir/${md5}_config.tar.gz
   
              echo "{'code':0,'message':'$md5'}"

              exit 0
         else 
              echo "{'code':1,'message':'错误! 目标目录 $dstDir 不存在或者权限被拒绝'}"

              exit 1
         fi

     else
          echo "{'code':1,'message':'错误! 源目录 $srcDir 不存在或者权限被拒绝'}"
     
          exit 1
     fi
fi
