#!/bin/bash
# NAME            : $0 
# Ver             : 0.3
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2012-05-29 10:05 
# Summary         : get reomte file/directory via rsync 

# usage : $0 <remote-ip> <moduleName> <path-to-file> <dstDir> <ruser> <rport> <passwd>

function print_log {
    
    local line
         
    while read line ; do
              
          echo "$(date +'%F %T')| $line"
                       
    done < <(echo "$*")
                          
    return
} 


export PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

binDir=$(cd $(dirname $0) && pwd -P)

pwdFile="$binDir/rsync-passwd.$$"

maxRate=3072

if [ $# -ne 7 ] ; then

     #echo "{'code':1,'message':'Err! paramater num err [$0 <remote-ip> <moduleName> <path-to-file> <dstDir> <ruser> <rport> <passwd>]'}" 

     echo "{'code':1,'message':'参数错误! $0 <remote-ip> <moduleName> <path-to-file> <dstDir> <ruser> <rport> <passwd>'}" 

     exit 1

elif [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ] || [ -z "$4" ] || [ -z "$5" ] || [ -z "$6" ] || [ -z "$7" ] ; then

     #echo "{'code':1,'message':'Err! found empty paramater : $@'}"

     echo "{'code':1,'message':'错误! 参数值存在空值 : $@'}"

     exit 1

elif [ ! -d $(dirname $4) ] ; then   

     #echo "{'code':1,'message':'Err! invalid output dir $(dirname $4)'}" 

     echo "{'code':1,'message':'错误! 目标路径 $(dirname $4) 不是目录'}" 

     exit 1
else
     rIp=$1 ; module=$2 ; path=$3 ; dstDir=$4 ; ruser=$5 ; rport=$6 ; passwd="$7"
     
     # enfore password file permission is 400

     echo "$passwd" > $pwdFile

     chmod 0400 $pwdFile

     # clear download log

     logFile=/data/pkg/public-scripts/log/rsync_download.log 

     size=$(stat --format='%s' $logFile)

     if [ $size -gt 52428800 ] ; then

          : > $logFile
     fi

     sudo mkdir -p $(dirname $logFile)

     sudo touch $logFile

     sudo chmod 666 $logFile

     print_log "Paramaters   = [$*]" >> $logFile
  
     print_log "        host = [$1]" >> $logFile
  
     print_log "rsync module = [$2]" >> $logFile
  
     print_log "   file path = [$3]" >> $logFile
  
     print_log "     dst Dir = [$4]" >> $logFile

     print_log "       ruser = [$5]" >> $logFile

     print_log "       rport = [$6]" >> $logFile

     print_log "===================== prepare to download ==========================" >> $logFile

     # rsync daemon connection test

     #nc -w 2 -nvzz $rIp $rport &>/dev/null || { echo "{'code':1,'message':'Err! rsync connection to host [$rIp:$rport] failed'}" ; rm -f $pwdFile ; exit 1; }  

     nc -w 2 -nvzz $rIp $rport &>/dev/null || { echo "{'code':1,'message':'错误! 无法连接 $rIp:$rport'}" ; rm -f $pwdFile ; exit 1; }  

     print_log "connection test for $rIp:$rport passed" >> $logFile

     # check if remote file/dir exist first

     sudo rsync --timeout=2 --password-file=$pwdFile --list-only rsync://$ruser@$rIp:$rport/$module/$path >> $logFile 2>&1

     rc=$?

     if [ $rc -eq 5 ] ; then

          #msg="Err! module [$2] NOT exist or access DENIED by rsync daemon $rIp:$rport"

          msg="错误! rsync 模块 [$2] 不存在,或者模块路径不存在, 或者权限被拒绝"

          print_log "$msg" >> $logFile

          echo "{'code':1,'message':'$msg'}"

          rm -f $pwdFile ; exit 1

     elif [ $rc -eq 23 ] ; then

          #msg="Err! remote file $path (in module $module) not exist"

          msg="错误! 远程文件 $path 不存在于 rsync 模块 \"$2\" 下,请检查传入的文件路径是否正确"

          print_log "$msg" >> $logFile

          echo "{'code':1,'message':'$msg'}"

          rm -f $pwdFile ; exit 1

     elif [ $rc -ne 0 ] ; then

          #msg="Err! get remote file info failed , download abort"

          msg="错误! 无法获取远程文件信息" 

          print_log "$msg" >> $logFile

          echo "{'code':1,'message':'$msg'}"

          rm -f $pwdFile ; exit 1
     else 
          sudo rsync --timeout=60 --bwlimit=$maxRate -avvz --password-file=$pwdFile rsync://$ruser@$rIp:$rport/$module/$path $dstDir >> $logFile 2>&1

          rc=$?

          print_log "==================== return code ===================================" >> $logFile

          if [ $rc -eq 0 ] ; then
    
                print_log "rc=succeed" >> $logFile
    
                #echo "{'code':0,'message':'Ok! download succeed'}"

                echo "{'code':0,'message':'rsync 下载成功'}"
    
                rm -f $pwdFile ; exit 0
          else
                case $rc in 
    
                   1)  msg="Err! Syntax or usage error" ;;
    
                   3)  msg="Err! Errors selecting input/output files, dirs" ;;
    
                   5)  msg="Err! Error starting client-server protocol" ;;
    
                   10) msg="Err! Error in socket I/O" ;;
    
                   11) msg="Err! Error in file I/O" ;;
    
                   12) msg="Err! Error in rsync protocol data stream" ;; 
    
                   23) msg="Err! source file not found or Partial transfer due to error" ;;
    
                   30) msg="Err! Timeout in data send/receive" ;;
    
                   35) msg="Err! Timeout waiting for daemon connection" ;;
    
                   *)  msg="Err! unknown error"
    
                esac 
    
                print_log "rc=failed" >> $logFile
    
                echo "{'code':1,'message':'$msg'}"
                 
                rm -f $pwdFile ; exit $rc
           fi

     fi
fi
