#!/bin/bash
# NAME            : $0 
# Ver             : 0.6
# Author          : boblin(linbobo@chinaduo.com)
# LastModified by : boblin(linbobo@chinaduo.com)
# LastModified    : 2014-09-05
# Summary         : duowan pkg release system background scripts
# Change-log      : add tar log && md5sum log for debug

export LANG=C

export LC_ALL=C

source /data/pkg/public-scripts/func/common-log.sh

source /data/pkg/public-scripts/func/common.sh

source /data/pkg/public-scripts/func/misc.sh

binDir="$(cd $(dirname $0) && pwd -P)"

errLog="$binDir/tar-error.log"

if [ $# -lt 5 ] || [ $# -gt 6 ] ; then

     echo "{'code':1,'message':'语法错误! tar.sh <srcdir> <dstdir> <pkg> <ver> <platform> [<maxMB>]'}"

	 elog "\$1=[$1] , \$2=[$2], \$3=[$3] , \$4=[$4] , \$5=[$5] , \$6=[$6]" >> $errLog 2>&1

     exit 1
else
     srcDir=$1

     dstDir=$2

     pkg=$3

     ver=$4

     platform=$5

     maxSize=${6:-300}

	 logDir="$binDir/log/$pkg"

     logFile="$logDir/md5.log"

	 mkdir -p $logDir &>/dev/null

	 dlog "$0 $*" >> $logFile 2>&1

     # force to integer

     maxSize=$(printf "%.0f" $maxSize)

     if [ "${dstDir#.}" != "$dstDir" ] ; then

        echo "{'code':1,'message':'错误! 目标目录 $dstDir 必须是全路径格式'}"

        elog "dst dir must be full-path" >> $logFile 2>&1

        exit 1
     fi

     if [ -d "$srcDir" ] && [ -r "$srcDir" ] && [ -x "$srcDir" ] ; then

         if [ -d "$dstDir" ] && [ -w "$dstDir" ] && [ -x "$dstDir" ] ; then

              # extra job (check mandatory directory)

              for dir in bin conf lib data log  ; do

                  if [ ! -d $srcDir/$dir ] ; then

                        echo "{'code':1,'message':'错误! 基础目录 $srcDir/$dir 不存在'}"

                        elog "basic dir [$srcDir/$dir] not exist" >> $logFile 2>&1

                        exit 1
                  fi

              done

              # find file

              x=$(find "$srcDir" -type f | xargs -n1 | grep -xv "${srcDir%\/}/init.conf")

              if [ -z "$x" ] ; then

                   echo "{'code':2,'message':'错误! 源目录 $srcDir 下没有找到任何文件'}"

				   elog "no files found under src dir $srcDir" >> $logFile 2>&1

                   exit 1
              fi

			  mkdir -p /data/tmp

              tmpDir=/data/tmp/$pkg-$ver

              rm -fr $tmpDir &>/dev/null

              mkdir -p $tmpDir &>/dev/null || { echo "{'code':1,'message':'错误! 创建临时目录 $tmpDir 失败'}" ;
			  
			                                    elog "create tmp dir $tmpDir failed"  >> $logFile 2>&1 ; exit 1; 
											  }

              cp -a $srcDir/* $tmpDir &>/dev/null

			  # remove testDoc/ dir under tmp dir , it shouldn't be included in pkg

              rm -fr $tmpDir/testDoc &>/dev/null

			  # check all UGC scripts ( start|stop|restart|monitor|resovle|post-install )

              postInstallScripts=$(sed -nr '/<post_install>/,/<\/post_install>/p' $srcDir/init.conf | sed -n -e '1!p' | sed -n -e '$!p')

              startScripts=$(sed -nr '/<start>/,/<\/start>/p' $srcDir/init.conf | sed -n -e '1!p' | sed -n -e '$!p')

              stopScripts=$(sed -nr '/<stop>/,/<\/stop>/p' $srcDir/init.conf | sed -n -e '1!p' | sed -n -e '$!p')

              restartScripts=$(sed -nr '/<restart>/,/<\/restart>/p' $srcDir/init.conf | sed -n -e '1!p' | sed -n -e '$!p')

              monitorScripts=$(sed -nr '/<monitor>/,/<\/monitor>/p' $srcDir/init.conf | sed -n -e '1!p' | sed -n -e '$!p')

              resolveScripts=$(sed -nr '/<resolve>/,/<\/resolve>/p' $srcDir/init.conf | sed -n -e '1!p' | sed -n -e '$!p')

			  syntaxChkResult=$(bash -n <<< "$postInstallScripts" 2>&1)

			  if [ $? -ne 0 ] ; then

				   echo "{'code':3,'message':'安装后脚本存在语法错误'}"

                   elog "post-install script found syntax err" >> $logFile 2>&1

				   exit 1
		      fi

              syntaxChkResult=$(bash -n <<< "$startScripts" 2>&1)

              if [ $? -ne 0 ] ; then

                   echo "{'code':3,'message':'启动脚本存在语法错误'}"

                   elog "start script found syntax err" >> $logFile 2>&1

                   exit 1
              fi

              syntaxChkResult=$(bash -n <<< "$stopScripts" 2>&1)

              if [ $? -ne 0 ] ; then

                   echo "{'code':3,'message':'停止脚本存在语法错误'}"

                   elog "stop script found syntax err" >> $logFile 2>&1

                   exit 1
              fi

              syntaxChkResult=$(bash -n <<< "$restartScripts" 2>&1)

              if [ $? -ne 0 ] ; then

                   echo "{'code':3,'message':'重启脚本存在语法错误'}"

                   elog "restart script found syntax err" >> $logFile 2>&1

                   exit 1
              fi

              syntaxChkResult=$(bash -n <<< "$monitorScripts" 2>&1)

              if [ $? -ne 0 ] ; then

                   echo "{'code':3,'message':'监控脚本存在语法错误'}"

                   elog "monitor script found syntax err" >> $logFile 2>&1
				
                   exit 1
              fi

              syntaxChkResult=$(bash -n <<< "$resolveScripts" 2>&1)

              if [ $? -ne 0 ] ; then

                   echo "{'code':3,'message':'异常处理脚本存在语法错误'}"

                   elog "resolve script found syntax err" >> $logFile 2>&1

                   exit 1
              fi

              # prepare to tar , first change file perm under bin/

              cd $tmpDir/bin && chmod 755 * &>/dev/null

			  cd $(dirname $tmpDir) && dlog "generate md5 list" >> $logFile 2>&1

              dlog "$(find $(pwd)/$pkg-$ver -type f | LC_CTIME=C xargs -n 10 -r ls -l --full-time)" >> $logFile 2>&1

			  dlog "$(find $(pwd)/$pkg-$ver -type f | xargs md5sum)" >> $logFile 2>&1

			  # do NOT use tar -zcvf , because it will case md5sum changed each time

			  # see (-n option) of gzip's manual
			  
			  tar -c $pkg-$ver 2>/dev/null | gzip -n > $dstDir/$pkg-$ver-$platform.tar.gz

              size=$(( 10#$(stat --format='%s' $dstDir/$pkg-$ver-$platform.tar.gz ) / 1024 / 1024 ))

              if [ $size -gt $maxSize ] ; then

                 rm -f $dstDir/$pkg-$ver-$platform.tar.gz &>/dev/null

                 rm -fr $tmpDir &>/dev/null

                 echo "{'code':1,'message':'错误! 生成的 TAR 包大小 (${size}MB) 超过 ${maxSize}MB'}"

				 elog "tarball size too large (>${maxSize}MB)" >> $logFile 2>&1

                 exit 1
              fi

              flag=0

			  while [ $flag -lt 5 ] ; do

                      md5=$(md5sum $dstDir/$pkg-$ver-$platform.tar.gz 2>>$logFile | awk '{print $1}')
   
                      if [ -n "$md5" ] ; then
   
                            rm -fr $tmpDir &>/dev/null
          
                            echo "{'code':0,'message':'$md5'}"
       
           			        dlog "$pkg-$ver-$platform.tar.gz $md5" >> $logFile 2>&1

							dlog "-- END --" >> $logFile 2>&1

							echo >> $logFile

                            exit 0
                       else
   				            sleep 0.5 

							((flag++))
                       fi							
     		  done

			  if [ $flag -gt 0 ] ; then

				   echo "{'code':1,'message':'错误! MD5 为空'}"

                   elog "md5 is null" >> $logFile 2>&1

				   exit 1
			   fi
         else 
              echo "{'code':1,'message':'错误! 目标目录 $dstDir 不存在或者权限被拒绝'}"

              elog "dst dir $dstDir not exist or permission denied" >> $logFile 2>&1

              exit 1
         fi

     else
          echo "{'code':1,'message':'错误! 源目录 $srcDir 不存在或者权限被拒绝'}"
     
          elog "src dir $srcDir not exist or permission denied" >> $logFile 2>&1

          exit 1
     fi
fi
