#!/bin/bash

function instance_status_changed {

   local lastCheckTs=$(cat $lastRunTsFile 2>/dev/null)

   local list=$(find /data/services/*/admin/cache/proc.info 2>/dev/null)

   local i

   for i in $list ; do

       if [ $(stat --format=%Z $i) -ge $lastCheckTs ] ; then

             debug_short_log "instance status [$i] changed since last check [$(date -d@$lastCheckTs +'%F %T')]"

             # no further check , return immediately

             return 0
       else
             debug_short_log "instance status [$i] not changed since last check [$(date -d@$lastCheckTs +'%F %T')]" 
       fi

   done

   return 1

}


function check_proc_startTime {

   local pidExeList="${1:?$FUNCNAME [ERR ] | pidExeList is NULL}"

   local proc="${2:?$FUNCNAME [ERR ] | proc is NULL}"

   local path="${3:?$FUNCNAME [ERR ] | path is NULL}"

   local pLevel="${4:?$FUNCNAME [ERR ] | pLevel is NULL}"

   local lastCheckTs=$(cat $lastRunTsFile 2>/dev/null)

   local pids startTs i

   : {lastCheckTs:=$(date +%s)}

   debug_short_log "last not running check time is $lastCcheckTs [$(date -d @$lastCheckTs +'%F %T')]"

   pids=$(echo "$pidExeList" | awk '{print $9}' | awk -F '/' '{print $3}')

   for i in $pids ; do

         startTs=$(ps --no-headers -olstart $i | xargs -i date -d {} +'%s')
   
         if [ $startTs -ge $lastCheckTs ] ; then
   
              # something happend with not-running-check after last round
   
              # but auto-start by monitor.sh . By now ,the  proc num is right
   
              log_unexp_restart "$todayAlarmFile" "$proc" "$pLevel" "$path" "$i" "$startTs"
   
              warn_short_log "!!!! pid [$i] restart unexpected on $(date -d@$startTs +'%F %T') !!!!"

              warn_short_log "skip other pids check"

              break  # only check 1 pid for each pkg-ver
         else
              debug_short_log "pid [$i] start time is $(date -d@$startTs +'%F %T') , before last check time , ok" 
         fi
   done

}

function get_unexp_restart_log {

   local startTs=${1:?$FUNCNAME [ERR ] | startTs is NULL}

   local file=${2:?$FUNCNAME [ERR ] | file is NULL}

   local path=${3:?$FUNCNAME [ERR ] | path is NULL}

   debug_short_log "begin restart-too-many check ...."

   debug_short_log "scan unexpect restart record within 10 minutes"

   debug_short_log "try to match $path"

   awk -v x=$startTs -v y="$path" '$4 == "unexpect_restart_at" && $1 >= x && $5 >= x && $10 == y' $file

   return

}


function get_unexp_restart_count {

   local startTs=${1:?$FUNCNAME [ERR ] | startTs is NULL}

   local file=${2:?$FUNCNAME [ERR ] | file is NULL}

   get_unexp_restart_log "$startTs" "$file" | wc -l

   return
}


function quit_if_vhost {

   if is_virtual_host ; then
   
        err_short_log "this is a virutal host , just quit"
   
        date +%s > $lastRunTsFile 
   
        return 1
   fi

   return
}


function quit_if_operating_host {

   local hostStatus

   hostStatus=$(get_host_status)
 
   debug_short_log "host status = [$hostStatus]"

   if [ $hostStatus -ne 1 ] ; then

        err_short_log "host status not 1(running) , just quit"

        date +%s > $lastRunTsFile

        return 1 
   fi

   return
}


function backup_cmdb_pinfo_file {

   cp $cmdbProcCacheFile $cmdbProcCacheFile.bak &>/dev/null

   return
}


function restore_cmdb_pinfo_file {

   debug_short_log "restore old cmdb proc info file"

   cp $cmdbProcCacheFile.bak $cmdbProcCacheFile

   return
}

function choose_cmdb_ip {

   local chosenIp

   chosenIp=$(cat $cmdbIpCacheFile)

   if [ -z "$chosenIp" ] ; then

       err_short_log "no suitable ip found , all Ip's pkg-loss rate > 0%"

       err_short_log "enforce to choose two ip randomly ( NOTE :: some \"sort\" command may not support '-R' opt )"

       chosenIp="$(echo $cmdbDefaultIpList $cncProxyIpList | xargs -n1 | { sort -R 2>/dev/null || sort ; } | xargs)"
   fi

   echo $chosenIp

   return
}


function download_cmdb_pinfo {
  
   local chosenIp eth0Ip i rc

   chosenIp=$(choose_cmdb_ip)

   eth0Ip=$(get_local_ip "eth0" | head -1)

   for i in $chosenIp ; do

       # real-time interface
   
       cmdbUrl="http://${i}:8088/webservice/pkgRep/listByip.do?ip=${eth0Ip}&returnType=text"
   
       debug_short_log "cmdbURL = $cmdbUrl" >> $logFile
   
       wget --read-timeout=10 --output-document=$tmpFile --tries=1 "$cmdbUrl" &>/dev/null
   
       rc=$?
   
       if [ $rc -eq 0 ] ; then
   
            debug_short_log "read from CMDB [$i] succeed , break"
   
            break
       else
            err_short_log "read from CMDB [$i] failed , try another ip"

       fi
   
   done

   # if all cmdb return failed , then exit

   if [ $rc -ne 0 ] ; then

        err_short_log "read from all cmdb faild with read timeout = 10 secs"

        date +%s > $lastRunTsFile

        return 1
   fi

   return

}


function show_cmdb_pinfo {

   debug_short_log "read proc info from cmdb ...."

   debug_short_log "$(cat -A $tmpFile)"

   if [ ! -s "$tmpFile" ] ; then

        warn_log "empty return from cmdb !!!"
   else
        debug_short_log "read finished Y ^_^ Y "
   fi

   return
}


function cmdb_pinfo_format {

   sed -i -r 's#,$##g' $tmpFile

   sort -u $tmpFile > $cmdbProcCacheFile

   return
}


function gen_local_pinfo_cache_file {

   debug_short_log "search all /data/services/*/admin/cache/proc.info file"

   cat /data/services/*/admin/cache/proc.info 2>/dev/null

   return
}

function get_proc_level {

   local verId=${1:?$FUNCNAME [ERR ] | verId is NULL}

   local plevel

   plevel=$(grep "version_id:${verId}," $cmdbProcCacheFile 2>/dev/null | egrep -o 'process_level:[0-9]+' | cut -d ':' -f 2)

   echo $plevel

   return
}


function gen_complete_local_pinfo_cache {

   gen_local_pinfo_cache_file | (

                                  while read line ; do

                                        plevel=$(get_proc_level $(echo "$line" | egrep -o 'version_id:[0-9]+' | cut -d ':' -f 2))  

                                        echo "$line,process_level:${plevel:=3}" 

                                  done | sort -t ',' -k10,10 > $localProcCacheFile

                                )

   return
}

function find_proc_proc_exe {

   local proc=${1:?$FUNCNAME [ERR ] | proc is NULL}

   local pList

   debug_short_log "try to find proc [$proc] through /proc/pid/exe ...."

   pList=$(ls /proc/[0-9]*/exe 2>/dev/null -l | sed 's/ (deleted)//' | awk -v _x="$binaryPath" '$NF==_x')

   [ -n "$pList" ] && echo "$pList" || return 1

}


function find_proc_pidof {
   
   local proc=${1:?$FUNCNAME [ERR ] | proc is NULL}

   local pList

   debug_short_log "try to find [$proc] through pidof (usually chroot) ...."

   pList=$(pidof $proc | xargs -n 1 | xargs -i ls -l /proc/{}/exe)

   [ -n "$pList" ] && echo "$pList" || return 1 

}


function find_proc_proc_status {
   
   local proc=${1:?$FUNCNAME [ERR ] | proc is NULL}
   
   local pList

   debug_short_log "try to find [$proc] through /proc/pid/status (usually change name) ...."

   #pList=$(egrep "^Name:[[:space:]]+${proc}$" /proc/[0-9]*/status 2>/dev/null | awk -F '/' '{print $3}' | xargs -n 1 -i ls -l /proc/{}/exe)

   pList=$(egrep "^Name:[[:space:]]+${proc}$" /proc/[0-9]*/status 2>/dev/null)

   debug_short_log "$(echo "$pList")"

   pList=$(echo "$pList" | awk -F '/' '{print $3}' | xargs -r -n 1 -i ls -l /proc/{}/exe)

   [ -n "$pList" ] && echo "$pList" || return 1

}


function binary_not_exist {

   if [ ! -e $1 ] ; then

        warn_short_log "can NOT find [$1] through /proc/pid/exe , pidof , /proc/pid/status"

        warn_short_log "maybe it's a garbage record , just ignore"

        debug_short_log "<<< finished >>>"
   
        debug_short_log ""

        return
   else
        return 1
   fi
}


function instance_stopped {

   if [ -e $1 ] && [ "$2" = "stop" ] ; then
   
        warn_short_log "binary file found , but instance status = stop"
      
        warn_short_log "maybe it's a garbage record , just ignore"
      
        debug_short_log "<<< finished >>>"

        debug_short_log ""
    
        return
   else
        return 1
   fi
}

function garbage_record {

   if [ -e $1 ] && [ -f $2 ] ; then

        warn_short_log "binary file found , .no-autostart flag file also found"

        debug_short_log "$(ls -l $2)"

        warn_short_log "maybe current pkg is uninstall-ing or stopp-ing or update-ing or rollback-ing or restart-ing, just ignore" 

        debug_short_log "<<< finished >>>"

        debug_short_log ""

        return
   else
        return 1
   fi

}



function not_run_alarm {

   local str

   #--------------------------------------------------#
   # set alarm msg according the different situations #
   #--------------------------------------------------#

   if [ $9 -ge 1 ] ; then   # not running actually happened

         if [ "${14}" = "1" ] ; then

               # also coredumped

               msg="${13}/${12} coredump : (${10} < $8 / autostart : $7)"
         else
               # just not running , no coredump happened

               msg="${13}/${12} num : (${10} < $8 / autostart : $7)"
         fi
   else
         if [ "${14}" = "1" ] ; then

              # coredump happened , but auto-start already

              msg="${13}/${12} coredump (autostart : $7)"
         fi
   fi

   str="id=${1}&sid=${2}&pre=${3}"
 
   str="${str}&alarm_level=${4}&alarm_type=${5}"

   str="${str}&verId=${6}&autostart=${7}"

   str="${str}&normal_count=${8}&less_count=${9}&cur_count=${10}"

   str="${str}&not_run_count='n/a'&procname=${12}&path=${13}&pname=${15}"

   str="${str}&msg='$msg'"

   echo "str"

   python /home/dspeak/yyms/yymp/yymp_report_script/yymp_report_alarm.py "$str" 2>&1

   return
}


function too_many_restart_alarm {

   local str

   str="id=${1}&sid=${2}&pre=${3}"

   str="${str}&alarm_level=${4}&alarm_type=${5}"

   str="${str}&verId=${6}&msg='${7}'"

   python /home/dspeak/yyms/yymp/yymp_report_script/yymp_report_alarm.py "$str" 2>&1

   return
}


function log_not_running {

   local now=$(date +%s)

   local file=${1:?$FUNCNAME [ERR ] | file is NULL}

   echo "$now $(date -d @$now +'%F %T') not_running proc=$2 level=$3 path=$4 coredump=$5 amount=$6 curNum=$7 less=$8 pkg=$9 ver=${10} pname=${11}" >> $file

   return
}


function log_unexp_restart {

   local now=$(date +%s)

   local file=${1:?$FUNCNAME [ERR ] | file is NULL}

   echo "$now $(date -d @$now +'%F %T') unexpect_restart_at $6 [$(date -d @$6 +'%F %T')] proc=$2 level=$3 path=$4 pid=$5" >> $file

   return
}

function log_call_monitor {

   local now=$(date +%s)

   local file=${1:?$FUNCNAME [ERR ] | file is NULL}

   echo "$now $(date -d @$now +'%F %T') run_monitor proc=$2 monitorScript=$3" >> $file

   return
}


function clear_old_alarm_record {

 find $alarmRecordDir -type f -name '????????' -mtime +30 -exec rm -f {} \;

  return
}


function update_today_link {

   if [ "$(date +'%H%M')" = "0000" ] ; then

        mkdir -p $alarmRecordDir ; rm -f $todayAlarmFile 2>/dev/null

        cd $alarmRecordDir && touch $alarmRecordFile && ln -s $alarmRecordFile today
   fi

   return
}

function find_s2s_pidFiles {

   local pidFiles

   pidFiles=$(find /data/services/*/admin/cache/s2s -type f -name '[0-9]*' 2>/dev/null) 

   echo "$pidFiles"

   return

}


function s2skill {

   local pidFiles="$@"

   local i appName pid logFile

   if [ -z "$pidFiles" ] ; then

        return
   else
	    for file in $pidFiles ; do

			pid=$(basename $file)

			appName=$(source ${file%%/cache/s2s/*}/common-var.conf && echo $APP_NAME)

            procName=$(basename $(ps --no-headers -o cmd $pid | awk '{print $1}') 2>/dev/null)

			: ${procName:='<proc-not-exist>'}

			if [ "$appName" != "$procName" ] ; then

				 debug_short_log "Invalid pid : [$pid] appName=[$appName] procName=[$procName] , not match"

                 debug_short_log "pidFile=$file"
                  
                 logFile="${file%/*}/cleanup.log"

				 ( cd ${file%%/admin/cache/s2s/*}/bin && \
					 
                   python $S2S_KILLER_SCRIPT $file >> $logFile )
            else
                 debug_short_log "Normal pid : [$pid] appName=[$appName] procName=[$procName] , match" 				

				 debug_short_log "pidFile=$file"
            fi				 
   
		done
   fi
}

function s2skill_for_stale {

  local stalePidFiles file logFile

  stalePidFiles=$(find /data/backup /data/bad_pkg -type f -path '/data/*/*/admin/cache/s2s/[0-9]*') 

  for file in $stalePidFiles ; do

	  logFile="${file%/*}/cleanup.log"

	  python $S2S_KILLER_SCRIPT $file >> $logFile

  done	  

}
