#!/bin/bash

function rotate_log {

   local logFile=${1:?$FUNCNAME [ERR ] | logFile is NULL}

   local maxSize=${2:?$FUNCNAME [ERR ] | maxSize is NULL}

   local logSize=$(stat --format='%s' $logFile 2>/dev/null)

   : ${logSize:=0}

   if [ $logSize -gt $(kmg_to_bytes $maxSize) ] ; then

        mv $logFile $logFile.bak &&  touch $logFile
   fi

   return
}


function err_log {

   local content

   if [ "${ENABLE_ERR_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | %30s | %30s | [ERR ] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}

function err_short_log {

   local content

   if [ "${ENABLE_ERR_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | [ERR ] | %s\n" "$(date +'%F %T')" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}



function note_log {

   local content

   if [ "${ENABLE_NOTE_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | %30s | %30s | [NOTE] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}


function note_short_log {

   local content

   if [ "${ENABLE_NOTE_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | [NOTE] | %s\n" "$(date +'%F %T')" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}


function info_log {

   local content

   if [ "${ENABLE_INFO_LOG:=yes}" = 'yes' ] ; then

        while read content  ; do

           printf "%s | %30s | %30s | [INFO] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content"

        done < <(echo "$*")
   fi

   return
}


function info_short_log {

   local content

   if [ "${ENABLE_INFO_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | [INFO] | %s\n" "$(date +'%F %T')" "$content"

        done < <(echo "$*")
   fi

   return
}

function warn_log {

   local content

   if [ "${ENABLE_WARN_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | %30s | %30s | [WARN] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}


function warn_short_log {

   local content

   if [ "${ENABLE_WARN_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | [WARN] | %s\n" "$(date +'%F %T')" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}



function debug_log {

   local content

   if [ "${ENABLE_DEBUG_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | %30s | %30s | [DEBU] | %s\n" "$(date +'%F %T')" "${FUNCNAME[1]}(${BASH_LINENO[1]})" "${FUNCNAME[2]:=N/A}(${BASH_LINENO[2]})" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}


function debug_short_log {

   local content

   if [ "${ENABLE_DEBUG_LOG:=yes}" = 'yes' ] ; then

        while read content; do

           printf "%s | [DEBU] | %s\n" "$(date +'%F %T')" "$content" 1>&2

        done < <(echo "$*")
   fi

   return
}


function elog {

   err_short_log "$@"

   return
}

function nlog {

   note_short_log "$@"

   return
}


function log {

   info_short_log "$@"

   return
}

function dlog {

   debug_short_log "$@"

   return
}

function wlog {

   warn_short_log "$@"

   return
}
