#!/bin/bash

function get_qdisc_sent_stats {

   tc -s qdisc | egrep '[0-9]+[[:space:]]+pkt.*dropped[[:space:]]+[0-9]+' -o | awk '{print $NF","$1}'

   return
}



function record_qdisc_sent_stats {

   local curMin=$(date +'%Y-%m-%d %H:%M:00')

   local curMinTs=$(date -d "$curMin" +%s)

   echo "$curMinTs $(get_qdisc_sent_stats)" > /tmp/qdisc.stats

   return
}


function calc_qdisc_sent_drop_ratio {

   local ratio

   local oldQdiscStats

   local curMinTs=$(date -d "$curMin" +%s)

   oldQdiscStats=$(tail -n 1 /tmp/qdisc.stats 2>/dev/null | awk '{print $2}')

   if [ -z "$oldQdiscStats" ] ; then

        # no init data

        #echo "no init data" >&2

        record_qdisc_sent_stats 

        #ls -l --full-time /tmp/qdisc.stats

        return
   else
        oldQdiscSent=$(echo "$oldQdiscStats" | awk -F ',' '{print $2}')

        oldQdiscSentDrop=$(echo "$oldQdiscStats" | awk -F ',' '{print $1}')

        if [ -z "$oldQdiscSent" ] || [ -z "$oldQdiscSentDrop" ] ; then

              # invalid data , quit

              #echo "empty data" >&2

              record_qdisc_sent_stats

              #ls -l --full-time /tmp/qdisc.stats

              return 1
        else
              curQdiscStats=$(get_qdisc_sent_stats)

              curQdiscSent=$(echo "$curQdiscStats" | awk -F ',' '{print $2}')

              curQdiscSentDrop=$(echo "$curQdiscStats" | awk -F ',' '{print $1}')

              deltaQdiscSent=$(( curQdiscSent - oldQdiscSent ))

              deltaQdiscSentDrop=$(( curQdiscSentDrop - oldQdiscSentDrop ))

              if [ $deltaQdiscSent -le 0 ] || [ $deltaQdiscSentDrop -lt 0 ] || [ $deltaQdiscSentDrop -ge $deltaQdiscSent ] ; then

                    # invalid delta , quit

                    #echo "invalid data" >&2

                    record_qdisc_sent_stats

                    #ls -l --full-time /tmp/qdisc.stats

                    return
              else
                    echo "$deltaQdiscSentDrop $deltaQdiscSent" | awk '{printf "%.2f\n",$1/$2*100}'

                    echo "$curMinTs $curQdiscStats" > /tmp/qdisc.stats

                    #ls -l --full-time /tmp/qdisc.stats
              fi
        fi
   fi


}




function netstat_stats {

   local statsResult

   statsResult=$(netstat -su) 

   echo "$statsResult"

   return

}

function netstat_udp_stats {

   local udpStats

   udpStats=$(netstat_stats | sed -r -n '/^Udp:/,/[^ ]+:$/p' | sed -n '$!p')

   echo "$udpStats"

   return
}


function get_cur_udp_recv_err {

   local x=$(netstat_udp_stats)

   local udpRecv udpRecvErr

   udpRecv=$(echo "$x" | grep 'packets received$' | awk '{print $1}')

   udpRecvErr=$(echo "$x" | grep 'RcvbufErrors' |  awk '{print $NF}') 

   : ${udpRecvErr:=0}

   echo "${udpRecvErr},${udpRecv}"

   return
}


function get_cur_udp_recv_err_simple {

   get_cur_udp_recv_err | awk -F ',' '{print $1}'

   return
}


function get_cur_udp_sent_err {

   local x=$(netstat_udp_stats)

   local udpSent udpSentErr

   udpSent=$(echo "$x" | grep 'packets sent$' | awk '{print $1}')

   udpSentErr=$(echo "$x" | grep 'SndbufErrors' |  awk '{print $NF}')

   : ${udpSentErr:=0}

   echo "${udpSentErr},${udpSent}"

   return
}


function get_cur_udp_sent_err_simple {

   get_cur_udp_sent_err | awk -F ',' '{print $1}'

   return

}

function record_udp_recv_err_stats {

   local curMin=$(date +'%Y-%m-%d %H:%M:00')

   local curMinTs=$(date -d "$curMin" +%s)

   echo "$curMinTs $(get_cur_udp_recv_err)" > /tmp/udp-recv.stats

   return
}



function record_udp_sent_err_stats {

   local curMin=$(date +'%Y-%m-%d %H:%M:00')

   local curMinTs=$(date -d "$curMin" +%s)

   echo "$curMinTs $(get_cur_udp_sent_err)" > /tmp/udp-sent.stats

   return
}



function calc_udp_recv_err_ratio {

   local ratio

   local oldUdpErr

   local curMinTs=$(date -d "$curMin" +%s)

   oldUdpErr=$(tail -n 1 /tmp/udp-recv.stats 2>/dev/null | awk '{print $2}')

   if [ -z "$oldUdpErr" ] ; then

        # no init data

        #echo "no init data" >&2

        record_udp_recv_err_stats

        #ls -l --full-time /tmp/udp-recv.stats

        return
   else
        oldUdpRecv=$(echo "$oldUdpErr" | awk -F ',' '{print $2}') 

        oldUdpRecvErr=$(echo "$oldUdpErr" | awk -F ',' '{print $1}') 

        if [ -z "$oldUdpRecv" ] || [ -z "$oldUdpRecvErr" ] ; then

              # invalid data , quit

              # echo "empty data" >&2

              record_udp_recv_err_stats

              #ls -l --full-time /tmp/udp-recv.stats

              return 1
        else
              curUdpRecvStats=$(get_cur_udp_recv_err)
 
              curUdpRecv=$(echo "$curUdpRecvStats" | awk -F ',' '{print $2}')

              curUdpRecvErr=$(echo "$curUdpRecvStats" | awk -F ',' '{print $1}')

              deltaRecv=$(( curUdpRecv - oldUdpRecv ))

              deltaRecvErr=$(( curUdpRecvErr - oldUdpRecvErr ))

              if [ $deltaRecv -le 0 ] || [ $deltaRecvErr -lt 0 ] || [ $deltaRecvErr -ge $deltaRecv ] ; then

                    # invalid delta , quit

                    #echo "invalid data : delta-recv=$deltaRecv , delta-recv-err=$deltaRecvErr" >&2

                    record_udp_recv_err_stats

                    #ls -l --full-time /tmp/udp-recv.stats

                    return
              else
                    #echo "$oldUdpRecvErr $curUdpRecvErr $deltaRecvErr $oldUdpRecv $curUdpRecv $deltaRecv" | awk '{printf "old-err:%d cur-err:%d delta-err:%d old-recv:%d cur-recv:%d delta-recv:%d ratio=%.2f\n",$1,$2,$3,$4,$5,$6,$3/$6*1000}'
                    echo "$deltaRecvErr $deltaRecv" | awk '{printf "%.2f\n",$1/$2*100}'

                    echo "$curMinTs $curUdpRecvStats" > /tmp/udp-recv.stats

                    #ls -l --full-time /tmp/udp-recv.stats
              fi
        fi
   fi

}



function calc_udp_sent_err_ratio {

   local ratio

   local oldUdpErr

   local curMinTs=$(date -d "$curMin" +%s)

   oldUdpErr=$(tail -n 1 /tmp/udp-sent.stats 2>/dev/null | awk '{print $2}')

   if [ -z "$oldUdpErr" ] ; then

        # no init data

        #echo "no init data" >&2

        record_udp_sent_err_stats

        #ls -l --full-time /tmp/udp-sent.stats

        return
   else
        oldUdpSent=$(echo "$oldUdpErr" | awk -F ',' '{print $2}') 

        oldUdpSentErr=$(echo "$oldUdpErr" | awk -F ',' '{print $1}') 

        if [ -z "$oldUdpSent" ] || [ -z "$oldUdpSentErr" ] ; then

              # invalid data , quit

              #echo "empty data" >&2

              record_udp_sent_err_stats

              #ls -l --full-time /tmp/udp-sent.stats

              return 1
        else
              curUdpSentStats=$(get_cur_udp_sent_err)
 
              curUdpSent=$(echo "$curUdpSentStats" | awk -F ',' '{print $2}')

              curUdpSentErr=$(echo "$curUdpSentStats" | awk -F ',' '{print $1}')

              deltaSent=$(( curUdpSent - oldUdpSent ))

              deltaSentErr=$(( curUdpSentErr - oldUdpSentErr ))

              if [ $deltaSent -le 0 ] || [ $deltaSentErr -lt 0 ] || [ $deltaSentErr -ge $deltaSent ] ; then

                    # invalid delta , quit

                    record_udp_sent_err_stats

                    # ls -l --full-time /tmp/udp-sent.stats

                    return
              else
                    echo "$deltaSentErr $deltaSent" | awk '{printf "%.2f\n", $1/$2*100}' 

                    echo "$curMinTs $curUdpSentStats" > /tmp/udp-sent.stats

                    # ls -l --full-time /tmp/udp-sent.stats
              fi
        fi
   fi

}
