#!/bin/bash
#
# 对于数据盘默认只是分一个区。系统盘分3个区，分别是根分区(30GB)、交换分区(4GB)、数据分区(剩下)
# 格式化硬盘
export PATH="/sbin:/usr/sbin:/bin:/usr/bin"
export LANG=C
export LC_ALL=C

export OK_DISK

tmp_file1=`mktemp`
tmp_file2=`mktemp`

[ "$DEBUG_PXE" = "YES" ] && set -x
FORMATI_PRINT_HELP() {
OUTPUT_RED "运行会导致丢失所有分区和数据，小心使用"
OUTPUT_GREEN "$0 [SYSTEM_DISK] --device [/dev/sda、/dev/sdb...]"
OUTPUT_GREEN "SYSTEM_DISK:将硬盘分区成 /(30G)、swap(4G)、/data(rest_of)"
rm -f $tmp_file1 $tmp_file2
exit 1
}

OUTPUT_GREEN() {
	echo -e "\033[32;40;1m$1\033[0m"
}

OUTPUT_RED(){
	echo -e "\033[31;40;1mError_msg:$1\033[0m"
}


function comm_formation () {
	NEED_FORMAT_DISK=$1
	grep -q "$NEED_FORMAT_DISK" /etc/mtab
	[ $? -eq 0 ] && OUTPUT_RED "$NEED_FORMAT_DISK处于挂载状态" && return 1
	if [ -b $NEED_FORMAT_DISK ]; then
	
		# try to umount disks
		DISK_BASENAME="${NEED_FORMAT_DISK##*/}"
		DISK_UMOUNT_NUM=$(grep "$DISK_BASENAME" /proc/partitions | awk '{print $NF}')
		for DISK_UMOUNT in $DISK_UMOUNT_NUM; do
			umount -f /dev/$DISK_UMOUNT > /dev/null 2>&1
		done
		
		DISK_VOLUME_CAP=$(parted -s $NEED_FORMAT_DISK print | awk '{if($0 ~ /^Disk/){print $NF;exit}}')
		if [ -z $DISK_VOLUME_CAP ];then
			parted -s $NEED_FORMAT_DISK mklabel gpt
			DISK_VOLUME_CAP=$(parted -s $NEED_FORMAT_DISK print | awk '{if($0 ~ /^Disk/){print $NF;exit}}')
		fi
		DISK_CAP=$(echo $DISK_VOLUME_CAP | tr -d '0-9.')
		DISK_VOLUME=$(echo $DISK_VOLUME_CAP | awk '{gsub("TB|GB|MB","",$0);print int($0)}')
		if [ -z "$DISK_VOLUME_CAP" ]; then
			OUTPUT_RED "$NEED_FORMAT_DISK can't not found"
			exit 1
		fi
		
		if [ "$DISK_CAP" = "GB" -a $DISK_VOLUME -ge 1000 ]; then
			PARTITION_TABLE="gpt"
		elif [ "$DISK_CAP" = "TB" ];then
			PARTITION_TABLE="gpt"
		else
			PARTITION_TABLE="msdos"
		fi
		
		PARTITION_NUM=$(parted -s $NEED_FORMAT_DISK print | sed -n "/^Number/,//p"|grep -E 'primary|extended|logical'|awk '{print $1}')
		
		for WHICH_PARTITION in $PARTITION_NUM; do
			parted -s $NEED_FORMAT_DISK rm $WHICH_PARTITION > /dev/null 2>&1
		done
		
		parted -s $NEED_FORMAT_DISK mklabel $PARTITION_TABLE
		parted -s $NEED_FORMAT_DISK mkpart primary ext4 0% 100%
		
		FORMAT_PARTITION_NUM=$(parted -s $NEED_FORMAT_DISK print | sed -n "/^Number/,//p"|grep -E 'primary|logical'|awk '{print $1}')
		
		for FORMAT_PARTITION in $FORMAT_PARTITION_NUM; do
			OUTPUT_GREEN "Formating ${NEED_FORMAT_DISK}${FORMAT_PARTITION} , wait......"
			mkfs.ext4 ${NEED_FORMAT_DISK}${FORMAT_PARTITION}
			OK_DISK="${OK_DISK} ${NEED_FORMAT_DISK}"
		done

	else
		OUTPUT_RED "$NEED_FORMAT_DISK不存在"
	fi
}

function system_formation () {
	NEED_FORMAT_DISK=$1
	if [ -b $NEED_FORMAT_DISK ]; then
	
		# try to umount disks
		DISK_BASENAME="${NEED_FORMAT_DISK##*/}"
		DISK_UMOUNT_NUM=$(grep "$DISK_BASENAME" /proc/partitions | awk '{print $NF}')
		for DISK_UMOUNT in $DISK_UMOUNT_NUM; do
			umount -f /dev/$DISK_UMOUNT > /dev/null 2>&1
		done
		
		DISK_VOLUME_CAP=$(parted -s $NEED_FORMAT_DISK print | awk '{if($0 ~ /^Disk/){print $NF;exit}}')
		if [ -z $DISK_VOLUME_CAP ];then
			parted -s $NEED_FORMAT_DISK mklabel gpt
			DISK_VOLUME_CAP=$(parted -s $NEED_FORMAT_DISK print | awk '{if($0 ~ /^Disk/){print $NF;exit}}')
		fi
		DISK_CAP="GB"
		DISK_VOLUME=`echo $DISK_VOLUME_CAP | awk '{
				if($0 ~ /MB/){
					gsub("MB","",$0);disk_volume=$0/1000;
				}else if($0 ~ /GB/){
					gsub("GB","",$0);disk_volume=$0;
				}else if($0 ~ /TB/){
					gsub("TB","",$0);disk_volume=$0*1000;
				};
		}
		END{
			print int(disk_volume);
		}'`
		
		if [ -z "$DISK_VOLUME_CAP" ]; then
			OUTPUT_RED "$NEED_FORMAT_DISK can't not found"
			exit 1
		fi
		
		[ $DISK_VOLUME -ge 1000 ] && PARTITION_TABLE="gpt" || PARTITION_TABLE="msdos"
		
		if [ $DISK_VOLUME -le 50 ]; then
			OUTPUT_YELLOW "$NEED_FORMAT_DISK volume less than 50GB"
			ROOT_PAR_SIZE=`echo $DISK_VOLUME|awk '{tmp=$1*0.3;if(tmp < 4){print "4"}else{print tmp}}'`
			SWAP_PAR_SIZE=`echo $DISK_VOLUME|awk '{tmp=$1*0.05;if(tmp >= 4){print "4"}else if(tmp < 0.5){print "0.5"}else{print tmp}}'`
			ROOT_SWAP_SIZE=`echo $ROOT_PAR_SIZE $SWAP_PAR_SIZE|awk '{print $1+$2}'`
		else
			ROOT_PAR_SIZE=30
			SWAP_PAR_SIZE=4
			ROOT_SWAP_SIZE=34
		fi
		[ $DISK_VOLUME -le 5 ] && OUTPUT_RED "$NEED_FORMAT_DISK volume less than 5GB, exit" && exit 1
		
		PARTITION_NUM=$(parted -s $NEED_FORMAT_DISK print | sed -n "/^Number/,//p"|grep -E 'primary|extended|logical'|awk '{print $1}')
		
		for WHICH_PARTITION in $PARTITION_NUM; do
			parted -s $NEED_FORMAT_DISK rm $WHICH_PARTITION > /dev/null 2>&1
			wait
		done
		sleep 1
		
		parted -s $NEED_FORMAT_DISK mklabel $PARTITION_TABLE
		if [ "$PARTITION_TABLE" = "msdos" ];then
			parted -s $NEED_FORMAT_DISK mkpart primary ext4 0% ${ROOT_PAR_SIZE}GB
			parted -s $NEED_FORMAT_DISK set 1 boot on
			parted -s $NEED_FORMAT_DISK mkpart primary linux-swap ${ROOT_PAR_SIZE}GB ${ROOT_SWAP_SIZE}GB
			mkswap ${NEED_FORMAT_DISK}2
			parted -s $NEED_FORMAT_DISK mkpart primary ext4 ${ROOT_SWAP_SIZE}GB 100%
			
		else
			# gpt 分区表需要预留开头2MB
			parted -s $NEED_FORMAT_DISK mkpart primary 0% 2MB
			parted -s $NEED_FORMAT_DISK set 1 bios_grub on
			parted -s $NEED_FORMAT_DISK mkpart primary ext4 2MB ${ROOT_PAR_SIZE}GB
			parted -s $NEED_FORMAT_DISK set 2 boot on
			parted -s $NEED_FORMAT_DISK mkpart primary linux-swap ${ROOT_PAR_SIZE}GB ${ROOT_SWAP_SIZE}GB
			mkswap ${NEED_FORMAT_DISK}3
			parted -s $NEED_FORMAT_DISK mkpart primary ext4 ${ROOT_SWAP_SIZE}GB 100%
		fi
		
		FORMAT_PARTITION_NUM=$(parted -s $NEED_FORMAT_DISK print | sed -n "/^Number/,//p"|grep -E 'primary|logical'|grep -vE 'bios_grub|swap'|awk '{print $1}')
			
		for FORMAT_PARTITION in $FORMAT_PARTITION_NUM; do
			OUTPUT_GREEN "Formating ${NEED_FORMAT_DISK}${FORMAT_PARTITION} , wait......"
			mkfs.ext4 ${NEED_FORMAT_DISK}${FORMAT_PARTITION}
			wait
			sleep 0.5
		done

	else
		OUTPUT_RED "$NEED_FORMAT_DISK does not exist"
	fi
}

until [ -z "$1" ]; do 
	case $1 in
		"--device")
			export FORMAT_DISK=yes
			shift;;
		/dev/*)
			if [ "$FORMAT_DISK" = "yes" ]; then
				comm_formation $1
				wait
				sleep 0.5
			else
				FORMATI_PRINT_HELP
			fi
			shift;;
		*)
			FORMATI_PRINT_HELP;;
	esac
done

[ "$FORMAT_DISK" != "yes" ] && FORMATI_PRINT_HELP

_OK_DISK=$(echo $OK_DISK | tr -d ' ')
if [ ! -z "$_OK_DISK" ];then
	OUTPUT_GREEN "成功分区格式化${OK_DISK}"
	blkid -o full > $tmp_file1
	for need_disk in $OK_DISK;
	do
		WILL_MOUNTPOINT="/data${i}"
		for((i=0;;i++));do
			if [ $i -eq 0 ]; then
				WILL_MOUNTPOINT="/data"
			else
				WILL_MOUNTPOINT="/data${i}"
			fi
			if ! mountpoint -q $WILL_MOUNTPOINT; then
				if ! grep -qw "$WILL_MOUNTPOINT" $tmp_file2; then
					break
				fi
			fi
		done
		[ ! -d $WILL_MOUNTPOINT ] && mkdir $WILL_MOUNTPOINT
		DISK_UUID=`grep "$need_disk" $tmp_file1 | awk '{for(i=1;i<=NF;i++){if($i ~ /UUID/){print $i}}}'`
		if [ ! -z "$DISK_UUID" ] && [ ! -z $WILL_MOUNTPOINT ]; then
			echo -e "$DISK_UUID\t$WILL_MOUNTPOINT\text4\tdefaults\t0\t0" >> $tmp_file2
			echo -e "$DISK_UUID\t$WILL_MOUNTPOINT\text4\tdefaults\t0\t0"
		fi
	done
fi

rm -f $tmp_file1 $tmp_file2