#!/bin/bash
# 通过iptables隔离网络流量
# Merge: 查看目前服务器上进程和端口信息列表

# return code:
# 51  错误
# 52  存在业务进程
# 53  存在业务监听端口
# 54  存在业务进程和监听端口

source /usr/local/i386/*/auto/function.sh
return_code=51

if [ -s para.config ];then
    source /usr/local/i386/public_repos/initial_system/snmp/ext/tools_system/commonFunction.sh
    op=$(awk -F '=' '{if($1 ~ /apply_op/){print $2}}' para.config)
    apply_user=$(awk -F '=' '{if($1 ~ /cur_user/){print $2}}' para.config)
else
    op=$1
fi

function help_func() {
    cat << EOF

$(basename $0) [isolate|release|query|query_proc|query_proc_ext <file>]
isolate:        隔离
release:        解除隔离
query:          查询隔离状态
query_proc:     查询进程端口信息
query_proc_ext: 查询进程端口信息排除file里面额外进程
EOF
    exit ${return_code}
}

case "$op" in
    isolate|1)
        apply_op="isolate";;
    release|2)
        apply_op="release";;
    query|3)
        apply_op="query_action";;
    query_proc|4)
        apply_op="query_proc";;
    query_proc_ext|5)
        apply_op="query_proc_ext"
        [ -f "$2" ] && ext_file=$2 || help_func
        ;;
    *)
        help_func
esac

BASENAME=$(basename $0)
IPTABLES_DIR="/usr/local/virus/iptables"
isolate_lock="$IPTABLES_DIR/.iptables_isolate_backup"
SYSOP_DIR="/var/log/sysop_manager"
IPADDR=$(get_one_ip)
exclude_perfix="/usr/local/i386/public_repos/initial_system/exclude_info/exclude_isolate"
run_time=$(date "+%Y-%m-%d %H:%M:%S")

# ACCEPT: ssh, yyms, tools
# DROP: agent_channel(包发布) and others

iptables_rule () {
        cat << 'EOF'
iptables -N ISOLATE_INPUT
iptables -I INPUT 2 -p all -g ISOLATE_INPUT
iptables -N ISOLATE_OUTPUT
iptables -I OUTPUT 1 -p all -g ISOLATE_OUTPUT

iptables -A ISOLATE_INPUT -p icmp -j ACCEPT
iptables -A ISOLATE_INPUT -p tcp -m multiport \
    --destination-port 32200,10029 -j ACCEPT
iptables -A ISOLATE_INPUT -p tcp -m multiport \
    --source-port 63217,63579,40000,40100,40200,40300,6389:6390,51300,44944,10025 -j ACCEPT
iptables -A ISOLATE_INPUT -p udp -m multiport \
    --source-port 123 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.167.101      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 211.103.26.244     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 221.228.209.180    -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 122.193.202.52     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.164.216      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 122.97.250.176     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 120.195.152.114    -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 221.228.209.48     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.164.215      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 122.97.250.175     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 120.195.152.113    -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 221.228.209.47     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.21.32.124       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 14.17.108.136      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.21.32.123       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 14.17.108.135      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.21.32.122       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 14.17.108.134      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.81.105       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 183.57.81.219      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 122.13.201.155     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.81.108       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 183.57.81.222      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 122.13.201.158     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.81.47        -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 113.107.237.47     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 122.13.200.239     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.1.82         -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 140.207.208.45     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 222.73.61.45       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.3.33         -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 101.226.185.57     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 140.207.209.57     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.96.143       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 113.106.100.79     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.52.10        -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 115.238.170.51     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.96.145       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 113.106.100.74     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.164.77       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 58.215.143.77      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 58.241.28.77       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 120.195.152.194    -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.80.210       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 122.13.200.148     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 183.61.143.180     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.96.116       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 183.61.2.157       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.96.63        -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 113.106.100.66     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.0.16         -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 101.226.185.72     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 140.207.209.72     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 220.181.86.195     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 118.26.226.195     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 112.81.159.51      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 58.215.46.51       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 119.147.160.121    -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 58.249.113.57      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 121.9.221.159      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 113.107.239.132    -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 218.60.71.24       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 218.60.71.12       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 121.14.39.139      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 58.249.117.107     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 119.188.71.161     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 119.188.71.154     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 124.95.146.50      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 121.14.37.77       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 58.249.114.109     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 113.108.228.226    -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 58.248.181.98      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 61.55.172.21       -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 121.14.241.40      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 222.186.49.44      -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 119.97.153.188     -p tcp -m multiport --source-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_INPUT -s 113.106.100.63     -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_INPUT -s 121.14.241.39      -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_INPUT -s 61.55.172.21       -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_INPUT -s 124.95.146.50      -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_INPUT -s 10.20.81.108       -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_INPUT -p all -j DROP

iptables -A ISOLATE_OUTPUT -o lo -j ACCEPT
iptables -A ISOLATE_OUTPUT -p tcp -m multiport \
    --source-port 32200,10029 -j ACCEPT
iptables -A ISOLATE_OUTPUT -p tcp -m multiport \
    --destination-port 63217,63579,40000,40100,40200,40300,6389:6390,51300,44944,10025 -j ACCEPT
iptables -A ISOLATE_OUTPUT -p udp -m multiport \
    --destination-port 123 -j ACCEPT
iptables -A ISOLATE_OUTPUT -p icmp -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.167.101     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 211.103.26.244    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 221.228.209.180   -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 122.193.202.52    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.164.216     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 122.97.250.176    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 120.195.152.114   -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 221.228.209.48    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.164.215     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 122.97.250.175    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 120.195.152.113   -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 221.228.209.47    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.21.32.124      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 14.17.108.136     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.21.32.123      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 14.17.108.135     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.21.32.122      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 14.17.108.134     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.81.105      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 183.57.81.219     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 122.13.201.155    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.81.108      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 183.57.81.222     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 122.13.201.158    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.81.47       -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 113.107.237.47    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 122.13.200.239    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.1.82        -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 140.207.208.45    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 222.73.61.45      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.3.33        -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 101.226.185.57    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 140.207.209.57    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.96.143      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 113.106.100.79    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.52.10       -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 115.238.170.51    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.96.145      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 113.106.100.74    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.164.77      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 58.215.143.77     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 58.241.28.77      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 120.195.152.194   -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.80.210      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 122.13.200.148    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 183.61.143.180    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.96.116      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 183.61.2.157      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.96.63       -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 113.106.100.66    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 10.20.0.16        -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 101.226.185.72    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 140.207.209.72    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 220.181.86.195    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 118.26.226.195    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 112.81.159.51     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 58.215.46.51      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 119.147.160.121   -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 58.249.113.57     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 121.9.221.159     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 113.107.239.132   -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 218.60.71.24      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 218.60.71.12      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 121.14.39.139     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 58.249.117.107    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 119.188.71.161    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 119.188.71.154    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 124.95.146.50     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 121.14.37.77      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 58.249.114.109    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 113.108.228.226   -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 58.248.181.98     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 61.55.172.21      -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 121.14.241.40     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 222.186.49.44     -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -d 119.97.153.188    -p tcp -m multiport --destination-port 9090,6379 -j ACCEPT
iptables -A ISOLATE_OUTPUT -s 113.106.100.63    -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_OUTPUT -s 121.14.241.39     -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_OUTPUT -s 61.55.172.21      -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_OUTPUT -s 124.95.146.50     -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_OUTPUT -s 10.20.81.108      -p tcp -m multiport --source-port 10025 -j ACCEPT
iptables -A ISOLATE_OUTPUT -p all -j DROP
EOF

}

function iptables_isolate () {
    [ -s $IPTABLES_DIR/iptables.add ] && \
    [ ! -s $isolate_lock ] && \
    mv $IPTABLES_DIR/iptables.add $isolate_lock

    [ ! -f $isolate_lock ] && \
    touch $isolate_lock

    iptables_rule > $IPTABLES_DIR/iptables.add
    echo "#start_isolate=${run_time}" >> $IPTABLES_DIR/iptables.add

    bash $IPTABLES_DIR/iptables.sh
}

function iptables_relieve_isolate () {
    if [ -f $isolate_lock ];then
        cat $isolate_lock > $IPTABLES_DIR/iptables.add
        rm -f $isolate_lock
        bash $IPTABLES_DIR/iptables.sh
    fi
}

function user_lock () {
    string="AllowUsers yuwanfu wuhaiting linminggui"
    if ! grep -q "${string}" /etc/ssh/sshd_config;then
        echo "${string}" >> /etc/ssh/sshd_config
        if [ "$LINUX_VERSION" = "ubuntu" ];then
            /etc/init.d/ssh restart > /dev/null 2>&1
        elif [ "$LINUX_VERSION" = "centos" ];then
            /etc/init.d/sshd restart > /dev/null 2>&1
        fi
    fi
}

function user_unlock () {
    string="AllowUsers yuwanfu wuhaiting linminggui"
    if grep -q "${string}" /etc/ssh/sshd_config;then
        sed -i "/${string}/d" /etc/ssh/sshd_config
        if [ "$LINUX_VERSION" = "ubuntu" ];then
            /etc/init.d/ssh restart > /dev/null 2>&1
        elif [ "$LINUX_VERSION" = "centos" ];then
            /etc/init.d/sshd restart > /dev/null 2>&1
        fi
    fi
}

function query_proc_port() {
    local r=0

    exclude_process=$(cat ${exclude_perfix}_proc | tr -s '\n' '|' | \
        sed -e "s/^|*//" -e "s/|*$//")
    exclude_port=$(cat ${exclude_perfix}_listenport |tr -s '\n' '|' | \
        sed -e "s/^|*//" -e "s/|*$//")

    exist_proc=$(ps -eo args | grep -ivE "${exclude_process}"|tr -d '%')
    if [ "$apply_op" == "query_proc_ext" ] && [ -s "$ext_file" ];then
        ext_process=$(cat $ext_file|tr -s '\n' '|'|sed -e "s/^|*//" -e "s/|*$//")
        exist_proc=$(printf "$exist_proc"|grep -viE "$ext_process")
    fi
    exist_proc=$(printf "$exist_proc" | \
        grep -vwE "sort|COMMAND|ps|su|tr|tail|$BASENAME" | \
        sort -u|tr -s '\n' '; ')

    if [ "x${exist_proc}" != "x" ];then
        echo -e "Result\t$IPADDR\t存在服务进程; ${exist_proc}"
        proc_r=52
    fi

    exist_conn=$(wc -l /proc/net/{tcp,udp}| grep total| cut -d ' ' -f 1)
    if [ $exist_conn -le 1000 ];then
        exist_port=$(netstat -lntup|awk '{printf("%s\t%s\n",$4,$NF)}' | \
            awk -F ':' '{print $NF}' | tr -s '/' ' ' | \
            awk '{printf("%s %s\n",$1,$NF)}' | \
            grep -vwE "$exclude_port"|tr -d '%')
        if [ "$apply_op" == "query_proc_ext" ] && \
            [ -s "$ext_file" ] && [ "x${ext_process}" != "x" ];then
            exist_port=$(printf "$exist_port" | grep -viE "$ext_process")
        fi
        exist_port=$(printf "$exist_port"|grep -E "[0-9]" | \
            awk '{print $NF}'|sort -u| tr -s '\n' '; ')

        if [ "x${exist_port}" != "x" ];then
            echo -e "Result\t$IPADDR\t存在端口监听进程; ${exist_port}"
            port_r=53
        fi
    else
        echo -e "Result\t$IPADDR\t存在大量网络连接:${exist_conn}"
        port_r=53
    fi

    [ "x${proc_r}" == "x" ] && [ "x${port_r}" == "x" ] && \
        echo -e "Result\t$IPADDR\t没有业务进程和监听端口"

    if [ "x${proc_r}" != "x" ] && [ "x${port_r}" != "x" ];then
        r=54
    elif [ "x${proc_r}" != "x" ];then
        r=$proc_r
    elif [ "x${port_r}" != "x" ];then
        r=$port_r
    fi
    return $r
}

return_code=0
if [ "$apply_op" = "isolate" ];then
    if [ ! -f $isolate_lock ];then
        iptables_isolate
        user_lock
    fi
    echo -e "Result\t$IPADDR\t隔离成功"
elif [ "$apply_op" = "release" ];then
    iptables_relieve_isolate
    user_unlock
    echo -e "Result\t$IPADDR\t解除隔离"
elif [ "$apply_op" = "query_action" ];then
    if [ -f $isolate_lock ];then
        start_time=$(grep -m1 'start_isolate' $IPTABLES_DIR/iptables.add|awk -F '=' '{print $NF}')
        echo -e "Result\t$IPADDR\t从${start_time}已经开始隔离"
    else
        echo -e "Result\t$IPADDR\t服务器没有进行隔离"
    fi
elif [ "$apply_op" = "query_proc" ] || [ "$apply_op" = "query_proc_ext" ];then
    query_proc_port
    return_code=$?
fi

if [ "x${apply_user}" = "x" ];then
    loguser="$run_time [network_isolate] "$(env | grep 'SUDO_USER'|awk -F '=' '{print $NF}')" as "$(whoami)
else
    loguser="$run_time [network_isolate] $apply_user tools_system"
fi
echo "$loguser $apply_op" >> $SYSOP_DIR/sysop_manager.log

[ "${apply_user}" != "" ] && printResult "0" "0"
exit $return_code

