#!/bin/bash

export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
export LANG=C
export LC_ALL=C
export DEBIAN_FRONTEND=noninteractive

[ "$(whoami)" != 'root' ] && echo 'Permission denied' && exit 1

tmpfile=$(mktemp /tmp/updatessl.XXXX)

ARCH_BIT=$(getconf LONG_BIT)
[ $ARCH_BIT -eq 64 ] && ARCH="amd64" || { ARCH="i386";echo "Not support arch=32";exit 1; }

grep -iqE 'Ubuntu|Debian' /etc/issue && LINUX_VERSION="ubuntu" && \
RELEASE=$(sed -n 's/.* \([0-9.]*\) .*/\1/p' /etc/issue) && \
CODENAME=$(awk -F '=' '{if($1=="DISTRIB_CODENAME"){print $2}}' /etc/lsb-release)
grep -iqE 'CentOS|red' /etc/issue && LINUX_VERSION="centos"

trap "rm -f $tmpfile;exit 0;" 0 1 2 9 15

OUTPUT_GREEN() {
    echo -e "\033[32;40;1m$1\033[0m"
}

OUTPUT_YELLOW(){
    echo -e "\033[33;40;1mWarning:$1\033[0m"
}

OUTPUT_RED(){
    echo -e "\033[31;40;1mError_msg:$1\033[0m"
}

if [ "$LINUX_VERSION" = "centos" ];then
    OUTPUT_YELLOW "Sorry，目前不支持centos系统openssl升级"
    exit 1
elif [ "$LINUX_VERSION" = "ubuntu" ];then
    dpkg -l > $tmpfile
    for ep in libssl-dev;
    do
        grep -w "$ep" $tmpfile|grep -q "^ii" && ext_packages="${ext_packages} $ep"
    done
    case $RELEASE in
        9.04*|11.04*)
            fix_version098="0.9.8za-1ubuntu1"
            fix_version=$fix_version098
            download_version=$fix_version098
            fix_packages="openssl libssl0.9.8"
            ;;
        10.04*)
            fix_version098="0.9.8k-7ubuntu8.27"
            fix_version=$fix_version098
            download_version=$fix_version098
            fix_packages="openssl libssl0.9.8"
            ;;
        12.04*)
            fix_version098="0.9.8za-1ubuntu1"
            fix_version100="1.0.1-4ubuntu5.15"
            fix_version=$fix_version100
            download_version="1.0.1-4ubuntu5.25"

            grep -w 'ia32-libs' $tmpfile |grep -q "^ii" && \
            fix_packages="openssl libssl0.9.8:i386 libssl0.9.8 libssl1.0.0:i386 libssl1.0.0" || \
            fix_packages="openssl libssl0.9.8 libssl1.0.0"
            ;;
        *)
            OUTPUT_YELLOW "非ubuntu 系统"
            exit 1
            ;;
    esac
fi

function ubuntu_update_libssl () {
    cur_version=$(awk '{if($2 == "openssl"){print $3;exit;}}' $tmpfile)

    dpkg --compare-versions "$cur_version" ge "${fix_version}" && \
    update_need="no" || update_need="yes"

    if [ "x${update_need}" == 'xno' ] && [ "x${cur_version}" != "x" ];then
        OUTPUT_GREEN "openssl已是最新"
        exit 0
    fi

    for get_package in $fix_packages $ext_packages;
    do
        [ "${get_package##*:}" == 'i386' ] && download_arch="i386" ||download_arch=$ARCH
        [ "${get_package%%:*}" == "libssl0.9.8" ] && get_version=$fix_version098 || get_version=$download_version
        gp="${get_package%%:*}_${get_version}_${download_arch}.deb"
        [ -f "$gp" ] && rm -f $gp
        printf "download %-40s ...  " $gp
        URL="http://mirror.yy.duowan.com:63782/ubuntu/dists/${CODENAME}-prisoft/main/packages/binary-${download_arch}/$gp"
        wget -q --timeout=4 --tries=3 "$URL" || \
        { echo "Download packages fail[$gp], try again later";exit 1; }
        echo '[ok]'
        install_packages="${install_packages} $gp"
    done

    echo "dpkg install ${fix_packages} ${ext_packages} ...."
    dpkg -i --force-all $install_packages >/dev/null
    rm -f $install_packages
    cur_version=$(dpkg -l | awk '{if($2 == "openssl"){print $3;exit;}}')
    dpkg --compare-versions "$cur_version" ge "${fix_version}" && \
    update_ok='yes' || update_ok='fail'

}

if [ "$LINUX_VERSION" = "ubuntu" ];then
    ubuntu_update_libssl
fi

if [ "x${update_need}" == 'xyes' ] && [ "x${update_ok}" == 'xyes' ];then
    OUTPUT_GREEN "更新ssl成功,如果有业务进程涉及相关库文件,请重启进程"
else
    OUTPUT_RED "更新失败,请检查"
fi
