#!/bin/bassh
# 监控OpenStack各个组件存活

# 可以配置以下各个组件是否打开监控，或者使用外部配置文件/root/mon_openstack.conf
# 根据hostname决定打开何种类型
# NOVA package
ENABLE_all_nova_warn=yes
WARN_nova_cert=yes
WARN_nova_consoleauth=yes
WARN_nova_novncproxy=yes
nova_novncproxy=6080
WARN_nova_scheduler=yes
WARN_nova_conductor=yes
WARN_nova_api=yes
nova_api_port1=8773
nova_api_port2=8774
nova_api_port3=8775

# Glance package
ENABLE_all_glance_warn=yes
WARN_glance_api=yes
glance_api=9292
WARN_glance_registry=yes
glance_registry_port=9191

# Cinder package
ENABLE_all_cinder_warn=yes
WARN_cinder_api=yes
cinder_api_port=8776
WARN_cinder_volume=yes
WARN_cinder_scheduler=yes

# Neutron package
ENABLE_all_neutron_warn=yes
WARN_neutron_metadata_agent=yes
WARN_neutron_l3_agent=yes
WARN_neutron_server=yes
neutron_server_api=9696
WARN_neutron_openvswitch_agent=yes
WARN_neutron_dhcp_agent=yes
WARN_dnsmasq=yes
WARN_ovsdb_server=yes
WARN_ovs_vswitchd=yes

# MISC package
ENABLE_all_misc_warn=yes
WARN_rabbitmq_server=yes
rabbitmq_server=5672
WARN_named=yes
named_port=53
WARN_memcached=yes
memcached_port=11211
WARN_ceilometer=no
WARN_haproxy=yes
WARN_keepalive=yes


export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
BASENAME=$(basename $0)
BASEDIR=/dev/sysop
LOGFILE=/var/log/${BASENAME%%.*}.log
PLOG=${BASEDIR}/${BASENAME}_ps.log
KEYFILE=/root/keystone_adm
GBLMSG=""


function noblock_lock () {
	local lockfile
	lockfile="${1:-${BASEDIR}/${BASENAME}.lock}"
	exec 200<>$lockfile
	flock -n -e 200 || { echo "$0 Already Running, exit!";exit 1; }
}

function noblock_unlock () {
	local lockfile
	lockfile="${1:-${BASEDIR}/${BASENAME}.lock}"
	exec 200<>$lockfile
	flock -u 200
	rm -f $lockfile
}

function addwarn_alarm () {
	GBLMSG="$GBLMSG $1"
}

function send_alarm () {
	local warn_id="12605"
	local warn_msg="health_warn"

    if [ "x${GBLMSG}" != "x" ];then
		msg=$(python /home/dspeak/yyms/yymp/yymp_report_script/yymp_report_alarm.py "$warn_id" "" "${warn_msg}")
        echo $GBLMSG
        echo "$(date "+%Y-%m-%d %H:%M:%S") $GBLMSG" >> $LOGFILE
        echo "$(date "+%Y-%m-%d %H:%M:%S") $msg" >> $LOGFILE
        return 1
    else
        echo 'Health_ok'
        return 0
    fi
}

function load_process () {
	/bin/ps -eo pcpu,pid,command > $PLOG
}

function dump_process () {
	cat $PLOG && return 0 || return 1
}

function process_alive_num () {
local pname=$(echo $1|tr -s '_' '-')
	local pnum=$2
	local nport=$3
	local pstat=""

	cpum=$(grep -wE "$pname" $PLOG | grep -vcwE 'grep|ssh|bash|tail|head')
	[ $cpum -ge $pnum ] && pstat='ok' || pstat='fail'

	if [ "x${nport}" != "x" ];then
	    /bin/nc -z 127.0.0.1 $nport || pstat='fail'
    fi

	[ "$pstat" == "ok" ] && return 0 || return 1
}

function service_alive () {
	# service: nova, neutron, cinder, glance ...
	# processor num default 1
	local service=$1
	eval "mps=\${!WARN_${service}*}"
	
	for mp in $mps;do
		mp_prefix="${mp#WARN_}"
		eval "mp_en=\$$mp"
		eval "mp_ports=\${!${mp_prefix}*}"
		[ "x${mp_en}" != "xyes" ] && continue

		if [ "x${mp_ports}" != "x" ];then
			for mp_port in $mp_ports;do 
				eval "eval_port=\$$mp_port"

				process_alive_num "$mp_prefix" "1" "$eval_port" || \
					addwarn_alarm "${mp_prefix}:${eval_port};"
			done
		else
			process_alive_num "${mp_prefix}" "1" || \
				addwarn_alarm "${mp_prefix}:down;"
		fi
		
	done
	return 0
}

function mon_keystone () {
	local service_timeout=5
    local keystone_name="keystone.cloud.yy.com"
    [ "x${OS_AUTH_URL}" == "x" ] && return 1

    # check remote gateway 
    local rgws=$(nslookup $keystone_name  | grep -A1 '^Name'| \
        awk '{if($1 ~ /Address/){print $2}}')
    for rgw in $rgws;do
        rgw="${rgw%.*}.1"
        ping -i0.2 -w2 $rgw >/dev/null 2>&1 && \
        { local rgw_stat='ok';break; }
    done

    if [ "x${rgw_stat}" == "x" ];then
        echo "$(date "+%Y-%m-%d %H:%M:%S") ping fail $keystone_name ($rgws)" >> $LOGFILE
        exit 1
    fi
	for port in 35357 5000;do
		keystone  --os-auth-url="http://${keystone_name}:$port/v2.0" \
			--timeout=$service_timeout role-list >/dev/null 2>&1 || \
			{ addwarn_alarm "keystone:$port:down";return 1; }
	done
	
}

function mon_nova () {
	local service="nova"
	local service_timeout=20
    [ "x${ENABLE_all_nova_warn}" != "xyes" ] && return 0

	service_alive $service

	nova_services=$(nova --timeout=$service_timeout service-list|tr -d '%')
	local down_agent=$(printf "$nova_services"|grep -w "$service"|tr -d ' '|\
		awk -F '|' '{if(tolower($6) !~ /up/){printf("%s:%s:down; ",$2,$3)}}')
	[ "x${down_agent}" != "x" ] && addwarn_alarm "$down_agent"

}

function mon_cinder () {
	local service=cinder
    [ "x${ENABLE_all_cinder_warn}" != "xyes" ] && return 0

	service_alive $service

	cinder_services=$(cinder service-list|tr -d '%')
	local down_agent=$(printf "$cinder_services"|grep -w "$service"|tr -d ' '|\
		awk -F '|' '{if(tolower($6) !~ /up/){printf("%s:%s:down; ",$2,$3)}}')

	[ "x${down_agent}" != "x" ] && addwarn_alarm "$down_agent"
}

function mon_glance () {
	local service=glance
    [ "x${ENABLE_all_glance_warn}" != "xyes" ] && return 0

	service_alive $service
}

function mon_neutron () {

    [ "x${ENABLE_all_neutron_warn}" != "xyes" ] && return 0

	service_alive "neutron"
	service_alive "dnsmasq"
	service_alive "ovsdb_server"
	service_alive "ovs_vswitchd"

    if [ "x${WARN_neutron_server}" == "xyes" ] && [ "x${WARN_dnsmasq}" == "xyes" ];then
	    neutron_services=$(neutron agent-list -f csv|tr -d '\"'|tr -s ' ' '-')
	    local down_agent=$(printf "$neutron_services"|grep -v 'admin_state_up'| \
	    	awk -F ',' '{if($4 !~ /:-\)/){printf("%s:%s:down; ",$2,$3)}}')
	    [ "x${down_agent}" != "x" ] && addwarn_alarm "$down_agent"

	    # check for dnsmasq num
	    local ALL_NETS=$(neutron net-list -f csv -F id|tr -d '\"\r'|grep "[0-9a-ZA-Z-]*-[0-9a-ZA-Z-]*")
	    for ALL_NET in $ALL_NETS;do
	    	local dns_num=$(neutron dhcp-agent-list-hosting-net $ALL_NET -f csv -F alive 2>/dev/null| wc -l)
	    	if [ $dns_num -lt 3 ];then
	    		local dns_net=$(neutron net-show $ALL_NET -f shell 2>/dev/null| \
                    awk -F '=' '{if($1 == "name"){print $2}}')
	    		addwarn_alarm "${dns_net}:dnsmasq:$((dns_num-1))"
	    	fi
	    done
    fi

	# check for ovsdb-server
	local services="ovsdb-server"
	if ovsdb-client list-dbs >/dev/null 2>&1;then
	    grep "${services}.*monitoring" $PLOG | grep -q'healthy' || addwarn_alarm "${services}:crash;"
    else
        addwarn_alarm "${services}:down;"
    fi

	# check for ovs-vswitchd
	local services="ovs-vswitchd"
	if ovs-ofctl show br-int >/dev/null 2>&1;then
	    grep "${services}.*monitoring" $PLOG | grep -q 'healthy' || addwarn_alarm "${services}:crash;"
    else
        addwarn_alarm "${services}:down;"
    fi

}

function mon_named () {
    local service=named

    [ "x${ENABLE_all_misc_warn}" != "xyes" ] && return 0
    service_alive "$service"
}

function mon_memcached () {
    local service=memcached

    [ "x${ENABLE_all_misc_warn}" != "xyes" ] && return 0
    service_alive "$service"
}

########
# main #
########

noblock_lock
trap "noblock_unlock;rm -f $PLOG;" 0 1 2 9 15


HOSTNAME=$(hostname)
case "${HOSTNAME%%-*}" in
    controller)
        source $KEYFILE && [ "x${OS_AUTH_URL}" != "x" ] || exit 1
        ENABLE_all_neutron_warn=no
        ;;
    neutron)
        source $KEYFILE && [ "x${OS_AUTH_URL}" != "x" ] || exit 1
        ENABLE_all_nova_warn=no
        ENABLE_all_cinder_warn=no
        ENABLE_all_glance_warn=no
        ENABLE_all_misc_warn=no
        WARN_neutron_openvswitch_agent=no
        ;;
    compute)
        ENABLE_all_nova_warn=no
        ENABLE_all_cinder_warn=no
        ENABLE_all_glance_warn=no
        ENABLE_all_misc_warn=no
        WARN_neutron_metadata_agent=no
        WARN_neutron_l3_agent=no
        WARN_neutron_server=no
        WARN_neutron_dhcp_agent=no
        WARN_dnsmasq=no
        ;;

    *)
        exit 1
        ;;
esac
[ -s /root/mon_openstack.conf ] && source /root/mon_openstack.conf


load_process
mon_keystone
mon_nova
mon_cinder
mon_glance
mon_neutron
mon_named
mon_memcached

send_alarm && exit 0 || exit 1

