#!/usr/bin/env python
# -*- coding:utf8 -*-
import os,sys
import time
import logging.config
import ConfigParser
from daemon import Daemon
import argparse


def singleton(cls):
  instances = {}
  def get_instance():
    if cls not in instances:
      instances[cls] = cls()
    return instances[cls]
  return get_instance()

#@singleton 低版本python不支持修饰符
class Logger():
  def __init__(self):
    if(not os.path.exists('./log')):
        os.mkdir('./log',0755)
    self.logger = logging.getLogger('root')
    self.logger.setLevel(logging.DEBUG)
    handler = logging.handlers.RotatingFileHandler('./log/lbsd_filter.log',maxBytes=5*1024*1024,backupCount=5)
    formatter = logging.Formatter('[01][%(asctime)s][%(levelname)s][%(filename)s][%(lineno)d][-][%(thread)d]=[%(message)s]=')
    handler.setFormatter(formatter)
    self.logger.addHandler(handler)
  def get_logger(self):
    return self.logger
Logger = singleton(Logger)

class Filter(object):
    def __init__(self):
        self.__lbsd_log = '/data/yy/log/lbsd_d/lbsd_d.log'
        if(not os.path.exists('./retcode')):
            os.mkdir('./retcode')
            
    def get_file_index(self):
        try:
            fd = open('./lbsd_filter.rec','r')
            index = int(fd.read())
            return index
        except:
            Logger.logger.exception("get_file_index")
            return 0

    def rec_file_index(self, index):
        try:
            fd = open('./lbsd_filter.rec','w')
            fd.write(str(index))
            fd.close()
        except:
            Logger.logger.exception("rec_file_index")
        
    def proc(self):
        try:
            if(not os.path.exists(self.__lbsd_log)):
                return
            fd = open(self.__lbsd_log, 'r')
            fd.seek(0,os.SEEK_END)
            file_index = self.get_file_index()
            rec_file_name = './retcode/result_code_%d'%file_index
            fd_rec = open(rec_file_name, 'a')
            sleep_time = 0
            while(True):
                lines = fd.readlines(1024*1024)
                for line in lines:
                    index = line.find('OT:')
                    if(index != -1):
                        fd_rec.write(line[index+3:index+4] + '\n')
                if(len(lines) > 0):
                    fd_rec.flush() #flush to disk
                    if(os.path.getsize(rec_file_name) >= 10*1024*1024):
                        fd_rec.close()
                        file_index += 1
                        self.rec_file_index(file_index)
                        rec_file_name = './retcode/result_code_%d'%file_index
                        fd_rec = open(rec_file_name, 'a')
                else:
                    time.sleep(1)
                    sleep_time += 1
                    if(sleep_time >= 5):
                        tmp = os.readlink('/proc/%d/fd/%d'%(os.getpid(),fd.fileno()))
                        if(os.path.basename(tmp) != os.path.basename(self.__lbsd_log)):
                            fd.close()
                            fd = open(self.__lbsd_log, 'r')
                        sleep_time = 0
        except:
            Logger.logger.exception("proc")
            self.rec_file_index(file_index)    

class MyDaemon(Daemon):
  def __init__(self, pid):
    super(self.__class__, self).__init__(pid)

  def run(self):
    filter = Filter()
    filter.proc()    

if __name__ == "__main__":

  parser = argparse.ArgumentParser()
  parser.add_argument('cmd', default="start", help='start, stop, restart')
  parser.add_argument('-p', '--pidfile' , default="./proc/daemon.pid")

  args = parser.parse_args()
  pidpath = os.path.abspath(args.pidfile);
  dirname = os.path.dirname(pidpath);
  if(not os.path.exists(dirname)):
      os.makedirs(dirname);
  daemon = MyDaemon(pidpath)
  if 'start' == args.cmd:
      daemon.start()
  elif 'stop' == args.cmd:
      daemon.stop()
  elif 'restart' == args.cmd:
      daemon.restart()
  else:
      print "Unknown command"
      sys.exit(2)
