#!/usr/bin/env python
# -*- coding: utf-8 -*-
##################################################
# parse new framework dbd log and generate json for YYMS
#
# Author: Wenwenliu <wenwenliu@chinaduo.com>
# Create: <2012-04-27 11:59>
# Usage:
#     use --help option to run this script
##################################################


__metaclass__ = type


import sys
import os
#from time import strftime, localtime
import datetime
from string import Template
import subprocess
#import shlex
from optparse import OptionParser
import re


debug_mode = False
output_job_stat = False


class AsyncJobLogInfo():
    '''Struct to store async job log information.'''
    def __init__(self, success=0, exception=0, timeout=0,
                 average_proc_time=0, max_proc_time=0, min_proc_time=0):
        self.success = success
        self.exception = exception
        self.timeout = timeout
        self.average_proc_time = average_proc_time
        self.max_proc_time = max_proc_time
        self.min_proc_time = min_proc_time


def parse(infile):
    '''Parse new dbd log and output log statistic in json format.'''

    dt = datetime.datetime.now() + datetime.timedelta(minutes=-1)
    regex_time = dt.strftime("%b %e %H:%M") + ":\d{2}"
    #regex_time = strftime("%b %d %H:%M", localtime()) + ":\d{2}"
    #regex_time = "\w{3} \d{2} \d{2}:\d{2}:\d{2}"
    regex_number = "\d{1,}"
    regex_double = "[0-9.]{1,}"
    regex_jobname = "[0-9a-zA-Z:_]{1,}"

    regex_proc_args = tuple([regex_time] + ([regex_number] * 4))
    regex_proc = "(%s) [a-zA-Z0-9_: ]{1,}\[\d{1,}\]: request:(%s) proc:(%s) droped:(%s) active:(%s)" % regex_proc_args
    regex_async_stat_args = tuple([regex_time] + ([regex_number] * 8))
    regex_async_stat = "(%s) [a-zA-Z0-9_: ]{1,}\[\d{1,}\]: task statistic droped:(%s), frameExpCnt:(%s), seqMissCnt:(%s), startAbortCnt:(%s), totalStartJobCnt:(%s), jobExpCnt:(%s), jobTimeoutCnt:(%s), transportError:(%s)" % regex_async_stat_args
    regex_async_jobs_stat_args = tuple([regex_time, regex_jobname] + ([regex_number] * 3) + [regex_double] + ([regex_number] * 2))
    regex_async_jobs_stat = "(%s) [a-zA-Z0-9_: ]{1,}\[\d{1,}\]: task:(%s) totalSuccessCnt:(%s), expCnt:(%s), timeoutCnt:(%s) last 10 second averTimeUs:(%s)us maxTime:(%s)us, minTime:(%s)us" % regex_async_jobs_stat_args

    pattern_proc = re.compile(regex_proc)
    pattern_async_stat = re.compile(regex_async_stat)
    pattern_async_jobs_stat = re.compile(regex_async_jobs_stat)

    stat_proc = {}
    stat_async = {}
    stat_async_jobs = {}
    latest_proc_time = ""
    get_first_async_stat = False
    get_async_stat_times = 0
    for line in infile:
        match = pattern_proc.match(line)
        if match:
            time = match.group(1)
            if time == latest_proc_time:
                proc_number = int(match.group(3))
                last_proc_number = stat_proc[latest_proc_time]["processed"]
                if proc_number > last_proc_number:
                    #stat_proc[latest_proc_time]["to_process"] = int(match.group(2))
                    stat_proc[latest_proc_time]["processed"] = proc_number
                    #stat_proc[latest_proc_time]["dropped_pkg"] = int(match.group(4))
                    #stat_proc[latest_proc_time]["active_thread"] = int(match.group(5))
            else:
                stat = {}
                #stat["to_process"] = int(match.group(2))
                stat["processed"] = int(match.group(3))
                #stat["dropped_pkg"] = int(match.group(4))
                #stat["active_thread"] = int(match.group(5))
                stat_proc[time] = stat
                latest_proc_time = time
        else:
            match = pattern_async_stat.match(line)
            if match:
                stat = {}
                stat["job_total"] = int(match.group(6))
                stat["abort_on_start"] = int(match.group(5))
                stat["job_except"] = int(match.group(7))
                stat["job_timeout"] = int(match.group(8))
                stat["job_seq_miss"] = int(match.group(4))
                stat["frame_except"] = int(match.group(3))
                stat["trans_error"] = int(match.group(9))
                stat["job_dropped"] = int(match.group(2))
                if get_first_async_stat == False:
                    stat_async["first"] = stat
                    stat_async["last"] = stat
                    get_first_async_stat = True
                else:
                    stat_async["last"] = stat
                get_async_stat_times += 1
            else:
                if output_job_stat == True:
                    match = pattern_async_jobs_stat.match(line)
                    if match:
                        time = match.group(1)
                        job_name = shorten(match.group(2).split("::").pop())
                        job_success = int(match.group(3))
                        job_exception = int(match.group(4))
                        job_timeout = int(match.group(5))
                        job_average_proc_time = float(match.group(6))
                        job_max_proc_time = int(match.group(7))
                        job_min_proc_time = int(match.group(8))
                        loginfo = AsyncJobLogInfo(
                            job_success, job_exception, job_timeout,
                            job_average_proc_time, job_max_proc_time,
                            job_min_proc_time)
                        if not job_name in stat_async_jobs.keys():
                            stat_async_jobs[job_name] = {time:loginfo}
                        else:
                            stat_async_jobs[job_name][time] = loginfo

    # output json
    (stat1, stat2, stat3) = cal_stat(stat_proc, stat_async, get_async_stat_times, stat_async_jobs)
    print_stat(stat1, stat2, stat3)


def shorten(name):
    if len(name) <= 8:
        return name
    ret_name = ""
    ret_name += name[0]
    for i in range(1, len(name)):
        ch = name[i]
        if ch.isupper() or ch.isdigit():
            ret_name += ch
    if len(ret_name) > 8:
        ret_name = ret_name[-8:]
    return ret_name


def cal_stat(stat_proc, stat_async, async_stat_times, stat_async_jobs):
    '''Calculate stat in summary.'''

    # calculate proc statistic
    # stat1 = {
    #     "to_process":0, "processed":0, "dropped_pkg":0, "active_thread":0}
    stat1 = {
        "processed":0}
    if len(stat_proc) > 0:
        for value in stat_proc.values():
            #stat1["to_process"] += value["to_process"]
            stat1["processed"] += value["processed"]
            #stat1["dropped_pkg"] += value["dropped_pkg"]
            #stat1["active_thread"] += value["active_thread"]

        total = len(stat_proc)
        for key, value in stat1.items():
            stat1[key] = value/total

    # calculate async summary statistic
    stat2 = {
        "job_total":0,
        "abort_on_start":0,
        "job_except":0,
        "job_timeout":0,
        "job_seq_miss":0,
        "frame_except":0,
        "trans_error":0,
        "job_dropped":0, }
    if async_stat_times > 1:
        for key in stat2.keys():
            stat2[key] = (stat_async["last"][key] - stat_async["first"][key]) / async_stat_times

    # calculate async each job statistic
    stat3 = {}
    for job_name, job_stat in stat_async_jobs.items():
         earlist_time = min(job_stat.keys())
         latest_time = max(job_stat.keys())
         n = len(job_stat)
         stat3[job_name + "_suc"] = (job_stat[latest_time].success - job_stat[earlist_time].success) / n
         stat3[job_name + "_exp"] = (job_stat[latest_time].exception - job_stat[earlist_time].exception) / n
         stat3[job_name + "_tout"] = (job_stat[latest_time].timeout - job_stat[earlist_time].timeout) / n
         stat3[job_name + "_avgtime"] = sum([stat.average_proc_time for stat in job_stat.values()]) / n
         stat3[job_name + "_mintime"] = min([stat.min_proc_time for stat in job_stat.values()])
         stat3[job_name + "_maxtime"] = max([stat.max_proc_time for stat in job_stat.values()])

    return (stat1, stat2, stat3)


def print_stat(*stats):
    '''Generate json according to log statistic.'''
    global debug_mode

    json_str = ""
    for stat in stats:
        for key, value in stat.items():
            json_str += ('"%s":"%s", ' % (key, value))
    json_str = json_str.rstrip(", ")
    json_str = '{"data":{' + json_str + '},"msg":{"Ntime":""}}'
    if debug_mode == True:
        import json
        json_object = json.loads(json_str)
        print >> sys.stdout, json.dumps(json_object, sort_keys=True, indent=4)
    else:
        print >> sys.stdout, json_str
    sys.stdout.flush()



if __name__ == '__main__':
    # check argument
    program_name = sys.argv[0]
    usage_template = Template('''
  Parse new framework dbd log and output json for YYMS

Synopsis:
  python $PROGRAM -p DBD_PROCESS_NAME [-d/--debug]

Example:
  For new framework dbd client_sessionapp, invole this common on terminal
    $$ python $PROGRAM -p db_client_sessionapp_d
  Or, for new framework dbd client_userapp, use this
    $$ python $PROGRAM --process db_client_userapp_d
  In each case, the arguement "db_*_d" is the process name.

  Specify option -d or --debug to enable debug mode, in which
  the json output is pretty printed.''')
    usage_str = usage_template.substitute(PROGRAM=program_name)
    version_template = Template('''$PROGRAM 0.5''')
    version_str = version_template.substitute(PROGRAM=program_name)

    parser = OptionParser(usage=usage_str, version=version_str)
    parser.add_option("-p", "--process", dest="process_name",
                      help="dbd process name, E.g. db_client_sessionapp_d.")
    parser.add_option("-d", "--debug",
                      action="store_true", dest="debug_mode", default=False,
                      help="enable debug mode to pretty print json result.")
    parser.add_option("-j", "--job",
                      action="store_true", dest='output_job_stat', default=False,
                      help="output async job statistic.")
    (options, args) = parser.parse_args()
    if options.process_name == None:
        parser.error("argument DBD_PROCESS_NAME missing")
    if options.debug_mode == True:
        debug_mode = True
    if options.output_job_stat == True:
        output_job_stat = True

    # set stdout to no buffer mode
    sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)

    # check log file's existence
    log_dir = "/data/yy/log"
    default_scan_line_number = 10000
    t = Template("$file_dir/$process_name/$process_name.log")
    file_path = t.substitute(file_dir=log_dir, process_name=options.process_name)
    if not os.path.isfile(file_path):
        print >> sys.stderr, "[Error] log file not exist: %s" % (file_path, )
        sys.exit(1)

    # parse log file
    args = ["tail", "-n", "%s" % (default_scan_line_number,), "%s" % (file_path, )]
    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    parse(p.stdout)
    sys.exit(0)
