#!/bin/bash
#Written by wuhaiting
#Readme: 批量刷IP，适用于客户端直接调用

#传入参数为:
#GATEWAY IP1 MASK1 IP2 MASK2 IP3 MASK3 ......GATEWAY

#返回值:
# 0:成功; 1:参数有误; 2:更新ip失败; 3:IP和网关匹配有误 4:未知错误

param_num=$BASH_ARGC;
param=$@
if [[ "$param_num" == "" ]] || [[ "$param_num" -lt 3 ]];then
	echo "传入参数为:"
	echo "IP1 MASK1 IP2 MASK2 IP3 MASK3 ......GATEWAY"
	exit 1
fi

TMP_FILE1=`mktemp /tmp/interface.XXXXXX`
TMP_FILE2=`mktemp /tmp/interface.XXXXXX`

cat /etc/issue|grep -i -E "ubuntu|Debian" > /dev/null 2>&1 && LINUX_VERSION=Ubuntu
cat /etc/issue|grep -i -E "centos|Red Hat" > /dev/null 2>&1 && LINUX_VERSION=Centos

function ubuntu_interface () {
	ubuntu_eth_dir=/etc/network
	#ubuntu_eth_dir=/home/wuhaiting
	GATEWAY=${BASH_ARGV[0]};GATEWAY_SUFFIX=`echo ${GATEWAY%.*}`
	for((i=$param_num-1;i>0;i=i-2));do
		j=`expr $i - 1`
		IP_ADDR=${BASH_ARGV[$i]};NETMASK=${BASH_ARGV[$j]};
	
		IP_SUFFIX=`echo ${IP_ADDR%.*}`
		if [ "$IP_SUFFIX" == "$GATEWAY_SUFFIX" ];then
			echo -e "# The loopback network interface\nauto lo\niface lo inet loopback\n" >> $TMP_FILE1
			
			echo -e "# The primary network interface\nauto eth0\niface eth0 inet static" >> $TMP_FILE1
			echo -e "\taddress $IP_ADDR\n\tnetmask $NETMASK\n\tgateway $GATEWAY\n" >> $TMP_FILE1
		
			break;
		fi
	done
	
	if [ "`grep -ce "auto eth0$" $TMP_FILE1`" -ne 1 ];then
		rm -f $TMP_FILE1 $TMP_FILE2
		echo "3:IP和网关匹配有误"
		exit 3
	fi
	
	unset IP_ADDR NETMASK GATEWAY IP_SUFFIX
	for((i=$param_num-1;i>0;i=i-2));do
		j=`expr $i - 1`
		IP_ADDR=${BASH_ARGV[$i]};NETMASK=${BASH_ARGV[$j]};IP_SUFFIX=`echo ${IP_ADDR%.*}`
		
		[[ "$GATEWAY_SUFFIX" == "$IP_SUFFIX" ]] && continue;
		
		echo $IP_ADDR | grep -qE "^10\.|^172\.|^192\." 2>/dev/null
		if [ "$?" != "0" ];then
			external_line=`grep -cw "auto.*eth0" $TMP_FILE1`
			echo -e "auto eth0:$external_line\niface eth0:$external_line inet static\n\taddress $IP_ADDR\n\tnetmask $NETMASK\n" >> $TMP_FILE1
			unset external_line
			continue;
		else
			external_line=`grep -cw "auto.*eth1" $TMP_FILE2`
			if [ "$external_line" == "0" ];then
				echo -e "# The second network interfaces\nauto eth1\niface eth1 inet static\n\taddress $IP_ADDR\n\tnetmask $NETMASK\n" >> $TMP_FILE2
			else
				echo -e "auto eth1:$external_line\niface eth1:$external_line inet static\n\taddress $IP_ADDR\n\tnetmask $NETMASK\n" >> $TMP_FILE2
			fi
		fi
	done
	[[ -s $TMP_FILE2 ]] && cat $TMP_FILE2 >> $TMP_FILE1
	
	if [ -s $TMP_FILE1 ];then
		wall_time=`date +%Y%M%d-%H-%S-%m`
		cp $ubuntu_eth_dir/interfaces /var/log/sysop_manager/interfaces.$wall_time
		cp -f $TMP_FILE1 $ubuntu_eth_dir/interfaces
		chmod 644 $ubuntu_eth_dir/interfaces
	fi

}

function centos_interface () {
	pkill -9 -f NetworkManager
	centos_eth_dir=/etc/sysconfig/network-scripts
	#centos_eth_dir=/home/wuhaiting
	wall_time=`date +%Y%M%d-%H-%S-%m`
	
	GATEWAY=${BASH_ARGV[0]};GATEWAY_SUFFIX=`echo ${GATEWAY%.*}`
	for((i=$param_num-1;i>0;i=i-2));do
		j=`expr $i - 1`
		IP_ADDR=${BASH_ARGV[$i]};NETMASK=${BASH_ARGV[$j]};
	
		IP_SUFFIX=`echo ${IP_ADDR%.*}`
		if [ "$IP_SUFFIX" == "$GATEWAY_SUFFIX" ];then
			echo -e "DEVICE=eth0\nBOOTPROTO=static\nNM_CONTROLLED=yes\nONBOOT=yes" > $TMP_FILE1
			echo -e "IPADDR=$IP_ADDR\nNETMASK=$NETMASK\nGATEWAY=$GATEWAY" >> $TMP_FILE1
			break;
		fi
	done
	
	if [ ! -s $TMP_FILE1 ];then
		rm -f $TMP_FILE1 $TMP_FILE2
		echo "3:IP和网关匹配有误"
		exit 3
	fi
	
	unset IP_ADDR NETMASK GATEWAY IP_SUFFIX
	for((i=$param_num-1;i>0;i=i-2));do
		j=`expr $i - 1`
		IP_ADDR=${BASH_ARGV[$i]};NETMASK=${BASH_ARGV[$j]};IP_SUFFIX=`echo ${IP_ADDR%.*}`
		
		[[ "$GATEWAY_SUFFIX" == "$IP_SUFFIX" ]] && continue;
	
		echo $IP_ADDR | grep -qE "^10\.|^172\.|^192\." && eth_index=1 || eth_index=0;
		external_line=`grep -w "^DEVICE=eth${eth_index}" $TMP_FILE1 /tmp/ifcfg-eth* 2>/dev/null|wc -l`
		
		if [ "$external_line" != "0" ];then
			echo -e "DEVICE=eth${eth_index}:${external_line}\nBOOTPROTO=static\nNM_CONTROLLED=yes\nONBOOT=yes" > /tmp/ifcfg-eth${eth_index}:${external_line}
			echo -e "IPADDR=$IP_ADDR\nNETMASK=$NETMASK\n" >> /tmp/ifcfg-eth${eth_index}:${external_line}
		else
			echo -e "DEVICE=eth${eth_index}\nBOOTPROTO=static\nNM_CONTROLLED=yes\nONBOOT=yes" > /tmp/ifcfg-eth${eth_index}
			echo -e "IPADDR=$IP_ADDR\nNETMASK=$NETMASK\n" >> /tmp/ifcfg-eth${eth_index}
		fi
	done
	
	cd $centos_eth_dir
	for a in `ls ifcfg-eth* 2>/dev/null`;do
		mv $a /var/log/sysop_manager/${a}.${wall_time}
	done
	cd - > /dev/null 
	
	mv $TMP_FILE1 $centos_eth_dir/ifcfg-eth0
	mv /tmp/ifcfg-eth* $centos_eth_dir/. 2>/dev/null
	sudo chmod 644 $centos_eth_dir/ifcfg-eth*
}

if [ "$LINUX_VERSION" == "Ubuntu" ];then
	ubuntu_interface
elif [ "$LINUX_VERSION" == "Centos" ];then
	centos_interface
fi
rm -f $TMP_FILE1 $TMP_FILE2