#!/bin/bash
#Date:2012-11-29
#Written by manifold.
#Readme:用户权限新增工具。


source commonFunction.sh

# IP地址获取
IP_ADDR=$(getSingleIp)

                         
# 参数获取
parseParameter || exit $?


# 判断此服务器是否启用新版权限系统，若未启用新版权限系统，不予开通
if [ ! -f /var/log/sysop_manager/start_user_privilege ];then

    code=100; msg="此服务器未启用新版权限系统，如需开通，请联系值班同事。"

    printResult "$code" "$msg"
     
    exit $code
    
fi


# 只允许开通服务器角色为C以及W的权限
cat /etc/motd | grep -q -E "Server Role:C|Server Role:W" || {

    LOGIN_USER=$(echo $cur_user | sed 's/^dw_//')
    
    id $LOGIN_USER || { 

        code=100; msg="$SCRIPT_NAME [ERR ] | 只允许开通网站事业部/YY普通前端服务器权限，或者是为别人开通自己有权限的服务器，不允许开通 DB 类服务器权限，此类权限请联系 吴海庭 进行开通"

        printResult "$code" "$msg" 
         
        exit $code ; }

}


# 禁止以非root用户运行此脚本
RUN_USER="root"
checkUser $RUN_USER || { code=101; msg="$SCRIPT_NAME [ERR ] | only $RUN_USER can run this script"
    
                         printResult "$code" "$msg" 
                         
                         exit $code ; }
                         


# 所需参数检查

# 申请原因
if [ -z "$PARA_APPLY_REASON" ];then

    code=102; msg="$SCRIPT_NAME [ERR ] | PARA_APPLY_REASON value is null."

    printResult "$code" "$msg" 

    exit $code
    
fi


# 操作人
PARA_OPERATOR="tool_system"


# 申请开通用户名
# PARA_USERNAME
if [ -z "$PARA_APPLY_USERNAME" ];then

    code=103; msg="$SCRIPT_NAME [ERR ] | PARA_APPLY_USERNAME value is null."

    printResult "$code" "$msg" 

    exit $code
    
fi


# 申请开通模式：1小时sudo权限/永久性sudo权限
# PARA_SUDO_MODE
if [ -f /var/log/sysop_manager/start_user_privilege ];then

    # 申请开通模式：强制 PARA_SUDO_MODE 为 temp，只给开通1个小时的sudo权限
    PARA_SUDO_MODE=${PARA_SUDO_MODE:="temp"}
    
else

    PARA_SUDO_MODE=${PARA_SUDO_MODE:="forever"}
    if [ "$PARA_SUDO_MODE" != "temp" ] && [ "$PARA_SUDO_MODE" != "forever" ];then

        code=106; msg="$SCRIPT_NAME [ERR ] | sudo 模式错误，请联系 余万福 进行分析处理"
        printResult "$code" "$msg" 
        exit $code

    fi

fi


ALL_USERNAME=$(echo "$PARA_APPLY_USERNAME" | sed 's/\\n/,/g' | sed -r 's/[[:space:]]+//g')


# 用户 ssh key 查询接口定义
function querySshKey () {

    WGET_TEMP_FILE=$(mktemp)
    wgetAction manager.repos.yy.duowan.com server_privileges_user_key.php $WGET_TEMP_FILE "action=query&username=$USER"
    
    if grep -q -E "ssh-rsa|ssh-dss" $WGET_TEMP_FILE;then
    
        USER_KEY=$(cat $WGET_TEMP_FILE|grep -E "ssh-rsa|ssh-dss")
    
    elif grep -q "username not found" $WGET_TEMP_FILE;then
    
        code=104; msg="$SCRIPT_NAME [ERR ] | 用户 $USER ssh-key 为空，若重试后依旧出现此错误，请联系 吴海庭 进行 ssh-key 新增"
        printResult "$code" "$msg" 
        exit $code
        
    else
    
        code=105; msg="$SCRIPT_NAME [ERR ] | 未知错误，请检查磁盘是否 readonly，若磁盘正常，重试后依旧出现此错误，请联系 余万福 进行分析处理"
        printResult "$code" "$msg" 
        exit $code
        
    fi
    
    rm -f $WGET_TEMP_FILE
    
}


# sudo 权限回收接口回调函数定义
function delSudo () {

    WGET_TEMP_FILE=$(mktemp)
    CONTENT="{\"apply_reason\":\"$PARA_APPLY_REASON\",\"operator\":\"$PARA_OPERATOR\",\"user\":\"$ALL_USERNAME\",\"ips\":\"$IP_ADDR\",\"recycle_time\":\"0000\",\"status\":\"2\"}"
    wgetAction manager.repos.yy.duowan.com server_privileges_del_sudo.php $WGET_TEMP_FILE "content=$CONTENT"
    rm -f $WGET_TEMP_FILE

}


# 用户新增函数定义
function addUser () {

    querySshKey
    
    if [ "$USER" == "yuwanfu" ] || [ "$USER" == "wuhaiting" ];then
        USER_GROUP=segroup
    elif [ "$USER" = "logreader" ] || [ "$USER" = "im_statistics" ];then
        USER_GROUP=logreader
    else
        USER_GROUP=execute
    fi
    
    /usr/sbin/groupadd $USER;
    /usr/sbin/useradd $USER -g $USER -G $USER_GROUP -s /bin/bash;
    /usr/sbin/usermod -G $USER_GROUP $USER
    sudo mkdir -p /home/$USER/.ssh;
    echo $USER_KEY > /home/$USER/.ssh/authorized_keys;
    sudo chown -R $USER.$USER /home/$USER;
    sudo chmod 700 /home/$USER/.ssh;
    sudo chmod 600 /home/$USER/.ssh/authorized_keys

}


# 用户新增函数定义
function addUserCMDB () {

    id $USER || { code=104; msg="$SCRIPT_NAME [ERR ] | 此服务器已启用新版权限系统，服务器上无此用户，请先在 cmdb 添加。"
    
                 printResult "$code" "$msg" 
                 
                 exit $code ; }
    
    if [ "$USER" == "yuwanfu" ] || [ "$USER" == "wuhaiting" ];then
        USER_GROUP=segroup
    elif [ "$USER" = "logreader" ] || [ "$USER" = "im_statistics" ];then
        USER_GROUP=logreader
    else
        USER_GROUP=execute
    fi
    
    /usr/sbin/usermod -G $USER_GROUP $USER

}



# 遍历添加所有用户
for USER in $(echo -e "$ALL_USERNAME" | awk -F ',' '{for(i=1;i<=NF;i++) print $i}');do

    USER=$(trimSpace $USER)
    
    # 判断是否为测试机，测试机全部可以开通永久权限
    cat /home/dspeak/yyms/hostinfo.ini | grep "status=1"
    if [ "$?" -eq 0 ];then
        # 判断启用新的权限系统
        [[ -f /var/log/sysop_manager/start_user_privilege ]] && addUserCMDB || addUser
    else
        PARA_SUDO_MODE="forever"
        addUser
    fi
    
done



# 当用户选择开通的是临时权限是，回写权限回收信息
if [ "$PARA_SUDO_MODE" == "temp" ];then

    delSudo
    
fi



# 最后的信息返回
code=0 ; msg="success"

printResult "$code" "$msg"

exit $code
