#!/bin/bash
#Date:2012-10-17
#Written by manifold.
#Readme:/etc/hosts变更操作工具。


source commonFunction.sh


# 禁止以非root用户运行此脚本
RUN_USER="root"
checkUser $RUN_USER || { code=101; msg="$SCRIPT_NAME [ERR ] | only $RUN_USER can run this script"
    
                         printResult "$code" "$msg" 
                         
                         exit $code ; }

                         
# 主脚本

# 参数获取

parseParameter || exit $?

# 所需参数检查：
# 1、host(禁止为空)
# 2、host对应的IP地址（禁止为空）
# 3、若原来无此条目，会进行添加；若原来/etc/hosts有相应host条目，是否强制替换。

# host

if [ -z "$PARA_ALTER_HOST" ]; then

    code=102 ; msg="$SCRIPT_NAME [ERR ] | PARA_ALTER_HOST value is null"
    
    printResult "$code" "$msg"
    
    exit $code
    
fi

# host对应的IP

if [ -z "$PARA_ALTER_HOST_IP" ]; then

    code=103 ; msg="$SCRIPT_NAME [ERR ] | PARA_ALTER_HOST_IP value is null"
    
    printResult "$code" "$msg"
    
    exit $code

else

    checkIsValidIp "$PARA_ALTER_HOST_IP" || { code=106; msg="$SCRIPT_NAME [ERR ] | ip format error."
    
                                              printResult "$code" "$msg" 
                                             
                                              exit $code ; }
    
fi

# PARA_ALTER_ENFORCE 默认不强制替换/变更

PARA_ALTER_ENFORCE=${PARA_ALTER_ENFORCE:="no"}


# 检查相应host是否已经存在于/etc/hosts（用awk可以无视host后面的空格等）

HOST_LINE=$( cat /etc/hosts | awk '{if($2=="'"${PARA_ALTER_HOST}"'") print}' | wc -l )


# 找到一条对应host条目

if [ "$HOST_LINE" -eq 1 ];then

    # 强制替换
    
    if [ "$PARA_ALTER_ENFORCE" = "yes" ]; then

        # 备份旧 host
        bakcupFileContent /etc/hosts
        
        # sed -r -i 's#.*\.[[:digit:]]+[ \t]+'"${PARA_ALTER_HOST}"'[ \t]*#'"${PARA_ALTER_HOST_IP}"' '"${PARA_ALTER_HOST}"'#g' /etc/hosts
        # change:2012-12-14 发现有 localhost www.test.com 这种情况
        sed -r -i 's#.*[ \t]+'"${PARA_ALTER_HOST}"'[ \t]*#'"${PARA_ALTER_HOST_IP}"' '"${PARA_ALTER_HOST}"'#g' /etc/hosts
        
    else
        
        # 不强制替换，将/etc/hosts相应条目带进告警信息打印出来
        
        code=104 ; msg="$SCRIPT_NAME [ERR ] | have found host entry:\"$( sed -n '/'"${PARA_ALTER_HOST}"'/p' /etc/hosts )\""
        
        printResult "$code" "$msg"
        
        exit $code
        
    fi

# 找过超过一条的对应hosts
    
elif [ "$HOST_LINE" -gt 1 ];then
    
    # 强制替换
    
    if [ "$PARA_ALTER_ENFORCE" = "yes" ]; then
    
        bakcupFileContent /etc/hosts
        
        # 删除旧条目
        sed -r -i '/.*[ \t]+'"${PARA_ALTER_HOST}"'[ \t]*/d' /etc/hosts
        
        echo "$PARA_ALTER_HOST_IP $PARA_ALTER_HOST" >> /etc/hosts
        
    else
    
        code=105 ; msg="$SCRIPT_NAME [ERR ] | more than one host entry:\"$( sed -n '/'"${PARA_ALTER_HOST}"'/p' /etc/hosts | tr '\n' ',' )\""
        
        printResult "$code" "$msg"
        
        exit $code
        
    fi
    
else

    # 无相应host，进行添加操作
    
    echo "$PARA_ALTER_HOST_IP $PARA_ALTER_HOST" >> /etc/hosts
    
fi


# 最后的信息返回

code=0; msg="$SCRIPT_NAME [INFO ] | successful."

printResult "$code" "$msg"

exit $code

