#!/bin/bash
#Date:2012-09-27
#Written by manifold.
#Readme:check host heartbeat.

source commonFunction.sh

# IP地址获取
IP_ADDR=$(getSingleIp)

# 基础变量定义
MYPID=$$

# 脚本名称定义
SCRIPT_NAME=$(basename $0)

# 禁止以非root用户运行此脚本
RUN_USER="root"
checkUser $RUN_USER || { code=101; msg="$SCRIPT_NAME [ERR ] | only $RUN_USER can run this script"
    
                         printResult "$code" "$msg" 
                         
                         exit $code ; }

                         
# 主脚本分析部分

# 检查系统并定义crontab的进程名字
checkOs ubuntu && CRON_NAME="cron"
checkOs centos && CRON_NAME="crond"

if [ -z "$CRON_NAME" ];then

    code=102; msg="$SCRIPT_NAME [ERR ] | CRON_NAME is null,the system is neither ubuntu nor centos,please contact dw_wuhaiting."
    
    printResult "$code" "$msg"
    
    exit $code
    
fi


# 检查crontab是否在运行
pidof "$CRON_NAME" || { # code=103; msg="$SCRIPT_NAME [ERR ] | crontab process $CRON_NAME is not running,please contact dw_wuhaiting."
                        code=103; msg="$SCRIPT_NAME [ERR ] | crontab进程未运行，请联系海庭进行处理。"
    
                        printResult "$code" "$msg" 
                        
                        exit $code ; }


# 告警判断条件：
# 有op_ccc.py及其他yyms相关文件，我们调用了，还是有告警，那就是尔平他们的问题（可能资产问题）
# 有op_ccc.py及其他yyms相关文件，我们未调用，那就是我们的问题，我们需要查查是否crontab没起作用，或者是否没起yymp_agent_d进程
# 无op_ccc.py，要细分：
# 是否需要连接管理机，若需要，没连管理机，那是我们问题；若无需连接管理机，那么这个告警就不应该出来，应该找尔平他们


# 判断yyms的相关文件/目录是否存在
if [ ! -f /home/dspeak/yyms/yyms_agent_2_d/cron/op_ccc.py ];then
    
    LOST_FILE="yes"
    
fi

for YYMS_DIR in \
                /home/dspeak/yyms/yyms_agent_2_d/cron \
                /home/dspeak/yyms/yymp \
                /home/dspeak/yyms/yymp/yymp_pylibs \
                /home/dspeak/yyms/yymp/yymp_pylibs/yymp_common
do

    if [ ! -d "$YYMS_DIR" ];then
        
        LOST_FILE="yes"
        
    fi
    
done


if [ "$LOST_FILE" = "yes" ]; then
    
    # code=104; msg="$SCRIPT_NAME [ERR ] | lost some yyms file,please contact dw_wuhaiting."
    code=104; msg="$SCRIPT_NAME [ERR ] | 丢失了部分 yyms 文件，请联系海庭进行处理。"
    
    printResult "$code" "$msg"
    
    exit $code
    
fi


# 检查是否有调用op_ccc.py
LAST_RUN_DATE=$(tail -n 1000 /var/log/sysop_manager/sysop_manager.log | awk '{if($NF~/op.ccc/) print $1}' | tail -n 1 | sed -r 's/(.*)-(.*)/\1 \2/g')

LAST_RUN_DATE_STRING=$(date --date "$LAST_RUN_DATE" "+%s")
NOW_DATE_STRING=$(date --date "now" "+%s")


# 判断调用时间是否超过10分钟
if [ -z "$LAST_RUN_DATE" ] || [ $(($NOW_DATE_STRING - $LAST_RUN_DATE_STRING)) -gt 600 ];then

    # 调用时间超过10分钟
    
    # code=105; msg="$SCRIPT_NAME [ERR ] | op_ccc.py have not been invoked in last 10 minutes,please contact dw_wuhaiting."
    code=105; msg="$SCRIPT_NAME [ERR ] | 在10分钟内未成功调用op_ccc.py，请联系海庭进行处理。"
    
    printResult "$code" "$msg"
    
    exit $code
    
else

    # 调用时间未超过10分钟，检查是否yymp没起
    
    pidof "yymp_agent_d" || { # code=106; msg="$SCRIPT_NAME [ERR ] | process yymp_agent_d is not running,please contact dw_wuhaiting."
                            code=106; msg="$SCRIPT_NAME [ERR ] | yymp_agent_d进程未运行请联系海庭进行处理。"
        
                            printResult "$code" "$msg" 
                            
                            exit $code ; }

fi


# 最后的信息返回
# code=0; msg="$SCRIPT_NAME [INFO ] | all environment is ok,if heartbeat also abnormal,please contact dw_wuerping."
code=0; msg="$SCRIPT_NAME [INFO ] | 所有环境检测通过，如果心跳告警依旧不正常，请联系海庭进行处理。"

printResult "$code" "$msg"

exit $code

