#!/bin/bash
#Date:2012-09-27
#Written by manifold.
#Readme:check host heartbeat.
#Change:2013-01-18
#Readme:改成管理机模式去跑，并加入 ping 等判断元素。


source commonFunction.sh


# 脚本名称定义
SCRIPT_NAME=$(basename $0)


# 禁止以非root用户运行此脚本
RUN_USER="root"
checkUser $RUN_USER || { code=101; msg="$SCRIPT_NAME [ERR ] | 禁止非 root 用户运行此脚本，请检查"
    
                         printResult "$code" "$msg" 
                         
                         exit $code ; }


# 主脚本分析部分

# 参数获取
parseParameter || exit $?


# 需进行处理的对应IP地址
if [ -z "$PARA_SEARCH_IP" ]; then

    code=102 ; msg="$SCRIPT_NAME [ERR ] | IP 地址为空，请检查"
    
    printResult "$code" "$msg"
    
    exit $code

fi


# 打印管理机IP地址（只用北上广3个点的管理机进行探测），上面有主机状态探测接口
function PRINT_MANAGER_SERVER () {

cat <<EOF
121.14.36.27
58.215.46.20
114.111.163.169
EOF

}


# 对端测试脚本
function testRemoteSystem {
    
    TOOL_SYSTEM_SCRIPT_DIR=/usr/local/i386/public_repos/initial_system/snmp/ext/tools_system
    
    FILE_NAME=$(mktemp)
    
    /home/yuwanfu/scp_channel_tools_system -P32200 -f -S $TOOL_SYSTEM_SCRIPT_DIR/checkHostHeartbeatAndPingRemoteScript.sh -D $CHECK_IP_ADDRESS:$FILE_NAME
    
    if [ "$?" != "0" ];then
    
        RETURN_INFO="RETURN_INFO\\t$CHECK_IP_ADDRESS\\t"
        
        code=103 ; msg="$RETURN_INFO [ERR ] | Scp 连接到对端服务器失败，请检查服务器是否负载过高 或者 磁盘只读 或者 网卡/其他硬件 有故障"
        
        echo "$msg"
        
        return $code
    
    fi
    
    BASE64_COMMAND_STRING=$(echo "/bin/bash $FILE_NAME ; rm -f $FILE_NAME" | base64 -w 0)
    
    /home/yuwanfu/ssh_channel_tools_system -h $CHECK_IP_ADDRESS -u yuwanfu -c "$BASE64_COMMAND_STRING"

    if [ "$?" != "0" ];then
    
        RETURN_INFO="RETURN_INFO\\t$CHECK_IP_ADDRESS\\t"
        
        code=103 ; msg="$RETURN_INFO [ERR ] | Ssh 连接到对端服务器失败，请检查服务器是否负载过高 或者 网卡/其他硬件 有故障"
        
        echo "$msg"
        
        return $code
    
    fi
    
}


# 遍历用户提交的 IP 地址，一台一台进行测试，判断是 网络问题 还是 系统问题（如crontab,负载高等原因）

for CHECK_IP_ADDRESS in $(echo "$PARA_SEARCH_IP" | sed 's/\\n/ /g'); do

    unset PING_STATUS
    
    # 先判断是否网络问题导致告警
    # 如果能 ping 通，直接进入 “系统内部问题判断” 步骤，否则进行管理机接口 ping 判断
    ping -w 2 $CHECK_IP_ADDRESS > /dev/null 2>&1
    
    if [ "$?" == "0" ];then
    
        # 进入系统内部判断逻辑
        PING_STATUS="OK"
        (testRemoteSystem)
        
    else
        
        for MANAGER_SERVER in $(PRINT_MANAGER_SERVER) ; do
        
            RESULT=$(curl -s -u manifold:dw_manifold "http://$MANAGER_SERVER:63217/revision/detect_ip_status.php?ip=$CHECK_IP_ADDRESS&timeout=2" 2>&1)
            
            if [ "$RESULT" == "0" ];then
            
                # 进入系统内部判断逻辑
                PING_STATUS="OK"
                (testRemoteSystem)
                
            else
            
                continue
                
            fi
            
        done
        
    fi

    if [ "$PING_STATUS" != "OK" ];then

        RETURN_INFO="RETURN_INFO\\t$CHECK_IP_ADDRESS\\t"
        
        code=104 ; msg="$RETURN_INFO [ERR ] | 北上广管理机无法 ping 通此服务器，请检查服务器是否已宕机 或者 整个机房网络故障"
        
        echo "$msg"

    fi

done


# 最后的信息返回
code=0; msg="$SCRIPT_NAME [INFO ] | successful."

printResult "$code" "$msg"

exit $code

