#!/bin/bash
#Date:2012-09-27
#Written by manifold.
#Readme:check host heartbeat.
#Change:2013-01-18
#Readme:改成管理机模式去跑，并加入 ping 等判断元素。
#Change:2013-05-22
#Readme:改用到网站事业部。


source commonFunction.sh


# 脚本名称定义
SCRIPT_NAME=$(basename $0)


# 禁止以非root用户运行此脚本
RUN_USER="root"
checkUser $RUN_USER || { code=101; msg="$SCRIPT_NAME [ERR ] | 禁止非 root 用户运行此脚本，请检查"
    
                         printResult "$code" "$msg" 
                         
                         exit $code ; }


# 主脚本分析部分

# 参数获取
parseParameter || exit $?


# 需进行处理的对应IP地址
if [ -z "$PARA_SEARCH_IP" ]; then

    code=102 ; msg="$SCRIPT_NAME [ERR ] | IP 地址为空，请检查"
    
    printResult "$code" "$msg"
    
    exit $code

fi


# 打印管理机IP地址（只用北上广3个点的管理机进行探测），上面有主机状态探测接口
function PRINT_MANAGER_SERVER () {

cat <<EOF
121.14.36.27
58.215.46.20
114.111.163.169
EOF

}


# 短信发送接口
SEND_SMS () {

    python send_sms_somebody_msg.pyc "$1" "$2"
    
    # echo "`date "+%F %T"` send sms:$1 $2" >> $LOG_DIR/sysop_manager.log
    
    # SEND_SMS 15989286245 "jump server-2 turn on iptables.server ip:$OA_ADDRESS."
    
}


# 对端测试脚本
function testRemoteSystem {
    
    TOOL_SYSTEM_SCRIPT_DIR=/usr/local/i386/public_repos/initial_system/snmp/ext/tools_system
    
    FILE_NAME=$(mktemp)
    
    # scp 传文件 retry 次数（包括首次尝试）
    RETRY_TIME=2
    
    until [ "$RETRY_TIME" -eq 0 ];do

        /home/yuwanfu/scp_channel_tools_system -P32200 -f -S $TOOL_SYSTEM_SCRIPT_DIR/checkHostHeartbeatAndPingRemoteScript.sh -D $CHECK_IP_ADDRESS:$FILE_NAME

        # 若文件传送失败，进行重试
        if [ "$?" != "0" ];then

            RETRY_TIME=$(($RETRY_TIME - 1))
            
            if [ "$RETRY_TIME" -gt 0 ];then
                
                continue
                    
            else
                
                RETURN_INFO="RETURN_INFO\\t$CHECK_IP_ADDRESS\\t"
                
                code=103 ; msg="$RETURN_INFO [ERR ] | 可PING通，但SCP到对端服务器失败，请检查是否服务器负载过高或是其他硬件故障。设备类型为：$SERVER_TYPE_INFO 。$REBOOT_INFO"
                
                echo "$msg" && return $code

            fi
            
        else
        
            # 若文件传送成功，执行相应脚本，然后执行中断
            BASE64_COMMAND_STRING=$(echo "/bin/bash $FILE_NAME ; rm -f $FILE_NAME" | base64 -w 0)
            
            /home/yuwanfu/ssh_channel_tools_system -h $CHECK_IP_ADDRESS -u yuwanfu -c "$BASE64_COMMAND_STRING"

            if [ "$?" != "0" ];then
            
                RETURN_INFO="RETURN_INFO\\t$CHECK_IP_ADDRESS\\t"
                
                code=103 ; msg="$RETURN_INFO [ERR ] | 可PING通，SCP成功，但SSH到对端服务器失败，请检查是否服务器负载过高或是其他硬件故障。"
                
                echo "$msg" && return $code

            fi            
            
            # 执行中断
            break
            
        fi
        
    done
    
}


# 获取设备类型名称（若为网站类机器，同时设备类型为B类的话，直接找俊廉；其他机器联系YY运维处理）
function getServerInfo {

    CMDB_INFO=$( getCmdbInfoByIp "$1" )
    
    if [ "$CMDB_INFO" == "" ];then
    
        SERVER_TYPE_INFO="设备类型为：获取设备类型失败"
        
        REBOOT_INFO="请确认后再操作"
        
        NET_MACHINE=no
        
    else
      
        SERVER_TYPE_ID=$( echo $CMDB_INFO | python -mjson.tool | grep server_type | awk -F [:,] '{print $2}' )
        
        [ -z "$SERVER_TYPE_ID" ] && SERVER_TYPE_INFO="设备类型为：获取设备类型失败" || SERVER_TYPE_INFO="设备类型为：$( getServerTypeNameById $SERVER_TYPE_ID )"

        if echo $CMDB_INFO | grep -q -E "网站事业部->|YY事业部->A线" ; then

            echo $SERVER_TYPE_INFO | grep -q -E "A|B" && REBOOT_INFO="网站类机器，请联系俊廉处理" || REBOOT_INFO="网站类机器，可直接重启"
            echo "$CMDB_INFO" | grep -q "网站事业部->网站一组->英雄联盟盒子[消息队列]" && REBOOT_INFO="网站类机器，请联系俊廉处理"
            
        else
 
            REBOOT_INFO="非网站类机器，联系YY运维同事处理"
            
            NET_MACHINE=no
            
        fi

    fi
    
}


# 遍历用户提交的 IP 地址，一台一台进行测试，判断是 网络问题 还是 系统问题（如crontab,负载高等原因）
for CHECK_IP_ADDRESS in $(echo "$PARA_SEARCH_IP" | sed 's/\\n/ /g'); do

    getServerInfo $CHECK_IP_ADDRESS
    
    if [ "$NET_MACHINE" == "no" ];then
    
        RETURN_INFO="RETURN_INFO\\t$CHECK_IP_ADDRESS\\t"
        
        code=105 ; msg="$RETURN_INFO [ERR ] | $SERVER_TYPE_INFO , $REBOOT_INFO"
        
        echo "$msg"

        continue
        
    fi
    
    unset PING_STATUS
    
    # 先判断是否网络问题导致告警
    # 如果能 ping 通，直接进入 “系统内部问题判断” 步骤，否则进行管理机接口 ping 判断
    ping -w 4 $CHECK_IP_ADDRESS > /dev/null 2>&1
    
    if [ "$?" == "0" ];then
    
        # 进入系统内部判断逻辑
        PING_STATUS="OK"
        (testRemoteSystem) ; exit $?
        
    else
        
        for MANAGER_SERVER in $(PRINT_MANAGER_SERVER) ; do
        
            RESULT=$(curl -s -u manifold:dw_manifold "http://$MANAGER_SERVER:63217/revision/detect_ip_status.php?ip=$CHECK_IP_ADDRESS&timeout=4" 2>&1)
            
            if [ "$RESULT" == "0" ];then
            
                # 进入系统内部判断逻辑
                PING_STATUS="OK"
                (testRemoteSystem) ; exit $?
                
            else
            
                continue
                
            fi
            
        done
        
    fi

    if [ "$PING_STATUS" != "OK" ];then

        RETURN_INFO="RETURN_INFO\\t$CHECK_IP_ADDRESS\\t"
        
        code=104 ; msg="$RETURN_INFO [ERR ] | 北上广管理机无法 ping 通此服务器，请检查服务器是否已宕机 或者 整个机房网络故障。$SERVER_TYPE_INFO 。$REBOOT_INFO"
        
        echo "$msg"

    fi

done


# 最后的信息返回
code=0; msg="$SCRIPT_NAME [INFO ] | successful."

printResult "$code" "$msg"

exit $code

