#!/bin/bash
#Date:2012-10-24
#Written by manifold.
#Readme:充值系统日志备份脚本。


# 任务说明：

: <<EOF
日志备份内容：
（1）备份规则
每天凌晨 01:00 之后备份昨天的日志文件，每天备份一次即可。
日志所在服务器目录：/data/yy/log/pay
日志文件名规则：*.log.日期*
日志在web服务器保留数量：3个月
例：假设今天是2012-10-24，昨天产生的文件日志名是：*.log.2012-10-23 或 *.log.2012-10-23.gz
注意文件名的前缀是不确定的。

（2）注意事项
各服务器产生的日志文件名可能是相同的，备份的时候需区分开服务器；

（3）需要备份的服务器列表
广州亚太双线：
113.108.228.93/58.248.184.93
113.108.228.94/58.248.184.94
113.108.228.150/58.248.181.22
113.108.228.152/58.248.181.24
119.147.160.100/58.249.113.36
119.147.160.111/58.249.113.47
119.147.160.123/58.249.113.59
119.147.160.124/58.249.113.60

广州BGP：
120.132.133.57
120.132.133.59

上海真如：
222.73.61.88/140.207.208.88
222.73.61.89/140.207.208.89
222.73.61.90/140.207.208.90
222.73.61.91/140.207.208.91

2012-11-13
上海真如-数据平台新增两台机器的日志备份
222.73.61.106
222.73.61.91

EOF


cd $(dirname $0)

source ../commonFunction.sh

PAY_SYSTEM_SCRIPT_DIR=/usr/local/i386/public_repos/initial_system/snmp/ext/tools_system/paySystem


# 禁止以非root用户运行此脚本
RUN_USER="root"
checkUser $RUN_USER || { code=101; msg="$SCRIPT_NAME [ERR ] | only $RUN_USER can run this script"
    
                         printResult "$code" "$msg" 
                         
                         exit $code ; }

                         
# 主脚本

# crontab任务定义

PARA_CRONTAB_ENTRY="30 01 * * * root /bin/bash $PAY_SYSTEM_SCRIPT_DIR/backupPaySystemLog.sh"

operateCrontab "show" "$PARA_CRONTAB_ENTRY" || operateCrontab "add" "$PARA_CRONTAB_ENTRY" || exit $?


# rsync 密码文件生成
RSYNC_SECRET_TEMP_FILE=$(mktemp)

USER_NAME=payloguser
USER_PASSWD=feqw2333f3W34dfS
echo "$USER_PASSWD" > $RSYNC_SECRET_TEMP_FILE
chmod 600 $RSYNC_SECRET_TEMP_FILE

RSYNC_SERVER_IP=58.215.143.209
RSYNC_MODULE=paylogbackup


# 查找需要备份的文件名，备份昨天的日志文件，将本地日志rsync出去，并删除旧日志文件（保留3个月）

RSYNC_FILE_TEMP_FILE=$(mktemp)

DATE_STRING=$(date --date "1 day ago $DATE_STRING" "+%Y-%m-%d")

for DIR in /data/yy/log/pay/ /data/yy/log/backup/ ; do

    cd $DIR && find . -maxdepth 1 -type f -name "*.log.$DATE_STRING*" -print > $RSYNC_FILE_TEMP_FILE
    
    rsync -avz --bwlimit=640 --password-file=$RSYNC_SECRET_TEMP_FILE --files-from=$RSYNC_FILE_TEMP_FILE $DIR $USER_NAME@$RSYNC_SERVER_IP::$RSYNC_MODULE/$IP_ADDR/ && \

    find $DIR -maxdepth 1 -type f -mtime +90 -exec rm -f {} \;

done


rm -f $RSYNC_FILE_TEMP_FILE

rm -f $RSYNC_SECRET_TEMP_FILE


