-- modified from http://lua-users.org/wiki/CommonFunctions
-- all/string.lua (all.string)
-- A Lua Library (ALL) - string utility functions.
-- This is compatible with Lua 5.1.
-- Licensed under the same terms as Lua itself.--DavidManura

module("string_ex", package.seeall)

function import()
  local env = getfenv(2)
  for k,v in pairs(all.string) do env[k] = v end
end

-- remove trailing and leading whitespace from string.
-- http://en.wikipedia.org/wiki/Trim_(8programming)
function trim(s)
  -- from PiL2 20.
  return (s:gsub("^%s*(.-)%s*$", "%1"))
end

-- remove leading whitespace from string.
-- http://en.wikipedia.org/wiki/Trim_(8programming)
function ltrim(s)
  return (s:gsub("^%s*", ""))
end

-- remove trailing whitespace from string.
-- http://en.wikipedia.org/wiki/Trim_(8programming)
function rtrim(s)
  local n = #s
  while n > 0 and s:find("^%s", n) do n = n - 1 end
  return s:sub(1, n)
end
-- The following more obvious implementation is generally not
-- as efficient, particularly for long strings since Lua pattern matching
-- starts at the left (though in special cases it is more efficient).
-- Related discussion on p.197 of book "Beginning Lua Programming".
--[[
function rtrim(s) return (s:gsub("%s*$", "")) end
]]

-- substitute variables into string.
-- Example: subst("a=$(a),b=$(b)", {a=1, b=2}) --> "a=1,b=2".
function subst(s, t)
  -- note: handle {a=false} substitution
  s = s:gsub("%$%(([%w_]+)%)", function(name)
    local val = t[name]
    return val ~= nil and tostring(val)
  end)
  return s
end

-- modified from http://lua-users.org/wiki/SplitJoin
function split(s, sep, max, reg_exp)
  assert(sep ~= '')
  assert(max == nil or max >= 1)

  local result = {}

  if #s > 0 then
    local plain = not reg_exp
    max = max or -1

    local field = 1
    local start = 1
    local first,last = s:find(sep, start, plain)
    while first and max ~= 0 do
      result[field] = s:sub(start, first-1)
      field = field+1
      start = last+1
      first, last = s:find(sep, start, plain)
      max = max-1
    end
    result[field] = s:sub(start)
  end

  return result
end

-- add bu chengguifeng
function startswith(s, prefix)
  return string.sub(s, 1, string.len(prefix)) == prefix
end
