import warnings
warnings.filterwarnings("ignore",category=DeprecationWarning)
import os
module_file_dir = os.path.dirname(__file__)
if(module_file_dir):
    os.chdir(module_file_dir)
g_caller = os.path.basename(__file__)
import sys,time
sys.path.append("../yymp_pylibs/yymp_common")
from agent_protocol import UnixUdpClient
import proxy_module
import urllib2
import re
import shutil
import fcntl
import socket
try:
    from hashlib import md5
except ImportError:
    from md5 import md5
import logging
import traceback
import random
from hostinfo import HostInfo
from app_conf import log_config

def http_download(url, download_target_path):
    request = urllib2.Request(url)
    #logging.debug(url)
    try:
        response = urllib2.urlopen(request)
        response_headers = response.info()
        # print response_headers.dict
        # length = response_headers["content-length"]
        filename = response_headers["content-disposition"].split('filename=')[1][1:-1]
        fullpath = download_target_path + filename;
        f = open(fullpath, 'wb')
        data = response.read() # read all ?
        while data:
            f.write(data)
            data = response.read()
        f.close()
        return (True, 200, fullpath)
    except urllib2.HTTPError, e:
        if(e.code == 304):
            # logging.debug("304 Not Modified")
            return (True, 304, "")
        else:
            logging.exception("url=%s, code=%s" % (url, e.code))
        return (False, e.code, "")
    except:
        UnixUdpClient("/tmp/yyac_worker_ant.api.sock").write_logid(100001, 0, long(time.time()), traceback.format_exc())
        logging.exception(url)
        return (False, 500, "")

class Strategy(object):
    def __init__(self):
        self.__hostinfo = HostInfo.get_hostinfo()
        self.__etc_path = "/home/dspeak/yyms/yymp/yymp_agent/etc/"
        self.__strategy_local_symbol_link = self.__etc_path + "strategy_local"
        self.__strategy_public_symbol_link = self.__etc_path + "strategy_public"
        self.__server_id = self.__hostinfo.get("server_id", None)
        self.__datanodes = self.__hostinfo.get("dnips").split(":")
        random.shuffle(self.__datanodes)

    def get_download_strategy_local_url(self, host, server_id, md5digest):
        url_tmpl = "http://%s:51300/get_strategy?serverId=%d&md5=%s"
        return url_tmpl % (host, server_id, md5digest)

    def get_download_public_strategy_url(self, host, md5digest):
        url_tmpl = "http://%s:51300/get_strategy?serverId=public_strategy&md5=%s"
        return url_tmpl % (host, md5digest)

    def rm_strategy_file(self):
        symbol_link_list = [self.__strategy_local_symbol_link,
                   self.__strategy_public_symbol_link]
        for symbol_link in symbol_link_list:
            if(not os.path.exists(symbol_link)):
                continue
            if(not os.path.islink(symbol_link)):
                os.remove(symbol_link)
                continue
            rp = os.readlink(symbol_link)
            os.unlink(symbol_link)
            os.remove(self.__etc_path + rp)

    def rm_old_strategy_file(self, serverid, fullpath):
        if fullpath == "":
            return
        for root, dirs, files in os.walk(self.__etc_path):
            for file in files:
                path = os.path.join(root, file)
                if re.match("^%s\.\S+" % serverid, file) and (not os.path.islink(path)):
                    if path != fullpath:
                        os.remove(path)
            break

    def rm_old_tar_file(self, fullpath, symbol_link):
        try:		
            if fullpath == "":
                return
            path = os.readlink(symbol_link)
            dir_name = os.path.dirname(path)
            if not dir_name.startswith(self.__etc_path):
                return
            file_name = os.path.basename(path)		
            for root, dirs, files in os.walk(dir_name):
                for file in files:
                    try:					
                        if file == file_name:
                            continue					
                        file_path = os.path.join(root, file)
                        os.remove(file_path)		
                    except:
                        traceback.print_exc()						
        except:
            traceback.print_exc()			

    def get_public_md5(self):
        return self.get_strategy_file_md5(self.__strategy_public_symbol_link)

    def get_local_md5(self):
        return self.get_strategy_file_md5(self.__strategy_local_symbol_link)

    def get_strategy_file_md5(self, symbol_link):
        md5digest = "0"
        if(not os.path.exists(symbol_link)):
            return md5digest
        if(not os.path.islink(symbol_link)):
            os.remove(symbol_link)
            return md5digest
        rf = os.readlink(symbol_link)
        (server_id, md5digest) = rf.split(".")
        if(self.__server_id == server_id):
            return md5digest
        else:
            return md5digest

    def tarxzf(self, tar_file, target_path):
        import tarfile
        tar = tarfile.open(tar_file)
        ret = None
        for tarinfo in tar:
            #tar.extract(tarinfo, target_path + tarinfo.name)
            tar.extract(tarinfo, target_path)
            ret = tarinfo.name
        tar.close()
        return ret

    def download_public(self):
        md5digest = self.get_public_md5()
        symbol_link = self.__strategy_public_symbol_link
        for host in self.__datanodes:
            url = self.get_download_public_strategy_url(host, md5digest)
            (retCode, httpCode, fullPath) = http_download(url, self.__etc_path)
            if(retCode):
                if(httpCode == 200):
                    filename = self.tarxzf(fullPath, self.__etc_path)
                    if(os.path.exists(symbol_link)):
                        os.unlink(symbol_link)
                    else:
                        try:						
                            os.remove(symbol_link)						
                        except:
                            traceback.print_exc()							
                    os.symlink(self.__etc_path + filename, symbol_link)
                self.rm_old_strategy_file("public_strategy", fullPath)
                self.rm_old_tar_file(fullPath, symbol_link)				
                break

    def download_local(self):
        symbol_link = self.__strategy_local_symbol_link
        md5digest = self.get_local_md5()
        for host in self.__datanodes:
            url = self.get_download_strategy_local_url(host, self.__server_id, md5digest)
            (retCode, httpCode, fullPath) = http_download(url, self.__etc_path)
            if(retCode):
                if(httpCode == 200):
                    if(os.path.exists(symbol_link)):
                        os.unlink(symbol_link)
                    else:
                        try:						
                            os.remove(symbol_link)						
                        except:
                            traceback.print_exc()                            							
                    os.symlink(fullPath, symbol_link)
                self.rm_old_strategy_file(self.__server_id, fullPath)
                break

class ProcList(object):
    def __init__(self):
        self.__proc_list_path__ = "/home/dspeak/yyms/proc_lists"
        self.__hostinfo = HostInfo.get_hostinfo()
        self.__datanodes = self.__hostinfo.get("dnips").split(":")
        self.__server_id = self.__hostinfo.get("server_id", None)

    def get_md5(self):
        try:
            m = md5()
            fd = open(self.__proc_list_path__, "r")
            m.update(fd.read())
            return m.hexdigest()
        except:
            logging.exception("")
            return None

    def download_proc_list(self):
        md5 = self.get_md5()
        for host in self.__datanodes:
            try:
                url = "http://%s:51300/proc_list?serverId=%d&md5=%s" % (host, self.__server_id, md5)
                (retCode, httpCode, fullPath) = http_download(url, "/home/dspeak/yyms")
                if retCode:
                    if httpCode == 200:
                        shutil.move(fullPath, self.__proc_list_path__)
                    break
            except:
                logging.exception("")

class AlarmBlock(object):
    def __init__(self):
        self.__hostinfo = HostInfo.get_hostinfo()
        self.__etc_path = "/home/dspeak/yyms/yymp/yymp_agent/etc/"
        self.__strategy_local_symbol_link = self.__etc_path + "block_local"
        self.__strategy_public_symbol_link = self.__etc_path + "block_public"
        self.__server_id = self.__hostinfo.get("server_id", None)
        self.__datanodes = self.__hostinfo.get("dnips").split(":")
        random.shuffle(self.__datanodes)

    def get_download_block_local_url(self, host, server_id, md5digest):
        url_tmpl = "http://%s:51300/get_alarm_block?serverId=%d&md5=%s"
        return url_tmpl % (host, server_id, md5digest)

    def get_download_public_block_url(self, host, md5digest):
        url_tmpl = "http://%s:51300/get_alarm_block?serverId=public_block&md5=%s"
        return url_tmpl % (host, md5digest)

    def rm_old_block_file(self, serverid, fullpath):
        if fullpath == "":
            return
        for root, dirs, files in os.walk(self.__etc_path):
            for file in files:
                path = os.path.join(root, file)
                if re.match("%s\.\S+" % serverid, file) and (not os.path.islink(path)):
                    if path != fullpath:
                        os.remove(path)
            break

    def get_public_md5(self):
        return self.get_block_file_md5(self.__strategy_public_symbol_link)

    def get_local_md5(self):
        return self.get_block_file_md5(self.__strategy_local_symbol_link)

    def get_block_file_md5(self, symbol_link):
        md5digest = "0"
        if(not os.path.exists(symbol_link)):
            return md5digest
        if(not os.path.islink(symbol_link)):
            os.remove(symbol_link)
            return md5digest
        rf = os.readlink(symbol_link)
        (tmp, server_id, md5digest) = rf.split(".")
        if(self.__server_id == server_id):
            return md5digest
        else:
            return md5digest

    def download_public(self):
        md5digest = self.get_public_md5()
        symbol_link = self.__strategy_public_symbol_link
        for host in self.__datanodes:
            url = self.get_download_public_block_url(host, md5digest)
            (retCode, httpCode, fullPath) = http_download(url, self.__etc_path)
            if(retCode):
                if(httpCode == 200):
                    if(os.path.exists(symbol_link)):
                        os.unlink(symbol_link)
                    else:
                        try:						
                            os.remove(symbol_link)						
                        except:
                            traceback.print_exc()							
                    os.symlink(fullPath, symbol_link)
                self.rm_old_block_file("alarm_block.public_block", fullPath)
                break

    def download_local(self):
        symbol_link = self.__strategy_local_symbol_link
        md5digest = self.get_local_md5()
        for host in self.__datanodes:
            url = self.get_download_block_local_url(host, self.__server_id, md5digest)
            (retCode, httpCode, fullPath) = http_download(url, self.__etc_path)
            if(retCode):
                if(httpCode == 200):
                    if(os.path.exists(symbol_link)):
                        os.unlink(symbol_link)
                    else:
                        try:						
                            os.remove(symbol_link)						
                        except:
                            traceback.print_exc()							
                    os.symlink(fullPath, symbol_link)
                self.rm_old_block_file("alarm_block.%s" %self.__server_id, fullPath)
                break

from unittest import TestCase

class TestInitStrategy(TestCase):
    def setUp(self):
        self.__strategy = Strategy()
        self.__strategy.rm_strategy_file()

    def test_get_md5digest(self):
        self.assertEqual("0", self.__strategy.get_public_md5())
        self.assertEqual("0", self.__strategy.get_local_md5())

    def tearDown(self):
        pass

class TestDownloadStrategy(TestCase):
    def setUp(self):
        self.__strategy = Strategy()

    def test_down_strategy(self):
        self.__strategy.download_local()
        self.__strategy.download_public()
        pass

    def tearDown(self):
        pass

def singleton():
    try:
        f = file("/tmp/strategy_sync", "w")
        fcntl.flock(f.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
    except:
        logging.exception("")
        return None
    return f

def unlock(fp):
    try:
        fcntl.flock(fp.fileno(), fcntl.LOCK_UN)
        fp.close()
    except:
        logging.exception("")

def main():
    fp = singleton()
    if fp is None:
        return
    global g_caller
    log_config(g_caller)
    if proxy_module.is_inner_machine():
        proxys = proxy_module.get_proxy_info()
        for proxy in proxys:
            try:
                proxy_module.proxy_init(proxy["ip"], proxy["port"])
                socket.setdefaulttimeout(30)
                strategy = Strategy()
                strategy.download_local()
                strategy.download_public()
                ProcList().download_proc_list()
                block = AlarmBlock()
                block.download_local()
                block.download_public()
                break
            except:
                logging.exception("")
    else:
        socket.setdefaulttimeout(30)
        strategy = Strategy()
        strategy.download_local()
        strategy.download_public()
        ProcList().download_proc_list()
        block = AlarmBlock()
        block.download_local()
        block.download_public()
    unlock(fp)

if __name__ == "__main__":
    main()
#    import unittest
#    unittest.main()


