import time
import struct
import socket


class AgentProtocol(object):
    @staticmethod
    def fid_pack(logid, logtype, timestamp, data):
        uri = (3001 << 8) | 11
        return AgentProtocol.protocol_pack(
            uri, logid, logtype, timestamp, data)

    @staticmethod
    def logid_pack(logid, logtype, timestamp, data):
        uri = (3000 << 8) | 11
        return AgentProtocol.protocol_pack(
            uri, logid, logtype, timestamp, data)

    @staticmethod
    def logid_unpack(data):
        return struct.unpack('<IIH', data)

    @staticmethod
    def protocol_pack(uri, logid, logtype, timestamp, data):
        pkg = ""
        pkg += struct.pack('<Q', logid)
        pkg += struct.pack('<H', logtype)
        pkg += struct.pack('<Q', timestamp)
        pkg += struct.pack('<H', len(data))
        pkg += data
        #pkg += struct.pack('<Q', 222)
        #pkg += struct.pack('<I', 44)
        return struct.pack('<IIH', 10 + len(pkg), uri, 200) + pkg


class UnixUdpClient(object):
    def __init__(self, sun_path="/tmp/yymp.agent.sock"):
        self._sun_path = sun_path
        self._sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        #self._protocol = AgentProtocol()

    def __del__(self):
        self._sock.close()

    def write_logid(self, logid, logtype, timestamp, data):
        send_buffer = AgentProtocol.logid_pack(logid, logtype, timestamp, data)
        self._sock.sendto(send_buffer, self._sun_path)

    def write_fid(self, logid, logtype, timestamp, data):
        send_buffer = AgentProtocol.fid_pack(logid, logtype, timestamp, data)
        self._sock.sendto(send_buffer, self._sun_path)


class LogServerProtocol(object):
    @staticmethod
    def logid_pack(logid, logtype, timestamp, data, hostid, roomid):
        uri = (3000 << 8) | 11
        return LogServerProtocol.protocol_pack(
            uri, logid, logtype, timestamp, data, hostid, roomid)

    @staticmethod
    def logid_unpack(data):
        return struct.unpack('<IIH', data)

    @staticmethod
    def protocol_pack(uri, logid, logtype, timestamp, data, hostid, roomid):
        pkg = ""
        pkg += struct.pack('<Q', logid)
        pkg += struct.pack('<H', logtype)
        pkg += struct.pack('<Q', timestamp)
        pkg += struct.pack('<H', len(data))
        pkg += data
        pkg += struct.pack('<Q', hostid)
        pkg += struct.pack('<I', roomid)
        return struct.pack('<IIH', 10 + len(pkg), uri, 200) + pkg

class TcpClient(object):
    def __init__(self, ip, port):
        self._ip = ip
        self._port = port
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def __del__(self):
        self._sock.close()

    def connection(self):
        self._sock.connect((self._ip, self._port))

    def write_logid(self, logid, logtype, timestamp, data, hostid, roomid):
        self.connection()
        send_buffer = LogServerProtocol.logid_pack(logid, logtype, timestamp, data, hostid, roomid)
        #print self._sock.send(send_buffer[:10])
        print self._sock.send(send_buffer)
        ret = self._sock.recv(1024)
        print LogServerProtocol.logid_unpack(ret)


if __name__ == '__main__':
    #socket.setdefaulttimeout(5)
    #client = UnixUdpClient()
    client = TcpClient("127.0.0.1", 9999)
    client.write_logid(12345, 1, long(time.time()), "1234567890", 222, 44)
    #client.write_fid(0, 0, long(time.time()), "Hello fid!")
