#!/usr/bin/env python
#-*-coding:utf-8-*-

"""
usage:
    yymp_report_alarm.py id pname msg
        or
    echo "id pname msg" | yymp_report_alarm.py
    也支持类似数据上报的调用方式：
    yymp_report_alarm.py "k=v&k=v&k=v"
        or
    echo "k=v&k=v&k=v" | yymp_report_alarm.py
    同时兼容以下调用方式:
    yymp_report_alarm.py id sid pre msg
        or
    echo "id sid pre msg" | yymp_report_alarm.py
    pre 设置此次为预警，因为同一sid预警与告警原来是通过阀值处理，纯告警无法区分，所以加 pre 字段， 1：预警， 0：告警
"""
import os
module_file_dir = os.path.dirname(__file__)
if(module_file_dir):
    os.chdir(module_file_dir)
import sys
sys.path.append("../yymp_pylibs/yymp_common")
import logging
import time

from app_conf import *


def handler(kv):
    comm_dir = Config.instance().config.get("common_dir", "dir")
    sys.path.append(comm_dir)
    try:
        from agent_protocol import UnixUdpClient
        conn = UnixUdpClient()
        conn.write_fid(1, 0, long(time.time()), kv)
    except Exception, e:
        logging.exception(e)
        sys.exit()


def parse_arg():
    kv = dict()
    if len(sys.argv) == 1:
        for line in sys.stdin:
            line = line.lstrip()
            line = line.split(" ", 3)   # 将id,sid,pre与msg切分开
            if len(line) == 4:
                try:
                    id = int(line[0])
                    sid = int(line[1])
                    pre = int(line[2])
                except ValueError, e:
                    logging.error("Can't convert id or sid to int")
                    sys.exit()
                else:
                    msg = line[3]
                    kv["id"] = id
                    kv["sid"] = sid
                    kv["alarm"] = 1
                    kv["pre"] = pre 
                    kv["msg"] = msg
            else:
                logging.error("Invalid input format.")
                sys.exit()
            break   # only one line in stdin, so break
    elif len(sys.argv) == 2:
        kv_list = sys.argv[1].split('&')
        for i in kv_list:
            if i.find("=") > 0:
                k, v = i.split("=")
                k = k.strip()
                v = v.strip()
                if k == "id" or k == "sid" or k == "alarm" or k == "pre":
                    try:
                        v = int(v)
                    except ValueError, e:
                        logging.error("Can't convert %s to int" % k)
                        sys.exit()
                kv[k] = v
            else:
                logging.error("Invalid input format")
                sys.exit()
        kv["alarm"] = 1
        if ("id" not in kv) or ("msg" not in kv):
            logging.error("Must has 'id' and 'msg' key")
            sys.exit()
    elif len(sys.argv) >= 5:
        try:
            id = int(sys.argv[1])
            sid = int(sys.argv[2])
            pre = int(sys.argv[3])
        except ValueError, e:
            logging.error("Can't convert id or sid to int")
            sys.exit()
        else:
            #兼容msg未加引号而且msg中包含空格的情况
            #如yymp_report_alarm.py id msg1 msg2 msg3 ...
            #msg1 msg2 msg3将会被当成一条msg
            msg = sys.argv[4]
            for arg in sys.argv[5:]:
                msg += " "
                msg += arg
            kv["id"] = id
            kv["sid"] = sid
            kv["alarm"] = 1
            kv["pre"] = pre 
            kv["msg"] = msg.replace("'", "")
    elif len(sys.argv) == 4:
        try:
            id = int(sys.argv[1])
            pname = sys.argv[2]
            msg = sys.argv[3]
        except ValueError:
            logging.exception("")
            sys.exit()
        kv["id"] = id
        kv["alarm"] = 1
        kv["pname"] = pname
        kv["msg"] = msg
    else:
        logging.error("Invalid input format")
        sys.exit()

    try:
        import json
        kv = json.dumps(kv)
    except:
        kv = str(kv).replace("'", '"')
    return kv


def main():
    log_config()
    json_str = dict()
    json_str = parse_arg()
    handler(json_str)
    print json_str

if __name__ == "__main__":
    main()
