#!/bin/bash
# Name   : log_clean.bash 
# Author : liaojunlian@yy.com
# Create : 2015-02-06
# Purpose: automatically clean logs in /data/weblog when /data usage > 90% .

#web日志统一清理， > 90% mtime + 7 delete , mtime + 3 delete, mtime +0 delete,  biggist file >1G truncate, 
#日志格式：*.log *.gz *.zip *.txt *.log.2015-01-12 等等

# directory to clean log
LOGDIR=/data/weblog

# directory to store executing log.
LOGBASE="/data/weblog/others/logclean"
LOGFILE="deleted.log"
LOG=$LOGBASE/$LOGFILE
CLEAN_THRESHOLD=90
SAFE_THRESHOLD=70


CURRENT_PTG=$(df -l|grep /data$|awk '{print $5}'|cut -d'%' -f1)
if [ "$CURRENT_PTG" == "" ]
then
	echo "Can not find /data used percentage."
	exit 1
fi

if [ ! -d $LOGDIR ]
then
	echo "log directory $LOGDIR does not exist."
	exit 2
fi

[ ! -d $LOGBASE ] && mkdir -p $LOGBASE
DATE=$(date +'%F %T')

cd $LOGDIR 
if [ $CURRENT_PTG -ge $CLEAN_THRESHOLD ] 
then
	echo -e "#################### $DATE\t deleted logs job start #################################" >> $LOG
	echo -e "$(df -lh)" >> $LOG
	echo "--------- log older than 7 days is deleting --------------------" >> $LOG
	find $LOGDIR -mtime +7 -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -exec ls -lh {} \; >> $LOG
	find $LOGDIR -mtime +7 -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -delete

	CURRENT_PTG=$(df -l|grep /data$|awk '{print $5}'|cut -d'%' -f1)
	if [ $CURRENT_PTG -gt $SAFE_THRESHOLD ] 
	then
		echo "--------- log older than 3 days is deleting --------------------" >> $LOG
		find $LOGDIR -mtime +3 -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -exec ls -lh {} \; >> $LOG
		find $LOGDIR -mtime +3 -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -delete
	fi

	CURRENT_PTG=$(df -l|grep /data$|awk '{print $5}'|cut -d'%' -f1)
	if [ $CURRENT_PTG -gt $SAFE_THRESHOLD ]
	then 
		echo "--------- log older than 0 days is deleting  --------------------" >> $LOG
		find $LOGDIR -mtime +0 -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -exec ls -lh {} \; >> $LOG
		find $LOGDIR -mtime +0 -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -delete
	fi
	
	CURRENT_PTG=$(df -l|grep /data$|awk '{print $5}'|cut -d'%' -f1)
	if [ $CURRENT_PTG -gt $SAFE_THRESHOLD ]
	then
		echo ">>>>>>>>>>>> logs size > 1G is descendingly truncating until under threshold <<<<<<<<<<<<<" >> $LOG
		find $LOGDIR  -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -size +1G -exec ls -lh {} \; >> $LOG
		echo "--------- below is log size > 1G list truncating  --------------------" >> $LOG
	fi

	while [ $CURRENT_PTG -gt $SAFE_THRESHOLD ]
	do
		# biggist file >1G truncate
		BIGFILE=$(find $LOGDIR  -type f -regextype posix-egrep -regex '.*\.(log|gz|zip|txt)([\._][_0-9-]*)?' -size +1G -exec stat -c "%s %n" {} \; | sort -nr |awk '{print $2}' | head -1 )
		if [ "$BIGFILE" == "" ]
		then
			break
		else
			ls -lh $BIGFILE >> $LOG
			> $BIGFILE
		fi

		CURRENT_PTG=$(df -l|grep /data$|awk '{print $5}'|cut -d'%' -f1)
	done

	echo -e "$(df -lh)" >> $LOG
	echo -e "#################### $DATE\t job DONE AT:$(date +'%F %T') #################################" >> $LOG
fi
