#!/bin/bash

UKER_12='3.2.0-72'
CKER_6='2.6.32-504.16.2.el6'


grep -iq 'ubuntu' /etc/issue && LINUX_VERSION='ubuntu'
grep -iq 'centos' /etc/issue && LINUX_VERSION='centos'
LONGBIT=$(getconf LONG_BIT)

function print_help(){
(
cat <<'EOF'
修复旧内核许多bug，更新内核，适用于64bit Ubuntu 12.04和Centos 6.x
更新完毕不会自动重启，需要手工执行重启操作。因此不会对线上业务产生影响
Attention: update kernel from Ubuntu 12.04 and Centos 6.x which has
fix many bug. You must reboot manually.
EOF
)
timeout=10
until [[ $timeout -le 0 ]]; do
    echo -n "$timeout "
    sleep 1
    (( timeout -= 1 ))
done
echo
}

function cklist() {
    down_tel='58.215.46.21'
    down_cnc='112.81.159.21'
    down_int='10.20.176.2'
    cname='mirror.yy.duowan.com'

    if ! grep -q "$cname" /etc/hosts;then
        if [ -s /var/log/sysop_manager/isp.log ];then
            first_isp=$(sort -n /var/log/sysop_manager/isp.log | head -n1)
            case "$first_isp" in
                5)
                    down_default=${down_tel};;
                10)
                    down_default=${down_int};;
                *)
                    down_default=${down_tel};;
            esac
        else
            down_default=${down_tel}
        fi

        echo "${down_default} ${cname}" >> /etc/hosts
        pidof nscd >/dev/null && /etc/init.d/nscd restart >/dev/null 2>&1
    fi
}

function ubuntu_update_kernel() {
    case $(lsb_release -a 2>/dev/null | grep 'Codename'|awk '{print $NF}') in
        precise)
            KER=${UKER_12};;
        *)
            return
    esac
    update_driver="basedirver-${KER}"
    if dpkg --compare-versions $(uname -r) lt ${KER}-generic;then
        rm -rf linux-image-${KER}-generic_*.deb linux-headers-${KER}_*.deb \
        linux-headers-${KER}-generic_*.deb ${update_driver}.tar.gz \
        ${update_driver}

        for pks in linux-image-${KER}-generic_${KER}.107_amd64.deb \
        linux-headers-${KER}_${KER}.107_all.deb \
        linux-headers-${KER}-generic_${KER}.107_amd64.deb;
        do
            wget --timeout=4 --tries=3 "http://mirror.yy.duowan.com:63782/ubuntu/pool/main/l/linux/${pks}"
            [ $? -ne 0 ] && { echo "Download packages fail, try again later";exit 1; }
        done
        wget -q --timeout=4 --tries=3 "http://mirror.yy.duowan.com:63782/ubuntu/pks_soft/${update_driver}.tar.gz"
        [ $? -ne 0 ] && \
            { echo "Download packages fail, try again later";exit 1; }

        for grubtag in "consoleblank=0" "processor.max_cstate=1" \
        "intel_idle.max_cstate=1";
        do
            splittag="${grubtag%%=*}"
            grep -wq "$splittag" /proc/cmdline || ctag="${ctag} ${grubtag}"
        done
        if [ "x${ctag}" != "x" ];then
            source /etc/default/grub
            ctag="${ctag} ${GRUB_CMDLINE_LINUX}"
            sed -i "/GRUB_CMDLINE_LINUX=.*/s/\(GRUB_CMDLINE_LINUX=\)\(.*\)/\1\"$ctag\"/" /etc/default/grub
        fi

        # Fix bug about kdump fail in ubuntu when crash kernel boot OOM.
        grep -q 'crashkernel=' /etc/grub.d/10_linux && \
        sed -i 's/crashkernel=384M-2G:64M,2G-:128M/crashkernel=384M-2G:128M,2G-:256M/' /etc/grub.d/10_linux
        sed -i 's/enabled=1/enabled=0/' /etc/default/apport 2>/dev/null
        sed -i "/run_apport_on_vmcore$/s/^/#/" /etc/init.d/kdump 2>/dev/null

        dpkg -i --force-all linux-image-${KER}-generic_*.deb \
        linux-headers-${KER}_*.deb linux-headers-${KER}-generic_*.deb
        if [ $? -eq 0 ];then
            tar zxf ${update_driver}.tar.gz
            bash ${update_driver}/install_script/rules
        fi

        rm -rf linux-image-${KER}-generic_*.deb linux-headers-${KER}_*.deb \
        linux-headers-${KER}-generic_*.deb ${update_driver}.tar.gz \
        ${update_driver}
    else
        echo "Current kernel newer than $KER"
        exit 2
    fi
}

function centos_update_kernel() {

    ARCH="x86_64"
    grep -q "6\." /etc/issue || { echo 'Only support release 6.x';exit 1; }
    [ -d /lib/modules/${CKER_6}.${ARCH} ] && \
    { echo "Have already update kernel to ${CKER_6}.${ARCH}";exit 1; }

    rm -rf kernel-devel-${CKER_6}.${ARCH}.rpm kernel-${CKER_6}.${ARCH}.rpm \
    kernel-firmware-${CKER_6}.noarch.rpm kernel-headers-${CKER_6}.${ARCH}.rpm \
    basedriver-${CKER_6}.tar.gz basedriver-${CKER_6}

    for pks in kernel-devel-${CKER_6}.${ARCH}.rpm \
    kernel-${CKER_6}.${ARCH}.rpm kernel-firmware-${CKER_6}.noarch.rpm \
    kernel-headers-${CKER_6}.${ARCH}.rpm;
    do
        wget --timeout=4 --tries=3 "http://mirror.yy.duowan.com:63782/centos/6/centos6-${ARCH}/updates/Packages/${pks}"
        [ $? -ne 0 ] && \
        { echo "Download packages fail, try again later";exit 1; }
    done

    wget --timeout=4 --tries=3 "http://mirror.yy.duowan.com:63782/centos/centos6-prisoft/x86_64/basedriver-${CKER_6}.tar.gz"
    [ $? -eq 0 ] || { echo "Download packages fail, try again later";exit 1; }
    rpm -ivh --force kernel-devel-${CKER_6}.${ARCH}.rpm \
    kernel-${CKER_6}.${ARCH}.rpm kernel-firmware-${CKER_6}.noarch.rpm \
    kernel-headers-${CKER_6}.${ARCH}.rpm;
    if [ $? -eq 0 ];then
        tar zxf basedriver-${CKER_6}.tar.gz
        echo 'Start to update driver, wait a few second.'
        /bin/bash basedriver-${CKER_6}/install_script/rules
        sed -i "s/default.*/default=0/g" /boot/grub/grub.conf
    fi
    rm -rf kernel-devel-${CKER_6}.${ARCH}.rpm kernel-${CKER_6}.${ARCH}.rpm \
    kernel-firmware-${CKER_6}.noarch.rpm kernel-headers-${CKER_6}.${ARCH}.rpm \
    basedriver-${CKER_6}.tar.gz basedriver-${CKER_6}
}

[ "$(whoami)" != "root" ] && { echo 'No permission!';exit 1; }
print_help
[ "${LONGBIT}" -ne 64 ] && exit 1
cklist
if [ "${LINUX_VERSION}" = 'ubuntu' ];then
    ubuntu_update_kernel
elif [ "$LINUX_VERSION" = 'centos' ];then
    centos_update_kernel
fi
echo "done更新完毕请自行安排时间重启"
